
BP 3D     Back Projection - 3D, using Euler angles, ||         BP 3D 
                                                           02/16/95

PURPOSE:  Calculates Back-Projection step of 3D reconstruction using
          three Eulerian angles. Optionally one of two possible
          weighting functions is applied.

SEE ALSO: BP 3    (Back Projection - 3D, iterative (ART))
          BP 3E   (Back Projection - 3D, using Euler angles)
          BP CTF  (Back Projection - 3D, CTF correction, ||)
          BP GW   (Back Projection - weighted, arbitrary geometry)
          BP GW3  (Back Projection - weighted, arbitrary geometry, 3 angles)
          BP MEM2 (Back Projection - 2D maximum entropy method)
          BP R2   (Back Projection - 2D, R**2 weighting of the image series)
          BP RP   (Back Projection - 3D, iterative, with constraints, ||)
          BP S2   (Back Projection - 2D, single tilt iterative, with constraints)
          BP W2   (Back Projection - 2D, filtered weighted)
          BP WX   (Back Projection - weighted, X)
          BP WY   (Back Projection - weighted, Y)
          BP XY   (Back Projection - simple for single axis & conical tilting)

USAGE:    BP 3D

          .ENTER TEMPLATE FOR 2-D IMAGE FILE: PROJ001
          [Enter the name of any projection from the set.]

          .SELECTION DOC FILE: images127
          [Enter the name of the document file containing projection
           file numbers in the first column.]

          .ANGLES DOC FILE: ANGLES27
          [Enter the name of the document file containing Eulerian
           angles for the projections used (psi, theta, phi).]
                
          .3D VOLUME: X,Y DIMENSIONS:  75,77
          [Enter X and Y (NSAM, NROW) dimensions of the volume to 
          be reconstructed.
          These are independent of the dimensions of the 
          projections.  The center is assumed to be at point (NSAM/2+1,
          NROW/2+1,NSLICE/2+1).]
          
          .3D VOLUME: Z DIMENSION: 35
          [Enter Z (NSLICE) dimension of the volume to be reconstructed.]

          .FIRST, LAST SLICE TO BE RECONSTRUCTED: 2,24
          [Enter Z borders of the volume to be reconstructed.]

          .SNR/DIAMETER: 1.0E+4
          [Depending on the value:'<br>'
     '<LI>'     0 - weighting will be ommited and only back projection
           will be done.'<br>'
     '<LI>'    >0 - Signal-to-Noise-Ratio for the Wiener filtration of 
           type one weighting function.
     '<LI>'    <0 - Diameter of the structure for the weighting in
           type two weighting function.] 
                
          .RECONSTRUCTED 3-D FILE: VOLUME1001
          [Name of the output file.]

NOTE:     1. The projections do not have to have power-of-2 dimensions.

          2. The output volume has dimensions defined by X and Y dimensions
             entered and has Z dimension defined by first and last slice
             entered.  According to the numbers used above the volume
             would be 75x77x23.

          3. Weighting function used are applied in 2D and the type is 
             decided by the sign of the parameter given in SNR/DIAMETER.'<br>'
             '<LI>'    For parameter equal zero no weighting is applied.
             '<LI>'    For parameter larger than zero type one weighting
             function is applied.  The smaller the parameter the
             stronger the lowpass filtration effect.
             This weighting function is a Gaussian function as
             described in:
             M. Radermacher, Weighted Back-Projection Methods. (p.102) 
             In: '<I>'Electron Tomography'</I>'. Ed.: J. Frank, 1992.

            '<LI>'     For parameter lower than zero type two weighting
             function is applied.  Its absolute value defines the
             diameter of the reconstructed volume.  Normally DIAMETER=NSAM.
             The smaller the parameter the stronger the lowpass filtration effect.
             This weighting function is a "triangular overlap function"
             as defined in:
             G.Harauz and M. van Heel, Exact filters for general
             geometry three dimensional reconstruction.'<I>'Optik'</I>' 73 (1986) 146-156.

          4. Implemented by Paul Penczek.

SUBROUTINES: BCQ, REANG, CANG, BPCQP, BPCQ, BPCMP, BPCM, WTF, WTM

CALLER:      VTIL2
