
BP S2     Back Projection - 2D, single tilt iterative, with constraints  BP S2
                                                                      01/05/94

PURPOSE:  Calculates 2-dimensional iterative reconstruction
          for single-tilt geometry using constraints.
          The dimensions of projections do not have to be powers of two.

SEE ALSO: BP 3    (Back Projection - 3D, iterative (ART))
          BP 3D   (Back Projection - 3D, using Euler angles, ||)
          BP 3E   (Back Projection - 3D, using Euler angles)
          BP CTF  (Back Projection - 3D, CTF correction, ||)
          BP GW   (Back Projection - weighted, arbitrary geometry)
          BP GW3  (Back Projection - weighted, arbitrary geometry, 3 angles)
          BP MEM2 (Back Projection - 2D maximum entropy method)
          BP R2   (Back Projection - 2D, R**2 weighting of the image series)
          BP RP   (Back Projection - 3D, iterative, with constraints, ||)
          BP W2   (Back Projection - 2D, filtered weighted)
          BP WX   (Back Projection - weighted, X)
          BP WY   (Back Projection - weighted, Y)
          BP XY   (Back Projection - simple for single axis & conical tilting)

USAGE:    BP S2

          .ENTER TEMPLATE FOR 2-D IMAGE NAME: PRJ***
          [Enter the prefix of the projection file series.]

          .SELECTION DOC FILE: SELECT
          [Enter the name of the document file containing projection
           file numbers in the first column.]

          .ANGULAR DOC FILE: ANGLES
          [Enter the name of the document file containing Eulerian
           angles for the projections used (psi, theta, phi).
           Only theta is actually used, psi and phi are ignored.]

          .RADIUS OF RECONSTRUCTED OBJECT, HEIGHT OF THE SLICE: 45,30
          [The reconstruction is calculated for the circle only
           and within the slab of specified height.  This height is
           NSLICE dimension of resulting 3D volume.]

          .RECONSTRUCTION FROM NROW1 TO NROW2: 2,73
          [The reconstruction is calculated between specified
           nrow borders. Default is nrow1=1, nrow2=nrow.
           3D volume will have dimensions:
            NSAM, NROW2-NROW1+1, height of the slice
           where NSAM is original 2D projection dimension.]

          .LAMBDA, CORRECTION LIMIT: 1.0e-4,0.0
          [Lambda determines the weight of corrections. Too small
           value results in long time of calculations, too large
           value will terminate the iterations too soon.]

          .ITERATION LIMIT, NUMBER OF INTERNAL STEPS: 20,5
          [Program will terminate after number of iterations given.
           For each iteration there is specified number of iterations
           per slice performed (internal steps).  Thus, the total
           number of steps is the product of two numbers.
           In the above example it is 20*5=100.
           Min and max constraints are enforced during each internal
           step, but smoothing is applied only pnce per main iteration
           (20 times in the example).
           If no smoothing is requested the iteration limit should
           be equal 1, and the all the iterations requested
           should be specified as internal steps.
           Usually 50 steps are sufficient.]

          .MODE: 8
          [Mode determines the constraints used:
           1 - smoothing,
           2 - min constraint,
           5 - max constraint.
           Any combination of constraints can be used. The mode answered
           should be the sum of any single modes, for example mode=7
           activates both min and max constraints, mode=8 all three of them.]

          .MINIMUM, MAXIMUM: 1.77,1.90
          [Values of min and max constraints. They relate to the values
           in 2D projection data.]

          .SMOOTHING CONSTANT : 0.5
          [Smoothing constant has to be 0<s<1. s=0 corresponds to no
           smoothing, s=1 is strong low-pass filtration.]

          .3-D OUTPUT FILE: uuu101
          [Name of the output file.]

NOTES: 1. The limit of number of projections is 93.

       2. Only theta angle from the angular document file is used.
          Two remaining angles (psi and phi) are ignored.

       3. The projections do not have to have power-of-2 dimensions.

       4. If the radius of the reconstructed circle is too large
          comparing to the size of projections the warning is printed.
          The reconstruction is calculated, but it is incorrect
          on the boundary.

       5. The alternative, faster way to calculate the 3D single-tilt
          reconstruction is to use weighted backprojection BP W2.

       6. Implemented by: Paul Penczek.

SUBROUTINES: BPS2,REDPRO2,ASTCYL,PREPSL_S,PRJS2,PRJC2,PRJC0,REPR2_S,
             CPPB,SMT3,DOMIN3_S,DOMAX3_S,DOCORR3_S,
             BMAX_C,BMAX_S,BMIN_C,BMIN_S,FMAX_C,FMAX_S,FMIN_C,FMIN_S

CALLER:      VTIL2
