
TF C      Transfer Function - Complex                      TF C
                                                            10/23/96

PURPOSE:  To compute the phase contrast transfer function for
          bright-field electron microscopy. The 'TF C' operation
          produces the transfer function in complex form so that
          it can be applied (by using 'MU') to the Fourier transform
          of a model object, for simulations of bright-field weak
          phase contrast. For literature, see Notes.

SEE ALSO: TF      (Transfer Function - defocus dependent)
          TF C3   (Transfer Function - complex 3D)
          TF CT   (Transfer Function - phase flipping, complex, ternary)
          TF CT3  (Transfer Function - complex, ternary 3D)
          TF CTF  (Transfer Function - CTF correction)
          TF D    (Transfer Function - display)
          TF DDF  (Transfer Function - determine defocus and amplitude contrast)
          TF DEV  (Transfer Function - determine envelope function)
          TF DNS  (Transfer Function - determine and delete noise background)
          TF FL   (Transfer Function - flip sign of Fourier transform)
          TF MFL  (Transfer Function - make filter file for 'TF FL')
  
USAGE:    TF C

          .OUTPUT FILE: TFC001
          [Enter the name of the file that will store the computed
          function.]

          .CS [MM]: 2.0
          [Enter the spherical aberration constant.]

          .DEFOCUS(ANGSTROEMS), LAMBDA(ANGSTROEMS): 2000,0.037
          [Enter the amount of defocus, in Angstroems.  Positive 
          values correspond to underfocus (the preferred region);
          negative values correspond to overfocus. Next, enter
          the wavelength of the electrons.  The value used in
          this example corresponds to 100kV. Other values are listed
          below:
         '<PRE>'
                     keV        A
                     100        0.03700
                     200        0.02501
                     300        0.01968
                     400        0.01643                        ]
         '</PRE>'

          .NUMBER OF SP. FREQU. PTS.: 128
          [Enter the dimension of the 2D array.  In our example,
          each element of the array (K,I) corresponds to a spatial
          frequency                                                   '<BR>'
                  Kx = (K-65) * DK                                    '<BR>'
                  Ky = (I-65) * DK                                    '<BR>'
          where DK is defined by the next input.]

          .MAXIMUM SPATIAL FREQUENCY [A-1]: 0.15
          [Enter the spatial frequency radius corresponding to the
          maximum radius ( = 128/2 in our example) of pixels in the
          array.  From this value, the spatial frequency increment
          (DK=0.15/64) is calculated.]

          .SOURCE SIZE[A-1], DEFOCUS SPREAD[A]: 0.005,250
          [Enter the size of the illumination source in reciprocal
          Angstroems. This is the size of the source as it appears in
          the back focal plane of the objective lens.  A small value
          results in high coherence; a large value, low coherence.
          Enter the estimated magnitude of the defocus variations  
          corresponding to energy spread and lens current fluctuations.]

          .ASTIGMATISM[A], AZIMUTH[DEG]: 400,30
          [Enter the defocus range due to axial astigmatism.
          The value given indicates a defocus range of +/- 400A around
          the nominal value as the azimuth is varied. Then,
          enter the angle, in degrees, that characterizes the  
          direction of astigmatism.  The angle defines the origin 
          direction where the astigmatism has no effect.]

          .Amplitude contrast ratio [0-1], Gaussian envelope halfwidth: 0.2,0.062
          [Enter ACR and GEH (in A^1), see below for definition.]

          .Sign (+1 or -1): -1
          [Application of the transfer function results in contrast
          reversal if underfocus (DZ positive) is used.  To compensate for
          this reversal, use sign switch -1.]
         
          The transfer function is then computed in complex form compatible 
          with the Fourier transform format.

NOTES: 1. Theory and all definitions of electron optical parameters
          are as in: J. Frank (1973)'<I>' Optik'</I>' 38:519, and R. Wade
          & J. Frank (1974) '<I>'Optik'</I>' 49:81.  Internally, the program
          uses the generalized coordinates defined in these papers.

       2. In addition, an optional cosine term has been added with a weight,
          and an ad hoc Gaussian fall off function has been added as discussed
          in Stewart et al., '<I>'EMBO J.'</I>' 12 (1993) 2589-2599.

          The complete expression is:                                    '<BR>'

          TF(K) = [(1-ACR)*sin(GAMMA) - ACR*cos(GAMMA)]*ENV(K)*exp[-(K/GEH)^2]

     3. To apply the transfer function to a model 2D structure, use the 
        following steps:                                                 '<BR>'
            (i)   use 'FT' to compute the Fourier transform of the model 
        structure,                                                       '<BR>'
            (ii)  use 'TF C' to compute the transfer function in complex 
        format,                                                          '<BR>'
            (iii) use 'MU' to multiply the Fourier transform with the 
        complex transfer function,                                       '<BR>'
            (iv)  use 'FT' to compute the inverse Fourier transform.
           
SUBROUTINES: TFD, TRAFD, TRAFC, TRAFC3

CALLER:      UTIL1 

