
TF D      Transfer Function - Display                          TF D    
                                                            10/23/96

PURPOSE:  Computes the phase contrast transfer function for
          bright-field electron microscopy. TF D produces 
          the transfer function (or its square, the envelope function) 
          in real, displayable form.

SEE ALSO: TF      (Transfer Function - defocus dependent)
          TF C    (Transfer Function - complex)
          TF C3   (Transfer Function - complex 3D)
          TF CT   (Transfer Function - phase flipping, complex, ternary)
          TF CT3  (Transfer Function - complex, ternary 3D)
          TF CTF  (Transfer Function - CTF correction)
          TF D    (Transfer Function - display)
          TF DDF  (Transfer Function - determine defocus and amplitude contrast)
          TF DEV  (Transfer Function - determine envelope function)
          TF DNS  (Transfer Function - determine and delete noise background)
          TF FL   (Transfer Function - flip sign of Fourier transform)
          TF MFL  (Transfer Function - make filter file for 'TF FL')
  
USAGE:    TF D
  
          .OUTPUT FILE: TFD001
          [Enter the name of the file that will store the computed
          function.]

          .CS [MM]: 2.0
          [Enter the spherical aberration constant.]

          .DEFOCUS(ANGSTROEMS), LAMBDA(ANGSTROEMS): 2000,0.037
          [Enter the amount of defocus, in Angstroems.  Positive  
          values correspond to underfocus ( = the preferred region);
          negative values correspond to overfocus. Next enter
          the wavelength of the electrons.  The value used in this
          example corresponds to 100kV. Other values are listed below:
          '<PRE>'
                     keV           A
                     100        0.03700
                     200        0.02501
                     300        0.01968
                     400        0.01643                            ]
          '</PRE>'  

          .NUMBER OF SP. FREQU. PTS.: 128
          [Enter the dimension of the 2D array.  In our example,
          each element of the array (K,I) corresponds to a spatial
          frequency                                               '<BR>'
                  Kx = (K-65) * DK                                '<BR>'
                  Ky = (I-65) * DK                                '<BR>'
          where DK is defined by the next input.]

          .MAXIMUM SPATIAL FREQUENCY [A-1]: 0.15
          [Enter the spatial frequency radius corresponding to the
          maximum radius ( = 128/2 in our example) of pixels in the
          array.  From this value, the spatial frequency increment
          (DK=0.15/64) is calculated.]

          .SOURCE SIZE[A-1], DEFOCUS SPREAD[A]: 0.005,250
          [Enter the size of the illumination source in reciprocal
          Angstroems.  It is the size of the source as it appears in
          the back focal plane of the objective lens.  A small value
          results in high coherence; a large value, low coherence.
          Next, enter the estimated magnitude of the defocus variations  
          corresponding to energy spread and lens current fluctuations.]

          .ASTIGMATISM[A], AZIMUTH[DEG]: 400,30
          [Enter the defocus difference due to axial astigmatism.
          The value given indicates a defocus range of +/- 400A around
          the nominal value as the azimuth is varied. Next, 
          enter the angle, in degrees, that characterizes the 
          direction of astigmatism.  The angle defines the origin  
          direction where the astigmatism has no effect.]

          .AMPLITUDE CONTRAST RATIO [0-1], GAUSSIAN ENVELOPE PARAMETER: 0.2,100
          [Enter ACR and GEP; see below for definitions.]

          .(D)IFFRACTOGRAM/(E)NVELOPE/(S)TRAIGHT: D
          [Either the transfer function is put into the array directly as
          computed (option 'S'), or its square (option 'D') is stored, or 
          else the envelope function describing the attenuation of the 
          transfer function due to partial coherence effects (option
          'E') is stored.]

NOTES: 1. Theory and all definitions of electron optical parameters
          are according to:  J. Frank (1973) '<I>'Optik'</I>' 38:519, and R. Wade
          & J. Frank (1974) '<I>'Optik'</I>' 49:81.  Internally, the program
          uses the generalized coordinates defined in these papers.

       2. In addition, an optional cosine term has been added with a weight,
          and an ad hoc Gaussian falloff function has been added as discussed
          in Stewart et al. (1993) '<I>'EMBO J.'</I>' 12:2589-2599.

          The complete expression is:                                   '<BR>'

          TF(K) = [(1-ACR)*sin(GAMMA) - ACR*cos(GAMMA)]*ENV(K)*exp[-GEP*K^2]


SUBROUTINES: TFD, TRAFD, TRAFC, TRAFC3

CALLER:      UTIL1
