C++************************************************************************
C
C  ADD.FOR                   OUTPUT FILES CHANGED AUG 96 al
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR       *
C *  LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR    *
C *  USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF     *
C *  THE CENTER FOR LABORATORIES AND RESEARCH   			   *
C **************************************************************************
C
C  ADD(LUN1, LUN2, LUN3, NSAM, NROW, NSLICE, SIGT)
C
C  PURPOSE:  ADD, SUBTRACT, OR MULTIPLY TWO IMAGES
C            OR MULTIPLY FOURIER TRANSFORMS
C            THIS SUBROUTINE ADDS, SUBTRACTS, OR MULTIPLIES AN IMAGE
C            STORED IN FILE 1 TO, FROM, OR WITH AN IMAGE STORED IN
C            FILE 2.  BOTH IMAGES HAVE TO HAVE THE SAME SIZE.
C
C********************CHANGED ON 7/21/86 BY MN**************************
C        LUN1         LOGICAL UNIT NUMBER OF INPUT FILE 1
C        LUN2         LOGICAL UNIT NUMBER OF INPUT FILE 2
C        LUN3         LOGICAL UNIT NUMBER OF OUTPUT FILE 
C        NSAM,NROW    X & Y DIMENSIONS OF IMAGES
C        NSLICE       Z DIMENSION OF IMAGES
C        SIGT        +1    1 IS ADDED TO 2, AND RESULT PUT INTO 3
C                    -1    1 IS SUBTRACTED FROM 2, AND RESULT PUT INTO 3
C                    +2    1 IS MULTIPLIED WITH 2, AND RESULT PUT INTO 3
C                    -2    1 IS DIVIDED BY 2, AND RESULT PUT INTO 3
C                          OR COMPLEX FOURIER MULTIPLICATION WITH CONJUGATE
C                    -3    COMPLEX 1 IS DIVIDED BY COMPLEX 2
C                    +3    1 IS SQUARED, RESULT PUT INTO 3
C                    +4    SQUARE ROOT OF 1 IS COMPUTED AND PUT INTO 3
C                    +5    1 IS MULTIPLIED USING ARITHMETIC OR WITH 2, 
C                             AND RESULT PUT INTO 3
C                          
C
C    VARIABLES: IFORM (TYPE)  FILE TYPE SPECIFIER. 
C	         +1    R    2-D IMAGE
C                +3   R3    3-D IMAGE
C               -11   O2    2-D FOURIER TRANSFORM, MIXED RADIX ODD
C               -12   E2    2-D FOURIER TRANSFORM, MIXED RADIX EVEN
C               -21   O3    3-D FOURIER TRANSFORM, MIXED RADIX ODD
C               -22   E3    3-D FOURIER TRANSFORM, MIXED RADIX EVEN
C
C   !!!  W A R N I N G  !!!
C   What in the comments is referred to as a "second file" is in fact
C   first file in the order of questions asked by the corresponding
C   operation, and what is referred to as a "first file" is
C   the second file in order of questions asked.
C   In other words in it is LUN3 = LUN2 "operation" LUN1
C
C23456789012345678901234567890123456789012345678901234567890123456789012
C--************************************************************************

         SUBROUTINE ADD(LUN1,LUN2,LUN3, NSAM,NROW,NSLICE,SIGT)

         INCLUDE 'CMBLOCK.INC'

         COMMON  BUF(1)
         COMPLEX CBUF(1)
         COMPLEX DIV
         EQUIVALENCE(BUF(1),CBUF(1))

CNO_SAVE

C       DOES NOT WORK ON SOME ODD FILE FORMATS NO LONGER IN USE
        IF (IFORM.EQ.0 .OR. IFORM.EQ.8 .OR. IFORM.EQ.11 .OR.
     &	     IFORM.EQ.12 .OR. IFORM.EQ.16 .OR. IFORM.EQ.-9)  THEN
            CALL ERRT(39,'ADD',NE)
            RETURN
        ENDIF


	NSAM1 = NSAM + 1
	NREC  = NROW * NSLICE

	IF (SIGT .EQ. +1.0)  THEN
C           SIGT        +1    1 IS ADDED TO 2, RESULT PUT INTO 3
            DO   I=1,NREC
		CALL REDLIN(LUN1,BUF,NSAM,I)
		CALL REDLIN(LUN2,BUF(NSAM1),NSAM,I)
            	DO  K=1,NSAM
  		   BUF(K) = BUF(K)+BUF(NSAM+K)
                ENDDO
 		CALL WRTLIN(LUN3,BUF,NSAM,I)
            ENDDO
		
	ELSEIF (SIGT .EQ. -1.0)  THEN
C          SIGT        -1    1 IS SUBTRACTED FROM 2, RESULT PUT INTO 3
	   DO  I=1,NREC
		CALL REDLIN(LUN1,BUF,NSAM,I)
		CALL REDLIN(LUN2,BUF(NSAM1),NSAM,I)
		DO   K=1,NSAM
  		   BUF(K)=-BUF(K)+BUF(NSAM+K)
                ENDDO
 		CALL WRTLIN(LUN3,BUF,NSAM,I)
           ENDDO

	ELSEIF (SIGT .EQ. +2.0)  THEN
C          SIGT        +2    1 IS MULTIPLIED WITH 2, RESULT PUT INTO 3
	   IF (IFORM .GT. 0)  THEN
C               REAL FILES
		DO  I=1,NREC
		   CALL REDLIN(LUN1,BUF,NSAM,I)
		   CALL REDLIN(LUN2,BUF(NSAM1),NSAM,I)
		   DO  K=1,NSAM
                      BUF(K)=BUF(K)*BUF(NSAM+K)
                   ENDDO
		   CALL WRTLIN(LUN3,BUF,NSAM,I)
                ENDDO

	   ELSEIF (IFORM .LT. 0)  THEN
C               FOURIER FILES	
		NSH = NSAM/2
		DO I=1,NREC
		   CALL REDLIN(LUN1,BUF,NSAM,I)
		   CALL REDLIN(LUN2,BUF(NSAM1),NSAM,I)
		   DO  N=1,NSH
    		      CBUF(N)=CBUF(N)*CBUF(NSH+N)
                   ENDDO
    		   CALL WRTLIN(LUN3,BUF,NSAM,I)
                ENDDO
	   ENDIF
	
	ELSEIF (SIGT .EQ. -2.0)  THEN
C          SIGT        -2    1 IS DIVIDED BY 2, AND RESULT PUT INTO 2
C                    OR COMPLEX FOURIER MULTIPLICATION WITH CONJUGATE
	   IF (IFORM.GT.0)  THEN
		IZCOUN = 0
		DO  I=1,NREC
		   CALL REDLIN(LUN1,BUF,NSAM,I)
		   CALL REDLIN(LUN2,BUF(NSAM1),NSAM,I)
		   DO  K=1,NSAM
		      IF (BUF(K) .NE. 0) THEN
		         BUF(K) = BUF(NSAM+K) / BUF(K)
		      ELSE
		         BUF(K) = 0.0
		         IZCOUN = IZCOUN + 1
		      ENDIF
                   ENDDO
 		   CALL WRTLIN(LUN3,BUF,NSAM,I)
                ENDDO

		IF (IZCOUN .GT. 0) WRITE(NOUT,160) IZCOUN
160	        FORMAT(' --- WARNING: ',I7,' OUTPUT PIXELS SET TO 0.0 ',
     &                 'WHEN DIVISION BY 0 ATTEMPTED')

	   ELSEIF (IFORM .LT. 0)  THEN
C               FOURIER MULTIPLICATION 	
		NSH = NSAM / 2
		DO I = 1,NREC
		   CALL REDLIN(LUN1,BUF,NSAM,I)
		   CALL REDLIN(LUN2,BUF(NSAM1),NSAM,I)
		   DO N=1,NSH
    		      CBUF(N) = CBUF(NSH+N) * CONJG(CBUF(N))
                   ENDDO
    		   CALL WRTLIN(LUN3,BUF,NSAM,I)
                ENDDO
	   ENDIF
C
	ELSEIF (SIGT .EQ. +3.0)  THEN
C          SIGT        +3    1 IS SQUARED, RESULT PUT INTO 3
	   IF (IFORM .GT. 0)  THEN
C               REAL IMAGE
		DO  I=1,NREC
		   CALL REDLIN(LUN1,BUF,NSAM,I)
		   DO  K=1,NSAM
                      BUF(NSAM+K)=BUF(K)*BUF(K)
                   ENDDO
 		   CALL WRTLIN(LUN3,BUF(NSAM1),NSAM,I)
                ENDDO

	   ELSEIF (IFORM .LT. 0)  THEN	
C               FOURIER IMAGES
		NSH = NSAM/2
		DO I=1,NREC
		   CALL REDLIN(LUN1,BUF,NSAM,I)
		   DO N=1,NSH
                      CBUF(N) = CBUF(N)*CONJG(CBUF(N))
                   ENDDO
                   CALL WRTLIN(LUN3,BUF,NSAM,I)
                ENDDO
	   ENDIF

	ELSEIF (SIGT .EQ. -3.0)  THEN
C          SIGT        -3    COMPLEX 1 IS DIVIDED BY COMPLEX 2
		NSH = NSAM / 2
		DO I = 1,NREC
		   CALL REDLIN(LUN1,BUF,NSAM,I)
		   CALL REDLIN(LUN2,BUF(NSAM1),NSAM,I)
		   DO N=1,NSH
    		      CBUF(N) = CBUF(NSH+N) / CBUF(N)
                   ENDDO
    		   CALL WRTLIN(LUN3,BUF,NSAM,I)
                ENDDO


	ELSEIF(SIGT .EQ. +4.0)  THEN
C           SIGT        +4    SQUARE ROOT OF 1 IS COMPUTED AND PUT IN 3
	    IF  (IFORM .GT. 0)  THEN
		DO  I=1,NREC
                   CALL REDLIN(LUN1,BUF,NSAM,I)
                   DO K=1,NSAM
                      BUF(NSAM+K)=SQRT(BUF(K))
                   ENDDO
                   CALL WRTLIN(LUN3,BUF(NSAM+1),NSAM,I)
                ENDDO
	    ELSE
C               NOT IMPLEMENTED FOR FOURIER
		CALL ERRT(2,'ADD',NE)
	    ENDIF

	ELSEIF (SIGT.EQ.+5.0)  THEN
C          SIGT        +5   1 IS MULTIPLIED WITH 2, RESULT PUT IN 3
C                           WITH ARITHMETIC OR
	   IF  (IFORM .GT. 0)  THEN
		DO  I=1,NREC
                   CALL REDLIN(LUN1,BUF,NSAM,I)
                   CALL REDLIN(LUN2,BUF(NSAM1),NSAM,I)
                   DO K=1,NSAM
                      B = BUF(NSAM+K)
                      IF (BUF(NSAM+K) .EQ. 0.) B = BUF(K)
                      BUF(K) = B
                   ENDDO
                   CALL WRTLIN(LUN3,BUF,NSAM,I)
                ENDDO
	   ELSE
C               NOT IMPLEMENTED FOR FOURIER
		CALL ERRT(2,'ADD',NE)
	   ENDIF
	ELSE
C          UNKNOWN SIGT
	   CALL ERRT(23,'ADD',NE)
	ENDIF

      RETURN
      END
      
