c
c
C==========================================================================
C CALLING PROGRAM FOR SIMULTANEOUS TRANSLATIONAL AND ROTATIONAL
C ALIGNMENT OF THREE-DIMENSIONAL VOLUMES. BRUT FORCE APPROACH.
C                                                        8/91 M.Radermacher 
C
C    LUN1,LUN2,LUN3 logical unit numbers for input,reference
c                   and ccf-file
C    MAXDIM         maximum array space
C
C    NCCF =161051 CORRESPONDS TO 11 SHIFT POSITIONS FOR EACH, X,Y, AND Z
C          COMBINED WITH 11 ROTATIONS FOR EACH ALPHA,BETA WITH GAMMA BEING
C          THE INVERSE OF ALPHA, OR TO NO SHIFT AND 401 ANGLES BETA WITH 
C          ALPHA/GAMMA COUPLED OR TO 51 INDEPENDENT ANGLES IN ALPHA,BETA
C          GAMMA. 
C
C==========================================================================
c
        SUBROUTINE AL3DCALL(LUN1,LUN2,LUN3,MAXDIM)

        INCLUDE 'CMBLOCK.INC' 
C   dimension 262144 fits 64**3 cube. Volume can be larger depending
C   on maxdim.
        PARAMETER (NCCF=161051)
C        DIMENSION SUM(NCCF)	
C        COMMON SPACE(256),BUF(1024),SUM,VOL(262144)
C	NOTE: LAYOUT SUM(NCCF),VOL(262144) ON BUFIMG(1) IN THAT ORDER.
C	ML. 10/15/96
        COMMON SPACE(256),BUF(1024),BUFIMG(1)
        COMMON /COMUN/FIL1,FIL2,FILO
        CHARACTER*81 FIL1,FIL2,FILO
        CHARACTER*1 NULL,YN
	INTEGER  P1
        LOGICAL LINK

C	POINTER TO VOL ARRAY
	P1 = NCCF + 1
        IREPEAT=0
        ISPACE1=NCCF+256+1024
        NULL=CHAR(0)
        CALL RDPRM(DEBUG,NLOOP,ILOOP,'DEBUG 0,1,OR 2$')
	CALL FILERD(FIL1,NLET1,NULL,'INPUT1',IRTFLG)
        IF(FIL1(1:1).EQ.'*') RETURN
	CALL OPENFB(BUF,FIL1,LUN1,NSAM1,NROW1,'OLD',NF,NSLIC1)

C here it is decided if there is enough buffer space, depending on
c volume dimensions:

        ISPACE=ISPACE1+NSAM1*NROW1*NSLIC1
        IF(ISPACE.GT.MAXDIM) THEN
        WRITE(NOUT,1005) ISPACE*4,MAXDIM*4 
1005    FORMAT(1X,'YOU NEED ',I7,' BYTES OF MEMORY, BUT HAVE ONLY'
     $  ,I7,/' IF YOU ARE NOT ALREADY USING THE BATCH VERSION OF SPIDER',
     $  /' TRY USING THAT. TRY USING A SMALLER VOLUME OR CHANGE MAXDIM '
     $  /' IN INILUNB.FOR OR LOWER THE DIMENSIONS IN AL3DCALL.FOR')
        CLOSE( LUN1)
        RETURN
        ENDIF
       
        CALL FILERD(FIL2,NLET2,NULL,'REFERENCE',IRTFLG)
        IF(FIL2(1:1).EQ.'*') THEN
        CLOSE(LUN1)
        RETURN
        ENDIF

	CALL OPENFB(BUF,FIL2,LUN2,NSAM2,NROW2,'OLD',NF,NSLIC2)
        CALL FILERD(FILO,NLETO,NULL,'CCF-FILE',IRTFLG)
        IF(FILO(1:1).EQ.'*') THEN
        CLOSE(LUN1)
        CLOSE(LUN2)
        RETURN
        ENDIF
        ICOMB=FLOAT(NCCF)**0.3333
1       WRITE(NOUT,1007) NCCF,ICOMB
1007    FORMAT(1X,
     $/' IN THE FOLLOWING SPECIFY THE NUMBER OF ROTATIONS AND SHIFTS.',
     $/' THE CROSSCORELATION IS CALCULATED FOR EACH POSSIBLE'
     $/' COMBINATION. THE NUMBER OF COMBINATIONS IS CURRENTLY'
     $/' LIMITED TO ',I8,' FOR EXAMPLE, NO VARIATION OF THE POSITION',
     $/' AND ',I4,' VALUES FOR EACH ANGLE OR VICE VERSA. IF THE',
     $/' NUMBER NA OF ANGULAR POSITIONS IS THE SAME FOR EACH ANGLE,',
     $/' AND THE NUMBER NS FOR SHIFT VALUES IS THE SAME FOR EACH',
     $/' COORDINATE, THE NUMBER OF COMBINATIONS IS NA**3*NS**3.')


C ASK IF ONLY TILTAXIS DIRECTION SHOULD BE VARIED, WITHOUT ROTATION
C OF VOLUME.

        CALL RDPRMC(YN,NA,.TRUE.,
     $  'LINK ALPHA TO -GAMMA? (Y/N)$',NULL,IRT)
        LINK=.FALSE.
        IF(YN.EQ.'Y') THEN
        LINK=.TRUE.
        WRITE(NOUT,*) '** GAMMA SET TO INVERSE OF ALPHA **'
        WRITE(NOUT,*) ' '
        ENDIF

201     WRITE(NOUT,*) 'NUMBER OF ANGULAR STEPS (ODD, MAXIMUM 11)'
        CALL RDPRMI(NALPHA,NBETA,NLOOP,ILOOP,'In alpha,beta')
        CALL RDPRMI(NGAMMA,NDUM,NLOOP,ILOOP,'and also in Gamma')
        IF(NALPHA.EQ.0) NALPHA=1
        IF(NBETA.EQ.0) NBETA=1
        IF(NGAMMA.EQ.0) NGAMMA=1

        IF(LINK) THEN
        NGAMMA=1
        WRITE(NOUT,*) '** NUMBER OF STEPS IN GAMMA SET TO 1 **'
        ENDIF

        CHA=INT(NALPHA/2)-FLOAT(NALPHA)/2.
        CHB=INT(NBETA/2)-FLOAT(NBETA)/2.
        CHG=INT(NGAMMA/2)-FLOAT(NGAMMA)/2.
        IF(CHA.EQ.0..OR.CHB.EQ.0..OR.CHG.EQ.0.) THEN
         WRITE(NOUT,*) '*** USE ONLY ODD NUMBERS *** PLEASE REENTER'
         IREPEAT=IREPEAT+1
         IF(IREPEAT.LT.10) THEN 
         GOTO 201
         ELSE
         WRITE(NOUT,*) 'THESE WERE ENOUGH TRIES, TAKE A REST'
         CLOSE(LUN1)
         CLOSE(LUN2)
         CLOSE(LUN3)
         RETURN
         ENDIF
        ENDIF
        WRITE(NOUT,*) 'AND NOW THE ANGULAR INCREMETS'
        CALL RDPRM2(DALP,DBET,NLOOP,ILOOP,'In alpha,beta')
        CALL RDPRM(DGAM,NLOOP,ILOOP,'and also in Gamma')
        IF(DALP.EQ.0.) DALP=1.
        IF(DBET.EQ.0.) DBET=1.
        IF(DGAM.EQ.0)  DGAM=1.

C THE FOLLOWING IS TAKEN CARE OF IN ALI3D.FOR:
        IF(LINK) WRITE(NOUT,*) 
     $  '** INCREMENTS IN GAMMA SET TO INVERSE OF ALPHA **'

        WRITE(NOUT,*) 'AND THE ANGLES AROUND WHICH TO VARY'
        CALL RDPRM2(ALPHC,BETC,NLOOP,ILOOP,'For alpha,beta')
        CALL RDPRM(GAMC,NLOOP,ILOOP,'and also for Gamma')

C THE FOLLOWING IS TAKEN CARE OF IN ALI3D.FOR:
        IF(LINK) WRITE(NOUT,*) 
     $  '** CENTRAL ANGLE OF GAMMA SET TO CENTRAL ANGLE OF ALPHA **'


        WRITE (NOUT,*) 'NOW LETS FACE THE SHIFTS,', 
     $  'AGAIN SIMILAR QUESTIONS'

202     WRITE (NOUT,*) 'NUMBER OF POSITIONS (ODD, MAXIMUM 11)'
        CALL RDPRMI(NPX,NPY,NLOOP,ILOOP,'In X,Y$')
        CALL RDPRMI(NPZ,NDUM,NLOOP,ILOOP,'and also Z$')
        IF(NPX.EQ.0) NPX=1
        IF(NPY.EQ.0) NPY=1
        IF(NPZ.EQ.0) NPZ=1
        CHX=INT(NPX/2)-FLOAT(NPX)/2.
        CHY=INT(NPY/2)-FLOAT(NPY)/2.
        CHZ=INT(NPZ/2)-FLOAT(NPZ)/2.
        IF(CHA.EQ.0..OR.CHB.EQ.0..OR.CHG.EQ.0.) THEN
         WRITE(NOUT,*) '*** USE ONLY ODD NUMBERS *** PLEASE REENTER'
         IREPEAT=IREPEAT+1
         IF(IREPEAT.LT.10) THEN 
         GOTO 202
         ELSE
         WRITE(NOUT,*) 'THESE WERE ENOUGH TRIES, TAKE A REST'
         CLOSE(LUN1)
         CLOSE(LUN2)
         CLOSE(LUN3)
         RETURN
         ENDIF
        ENDIF
        NPOS=NALPHA*NBETA*NGAMMA*NPX*NPY*NPZ
        IF(NPOS.GT.NCCF) THEN 
        WRITE(NOUT,*) 'TOO MANY ANGLE/SHIFT COMBINATIONS. TRY AGAIN'
        GOTO 1
        ENDIF 

        WRITE(NOUT,*) 'SHIFT INCREMENTS'
        CALL RDPRM2(DX,DY,NLOOP,ILOOP,'In X,Y$')
        CALL RDPRM(DZ,NLOOP,ILOOP,'and also Z$')
        IF(DX.EQ.0.) DX=.1
        IF(DY.EQ.0.) DY=.1
        IF(DZ.EQ.0.) DZ=.1

        WRITE(NOUT,*) 'AND THE SHIFT AROUND WHICH TO VARY'
        CALL RDPRM2(SX0,SY0,NLOOP,ILOOP,'In X,Y$')
        CALL RDPRM(SZ0,NLOOP,ILOOP,'and also Z$')
        CALL RDPRM(RADIUS,NLOOP,ILOOP,'outside mask radius$')
        CALL RDPRM(RADIN,NLOOP,ILOOP,'inside mask radius$')
        CALL RDPRMI(ISAMPLE,IDUM,NLOOP,ILOOP,'SAMPLING INTERVAL')
        IF(ISAMPLE.EQ.0) ISAMPLE=1
        CALL RDPRMI(NSAMO,IDUM,NLOOP,ILOOP,'LINE LENGTH OF CCF-FILE')
        IF(NSAMO.EQ.0) NSAMO=3
        CALL RDPRMI(NITER,IDUM,NLOOP,ILOOP,'MAX. NUMBER OF ITERATIONS')
        WRITE(NOUT,*) 'THATS IT, NOW PLEASE BE PATIENT'
        NANGLE=NALPHA*NBETA*NGAMMA
        NPOSITION=NPX*NPY*NPZ   
        NCOEFF=NANGLE*NPOSITION 

        ROWO=FLOAT(NCOEFF)/FLOAT(NSAMO)
        TEST=FLOAT(INT(ROWO))
        IF(TEST.NE.ROWO) THEN 
        NROWO=ROWO+1
        ELSE
        NROWO=ROWO
        ENDIF

        NROWOPEN=NROWO*NITER
	CALL OPENFB(BUF,FILO,LUN3,NSAMO,NROWOPEN,'UNKNOWN',NF,1)
        ITER=0
        IOFF=0
        FMAXO=-10.
        WRITE(NOUT,1001)
1001    FORMAT(1X, 
     $  'CCF VALUE   ALPHA       BETA        GAMMA',
     $ '       XSHIFT     ','       YSHIFT     ','       ZSHIFT     ',
     $ 'ITERATION')
100     ITER=ITER+1

        WRITE(NOUT,1002)ALPHC,BETC,GAMC,SX0,SY0,SZ0

        DO I=1,NCCF
C3       SUM(I)=0.
        BUFIMG(I)=0.
	ENDDO

C	ML;
C	IN ALI3D.F VOL(NSAM,NROW,NSLIC) & 
C		   SUM(NPX,NPY,NPZ,NGAMMA,NBETA,NALPHA)

C       CALL ALI3D(NOUT,LUN1,LUN2,VOL,SUM,BUF,NALPHA,NBETA,
        CALL ALI3D(NOUT,LUN1,LUN2,BUFIMG(P1),BUFIMG,BUF,NALPHA,NBETA,
     $  NGAMMA,NPX,
     $  NPY,NPZ,DALP,DBET,DGAM,DX,DY,DZ,ALPHC,BETC,GAMC,SX0,SY0,SZ0,
     $  NSAM1,NROW1,NSLIC1,EULER,FMAX,ALMAX,BEMAX,
     $  GAMAX,XMAX,YMAX,ZMAX,RADIUS,RADIN,DEBUG,ISAMPLE,LINK)

	IF(NCCF.LT.(NROWO*NSAMO)) THEN
	   KK=NROWO*NSAMO
	   WRITE(NOUT,44) NCCF,KK
44         FORMAT(' BOUNDARY CHECK: OUTSIDE OF SUM ARRAY' /
     $             ,' SIZE: ',I4,' READING ENTRY AT:',I4)
           RETURN
	END IF
        DO  I=1,NROWO
           LINE=I+IOFF
           KK=(I-1)*NSAMO+1
C4         CALL WRTLIN(LUN3,SUM(KK),NSAMO,LINE)
           CALL WRTLIN(LUN3,BUFIMG(KK),NSAMO,LINE)
	ENDDO
c 4       CONTINUE

        IOFF=IOFF+NROWO
        WRITE(NOUT,1000) FMAX,ALMAX,BEMAX,GAMAX,XMAX,YMAX,ZMAX,ITER
1000    FORMAT(' ',7(E10.4,2X),I5)
C        WRITE(NDAT,1004) (SUM(I),I=1,NCOEFF)
C        WRITE(NDAT,1004) (BUFIMG(I),I=1,NCOEFF)
1004    FORMAT(1X,'SUM:',10(E10.4,1X))
        IF(ITER.EQ.NITER) GOTO 101
        IF(FMAX.LT.FMAXO) GOTO 101
        ALPHC=ALMAX
        BETC=BEMAX
        GAMC=GAMAX
        SX0=XMAX
        SY0=YMAX
        SZ0=ZMAX
        WRITE(NOUT,1002)ALPHC,BETC,GAMC,SX0,SY0,SZ0
1002    FORMAT(1X,'NEW START AT:',6(E10.4,2X))
        GOTO 100
101     CLOSE (LUN1)
        CLOSE (LUN2)
        CLOSE (LUN3)
        WRITE(NOUT,1003)ALMAX,BEMAX,GAMAX,XMAX,YMAX,ZMAX,FMAX,
     $  RADIUS,RADIN
1003    FORMAT(/1X,'SOLUTION:'/,' ALPHA=',F10.4,' BETA=',F10.4,
     $  ' GAMMA=',F10.4,/,' X-SHIFT=',F10.4,' Y-SHIFT=',F10.4,
     $  ' Z-SHIFT=',F10.4,/,' CCF-VALUE:',E10.4,' OUTER/INNER RADII:',
     $  2F12.4)
        WRITE(NOUT,*) 
     $'THANK YOU FOR YOUR PATIENCE'
        IF(NSEL(1).EQ.0) GOTO 2000
        PARAM(NSEL(1))=ALMAX
        IF(NSEL(2).EQ.0) GOTO 2000
        PARAM(NSEL(2))=BEMAX
        IF(NSEL(3).EQ.0) GOTO 2000
        PARAM(NSEL(3))=GAMAX
        IF(NSEL(4).EQ.0) GOTO 2000
        PARAM(NSEL(4))=XMAX
        IF(NSEL(5).EQ.0) GOTO 2000
        PARAM(NSEL(5))=YMAX
        IF(NSEL(6).EQ.0) GOTO 2000
        PARAM(NSEL(6))=ZMAX
        IF(NSEL(7).EQ.0) GOTO 2000
        PARAM(NSEL(7))=FMAX
2000    RETURN
        END
