
        SUBROUTINE ALI3D(NOUT,LUN1,LUN2,VOL,SUM,BUF
     $  ,NALPHA,NBETA,NGAMMA,NPX,
     $  NPY,NPZ,DALPHI,DBETI,DGAMI,DX,DY,DZ,ALPHC,BETC,GAMC,SX0,SY0,SZ0,
     $  NSAM,NROW,NSLIC,EU,FMAX,ALMAX,BEMAX,
     $  GAMAX,XMAX,YMAX,ZMAX,RAD,RADIN,DFLAG,ISAMPLE,LINK)

C========================================================================
C THREE-DIMENSIONAL ROTATIONAL AND TRANSLATIONAL ALIGNMENT. 
C                                                     8/91 M.Radermacher 
C
C
C        NOUT         output logical unit number (note that cmblock is not 
c                      included)
c        LUN1,LUN2,   logical unit numbers for input,reference,ccf-file
c        LUN3 (not used ml 2/2/95)
c        VOL          array receiving input volume (reference read line by
c                      line)
c        SUM           array for non-normalized ccf-coefficients
c        BUF           buffer
c        NALPHA,NBETA,NGAMMA number of angle values to be tested
c        NPX,NPY,NPZ   number of positions in x,y,z to be tested
c        DALPHI,DBETI,DGAMI angular intervals
c        DX,DY,DZ      positional intervals
c        ALPHC,BETC,GAMC  center angles around which to vary
c        SX0,SY0,SZ0   center shifts around which to vary
c        NSAM,NROW,NSLIC volume dimensions
c        EU            euler rotation matrix
c        FMAX          maximum nonnormalized ccf value
c        ALMAX,BEMAX,GAMAX  angles at FMAX
c        XMAX,YMAX,ZMAX shifts at FMAX
c        RAD,RADIN      outer/inner radius of spherical mask
c        DFLAG          flag, 1 = some debug output, 2=lots of debug output
c        ISAMPLE        skipping factor for undersampling
c=========================================================================


C       INCLUDE 'COMMON1:CMBLOCK.FOR'
        DIMENSION SUM(NPX,NPY,NPZ,NGAMMA,NBETA,NALPHA)
        DIMENSION BUF(1024),EU(3,3)
        DIMENSION VOL(NSAM,NROW,NSLIC)
        LOGICAL LINK
        IF(DFLAG.GE.1) WRITE(NOUT,100) 
     $  NALPHA,NBETA,NGAMMA,NPX,
     $  NPY,NPZ,DALPHI,DBETI,DGAMI,DX,DY,DZ,ALPHC,BETC,GAMC,SX0,
     $  SY0,SZ0,NSAM,NROW,NSLIC,FMAX,ALMAX,BEMAX,
     $  GAMAX,XMAX,YMAX,ZMAX,RAD
100     FORMAT(1X,6I6,/1X,12E10.4,/1X,3I6,/1X,8E10.4)
        RADINLIM=RADIN*RADIN
        IOUTS=0
        ANG=-1.
        PI=ACOS(ANG)
        DEG=180./PI
        RD=PI/180.
        ALPHC=ALPHC*RD
        BETC=BETC*RD
        GAMC=GAMC*RD
        DALPH=DALPHI*RD
        DBET=DBETI*RD
        DGAM=DGAMI*RD
C READ FIRST VOLUME:
        DO  I=1,NSLIC
        LIN=(I-1)*NROW
        DO  K=1,NROW
        LINE=LIN+K
        CALL REDLIN(LUN1,VOL(1,K,I),NSAM,LINE)
	ENDDO
	ENDDO
c 101     CONTINUE

C READ THE SECOND VOLUME DURING THE CALCULATION (JUST ONCE)
C GET SHIFT AND ANGULAR LOOPS LIMITS (NOTE THAT 0 IS ONE OF THE POINTS)
        NX=NPX/2
        NY=NPY/2
        NZ=NPZ/2
        NALPH=NALPHA/2
        NBET=NBETA/2
        NGAM=NGAMMA/2 
C GET CENTER:
        NXCENT=NSAM/2+1
        NYCENT=NROW/2+1
        NZCENT=NSLIC/2+1
C GET LOOP LIMITS:
        ISLICL=NZCENT-RAD
        ISLICH=NZCENT+RAD
        DO  I=ISLICL,ISLICH,ISAMPLE
        Z=FLOAT(I-NZCENT)
        RZ=Z*Z
        IPLANE=(I-1)*NROW
        YRAD=SQRT(RAD*RAD-Z*Z)
        IYRAD=INT(YRAD+0.5)
        IYL=NYCENT-IYRAD
        IYH=NYCENT+IYRAD
        DO J=IYL,IYH,ISAMPLE
        Y=FLOAT(J-NYCENT)
        RYZ=RZ+Y*Y
        XRAD=SQRT(IYRAD*IYRAD-Y*Y)
        IXRAD=INT(XRAD+0.5)
        IXL=NXCENT-IXRAD
        IXH=NXCENT+IXRAD
        IROW=IPLANE+J
        CALL REDLIN(LUN2,BUF,NSAM,IROW)
        IF (DFLAG.GE.1) WRITE(NOUT,502) (BUF(LL),LL=1,NSAM)
502     FORMAT(' BUF:',10(E10.4,2X))
C THE LOOP OVER THE LINE WILL BE A LOT LATER.
        IF (DFLAG.GT.1.5) WRITE(NOUT,*) L,ALPH*DEG,CALPH,
     $      SALPH,M,BET*DEG,CBET,SBET,N,GAM*DEG,CGAM,SGAM,EU
        DO  L=-NALPH,NALPH
        LC=L+NALPH+1
        ALPH=ALPHC+FLOAT(L)*DALPH
        CALPH=COS(ALPH)
        SALPH=SIN(ALPH)
       
        DO  M=-NBET,NBET
        MC=M+NBET+1
        BET=BETC+FLOAT(M)*DBET
        CBET=COS(BET)
        SBET=SIN(BET)

        DO  N=-NGAM,NGAM
        NC=N+NGAM+1
        GAM=GAMC+FLOAT(N)*DGAM
        IF(LINK) GAM=-ALPH
        CGAM=COS(GAM)
        SGAM=SIN(GAM)
       
        EU(1,1)=CGAM*CBET*CALPH-SGAM*SALPH
        EU(2,1)=CALPH*SGAM+CGAM*CBET*SALPH
        EU(3,1)=-CGAM*SBET

        EU(1,2)=-CBET*CALPH*SGAM-CGAM*SALPH
        EU(2,2)=CGAM*CALPH-CBET*SGAM*SALPH
        EU(3,2)= SGAM*SBET

        EU(1,3)=CALPH*SBET
        EU(2,3)=SBET*SALPH
        EU(3,3)=CBET

        IF(DFLAG.GT.1.5) WRITE(NOUT,505) L,ALPH*DEG,CALPH,SALPH,M,
     $     BET*DEG,CBET,SBET,N,GAM*DEG,CGAM,SGAM
        IF(DFLAG.GT.1.5) WRITE(NOUT,508)EU
505     FORMAT(' ',I6,3(F10.4,2X),I6,3(F10.4,2X),I6,3(F10.4,2X)) 
508     FORMAT(3(1X,F10.4,2X))
        DO 3 K=IXL,IXH,ISAMPLE
        X=FLOAT(K-NXCENT)
        RR=X*X+RYZ
        IF(RR.LT.RADINLIM) GOTO 3

        XR=EU(1,1)*X+EU(2,1)*Y+EU(3,1)*Z
        YR=EU(1,2)*X+EU(2,2)*Y+EU(3,2)*Z
        ZR=EU(1,3)*X+EU(2,3)*Y+EU(3,3)*Z

        IF(DFLAG.GE.2) WRITE(NOUT,600) X,Y,Z,XR,YR,ZR
600     FORMAT(' XYZ: ',3(F10.4,1X),' XR,YR,ZR: ',3(F10.4,1X))

        DO 21 IZ=-NZ,NZ
        IZC=IZ+NZ+1
        ZS=ZR+FLOAT(IZ)*DZ+SZ0+NZCENT
        IZS=INT(ZS)
         IF(IZS.LE.0.OR.IZS.GE.NSLIC) THEN
           IOUTS=IOUTS+1
           GOTO 21
         ENDIF
        DELTZ=ZS-IZS
        DO 22 IY=-NY,NY
        IYC=IY+NY+1
        YS=YR+FLOAT(IY)*DY+SY0+NYCENT
        IYS=INT(YS)
         IF(IYS.LE.0.OR.IYS.GE.NROW) THEN
           IOUTS=IOUTS+1
           GOTO 22
         ENDIF
        DELTY=YS-IYS
        DO 23 IX=-NX,NX
        IXC=IX+NX+1
        XS=XR+FLOAT(IX)*DX+SX0+NXCENT
        IXS=INT(XS)
         IF(IXS.LE.0.OR.IXS.GE.NSAM) THEN
           IOUTS=IOUTS+1
           GOTO 23
         ENDIF
        DELTX=XS-IXS
C TRILINEAR INTERPOLATION:
        C1=VOL(IXS,IYS,IZS)
        C2=VOL(IXS+1,IYS,IZS)
        C3=VOL(IXS,IYS+1,IZS)
        C4=VOL(IXS+1,IYS+1,IZS)
        C5=VOL(IXS,IYS,IZS+1)
        C6=VOL(IXS+1,IYS,IZS+1)
        C7=VOL(IXS,IYS+1,IZS+1)
        C8=VOL(IXS+1,IYS+1,IZS+1)
        D1=C2*DELTX+C1*(1-DELTX)
        D2=C4*DELTX+C3*(1-DELTX)
        E1=D2*DELTZ+D1*(1-DELTZ)
        D3=C6*DELTX+C5*(1-DELTX)
        D4=C8*DELTX+C7*(1-DELTX)
        E2=D4*DELTZ+D3*(1-DELTZ)
        F1=E2*DELTY+E1*(1-DELTY)
        if(dflag.ge.2) write(nout,507) XS,YS,ZS,IXS,IYS,IZS,IXC,
     $  IYC,IZC,NC,MC,LC
507     format(1x,'s:',3(f10.4,1x),'is:',3(i4,1x),
     $  ' ic:',6(i4,1x))
C
        SUM(IXC,IYC,IZC,NC,MC,LC)=
     $  SUM(IXC,IYC,IZC,NC,MC,LC)+BUF(K)*F1

        IF(DFLAG.GE.2) WRITE(NOUT,503) 
     $     SUM(IXC,IYC,IZC,NC,MC,LC)
     $     ,IPLANE+1,J,K,BUF(K),
     $     C1,C2,C3,C4,C5,C6,C7,C8,D1,D2,D3,D4,E1,E2,F1
503     FORMAT(' ',E10.4,1X,3(I3,1X),E10.4,
     $  /1X,7(F12.4,1X)/1X,8(F12.4,1X))

23      CONTINUE
22      CONTINUE
21      CONTINUE
3       CONTINUE
	ENDDO
c13      CONTINUE
	ENDDO
c12      CONTINUE
	ENDDO
c11      CONTINUE
	ENDDO
c2       CONTINUE
	ENDDO
c1       CONTINUE

        IF(DFLAG.GE.1) WRITE(NOUT,506) SUM 
506     FORMAT(1X,9(2X,E10.4))


C NOW FIND THE MAXIMUM IN SUM
C INITIALIZE:
        FMAX=SUM(1,1,1,1,1,1)
        IXM=1
        IYM=1
        IZM=1
        IGM=1
        IBM=1
        IAM=1
C SEARCH:
        DO L=1,NALPHA
        DO M=1,NBETA
        DO N=1,NGAMMA
        DO IZ=1,NPZ 
        DO IY=1,NPY
        DO IX=1,NPX
        VAL=SUM(IX,IY,IZ,N,M,L)
        IF(VAL.GE.FMAX) THEN
        FMAX=VAL
        IXM=IX
        IYM=IY
        IZM=IZ
        IGM=N
        IBM=M
        IAM=L
	ENDIF
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
        ALMAX=(ALPHC+(IAM-NALPH-1)*DALPH)*DEG
        BEMAX=(BETC+(IBM-NBET-1)*DBET)*DEG
        GAMAX=(GAMC+(IGM-NGAM-1)*DGAM)*DEG
        IF(LINK) GAMAX=-ALMAX
        XMAX=SX0+(IXM-NX-1)*DX
        YMAX=SY0+(IYM-NY-1)*DY
        ZMAX=SZ0+(IZM-NZ-1)*DZ
        WRITE(NOUT,2001) IOUTS
2001    FORMAT(' ALI3D: ',I5,' VALUES OUTSIDE VOLUME')
        RETURN
        END
