
C++************************************************************************
C
C  ALLOCIT.F                                          A. LEITH JUNE 95
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1995, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR       *
C *  LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR    *
C *  USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF     *
C *  THE CENTER FOR LABORATORIES AND RESEARCH   			   *
C **************************************************************************
C
C  ALLOCIT(MEMWANT,RUNTIME,IPOINTER,CALLER,IRTFLG)
C
C  PURPOSE:         RUN-TIME ALLOCATION OF ARRAY SPACE
C                   WORKS ON SGI
C
C  PARAMETERS: 
C         MEMWANT    NUMBER OF 32 BIT VARIABLES TO BE ALLOCATED  (SENT)
C                    IF ZERO AND RUNTIME, POINTER IS FREED!
C         RUNTIME    LOGICAL VARIBLE SHOWING THAT RUNTIME ALLOCATION
C                    IS ACTIVE                              (SENT/RETURNED)
C         IPOINTER   POINTER TO ALLOCATED SPACE                  (RETURNED)
C         CALLER     CHAR. STRING OF NAME OF CALLING PGM.        (SENT)
C         IRTFLG     ERROR RETURN FLAG (0 IS NORMAL)             (RETURNED)
C         
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*********************************************************************
 
        SUBROUTINE ALLOCIT(MEMWANT,RUNTIME,IPOINTER,CALLER,IRTFLG)

        INCLUDE 'CMBLOCK.INC'

C DO NOT COMPILE WITH STATIC
CNO_SAVE

        CHARACTER *(*) CALLER
        LOGICAL        RUNTIME

        DIMENSION DUMMY(1)
        POINTER   (IPOINTER,DUMMY)

        IRTFLG  = 0

C        USE RUN-TIME ARRAY ALLOCATION FOR Q
         IF (MEMWANT .EQ. 0 .AND. RUNTIME .AND. IPOINTER .NE. 0) THEN
C           ARE FREEING ALREADY ALLOCATED MEMORY
            CALL CHKALLOC(2,IPOINTER,IDUM,IRTFLG)
            CALL FREE(IPOINTER)
            RUNTIME = .FALSE. 

         ELSEIF (MEMWANT .GT. 0) THEN
C           WANT TO ALLOCATE NEW MEMORY, CHECK AMOUNT AVAILABLE
            CALL CHKALLOC(0,IDUM,4*MEMWANT,IRTFLG)
            IF (IRTFLG .NE. 0) THEN
               RUNTIME = .FALSE.
               IRTFLG  = 1
               RETURN
            ENDIF
                 
            IPOINTER = MALLOC(4*MEMWANT) 
            IF (IPOINTER .EQ. 0) THEN
C              ALLOCATION FAILED
               ITEMP = MEMWANT * 4
               WRITE(NOUT,90) ITEMP, CALLER 
 90            FORMAT(' *** MEMORY ALLOCATION OF: ',I10,
     &                ' BYTES FAILED IN: ',A)
               RUNTIME = .FALSE.
               IRTFLG  = 1
               RETURN
            ENDIF

C           RECORD AMOUNT STILL AVAILABLE
            CALL CHKALLOC(1,IPOINTER,4*MEMWANT,IRTFLG)
 
C           RUNTIME ALLOCATION SUCCEEDED
            RUNTIME = .TRUE. 
         ELSE
C           NO MEMORY REQUESTED AND .NOT. RUNTIME
            RUNTIME = .FALSE.
         ENDIF 

         RETURN
         END



        SUBROUTINE CHKALLOC(IOP,IPOINTER,MEM,IRTFLG)
 
        INCLUDE 'CMBLOCK.INC'

        LOGICAL        ONSUN,ONUNIX,ONSGI,ONOSF
        COMMON /OPSYS/ ONSUN,ONUNIX,ONSGI,ONOSF,MEMALLOCABLE

C       NUMPTRS IS MAX NUMBER OF CURRENTLY ALLOCATED MEMORY POINTERS
        PARAMETER  (NUMPTRS = 25)
#ifdef SP_NT
        INTEGER  MEMPTR(NUMPTRS)
#else
        INTEGER *8 MEMPTR(NUMPTRS)
#endif
        DIMENSION  MEMALLOCATED(NUMPTRS) 

        DIMENSION DUMMY(1)
        POINTER   (IPOINTER,DUMMY)

#ifdef SP_NT
        INTEGER MEMWANTED,ITEMP,MEMALLOCNOW
#else
        INTEGER * 8 MEMWANTED,ITEMP,MEMALLOCNOW
#endif

        SAVE      MEMALLOCATED,MEMPTR,MEMALLOCNOW

        DATA      MEMALLOCATED/NUMPTRS*0/
        DATA      MEMALLOCNOW/0/

C       SET ERROR RETURN
        IRTFLG = 1

        IF (IOP .EQ. 0) THEN
C          ARE TRYING TO ALLOCATE NEW MEMORY
           MEMWANTED = MEM + MEMALLOCNOW
           ITEMP     = MEMALLOCABLE
           ITEMP     = ITEMP * 1000000
           IF (MEMWANTED .GT. ITEMP) THEN
C             TOO MUCH MEMORY REQUESTED
              WRITE(NOUT,91) MEMWANTED, ITEMP 
 91           FORMAT(' *** TOTAL REQUESTED MEMORY (BYTES): ',I10,
     &               ' EXCEEDS YOUR ALLOCABLE SETTING: ',I10)

              WRITE(NOUT,92)  
 92           FORMAT(' *** (ALLOCABLE SETTING CAN BE INCREASED WITH',
     &               ' "MD", "SET MEM")./')
              RETURN
           ENDIF
           IRTFLG = 0
           RETURN

        ELSEIF (IOP .EQ. 1) THEN
C          HAVE ALLOCATED NEW MEMORY, RECORD IT
           MEMALLOCNOW = MEM + MEMALLOCNOW

           DO I = 1,NUMPTRS
              IF (MEMPTR(I) .EQ. 0) THEN
                MEMPTR(I)       = IPOINTER  
                MEMALLOCATED(I) = MEM
                IRTFLG          = 0
                RETURN
              ENDIF
           ENDDO
           CALL ERRT(101,'NUMPTRS IN ALLOCIT TOO SMALL',NE)
           RETURN

        ELSE
C          ARE FREEING OLD MEMORY
           DO I = 1,NUMPTRS
              IF (MEMPTR(I) .EQ. IPOINTER) THEN
                 MEMALLOCNOW     = MEMALLOCNOW - MEMALLOCATED(I)
                 MEMPTR(I)       = 0  
                 MEMALLOCATED(I) = 0
                 IRTFLG          = 0
                 RETURN
              ENDIF
           ENDDO
           WRITE(NOUT,*)'*** WARNING, ALLOCIT FREEING UNRECORDED MEMORY'
        ENDIF

        RETURN
        END
                     

