
C++*************************************************************************
C
C  BOXX.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C  BOXX(LUN1,LUN2,NSAM,NROW,NSLICE,MAXDIM)
C
C  PARAMETERS:
C        LUN1     LOGICAL UNIT NUMBER OF FILE
C        LUN2     LOGICAL UNIT NUMBER OF FILE
C        NSAM     NUMBER OF SAMPLES
C        NROW     NUMBER OF ROWS
C	 NSLICE   NUMBER OF SLICES
C        MAXDIM   MAXIMUM BUFFER SPACE AVAILABLE
C
C        BOXX DOES OPERATIONS INVOLVING  LOCAL BOX
C         MODE 1 > SUBTRACTIVE BOX FILTERING
C         MODE 2 > RETURNS FMIN IF PT.LT.XAVG
C                          FMAX IF PT.GE.XAVG
C         MODE 3 > RETURNS LOCAL AVERAGE FOR POINT
C	  MODE 4 > DIVISIVE CONTRAST CORRECTION
C
C        0         2         3         4         5         6         7     
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*********************************************************************

	SUBROUTINE BOXX(LUN1,LUN2,NSAM,NROW,NSLICE,MAXDIM)
         INCLUDE 'CMBLOCK.INC'
	COMMON X(1)

	INTEGER      DS,DR,DL,A1
        CHARACTER *1 ANS,NULL

        NULL = CHAR(0)

 8877	CALL RDPRMC(ANS,NC,.TRUE.,
     &   '(H)IGH PASS / (L)OW PASS / (T)HRES / (D)IVIS',NULL,IRTFLG)

        MODE = INDEX('HTLD',ANS)

	IF (MODE .LE. 0)  THEN
          CALL ERRT(23,'BOXX',NERROR)
          GOTO  8877
	ENDIF

C       INITIALIZATION
	FMINN = FMIN
	FMAXX = FMAX
	IF (FMIN .GE. FMAX) THEN
           FMINN = 0.0
           FMAXX = 2.0
        ENDIF

 3355	CALL RDPRMI(DS,DR,NLOOP,ILOOP,' LOCAL AREA SIZE  COL,ROWS')
        IF (DS .LE. 0) THEN
           CALL ERRT(31,'BOXX',NERROR)
           GOTO  3355
        ENDIF
        IF (DR .EQ. 0)   DR = DS
 
         
	IF (NSLICE .GT. 1)  THEN
C         FIND SLICE DIMENSION OF BOX         
	  CALL RDPRMI(DL,IDXUTV,NLOOP,ILOOP,' LOCAL AREA SIZE  SLICES')
          IF (DL .EQ. 0)  DL = DS

          WRITE(NOUT,90) DS,DR,DL
 90       FORMAT(' BOX SIZE: ',I3,' x',I3,' x',I3)
        ELSE
          WRITE(NOUT,91) DS,DR
 91       FORMAT(' BOX SIZE: ',I3,' x',I3)
        ENDIF

        IF (MODE .EQ. 4) THEN   
          IF(IMAMI.NE.1) CALL NORM3(LUN1,NSAM,NROW,NSLICE,FMAX,FMIN,AV)
          AV1 = AV * 0.1
        ENDIF

        CALL RDPRM(F,NLOOP,ILOOP,'FILTER WEIGHT (0.0->1.0)')
        FC = 1.0 - F

	IF (NSLICE .LE. 1)  THEN
C         2-D
C----------------------------------------------------------------------
	  A1 = DR * NSAM
          IF (A1 + NSAM .GT. MAXDIM) THEN
C           INSUFFICIENT BUFFER SPACE
            CALL ERRT(6,'BOXX',NERROR)
            RETURN
          ENDIF

          KT     = 1
          KR     = 1
          MOVWAY = 2
          I      = -1

	  DO J=1,NROW
           JOFF = MOD(J-1,DR)*NSAM
           IF (MOVWAY .EQ. 3) KT=NSAM
C          *** START OFF OR GO DOWN ***
           GO TO(100,200,300,400),MODE

c+++         moved labels 100,200,300,400 out of do loop 10
c	     just in case mode can be > 4, add goto 5
	     GOTO  5 
	     
C            *** MODE 1 EQUATIONS ***
100	     CALL AVERG(LUN1,XAVG,J,KT,NSAM,NROW,DS,DR,KR)
	     POINT=X(JOFF+KT)
	     X(A1+KT)=F*(POINT-XAVG)+POINT*FC
             GO TO 5

C            *** MODE 2 EQUATIONS ***
200          CALL AVERG(LUN1,XAVG,J,KT,NSAM,NROW,DS,DR,KR)
             POINT=X(JOFF+KT)
             X(A1+KT)=F*FMINN+FC*POINT
             IF(POINT.GE.XAVG)X(A1+KT)=F*FMAXX+FC*POINT
             GO TO 5

C            *** MODE 3 EQUATIONS ***
300          CALL AVERG(LUN1,XAVG,J,KT,NSAM,NROW,DS,DR,KR)
             X(A1+KT)=XAVG*F + FC*X(JOFF+KT)
             GO TO 5
	     
C            *** MODE 4 EQUATIONS ***
400          CALL AVERG(LUN1,XAVG,J,KT,NSAM,NROW,DS,DR,KR)
             POINT=X(JOFF+KT)
             X(A1+KT)=F*POINT/(XAVG+AV1)+POINT*FC

5	   DO 10 K=2,NSAM
             KT=K
             IF (MOVWAY.EQ.3)KT=NSAM+1-K

C            *** MOVE RIGHT OR LEFT ***
	     GO TO (150,250,350,450),MODE
	     
c+++         moved labels 100,200,300,400 out of do loop 10
c            just in case mode > 4, add goto 100
	     GOTO 100
	     
150	     CALL AVERG(LUN1,XAVG,J,KT,NSAM,NROW,DS,DR,MOVWAY)
             POINT=X(JOFF+KT)
             X(A1+KT)=F*(POINT-XAVG)+POINT*FC
             GO TO 10

250          CALL AVERG(LUN1,XAVG,J,KT,NSAM,NROW,DS,DR,MOVWAY)
             POINT=X(JOFF+KT)
             X(A1+KT)=F*FMINN+FC*POINT
             IF (POINT .GE. XAVG) X(A1+KT)=F*FMAXX+FC*POINT
             GO TO 10

350          CALL AVERG(LUN1,XAVG,J,KT,NSAM,NROW,DS,DR,MOVWAY)
             X(A1+KT)=XAVG
             GO TO 10

450          CALL AVERG(LUN1,XAVG,J,KT,NSAM,NROW,DS,DR,MOVWAY)
             POINT=X(JOFF+KT)
             X(A1+KT)=F*POINT/(XAVG+AV1)+POINT*FC
10        CONTINUE
          I = -I
          MOVWAY = MOVWAY+I
          KT = 1
          CALL WRTLIN(LUN2,X(A1+1),NSAM,J)
          KR = 4
	ENDDO

        IF (MODE .EQ. 2) CALL SETPRM(LUN2,NSAM,NROW,2.,0.,1.,'R')
30      CONTINUE

C       3D
C-------------------------------------------------------------------------
	ELSE

        A1 = DL*NSAM*NROW
        IF (A1+NSAM .GT. MAXDIM) THEN  
C           INSUFFICIENT BUFFER SPACE
            CALL ERRT(6,'BOXX',NERROR)
            RETURN
        ENDIF

	I      = -1
	MOVWAY = 2
	IS     = -1
	MOVSID = 4
	KR     = 1

	DO    L=1,NSLICE
           LOFF=MOD(L-1,DL)*NSAM*NROW
           DO    J=1,NROW
             IF (MOVWAY.EQ.3)  THEN
                KT=NSAM
             ELSE
                KT=1
             ENDIF
             IF (MOVSID .EQ. 5) THEN
               KJ=NROW+1-J
             ELSE
               KJ=J
             ENDIF
             IF (J .EQ. 1)  THEN
                GOTO  (1008,2008,3008,4008),MODE
             ELSE
                GOTO  (1009,2009,3009,4009),MODE
             ENDIF
	     
C	     moved 1008,2008,3008,4008 out of do loop 810;
C            just in case add goto 85.
             GOTO 85
	       
 1008	       CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                   DS,DR,DL,KR)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(NO)=F*(POINT-XAVG)+POINT*FC
               GOTO  85

 2008          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                     DS,DR,DL,KR)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(NO)=F*FMINN+FC*POINT
               IF (POINT .GE. XAVG) X(NO)=F*FMAXX+FC*POINT
               GOTO  85

 3008          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                     DS,DR,DL,KR)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(NO)=XAVG*F + FC*POINT
               GOTO  85

 4008          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                     DS,DR,DL,KR)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(A1+KT)=F*POINT/(XAVG+AV1)+POINT*FC
               GOTO  85

 1009          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                     DS,DR,DL,MOVSID)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(NO)=F*(POINT-XAVG)+POINT*FC
               GOTO  85

 2009          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                    DS,DR,DL,MOVSID)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(NO)=F*FMINN+FC*POINT
               IF(POINT.GE.XAVG)X(NO)=F*FMAXX+FC*POINT
               GOTO  85

 3009          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                     DS,DR,DL,MOVSID)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(NO)=XAVG*F + FC*POINT
               GOTO 85

 4009          CALL  AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                      DS,DR,DL,MOVSID)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(A1+KT)=F*POINT/(XAVG+AV1)+POINT*FC
               GOTO  85
	     
 85          DO  810  K=2,NSAM
               IF(MOVWAY.EQ.3)  THEN
                 KT=NSAM+1-K	
               ELSE
                 KT=K
               ENDIF
               GOTO  (1508,2508,3508,4508),MODE

C	       moved 1008,2008,3008,4008 out of do loop 810;
c	       mode > 5 ? add goto 1008 just in case.
	       GOTO 1008

 1508          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                     DS,DR,DL,MOVWAY)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(NO) = F*(POINT-XAVG)+POINT*FC
               GOTO  810

 2508          CALL  AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                      DS,DR,DL,MOVWAY)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(NO)=F*FMINN+FC*POINT
               IF(POINT.GE.XAVG)X(NO)=F*FMAXX+FC*POINT
               GOTO  810


 3508          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                     DS,DR,DL,MOVWAY)
               NO=A1+KT
               X(NO)=XAVG
               GOTO  810
 4508          CALL AVERG3(LUN1,XAVG,KJ,KT,L,NSAM,NROW,NSLICE,
     &                     DS,DR,DL,MOVWAY)
               NI=LOFF+(KJ-1)*NSAM+KT
               POINT=X(NI)
               NO=A1+KT
               X(A1+KT)=F*POINT/(XAVG+AV1)+POINT*FC
 810         CONTINUE
             I      = -I
             MOVWAY = MOVWAY+I
             NLIN   = (L-1)*NROW+KJ
             CALL  WRTLIN(LUN2,X(A1+1),NSAM,NLIN)
	ENDDO

           IS     = -IS
           MOVSID = MOVSID+IS
           KR     = 6
 	ENDDO

	ENDIF

	END

