
C**************************************************************************
C
C    CNGOUL.FOR  -- CREATED JULY 85
C                   ALTERED JAN 88
C **************************************************************************
C *  AUTHOR: ARDEAN LEITH                                                  *
C *  COPYRIGHT (C)1985,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C $$ CNGOUL (FILNAM,FILEXT,LUN,ISEL,SUBTAS,TOFILE,IRTFLG)
C
C    PURPOSE:  TO OPEN THE CONTOUR STORAGE FILE AND READ THE CONTOUR AND 
C              TEXT DATA FROM THE FILE AND PLOT THEM ON THE GOULD PLOTTER
C
C    PARAMETERS:
C           FILNAM   THE NAME OF THE CONTOUR STORAGE FILE IN CHARACTER FORMAT.
C
C           FILEXT   THE EXTENSION OF THE CONTOUR STORAGE FILE (CHARACTER * 3)
C
C           LUN      UNIT NO. USED TO READ THIS FILE
C
C           ISEL     FLAG FOR SELECTION OF VIEWPORT SIZE
C                    = 1    ASK USER FOR THE VIEWPORT SIZE
C                    = 0    USE DEFAULT HARDWARE VIEWPORT SIZE
C
C           SUBTAS   LOGICAL FLAG THAT CALL IS FROM SUBTASK SO IT CAN NOT
C                    CREATE ANOTHER SUBTASK TO CHANGE SPEED OF PLOTTER
C
C           TOFILE   LOGICAL FLAG FOR OUTPUT TO FILE
C
C           IRTFLG   ERROR RETURN FLAG (0 = NO ERROR, 1 = ERROR IN PLOT)
C
C       CALLED BY:   PLOT1 (SPIDER)     CNGRAF
C
C       CALLS:       GOULDR     RDPRMC   RDPRM2S   CSVMS
C
C--********************************************************************

      SUBROUTINE CNGOUL(FILNAM,FILEXT,LUN,ISEL,SUBTAS,TOFILE,IRTFLG)

      INCLUDE 'CONSTG.INC'
      COMMON /UNITS/LUNDOC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
      COMMON /BATCH/IBCNT,NLOOP,IABSLP,ILOOP

      COMMON  DATA
      
        PARAMETER      (NSIZE=2000)
        DIMENSION      DATA(3,NSIZE)
        CHARACTER *(*) FILNAM,FILEXT
        CHARACTER * 80 LINE,PLFILE
        CHARACTER * 3  DISP
        CHARACTER * 1  NULL,ANSW
        LOGICAL        MIRROR,SUBTAS,TOFILE,ERRI2
        LOGICAL        TEXT,DEBUG,TITLE,WINDOW
        CHARACTER * 4  CDATA
        EQUIVALENCE    (CDATA,FDATA)

	NULL=CHAR(0)

        IRTFLG = 0
        IXSIZE = 14000
        IYSIZE = 10000

C..      ASSIGN DEFAULT VIEWPORT SIZE OF 10000 FOR GOULD PLOTTER
C..      (THIS VALUE WILL CHANGE FOR DIFFERENT HARDWARES)
         IF (TOFILE) THEN
            IVPTSZ = 6000
         ELSE
            IVPTSZ = 10000
         ENDIF
         XXTRA  = 900
         YXTRA  = 500

   10    IF (ISEL .EQ. 1) THEN
C..         ASK IF USER WANTS TO CHOOSE THE VIEWPORT SIZE
            WRITE(NOUT,9023) IVPTSZ
9023        FORMAT(' CURRENT PLOT PAGE SIZE: ',I6)
            CALL RDPRIS(IVPTSZ,IDUM,NLOOP,ILOOP,
     &      '.ENTER NEW PAGE SIZE (30 - 10000) OR <CR>',IRTFLG)
            IF (IRTFLG .EQ. -1) RETURN
            IF (ERRI2(IVPTSZ,IDUM,2,30,14000,0,14000)) GOTO 10
         ENDIF

C..      SET DEFAULT WINDOW
20       CONTINUE
         WXMINP = 0.0
         WYMINP = 0.0
         WXMAXP = 1.0
         WYMAXP = 1.0

C..      ASK IF USER WANTS TO WINDOW THE FILE
5678     CALL RDPRMC(ANSW,N,.TRUE.,'.DO YOU WANT TO WINDOW? (N/Y)d',
     &    NULL,IRTFLG)
         IF (IRTFLG .EQ. -1) THEN
            IF (ISEL .EQ. 1) GOTO 10
            RETURN
         ENDIF

         DEBUG = .FALSE.
         IF (N .GT. 0 .AND. (ANSW .EQ. 'n' .OR. ANSW .EQ. 'y'))
     &      DEBUG = .TRUE.

         WINDOW = .FALSE.
         IF (N .GT. 0 .AND. (ANSW .EQ. 'Y' .OR. ANSW .EQ. 'y')) THEN
C..          USER WANTS TO WINDOW SO ASK FOR CO-ORDINATES
             WINDOW = .TRUE.
         ELSE
	     GOTO 4013
         ENDIF
	   
 4002        CALL RDPRM2S(WXMINP,WYMINP,NLOOP,ILOOP,
     &       '.ENTER X & Y CO-ORD. OF BOTTOM LEFT CORNER (0.0 - 1.0)',
     &        IRTFLG)
             IF (IRTFLG .EQ. -1) GOTO 20

 4003        CALL RDPRM2S(WXMAXP,WYMAXP,NLOOP,ILOOP,
     &       '.ENTER X & Y CO-ORD. OF TOP RIGHT CORNER (0.0 - 1.0)',
     &        IRTFLG)
             IF (IRTFLG .EQ. -1) GOTO 4002
	     
4013     CONTINUE

C..      UPDATE THE SCALING BECAUSE OF WINDOW
         YDIFF = WYMAXP - WYMINP
         XDIFF = WXMAXP - WXMINP
         IF (XDIFF .GT. YDIFF) THEN
            DIFF = XDIFF
         ELSE
            DIFF = YDIFF
         ENDIF
         PLSCL = 0.95 * IVPTSZ / DIFF

4004     WRITE(NOUT,9024)PLSCL
9024     FORMAT(' CURRENT PLOTTING SCALE: ',F8.3)
         CALL RDPRM2S(PLSCL,FDUM,NLOOP,ILOOP,
     &   'ENTER NEW PLOTTING SCALE OR <CR>',IRTFLG)
         IF (IRTFLG .EQ. -1) THEN
            IF (WINDOW) GOTO 4003
            GOTO 20
         ENDIF

C        FIND CLIP WINDOW
         WXMIN  = WXMINP * PLSCL + XXTRA
         WYMIN  = WYMINP * PLSCL + YXTRA
         WXMAX  = WXMAXP * PLSCL + XXTRA
         WYMAX  = WYMAXP * PLSCL + YXTRA

C        DO NOT ALLOW CLIP WINDOW TO GO OFF THE PAGE
         IF (WXMAX .GT. IXSIZE) THEN
            WXMAX = IXSIZE
         ELSE IF (WYMAX .GT. IYSIZE) THEN
            WYMAX = IYSIZE
         ENDIF

         MIRROR = .FALSE.
         PLSCLY = PLSCL
 4343    CALL RDPRMC(ANSW,N,.TRUE.,'.PRINT TEXT? (Y/N/M)',NULL,IRTFLG)
         IF (IRTFLG .EQ. -1) GOTO 4004
         IF (N .GT. 0 .AND. (ANSW .EQ. 'M' .OR. ANSW .EQ. 'm')) THEN 
            MIRROR = .NOT. MIRROR
            IF (MIRROR) THEN
              WRITE(NOUT,*) ' Y COORDINATES MIRRORED AROUND X AXIS'
              PLSCLY = -PLSCL
              YXTRA = YXTRA + PLSCL
            ELSE
              WRITE(NOUT,*) ' Y COORDINATES NOT MIRRORED AROUND X AXIS'
            ENDIF
            GOTO 4343
         ELSE
            TEXT = .TRUE.
            IF (N .GT. 0 .AND. (ANSW .EQ. 'N' .OR. ANSW .EQ. 'n')) 
     &         TEXT = .FALSE.
         ENDIF

         IF (TOFILE) THEN
C          GET FILE NAME FOR PLOT OUTPUT
           IPLUN = 17
           DISP = 'UNK'
           CALL AUXFIL(PLFILE,NLET,
     &     'ENTER PLOTTER FILE NAME$',DISP,FILEXT,IPLUN,IRTFLG)
           IF (IRTFLG .EQ. -1) GOTO 4343
 
C          solve a goto 4500 inside if then loop. add goto 6000.          
	   GOTO 6000
	 END IF
C         ELSE
C..        ASSIGN UNIT NO. TO THE PLOTTING HARDWARE
           IPLUN = NOUT
        
           WRITE(IPLUN,124)
  124      FORMAT(/,' LOAD PENS 1 THROUGH 10. USUAL COLORS ARE:',/,
     &     ' BLACK, RED, GREEN, BLUE, YELLOW, MAGENTA'/
     &     ' TURQUOISE, BROWN, ORANGE, AQUA'/)

 4500      WRITE(IPLUN,125)
  125      FORMAT(' Press RESET, CHART LOAD, and PAUSE on plotter.'
     &       ,/,' Note: To abort plot, terminal speed must be 2400.')

           IF (SUBTAS) WRITE(NOUT,9056)
9056       FORMAT('0','PORT SPEED MUST BE SET TO 2400'/)
4006       CALL RDPRMC(ANSW,N,.TRUE.,'.OK TO PLOT NOW? (N/Y)',
     &     NULL,IRTFLG)
           IF (IRTFLG .EQ. -1) GOTO 4343

           IF (N .GT. 0 .AND. (ANSW .NE. 'Y' .AND. ANSW .NE. 'y')) 
     &        GOTO 4006

           IF (.NOT. SUBTAS) THEN
C            NOT CALLED FROM SUBTASK, CAN USE CSVMS
             LINE(1:20) = 'SET TERM/SPEED=2400'//NULL
             CALL CSVMS(LINE,IERR)
           ENDIF
C         ENDIF

6000      CONTINUE

C..      INITIALIZE THE PLOTTER
         CALL GDSTRT(IPLUN)
         CALL GDGO(IPLUN)

C        SET WINDOW 
         CALL GDWIND(IPLUN,WXMIN,WYMIN,WXMAX,WYMAX)

         LASPEN = -1
       
C..      GET CONTOUR DATA AND PLOT THEM
         DO 1001 I = 1, LSTCON
C..         HANDSHAKE ADDED
            IF (.NOT. TOFILE) CALL GDHAND(IPLUN)

            IF (CONTRL(1,I) .LE. 0.0) GOTO 1001
            ICNTR = I
            CALL CSGET(ICNTR,NSIZE,DATA,NDATA,LUN,IRTFLG)
            IF (NDATA .LE. 0 .OR. IRTFLG .NE. 0) GOTO 1001

C..         SET PEN NUMBER (COLOR)
            IF (IPEN .NE. LASPEN) CALL GDPEN(IPLUN,IPEN)
            LASPEN = IPEN
C..         SET LINE TYPE
            CALL GDLINE(IPLUN,LINTP)

            IF (IFRMCS .EQ. -6 .AND. MARK .EQ. -2) THEN
C               NEW STYLE TEXT CONTOUR
                XM = DATA(1,1) * PLSCL  + XXTRA
                YM = DATA(2,1) * PLSCLY + YXTRA
                NCHAR = DATA(1,2)
                FDATA = DATA(2,2)
                CALL GDTEX(IPLUN,CDATA,NCHAR,XM,YM,ITSIZE,
     &                       ITANGL,JUST)

            ELSEIF (IFRMCS .EQ. 6 .AND. MARK .EQ. 2) THEN
C..            THIS IS AN OLD STYLE TEXT CONTOUR
               ITANGL = 0
               ITSIZE = 20

               DO  II = 1, NDATA
                  XM = DATA(1,II) * PLSCL + XXTRA
                  YM = DATA(2,II) * PLSCLY + YXTRA
                  CALL GDTEX(IPLUN,CSYMBL(1:1),1,XM,YM,ITSIZE,
     &                 ITANGL,JUST)
	       ENDDO

            ELSE 
C..            NORMAL CONTOUR
               XM = DATA(1,1) * PLSCL + XXTRA
               YM = DATA(2,1) * PLSCLY + YXTRA
               CALL GDMOVE(IPLUN,XM,YM)
               CALL GDDOWN(IPLUN)
               DO  II = 2, NDATA
                 XM = DATA(1,II) * PLSCL + XXTRA
                 YM = DATA(2,II) * PLSCLY + YXTRA
                 CALL GDDRAW(IPLUN,XM,YM)
	       ENDDO
          ENDIF
 1001    CONTINUE


C..      GO FROM 1 TO LSTTXT & PLOT NON-DELETED TEXTS 
         DO 1003 I = 1, LSTTXT
            IF (TXTCTR(1,I) .LT. 0.0) GOTO 1003
               ITEXT = I
               CALL CSGTTX(LINE,NCHAR,LUN,IRTFLG,ITEXT)
               IF (NCHAR .GT. 0) THEN
                  XM = TXTCTR(4,I) * PLSCL + XXTRA
                  YM = TXTCTR(5,I) * PLSCLY + YXTRA
                  IF (IPEN .NE. LASPEN) CALL GDPEN(IPLUN,IPEN)
                  LASPEN = IPEN
                  CALL GDFONT(IPLUN,IFONT)
                  CALL GDTEX(IPLUN,LINE,NCHAR,XM,YM,ITSIZE,ITANGL,JUST)   
               ENDIF
 1003    CONTINUE

C        SET WINDOW TO FULL PAGE
         CALL GDWIND(IPLUN,-1000.0,-1000.0,32000.0,32000.0)

         TITLE = .TRUE.

         IF (TITLE .AND. TITLCS(1:18) .NE. '                  ' .AND.
     &       TITLCS(1:1) .NE. NULL) THEN
C..         WRITE THE TITLE AND DATES
            LINE(1:48) = 'TITLE: '//TITLCS
            X1 = 30
            Y1 = 30
            NCHAR = 48
            CALL GDMOVE(IPLUN,X1,Y1)
            CALL GDDOWN(IPLUN)
            IPEN = 0
            IF (IPEN .NE. LASPEN) CALL GDPEN(IPLUN,IPEN)
            LASPEN = IPEN
            CALL GDTEX(IPLUN,LINE,NCHAR,XI,YI,ITSIZE,ITANGL,JUST)
         ENDIF

C..      TURN OFF THE PLOTTER
         CALL GDPEN(IPLUN,0)
         LASPEN = -1
         CALL GDEND(IPLUN)

         IF (TOFILE) THEN
           CLOSE(IPLUN)
         ELSE
           LINE(1:20)= 'SET TERM/SPEED=9600'//NULL
           IF (.NOT. SUBTAS) CALL CSVMS(LINE,IERR)
4544       WRITE(NOUT,122)
  122      FORMAT(' Plot finished. Port speed has been reset.')
         ENDIF

         CALL RDPRMC(ANSW,N,.TRUE.,'.PLOT AGAIN? (N/Y)',NULL,IRTFLG)
         IF (ANSW .EQ. 'Y') GOTO 4500

      RETURN
      END         

