
C**************************************************************************
C
C   CNINT3.FOR  -- CREATED MAY 87
C
C **************************************************************************
C *  AUTHOR: ArDean Leith 
C *  COPYRIGHT (C)1987 , WADSWORTH CENTER FOR LABORATORIES AND RESEARCH    *
C *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C      CNINT3(LUNCS,DISPL)
C
C      PURPOSE:     READS  MULTIPLE SPIDER PICTURE FILES, EXTRACTS CONTOURS, 
C                   AND TRANSFERS THEM TO A CNT META-FILE
C
C      PARAMETERS:  UNUSED
C                   MAXDIM  = COMMON BUFFER SIZE
C
C      CALLED BY:   PLOT1
C
C      CALLS:       REDLIN     CNTUR   CSSCALT                
C
C      CALL TREE:   PLOT1..CNINT3..CNTUR..CNSCAN..CNTRCE..CSPUSH
C                        ..CNINT                ..CNCALC
C                      
C        0         2         3         4         5         6         7
C23456789012345678901234567890123456789012345678901234567890123456789012
C--********************************************************************

       SUBROUTINE CNINT3(LUNCS,UNUSED,MAXDIM)
  
        INCLUDE 'CONSTG.INC'
        INCLUDE 'CMBLOCK.INC'

C-------- START OF EM-PLOTT-COMMON-------------------------------------
C     INTEGERS
      COMMON /CONT1/ ICALL, IDIDIT, IDONE, IDX, IDY, ILINE, INTT,
     &               IRCD, ISS, ISTART, ISUM1, ISUM2, ISUM3, IT, IV, 
     &               IXX1, IXX2, IXX3, IX, IY, JSUM1, JSUM2, JSUM3, JT,
     &               LEVEL, LW, M, MF, MI, MT, N, NDIV, NF, NI, NT, NW

C     FLOATING POINT
      COMMON /CONT2/ APDIV, APDIVX, CV, DL, PY, RA, RC, RS, SKALE, THE,
     &               SX, SY, DENSL

C     ARRAYS
      COMMON /CONT3/ INCX(3), IORGX(3), INX(8),
     &               INY(8),  IPT(3,3), IMAP(12), NG(3), NP(3)

      COMMON /CONT4/ CTRI(6),FCTR(6),CTRDEL(6),ICNDSH(6),ICNCEL

C--------END OF EM-PLOTT-COMMON----------------------------------------

        PARAMETER        (MAXIRR  = 80000)
        PARAMETER        (NSIZE   = 2000)

        COMMON           BUF(MAXIRR),X(NSIZE),Y(NSIZE),AM(1)
        COMMON /SPI_BUF/ DATA(3,NSIZE),WORK(2,NSIZE),NUMBER(200)
        COMMON /POLY/    MINPTS
      
        CHARACTER *81    FILPAT,FILNAM
        CHARACTER * 1    NULL,ANSW

        DATA             LUNIM/11/
	NULL=CHAR(0)

C*      NMAXI IS DIMENSION OF X AND Y ARRAYS
        NMAXI = NSIZE

C       MINIMUM NUMBER OF POINTS SAVED IN CONTOUR FILE / CONTOUR
        MINPTS = 4

C       MAX NUMBER OF POINTS DESIRED ON CONTOUR FILE CONTOUR
        MAXPTS = 800

        ISTFLG = 1

C       GET IMAGE FILE NAMES
4004    WRITE(NOUT,*) 'FOR IMAGE FILE NAMES:'
        NIMAG = 200
        CALL FILSEQP(FILPAT,NLET,NUMBER,200,NIMAG, 
     $  'FILE PREFIX OR TEMPLATE (EG. PIC****)', IRTFLG)
     
        
        DO 1000 IFIL = 1,NIMAG
          CALL FILGET(FILPAT,FILNAM,NLET,NUMBER(IFIL),IERR)
          IF (IERR .NE. 0) GOTO 1000
         
C         OPEN A SPIDER IMAGE TYPE FILE AS INPUT
          MAXIM = 0
          CALL OPFIL(.FALSE.,FILNAM,LUNIM,'O',NSAM,NROW,NSLICE,
     &             MAXIM,' ',.FALSE.,IRTFLG)
          IF (IRTFLG .NE. 0) GOTO 1000

C         ESTABLISH FMIN AND FMAX IF NOT IN COMMON YET
          IF (IMAMI .NE. 1) 
     &       CALL NORM3(LUNIM,NSAM,NROW,NSLICE,FMAX,FMIN,AV)

          MAXPIX = MAXDIM - 3 * NSIZE 
          IF ((NSAM * NROW) .GT. MAXPIX) THEN
            WRITE(NOUT,9945) MAXPIX
 9945       FORMAT(' SORRY, PGM LIMITED TO: ',I8,' PIXELS'/)
            CALL ERRT(100,'CNINT3',NE)
            RETURN
          ENDIF

C         SET X AND Y EXTENT FOR CONTOUR FILE
          XMIN  = 1.0
          YMIN  = 1.0
          XMAX  = NSAM
          YMAX  = NROW

C         PUT IN FRAME AROUND PLOT
          DATA(1,1) = 1.0
          DATA(2,1) = 1.0

          DATA(1,2) = NROW
          DATA(2,2) = 1.0

          DATA(1,3) = NROW
          DATA(2,3) = NSAM

          DATA(1,4) = 1.0
          DATA(2,4) = NSAM

          DATA(1,5) = 1.0
          DATA(2,5) = 1.0

          NDATA     = 5
          CALL CSPUSH(LUNCS,DATA,NDATA,NERR,ISTFLG,ICNTR)

C         READ THE SPIDER FILE INTO AM ARRAY
C         FLIP IS FOR MIRRORING INPUT PRIOR TO CONTOUR STORAGE
          J = NSAM * NROW + 1
          DO  I = 1,NROW
             J = J - NSAM
             CALL REDLIN (LUNIM,AM(J),NSAM,I)
          ENDDO

C         DISPLAY MAX AND MIN VALUE OF PICTURE , ASK FOR THE CONTOUR LEVELS
          WRITE(NOUT,106) FMIN,FMAX
  106     FORMAT(' IMAGE RANGE: ',1PG11.3,'....',1PG11.3)


 1009     CALL RDPRM2S(BLEV,ELEV,NLOOP,ILOOP,
     &        'STARTING AND ENDING CONTOUR LEVELS',IRTFLG)
          IF (IRTFLG .EQ. -1) GOTO 4004

 1010     CALL RDPRM2S(RINC,FDUM,NLOOP,ILOOP,
     &    'CONTOUR LEVEL INCREMENT (USE INCR. > END FOR ONE LEVEL)',
     &     IRTFLG)
          IF (IRTFLG .EQ. -1) GOTO 1009

C         CNTUR EXTRACTS THE CONTOUR FROM THE SPIDER FILE AND PLACES
C         IT IN THE CONTOUR FILE
          CALL CNTUR(AM,NSAM,NROW,BLEV,ELEV,RINC,BUF,X,Y,
     &                NMAXI,LUNCS,.FALSE.,MAXPTS,MAXIRR)

C         SCALE AND WRITE THE CONTROL DATA INTO THE FILE
          CALL CSSCALT(LUNCS,1)
          CALL CSTRAP(LUNCS)

          CLOSE(LUNIM)

1000    CONTINUE

C       CLOSE CONTOUR STORAGE FILE 
        CLOSE(LUNCS)

        RETURN
        END
