
C***************************************************************************
C
C  CNMOV.FOR  -- CREATED JAN 86
C                   
C **************************************************************************
C *  AUTHOR: ArDean Leith                                                  *
C *  COPYRIGHT (C)1986 , WADSWORTH CENTER FOR LABORATORIES AND RESEARCH    *
C *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C CNMOV(LUNCS,DISPL)
C
C      PURPOSE:
C        READS  CNT FILES, EXTRACTS CONTOURS, AND TRANSFERS
C          THEM TO A MOVIE.BYU MOSAIC INPUT FILE
C
C      PARAMETERS:
C          DISPL = FLAG FOR CONTOURS DISPLAY
C
C      CALLED BY:
C
C           PLOT1
C
C      CALLS:
C           CSSCALT             TMEND             CSOPEN
C           RDPRC              TMSTRT            AUXFIL
C           TMCLR              CSGET             
C           SSDISP             
C
C      VARIABLES:
C          MOFLNM = MOVIE FILE NAME
C          MOEXT  = MOVIE FILE EXTENSION
C          DATAT2 = TEMPORARY ARRAY FOR COORD. OUTPUT
C          LUNM   = UNIT NUMBER FOR MOVIE FILE
C          LUNCS  = UNIT NUMBER FOR CONTUR FILE 
C          N     = NUMBER OF POINTS ON CLOSED LOOP
C          NDATA = NUMBER OF POINTS ON CNT FILE LOOP
C          CHECK = FLAG TO INDICATE THAT CONTOURS SHOULD BE
C                  SCREENED BEFORE TRANSFER
C          INVERZ= FLAG FOR INVERTING Z LEVELS
C          CWISE = FLAG FOR INVERTING CONTOUR TO CLOCKWISE
C          MIRROR= FLAG FOR MIRRORING AROUND Y AXIS
C          DISPL = FLAG FOR VT125 DISPLAY DESIRED
C
C--********************************************************************

      SUBROUTINE CNMOV(LUNCS,DISPL)
  
      INCLUDE 'CONSTG.INC'
      INCLUDE 'CMBLOCK.INC'

        COMMON    DATA,DATAT2
       
        PARAMETER (NSIZE  = 2000)
C..     MAXIMUM OF 2000 POINTS / CONTOUR

        DIMENSION DATA(3,NSIZE), DATAT2(3,NSIZE)

        CHARACTER * 81 MOFLNM
        CHARACTER * 80 PROMPT
        CHARACTER * 8  DISP
        CHARACTER * 4  MOEXT
        CHARACTER * 1  NULL,ANSW

        LOGICAL        CHECK,DISPL,CWISE,INVERZ,MIRROR
        LOGICAL        SWITCH

        DATA XFLAG/9.99E33/
	NULL=CHAR(0)
        LUNDOC = 20
        LUNM   = 12
        IRTFLG = 0

 5001   CONTINUE
C..     ASK FOR MOVIE FILE NAME
        MOEXT  = 'MOV'
        DISP   = 'U'
        PROMPT = '.ENTER MOVIE FILE NAME'
        CALL AUXFIL(MOFLNM,NLET,PROMPT,DISP,MOEXT,LUNM,IRTFLG)
        IF (IRTFLG .NE. 0) RETURN

C..     IS CONTOUR TO BE OK'd BEFORE TRANFER?   
 2043   CONTINUE
        DISPL = .FALSE.
        CHECK = .FALSE.
        MIRROR= .TRUE.
        INVERZ= .FALSE.
        WRITE(NOUT,3112)
 3112   FORMAT(' Answer yes if some contours have holes.')
        CALL RDPRMC(ANSW,N,.TRUE.,
     &     'DO YOU WANT TO OK EACH CONTOUR TRANSFER? (Y/N)',NULL,IRT)
        IF (IRT .EQ. -1) GO TO 5001
        IF (ANSW .EQ. 'H' .OR. ANSW .EQ. 'h')THEN
          WRITE(NOUT,*) ' LOWER CASE DOES NOT MIRROR Y AXIS'
          GOTO 2043
        ENDIF

        IF ((ANSW .EQ.'Y') .OR. (ANSW .EQ. 'y')) CHECK  = .TRUE.
        IF ((ANSW .EQ.'y') .OR. (ANSW .EQ. 'n')) THEN
           MIRROR = .FALSE.
           WRITE(NOUT,*) ' HIDDEN NO MIRROR COMMAND SELECTED'
        ENDIF

C..     IS CONTOUR TO BE DISPLAYED?   
C..     USE LOWER CASE TO ACTIVATE HIDDEN COMMAND FOR INVERTING Z LEVELS

 2003   CALL RDPRMC(ANSW,N,.TRUE.,
     &       'DO YOU WANT TO VIEW EACH CONTOUR? (Y/N)',NULL,IRTFLG)
        IF (IRTFLG .EQ. -1) GO TO 2043
        IF ((ANSW .EQ. 'H') .OR. (ANSW .EQ. 'h')) THEN
           WRITE(NOUT,*)  ' LOWER CASE INVERTS Z STACK'
           GOTO 2003
        ENDIF
        IF ((ANSW .EQ. 'Y') .OR. (ANSW .EQ. 'y')) DISPL=.TRUE.
        IF ((ANSW .EQ. 'y') .OR. (ANSW .EQ. 'n')) THEN
          INVERZ = .TRUE.
          WRITE(NOUT,*) ' HIDDEN STACK INVERSION SELECTED'
        ENDIF

 4367   CONTINUE
C       SET FLAG FOR DATA INVERSION IF ORIGINALLY CLOCKWISE.
        CWISE = .TRUE.

        CALL RDPRMC(ANSW,N,.TRUE.,
     &     'INVERT CLOCKWISE DIRECTION (N/Y)',NULL,IRTFLG)
        IF (IRTFLG .EQ. -1) GO TO 2043
        IF ((ANSW .EQ. 'H') .OR. (ANSW .EQ. 'h')) THEN
           WRITE(NOUT,*) ' NO HIDDEN OPTION'
           GOTO 4367
        ENDIF
        IF (ANSW .EQ. 'Y'.OR. ANSW .EQ. 'y') CWISE=.FALSE.

C..      GO FROM 1 TO LSTCON & PROCESS VALID CONTOURS ONLY
         DO 1001 I = 1, LSTCON
           IF (CONTRL(1,I) .LT. 0.0) GOTO 1001
           ICNTR= I
           CALL CSGET(ICNTR,NSIZE,DATA,NDATA,LUNCS,NF)

C.         DISPLAY THE CONTOUR IF DESIRED
           ITRM = 1
           IF (DISPL) CALL SSDISP(DATA,NDATA,ITRM,.TRUE.)

           WRITE(NOUT,114) I,NDATA,ZCOO,MOFLNM
  114      FORMAT
     &        (' Contour ',I3,' with ',I4,' points at z-level ',
     &           F4.1,' will be stored in : ',A)

           SWITCH = .FALSE.
           IF (CHECK) THEN
             CALL RDPRMC(ANSW,N,.TRUE.,
     &          'TRANSFER CONTOUR, HOLE, OR ABORT? (C/H/A)',
     &          NULL,IRTFLG)

             IF (ANSW .EQ. 'H' .OR. ANSW .EQ. 'h')THEN
                CWISE  = .NOT. CWISE
                SWITCH = .TRUE.
             ENDIF

             IF (ANSW .EQ. 'A' .OR. ANSW .EQ. 'a') THEN
                WRITE(NOUT,*) ' Transfer aborted upon command.'
                GO TO 1003
             ENDIF
           ENDIF

C..        PUT CONTOUR POINTS INTO MOVIE FILE

C          DECODE FILE NUMBER TO GET A Z LEVEL.
C          UNLESS THE INVERZ FLAG IS SET, THE FILES ARE ASSUMED TO
C          BE NUMBERED FROM TOP TO BOTTEM
C           CSNUMB = CSFLNM(4:6)
C           DECODE(3,8897,CSNUMB)IZZ
C 8897      FORMAT(I3)
C           ZZLEVL = -IZZ

           ZZLEVL = -ZCOO
           IF (INVERZ) ZZLEVL = -ZZLEVL

C..        RECOVER ORIGINAL UNSCALED DATA
C..        ENSURE NO POINTS ARE REPEATED AS MOVIE.BYU DOESNT LIKE THEM!
           N = 1
           TEMPX   = DATA(1,1)* SCALE + XMIN
           IF(MIRROR) THEN
             TEMPY = (1.0-DATA(2,1))* SCALE + YMIN
           ELSE 
             TEMPY = DATA(2,1)* SCALE + YMIN
           ENDIF
           DATAT2(1,N) = TEMPX
           DATAT2(2,N) = TEMPY

           DO  J = 2,NDATA

             TEMPX2   =   DATA(1,J)* SCALE + XMIN
             IF(MIRROR) THEN
                TEMPY2 = (1.0 - DATA(2,J))* SCALE + YMIN
             ELSE
                TEMPY2 =  DATA(2,J)* SCALE + YMIN
             ENDIF

C
C            IF((ABS(TEMPX2-TEMPX)).GT.(TEMPX*10E-4).OR.
C     &         (ABS(TEMPY2-TEMPY)).GT.(TEMPY*10E-4)) THEN
             IF((TEMPX2 .NE. TEMPX).OR.
     &         (TEMPY2 .NE. TEMPY)) THEN
               N = N + 1
               DATAT2(1,N) = TEMPX2
               DATAT2(2,N) = TEMPY2
               TEMPX       = TEMPX2
               TEMPY       = TEMPY2   
            ENDIF
            IF((N .EQ. (NSIZE-1)) .AND. (J .LT. NDATA)) THEN
                WRITE(NOUT,925) NSIZE
  925           FORMAT(' CONTOUR TRUNCATED AT ',I5,' POINTS') 
               GOTO 1235
            ENDIF
 	  ENDDO
 1235     CONTINUE

C..       INVERT ARRAYS TO CLOCKWISE IF NECESSARY
          IF (CWISE) THEN
             IGO1   = 1
             ISTOP1 = N
             IINCR  = 1
          ELSE
             IGO1   = N
             ISTOP1 = 1
             IINCR  = -1
          ENDIF

          IF(SWITCH) CWISE= .NOT.CWISE

C..       TRANSFER DATA
CC          WRITE(LUNM,921) N,ZZLEVL
	  WRITE(LUNM,921) N,ZZLEVL,ZCOO
  921     FORMAT(I12,1PE12.5,5X,' CONTUR   TOP:',1PE12.5)

          WRITE(LUNM,920) (DATAT2(1,K),DATAT2(2,K),K=IGO1,
     &                    ISTOP1,IINCR)
  920     FORMAT(1P6E12.5)              
  
 1003   CONTINUE
         IF( DISPL ) THEN 
C..        CLEAR THE SCREEN
           CALL TMSTRT(NOUT)
           CALL TMCLR(NOUT)
           CALL TMEND(NOUT)
         ENDIF

         IF(CHECK .AND. (IRTFLG.EQ.0)) THEN
            CALL RDPRMC(ANSW,N,.TRUE.,'Transfer more? (Y/N)',
     &           NULL,IRTFLG)
            IF (ANSW .EQ. 'N')  GO TO 9004
         ENDIF

 1001  CONTINUE

       
C-------------------------------------------TRANSFERS COMPLETED

C..     ENTER END OF DATA INDICATOR IN MOVIE FILE
 9004   LS = 0
        ZLEVEL = XFLAG
        WRITE(LUNM,921) LS,ZLEVEL,ZCOO

  923   FORMAT(3X,A6,'   CLASS: ',A4,3X,'CNT: ',I4)

C..     CLOSE MOVIE FILE 
        CLOSE (UNIT = LUNM)

      RETURN
      END

