
C*************************************************************************** 
C
C CNOPN.FOR  -- CREATED MAY 87
C               ICNTFMT ADDED JULY 92 al
C
C **************************************************************************
C *  AUTHOR: ArDean Leith 
C *  COPYRIGHT (C)1987 , WADSWORTH CENTER FOR LABORATORIES AND RESEARCH    *
C *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C    CNOPN(FILNAM,DISP,LUN,IRCLND,IRTFLG)
C
C    PURPOSE:    OPENS AN OLD OR  NEW CNT FILE FOR STORAGE OR RECOVERY OF DATA.
C                ADAPTED FROM CSOPEN VIA CNOPEN. USE ALFILE TO SET THE
C                FILE NAME BEFORE CALLING THIS PGM.
C
C    PARAMETERS: FILNAM = WHOLE FILE NAME FOR FILE TO BE OPENED(WITH EXTENSION)
C                DISP   = CHAR FLAG THAT FILE IS OLD, NEW, OR UNKNOWN
C                LUN    = LOGICAL UNIT FOR OPENING FILE IF NECESSARY
C                IRCLND = RECORD LENGTH OF FILE
C                IRTFLG = ERROR FLAG
C
C    NOTE:       IF ITFLG IS "1" THEN TITLE IS SOLICITED.  ITFLG IS
C                PASSED IN COMMON
C
C    CALLED BY: 
C
C    CALLS:      CSRDCT    DATE     TIME      WRTLIN     RDPRMC 
C
C--********************************************************************

        SUBROUTINE CNOPN(FILNAM,DISP,LUN,IRCLND,IRTFLG)

        INCLUDE 'CONSTG.INC'

        COMMON /UNITS/LUNDOC,NIN,NOUT
        COMMON /LUNARA/ LUNARA(100),LUNSTK(100)

        COMMON    /BATCH/IBCNT,NLOOP,IABSLP,ILOOP
        INTEGER          IBCNT,NLOOP,IABSLP,ILOOP
       
        CHARACTER * 80 LINE
        CHARACTER * 1  NULL
        CHARACTER *(*) FILNAM,DISP
        LOGICAL        EX,ERRI2
        EQUIVALENCE    (CBUF(1),LINE(1:1))

	NULL=CHAR(0)

C       SET NO ERROR RETURN
        IRTFLG = 0

C       INITIALIZE DEFAULT VALUES
        MARK  = 0
        IPEN  = 0
        LINTP = 0
        INTEN = 9
        JUST  = 0
        IORN  = 0
        IFON  = 1

C       FIND LENGTH OF EXTENDED FILE NAME
        NLET   = INDEX(FILNAM,NULL)
        IF (NLET .EQ. 0) THEN
           NLET = LEN(FILNAM)
        ELSE
           IF (NLET .LE. 1) GOTO 5001
           NLET = NLET - 1
        ENDIF

        IF (LUN .LE. 0 .OR. LUN .GT. 100) THEN
           WRITE(NOUT,1093) LUN
1093       FORMAT(' ERROR: LUN=',I4,' MUST BE 1...100, SET TO 99'/)
           LUN = 99
        ENDIF
        LUNARA(LUN) = 1
        LUNSTK(LUN) = 0

C       CHECK FOR FILE EXISTENCE 
        INQUIRE (FILE=FILNAM,ERR=5001,EXIST=EX) 

        IF (.NOT. EX .AND. (DISP(1:1) .EQ. 'O' .OR.
     &                      DISP(1:1) .EQ. 'C')) THEN
          WRITE(NOUT,902)  FILNAM(1:NLET)
902       FORMAT(' *** DOES NOT EXIST: ',A)
          IRTFLG = 1
          RETURN

        ELSEIF (DISP(1:1) .EQ. 'O' .OR. DISP(1:1) .EQ. 'C') THEN

C         THE FILE ALREADY EXISTS, OPEN THE FILE, READ THE RECORDS 
C         AND RESTORE ALL COMMON BLOCK VARIABLES.

#ifdef __osf__
C         RECL IS IN UNITS OF FLOATS ON OSF 
          LENCNT4 = 1080
          IRECLN  = LENCNT4  
#else
C         RECL IS IN UNITS OF BYTES ON SGI AND SUN
          LENCNT4 = 4320
          IRECLN  = LENCNT4  / 4
#endif

          OPEN(UNIT=LUN,FILE=FILNAM,STATUS='OLD',FORM='UNFORMATTED',
     &    ACCESS='DIRECT',RECL=LENCNT4,ERR=5001)

          WRITE(NOUT,2109) FILNAM(1:NLET)
 2109     FORMAT(' FILE OPENED: ',A)

          CALL CSRDCT(LUN,FILNAM,IRTFLG)

          IRCLND = IRECLN
          RETURN

      ELSEIF (DISP(:1) .EQ. 'U' .OR. DISP(:1) .EQ. 'N') THEN
C        FILE DOES NOT EXIST, AND IT IS SPECIFIED AS 'UNKNOWN'
C*       OR IT EXISTS AND IS SPECIFIED AS 'NEW'
C        NON-ZERO MAXCNT AND IRECLN NEED TO BE SUPPLIED

         IF (MAXCNT .LE. 10 .OR. MAXCNT .GT. IMAXCT) THEN
1019       MAXCNT = IMAXCT-1
           CALL RDPRIS(MAXCNT,IDUM,NLOOP,ILOOP,
     &         'ENTER MAX NUMBER OF CONTOURS EXPECTED',IRTFLG)
           IF (IRTFLG .EQ. -1) RETURN
           IF (ERRI2(MAXCNT,IDUM,1,2,IMAXCT,IDUM,IDUM)) GOTO 1019
         ENDIF

         IRECLN  = 1080
         LENCNT  = IRECLN

#ifdef __osf__
         IRECLN4 = IRECLN 
#else
         IRECLN4 = IRECLN * 4
#endif

         OPEN(UNIT=LUN,FILE=FILNAM,STATUS='NEW',FORM='UNFORMATTED',
     &       ACCESS='DIRECT',RECL=IRECLN4,ERR=5001)

         WRITE(NOUT,2104)  FILNAM(1:NLET)
2104     FORMAT(' FILE OPENED: ',A)

C        GET DATE, TIME AND TITLE AND PUT THEM INTO THE FILE
	 CALL DATE(DATECS)
	 CALL TIME(TIMECS)
         IF (ITFLG .EQ. 1) THEN
           CALL RDPRMC(TITLCS,N,.TRUE.,
     &          'Enter title (max 40 char.)',NULL,IRTFLG)
           IF (IRTFLG .EQ. -1) RETURN
         ENDIF
         LINE(1:80) = DATECS(1:10) // TIMECS(1:8) // TITLCS(1:40) //
     &               '            ' // NULL
	 CALL WRTLIN(LUN,CBUF,IRECLN,1)

C        INITIALIZE ALL VARIABLES AND ARRAYS
         DO  K = 1,IRECLN
           CBUF(K) = 0.0
         ENDDO

C        INTIALIZE RECORD NO 3 & 4
         CALL WRTLIN(LUN,CBUF,IRECLN,2)
         CALL WRTLIN(LUN,CBUF,IRECLN,3)

         K  = IRECLN / 6
         K1 = MAXCNT / K
         K2 = MAXCNT - K * K1
         IF (K2 .GT. 0) K1 = K1 + 1

C        NDTCTL IS NO. OF CONTROL RECORDS IN CNT FILE
         NDTCTL = K1
                  
         IF (.NOT. SERSEC) THEN
C           INCREASE MAXCNT TO USE A FULL LINE
            MAXCNT = K * NDTCTL
            MAXCNT = MIN(MAXCNT,IMAXCT)    
C           CAN NOT EXCEED IMAXCT ARRAY LIMIT
         ENDIF

         MAXTXT = IRECLN / 5
         DO  K = 4, NDTCTL+3
           CALL WRTLIN(LUN,CBUF,IRECLN,K)
         ENDDO

C        NOW WRITE THE FIRST RECORD
         CBUF(1)  = MAXCNT
         CBUF(2)  = MAXTXT

C        MAXREC IS CURRETN NUMBER OF RECORDS USED
         CBUF(3)  = NDTCTL + 5

C        NDTSUF
         CBUF(4)  = 1  
             
C        NTXSUF
         CBUF(5)  = 1 
              
C        FILE TYPE
         CBUF(6)  = IFRMCS  
       
C        SCALE
         CBUF(7)  =  0.0 

C        XMAX     
         CBUF(8)  = -9.0E20       
C        XMIN
         CBUF(9)  =  9.0E20      
C        YMAX
         CBUF(10) = -9.0E20    
C        YMIN
         CBUF(11) =  9.0E20
    
C        ICOMP (COMPRESSION FLAG, UNUSED)
         CBUF(12) = 0 
         
C        NDTCT
         CBUF(13) = NDTCTL   
   
C        NCNTRS
         CBUF(14) = 0 
          
C        ISTCON
         CBUF(15) = 1.0 
        
C        LSTCON
         CBUF(16) = 0.0
   
C        ISTTXT
         CBUF(17) = 1.0 
      
C        LSTTXT
         CBUF(18) = 0.0
        
C        NTXT
         CBUF(19) = 0         

C        SET DIGITIZER CONSTANT 
         PIXPCM   = 1.0
         CBUF(20) = PIXPCM

C*       VAX RECORD LENGTH BUG CORRECTED USING LENCOR
         CBUF(21) = 1           
         CBUF(22) = LENCNT
           
         CALL WRTLIN(LUN,CBUF,IRECLN,0)   

         CALL CSRDCT(LUN,FILNAM,IRTFLG)    

C        RETURNS IRECLN TO CALLER
         IRCLND = IRECLN 
    
C        LSRECD REMEMBERS WHAT CNT RECORD IS IN CBUF
         LSRECD = 1  
       
         IRTFLG = 0
       RETURN
      ENDIF


C     REPORT ERRORS 
5001  WRITE(NOUT,102) LUN,FILNAM(1:NLET)
  102 FORMAT(' ERROR ON UNIT:',I3,' OPENING FILE: ',A)
      IRTFLG = 1

      RETURN
      END
