
C++*************************************************************************
C
C    CNPOST.FOR  -- ADAPTED AUG 88 FROM CNRAST
C                   REWRITTEN JULY 92 - al
C **************************************************************************
C *  AUTHOR: ArDean Leith                                                  *
C *  COPYRIGHT (C)1986 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C CNPOST(
C
C    PURPOSE:  TO READ CONTOUR AND TEXT DATA FROM A FILE AND PLOT
C              THEM ON A RASTER TECH. 1/80 TERMINAL.
C
C    PARAMETERS:
C           FILNAM   THE NAME OF THE CONTOUR STORAGE FILE IN CHARACTER FORMAT
C           FILEXT   THE EXTENSION OF THE CONTOUR STORAGE FILE
C                    IN CHARACTER * 3 FORMAT
C           LUNCS    UNIT NO. USED TO READ CNT FILE
C           IRTFLG   ERROR RETURN FLAG
C                    (= 0   NO ERROR;  = 1   ERROR IN PLOTTING)
C           INVERZ   LOGICAL FLAG TO INVERT CONTOUR ORDER
C
C    CALLED BY:   CNDISP
C
C    CALLS:      CSGTTX       POSTDR
C 
C        0         2         3         4         5         6         7
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*********************************************************************

        SUBROUTINE CNPOST(CSFILE,POSTFILE,LUNCS,LUNPOST,PLSCL,   
     &                     CENTER,FLIP,HIDE,FILLED,INVERZ,FRAMES,
     &                     FIRST,LAST,DOTEXT,IRTFLG)

        INCLUDE 'CONSTG.INC'

        LOGICAL ONSUN,ONUNIX,ONSGI,ONOSF
        COMMON /OPSYS/ ONSUN,ONUNIX,ONSGI,ONOSF,MEMALLOCABLE

        COMMON /UNITS/LUNDOC,NIN,NOUT

        PARAMETER      (NSIZE=2000)
        COMMON         DATA,IDATA(2,NSIZE)

        DIMENSION      DATA(3,NSIZE)
        CHARACTER *(*) CSFILE,POSTFILE
        CHARACTER * 80 LINE
        EQUIVALENCE    (CDATA,FDATA)
        CHARACTER * 4  CDATA
        CHARACTER * 1  NULL,ANSW

        LOGICAL        CENTER,FLIP,HIDE,FRAMES,FILLED,DOTEXT,
     &                 INVERZ,FIRST,LAST

        DATA MININT/-32768/,MAXINT/32767/,FLTMIN/-10E30/,
     &      FLTZER/10E-30/,FLTMAX/10E30/

        NULL  = CHAR(0)

        IF (FIRST) THEN
C         ASSIGN VIEWPORT SIZE
          IVPTSZ = 100
          IVPTD2 = IVPTSZ / 2

          IF (CENTER) THEN
C           FIND  OFFSET PARAMETERS FOR CENTERING
            XCENT  = (XMAX - XMIN) / (2.0 * SCALE)
            YCENT  = (YMAX - YMIN) / (2.0 * SCALE)
            YXTRA  = -PLSCL * YCENT
            XXTRA  = -PLSCL * XCENT
 
          ELSE   
C           FIND NON-CENTERED OFFSET PARAMETERS
            YXTRA = -PLSCL * 0.5
            XXTRA = -PLSCL * 0.5
          ENDIF

C         SET LOWER LEFT CORNER VALUES
          WYMIN = FLTMAX
          WXMIN = FLTMAX
       
          PLSCLY = PLSCL
          IF (FLIP) THEN 
            PLSCLY = -PLSCL
            YXTRA  = PLSCLY + YXTRA 
          ENDIF

          CALL POSTRT(LUNPOST)
        ENDIF


C       GO FROM 1 TO LSTCON & PLOT CONTOURS FROM CONTOUR DATABANK
         IF (INVERZ) THEN
            IGO   = LSTCON
            ISTOP = 1
            III   = -1
         ELSE
            IGO   = 1
            ISTOP = LSTCON
            III   = 1
         ENDIF

         DO 1001 IJ = IGO,ISTOP,III

            IF (CONTRL(1,IJ) .LE. 0.0) GOTO 1001
            ICNTR = IJ
            CALL CSGET(ICNTR,NSIZE,DATA,NDATA,LUNCS,NF)
            IF (NDATA .LE. 0 .OR. NF .NE. 0) GOTO 1001

            CALL POLINE(LUNPOST,INTEN,IPEN,LINTP)  

            IF (IFRMCS .EQ. -6 .AND. MARK .EQ. -2) THEN
C              NEW TYPE TEXT CONTOUR
               NCHAR = DATA(1,2)
               FDATA = DATA(2,2)
               XP    = DATA(1,1) * PLSCL  + XXTRA
               YP    = DATA(2,1) * PLSCLY + YXTRA
               WXMIN = MIN(WXMIN,XP)
               WYMIN = MIN(WYMIN,YP)
               CALL POTEX(LUNPOST,CDATA,NCHAR,XP,YP,ITSIZE,ITANGL,JUST)

            ELSEIF (IFRMCS .EQ. 6 .AND. MARK .EQ. 2) THEN
C              THIS IS A TEXT CONTOUR
               DO  II = 1, NDATA
                 XP    = DATA(1,II) * PLSCL  + XXTRA
                 YP    = DATA(2,II) * PLSCLY + YXTRA
                 WXMIN = MIN(WXMIN,XP)
                 WYMIN = MIN(WYMIN,YP)
                 CALL POTEX(LUNPOST,CSYMBL,1,XP,YP,ITSIZE,ITANGL,JUST)
               ENDDO

            ELSE
C              NORMAL GRAPHICS CONTOUR
               XP   = DATA(1,1) * PLSCL  + XXTRA
               YP   = DATA(2,1) * PLSCLY + YXTRA
               CALL POPATH(LUNPOST,XP,YP)

C              PROCESS EACH POINT ON THIS LINE
               DO IDIS = 1,NDATA

C                IDATA IS FILLED X-->Y FOR SPEED IN POSTDR
                 XT    = DATA(1,IDIS) * PLSCL  + XXTRA
                 YT    = DATA(2,IDIS) * PLSCLY + YXTRA
                 IDATA(2,IDIS) = XT
                 IDATA(1,IDIS) = YT
                 WXMIN = MIN(WXMIN,XT)
                 WYMIN = MIN(WYMIN,YT)
               ENDDO

               IF (HIDE) THEN           
C                WANT HIDDEN LINE OUTPUT
                 IF (DATA(1,1) .NE. DATA(1,NDATA) .OR.
     &               DATA(2,1) .NE. DATA(2,NDATA)) THEN
                   WRITE(NOUT,*) ' SORRY: OPEN CONTOUR NOT FILLED.'
c avoid jump inside loop       GOTO 105
C                  SIMPLE POLYLINE OUTPUT, DRAW LINES
                   CALL POARAY(LUNPOST,IDATA,NDATA,.FALSE.,.FALSE.)
                   goto 1001
                 ENDIF

C                SET POLYGON TO BACKGROUND COLOR IF NOT FILLED
                 IF (.NOT. FILLED) CALL POLINE(LUNPOST,0,0,0)

C                DRAW SOLID POLYGON
                 CALL POARAY(LUNPOST,IDATA,IDIS,.TRUE.,.TRUE.)
  
                 IF (.NOT. FILLED) CALL POLINE(LUNPOST,INTEN,IPEN,LINTP)
                 CALL POARAY(LUNPOST,IDATA,IDIS,.FALSE.,.FALSE.)

               ELSE
C                SIMPLE POLYLINE OUTPUT, DRAW LINES
105              CALL POARAY(LUNPOST,IDATA,NDATA,.FALSE.,.FALSE.)
               ENDIF
            ENDIF
1001     CONTINUE

         LINTP = 0

         IF (DOTEXT) THEN
           DO I = 1, LSTTXT
C..          GO FROM 1 TO LSTTXT & PROCESS EXISTING TEXTS
             IF (TXTCTR(1,I) .GE. 0.0) THEN
               ITEXT = I
               CALL CSGTTX(LINE,NCHAR,LUNCS,NF,ITEXT)

               IF (NCHAR .GT. 0 .AND. NF .EQ. 0) THEN
                 TX1 = TXTCTR(4,I) * PLSCL  + XXTRA
                 TY1 = TXTCTR(5,I) * PLSCLY + YXTRA

                 WXMIN = MIN(WXMIN,TX1)
                 WYMIN = MIN(WYMIN,TY1)

C                SET TEXT INTENSITY
                 CALL POLINE(LUNPOST,INTEN,IPEN,LINTP)  
C                DRAW TEXT
                 CALL POTEX(LUNPOST,LINE,NCHAR,TX1,TY1,ITSIZE,
     &                      ITANGL,JUST)
               ENDIF
             ENDIF
           ENDDO
         ENDIF

         IF (FRAMES) THEN
C           DRAW WHITE FRAME AROUND PLOT
            X1     = -IVPTD2
            Y1     =  IVPTD2
            X2     =  IVPTD2
            Y2     = -IVPTD2

            IPEN  = 0
            INTEN = 9
            CALL POLINE(LUNPOST,INTEN,IPEN,LINTP)    
            CALL POPATH(LUNPOST,X1,Y1)
            CALL PODRAW(LUNPOST,X1,Y2)
            CALL PODRAW(LUNPOST,X2,Y2)
            CALL PODRAW(LUNPOST,X2,Y1)
            CALL POSHOW(LUNPOST,.TRUE.,.FALSE.)
            WXMIN = MIN(WXMIN,X1)
            WYMIN = MIN(WYMIN,Y1)
         ENDIF


         IF (LAST) THEN
           CALL RDPRMC(LINE,NALP,.TRUE.,
     &                'COMMENT (OR <CR> FOR NONE)',NULL,IRT)
           IF (NALP .GT. 0) THEN
             CALL POLL(LUNPOST)
C**             CALL POPATH(LUNPOST,WXMIN,WYMIN-48.0)
             CALL POTEXT(LUNPOST,LINE,NALP)
           ENDIF
           CALL POEND(LUNPOST)         
           CLOSE(LUNPOST)

           CALL RDPRMC(ANSW,NA,.TRUE.,'PRINT NOW? (Y/N)',NULL,IRTFLG)
           IF (ANSW .NE. 'N' .AND. ANSW .NE. 'n') THEN
C            THIS WILL HAVE TO BE ALTERED AT DIFFERENT SITES!!!!

             WRITE(NOUT,*) ' WARNING: SITE SPECIFIC COMMAND IN CNPOST'
             IF (ONUNIX) THEN
                LINE = 'lpr ' // POSTFILE
             ELSE
                LINE = 'PRINT/NOFLAG/QUE=SPIDER_POST_QUE ' // POSTFILE
             ENDIF

             CALL CSVMS(LINE,IERR)
             WRITE(NOUT,*) ' '
           ENDIF
        ENDIF

      RETURN
      END         

