C++************************************************************************
C
C $$ COMBEUL.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR       *
C *  LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR    *
C *  USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF     *
C *  THE CENTER FOR LABORATORIES AND RESEARCH   			   *
C **************************************************************************
C
C
C
C--************************************************************************
C
C $$ COMBEUL.FOR
C
C------------------------------------------------------------------------
C   subroutine to combine two euler rotations and calculate the
C   three angles of the corresponding single euler rotation.
C   4/27/90 M.Radermacher
C          The definition of the Euler rotations is as follows:
C
C          r' = D(psi) D(theta) D(phi) r
C
C          with:
C                       | cos(psi) sin(psi)  0 |
C                D(psi)=|-sin(psi) cos(psi)  0 |
C                       |    0        0      1 |
C
C                       | cos(theta)  0 -sin(theta) |
C              D(theta)=|     0       1      0      |
C                       | sin(theta)  0  cos(theta) |
C
C                       | cos(psi) sin(psi)  0 |
C                D(psi)=|-sin(psi) cos(psi)  0 |
C                       |    0        0      1 |
C
C      with a,b,c1 = psi,theta,phi of first rotation and
C           c2,d,e = psi,theta,phi of second rotation.
C
C      w = logical flag for control of printout
C-------------------------------------------------------------------

         SUBROUTINE COMBEUL(A,B,C1,C2,D,E,PSI,THETA,PHI,W)

         INCLUDE 'CMBLOCK.INC'

         REAL*4 M11,M12,M13,M21,M22,M23,M31,M32,M33
         COMMON AM(3,3),BM(3,3),CM(3,3),DM(3,3),EM(3,3),
     $   FM(3,3),GM(3,3),HM(3,3),S(3,3),T(3,3),SFV(2),CFV(2)
     $   ,TT(3,3),SGT(2),TEST(3)
         LOGICAL W,W1
	 
         W1=.FALSE.
         PI=4.*DATAN(1.D0)
         WRITE(NOUT,500) A,B,C1,C2,D,E
500      FORMAT(' input angles:'/,
     $   '  psi1=',f10.4,'  theta1=',f10.4,'  phi1=',f10.4,
     $   /'  psi2=',f10.4,'  theta2=',f10.4,'  phi2=',f10.4)
         C=C1+C2
         SA=SIND(A)
         SB=SIND(B)
         SC=SIND(C)
         SD=SIND(D)
         SE=SIND(E)
         CA=COSD(A)
         CB=COSD(B)
         CC=COSD(C)
         CD=COSD(D)
         CE=COSD(E)
Cw     write(nout,200) sa,ca,sb,cb,sc,cc,sd,cd,se,ce
200      FORMAT(' sin(a)=',e12.4,' cos(a)=',e12.4,/
     $   ' sin(b)=',e12.4,' cos(b)=',e12.4,/
     $   ' sin(c)=',e12.4,' cos(c)=',e12.4,/
     $   ' sin(d)=',e12.4,' cos(d)=',e12.4,/
     $   ' sin(e)=',e12.4,' cos(e)=',e12.4)
C build matrix am

         AM(1,1)=CA
         AM(1,2)=SA
         AM(1,3)=0
         AM(2,1)=-SA
         AM(2,2)=CA
         AM(2,3)=0
         AM(3,1)=0
         AM(3,2)=0
         AM(3,3)=1
Cw     write(nout,11)
11       FORMAT(' ',/' matrix am:')
Cw     write(nout,100) am
C build matrix bm
         BM(1,1)=CB
         BM(1,2)=0
         BM(1,3)=-SB
         BM(2,1)=0
         BM(2,2)=1
         BM(2,3)=0
         BM(3,1)=SB
         BM(3,2)=0
         BM(3,3)=CB
Cw      write(nout,12)
12       FORMAT(' ',/' matrix bm:')
Cw     write(nout,100) bm
C build matrix cm
         CM(1,1)=CC
         CM(1,2)=SC
         CM(1,3)=0
         CM(2,1)=-SC
         CM(2,2)=CC
         CM(2,3)=0
         CM(3,1)=0
         CM(3,2)=0
         CM(3,3)=1
Cw     write(nout,13)
13       FORMAT(' ',/' matrix cm:')
Cw     write(nout,100) cm
C build matrix dm
         DM(1,1)=CD
         DM(1,2)=0
         DM(1,3)=-SD
         DM(2,1)=0
         DM(2,2)=1
         DM(2,3)=0
         DM(3,1)=SD
         DM(3,2)=0
         DM(3,3)=CD
Cw     write(nout,14)
14       FORMAT(' ',/' matrix dm:')
Cw     write(nout,100) dm
C build matrix em
         EM(1,1)=CE
         EM(1,2)=SE
         EM(1,3)=0
         EM(2,1)=-SE
         EM(2,2)=CE
         EM(2,3)=0
         EM(3,1)=0
         EM(3,2)=0
         EM(3,3)=1
Cw     write(nout,15)
15       FORMAT(' ',/' matrix em:')
Cw     write(nout,100) em
C multiply them:
         CALL MRMAMULE(AM,BM,3,3,3,S,W1)
Cw     write(nout,16)
16       FORMAT(' ',/' matrix am * bm:')
Cw     write(nout,100) s
         CALL MRMAMULE(S,CM,3,3,3,T,W1)
Cw     write(nout,17)
17       FORMAT(' ',/' matrix am * bm *cm :')
Cw      write(nout,100) t
         CALL MRMAMULE(T,DM,3,3,3,S,W1)
Cw      write(nout,18)
18       FORMAT(' ',/' matrix am * bm * cm * dm:')
Cw     write(nout,100) s
         CALL MRMAMULE(S,EM,3,3,3,TT,W1)
         IF(W) WRITE(NOUT,19)
19       FORMAT(' ',/' matrix am * bm * cm * dm * em:')
         IF(W) WRITE(NOUT,100) TT
C
C solutions:
         M11=TT(1,1)
         M12=TT(1,2)
         M13=TT(1,3)
         M21=TT(2,1)
         M22=TT(2,2)
         M23=TT(2,3)
         M31=TT(3,1)
         M32=TT(3,2)
         M33=TT(3,3)
Cw     write(nout,300) m11,m12,m13,m21,m22,m23,m31,m32,m33
300      FORMAT(' m11,m12,m13',3e12.4,/
     $   ' m21,m22,m23',3e12.4,/
     $   ' m31,m32,m33',3e12.4)

         CG=M33
C there are 3 ways to calculate sin(g). Choose the one that does
C not attempt to divide by 0.
         IF(M11.NE.0) THEN
           SGT(1)=SQRT((-M31*M33*M13-M32*M23)/M11)
           TEST(1)=SGT(1)
           GOTO 10
         ENDIF
         IF(M12.NE.0) THEN
           SGT(1)=SQRT((-M32*M33*M13+M23*M31)/M12)
           TEST(2)=SGT(1)
           GOTO 10
         ENDIF
         IF(M22.NE.0) THEN
           SGT(1)=SQRT((-M33*M23*M32-M13*M31)/M22)
           TEST(3)=SGT(1)
           GOTO 10
         ENDIF
         IF(W) WRITE(NOUT,77) TEST
77       FORMAT(' test:',3(2x,e12.4))
10       SGT(2)=-SGT(1)
         DO  I=1,1
         SG=SGT(I)
         IF (SG.NE.0) THEN
         SF = M23/SG
         CF =-M13/SG
         SH = M32/SG
         CH = M31/SG
         ELSE
C if sg = sin(g) = 0 we have only rotations around the z-axis, which
C is accomplished by a single angle h.
         SF=0
         CF=1
         CH=M11
         SH=M12
         ENDIF
C the following section is to confirm that the resulting rotation
C is correct, so its only done if it is also written out (flag w =
C .true.:
         IF(.NOT.W) GOTO 1001
C now build all necessary matrices
C build matrix fm
         FM(1,1)=CF
         FM(1,2)=SF
         FM(1,3)=0
         FM(2,1)=-SF
         FM(2,2)=CF
         FM(2,3)=0
         FM(3,1)=0
         FM(3,2)=0
         FM(3,3)=1
Cw     write(nout,20)
20       FORMAT(' ',/' matrix fm:')
Cw     write(nout,100) fm
C build matrix gm
         GM(1,1)=CG
         GM(1,2)=0
         GM(1,3)=-SG
         GM(2,1)=0
         GM(2,2)=1
         GM(2,3)=0
         GM(3,1)=SG
         GM(3,2)=0
         GM(3,3)=CG
Cw     write(nout,21)
21       FORMAT(' ',/' matrix gm:')
Cw     write(nout,100) gm
C build matrix hm
         HM(1,1)=CH
         HM(1,2)=SH
         HM(1,3)=0
         HM(2,1)=-SH
         HM(2,2)=CH
         HM(2,3)=0
         HM(3,1)=0
         HM(3,2)=0
         HM(3,3)=1
Cw     write(nout,22)
22       FORMAT(' ',/' matrix hm:')
Cw     write(nout,100) hm
         CALL MRMAMULE(FM,GM,3,3,3,S,W1)
Cw     write(nout,23)
23       FORMAT(' ',/' matrix fm * gm:')
Cw     write(nout,100) s
         CALL MRMAMULE(S,HM,3,3,3,T,W1)
         IF(W)WRITE(NOUT,24)
24       FORMAT(' ',/' matrix fm * gm * hm:')
         IF(W) WRITE(NOUT,100) T
100      FORMAT(3(1x,e12.4))
1001     CONTINUE
         THETA=ATAN2(SG,CG)/PI*180.
         PSI  =ATAN2(SF,CF)/PI*180.
         PHI  =ATAN2(SH,CH)/PI*180.
         WRITE(NOUT,400) I
400      FORMAT(' resulting angles set # ',i2)
         WRITE(NOUT,401) PSI,THETA,PHI
401      FORMAT(' psi=',e12.4,'  theta=',e12.4,'  phi=',e12.4)
	 ENDDO
1000     RETURN
         END
