
C ++********************************************************************
C
C COPYBRIX
C
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1993, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C 
C PURPOSE: CONVERTS 3D SPIDER IMAGES FOR INPUT DIRECTLY TO O.
C          COMPATIBLE WITH O VERSION 5.10.  JF 3/14/95
C
C        0         2         3         4         5         6         7
C23456789012345678901234567890123456789012345678901234567890123456789012
C**************************************************************************

        SUBROUTINE COPYBRIX(LUN1,LUN2,NSAM,NROW,NSLICE)

        INCLUDE 'CMBLOCK.INC'
CNO_SAVE
        COMMON          BUF(20000)

        INTEGER         EXTENT(3), ORIGIN(3), GRID(3), PLUS, ERRCOD
        INTEGER         XYZBRIX(3)
        REAL            CELL(6), PROD, SIGMA, GINT

        CHARACTER *81   FILNAM
        CHARACTER *80   EXTEN
        COMMON /COMMUN/ FILNAM,EXTEN
        CHARACTER*1     ANS,NULL

        NULL  = CHAR(0)
   
	IF (IMAMI.NE.1) CALL NORM3(LUN1,NSAM,NROW,NSLICE,FMAX,FMIN,AV)
        NPT = NSAM*NROW*NSLICE

C       GET FILENAME,  ALLOW DIFFERENT FILE EXTENSION
        CALL AUXFILERD(FILNAM,NCHAR,EXTEN,'BRIX OUTPUT',IRTFLG)
        IF (IRTFLG .NE. 0) GOTO 999

C       OPEN BRIX FILE
	OPEN(UNIT=LUN2,FILE=FILNAM, STATUS= 'UNKNOWN', ACCESS='DIRECT',
     &          RECL=512,FORM='UNFORMATTED')

        CALL RDPRM(GINT,NLOOP,ILOOP,'SAMPLING DISTANCE')

        CALL RDPRMC(ANS,NCHAR,.TRUE.,
     &     'DEFAULT UNIT CELL IS NSAM,NROW,NSLICE. OVERWRITE (Y/N)?',
     &     NULL,IRTFLG)

C       PREPARE PARAMETERS FOR HEADER

        ORIGIN(1) = -NSAM/2
        ORIGIN(2) = -NROW/2
	IF (MOD(NSLICE,2).EQ.0) THEN
	   ORIGIN(3) =  -NSLICE/2+1
	ELSE
           ORIGIN(3) = -NSLICE/2
	ENDIF

        EXTENT(1) = NSAM 
        EXTENT(2) = NROW 
        EXTENT(3) = NSLICE

        CELL(1)   = EXTENT(1) * GINT
        CELL(2)   = EXTENT(2) * GINT
        CELL(3)   = EXTENT(3) * GINT
        DO I =4,6
          CELL(I) = 90.0
        ENDDO

        GRID(1)   = NSAM
        GRID(2)   = NROW
        GRID(3)   = NSLICE

C       ALL DATA WILL BE RESCALED TO 1000, BECAUSE OF THE
C       FAILURE OF O TO READ IN CONTOUR LEVEL SPECIFICATIONS INVOLVING
C       DIGITS BEHIND THE DECIMAL POINT.

        FMAX  = FMAX*1000.
        FMIN  = FMIN*1000.
        PROD  = 255./(FMAX-FMIN)
        PLUS  = -FMIN*PROD
        SIGMA = SIG* 1000.

        IF (ANS.NE.'N' .AND. ANS.NE.'n') THEN
C          CALL RDPRMI(NX,NY,NLOOP,ILOOP,'ENTER UNIT CELL DIMS (X,Y)')
C          CALL RDPRMI(NZ,NDUM,NLOOP,ILOOP,'ENTER UNIT CELL DIMS (Z)')
           CALL RDPRI3S(NX,NY,NZ,NLOOP,ILOOP,
     &                 'ENTER UNIT CELL DIMS (X,Y,Z)',IRTFLG)
           IF (IRTFLG .EQ. -1) THEN
              CLOSE (LUN1)
              RETURN
           ENDIF

          CELL(1) = NX* GINT
          CELL(2) = NY* GINT
          CELL(3) = NZ* GINT
        ENDIF 

        DO I=1,3
           XYZBRIX(I) = EXTENT(I)/8
           IF(MOD(EXTENT(I),8) .GE. 1) XYZBRIX(I) = XYZBRIX(I) +1
        ENDDO

C       SET FORMAT SWITCH TO "NEW"
        CALL PAGED_FORMAT('NEW')

C       NOW WRITE HEADER
        CALL PAGEDHDR(LUN2, ORIGIN, EXTENT, GRID, CELL, 
     &               SIGMA, PROD, PLUS, ERRCOD)
        IF (ERRCOD .NE. 0) THEN
            WRITE (NOUT,*)  '*** ERROR WRITING BRIX HEADER'
            CALL ERRT(100,'COPYBRIX',NE)
        ENDIF

C       NOW READ SPIDER VOLUME AND RECORD IN BRIX FILE
C       READ UPSIDE DOWN TO CORRECT HANDEDNESS FLIP
        INUM = 1
        DO ISEC = 1, NSLICE
           IRECO = (NSLICE-ISEC)*NROW
	   DO J = 1,NROW
              ISTART = (J-1) * NSAM + 1
              CALL REDLIN(LUN1, BUF(ISTART), NSAM, IRECO+J)
              DO I = 1,NSAM
                 BUF(ISTART+I-1) = (BUF(ISTART+I-1)*1000.-FMIN) * PROD
              ENDDO
           ENDDO
           CALL PAGED (LUN2, BUF(1), BUF(NSAM*NROW+1),
     &        NSAM, NROW, XYZBRIX(1), XYZBRIX(2), ISEC, INUM, ERRCOD)
           IF (ERRCOD .NE. 0) THEN
              WRITE (NOUT,*)  '*** ERROR WRITING BRICK'
              CALL ERRT(100,'COPYBRIX',NE)
           ENDIF
        ENDDO
C       END ISEC LOOP OVER SLICES

        ISEC = 0
        CALL PAGED (LUN2, BUF(1), BUF(NSAM*NROW+1),
     &     NSAM, NROW, XYZBRIX(1), XYZBRIX(2), ISEC, INUM, ERRCOD)

999     CLOSE (LUN1)
        CLOSE (LUN2)

        RETURN
        END


