
C++*********************************************************************
C
C COPYSHORT.F  -- ADAPTED FROM OPENFB.F AUG 96 al
C                                          
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK        *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND     *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.  *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS *
C *    OR USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN        *
C *    APPROVAL OF THE CENTER FOR LABORATORIES AND RESEARCH            *
C **********************************************************************
C
C    COPYSHORT(LUNS,LUNL)
C
C    PURPOSE:       TO OPEN A NEW OR  EXISTING DATA FILE FOR RANDOM 
C                   ACCESS READING/WRITING.
C
C    PARAMETERS:
C        BUF        WORKING BUFFER
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************
#ifdef SP_F90

       SUBROUTINE COPYSHORT

       COMMON /UNITS/LUNC,NIN,NOUT

       WRITE(NOUT,*) 'DUMMY CALL: copyshort'
       RETURN
       END
#else

      SUBROUTINE COPYSHORT(LUNS,LUNL)

      INCLUDE 'CMBLOCK.INC'

      COMMON /LUNARA/ LUNARA(100)
      COMMON /COMMUN/ FILOLD,FILNEW
      COMMON          BUF(1024)

      CHARACTER * 81 FILOLD,FILNEW
      CHARACTER      NULL
      LOGICAL        EX
      DIMENSION      VALUES(3)
      LOGICAL*2      L2TIM(4),L2DAT(5),L2TIT(20)
      EQUIVALENCE    (CDAT,L2DAT),(CTIM,L2TIM),(CITI,L2TIT)
      DATA NULL/0/

C     GET FILENAME FOR EXISTING SHORT HEADER IMAGE FILE
10    CALL FILERD(FILOLD,NLETO,NULL,'OLD SHORT HEADER',IRTFLG)
      IF (IRTFLG .NE. 0) RETURN

      CALL FILCAN(FILOLD,NLETO,NULL,NULL,FILOLD,DATEXC,IRTFLG)
      IF (IRTFLG .NE. 0) GOTO 10

C     SEE IF DATA FILE EXISTS
#ifdef SP_VMS
      INQUIRE(FILE=FILOLD,IOSTAT=IOERR,EXIST=EX,RECL=NSAMO)
      NSAM  = NSAMO / 4
#else
      INQUIRE(FILE=FILOLD,IOSTAT=IOERR,EXIST=EX)
#endif
      IF (.NOT. EX) THEN
C          ERROR -- FILE DOES NOT EXIST
           CALL ERRT(18,'COPYSHORT',NE)
           RETURN
      ELSEIF (IOERR) THEN
C          ERROR OPENING FILE
           CALL ERRT(4,'COPYSHORT',NE)
           RETURN
      ENDIF

#ifdef SP_VMS
      OPEN(UNIT=LUNS,FILE=FILOLD,STATUS='OLD',RECORDTYPE='FIXED',
     1         ACCESS='DIRECT',ERR=9999)
#else
      OPEN(LUNS,FILE=FILOLD,STATUS='OLD',ACCESS='DIRECT',
     &     FORM='UNFORMATTED',RECL=256,ERR=9999)
#endif

C     READ HEADER FROM FILE            	       
      LUNARA(LUNS) = 0
      IERR         = 0

#ifdef SP_VMS
C     NSAM COMES FROM INQUIRE
      CALL REDLIN(LUNS,BUF,NSAM,1)
#else
C     UNIX DOES NOT KNOW NSAM!!
11    WRITE(NOUT,*) 'NSAM IS USUALLY DETERMINED BY VAX/VMS INQUIRE.'
      WRITE(NOUT,*) 'ON UNIX THIS IS NOT AVAILABLE.  YOU MUST ENTER'
      WRITE(NOUT,*) 'A GUESS FOR NSAM'
      NSAM = 64
      CALL RDPRIS(NSAM,NDUM,NLOOP,ILOOP,'NSAM',IRTFLG)
      IF (NSAM .LE. 0) GOTO 11
      IF (IRTFLG .EQ. -1) GOTO 10

      CALL REDLIN(LUNS,BUF,NSAM,1)
#endif
      IF (IERR .NE. 0) THEN
          WRITE(NOUT,*) '*** ERROR READING: ',FILOLD(:NLETO)
          CALL ERRT(100,'COPYSHORT',NE)
          RETURN
      ENDIF

      IF (BUF(1) .LT. 0) THEN
C        A NEGATIVE VALUE FOR NSLICE IS FLAG FOR LONG HEADER ON VAX
         WRITE(NOUT,*) '*** NOT A SHORT HEADER FILE?'
         CALL ERRT(100,'COPYSHORT',NE)
         RETURN
      ENDIF

      NSLICE   = ABS(BUF(1))+0.5
      NROW     = BUF(2)+0.5
      IFORM    = BUF(5)
      IF (BUF(5) .LT. 0.0) IFORM = BUF(5)-0.5
      IF (IABS(IFORM) .EQ. 1) NSLICE = 1
      IMAMI    = BUF(6)+0.5

C      READ SHORT LABEL FORMAT DATE, TIME & TITLE
       IF (IFORM .EQ. -1 .OR. IFORM .EQ. -3 .OR. IFORM .EQ. -5
     &         .OR. IFORM .EQ. -7 .OR. IFORM .EQ. -9)  THEN
C         RECOVER ANGLES FROM FMAX AND FMIN
          IANGLE = IMAMI
          PHI    = BUF(7)
          THETA  = BUF(8)
          SCALE  = BUF(9)
          VALUES(1) = IANGLE
          VALUES(2) = PHI
          VALUES(3) = THETA
          CALL SETLAB(LUNL,NSAM,BUF,14,3,VALUES,'U',IRTFLG)
          VALUES(1) = SCALE
          CALL SETLAB(LUNL,NSAM,BUF,21,1,VALUES,'U',IRTFLG)
       ELSE
C         ANGLES MAY BE IN TITLE
          CALL REDLIN(LUNS,BUF,NSAM,0)
          CALL RDPRMT(VALUES(2),NGOT,BUF,IRTFLG)
          IF (NGOT  .GT. 0) THEN
             VALUES(1) = 1
             CALL SETLAB(LUNL,NSAM,BUF,14,NGOT+1,VALUES,'U',IRTFLG)
          ENDIF
       ENDIF

C      COPY IN THE TITLE
       IF (NSAM .GT. 16) THEN
C         FULL DATE AVAILABLE
          DO I=1,5
             L2DAT(I) = BUF(11+I)+0.5
          ENDDO
          CDAT(10:10) = ' '
       ELSE
          CDAT = '         '
       ENDIF

       IF (NSAM .GT. 20) THEN
C         TIME AND AT LEAST SOME TITLE ARE AVAILABLE              
          DO I=1,4
             L2TIM(I) = BUF(16+I) + 0.5
          ENDDO
          DO I=1,20
             L2TIT(I) = BUF(20+I) + 0.5
          ENDDO
       ELSE
C         TIME NOT AVAILABLE IN SHORT LABEL LENGTH
          CTIM   = '        '
          CTIT   = '          '
       ENDIF

C      OPEN REGULAR SPIDER IMAGE FILE, KEEP THE DATE/TIME
       IRTFLG = -1
       CALL OPFIL(.TRUE.,FILNEW,LUNL,'U',NSAM,NROW,NSLICE,
     &               IDUM,'LONG HEADER',.TRUE.,IRTFLG)
       IF (IRTFLG .NE. 0) GOTO 9000

C      SET TITLE IN NEW FILE
       CALL TITLE(LUNL,NSAM)

C      COPY IMAGE DATA INTO FILE
       LUNARA(LUNS) = 1
       DO I = 1,NROW*NSLICE
          CALL REDLIN(LUNS,BUF,NSAM,I)
          CALL WRTLIN(LUNL,BUF,NSAM,I)
       ENDDO
       GOTO 9000


9999    WRITE(NOUT,*) '*** ERROR OPENING FILE',FILOLD(1:NLETO)
        CALL ERRT(100,'OPENFB',NE)
        RETURN

9000   CLOSE(LUNS)
       CLOSE(LUNL)
       RETURN

       END



C++************************************************************************
C
C   RDPRMT.FOR                           DATE LAST MODIFIED: 9/10/90
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C  RDPRMT(FNUMB,NNN,BUF,IRTFLG)
C
C  PURPOSE:     READ A STRING CONTAINING THE TILT ANGLE AND CONVERT
C               IT INTO A REAL/INTEGER NUMBER.
C               STRING IS OF THE FORM (XX.,XX.) OR (XX)
C               SHOULD WORK WITH <11 NUMBERS IN PARENTHESIS
C
C PARAMETERS:
C               FNUMB  ARRAY WHERE NUMBERS ARE RETURNED
C               NNN    (OUTPUT) NUMBER OF NUMBERS FOUND
C               BUF    CONTAINS THE INPUT STRING IN REAL FORMAT
C 
C--*******************************************************************

	SUBROUTINE RDPRMT(FNUMB,NDUM,BUF,IRTFLG)

        INCLUDE 'CMBLOCK.INC'
        
	DIMENSION BUF(*),FNUMB(*),IN(10)
      
        CHARACTER * 40 TITLE
        CHARACTER * 10 STR,STR2
        INTEGER   * 2  ITITD(20)

        EQUIVALENCE (ITITD(1),TITLE(1:1))


C------- DECODE TITLE -------------------------------------------------

        DO  I=1,20
           ITITD(I) = BUF(20+I)+0.5
	ENDDO

C------- LOOK FOR PARENTHESES AND COMMA --------------------------------

        ICOM = 1
	DO  I=1,40
	   IF (TITLE(I:I) .EQ. '(' .OR. TITLE(I:I) .EQ. ')' .OR.
     &         TITLE(I:I) .EQ. ',') THEN
               IN(ICOM) = I
               IF (TITLE(I:I) .EQ. ')') GOTO 2101
               ICOM = ICOM+1
           ENDIF
	ENDDO

C------- EXTRACT NUMBERS -----------------------------------------------

2101    CONTINUE
        DO  I=1,ICOM-1
           IA = IN(I)+1
           IE = IN(I+1)-1
           K  = 0
           IP = 0
C------    LOOK IF NUMBER IS GIVEN IN REAL OR INTEGER FORMAT -------------
           DO  L=IA,IE
              K = K + 1
              STR(K:K) = TITLE(L:L)
              IF (STR(K:K) .EQ. '.') IP=K
	   ENDDO

           IDIF=IE-IA+1
           STR2(1:8) = '        '

2002       IF (IP .EQ. 0) THEN

C------       INPUT IS IN INTEGER FORMAT, CODE IT IN REAL FORMAT ---------
	      ID = 4-IDIF
              STR2(ID+1:ID+IDIF) = STR(ID+1:ID+IDIF)
              STR2(5:5) = '.'
           ELSE

             IFILL = 5-IP
C-------     FILL IN LEADING BLANKS -------------------------------------
	     IK=0
	     DO  L=IFILL+1,IDIF+IFILL
	        IK=IK+1
                STR2(L:L) = STR(IK:IK)
	     ENDDO
           ENDIF


C-------   TRANSFER NUMBER TO FNUMB(I) ------------------------------------

           READ(STR2,8000,ERR=9999) FNUMB(I)
8000	   FORMAT(F8.3)

	ENDDO

        NNN = ICOM-1
	WRITE(NOUT,7) (FNUMB(I),I=1,NNN)
7	FORMAT(1X,1PE10.3)
        IRTFLG = 0
        RETURN

9999    IRTFLG = 1
        WRITE(NOUT,91)
91      FORMAT(' *** NO TILT ANGLES IN TITLE.')
        RETURN

	END


#endif
