C++*******************************************************************
C
C $$ CRCSEC.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987 WADSWORTH CENTER FOR LABORATORIES AND          *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C $$ CRCSEC:    SUBROUTINE CALLED BY ORNTS.  SELECTS L AZIMUTHALLY
C               EQUISPACED SAMPLES FROM A HALF CIRCLE (RADIUS R)
C               AROUND (NSAM/2+1,NROW/2+1).  BILINEAR INTERPOLATION
C               USED.  SORTING USED FOR MOST EFFICIENT READ ACCESS.
C
C       CALL CRCSEC(LUN,BUF,NSAM,NROW,SEC,L,NRING,DTH)
C         LUN     LOGICAL UNIT NUMBER OF IMAGE FILE
C         BUF     BUFFER ARRAY OF LENGTH 2*NSAM+3*L
C         NSAM    NUMBER OF SAMPLES
C         NROW    NUMBER OF ROWS
C         SEC     REAL ARRAY TO RECEIVE L SAMPLES FROM CIRCLE OR HALF CIRCLE
C         L       NUMBER OF SAMPLES ON CIRCLE OR HALFCIRCLE
C         R       RADIUS OF CIRCLE
C	  DTH	  ANGULAR INCREMENT. THIS CONTROLS THE MODE OF OPERATION:
C		   IF L*DTH=180 DEGREES, WE HAVE ACF SEARCH
C		   IF L*DTH=360 DEGREES, WE HAVE DIRECT SEARCH
C
C--*******************************************************************
C
C
      SUBROUTINE CRCSEC(LUN,BUF,NSAM,NROW,SEC,L,R,DTH)
      DIMENSION BUF(1),SEC(1)
      INTEGER A1,A2,A3
C     DTH = 3.1415926/FLOAT(L)
      COD = COS(DTH)
      SID = SIN(DTH)
      MB2 = NSAM/2+1
      NB2 = NROW/2+1
      LP2 = L*2
      A1 = 2*NSAM
      A2 = A1 + L
      A3 = A2 + L
      XO = MB2
      YO = NB2
      CO = R
      SI = 0.
      DO  KTH = 1,L
      X = XO + CO
      Y = YO + SI
      SEC(KTH) = 0.
      BUF(A1+KTH) = X
      BUF(A2+KTH) = Y
      BUF(A3+KTH) = KTH
      X = CO
      CO = CO*COD - SI*SID
      SI = SI*COD + X*SID
      ENDDO
      CALL SORT(BUF(A2+1),BUF(A1+1),BUF(A3+1),L)
C     DO 51 I = 1,L
C51   WRITE(3,52) BUF(I+A2),BUF(I+A1),BUF(I+A3)
C52   FORMAT(1X,3F10.2)
      JPREV = 0
      DO  LIN = 1,L
      J = BUF(A2+LIN)
      Y = BUF(A2+LIN) - J
      YREM = 1.-Y
      JP1 = J+1
      IF(J.LT.1.OR.JP1.GT.NROW) CALL ERRT(7,'CRCSEC',NE)
      IF(J.EQ.JPREV) GOTO 65
      IF(J.NE.JPREV+1) GOTO 55
      CALL REDLIN(LUN,BUF(NS1),NSAM,JP1)
      NS1P = NS1
      NS1 = NS2
      NS2 = NS1P
      GOTO 60
55    NS1 = 1
      NS2 = NSAM + 1
      CALL REDLIN(LUN,BUF(NS1),NSAM,J)
      CALL REDLIN(LUN,BUF(NS2),NSAM,JP1)
60    JPREV = J
65    K = BUF(A1+LIN)
      NLIN = BUF(A3+LIN)+.5
      X = BUF(A1+LIN) - K
      XREM = 1.-X
      NLINE = (NLIN-1)/L * (L*2)
      NLINS = NLINE + MOD(NLIN-1,L)+1
      SEC(NLINS) = YREM*(XREM*BUF(NS1+K-1)+X*BUF(NS1+K))
     1            + Y  *(XREM*BUF(NS2+K-1)+X*BUF(NS2+K))
      ENDDO
      RETURN
      END
