

C++************************************************************************
C
C CSCLIP.FOR  -- CREATED JAN 85
C
C **************************************************************************
C *  AUTHOR: R. BANERJEE                                                   *
C *  COPYRIGHT (C)1985 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C $$ CSCLIP    CLIPPING OF LINES FOR THE USUAL CASE OF AN UPRIGHT
C              RECTANGULAR WINDOW.
C
C    PURPOSE:  TO CLIP LINES JOINING THE TWO POINTS SUCH THAT ONLY THE
C              PORTION INSIDE THE WINDOW IS RETAINED.
C              WE USE THE COHEN-SUTHERLAND ALGORITHM AS DISCUSSED IN
C              FUNDAMENTALS OF INTERACTIVE COMPUTER GRAPHICS,
C              J.D.FOLEY AND A.VAN DAM ;PAGES 146-149 
C
C    CSCLIP(X1,Y1,X2,Y2,XMIN,XMAX,YMIN,YMAX,ICLIP)
C
C        X1,Y1    CO-ORDINATE OF POINT P1 FOR LINE P1P2
C 
C        X2,Y2    CO-ORDINATE OF POINT P2 FOR LINE P1P2
C
C        XMIN,YMIN  CO-ORDINATE OF BOTTOM LEFT CORNER OF WINDOW
C
C        XMAX,YMAX  CO-ORDINATE OF TOP RIGHT CORENR OF WINDOW
C
C        X1,Y1 AND X2,Y2 CONTAINS THE MODIFIED CO-ORDINATE FOR
C        THE LINE AFTER CLIPPING.
C
C        ICLIP     A FLAG INDICATING THE CLIPPING STATUS AS FOLLOWS
C                  = 0    THE LINE WAS REJECTED BECAUSE IT WAS OUTSIDE
C                  = 1    THE LINE WAS ACCEPTED BECAUSE IT WAS INSIDE
C                  = 2    THE LINE WAS CLIPPED
C
C        THIS FLAG IS USED TO CONTROL THE MOVE AND DRAW COMMAND 
C        
C       CALLS:     CSOTCD
C
C--********************************************************************

       SUBROUTINE CSCLIP(X1,Y1,X2,Y2,XMIN,XMAX,YMIN,YMAX,ICLIP)
 
       LOGICAL ACCEPT,REJECT,DONE,OUTCD1(4),OUTCD2(4),TLOG
 
C..    INITIALIZE
       ACCEPT = .FALSE.
       REJECT = .FALSE.
       DONE = .FALSE.
       ICLIP = 0
 
C..      FIND THE OUT-CODE FOR POINT P2
         CALL CSOTCD(X2,Y2,XMIN,XMAX,YMIN,YMAX,OUTCD2)
 
C..    REPEAT THE LOOP
 
 5001 CONTINUE
 
C..      FIND THE OUT-CODE FOR POINT P1
         CALL CSOTCD(X1,Y1,XMIN,XMAX,YMIN,YMAX,OUTCD1)
 
C..      CHECK TRIVIAL REJECT (AT LEAST ONE SUFFIX FOR BOTH OUTCODES
C..      ARE TRUE)
 
         DO  I = 1, 4
            IF (OUTCD1(I) .AND. OUTCD2(I)) THEN
               REJECT = .TRUE.
               GOTO 4001
            ENDIF
	 ENDDO
 4001    CONTINUE
         IF (REJECT) THEN
            DONE = .TRUE.
            ICLIP = 0
            RETURN
         ENDIF
 
C..      CHECK TRIVIAL ACCEPT ( ALL OUTCODES ARE FALSE)
 
         DO  I = 1, 4
            IF (OUTCD1(I)) GOTO 4002
C..
            IF (OUTCD2(I)) GOTO 4002
 
            IF (I .EQ. 4) ACCEPT = .TRUE.
	 ENDDO
 4002    CONTINUE
         IF (ACCEPT) THEN
            DONE = .TRUE.
            IF (ICLIP .LT. 2) ICLIP = ICLIP + 1
            RETURN
         ENDIF
 
C..      SUBDIVIDE THE LINE SINCE AT MOST ONE END POINT IS INSIDE
C..      FIRST IF P1 IS INSIDE WINDOW, EXCHANGE POINTS 1 AND 2 AND 
C..      THEIR OUTCODES TO GUARANTEE THAT P1 IS OUTSIDE WINDOW.
 
         ICLIP = 1
         DO  I = 1, 4
       
C..         IF P1 IS OUTSIDE WINDOW DON'T BOTHER
 
            IF (OUTCD1(I)) GOTO 4003
 
C..         IF I = 4 AND WE HAVE NOT GONE OUT THEN P1 IS INSIDE WINDOW
 
            IF (I .EQ. 4) THEN
 
C..            SWAP P1 AND P2
 
               TEMP = X1
               X1 = X2
               X2 = TEMP
               TEMP = Y1
               Y1 = Y2
               Y2 = TEMP
               DO  J = 1,4
                  TLOG = OUTCD1(J)
                  OUTCD1(J) = OUTCD2(J)
                  OUTCD2(J) = TLOG
	       ENDDO
 
            ENDIF
 
 	 ENDDO
 4003    CONTINUE
 
C..      NOW PERFORM A SUBDIVISION, MOVE P1 TO THE INTERSECTION POINT
C..      USE THE FORMULAS   Y = Y1 + (X - X1) * SLOPE
C..                         X = X1 + (Y - Y1) / SLOPE
 
         IF (OUTCD1(1)) THEN
 
C..         DIVIDE LINE AT THE TOP OF WINDOW
 
            X1 = X1 + (X2 - X1) * (YMAX - Y1) / (Y2 - Y1)
            Y1 = YMAX
            GOTO 4004
         ENDIF
 
         IF (OUTCD1(2)) THEN
C..
C..         DIVIDE LINE AT BOTTOM OF WINDOW
C..
            X1 = X1 + (X2 - X1) * (YMIN - Y1) / (Y2 - Y1)
            Y1 = YMIN
            GOTO 4004          
         ENDIF
 
         IF (OUTCD1(3)) THEN
 
C..         DIVIDE LINE AT THE RIGHT EDGE OF WINDOW
 
            Y1 = Y1 + (Y2 - Y1) * (XMAX - X1) / (X2 - X1)
            X1 = XMAX
            GOTO 4004
         ENDIF
 
         IF (OUTCD1(4)) THEN
 
C..         DIVIDE LINE AT THE LEFT EDGE OF WINDOW
 
            Y1 = Y1 + (Y2 - Y1) * (XMIN - X1) / (X2 - X1)
            X1 = XMIN
         ENDIF
4004     CONTINUE
 
C..      GO BACK IF NOT DONE
 
         IF (.NOT. (DONE)) GOTO 5001
 5101 CONTINUE
           
 
	RETURN
	END
 
