C++*******************************************************************
C
C CSPSTX.FOR  -- CREATED DEC 84
C
C **************************************************************************
C *  AUTHOR: R. BANERJEE                                                   *
C *  COPYRIGHT (C)1984 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C    CSPSTX(LINE,NCHAR,LUN,NF,ISTFLG,ITEXT)
C
C    PURPOSE:  TO PUSH THE  TEXT INTO A CONTOUR STORAGE FILE AND 
C              UPDATE THE TXTCTR ARRAY 
C
C    PARAMETERS:
C        LINE           160 CHARACTERS CONTAINING THE LINE OF TEXT
C        NCHAR          NO. OF CHARACTERS IN THE TEXT 
C        ITEXT          A NUMBER RETURNED TO CALLER (TEXT I.D.) 
C        LUN            LOGICAL UNIT NUMBER TO BE ASSIGNED TO FILNAM.
C        NF             ERROR RETURN FLAG.
C                       NF = 0    NORMAL RETURN
C                       NF = 1    ERROR RETURN
C        ISTFLG         = 0 STORE THE BUFFER ONLY WHEN IT IS FULL
C                       = 1 STORE THE BUFFER EVEN IF IT IS NOT FULL   
C
C   VARIABLES:
C        CBUF           BUFFER FOR READING THE TEXT       
C        JOFP           ANGLE, SIZE, JUSTIFICATION, ORIENTATION, FONTS
C                       AND PEN TYPE FOR TEXT
C        XPOS           X-CO-ORDINATE OF TEXT
C        YPOS           Y-CO-ORDINATE OF TEXT
C        IRECLN         RECORD LENGTH.
C
C        
C        NOTE     THE FILE IS ALREADY OPENED BY CALLER
C                 CALLER SHOULD ASSIGN JOFP,XPOS,YPOS BEFORE CALLING
C                 THIS ROUTINE.
C                 THE LAST CALL TO THIS ROUTINE SHOULD BE MADE
C                 ALWAYS WITH ISTFLG = 1 SUCH THAT THE TEXT IS STORED. 
C
C                 IF THE FILE IS ALREADY SCALED (SCALE > 0.) THEN
C                 SCALE THE VALUES ELSE
C                 UPDATE XMAX,XMIN,YMAX,YMIN VALUES
C   
C   CALLED BY:    DENDRO        DPROFL        HISMAP        ?????
C                 DPROFD
C
C   CALLS:        WRTLIN        CSGTBN        REDLIN
C
C--********************************************************************

	SUBROUTINE CSPSTX(LINE,NCHAR,LUN,NF,ISTFLG,ITEXT)

        INCLUDE 'CONSTG.INC'

        COMMON /UNITS/LUNDOC,NIN,NOUT

        CHARACTER   * 8192 TXTARA
        CHARACTER   * 160  LINE
        EQUIVALENCE        (CBUF(1), TXTARA(1:1))
        EQUIVALENCE        (ISJIFP,FSJIFP)

C..     SET GOOD ERROR RETURN
        NF = 0

C..     CHECK WHETHER CAN STORE TEXT OR NOT
        IF (NTXT .GE. MAXTXT) THEN
           NF = 1
           WRITE(NOUT,*) 'NO MORE ROOM TO STORE TEXT'
           RETURN
        ENDIF

        IF (NCHAR .GT. 160) THEN
            WRITE(NOUT,*) 'TEXT LINE TRUNCATED TO 160 CHAR.'
            NCHAR = 160
        ENDIF

C..     CHECK WHETHER FILE IS ALREADY SCALED OR NOT
        IF (SCALE .EQ. 0.0) THEN

C..          FILE NOT SCALED, UPDATE XMAX,XMIN,YMAX,YMIN
             IF (XPOS .GT. XMAX) XMAX = XPOS
             IF (XPOS .LT. XMIN) XMIN = XPOS
             IF (YPOS .GT. YMAX) YMAX = YPOS
             IF (YPOS .LT. YMIN) YMIN = YPOS
        ELSE

C..          FILE IS SCALED, CHECK FOR RANGE AND THEN SCALE XPOS & YPOS

             IF ((XPOS .GT. XMAX) .OR. (XPOS .LT. XMIN)) THEN     
                NF = 1
                WRITE(NOUT,101) XPOS,XMAX,XMIN
  101           FORMAT(' TRYING TO PLACE TEXT AT XPOS = ',F8.1,
     &          /,1X,' INTO FILE WITH XMAX = ',F8.1,' AND XMIN = ',
     &          F8.1)
                RETURN
             ENDIF
             IF ((YPOS .GT. YMAX) .OR. (YPOS .LT. YMIN)) THEN     
                NF = 1
                WRITE(NOUT,102) YPOS,YMAX,YMIN
  102           FORMAT(' TRYING TO PLACE TEXT AT YPOS = ',F8.1,
     &          /,1X,' INTO FILE WITH YMAX = ',F8.1,' AND YMIN = ',
     &          F8.1)
                RETURN
             ENDIF

C..          SCALE XPOS,YPOS
             XPOS = (XPOS - XMIN) / SCALE
             YPOS = (YPOS - YMIN) / SCALE
        ENDIF      

C..     READ THE 4TH REC,ONLY IF 4TH REC IS NOT THE LAST RECORD READ/WRITTEN
        IF (LSRECD .NE. 4) CALL REDLIN(LUN,CBUF,IRECLN,3)

C..     INITIALIZE THE STARTING POSITION   
 5001 CONTINUE  
        IFRSUF = NTXSUF
        MAXSUF = IRECLN * 4    
C       ! SINCE RECORD LENGTH IS IN WORDS 
        J      = IFRSUF
        KTEXT  = 1
C..
C..     STORE THE TEXT AND CHECK IF RECORD LENGTH HAS BEEN EXCEEDED
 4001   CONTINUE
          IF (J .GT. MAXSUF) THEN

C..         NO MORE ROOM 
C..         STORE THE CURRENT RECORD AND CLEAR THE BUFFER
            CALL WRTLIN(LUN,CBUF,IRECLN,3)
            DO K= 1, IRECLN
  	        CBUF(K) = 0.0
	    ENDDO

C..         FLAG ERROR AND RETURN
            WRITE(NOUT,104)
  104       FORMAT(' NO MORE ROOM, TEXT TRUNCATED')
            NF = 1
            RETURN
         ENDIF

C..       STORE TEXT
          TXTARA(J:J) = LINE(KTEXT:KTEXT)
          J = J + 1   
          KTEXT = KTEXT + 1
          IF ( KTEXT .LE. NCHAR) GOTO 4001
 4101 CONTINUE

C..       UPDATE THE ENDING RECORD POSITION
          ITOSUF = J - 1

C..       IF ISTFLG IS SET THEN STORE THE RECORD
          IF (ISTFLG .EQ. 1) THEN
             CALL WRTLIN(LUN,CBUF,IRECLN,3)
          ENDIF

C..       NOW UPDATE THE TXTCTR RECORD 
C..       GET A CELL NO. FROM THE TXTCTR ARRAY
          CALL CSGTBN('TXTCTR',TXTCTR,5,MAXTXT,ISTTXT,LSTTXT,NOBIN)

C..       IF NOBIN <= 0 THEN ERROR
          IF (NOBIN .LE. 0) THEN
             WRITE(NOUT,103) NOBIN
  103        FORMAT(' ILLEGAL BIN NO. = ',I8)
             NF = 1
             RETURN
          ENDIF

          TXTCTR(1,NOBIN) = IFRSUF
          TXTCTR(2,NOBIN) = ITOSUF

C******************DEBUG
C          WRITE(46,9993) LINE(1:33),NCHAR,ITANGL,ITSIZE,JUST,
C     &                   INTEN,IFON,IPEN
C 9993     FORMAT(1X,A30,' NCH,<,SIZE,JUST,INT,FON,PEN: ',7I5)
C******************************************888

          ISJIFP = ITANGL * 1000000 + ITSIZE * 10000 + JUST * 1000 + 
     &             INTEN  * 100     + IFON   * 10    + IPEN

          TXTCTR(3,NOBIN) = FSJIFP
          TXTCTR(4,NOBIN) = XPOS
          TXTCTR(5,NOBIN) = YPOS
          ITEXT = NOBIN

C..       UPDATE THE NTXSUF & # OF TEXT
          NTXSUF = ITOSUF + 1
          NTXT   = NTXT + 1
          LSRECD = 4
C.. 
      RETURN
      END

