
C **************************************************************************
C
C CSRDCT.FOR               PIXPCM ADDED SEPT 86 al
C                          LENCOR ADDED APRIL 87 al
C                          ICNTFMT ADDED JULY 92 al
C
C **************************************************************************
C *  AUTHOR: R. BANERJEE                                                   *
C *  COPYRIGHT (C)1985 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C    CSRDCT(LUN)
C
C    PURPOSE:         TO READ CONTROL DATA FROM A CONTOUR STORAGE FILE
C
C    PARAMTERS:
C        LUN          LOGICAL UNIT NUMBER TO BE ASSIGNED TO FILNAM.
C
C    VARIABLES:
C        CBUF         BUFFER FOR READING THE DATA (IN COMMON BLOCK)      
C        IRECLN       RECORD LENGTH. (COMMON BLOCK)
C
C    CALLS:           REDLIN
C
C    CALLED BY:       CNOPN      OTHERS
C
C        0         2         3         4         5         6         7
C23456789012345678901234567890123456789012345678901234567890123456789012
C--********************************************************************

	SUBROUTINE CSRDCT(LUN,FILNMC,IRTFLG)

        INCLUDE 'CONSTG.INC'

        COMMON /UNITS/LUNDOC,NIN,NOUT
        COMMON /IOERR/IERR

        CHARACTER *(*) FILNMC
        CHARACTER * 1  NULL
        CHARACTER * 80 LINE
        DIMENSION      CONTRL2(6*IMAXCT),TXTCTR2(5*IMAXTX)
        EQUIVALENCE    (CBUF(1),LINE(1:1)),(CONTRL,CONTRL2)
        EQUIVALENCE    (TXTCTR,TXTCTR2)

	NULL=CHAR(0)

        IERR = 0

C       READ THE FIRST RECORD
        CALL REDLIN(LUN,CBUF,IRECLN,0)
        IF (IERR .NE. 0) THEN
           WRITE(NOUT,*)' *** CNT REDLIN ERROR ON RECORD: 1'
           STOP
        ENDIF
            
C       RECOVER THE CONTROL VALUES FROM THE FILE

C       MAXCNT IS MAX. NO. CONTOURS CONTROL RECORDS HOLD
        MAXCNT = CBUF(1)            

C       MAXTXT IS MAX. NO. TEXT THAT TEXT CONTROLS HOLD
        MAXTXT = CBUF(2)            

C       NO. OF RECORDS IN FILE
        MAXREC = CBUF(3)            

C       
        NDTSUF = CBUF(4)
             
C       NEXT TEXT SUFFIX
        NTXSUF = CBUF(5)            

C       FILE TYPE
        IFRMCS = CBUF(6)            

C       SCALING FACTOR (ALL DATA SCALED 0..1)
        SCALE  = CBUF(7)            

C       XMAX IS MAX. X IN DATA
        XMAX   = CBUF(8)            

C       XMIN IS THE MIN. X IN DATA
        XMIN   = CBUF(9)            

C       YMAX IS THE MAX. Y IN DATA
        YMAX   = CBUF(10)           

C       YMIN IS THE MIN. Y IN DATA
        YMIN   = CBUF(11)           

C       ICOMP IS A COMPRESSION FLAG (CURRENTLY UNUSED)
        ICOMP  = CBUF(12)           

        NDTCTL = CBUF(13)
           
C       NCNTRS IS NO. OF CONTOURS IN FILE
        NCNTRS = CBUF(14)           

C       ISTCON IS FIRST CONTOUR LOCATION
        ISTCON = CBUF(15)           
C       LSTCON IS LAST CONTOUR LOCATION
        LSTCON = CBUF(16)           
       
C       ISTTXT IS FIRST TEXT LOCATION      
        ISTTXT = CBUF(17)           

C       LSTTXT IS LAST TEXT LOCATION      
        LSTTXT = CBUF(18)           
       
C       NTXT IS NO. OF TEXT IN FILE
        NTXT   = CBUF(19)           

C       PIXPCM IS CONVERSION FACTOR FROM DATA UNITS TO CM.
        PIXPCM = CBUF(20)           
        IF (PIXPCM .LE. 0.0) PIXPCM = 1.0
 
C       LENCOR = used record length
        LENCOR = CBUF(21)           

C       LENCNT IS PHYSICAL LENGTH AS RECORDED IN THE FILE
        LENCNT = CBUF(22)

        IF (LENCNT .NE. IRECLN) THEN
          WRITE(NOUT,*) 
     &    'WARNING LENCNT:',LENCNT,' NOT EQUAL IRECLN:',IRECLN 
           LENTEMP = IRECLN
           CALL RDPRIS(IRECLN,LENTEMP,NLOOP,ILOOP,
     &        'Enter record length for file and data',IRTFLG)
           IF (IRTFLG .EQ. 0) THEN
              CLOSE(LUN)
#ifdef __osf__
              IRECLN4 = IRECLN
#else
              IRECLN4 = IRECLN * 4
#endif
              OPEN(UNIT=LUN,FILE=FILNMC,STATUS='OLD',FORM='UNFORMATTED',
     &             ACCESS='DIRECT',RECL=IRECLN4,ERR=5001)
C             ONCE OPENED IRECLN IS ONLY USED TO SET LENGTH OF READS AND WRITES
              IRECLN = LENTEMP
              LENCNT = LENTEMP
           ENDIF
        ENDIF

C       READ THE 2ND RECORD
        CALL REDLIN(LUN,CBUF,IRECLN,1)
        IF (IERR .NE. 0) THEN
           WRITE(NOUT,*)'*** CNT REDLIN ERROR ON RECORD: 2'
           STOP
        ENDIF

C       GET DATE, TIME AND TITLE AND PUT THEM INTO THE VARIABLES
        DATECS(1:10) = LINE(1:10)
        TIMECS(1:8)  = LINE(11:18)
        TITLCS(1:40) = LINE(19:58)

C       READ RECORD NO 3 & INITIALIZE TXTCTR
        CALL REDLIN(LUN,CBUF,IRECLN,2)
        IF (IERR .NE. 0) THEN
           WRITE(NOUT,*)'*** CNT REDLIN ERROR ON RECORD: 3'
           STOP
        ENDIF

        MAXT = MIN((IMAXTX*5),(MAXTXT * 5))
        DO  I = 1, MAXT
            TXTCTR2(I) = CBUF(I)
        ENDDO

C       READ RECORD NOS. 5 THRU NDTCTL + 4 AND INITIALIZE ARRAY CONTRL
C       CALCULATE MAXIMUM VALID NUMBERS MAXSUF STORED IN ONE RECORD
C       THEN READ ONE RECORD AT A TIME AND COPY MAXSUF NUMBERS TO 
C       THE CONTROL ARRAY AND THEN WE READ THE NEXT RECORD.

        NREC   = 4
        MAXSUF = IRECLN / 6
        MAXSUF = MAXSUF * 6
        MXCNTZ = MAXCNT * 6
        CALL REDLIN(LUN,CBUF,IRECLN,NREC)
        IF (IERR .NE. 0) THEN
           WRITE(NOUT,*)'*** CNT REDLIN ERROR ON NREC: ',NREC
           STOP
        ENDIF 
        J = 0

        DO  I = 1, MXCNTZ
           J = J + 1
           IF (J .GT. MAXSUF) THEN
              NREC = NREC + 1
              J = 1
              CALL REDLIN(LUN,CBUF,IRECLN,NREC)
              IF (IERR .NE. 0) THEN
                WRITE(NOUT,*)'*** CNT REDLIN ERROR ON RECORD: ',NREC+1
                STOP
              ENDIF
            ENDIF
            CONTRL2(I) = CBUF(J)
        ENDDO
 
        IF (ICSPR .GE. 1) THEN
C         PRINT THE FILE PARAMETERS

          WRITE(NOUT,107) DATECS,TIMECS,TITLCS
  107     FORMAT(' CREATION DATE: ',A10,' TIME: ',A8,/,
     &    '  TITLE: ',A40/)

          WRITE(NOUT,101) MAXCNT,MAXTXT,IFRMCS,MAXREC,NDTSUF,NTXSUF
          WRITE(NOUT,102) SCALE,XMAX,XMIN,YMAX,YMIN
          WRITE(NOUT,103) ICOMP,NDTCTL,NCNTRS,NTXT,
     &                    ISTCON,LSTCON,ISTTXT,LSTTXT
          WRITE(NOUT,*)   'IRECLN:',IRECLN
          WRITE(NOUT,*)   'LENCOR:',LENCOR
          WRITE(NOUT,*)   'LENCNT:',LENCNT

  101   FORMAT(1X,'FILE OPENED WITH FOLLOWING CONTROL PARAMETERS:',
     & /,1X,'MAX. # OF CONTOURS(FILE CAPACITY) = ',I6,
     & /,1X,'MAX. # OF TEXT(FILE CAPACITY)     = ',I6,
     & /,1X,'FILE FORMAT   = ',I5,
     & /,1X,'MAX. REC USED = ',I5,
     & /,1X,'NEXT DATA SUF = ',I5,5X,' NEXT TEXT SUF = ',I5)
  102   FORMAT(
     &   1X,'SCALE = ',E10.4,2X,' MAX-X = ',E10.4,2X,
     & ' MIN-X = ',E10.4,2X,
     & /,22X,' MAX-Y = ',E10.4,2X,' MIN-Y = ',E10.4)
  103   FORMAT(
     &   1X,'FILE COMP. =     ',I6,5X,' # OF CTRL. REC.  = ',I6,
     & /,1X,'# OF CONTOURS  = ',I6,5X,'       # OF TEXT  = ',I6,
     & /,1X,'NEXT BIN (CON) = ',I6,5X,' HIGHEST BIN(CON) = ',I6,
     & /,1X,'NEXT BIN (TXT) = ',I6,5X,' HIGHEST BIN(TXT) = ',I6)  

      ENDIF

      LSRECD = NREC + 1
      RETURN

5001  WRITE(NOUT,*) '*** ERROR IN CSRDCT OPENING CNT FILE'
      STOP
      END
