
C++*******************************************************************
C
C  CSSCAL.FOR  -- CREATED DEC 84
C                 MODIFIED JULY 86 al
C                 ADAPTED FOR 3-D DEC 86 al
C                 ADAPTED FOR NEW TEXT CONTOUR MARCH 88 al
C  CSSCALT.FOR    RENAMED IN 1993 TO AVOID CLASH WITH INTRINSIC ROUTINE
C **************************************************************************
C *  AUTHOR: R. BANERJEE                                                   *
C *  COPYRIGHT (C)1984 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C    CSSCALT(LUN,ISERCH)
C
C    PURPOSE:  TO SCALE THE CONTOUR DATA SUCH THAT ALL CO-ORDINATES
C              ARE BETWEEN 0.0 TO 1.0
C              THE XMAX,XMIN AND YMAX,YMIN VALUES HAVE ALREADY
C              BEEN UPDATED WHILE WE WERE PUSHING THE CONTOURS.
C              HOWEVER, IF WE HAD DELETED A CONTOUR THEN THESE VALUES
C              WILL ALSO INCLUDE THE DELETED CONTOURS ALSO.
C              FOR THIS REASON A FLAG ISERCH IS USED.
C              WHEN ISERCH = 0 THEN WE USED THE EXISTING VALUES OF
C              XMAX,XMIN,YMAX,YMIN
C              IF ISERCH = 1 THEN WE FIND THE VALUE OF XMAX,XMIN,
C              YMAX,YMIN BY ONE PASS THRU THE WHOLE FILE.
C  
C   PARAMETERS:   LUN            LOGICAL UNIT NUMBER TO BE ASSIGNED TO FILNAM.
C                 ISERCH
C
C   VARIABLES:    CBUF           FOR READING THE DATA
C                 IRECLN         RECORD LENGTH.
C
C--********************************************************************

	SUBROUTINE CSSCALT(LUN,ISERCH)

        COMMON /UNITS/LUNDOC,NIN,NOUT
        INCLUDE 'CONSTG.INC'

        LOGICAL SCLFIL
        EQUIVALENCE (ISJIFP,FSJIFP)
       
        DATA MININT/-32768/,MAXINT/32767/,FLTMIN/-10E30/,
     &       FLTZER/10E-30/,FLTMAX/10E30/

C..     SET THE FLAG IF THE FILE IS ALREADY SCALED ONCE BEFORE
        ISERC  = ISERCH
        SCLFIL = .FALSE.

C*********************8DEBUG
C**        WRITE(NOUT,*) 'IN CSSCALT, SCALE= ',SCALE
C****************************

        IF (SCALE .NE. 0.0) THEN 
C..        FILE IS ALREADY SCALED ONCE SO WE HAVE TO DO 2 PASSES.
C..        ON FIRST PASS USE SCALE TO GET TO THE ORIGINAL X-VAL,Y-VAL
C..        THEN FIND THE XMAX,YMAX ETC.

           SCLFIL = .TRUE.
           XMINPR = XMIN
           YMINPR = YMIN
           ISERC  = 1
        ENDIF

C..     IF FLAG IS SET THEN DO THE FIRST PASS, FLAG WILL BE SET FOR 
C..     UNSCALED FILES AND FILES WITH RECENT DELETIONS.  SCLFIL IS 
C..     TRUE ONLY FOR SCALED FILES.

        IF (ISERC .EQ. 1) THEN
            XMAX = -9.99E10
            XMIN = -XMAX
            YMAX = XMAX
            YMIN = XMIN

            DO  K2 = 1, LSTCON
C..            PROCESS THE VALID CONTOURS ONLY
               ISTREC = CONTRL(1,K2)
               IF (ISTREC .GT. 0) THEN

                  ISTSUF = CONTRL(2,K2)
                  LSTREC = CONTRL(3,K2)
                  LSTSUF = CONTRL(4,K2)
                  IRECD  = ISTREC

                  FSJIFP = CONTRL(5,K2)
                  THREED = .FALSE.
                  IF (IFRMCS .EQ. 4) THEN
C                    THIS IS A 3-D CONTOUR CONTAINING FILE
                     MARK = MOD(CONTRL(5,K2) / 1000,10.0)
                     IF (MARK .EQ. 1 .OR. MARK .EQ. 6) THREED = .TRUE.

                  ELSEIF (IFRMCS .EQ. -6 .AND. ISJIFP .LT. 0) THEN
C                    NEW STYLE TEXT CONTOUR
                     IF ((ISTSUF + 1) .GT. IRECLN) THEN
                        LSTSUF = 1
                     ELSE
                        LSTSUF = ISTSUF + 1
                        LSTREC = ISTREC
                     ENDIF
                  ENDIF

C**************************DEBUG
C        IF (IREP .LT. 10) THEN  
C          WRITE(0,9911) IFRMCS,ISJIFP,ISTREC,ISTSUF
C9911      FORMAT(' CSSCALT; IFRMCS,ISJIFP,ISTREC,ISTSUF:',/,
C     &    I4,I10,4I6)
C          IREP = IREP + 1
C        ENDIF
C***********************************************8

C..               READ THE IRECD-TH REC, IF IT IS NOT THE LAST RECORD READ
                  IF (LSRECD .NE. IRECD) THEN
C..                 IF THE FILE IS SCALED THEN STORE THE BUFFER
C..                 BEFORE READING ANOTHER RECORD
                    IF (SCLFIL) THEN
                      IRECM1 = LSRECD - 1
                      CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)
                    ENDIF
                    IRECM1 = IRECD - 1
                    CALL REDLIN(LUN,CBUF,IRECLN,IRECM1)
                    LSRECD = IRECD 
                  ENDIF

                  J = ISTSUF
 4001             CONTINUE
                  XDAT = CBUF(J) 
                  IF (SCLFIL) THEN
                     XDAT    = XDAT * SCALE + XMINPR
                     CBUF(J) = XDAT
                  ENDIF
C..               UPDATE THE VALUE OF XMAX, XMIN ETC.
                  IF (XDAT .GT. XMAX) XMAX = XDAT
                  IF (XDAT .LT. XMIN) XMIN = XDAT

                  IF (J .EQ. IRECLN) THEN
C..                  THE CBUFF IS COMPLETLY COPIED, SO NEED TO READ ANOTHER ONE.
                     IF (LSTREC .GT. IRECD) THEN
C..                     IF THE FILE IS SCALED THEN STORE THE BUFFER
C..                     BEFORE READING ANOTHER RECORD
                        IF (SCLFIL) THEN
                           IRECM1 = IRECD - 1
                           CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)
                        ENDIF
                        IRECD  = IRECD + 1
                        IRECM1 = IRECD - 1
                        CALL REDLIN(LUN,CBUF,IRECLN,IRECM1)
                        LSRECD = IRECD
                        J = 0
                     ENDIF
                  ENDIF
                  J = J + 1


                  YDAT = CBUF(J)
                  IF (SCLFIL) THEN
                     YDAT    = YDAT * SCALE + YMINPR
                     CBUF(J) = YDAT
                  ENDIF
                  IF (YDAT .GT. YMAX) YMAX = YDAT
                  IF (YDAT .LT. YMIN) YMIN = YDAT     
                  IF (J .EQ. IRECLN) THEN
C..                  THE CBUFF IS COMPLETLY COPIED, SO NEED TO READ ANOTHER ONE.
                     IF (LSTREC .GT. IRECD) THEN
C..                     IF THE FILE IS SCALED THEN STORE THE BUFFER
C..                     BEFORE READING ANOTHER RECORD
                        IF (SCLFIL) THEN
                           IRECM1 = IRECD - 1
                           CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)
                        ENDIF
                        IRECD  = IRECD + 1
                        IRECM1 = IRECD - 1
                        CALL REDLIN(LUN,CBUF,IRECLN,IRECM1)
                        LSRECD = IRECD
                        J = 0
                     ENDIF
                  ENDIF
                  J = J + 1

                  IF (THREED) THEN
                    IF (J .EQ. IRECLN) THEN
C..                   NEED TO READ ANOTHER CBUFF
                       IF (LSTREC .GT. IRECD) THEN
C..                      IF THE FILE IS SCALED THEN STORE THE BUFFER
C..                      BEFORE READING ANOTHER RECORD
                         IF (SCLFIL) THEN
                           IRECM1 = IRECD - 1
                           CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)
                         ENDIF
                         IRECD  = IRECD + 1
                         IRECM1 = IRECD - 1
                         CALL REDLIN(LUN,CBUF,IRECLN,IRECM1)
                         LSRECD = IRECD
                         J = 0
                      ENDIF
                    ENDIF
                    J = J + 1
                  ENDIF

C..               CHECK IF WE HAVE REACHED THE END OF CONTOUR DATA
                  IF ((IRECD .LT. LSTREC) .OR. (J .LE. LSTSUF))
     &            GOTO 4001

C*******************8 DEBUG
C                  IF (NBUG .LE. 0 .AND. XMAX .GT. 100000) THEN
C                      NBUG = 10
C                      WRITE(6,*)'K2,ISTREC,XMAX,YMAX,ISTCON'
C                      WRITE(6,*) K2,ISTREC,XMAX,YMAX,ISTCON
C
C                      WRITE(6,*) 'ISTSUF,LSTREC,LSTSUF'
C                      WRITE(6,*) ISTSUF,LSTREC,LSTSUF
C                  ENDIF
C********************************
               ENDIF
	    ENDDO

C..         STORE THE LAST RECORD READ TO BE SURE THAT THE DATA IS UPDATED
            IRECM1 = LSRECD - 1
            CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)

C..         COMPARE THE TEXT POSITIONS NOW
            DO  J = 1,LSTTXT
               IF (TXTCTR(1,J) .GT. 0) THEN

C..               PROCESSING ONLY VALID TEXT
                  XPOS = TXTCTR(4,J)
                  YPOS = TXTCTR(5,J)
                  IF (SCLFIL) THEN
                     XPOS = XPOS * SCALE + XMINPR
                     YPOS = YPOS * SCALE + YMINPR
                     TXTCTR(4,J) = XPOS
                     TXTCTR(5,J) = YPOS
                  ENDIF 
                  IF (XPOS .GT. XMAX) XMAX = XPOS
                  IF (XPOS .LT. XMIN) XMIN = XPOS
                  IF (YPOS .GT. YMAX) YMAX = YPOS
                  IF (YPOS .LT. YMIN) YMIN = YPOS
               ENDIF
 	    ENDDO

        ENDIF

C..     XMAX,XMIN, YMAX, YMIN ARE ALL CORRECT NOW, SO FIND THE SCALE FACTOR.
        XSCAL = (XMAX - XMIN) 
        YSCAL = (YMAX - YMIN) 
        SCALE = YSCAL
        IF (XSCAL .GT. SCALE) SCALE = XSCAL 

        IF (SCALE .LT. FLTZER) THEN
           WRITE(NOUT,1199)
1199       FORMAT(' PROGRAMMING ERROR: ATTEMPTED SCALING BY 0.0',/,
     &            ' PROGRAM HALTED TO AVOID DAMAGE TO DATA. CONSULT',
     &            ' YOUR FRIENDLY PROGRAMMER.',//)
           STOP
        ENDIF
        SCALE1 = 1.0 / SCALE
 
C..     NOW DO THE SECOND PASS, SCALE ALL THE DATA POINTS AND TEXT POINTS
        DO 1003 K2 = 1, LSTCON

C..        PROCESS ONLY THE VALID CONTOURS ONLY
           ISTREC = CONTRL(1,K2)
           IF (ISTREC .GT. 0) THEN
              ISTSUF = CONTRL(2,K2)
              LSTREC = CONTRL(3,K2)
              LSTSUF = CONTRL(4,K2)
              FSJIFP = CONTRL(5,K2)
              IRECD  = ISTREC

              THREED = .FALSE.
              IF (IFRMCS .EQ. 4) THEN
C                THIS IS A 3-D OR VIEWS CONTOUR CONTAINING FILE
                 MARK = MOD(CONTRL(5,K2) / 1000,10.0)
                 IF (MARK .EQ. 1 .OR. MARK .EQ. 6) THREED = .TRUE.

              ELSEIF (IFRMCS .EQ. -6 .AND. ISJIFP .LT. 0) THEN
C                NEW STYLE TEXT CONTOUR
                 IF ((ISTSUF + 1) .GT. IRECLN) THEN
                    LSTSUF = 1
                 ELSE
                    LSTSUF = ISTSUF + 1
                    LSTREC = ISTREC
                 ENDIF

C**************************DEBUG
C        IF (IREP2 .LT. 6) THEN  
C          WRITE(0,5911) ISJIFP,ISTREC,ISTSUF,LSTREC,LSTSUF
C5911      FORMAT(' CSSCAL2; ISJIFP,1REC,1SUF,LREC,LSUF:',/,I10,4I6)
C          XT = CBUF(ISTSUF)
C          IF ((ISTSUF + 1) .gt. maxsuf) goto 6548
C          YT = CBUF(ISTSUF+1)
C          IF ((ISTSUF + 2) .gt. maxsuf) goto 6548
C          NCHAR = CBUF(ISTSUF+2)
C          IF ((ISTSUF + 3) .gt. maxsuf) goto 6548
C          FDATA = CBUF(ISTSUF+3)
C6548      WRITE(0,9844) XT,YT,NCHAR,CDATA,IRECD
C9844      FORMAT(' CSSCAL2; (',G11.3,',',G11.3,')',I5,':',A4,
C     &            ': IREC',I5)
C          IREP2 = IREP2 + 1
C        ENDIF
C********************************************
               ENDIF

C..            READ THE IRECD-TH REC, IF IT IS NOT THE LAST RECORD READ
               IF (LSRECD .NE. IRECD) THEN
C..               BEFORE READING STORE THIS RECORD IN THE FILE
                  IRECM1 = LSRECD - 1
                  CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)
                  IRECM1 = IRECD - 1
                  CALL REDLIN(LUN,CBUF,IRECLN,IRECM1)
                  LSRECD = IRECD 
               ENDIF

               J = ISTSUF
 4002          CBUF(J) = (CBUF(J) - XMIN) * SCALE1
               IF (J .EQ. IRECLN) THEN
C..               THE CBUFF IS COMPLETLY EMPTY. READ ANOTHER ONE.
                  CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)   
                  IRECD = IRECD + 1
                  IRECM1 = IRECD - 1
                  CALL REDLIN(LUN,CBUF,IRECLN,IRECM1)
                  LSRECD = IRECD  
                  IF (IRECD .GT. LSTREC) GOTO 1003
                  J = 0
               ENDIF
               J = J + 1

               CBUF(J) = (CBUF(J) - YMIN)  * SCALE1
               IF (J .EQ. IRECLN) THEN
C..               THE CBUFF IS COMPLETLY EMPTY. READ ANOTHER ONE.
                  CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)   
                  IRECD = IRECD + 1
                  IRECM1 = IRECD - 1
                  CALL REDLIN(LUN,CBUF,IRECLN,IRECM1)
                  LSRECD = IRECD  
                  IF (IRECD .GT. LSTREC) GOTO 1003
                  J = 0
               ENDIF
               J = J + 1

               IF (THREED) THEN
                 IF (J .EQ. IRECLN) THEN
C..                THE CBUFF IS COMPLETLY EMPTY. READ ANOTHER ONE.
                   CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)   
                   IRECD = IRECD + 1
                   IRECM1 = IRECD - 1
                   CALL REDLIN(LUN,CBUF,IRECLN,IRECM1)
                   LSRECD = IRECD  
                   IF (IRECD .GT. LSTREC) GOTO 1003
                   J = 0
                 ENDIF
                 J = J + 1
               ENDIF

C..            CHECK IF WE HAVE REACHED THE END OF THIS CONTOUR 
               IF ((IRECD .LT. LSTREC) .OR. (J .LE. LSTSUF))GOTO 4002

C**************************DEBUG
C        IF (ISJIFP .LT. 0 .AND. IREP2 .LT. 6) THEN  
C          XT = CBUF(ISTSUF)
C          YT = CBUF(ISTSUF+1)
C          NCHAR = CBUF(ISTSUF+2)
C          FDATA = CBUF(ISTSUF+3)
C          WRITE(0,3984) XT,YT,NCHAR,CDATA
C3984      FORMAT(' CSSCAL2END; (',G11.3,',',G11.3,')',I5,':',A4)
C          IF (J .GT. (MAXSUF-4)) GOTO 4455
C          X2T = CBUF(J-2)
C          Y2T = CBUF(J-1)
C          NCHAR = CBUF(J)
C          FDATA = CBUF(J+1)
C4455      WRITE(0,3983) X2T,Y2T,NCHAR,CDATA,J
C3983      FORMAT(' (J-1),(J-1),(J),(J+1); (',G11.3,',',G11.3,')',I5,
C     &            ':',A4,': J=',I5)
C        ENDIF
C********************************************

           ENDIF
1003	 CONTINUE 	 

C..      STORE THE LAST RECORD READ TO BE SURE THAT THE DATA IS UPDATED
         IRECM1 = LSRECD - 1
         CALL WRTLIN(LUN,CBUF,IRECLN,IRECM1)

C..      SCALE THE TEXT POSITIONS NOW
         DO  J = 1,LSTTXT
            IF (TXTCTR(1,J) .GT. 0) THEN
C..            PROCESSING ONLY VALID TEXT
               TXTCTR(4,J) = (TXTCTR(4,J) - XMIN)  * SCALE1
               TXTCTR(5,J) = (TXTCTR(5,J) - YMIN)  * SCALE1
            ENDIF
 	 ENDDO


C*********************8DEBUG
C**        WRITE(NOUT,*) 'IN CSSCALT, FINAL SCALE= ',SCALE
C****************************

      RETURN
      END

