
C**********************************************************************
C
C  DENDRO.FOR-- ADAPTED FOR METAFILE FORMATION 3 NOVEMBER 86 al
C
C**********************************************************************
C DRAWS DENDROGRAM AND FORMS A PLOT METAFILE FOR THE DENDROGRAM
C
C BASED ON ARBRE.FOR,    A DENDROGRAM PGM BY - JEAN-PIERRE BRETAUDIERE 
C THE UNIVERSITY OF TEXAS HEALTH SCIENCE CENTER AT HOUSTON                      
C MEDICAL SCHOOL - DEPARTMENT OF PATHOLOGY AND LABORATORY MEDICINE              
C P.O. BOX 20708, HOUSTON, TX 77225.                                            
C*--------------------------------------------------------------------*         
C*                                                                
C*      TREE DESCRIPTION 
C*                                                                 
C*      NKLA SUMMITS              JFIN = 2 * NKLA - 1                
C*      ARRAYS PROVIDED BY CHAVA  VAL(JFIN), LA(NKLA), LB(NKLA)     
C*                                PK(JFIN), IDK(NKLA)      
C*                                                              
C*      WORKING ARRAYS            NO(JFIN), V(NKLA), NT(NKLA),     
C*                                IW(NKLA), W(NKLA), IV(NKLA),    
C*                                NUM(NKLA)                          
C*
C       CALLS:         CNFILE      CSOPEN       CSPSTX
C                      CSSCALT      CSPUSH
C
C       CALLED BY:     NOYAU
C
C*--------------------------------------------------------------------*         

      SUBROUTINE DENDRO(NKLA, JFIN, VAL, LA, LB, PK, IDK, 
     &                 NO,NUM,NT,IV,IW,V,W,VMIN,VMAX)

      INCLUDE 'CONSTG.INC'
      INCLUDE 'CMBLOCK.INC' 

      COMMON  /POLY/  HOLES
      PARAMETER       (NSIZE=2000)
      COMMON /COMMUN/ DATA(3,NSIZE)
      
C     LIMITED TO 200 LEAVES ON TREE BY NKMAX
      PARAMETER      (NKMAX = 200)

      DIMENSION      VAL(JFIN),PK(JFIN),V(NKLA),W(NKLA)
      INTEGER        LA(NKLA),LB(NKLA),IDK(NKLA),NO(JFIN),NT(NKLA)
      INTEGER        IV( NKLA),NUM(NKLA),IW(NKLA),INUM
      DIMENSION      YT(NKMAX),X(NKMAX)
      LOGICAL        OLD,IOPEN,HOLES
      CHARACTER * 81 CSFILE
      CHARACTER * 80 LINE
      CHARACTER * 3  CVJ
      CHARACTER * 7  DISP
      CHARACTER * 6  CVMIN,CVMAX
      CHARACTER * 4  CNUM,CPOID
      EQUIVALENCE    (INUM,CNUM)
      CHARACTER * 1  NULL,LIGN(102),IBLAN,ISTAR

      DATA   LIG/102/,IBLAN/' '/ , ISTAR/'.'/
CNO_SAVE
      NULL=CHAR(0)

C     DON'T CHECK FOR COUNTER-CLOCKWISE CONTOURS
      HOLES = .FALSE.
      ICSPR  = -2

C..   FIND META-FILE NAME
      LUN    = 10
C     SET MAXIMUM NO. OF CONTOURS TO 200 ( I ESTIMATE THIS IS ENOUGH)
      MAXCNT = 200
      DISP   = 'UNK'
      CALL CNFILE(CSFILE,NLETC,'DENDROGRAM',
     &      DISP,DATEXC,LUN,IRTFLG)
      IF (IRTFLG .EQ. -1) RETURN
      
C     SET LINE AND TEXT PARAMETERS
      MARK   = 0
      ISHAP  = 0
      IPEN   = 0
      INTEN  = 9
      LINTP  = 0
      ITANGL = 0
      ITSIZE = 24
      IORN   = 0
      JUST   = 0
      IFON   = 0
      IPEN   = 0
      ZCOO   = 0.0

      JDEB   = NKLA + 1     
      VMIN   = VAL(JDEB)    
      VMAX   = VAL(JFIN)          

      DO    J = JDEB,JFIN                                                   
        IR    = J - JDEB + 1           
        IA    = LA(IR)        
        IB    = LB(IR)         
        IF (VAL(J) .LT. VMIN) VMIN = VAL(J) 
        IF (VAL(J) .GT. VMAX) VMAX = VAL(J)
        NO(IA)= J          
        NO(IB)= J         
      ENDDO
      NO(JFIN)= JFIN  

C     PRINT DESCRIPTION OF THE HIERARCHY CLASSES               
      WRITE (NDAT,600)     
  600 FORMAT ('1','NODE  INDEX  SENIOR  JUNIOR  SIZE'    ,5X,  
     #  'DESCRIPTION OF THE HIERARCHY CLASSES'     /' ',64('. ') / )

      DO   J = JDEB,JFIN         
        NT(1) = J 
        KPT   = 0 
        JI    = 1 
   20   IF (NT(JI) .GT. NKLA) GO TO  30    
        K       = NT(JI)   
        KPT     = KPT + 1  
        IV(KPT) = IDK(K)  
        IW(KPT) = K       
        NUM(NKLA-KPT+1) = IDK(K)
        JI    = JI - 1         
        GO TO  40              
   30   IJ    = JI + 1         
        NI    = NT(JI) - JDEB + 1
        NT(IJ)= LA(NI)           
        NT(JI)= LB(NI)           
        JI    = JI + 1           
   40   IF (JI .NE. 0) GO TO  20 
        IR    = J - JDEB + 1     
        WRITE (NDAT,610)  J, VAL(J),LA(IR),LB(IR),KPT,(IV(KK),KK=1,KPT)
  610   FORMAT (//1X,I4,F9.3,I5,I6,I7,5X,18(1X,A4),/' ',36X,18(1X,A4) ,
     #  /' ',36X,18(1X,A4)/' ',36X,18(1X,A4),/' ',36X,18(1X,A4 ))
        I1    = IW(1)            
        I2    = IW(KPT)          
        W(I1) = NO(J) + 0.0001   
        W(I2) = NO(I2) + 0.0001  
      ENDDO

C..   PRINT GRAPH OF DENDROGRAM  
      WRITE (NDAT,620)  VMIN, VMAX
  620 FORMAT ('1',5X, 'WEIGHT',4X, 'INDEX' ,9X,'DENDROGRAM',    
     #        4X, '(SCALE  ',2F7.2,' )',//)                 

      DO  J = 1,NKLA
        K     = W(J)      
        W(J)  = VAL(K)
      ENDDO

      DO  J = 1,NKLA   
        IWI   = IW(J)     
        V(NKLA-J+1)= W(IWI)
      ENDDO

      DO  L = 1,LIG                                                        
        LIGN(L)= IBLAN
      ENDDO

      PAS = FLOAT(LIG) /(VMAX - VMIN)

C     SET WINDOW SIZE (SCALING WILL BE DONE IN PLOT ROUTINES)
      XWIN = 120
      YWIN = 100

C     FIND SCALE FOR TREE (NOT INCLUDING LABELS)
      YSCALE = YWIN / (VMAX - VMIN)   
C     FIND HORIZ DISTANCE BETWEEN LEAVES
      XDIS = XWIN / (NKLA - 1)
C     LENGTH OF LEAVES
      YLEAF = -5.0

      ISTFLG = 0

C     PUT FILENAME AT TOP
      YPOS = 110.0
      XPOS = 0.0
      CALL CSPSTX(CSFILE,NLETC,LUN,NF,0,ITEXT)

C     RIGHT JUSTIFIED Y LABELS
      JUST = 2

C     LABEL Y AXIS
      XPOS = -6.0

      YPOS = -21.0
      LINE = 'WEIGHT'
      CALL CSPSTX(LINE,6,LUN,NF,0,ITEXT)

      YPOS = -14.0
      LINE = 'INDEX '
      CALL CSPSTX(LINE,6,LUN,NF,0,ITEXT)
 
      YPOS = -17.0
      LINE = 'X1000 '
      CALL CSPSTX(LINE,6,LUN,NF,0,ITEXT)
      
      YPOS = -7.0
      LINE = 'CLASS '
      CALL CSPSTX(LINE,6,LUN,NF,0,ITEXT)

      YPOS = 0.0           
      WRITE(CVMIN,8805)VMIN
 8805 FORMAT(F5.3)
      CALL CSPSTX(CVMIN,6,LUN,NF,0,ITEXT)

      YPOS = 100.0
      WRITE(CVMAX,8805)VMAX
      CALL CSPSTX(CVMAX,6,LUN,NF,0,ITEXT)

      YPOS = 50.0
      ITANGL = 90
      LINE   = 'SCALE '
      CALL CSPSTX(LINE,6,LUN,NF,0,ITEXT)

      ITANGL = 0
C     CENTER X AXIS LABELS
      JUST = 1

      DO   J = 1,NKLA
C      DO   J = NKLA,1,-1

         LX   = (V(J) - VMIN) * PAS + 1.0
         IF (LX .LT. 1)   LX = 1
         IF (LX .GT. LIG) LX = LIG

         DO  II = 1,LX
            LIGN(II) = ISTAR
   	 ENDDO

C        FIND NODE HEIGHT (Y)
         X(J)    = (J-1) * XDIS
         YT(J)   = (V(J) - VMIN) * YSCALE 
         JJ      = IW(NKLA-J+1)
         POID    = PK(JJ)

C        SET ISTFLG TO PUSH LAST LABELS AND LINES
         IF (J .EQ. NKLA) ISTFLG = 1
C$         IF (J .EQ. 1) ISTFLG = 1

C        SET LEAF POSITION
         XPOS  = X(J) 

C        LABEL THE LEAF WITH WEIGHT
         YPOS  = -21.0
         IPOID = POID
         WRITE(CPOID,8800)IPOID
 8800    FORMAT(I4)
         IT = 1
         IF (CPOID(:1) .EQ. ' ') IT = 2
         CALL CSPSTX(CPOID(IT:4),5-IT,LUN,NF,0,ITEXT)

C        LABEL LEAF WITH CLASS NUMBER NUM(J)
         YPOS = -7.0
         INUM = NUM(J)
         CALL CSPSTX(CNUM(2:4),3,LUN,NF,0,ITEXT)

         IF (J .NE. NKLA) THEN
C           LABEL THE LEAF WITH THE INDEX V(J) TIMES 1000
            YPOS = -14.0
            IVT  = V(J) * 1000
            WRITE(CVJ,8801)IVT
 8801       FORMAT(I3)
            CALL CSPSTX(CVJ,3,LUN,NF,0,ITEXT)

            WRITE (NDAT,630)  POID, V(J), NUM(J), (LIGN(L),L=1,LIG)
  630       FORMAT (' ',2F10.3,1X,A4,2X,'..',102A1)   
 
         ELSE
C           LABEL THE LEAF WITH NULL INDEX
            YPOS = -14.0
            LINE = ' --'
            CALL CSPSTX(LINE,3,LUN,NF,0,ITEXT)

            WRITE (NDAT,650)  POID, NUM(NKLA), (LIGN(L),L=1,LIG)  
  650       FORMAT (' ',F10.3,4X,6('-'),1X,A4,2X,'..',102A1)
         ENDIF

         IF (LX .EQ. 1) GO TO  110                               
         LX    = LX - 1 

         DO   LL = 1,LX
            LIGN(LL)= IBLAN
  	 ENDDO

  110    WRITE (NDAT,640)  (LIGN(L),L=1,LIG)
  640    FORMAT (' ',29X,102A1)

  
      ENDDO	

C     PUSH A DUMMY CHAR. INTO TEXT SO THAT RIGHT BOTTEM MARGIN IS IMPROVED
      XPOS = XPOS + 8.0
      YPOS = -22.0
      LINE = ' '
      CALL CSPSTX(LINE,1,LUN,NF,0,ITEXT)

C     PUSH  DUMMY CHAR. INTO TEXT SO THAT LEFT BOTTEM MARGIN IS IMPROVED
      XPOS = -22.0
      CALL CSPSTX(LINE,1,LUN,NF,1,ITEXT)
      
C     START I SWEEP
      DO 300 I=1,NKLA
         YTI   = YT(I)

C        DRAW VERTICAL LINE
         DATA(1,1) = X(I)
         DATA(2,1) = YLEAF
         DATA(1,2) = X(I)
         DATA(2,2) = YTI
         NDATA     = 2
         IF (I .EQ. NKLA) GOTO 340

C        FIND LENGTH OF HORIZONTAL LINE
         DO  J = I+1,NKLA
            YTJ   = YT(J)
            IF (YTJ .GE. YTI) GOTO 330
  	 ENDDO
C        NO HIGHER BRANCH FOUND, POSSIBLE ERROR
         GOTO 300

C        DRAW HORIZONTAL LINE
  330    DATA(1,3) = X(J)
         DATA(2,3) = YTI
         NDATA = 3

C        PUSH DATA INTO FILE
  340    CALL CSPUSH(LUN,DATA,NDATA,NF,1,ICNTR)

  300 CONTINUE

C     TICK MARKS AT Y = 0 AND Y = YMAX
      NDATA = 2
      DATA(1,1) = -3.0
      DATA(2,1) =  0.0
      DATA(1,2) = -1.0
      DATA(2,2) =  0.0
      CALL CSPUSH(LUN,DATA,NDATA,NF,1,ICNTR)

      DATA(2,1) = 100.0
      DATA(2,2) = 100.0
      CALL CSPUSH(LUN,DATA,NDATA,NF,1,ICNTR)

C..   SCALE THE META-FILE, IF ISERCH = 0 THEN SCALING IS DONE BY ONE PASS
      ISERCH = 0
      CALL CSSCALT(LUN,ISERCH)

C..   CLOSE THE META-FILE 
      CALL CSTRAP(LUN)
      CLOSE(LUN)
      END
