C *NAME* = DEUCL.FOR                   31-JAN-86      13:23:06                  
C----------------------------------=*=----------------------------------        
C           COPYRIGHT 1986 - JEAN-PIERRE BRETAUDIERE                            
C THE UNIVERSITY OF TEXAS HEALTH SCIENCE CENTER AT HOUSTON                      
C MEDICAL SCHOOL - DEPARTMENT OF PATHOLOGY AND LABORATORY MEDICINE              
C P.O. BOX 20708, HOUSTON, TX 77225.                                            
C DISCLAIMER: NO GUARANTEES OF PERFORMANCE ACCOMPANY THIS SOFTWARE,             
C NOR IS ANY RESPONSIBILITY ASSUMED ON THE PART OF THE AUTHOR                   
C OR THE INSTITUTION.  ALL THE SOFTWARE HAS BEEN TESTED EXTENSIVELY             
C AND EVERY EFFORT HAS BEEN MADE TO INSURE ITS RELIABILITY.                     
C THE FOLLOWING SOFTWARE MAY NOT BE COMMUNICATED TO A THIRD PARTY               
C UNDER ANY FORM WITHOUT WRITTEN PERMISSION OF THE AUTHOR.                      
C PLEASE REPORT ANY BUG(S) TO JPB AT (713) 792 52 00.                           
C OR THRU ARPANET AT  ......!rice!ots!ut-path!suprim                            
C*--------------------------------------------------------------------*         
C*                                                                    *         
C*      CONSTRUCTION OF THE MONO-INDEXED TABLE OF DISTANCES BETWEEN   *         
C*      THE NKLA CLASSES IN THE EUCLIDEAN SPACE CHARACTERIZED BY THE  *         
C*      KFAC FIRST FACTORIAL COORDINATES.                             *         
C*                                                                    *         
C*      INPUT:                                                        *         
C*      NKDIM        = MAJORIZER OF NKLA FOR T(*,KFAC)                *         
C*      NKLA         = NUMBER OF CLUSTERS FOR THE GIVEN PARTITION     *         
C*      MCARD        = NKLA * (NKLA-1) / 2  = DIMENSION FOR D(*)      *         
C*      JFIN         = 2 * NKLA - 1   = DIMENSION OF WORKING ARRAYS   *         
C*      ICARD        = NUMBER OF OBJECTS TO BE CLUSTERED              *         
C*      KFAC,NFAC    = FIRST KFAC FACTORIAL COORDINATES FROM A TOTAL  *         
C*                     OF NFAC                                        *         
C*      KLAS(ICARD)  = GIVEN PARTITION IN NKLA CLUSTERS.              *         
C*      NGUS         = FILE CONTAINING THE FACTORIAL COORDINATES OF   *         
C*                     THE GRAVITY CENTERS OF THE CLUSTERS.                     
C*                                                                    *         
C*      OUTPUT:                                                       *         
C*      T(NKLA,KFAC) = TABLE CONTAINING THE COORDINATES OF THE NKLA   *         
C*                     CENTERS                                        *         
C*      PK(JFIN)    = WEIGHT OF THE NKLA CLUSTERS                     *         
C*      D(MCARD)    = MATRIX OF DISTANCES BETWEEN CLUSTERS            *         
C*                                                                    *         
C*      WORKING ARRAY ... CI(KFAC)                                    *         
C*      INTERNAL FUNCTION  ... MONO.                                  *         
C*                                                                    *         
C==02.03.81                                                           *         
C==01.12.86                                                           *         
C*--------------------------------------------------------------------*         
      SUBROUTINE DEUCL (NKDIM,NKLA,MCARD,ICARD,KFAC,NFAC,                       
     #                  KLAS, D, T, PK, CI, NGUS )                              
C                                                                               
      DIMENSION KLAS(ICARD),D(MCARD),PK(NKLA),CI(NFAC),                         
     #          T(NKDIM,KFAC)                                                   
C                                                                               
C......... MONO-INDEXING OF THE TABLE OF DISTANCES                              
C                                                                               
      MONO(K1,K2)= MIN0(K1,K2) + ((MAX0(K1,K2)-1)*(MAX0(K1,K2)-2)/2)            
C                                                                               
C......... GRAVITY CENTER OF THE NKLA CLASSES                                   
C 
CNO_SAVE                                                                              
      CALL REW(NGUS, 0)                                                         
        DO  J = 1,NKLA                                                       
      PK(J) = -1.0                                                             
	ENDDO 
        DO 50  I = 1,ICARD                                                      
      READ (NGUS)  (CI(K), K=1,KFAC)                                            
      J     = KLAS(I)                                                           
        IF (PK(J) .GT. -0.99)          GO TO  30                                
      PK(J) = 1.0                                                               
        DO   K = 1,KFAC                                                       
      T(J,K)= CI(K)
	ENDDO                                                             
                                       GO TO  50                                
   30 PK(J) = PK(J) + 1.0                                                       
      PKJ   = 1.0 / PK(J)                                                       
        DO   K = 1,KFAC                                                       
      CI(K) = CI(K) - T(J,K)                                                    
      T(J,K)= T(J,K) + PKJ*CI(K)                                                
	ENDDO
   50   CONTINUE                                                                
C                                                                               
C......... CALCULATION OF THE MATRIX OF DISTANCES BETWEEN OBJECTS               
C                                                                               
        DO   K1 = 2,NKLA                                                      
      K3    = K1 - 1                                                            
        DO   K2 = 1,K3                                                        
      K1K2  = MONO (K1, K2)                                                     
      D(K1K2)= 0.0                                                              
        DO   K = 1,KFAC                                                       
      AJ    = T(K1,K) - T(K2,K)                                                 
      D(K1K2)= D(K1K2) + AJ*AJ                                                  
	ENDDO

   	ENDDO
	ENDDO
        DO   K1 = 2,NKLA                                                      
      K3    = K1 - 1                                                            
        DO   K2 = 1,K3                                                        
      K1K2  = MONO (K1, K2)                                                     
      D(K1K2)= PK(K1)*PK(K2)*D(K1K2) / (PK(K1)+PK(K2))                          
   	ENDDO
	ENDDO
         END                                                                    
