
C++*************************************************************************
C
C DOCINFO.F                                       NEW SEPT 96 
C                                                 AUTHOR: ARDEAN LEITH
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.    AUTHOR: J.FRANK      *
C *  COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, ALBANY, NY. *
C * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH            *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED        *
C * FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF       *
C * HEALTH RESEARCH INC.                                               *
C **********************************************************************
C
C  DOCINFO(DOCNAM,NDOC,MAXKEYT,MAXREGT,SAYIT,IRTFLG)
C
C  PURPOSE:  DETERMINES MAXKEY AND MAXREG INSIDE A DOCUMENT FILE. 
C
C  PARAMETERS:
C         DOCNAM        DOC. FILE NAME INCLUDING EXTENSION        (SENT)
C         NDOC          LOGICAL UNIT FOR DOC FILE                 (SENT)
C         MAXKEYT       NUMBER OF HIGHEST KEY                 (RETURNED)
C         MAXREGT       MAX. NUMBER OF REGISTERS PER LINE        (RETURNED)
C                       (THIS IS 1 LESS THAN COLS. NEEDED FOR
C                       RECOVERY WITH UNSDAL, ETC.)
C         IRTFLG        ERROR FLAG (O IS NORMAL RETURN)       (RETURNED)
C     
C  TYPICAL DOC FILE LINES:
C        KEY #REGS/LINE    VALUES ........
C  COL: 123456789 123456789 123456789 123456789 123456789 1234565789
C          1 4   20.070000   17.379999   17.379999   17.379999
C        99994   20.070000   17.379999   17.379999   17.379999
C         -1 5   21.070000   12.379999   12.379999   16.379999
C        ; COMMENT LINE (PREVIOUS LINE IS A CONTINUATION FOR KEY 9999)
C
C--*********************************************************************

        SUBROUTINE DOCINFO(DOCNAM,NDOC,MAXKEYT,MAXREGT,SAYIT,IRTFLG)

	INCLUDE 'CMBLOCK.INC' 
CNO_SAVE

        CHARACTER *80   RECLIN
	CHARACTER *(*)  DOCNAM
	LOGICAL         NEWFILE,SAYIT

C       OPEN THE DOC FILE USING EXTENSION FROM DOCNAM
        CALL OPENDOC(DOCNAM,.FALSE.,NLET,NDOC,.FALSE.,
     &               .TRUE.,.TRUE.,.TRUE.,NEWFILE,IRTFLG)
        IF (IRTFLG .NE. 0) RETURN
 
C       WANT TO FIND MAXIMUM KEY & REGISTER NUMBER 
        MAXKEYT = 0
        MAXREGT = 0

C       READING LOOP
710        READ (NDOC,518,END=799) RECLIN
518        FORMAT(A80)
C          SKIP PLAIN COMMENTS

           IF (RECLIN(3:3) .EQ. '-') THEN
C             CONTINUATION LINE FOR PREVIOUS KEY
              READ(RECLIN,620,ERR=999)NKEY,NREGPLINE2
620           FORMAT(I5,I2)
              NREGPLINE = NREGPLINE + NREGPLINE2 
              IF (NREGPLINE .GT. MAXREGT) MAXREGT = NREGPLINE

           ELSEIF (RECLIN(2:2) .NE. ';' .AND. RECLIN(3:3) .NE. ';')THEN
C             SKIP COMMENTS
              READ(RECLIN,620,ERR=999) NKEY,NREGPLINE
              IF (NKEY      .GT. MAXKEYT) MAXKEYT = NKEY
              IF (NREGPLINE .GT. MAXREGT) MAXREGT = NREGPLINE
           ENDIF
           GOTO 710
C       LOOP UNTIL END

799	CLOSE(NDOC)
        IF (SAYIT) THEN
           WRITE(NOUT,97) MAXREGT, MAXKEYT
97         FORMAT(' DOC FILE CONTAINS: ',I2,'  REGISTERS AND: ',I5,
     &            ' KEYS',/)
        ENDIF

        IRTFLG = 0
	RETURN

999     WRITE(NOUT,*) '*** ERROR READING DOC FILE:'
        WRITE(NOUT,*) RECLIN
        CALL ERRT(100,'DOCINFO',NE)
        IRTFLG = 1
        CLOSE(NDOC)

        RETURN            
	END


