
C++*************************************************************************
C
C DPROFL.FOR -- CREATED JAN 87
C               EXTENSIVELY REWRITTEN JULY 87
C **************************************************************************
C *  AUTHOR:  ArDean Leith                                                 *
C *  PART OF SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK    *
C *  COPYRIGHT (C)1987 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C    DPROFL(IMFILE,NLETI,CSFILE,NLETC,LUNI,LUNCS,NSAM,NROW)
C
C    PARAMETERS:
C         IMFILE    CHAR. VARIABLE CONTAINING IMAGE FILE NAME
C         NLETI     LENGTH OF IMAGE FILE NAME
C         CSFILE    CHAR. VARIABLE CONTAINING CONTOUR FILE NAME (NEW)
C         NLETC     LENGTH OF CONTOUR FILE NAME
C         LUNI      LOGICAL UNIT NUMBER OF IMAGE FILE
C         NSAM      NUMBER OF SAMPLES ON AN IMAGE LINE
C         NROW      NUMBER OF ROWS IN IMAGE
C
C    CALLS:         RDPRMC   RDPRAI     REDLIN    CSAXIS    CSPUSH
C                   CSPSTX   CSTRAP     CSSCALT
C
C    CALLED BY:     PLOT1
C
C--**************************************************************************

      SUBROUTINE DPROFL(IMFILE,NLETI,CSFILE,NLETC,LUNI,LUNCS,NSAM,NROW)

      INCLUDE 'CONSTG.INC'
      INCLUDE 'CMBLOCK.INC'

C     ACTUAL BUFF LENGTH IS SET IN PLOT1
      PARAMETER      (NSIZE = 2000)
      COMMON         ADUM(80),BUFF(2048),FDATA(3,NSIZE),ILIST
      COMMON /POLY/  HOLES,REVERS
       
      CHARACTER *(*) IMFILE,CSFILE
      CHARACTER * 80 LINE
      CHARACTER * 8  FMT1
      CHARACTER * 1  NULL
      PARAMETER      (NTOTL = 100)
      DIMENSION      ILIST(NTOTL)
      LOGICAL        SUMALL,GLOBAL
       
C                12345678
      DATA FMT1/'(I     )'/ 

      DATA MININT/-32768/,MAXINT/32767/,FLTMIN/-10E30/,FLTZER/10E-30/,
     &     FLTMAX/10E30/

        NULL   = CHAR(0)

        NTOTLN = NTOTL

        IF (NSAM .LE. 0 .OR. NROW .LE. 0) THEN
C         NO. COL OR ROW NOT SPECIFIED SO IT IS A BLANK IMAGE
          WRITE(NOUT,*) ' *** ERROR: UNSPECIFIED NSAM, NROW'
          RETURN
        ENDIF

 1      CALL RDPRMC(LINE,NC,.TRUE.,
     &    'INDIVIDUAL,  GLOBAL,  OR SUM  SCALE PLOT? (I/G/S)',
     &    NULL,IRTFLG)
        IF (IRTFLG .EQ. -1) THEN
           CLOSE(LUNCS)
           CLOSE(LUNI)
           RETURN
        ENDIF

        GLOBAL = .FALSE.
        SUMALL = .FALSE.
        IF (LINE(1:1) .EQ. 'G')THEN
C         SCALE PLOT BASED ON MAX AND MIN Y VALUES IN WHOLE FILE (FMIN, FMAX)
          GLOBAL = .TRUE.
          SYMIN = FMIN
          SYMAX = FMAX

        ELSE IF (LINE(1:1) .EQ. 'S') THEN
C         SUM ALL LINES AND SCALE PLOT BASED ON MAX AND MIN Y OF THE SUM
          SUMALL = .TRUE.
          SYMIN = FLTMAX
          SYMAX = FLTMIN
          DO  I=1,NSAM
             BUFF(NSAM+I) = 0.0
          ENDDO

        ELSE
C         SCALE THE PLOT BASED ON MAX AND MIN Y VALUES OF PLOTTED LINES ONLY
          SYMIN = FLTMAX
          SYMAX = FLTMIN
        ENDIF

	WRITE(NOUT,92) NROW
   92   FORMAT('  FILE HAS:',I4,' ROWS')

   18   NLIST = NTOTL
        CALL RDPRAI(ILIST,NTOTL,NLIST,1,NROW,'ENTER ROW NUMBER(S)',
     &              NULL,IRTFLG)
        IF (IRTFLG .EQ. -1) GOTO 1

        IF (.NOT. GLOBAL) THEN
C          GO THRU ALL REQUESTED LINES TO FIND MIN AND MAX
           DO  IT = 1,NLIST
              I = ILIST(IT)
              CALL REDLIN(LUNI,BUFF,NSAM,I)
              DO  J = 1,NSAM
                 VAL = BUFF(J)
                 IF (SUMALL) THEN
                    BUFF(NSAM+J) = BUFF(NSAM + J) + VAL
                 ELSE
                    IF (VAL .LT. SYMIN) SYMIN = VAL
                    IF (VAL .GT. SYMAX) SYMAX = VAL
                 ENDIF
              ENDDO
	   ENDDO
        ENDIF

        IF (SUMALL) THEN
C          FIND MAX AND MIN Y VALUES OF THE SUMS
           DO  J = 1,NSAM
              VAL = BUFF(J + NSAM)
              IF (VAL .LT. SYMIN) SYMIN = VAL
              IF (VAL .GT. SYMAX) SYMAX = VAL
          ENDDO
        ENDIF


        INTEN  = 9
        MARK   = 0
        ITANGL = 0
        ITSIZE = 32
        IFON   = 0
        IPEN   = 0
        FNSAM  = NSAM
        SXMIN  = 0.0
 
        CALL CSAXIS('X',SXMIN,FNSAM,0.0,0.0,120.0,100.0,XFACTR,LUNCS,
     &  IRTFLG)

        CALL CSAXIS('Y',SYMIN,SYMAX,0.0,0.0,120.0,100.0,YFACTR,LUNCS,
     &  IRTFLG)

        IF (.NOT. SUMALL) THEN
C          GET THE VALUES AGAIN
           DO  L = 1,NLIST
              I = ILIST(L)
              CALL REDLIN(LUNI,BUFF,NSAM,I)
              LINTP = MOD(L,10)

C****************
C**              IPEN  = MOD(L,10)  DISABLED FOR WORKSTATION
C*****************

              NDATA = 0
              DO J = 1,NSAM
                 NDATA = NDATA + 1
                 FDATA(1,NDATA) = J * XFACTR
                 FDATA(2,NDATA) = (BUFF(J) - SYMIN) * YFACTR
                 IF (NDATA .GE. NSIZE) THEN
C                   ARRAY FULL, PUSH INTO FILE            
                    CALL CSPUSH(LUNCS,DATA,NDATA,NF,1,ICNTR)

                    FDATA(1,1) = FDATA(1,NDATA)
                    FDATA(2,1) = FDATA(2,NDATA)
                    NDATA = 1
                 ENDIF
              ENDDO
              IF (NDATA .GT. 0) 
     &           CALL CSPUSH(LUNCS,FDATA,NDATA,NF,1,ICNTR)
	  ENDDO

        ELSE IF (SUMALL) THEN
           LINTP = 0
           IPEN  = 0
           NDATA = 0
           DO  J = 1,NSAM
              NDATA = NDATA + 1
              FDATA(1,NDATA) = J * XFACTR
              FDATA(2,NDATA) = (BUFF(J+NSAM) - SYMIN) * YFACTR
              IF (NDATA .GE. NSIZE) THEN
C                ARRAY FULL, PUSH INTO FILE            
                 CALL CSPUSH(LUNCS,FDATA,NDATA,NF,1,ICNTR)

                 FDATA(1,1) = FDATA(1,NDATA)
                 FDATA(2,1) = FDATA(2,NDATA)
                 NDATA = 1
              ENDIF
	   ENDDO
           IF (NDATA .GT. 0) CALL CSPUSH(LUNCS,FDATA,NDATA,NF,1,ICNTR)
        ENDIF          

C     PUT CNT FILENAME AT TOP,  PUT IMAGE FILENAME AT TOP
      JUST = 0
      IPEN = 0
      LINE = CSFILE(1:NLETC-1) // '      ' // IMFILE(1:NLETI) // 
     &                            '      '

      NC = NLETC + NLETI + 12

      NNUM = 4
C     PUT UP TO NNUM LINE NUMBERS ON TOP OF PLOT
      XPOS = 66.0

      IF (SUMALL) THEN
         LINE(NC+1:NC+7) = 'SUM OF '
         NC = NC + 7
      ENDIF
      IF (NLIST .GT. 1) THEN
         LINE(NC+1:NC+6 ) = 'ROWS: '
         NC = NC + 6
      ELSE
         LINE(NC+1:NC+5)  = 'ROW: '
         NC = NC + 5
      ENDIF

      DO 60 I = 1,NLIST
         IF (I .GT. NNUM) THEN
          WRITE(NOUT,89)NNUM
   89     FORMAT(/,' SORRY: LABEL LIMITED TO ',I3,' LINES')
          GOTO 61
         ENDIF
         L = ILIST(I)
         IF (L .GE. 1000) THEN
            LENL = 4
         ELSE IF (L .GE. 100) THEN
            LENL = 3
         ELSE IF (L .GE. 10) THEN
            LENL = 2
         ELSE 
            LENL = 1
         ENDIF

C        ADD ROW NUMBER
         NC = NC + 1
         IF ((NC + LENL) .GT. 50) GOTO 61

         NLEN = NUMDIG(L,1)
         NLEN = MAX(NLEN,5)
         WRITE(FMT1(3:3+NLEN-1),8009) L
8009     FORMAT(I5)
         WRITE(LINE(NC+1:NC+LENL-1),FMT1,ERR=60) L

C**      WRITE(LINE(NC:NC+LENL-1),8003,ERR=9999)L
C** 8003 FORMAT(I<LENL>)

         NC = NC + LENL -1
         IF (I .EQ. NLIST .OR. I .EQ. 10) GOTO 61
         LINE(NC+1:NC+2) = ', '
         NC = NC + 2
   60 CONTINUE

   61 YPOS = 114.0
      XPOS = 0.0
      NC   = NC - 2
      CALL CSPSTX(LINE,NC,LUNCS,NF,1,ITEXT)


 9999 WRITE(NOUT,9045)CSFILE(1:NLETC)
9045  FORMAT(/,'  GRAPH PLACED IN: ',A)

C..   SCALE THE META-FILE, IF ISERCH = 0 THEN SCALING IS DONE BY ONE PASS
      ISERCH = 0
      CALL CSSCALT(LUNCS,ISERCH)

C..   CLOSE THE META-FILE 
      CALL CSTRAP(LUNCS)
      CLOSE(LUNCS)

      RETURN
      END

