
C++************************************************************************
C
C    ENHANC.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C    ENHANC(FILNAM,LUNI,LUNO,NSAM,NROW)
C
C    PURPOSE:  MAIN IMAGE ENHANCEMENT ROUTINE.  CALLS OTHER ROUTINES FOR
C              HISTOGRAM BASED ENHANCEMENTS AND DOES THRESHOLDING.
C
C    PARAMETERS:
C         FILNAM     NAME OF FILE
C         LUNI       LOGICAL UNIT NUMBER OF INPUT FILE
C         LUNO       LOGICAL UNIT NUMBER OF OUTPUT FILE
C         NSAM,NROW  DIMENSIONS OF FILE
C
C--*******************************************************************

      SUBROUTINE ENHANC(FILNAM,LUNI,LUNO,NSAM,NROW,NSLICE)

C     BE CAREFUL ALTERING ULABELED COMMON, IT IS USED TO COMMUNICATE HISTOGRAM
      COMMON IPLOT(128),BUF(1)

      INCLUDE 'CMBLOCK.INC'

      DIMENSION      VAL(4)
      CHARACTER *(*) FILNAM
      CHARACTER *3   ANS
      CHARACTER      NULL,MODE,IRESH

      EQUIVALENCE(B1,VAL(1)),(T1,VAL(2)),(B2,VAL(3)),(T2,VAL(4))
CNO_SAVE
      NULL = CHAR(0)

      MAPA = NSAM+128
 4    CALL RDPRMC(MODE,NC,.TRUE.,
     &  '(S)TRETCH, (H)ISTOGRAM EQUALIZE OR (L)OCAL? (S/H/L)',
     &     NULL,IRT)
      IF (IRT .NE. 0) RETURN

      IF (MODE .EQ. 'H') THEN
C       HISTOGRAM EQUALIZATION
        CALL EHIST(FILNAM,LUNI,LUNO,NSAM,NROW,NSLICE)
        RETURN

      ELSEIF (MODE .EQ. 'L') THEN
C       ADAPTIVE HISTOGRAMING
        CALL LOCAL(LUNI,LUNO,NSAM,NROW,NSLICE)
        RETURN

      ELSEIF (MODE .NE. 'S') THEN
        CALL ERRT(15,'ENHANC ',NE)
        RETURN
      ENDIF

      CALL RDPRMC(ANS,NC,.TRUE.,
     & '(S)INGLE, (A)UTOMATIC OR (D)OUBLE MAPPING? (S/A/D)',NULL,IRT)
      IF (IRT .EQ. -1) GOTO 4

      IF (ANS .EQ. 'A') THEN
         CALL RDPRM(PERC,NLOOP,ILOOP,'INTEGRAL THRESHOLD PERCENT')
         CALL RDPRMC(IRESH,NC,.TRUE.,'PLOT RESULT HISTOGRAM? (Y/N)',
     &               NULL,IRT)
      ENDIF

      IF (ANS .EQ. 'S' .OR. ANS .EQ. 'A') THEN
         CALL RDPRM(BOTTOM,NLOOP,ILOOP,'BOTTOM DENSITY VALUE$')
         CALL RDPRM(TOP,NLOOP,ILOOP,'TOP DENSITY VALUE$')
         IF (BOTTOM .GE. TOP) GOTO 650
      ENDIF

C     DETERMINE HISTOGRAM FROM IMAGE AND STORE IN HISTOGRAM RECORDS OF FILE
      CALL HIST(LUNI,NSAM,NROW,NSLICE,0,0,.FALSE.)

94    HINCO = BUF(NSAM+130) / 127.
      HMIN  = BUF(NSAM+129)
      WRITE(NOUT,1111) HMIN,BUF(NSAM+130)
1111  FORMAT(' $$$ HISTOGRAM MINIMUM =',F10.5,
     &       '  HISTOGRAM RANGE=',F10.5)

      IF (ANS .EQ. 'D') THEN
C        NON-UNIQUE MAPPING WITH DOUBLE LINEAR MAPPING FUNCTION
100      CALL RDPRM(B1,NLOOP,ILOOP,'BOTTOM1')
         IF (B1 .LT. FMIN) B1 = FMIN
         CALL RDPRM(T1,NLOOP,ILOOP,'TOP1')
         CALL RDPRM(B2,NLOOP,ILOOP,'BOTTOM2')
         CALL RDPRM(T2,NLOOP,ILOOP,'TOP2')
         IF (T2 .GT. FMAX) T2 = FMAX
C        CHECK IF B1,T1,B2,T2 ARE ORDERED ACCORDING TO INCREASING VALUE
         A = VAL(1)
         DO I = 1,4
           A = AMAX1(A,VAL(I))
           IF (A .NE. VAL(I)) GOTO 650
	 ENDDO

         IF (B1 .NE. T1) THEN
            IF (B2 .NE. T2) GOTO 130
            BOTTOM = B1
            TOP = T1
         ELSE
            BOTTOM = B2
            TOP = T2
         ENDIF
         GOTO 30


130      NB1 = (B1-HMIN) / HINCO + 1.5
         NT1 = (T1-HMIN) / HINCO + 1.5

C        NEW DENSITY INCREMENT ON LEFT HAND SIDE
         HINCN1 = 2.0 / FLOAT(NT1-NB1)
         NB2    = (B2-FMIN) / HINCO+1.5
         NT2    = (T2-FMIN) / HINCO+1.5
C        NEW DENSITY INCREMENT ON RIGHT HAND SIDE
         HINCN2 = 2.0 / FLOAT(NT2-NB2)
         IF (NB1 .GE. 2) THEN
           DO  K = 1,NB1-1
           BUF(MAPA+K) = 0.
	   ENDDO
         ENDIF
         DO  K = NB1,NT1
          BUF(MAPA+K) = FLOAT(K-NB1)* HINCN1
	 ENDDO
         IF (NT1 .NE. NB2) THEN
           DO  K = NT1+1,NB2-1
            BUF(MAPA+K) = 2.
	   ENDDO
         ENDIF
         DO  K = NB2,NT2
          BUF(MAPA+K) = FLOAT(K-NB2)*HINCN2      
	 ENDDO
         IF (NT2 .NE. 128) THEN
           DO K = NT2+1,128
            BUF(MAPA+K) = 2.
	   ENDDO
         ENDIF
         GOTO 500

      ELSEIF (ANS .EQ. 'A') THEN
        P = PERC*FLOAT(NSAM)*FLOAT(NROW)/100.
        ADD   = 0.0
        DO  I = 1,128
          IM  = I
          ADD = ADD + BUF(NSAM+I)
          WRITE(NDAT,2222)I,P,ADD,BUF(NSAM+I)
2222      FORMAT(1X,I3,3F10.5)
          IF (ADD .GT. P) GOTO 97
	ENDDO

97      BOTTOM = FLOAT(IM)*HINCO+HMIN
        ADD = 0.
        DO  I = 128,IM,-1
          IC = I
          ADD = ADD+BUF(NSAM+I)
          IF (ADD .GT. P) GOTO 99
	ENDDO

99      TOP = FLOAT(IC)*HINCO+HMIN
      ENDIF



30    IF (BOTTOM .LT. FMIN) BOTTOM = FMIN
      IF (TOP    .GT. FMAX) TOP = FMAX
      IF (BOTTOM .EQ. FMIN .AND. TOP .EQ. FMAX) RETURN

      NB = (BOTTOM-HMIN)/HINCO+1.5
      NT = (TOP-HMIN)/HINCO+1.5
      WRITE(NOUT,31)BOTTOM,TOP
31    FORMAT(' BOTTOM DENSITY ',G10.2,' , TOP DENSITY ',G10.2)
      HINCN = 2./FLOAT(NT-NB)
      IF (NB .GE. 2) THEN
        DO  K = 1,NB-1
         BUF(MAPA+K) = 0.
	ENDDO
      ENDIF
70    DO  K = NB,NT
       BUF(MAPA+K) = FLOAT(K-NB)*HINCN
      ENDDO
      IF (NT .NE. 128) THEN
        DO  K = NT+1,128
         BUF(MAPA+K) = 2.
	ENDDO
      ENDIF

C     NOW APPLY MAPPING FUNCTION TO DATA.
C     RESULT IS NORMALIZED BETWEEN 0. AND 2.

500   CALL GRAPHS(BUF(MAPA+1),128,1)
      AV = 0.
      DO  I = 1,NROW
         CALL REDLIN(LUNI,BUF,NSAM,I)
         DO  K = 1,NSAM
            MAP = (BUF(K)-HMIN)/HINCO+1.5
            T   = BUF(MAPA+MAP)
            AV  = AV +T
         BUF(K) = T
	 ENDDO
      CALL WRTLIN(LUNO,BUF,NSAM,I)
      ENDDO

      AV    = AV/(FLOAT(NSAM)*FLOAT(NROW))
      SIG   = -1.
      IHIST = 0
      FMAX  = 2.
      FMIN  = 0.
      CALL SETPRM(LUNO,NSAM,NROW,FMAX,FMIN,AV,'U')
      IF (IRESH .EQ. 'Y') CALL HIST(LUNO,NSAM,NROW,NSLICE,0,0,.FALSE.)
      RETURN

650   CALL ERRT(14,'ENHANC',NE)
      RETURN

      END
