
C++*************************************************************************
C
C    FILERD.F     ADAPTED OCT 88 FROM FILRD AND SPFILE al
C                 TOTALLY REWRITTEN JUNE 97 FOR NEW SUBSTITUTION al
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    FILERD(FILN,NLET,EXTEN,PROMPT,IRTFLG)
C
C    PARAMETERS:  FILN    CHARACTER ARRAY FOR FILE NAME       (RETURNED)
C                 NLET    NO. OF CHARACTERS IN FILE NAME      (RETURNED)
C                 EXTEN   OPTIONAL FILE NAME EXTENSION   (SENT/RETURNED)
C                 PROMPT  FILE NAME SOLICITATION PROMPT           (SENT)
C                 IRTFLG  ERROR FLAG, ZERO IS NORMAL          (RETURNED)
C
C    PURPOSE:     READS IN CHARACTER FILE NAME  AND TITLE.  CAN ALSO 
C                 SUBSTITUTE REGISTERS AND DO LOOP INDICES INTO
C                 CERTAIN POSITIONS OF FILENAME
C
C    DESCRIPTION: PRINTS OUT <PROMPT> MESSAGE:,
C                 READS IN FILE NAME FROM INPUT DEVICE. 
C                 SUBSTITUTES FOR REGISTER AND LOOP INDEX. 
C                 EXAMPLES IF (X0 = 1, X11=11, X21=1111, INDEX:A=44)
C                     avgx0       ---> avg001
C                     avg1x0      ---> avg001 (no error reported)
c                     avgx11      ---> avg011
c                     avgx21      ---> av1111 (no error reported)
c                     avg*x11     ---> avg0001
c                     avg***x11   ---> avg000001
c                     avg1x10     ---> avg1001
C
C                     avg0a       ---> avg044
C                     avg00a      ---> avg044
C                     avg**00a    ---> avg00044
C
C                     avg{***x0}  ---> avg001
C                     avg{****x0} ---> avg001
C                     avg{****}   ---> avg***
c                     avg{***x21} ---> ERROR DAMAGES INVARIANT PART OF 
C                                      FILE NAME
C
C
C                 IF THE FIRST CHARACTER IS A '$', FILNAM FROM PREVIOUS 
C                 CALL REMAINS UNCHANGED.  
C                 AFTER THE FILE NAME, A TITLE UP TO 160 
C                 CHARACTERS CAN BE ENTERED WHICH IS ATTACHED TO THE 
C                 IMAGE FILE IN THE HEADER WHEN OPEN IS 
C                 SUBSEQUENTLY CALLED.  NEW TITLE INFORMATION 
C                 SUPERSEDES THE TITLE ALREADY PRESENT IN THE HEADER.  
C                 TO ENTER A TITLE TYPE '|' AFTER THE FILE NAME.
C                 EXAMPLE: ABC001|THIS IS A TITLE
C
C  NOTE:          IF IRTFLG = 9 FILERD ACCEPTS AND RETURNS AN EXTENSION
C                 FROM THE INPUT
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*******************************************************************

	SUBROUTINE FILERD(FILN,NLET,EXTEN,PROMPT,IRTFLG)

        INCLUDE 'CMBLOCK.INC'
CNO_SAVE

        CHARACTER *(*)  FILN,PROMPT,EXTEN
        CHARACTER  *80  FILNAM
        CHARACTER  *100 FMT
	CHARACTER   *1  NULL,JCHAR,JM1CHAR,JM2CHAR
        CHARACTER  * 6  FMT1
        CHARACTER  * 9  DIGITS
        CHARACTER * 26  ALFABT
        LOGICAL         NOX,EXTENOK,GOTEX,SAYIT,KEEPGO,ISDIGI,ISCHAR
        LOGICAL         OLD1,OLD2,OLD3


        DATA DIGITS/'123456789'/
        DATA ALFABT/'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
        DATA FMT1/'(I . )'/
        DATA OLD1,OLD2,OLD3/.TRUE.,.TRUE.,.TRUE./

        NULL   = CHAR(0)

        EXTENOK = (IRTFLG .EQ. 9) 

C       READ THE FILE NAME FROM THE INPUT, DO NOT UPPERCASE IT!
        LENF  = MAX(1,lnblnk(PROMPT))
        FMT   = PROMPT(1:LENF) // ' FILE' // NULL
        SAYIT = .NOT. SILENT
	CALL RDPR(FMT(1:LENF+5),NLET,FILNAM,.FALSE.,SAYIT,IRTFLG)

        IF (NLET .LE. 0) THEN
C          NO FILE NAME IN INPUT
           IRTFLG = -1
           RETURN
        ENDIF
        IRTFLG = 1

C       write(nout,*)' '
C       write(nout,*)'input:      ',filnam(1:nlet)

C       REMOVE ANY INITIAL BLANKS
        I = 0
        DO WHILE (FILNAM(I+1:I+1) .EQ. ' ' .OR. 
     &           FILNAM(I+1:I+1) .EQ. CHAR(11))
C          INITIAL BLANK/TAB IN FILENAME
           I = I + 1
        ENDDO

        IF (I .GT. 1) THEN
C          REMOVE INITIAL BLANKS
           NLET2           = NLET - I 
           FILNAM(1:NLET2) = FILNAM(I+1:NLET)
           NLET            = NLET2
        ENDIF

	IF (FILNAM(1:1) .EQ. '$') THEN
C          DO NOT ALTER FILNAM IF INPUT IS "$"
           IRTFLG = 0
           GOTO 999 
   
        ELSEIF (FILNAM(1:1) .EQ. '^' .OR. FILNAM(1:1) .EQ. '*') THEN
C          RETURN IF  INPUT IS "^" OR "*"
           FILN(1:) = FILNAM(1:1)
           IRTFLG   = -1
           GOTO 999
        ENDIF
 
C       REMOVE ANY TRAILING COMMENT FROM THE INPUT LINE
        ISEMI = INDEX(FILNAM(1:NLET),';')
        IF (ISEMI .GT. 1) NLET = ISEMI - 1

C       REMOVE ANY EXTENSION FROM THE FILENAME (UNLESS IT IS WANTED)
        KEEPGO = .TRUE.
        J      = NLET
        GOTEX  = .FALSE.
        DO WHILE (J .GE. 1 .AND. KEEPGO)
           JCHAR = FILNAM(J:J)

           IF (JCHAR .EQ. ' ') THEN
C             DISCARD THIS TRAILING BLANK
              NLET = J - 1

           ELSEIF (JCHAR .EQ. ']' .OR. 
     &            JCHAR .EQ. ':' .OR. JCHAR .EQ. '/' ) THEN
C             BEGINNING OF DEVICE OR DIRECTORY, HALT EXTENSION SEARCH
              KEEPGO = .FALSE.

           ELSEIF (JCHAR .EQ. '.') THEN
              IF (EXTENOK) THEN
C                WANT TO USE FILE EXTENSION
                 EXTEN = FILNAM(J+1:NLET) // NULL
                 GOTEX = .TRUE.
              ENDIF
              NLET  = J - 1
           ENDIF
           J = J - 1
        ENDDO

        IF (NLET .LE. 0) THEN 
           WRITE(NOUT,*) '*** ABNORMAL FILENAME'
           CALL ERRT(100,'FILERD',NE)
           RETURN
        ENDIF

C       LOCATE OLD-STYLE SUBSTITUTION SPECIAL CHARACTERS
        IFILEND = NLET 

        JCHAR   = FILNAM(NLET:NLET)
        JM1CHAR = CHAR(0)
        IF (NLET .GT. 1) JM1CHAR = FILNAM(NLET-1:NLET-1)
        JM2CHAR = CHAR(0)
        IF (NLET .GT. 2) JM2CHAR = FILNAM(NLET-2:NLET-2)

        IF (ISCHAR(JCHAR) .AND. ISDIGI(JM1CHAR)) THEN
C          FOUND OLD-STYLE INDEX SUBSTITUTION AT END OF FILE NAME *0I
           IFILEND = NLET - 3

C          GOT >= ONE ZERO BEFORE INDEX SO SUBSTITUTE LOOP COUNTER
           CALL SSUPCAS(JCHAR)
           IDX  = INDEX(ALFABT,JCHAR)
           IVAL = IALPHA(IDX)
cc           IF (OLD1) THEN
ccC             COMPLAIN ABOUT OLD-STYLE SUBSTITUTION
cc              WRITE(NOUT,90) 
cc 90           FORMAT(/,' --- PLEASE USE MODERN FILENAME SUBSTITUTION ',
cc     &               'SYNTAX  (E.G. aaaa{***i})',/,
cc     &               ' --- THIS MESSAGE ONLY OCCURS ONCE.',/)
cc              OLD1 = .FALSE.
cc           ENDIF

        ELSEIF (ISDIGI(JCHAR)) THEN
C          POSSIBLE OLD-STYLE REGISTER (X) SUBSTITUTION
           IF (JM1CHAR .EQ. 'X' .OR. JM1CHAR .EQ. 'x') THEN
C             FOUND "X0" TYPE  OLD-STYLE REGISTER (X) SUBSTITUTION
              IFILEND = NLET - 3
              IREG    = INDEX(DIGITS,JCHAR) 
C             SPECIAL CODE FOR PICX0
              IF (NLET .EQ. 5) THEN
C                SPECIAL CODE FOR PICX0
                 IFILEND = NLET - 2
                 NLET    = 6
              ENDIF
cc              IF (OLD2) THEN
ccC                COMPLAIN ABOUT OLD-STYLE SUBSTITUTION
cc                 WRITE(NOUT,91) 
cc 91              FORMAT(/,' --- PLEASE USE MODERN FILENAME ',
cc     &                  'SUBSTITUTION  SYNTAX  (E.G. aaaa{***X0})',/,
cc     &               ' ---THIS MESSAGE WILL NOT OCCUR AGAIN.',/)
cc                 OLD2 = .FALSE.
cc              ENDIF

           ELSEIF (ISDIGI(JM1CHAR) .AND.
     &             JM2CHAR .EQ. 'X' .OR. JM2CHAR .EQ. 'x') THEN
C             FOUND "X00" TYPE  OLD-STYLE REGISTER (X) SUBSTITUTION
              IFILEND = NLET - 3
              IREG    = INDEX(DIGITS,JM1CHAR) * 10 + INDEX(DIGITS,JCHAR) 

cc              IF (OLD3) THEN
ccC                COMPLAIN ABOUT OLD-STYLE SUBSTITUTION
cc                 WRITE(NOUT,92) 
cc 92              FORMAT(/,' --- PLEASE USE MODERN FILENAME ',
cc     &                  'SUBSTITUTION  SYNTAX  (E.G. aaaa{***X11})',/,
cc     &               ' ---THIS MESSAGE WILL NOT OCCUR AGAIN.',/)
cc                 OLD3 = .FALSE.
cc              ENDIF
           ENDIF

           IF (IFILEND .LT. NLET) THEN
C             REGISTER SUBSTITUTION NEEDED
              IF (IREG .LT. 0 .OR. IREG .GT. 100)  THEN
                 WRITE(NOUT,*) 
     &            '*** BAD REGISTER: ',IREG,' IN FILENAME SUBSTITUTION'
                 CALL ERRT(100,'FILERD',NE)
                 RETURN
              ENDIF

C             FIND REGISTER CONTENTS
              IVAL = PARAM(IREG+1) + 0.5
              IF (IVAL .LT. 0)  THEN
                 WRITE(NOUT,*) 
     &              '*** NEGATIVE REGISTER CONTENT: ',IVAL,
     &              ' NOT ALLOWED IN FILENAME SUBSTITUTION'
                 CALL ERRT(100,'FILERD',NE)
                 RETURN
              ENDIF
           ENDIF
        ENDIF


        IF (IFILEND .LT. NLET) THEN
C          SUBSTITUTE FOR A PORTION OF THE FILE NAME
C          FIND NUMBER OF DIGITS TO BE SUBSTITUTED (>=3) 

           NDIGITS = NUMDIG(IVAL,3)

C          FIND NUMBER OF IMMEDIATLY PRECEEDING ASTERICKS (IF ANY)
           DO WHILE (IFILEND .GE. 1 .AND. 
     &              (FILNAM(IFILEND:IFILEND) .EQ. '*'))
C             INCREMENT ASTERICKS NEEDED
              IF ((NLET - IFILEND) .GE. NDIGITS) NDIGITS = NDIGITS + 1
              IFILEND = IFILEND - 1
           ENDDO  

           IFILEND = NLET - NDIGITS 
           IF (IFILEND .LT. 0) THEN
              WRITE(NOUT,*) '*** FILENAME TOO SHORT FOR SUBSTITUTION'
              CALL ERRT(100,'FILERD',NE)
              GOTO 999
           ENDIF

C          PREPARE FORMAT FOR WRITING IVAL INTO FILENAME
           FMT1(3:3) = DIGITS(NDIGITS:NDIGITS)
           FMT1(5:5) = DIGITS(NDIGITS:NDIGITS)

C          WRITE REGISTER CONTENTS INTO THE FILE NAME
           WRITE(FILNAM(IFILEND+1:IFILEND+NDIGITS),FMT1) IVAL
        ENDIF
        
C       write(nout,*) 'evaluating: ',filnam(1:nlet)

C       SUBSTITUTE FOR STRINGS WITHIN {...} PORTION(S) OF THE FILE NAME
        CALL FILNAMSUB(FILNAM,NLET,.FALSE.,0,IRTFLG)
        IF (IRTFLG .NE. 0) RETURN

C       write(nout,*) 'final:      ',filnam(1:nlet),' : ',nlet

        FILN = NULL
        FILN(1:NLET+1) = FILNAM(1:NLET) // NULL

        IF (EXTEN(1:1) .NE. NULL .AND. EXTENOK) THEN
C          EXTENSION RECEIVED FROM CALLER OR ON INPUT. 
C          ADD IT TO FILE NAME
           LENE = INDEX(EXTEN,NULL)
           IF (LENE .EQ. 0) LENE = LEN(EXTEN)
           LENB = INDEX(EXTEN,' ')
           IF (LENB .GT. 0) LENE = MIN(LENE,LENB)
           IF (LENE + NLET .LE. LEN(FILN)) THEN
              FILN = FILN(1:NLET) // '.' // EXTEN(1:LENE) // NULL
              NLET = NLET + LENE
           ELSE
              WRITE(NOUT,*) '*** FILENAME TOO LONG'
               CALL ERRT(100,'FILERD',NE)
              IRTFLG = 1
              RETURN
           ENDIF
        ENDIF

C       SET NORMAL ERROR RETURN
        IRTFLG = 0

999     IF (.NOT. SILENT) WRITE(NECHO,911) FILN(1:NLET)
        IF (NPROC .NE. 0) WRITE(NPROC,911) FILN(1:NLET)
911     FORMAT(5X,A)


        RETURN
        END


