
C++*************************************************************************
C
C    FILNAMSUB.F  NEW JULY 97 al
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    FILNAMSUB(FILNAM,NLET,USEVAL,PARAM,IRTFLG)
C
C    PARAMETERS: 
C           FILNAM     CHARACTER STRING FOR SUBSTITUION  (SENT/RETURNED)
C           NLET       NO. OF CHARACTERS IN STRING       (SENT/RETURNED)
C           USEVAL     FLAG TO USE SENT REG. VALUES      (SENT)
C           CURPARAM   SET OF REG. VALUES TO BE USED     (SENT)
C           IRTFLG     ERROR FLAG, ZERO IS NORMAL        (RETURNED)
C
C    PURPOSE:    SUBSTITUTE REGISTERS AND DO LOOP INDICES INTO
C                CERTAIN POSITIONS OF FILENAM (NOT NECESSARILY A
C                FILENAME IT CAN BE ANY STRING)
C
C    DESCRIPTION: SUBSTITUTES FOR REGISTER AND LOOP INDEX. 
C                 EXAMPLES IF (X0 = 1, X11=11, X21=1111, INDEX:A=44)
C                     avgx0       ---> avg001
C                     avg1x0      ---> avg001 (no error reported)
c                     avgx11      ---> avg011
c                     avgx21      ---> av1111 (no error reported)
c                     avg*x11     ---> avg0001
c                     avg***x11   ---> avg000001
c                     avg1x10     ---> avg1001
C
C                     avg0a       ---> avg044
C                     avg00a      ---> avg044
C                     avg**00a    ---> avg00044
C
C                     avg{***x0}  ---> avg001
C                     avg{****x0} ---> avg001
C                     avg{****}   ---> avg***
c                     avg{***x21} ---> ERROR DAMAGES INVARIANT PART OF 
C                                      FILE NAME
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

      SUBROUTINE FILNAMSUB(FILNAM,NLET,USEVAL,CURPARAM,IRTFLG)

      INCLUDE 'CMBLOCK.INC'
CNO_SAVE

      CHARACTER *(*)   FILNAM
      DIMENSION        CURPARAM(103)

      CHARACTER *1     JCHAR,JP1CHAR,JP2CHAR
      CHARACTER  * 6   FMT1
      CHARACTER  * 9   DIGITS
      CHARACTER * 26   ALFABT
      LOGICAL          ISDIGI,ISCHAR,USEVAL

      DATA DIGITS/'123456789'/
      DATA ALFABT/'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      DATA FMT1/'(I . )'/

      IRTFLG = 1
C     USE IVALT IN CASE IVAL IS A CONSTANT 
      IVALT  = IVAL

10    CONTINUE

C     SEE IF "{...}" IS PRESENT      
      CALL CHARINSIDE(FILNAM(1:NLET),'{','}',.FALSE., IGO,IEND,NCHAR)

      IF (NCHAR .GT. 0) THEN
C        NEED TO SUBSTITUTE INTO FILNAM
         J    = IGO + 1
         NAST = 0
         DO WHILE (J .LT. IEND .AND. FILNAM(J:J) .EQ. '*')
            NAST = NAST + 1
            J    = J + 1
         ENDDO

         IF (NAST .LE. 0) THEN
C           NO ASTERICKS IS AN ERROR
            WRITE(NOUT,*) 
     &         '***NO ASTERICKS IN FILENAME SUBSTITUTION STRING'
            CALL ERRT(100,'FILNAMSUB',NE)
            RETURN
         ELSEIF ((NAST + 2) .EQ. NCHAR) THEN
C           SUBSTITUTION STRING HAS ONLY ASTERICKS, LEAVE IT ALONE
            IRTFLG = 0
            RETURN
         ENDIF

         JCHAR   = FILNAM(J:J)
         JP1CHAR = CHAR(0)
         IF ((J + 1) .LT. IEND) JP1CHAR = FILNAM(J+1:J+1)
         JP2CHAR = CHAR(0)
         IF ((J + 2) .LT. IEND) JP2CHAR = FILNAM(J+2:J+2)
         CALL SSUPCAS(JCHAR)

         IF (JCHAR .EQ. 'X' .AND. ISDIGI(JP1CHAR)) THEN
C           REGISTER SUBSTITUTION WANTED
            IREG = INDEX(DIGITS,JP1CHAR) 
            IF (ISDIGI(JP2CHAR)) IREG = IREG * 10 + 
     &          INDEX(DIGITS,JP2CHAR) 

            IF (IREG .LT. 0 .OR. IREG .GT. 100) THEN
               WRITE(NOUT,*) 
     &            '*** BAD REGISTER: ',IREG,' IN FILENAME SUBSTITUTION'
               CALL ERRT(100,'FILNAMSUB',NE)
               RETURN
            ENDIF

C           FIND REGISTER CONTENTS
            IF (USEVAL) THEN
               IVALT = CURPARAM(IREG+1) + 0.5
            ELSE
               IVALT = PARAM(IREG+1) + 0.5
            ENDIF

         ELSEIF (ISCHAR(JCHAR) .AND. .NOT. USEVAL) THEN
C           LOOP COUNTER SUBSTITUTION WANTED
            CALL SSUPCAS(JCHAR)
            IDX   = INDEX(ALFABT,JCHAR)
            IVALT = IALPHA(IDX)
         ENDIF

C        FIND DIGITS IN IVALT
         NDIGITS = NUMDIG(IVALT,0)
         IF (NDIGITS .GT. NAST) THEN
            
            WRITE(NOUT,90)IVALT,FILNAM(1:NLET)
90          FORMAT(' *** SUBSTITUTING: ',I6, ' INTO:',A,/,
     &             '     DAMAGES INVARIANT PART OF FILE NAME')
            CALL ERRT(100,'FILNAMSUB',NE)
            RETURN
         ENDIF

C        PREPARE FORMAT FOR WRITE
         FMT1(3:3) = DIGITS(NAST:NAST)
         FMT1(5:5) = DIGITS(NAST:NAST)

         NLEN    = LEN(FILNAM)
         NLET = IGO + NAST + (NLET - IEND) - 1

         IF (NLET .GT. NLEN) THEN
            WRITE(NOUT,*) '*** SUBSTITUTED FILENAME TOO LONG: ',NLET
            CALL ERRT(100,'FILNAMSUB',NE)
            RETURN
         ENDIF
  
C        PRESERVE END OF FILENAME AFTER SUBSTITUTION AREA    
         FILNAM(IGO+NAST:) = FILNAM(IEND+1:)

C        WRITE SUBSTITUTION CONTENTS INTO THE FILE NAME
         WRITE(FILNAM(IGO:IGO+NAST-1),FMT1,IOSTAT=IERR) IVALT
         IF (IERR .NE. 0) THEN
            WRITE(NOUT,*) '*** ERROR SUBSTITUTING INTO FILENAME ',IVALT
            CALL ERRT(100,'FILNAMSUB',NE)
            RETURN
         ENDIF

C        SEE IF ANY MORE SUBSTITUTIONS ARE NEEDED
         GOTO 10
      ENDIF

      IRTFLG = 0

      RETURN
      END
