C++************************************************************************
C
C $$ FMRS_1.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR       *
C *  LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR    *
C *  USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF     *
C *  THE CENTER FOR LABORATORIES AND RESEARCH                              *
C **************************************************************************
C  1D real mixed radix FFT.
C INPUT:  X(N) - real array
C OUTPUT: N even  X(N+2)
C   Order of elements:
C  R(0),0.0, R(1), I(1), R(2), I(2), ....., R(N/2-1), I(N/2-1), R(N/2),0.0
C
C         N odd  X(N+1)
C  R(0),0.0, R(1), I(1), R(2), I(2), ....., R(N/2-1), I(N/2-1), R(N/2),I(N/2-1)
C 
C INV: +1 forward FFT
C      -1 inverse FFT
C on output INV=0 indicates error
C
C--************************************************************************
C
C $$ FMRS_1.FOR
C
         SUBROUTINE  FMRS_1(X,N,INV)
         PARAMETER (LBUF=5000)
         DIMENSION  X(*),WORK(LBUF)
CNO_SAVE
#ifdef SP_LIBFFT
         IF(N+15.GT.LBUF)  THEN
         INV=0
C Insufficient buffer, increase LBUF and compile SPIDER
         CALL  ERRT(6,'FMRS_1',NE)
         RETURN
         ENDIF
	CALL  SCFFT1DUI(N,WORK)
	LDA=1
	IF(INV.GT.0)  THEN
	CALL  SCFFT1DU(INV,N,X,LDA,WORK)
	ELSE
	CALL  CSFFT1DU(INV,N,X,LDA,WORK)
	QT=1.0/FLOAT(N)
	CALL  SSCAL1D(N,QT,X,LDA)
	ENDIF
#else
         IF(N.GT.LBUF)  THEN
         INV=0
C Insufficient buffer, increase LBUF and compile SPIDER
         CALL  ERRT(6,'FMRS_1',NE)
         RETURN
         ENDIF
C inv can be +1 (forward FFT) or -1 (inverse FFT)
         IF(INV)  2,2,1
1        DO    I=1,N
         WORK(I)=0.0
	 ENDDO
         CALL FFTMCF(X,WORK,N,N,N,INV)
         IF(MOD(N,2))  12,12,13
12       DO    I=N+1,3,-2
         X(I)=X((I+1)/2)
         X(I+1)=WORK((I+1)/2)
	 ENDDO
         X(2)=0.0
         X(N+2)=0.0
         RETURN
13       DO    I=N,3,-2
         X(I)=X(I/2+1)
         X(I+1)=WORK(I/2+1)
	 ENDDO
         X(2)=0.0
         RETURN
C
2        DO    I=2,N/2+1
         WORK(I)=X(2*I)/N
  	 WORK(N-I+2)=-WORK(I)
	 ENDDO
         WORK(1)=0.0
C
         DO    I=1,N/2+1
	 X(I)=X(2*I-1)/N
	 ENDDO
         DO    I=N,N/2+2,-1
         X(I)=X(N-I+2)
	 ENDDO
C
         CALL FFTMCF(X,WORK,N,N,N,INV)
#endif
         END
