C++*******************************************************************
C
C $$ FQ_Q.FOR 
C 12/22/94           
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C
C $$ FQ_Q: QUICK FILTERING OF THE REAL-SPACE FILE BY IN-CORE FFT
C
C      CALL FQ_Q(LUN,LUNO,B,N2S,N2R,NSAM,NROW,IOPT)
C        LUN         LOGICAL UNIT NUMBER OF REAL-SPACE FILE TO BE FILTERED
C        LUNO        LOGICAL UNIT NUMBER OF REAL-SPACE OUTPUT FILE 
C        IOPT        TYPE OF THE FILTER
C        B           BUFFER 
C        NSAM,NROW   DIMENSIONS OF REAL-SPACE FILE
C        N2S=2*NSAM  AT LEAST
C        N2R=2*NROW  "    "
C  
C--*******************************************************************
C
C
        SUBROUTINE FQ_Q(LUN,LUNO,B,LSD,N2S,N2R,NSAM,NROW,IOPT)
	INCLUDE 'CMBLOCK.INC'
	DIMENSION  B(LSD,N2R)
	DOUBLE PRECISION  AVE
CNO_SAVE
C
C  READ  IMAGE
	DO    I=1,NROW
 	CALL  REDLIN(LUN,B(1,I),NSAM,I)
	ENDDO
C  AVERAGE PADDING
	IF(N2S.NE.NSAM.AND.N2R.NE.NROW)  THEN
	AVE=0.0
c$doacross local(i,j),reduction(ave)
	DO    J=1,NROW
	DO    I=1,NSAM
	AVE=AVE+B(I,J)
	ENDDO
	ENDDO
	AVE=AVE/FLOAT(NSAM)/FLOAT(NROW)
c$doacross local(i,j)
	DO    J=1,N2R
	DO    I=NSAM+1,N2S
	B(I,J)=AVE
	ENDDO
	ENDDO
c$doacross local(i,j)
	DO    J=NROW+1,N2R
	DO    I=1,NSAM
	B(I,J)=AVE
	ENDDO
	ENDDO
	ENDIF
C
	INV=1
	CALL  FMRS_2(B,N2S,N2R,INV)
	IF (INV.EQ.0) THEN
	IOPT=-1
	RETURN
	ENDIF
C
C BUTTERWORTH FILTER***********************
C
	IF(IOPT.EQ.7.OR.IOPT.EQ.8)  THEN
	EPS=0.882
	AA=10.624
	CALL RDPRM2(FP,FS,NLOOP,ILOOP,
     &  'PASS-BAND FREQUENCY & STOP-BAND FREQUENCY?')
	ORD=2.*ALOG10(EPS/SQRT(AA**2-1.0))
	ORD=ORD/ALOG10(FP/FS)
	PARM1=FP/(EPS)**(2./ORD)
C
C
	ELSEIF(IOPT.GE.1.OR.IOPT.LE.6)  THEN
C
C
C
	CALL RDPRM(PARM1,NLOOP,ILOOP,'FILTER RADIUS')
	IF(PARM1.LT.0.0.OR.PARM1.GT.0.5)  PARM1=0.5*PARM1/(NSAM/2)
	 IF(IOPT.EQ.5.OR.IOPT.EQ.6)  THEN
C
C FERMI DISTRIBUTION FILTER ********************
C
	 CALL RDPRM(TEMP,NLOOP,ILOOP,'TEMPERATURE(0=CUTOFF)')
C
C  EXPONENTIAL FOR HIGH-PASS OPTION
	 IF(IOPT.EQ.6) TEMP=-TEMP
	 ENDIF
	ENDIF
C
	NR2=N2R/2
	X1=FLOAT(NSAM/2)**2
	Y1=FLOAT(NR2)**2
	PARM=PARM1**2
C
c$doacross local(i,j,ix,iy,f)
	DO 35 J=1,N2R
	IY=(J-1)
	IF(IY.GT.NR2) IY=IY-N2R
	DO 35 I=1,LSD,2
	IX=(I-1)/2
C
	IF(IOPT.EQ.1) THEN
C LOWPASS ********************************
        IF(0.25*(FLOAT(IX*IX)/X1+FLOAT(IY*IY)/Y1) .GT. PARM)  THEN
	B(I,J)=0.0
	B(I+1,J)=0.0
	ENDIF
	ELSEIF (IOPT.EQ.2) THEN	
C HIGH PASS ******************
	IF((IX.NE.0 .OR. IY.NE.0) .AND.
     &  0.25*(FLOAT(IX*IX)/X1+FLOAT(IY*IY)/Y1) .LE. PARM)  THEN
	B(I,J)=0.0
	B(I+1,J)=0.0
	ENDIF
	ELSEIF(IOPT.EQ.3)  THEN
C GAUSSIAN LOW PASS ***************************
	F=EXP(-0.25*(FLOAT(IX*IX)/X1+FLOAT(IY*IY)/Y1)/(2.*PARM))
        B(I,J)=B(I,J)*F
        B(I+1,J)=B(I+1,J)*F
	ELSEIF(IOPT.EQ.4)  THEN	
C GAUSSIAN HIGH PASS ********************************
	IF(IX.NE.0 .OR. IY.NE.0)  THEN
	F=(1.0-EXP(-0.25*(FLOAT(IX*IX)/X1+FLOAT(IY*IY)/Y1)/(2.*PARM)))
        B(I,J)=B(I,J)*F
        B(I+1,J)=B(I+1,J)*F
	ENDIF
C FERMI DISTRIBUTION FILTER ********************
	ELSEIF(IOPT.EQ.5.OR.IOPT.EQ.6)  THEN
	F=(0.5*SQRT(FLOAT(IX*IX)/X1+FLOAT(IY*IY)/Y1)-PARM1)/TEMP
	F=AMIN1(AMAX1(F,-10.0),10.0)
	IF(IOPT.EQ.6 .AND. IX.EQ.0 .AND.IY.EQ.0)  GO TO 35
        F=(1.0/(1.0+EXP(F)))
        B(I,J)=B(I,J)*F
        B(I+1,J)=B(I+1,J)*F
C BUTTERWORTH LOWPASS FILTER **************
	ELSEIF (IOPT.EQ.7)THEN
	F=0.5*SQRT(FLOAT(IX*IX)/X1+FLOAT(IY*IY)/Y1)
	F=SQRT(1.0/(1.0+(F/PARM1)**ORD))
        B(I,J)=B(I,J)*F
        B(I+1,J)=B(I+1,J)*F
C BUTTERWORTH HIGHPASS FILTER *************
	ELSEIF (IOPT.EQ.8)THEN
        IF(IX.NE.0 .OR. IY.NE.0) THEN
	F=0.5*SQRT(FLOAT(IX*IX)/X1+FLOAT(IY*IY)/Y1)
	F=(1.0-SQRT(1.0/(1.0+(F/PARM1)**ORD)))
        B(I,J)=B(I,J)*F
        B(I+1,J)=B(I+1,J)*F
	ENDIF
	ENDIF
 35   	CONTINUE
C  WRITE  IMAGE
665     CONTINUE
	INV=-1
	CALL  FMRS_2(B,N2S,N2R,INV)
	DO    I=1,NROW
 	CALL  WRTLIN(LUNO,B(1,I),NSAM,I)
	ENDDO
        END
