C    
C                                     DOC LONGFILENAME JULY 89 AL
C
C *******************************************************************
C
C	PROGRAM TO CALCULATE THE WEIGHTING FUNCTION ALLONG A SECTION
C       IN FOURIER SPACE CORRESPONDING TO A PROJECTION. THE WEIGHTING
C       FUNCTION IS THE INVERS OF THE SUM OF THE SINC-FUNCTIONS ALONG
C       THE Z-DIRECTIONS IN THE COORDINATE SYSTEMS OF EACH PROJECTION
C       
C       LET PW BE THE PROJECTION TO BE WEIGHTED AND PG THE PROJECTION
C       GENERATING THE SINC-FUNCTION. LET RW(W) BE A VECTOR IN THE CO-
C       ORDINATE SYSTEM BELONGING TO PW AND RW(G) THE SAME VECTOR RE-
C       PRESENTED IN THE COORDINATE SYSTEM OF PG. THE ARGUMENT OF THE
C       SINC-FUNCTION IS THE PLANE RW(G) WITH ZW=0 (ZW Z-COORDINATE IN
C       PW)
C
C       LET R BE A VECTOR IN THE SPACE FIXED COORDINATE SYSTEM. [DW]
C       THE ROTATION MATRIX FROM R TO RW (R(W)=[DW]*R). ANALOGOUS LET
C       [DG] BE THE ROTATION MATRIX FROM R TO RG (R(G)=[DG]*R). THE
C       ARGUMENT OF THE SINC FUNCTION THEN IS: PROJECTION ONTO ZG OF
C
C                                -1
C       RW(G)=[D]*RW(W)=[DG]*[DW]  *RW(W)
C                     :=[DGTHE]*[DGPHI]*[DWPHI]*[DWTHE]*RW(W).
C
C       THE MATRICES USED ARE (CONSISTEND WITH MRQBXS USED IN THE SIMPLE
C       BACKPROJECTION CALLED IN SPIDER BY BP XY):
C            
C           |COS(THETA(G)) 0 -SIN(THETA(G))| | COS(PHI(G)) SIN(PHI(G) 0|
C     [DG]= |       0       1       0      |*|-SIN(PHI(G)) COS(PHI(G) 0|
C           |SIN(THETA(G)) 0  COS(THETA(G))| |      0          0      1|
C
C        -1 |COS(PHI(W)) -SIN(PHI(W)) 0| | COS(THETA(W)) 0 SIN(THETA(W))|
C    [DW]  =|SIN(PHI(W))  COS(PHI(W)) 0|*|       0       1       0      |
C           |     0            0      1| |-SIN(THETA(W)) 0 COS(THETA(W))|
C
C       THE MATRIX MULTIPLICATION IS DONE EXPLICITELY TO FACILITATE LATER
C       CHANGES.
C
C ************************************************************************

        SUBROUTINE GENW

        PARAMETER (IBUF=33280)

C IARRY,IARRY2 ARRAYS FOR GETNUM, PLIST FOR DOCUMENT FILE BUFFER.
C        DIMENSION IARRY(4000),IARRY2(4000),PLIST(10),N(3)

        COMMON /DEBUG/DFLAG
        COMMON /COMMUN/DOCF

C     ADUM    DUMMY
C     SINT   FOR TABULARIZED SINE FUNCTION
C     W       WEIGHTING FUNCTION
C     THE REST ARE BUFFERS.
        COMMON ADUM(80),SINT(12800),BB(100),BUF(256),W(IBUF),
     $  A(IBUF),
     $  PHIG(4000),THEG(4000),PHIW(4000),THEW(4000),ANG(4),
     $  IARRY(4000),IARRY2(4000),PLIST(10),N(3),
     $  CDEL(4000),CGAM(4000),SGAM(4000),CTHE(4000),STHE(4000),
     $  CPHI(4000)
     $  ,SPHI(4000),SDEL(4000)

        INCLUDE 'CMBLOCK.INC'
        INCLUDE 'LABLOCK.INC'

        DOUBLE PRECISION DARG,DXA,PI
        CHARACTER*1 YESNO,FA,FB  
        LOGICAL DFLAG,DBFLAG   
        CHARACTER * 81 DOCF,FILNAM,PREF,FILO,WNAM
        CHARACTER * 81 FILPAT1,FILPAT3
        CHARACTER * 1  NULL,CDUM

C 	ARRAY JUST FOR DEBUG :
C        DIMENSION ZZZ(IBUF),FFF(IBUF)

        DATA PI/3.1415927/
        DATA LUN/15/,NDOC/16/,LUNW/27/
CNO_SAVE	
	
        NULL=CHAR(0)
        DFLAG=.FALSE.
        DBFLAG=.FALSE.
        CALL RDPRM(CRIT,NLOOP,ILOOP,'CRITICAL VALUE FOR W')
        IF(CRIT.EQ.0) CRIT=10.E-5

        CALL RDPRM(RAY,NLOOP,ILOOP,'3D VOLUME DIAMETER')
C
C     GET FILE NAMES AND NUMBERS:
C
C	NO NEED TO UPPER CASE IN RDPRMC.F, set IRTFLG = -999
        IRTFLG = -999
        NC=1
        CALL RDPRMC(FA,NC,.TRUE.,
     $  'GENER.ANGLES IN (L)ABEL,(D)OCF,(E)XTERNAL',CDUM,IRTFLG)
        IRTFLG = -999
        NC=1
        CALL RDPRMC(FB,NC,.TRUE.,
     $  'WGT. ANGLES IN (L)ABEL,(D)OCF,(E)XTERNAL',CDUM,IRTFLG)
C
C   FB='E' INDICATES THE ANGLES OF PW ARE GIVEN EXTERNALLY.
C   FB='D' INDICATES THE ANGLES OF PW ARE CONTAINED IN A DOCUMENT FILE.
C   OTHERWISE THE ANGLES OF PW ARE CONTAINED IN IMAGE LABELS.
C   FA='D' INDICATES THE ANGLES OF PG ARE CONTAINED IN A DOCUMENT FILE.
C   FA='E' INDICATES THE ANGLES OF PG ARE GIVEN EXTERNALLY.
C   OTHERWISE THEY ARE CONTAINED IN THE LABEL OF THE IMAGES.
C   THUS THE DEFAULT IS, THAT ALL ANGLES ARE CONTAINED IN THE IMAGE LABELS.
C 

        IRTFLG = -999
        CALL RDPRMC( FILPAT1,NCHAR1,.TRUE.,
     $  'TEMPLATE OF INPUT FILE SERIES',CDUM,IRTFLG)
        FILPAT1(NCHAR1+1:NCHAR1+1)=NULL
        IRTFLG = -999
        CALL RDPRMC(FILPAT3,NCHAR3,.TRUE.,
     $  'TEMPLATE OF OUTPUT FILE SERIES',CDUM,IRTFLG) 
        FILPAT3(NCHAR3+1:NCHAR3+1)=NULL

        IHOW=4000
        IHI=4000
        ILOW=0
        CALL RDPRAI(IARRY,4000,IHOW,ILOW,IHI,
     $  '.FILE NUMBERS OF GENERATING FILES:',CDUM,IRTFLG)
        IHOW2=4000
        CALL RDPRAI(IARRY2,4000,IHOW2,ILOW,IHI,
     $  '.FILE NUMBERS OF TO BE WEIGHTED FILES:',CDUM,IRTFLG)

C     GET THE ANGLES FOR ALL FILES:
C
        IF(FA.EQ.'E' .OR. FA.EQ.'e' ) THEN 
C
C     ANGLES OF PG GIVEN EXTERNALLY:
C
        DO  I=1,IHOW
        WRITE(NOUT,100)
100     FORMAT(' INPUT DATA OF THE PROJECTION SET:')
        CALL RDPRM(GTHE,NLOOP,ILOOP,'THETA')
        CALL RDPRM(GPHI,NLOOP,ILOOP,'PHI')
        PHIG(I)=GPHI/180.*PI
        THEG(I)=GTHE/180.*PI
	ENDDO
        ENDIF

        IF(FA.EQ.'D' .OR. FA.EQ.'d' ) THEN 
C
C     ANGLES OF PG CONTAINED IN DOCUMENT FILE:
C
        CALL FILERD(DOCF,NLET,NULL,'DOC1',IRTFLG)
        NOPEN=0
        NLIST=3
        KK=0
        DO  K=1,IHOW
        CALL UNSAV(DOCF,NOPEN,NDOC,IARRY(K),PLIST,NLIST,LERR,1)
        NOPEN=1
        IF(LERR.NE.0) CALL ERRT(4,' GENW',IE)
        IF(PLIST(3).EQ.1) THEN 
        KK=KK+1
        THEG(KK)=PLIST(1)/180.*PI
	PHIG(KK)=PLIST(2)/180.*PI
        IARRY(KK)=IARRY(K)
        ENDIF
	ENDDO
        CLOSE(NDOC)
        IHOW=KK
        ENDIF

        IF(FA.EQ.'L' .OR. FA.EQ.'l' ) THEN 

C     ANGLES OF PG CONTAINED IN FILE TITLE OR LABEL
C
        II=0

        DO 3 I=1,IHOW
        CALL FILGET(FILPAT1,FILNAM,NCHAR1,IARRY(I),IRTFLAG)
        NDUM=1
	CALL OPENFB(BUF,FILNAM,LUN,NSAM,NROW,'OLD',IER,NDUM)
        IF(IER.NE.2) THEN
        WRITE(NOUT,*) '*** FILE NOT FOUND ***'
        GOTO 3
        ENDIF

        IF(NSAM.GT.128.OR.NROW.GT.128) THEN 
        WRITE(NOUT,123)
123     FORMAT(' MAXIMUM IMAGE SIZE 128**2')
        CALL ERRT(6,'GENW',IE)
        CLOSE (LUN)
        RETURN
        ENDIF

        II=II+1
        NN = 2
        CALL GETANG(LUN,NSAM,BUF,15,NN,ANG,IRTFLG)
        PHIG(II)=ANG(1)/180.*PI
        THEG(II)=ANG(2)/180.*PI
        IARRY(II)=IARRY(I)

        CLOSE(LUN)
3	CONTINUE
        IHOW=II
        ENDIF
C>>>
C***************************************************************
C GET NSAM AND NROW:
C
        CALL FILGET(FILPAT1,FILNAM,NCHAR1,IARRY(1),IRTFLG)
        NDUM=1
	CALL OPENFB(BUF,FILNAM,LUN,NSAM,NROW,'OLD',IER,NDUM)
        IF(IER.NE.2) CALL ERRT(4,' GENW',IE)
C>>>
        IF(NSAM.GT.128.OR.NROW.GT.128) THEN 
        WRITE(NOUT,123)
        CALL ERRT(6,'GENW',IE)
        ENDIF
        CLOSE (LUN)
        UNIT=NSAM

C     THE ANGULAR DATA OF THE GENERATING PROJECTIONS ARE KNOWN
C     NOW. 
C     FIND ANGLES OF THE SET OF PROJECTIONS WHICH ARE TO BE WEIGHTED.

C     FIND OUT IF BOTH ARE IDENTICAL
C     IF THEY ARE , ALL ANGLES ARE KNOWN ALLREADY.
C
        IRTFLG = -999
        NC=1
	CALL RDPRMC(YESNO,NC,.TRUE.,
     $  'BOTH IMAGE SETS IDENTICAL (Y/N)',CDUM,IRTFLG)

        IF(YESNO.EQ.'Y') THEN 
        DO  I=1,IHOW2
        PHIW(I)=PHIG(I)
        THEW(I)=THEG(I)
	ENDDO
        GOTO 1007
        ENDIF

        IF(FB .EQ. 'E' .OR. FB .EQ.'E' ) THEN 
C
C     ANGLES OF PW GIVEN EXTERNALLY:
C
        DO  I=1,IHOW2
        WRITE(NOUT,103)
103     FORMAT(' INPUT DATA OF THE PROJECTION SET TO BE WEIGHTED:')
        CALL RDPRM(WPHI,NLOOP,ILOOP,'PHI')
        CALL RDPRM(WTHE,NLOOP,ILOOP,'THETA')
        PHIW(I)=WPHI/180.*PI
        THEW(I)=WTHE/180.*PI
	ENDDO
        ENDIF
 
        IF(FB.EQ.'D'  .OR. FB .EQ.'d' ) THEN 
C
C     ANGLES OF PW CONTAINED IN DOCUMENT FILE:
C
        CALL FILERD(DOCF,NLET,NULL,'DOC1',IRTFLG)
        NOPEN=0
        NLIST=3
        KK=0
        DO  K=1,IHOW2
        CALL UNSAV(DOCF,NOPEN,NDOC,IARRY2(K),PLIST,NLIST,LERR,1)
        NOPEN=1
        IF(LERR.NE.0) CALL ERRT(4,' GENW',IE)
        IF(PLIST(3).NE.0) THEN
        KK=KK+1
        THEW(KK)=PLIST(1)/180.*PI
	PHIW(KK)=PLIST(2)/180.*PI
        ENDIF
	ENDDO
        CLOSE(NDOC)
        IHOW2=KK
        ENDIF
 
        IF(FB.EQ.'L'  .OR. FB .EQ.'l' ) THEN 

C     ANGLES OF PW CONTAINED IN FILE TITLE OR LABEL
C
        II=0
        DO 333 I=1,IHOW2
        CALL FILGET(FILPAT1,FILNAM,NCHAR1,IARRY2(I),IRTFLG)
        NDUM=1
	CALL OPENFB(BUF,FILNAM,LUN,NSAM,NROW,'OLD',IER,NDUM)
        IF(IER.NE.2) THEN
        WRITE(NOUT,*) '*** FILE NOT FOUND ***'
        GOTO 333
        ENDIF
        II=II+1
        NN = 2
        CALL GETANG(LUN,NSAM,BUF,15,NN,ANG,IRTFLG)
        PHIW(II)=ANG(1)/180.*PI
        THEW(II)=ANG(2)/180.*PI
        IARRY2(II)=IARRY2(I)
        CLOSE(LUN)
 333	CONTINUE
        IHOW2=II
        ENDIF
 
1007    CONTINUE


C
C     NOW ALL ANGLES ARE KNOWN. PHIG,THEG FOR THE GENERATING PROJECTIONS
C     PHIW,THEW FOR THE PROJECTION TO BE WEIGHTED.
C
C     FIRST CALCULATE THE WEIGHTING FUNCTIONS (REAL)
C
C     BEGIN TO CALCULATE ALL MATRICES.
C
C     MATRICES TO TRANSFORM FROM THE GENERATING PROJECTIONS:
C
        DO  J=1,IHOW
        CTHE(J)=COS(THEG(J))
        STHE(J)=SIN(THEG(J))
 
        CPHI(J)=COS(PHIG(J))
        SPHI(J)=SIN(PHIG(J))
 
	ENDDO


C PRINT OUT THE GIVEN ANGLES:

        WRITE(NOUT,299) 
299     FORMAT(' ANGLES USED TO GENERATE THE WEIGHTING FUNCTION')
        WRITE(NOUT,298) (K,IARRY(K),THEG(K)/PI*180.,
     $  PHIG(K)/PI*180.,K=1,IHOW) 
298     FORMAT(' PROJ.# ',I5,'IMAGE # ',I5,' THETA:',F10.4,
     $  '  PHI:',F10.4)
  

        WRITE(NOUT,499) 
499     FORMAT(' ANGLES OF PROJECTIONS THAT ARE BEING WEIGHTED')
        WRITE(NOUT,498) (K,IARRY2(K),
     $  THEW(K)/PI*180.,PHIW(K)/PI*180.,K=1,IHOW2) 
498     FORMAT(' PROJ.# ',I5,'IMAGE # ',I5,
     $  ' THETA:',F10.4,' PHI:',F10.4)


C      MATRICES TO TRANSFORM TO THE WEIGHTED PROJECTIONS:
C
        DO  KW=1,IHOW2
 
C
        CDEL(KW)=COS(THEW(KW))
        SDEL(KW)=SIN(THEW(KW))
C
        CGAM(KW)=COS(PHIW(KW))
        SGAM(KW)=SIN(PHIW(KW))

        WRITE(NOUT,501) 
501     FORMAT(' ANGLE OF PROJECTION THAT IS BEING WEIGHTED')
        WRITE(NOUT,502) KW,(THEW(KW)/PI*180.),CDEL(KW),
     $   (PHIW(KW)/PI*180.),CGAM(KW),SGAM(KW)
502     FORMAT(' PROJ.# ',I5,' THETA:',F10.4,' COS:',F10.4,
     $  ' PHI:',F10.4,' COS:',F10.4,' SIN:',F10.4)

 
C  GET FILE DIMENSIONS: 

        IF (KW.EQ.1) THEN
           CALL FILGET(FILPAT1,FILNAM,NCHAR1,IARRY(1),IRTFLG)
           CALL OPENFB(BUF,FILNAM,LUN,NSAM,NROW,'OLD',IER,NDUM)
           CLOSE (LUN)
        ENDIF

C
C CLEAR THE ARRAY FOR THE WEIGHTING FUNCTION: 
        DO  I=1,NROW
        DO  K=1,NSAM+2
        W((I-1)*(NSAM+2)+K)=0.
	ENDDO
	ENDDO
C
C
        DO  J=1,IHOW
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,199) J
199     FORMAT(' J=',I4)
C
C CALCULATE THE MATRIX TO TRANSFORM THE COORDINATES IN THE SYSTEM OF 
C THE J-TH GENERATING PROJECTION INTO THE SYSTEM OF THE KW-TH TO BE
C WEIGHTED PROJECTION. 
C

        FX=CDEL(KW)*STHE(J)*(CPHI(J)*CGAM(KW)
     $               +SPHI(J)*SGAM(KW))-SDEL(KW)*CTHE(J)
        FY=STHE(J)*(-SGAM(KW)*CPHI(J)+CGAM(KW)*SPHI(J))


C
C       NOW GO OVER THE IMAGE AND ADD THE CONTRIBUTION OF THE J-TH
C       PROJECTION TO THE WEIGHTING FUNCTION
C       CALCULATE WEIGHTING FUNCTION IN CORE
C
        KEND=NSAM/2+1
        DO  I=1,NROW/2
        IY=I-1
        INDC=(I-1)*KEND
        DO  K=1,KEND
        INDX=INDC+K
        IK=K-1
C
C       IXR=INDEX OF REAL PART
C       IXI=INDEX OF IMAGINARY PART
C
        IXI=2*INDX
        IXR=IXI-1
        Z=FX*FLOAT(IK)+FY*FLOAT(IY) 
	
C IF SINC FUNCTION SHOULD BE NOT LARGER THAN 1, OTHERWISE USE LINE ABOVE
        FUNCT=1
        DXA=PI*Z/UNIT
        DARG=RAY*DXA
	
C IF SINC FUNCTION SHOULD BE NOT LARGER THAN 1, OTHERWISE USE LINE ABOVE
        IF(DXA.NE.0) FUNCT=SIN(DARG)/DARG
        W(IXR)=W(IXR)+FUNCT
        W(IXI)=W(IXR)
	ENDDO
        
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,107)I
107     FORMAT(' REAL LINE #:',I5)
        INDW=2*INDC
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,108) (W(INDW+LLL),LLL=1,NSAM+2,2)
108     FORMAT(' REW:',9F7.2)
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,907)I
907     FORMAT(' IMAGINARY LINE #:',I5)
        INDW=2*INDC
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,108) (W(INDW+LLL),LLL=2,NSAM+2,2)
908     FORMAT(' IMW:',9F7.2)
	ENDDO
787     FORMAT(' Z-COORD.:',8(I3,1X,F9.3,1X))
786     FORMAT(' FFF=: ',9F7.2)

        DO  I=NROW/2+1,NROW
        IY=NROW-I+1
        INDC=(I-1)*KEND
        DO  K=1,KEND
        INDX=INDC+K
        IK=K-1
C
C       IXR=INDEX OF REAL PART
C       IXI=INDEX OF IMAGINARY PART
C
        IXI=2*INDX
        IXR=IXI-1
        Z=FX*FLOAT(IK)+FY*FLOAT(-IY)
C IF SINC FUNCTION SHOULD BE NOT LARGER THAN 1, OTHERWISE USE LINE ABOVE
        FUNCT=1
        DXA=PI*Z/UNIT
        DARG=RAY*DXA
C IF SINC FUNCTION SHOULD BE NOT LARGER THAN 1, OTHERWISE USE LINE ABOVE
        IF(DXA.NE.0) FUNCT=SIN(DARG)/DARG
        W(IXR)=W(IXR)+FUNCT
        W(IXI)=W(IXR)
	ENDDO
        
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,107)I
        INDW=2*INDC
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,108) (W(INDW+LLL),LLL=1,NSAM+2,2)
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,907)I
        INDW=2*INDC
        IF(DFLAG.AND.DBFLAG)WRITE(NOUT,108) (W(INDW+LLL),LLL=2,NSAM+2,2)
	ENDDO

C   TAKE NEXT PG
C
	ENDDO
C
C HERE THE WEIGHTING FUNCTION FOR THE KW-TH PROJECTION IS READY
C PUT IT INTO THE CORRECT FORMAT FOR FOURIER MULTIPLICATION
C

        IF(DFLAG) WRITE(NOUT,750)
750     FORMAT(' THE COMPLETE WEIGHTING FUNCTION:')
        NTOT1=(NSAM+2)*NROW
        IF(DFLAG) WRITE(NOUT,751) (W(LLL),LLL=1,NTOT1)
751     FORMAT(' ',9F7.2)

C       NOW W CONTAINS THE TRANSFER FUNCTION
C
C       READ IN PW 
C
        CALL FILGET(FILPAT1,FILNAM,NCHAR1,IARRY2(KW),IRTFLG)
        CALL OPENFB(BUF,FILNAM,LUN,NSAM,NROW,'OLD',IER,NDUM)
        IF(IER.NE.2) CALL ERRT(4,' GENW',IE)
        DO  I=1,NROW
        INDEX=(I-1)*NSAM+1
	CALL REDLIN(LUN,A(INDEX),NSAM,I)
	ENDDO
        IF(DFLAG)WRITE(NOUT,110) (A(IN),IN=1,NROW*NSAM)
110     FORMAT(' R',8F7.2)
        CLOSE(LUN)
C
C   DO THE ACTUAL WEIGHTING:
C
C  FOURIER TRANSFORM IMAGE, MULTIPLY THE TRANSFORM W
C
        N(1)=NSAM
        N(2)=NROW
        N(3)=1
        NTOT1=(NSAM+2)*NROW
        NTOT2=(NSAM)*NROW
        SCF1=SQRT(FLOAT(NTOT2/4))
        SCF2=SQRT(FLOAT(NTOT2))
C
C  IN CORE FOURIER TRANSFORM:
C
        CALL FOUR2(A,N,2,-1,0)
        DO  I=1,NTOT1
        A(I)=A(I)/SCF2
	ENDDO
        IF(DFLAG)WRITE(NOUT,111) (A(IN),IN=1,NTOT1)
111     FORMAT(' F',9F6.3)
C
C  DO THE MULTIPLICATION:
C
        IMINI=NTOT1
        RMIN=NSAM
        IXMIN=NSAM
        IYMIN=NROW
        DO  I=1,NTOT1
        IF(W(I).GT.CRIT) THEN 
        W(I)=1/W(I)
        ELSE
        W(I)=1/CRIT
        ENDIF
        A(I)=A(I)*W(I)
	ENDDO

        IF(DFLAG)WRITE(NOUT,109) (A(I),I=1,NTOT1)
109     FORMAT(' W',9F6.3)
C
C  IN CORE BACK TRANSFORM:
C
        CALL FOUR2(A,N,2,+1,-1)
        DO  I=1,NTOT2
        A(I)=A(I)/SCF1
	ENDDO
        IF(DFLAG)WRITE(NOUT,115) (A(I),I=1,NTOT2)
115     FORMAT(' BT',8F6.3)
        CALL FILGET(FILPAT3,FILO,NCHAR3,IARRY2(KW),IRTFLG)
        NDUM=1
	CALL OPENFB(BUF,FILO,LUN,NSAM,NROW,'UNKNOWN',IER,NDUM)
        IF(IER.NE.2) CALL ERRT(4,' GENW',IE)
        DO  I=1,NROW
        IND1=(I-1)*NSAM+1
        IF(DFLAG)WRITE(NOUT,112) (A(IND1-1+IN),IN=1,NSAM)
112     FORMAT(' B',16F6.3)
        CALL WRTLIN(LUN,A(IND1),NSAM,I)
	ENDDO

C TRANFER ANGLES INTO OUTPUT FILES:
        ANG(1)=IANGLE
C QUICK FIX TO CORRECT TITLE:
        ANG(2)=PHIW(KW)/PI*180.
        ANG(3)=THEW(KW)/PI*180.
        NOPRNT=-1
        CALL SETLAB(LUN,NSAM,BUF,14,3,ANG,'U',NOPRNT)

        CLOSE (LUN)

        WRITE(NOUT,223) RMIN,IXMIN,IYMIN
223     FORMAT(' CRITICAL VALUES OF W FOUND AT FOURIER RADIUS:',
     $  F10.2,' K=',I5,' L=',I5)
        WRITE(NOUT,224) IMINI 
224     FORMAT(' CRITICAL VALUES OF W FOUND AT INDEX:'
     $  ,I5)
	ENDDO
        RETURN
        END
