
C *******************************************************************
C
C	PROGRAM TO CALCULATE THE WEIGHTING FUNCTION IN THREE DIMENSIONS
C       IN FOURIER SPACE. THE WEIGHTING FUNCTION IS THE INVERS OF THE 
C       SUM OF THE SINC-FUNCTIONS ALONG
C       THE Z-DIRECTIONS IN THE COORDINATE SYSTEMS OF EACH PROJECTION PG.
C       
C      LET R BE THE POINT IN THE THREE-DIMENSIONAL WEIGHTING
C      FUNCTION. PG THE PROJECTION GENERATING THE SINC-FUNCTION.  LET
C      RG BE THE SAME VECTOR REPRESENTED IN THE COORDINATE SYSTEM OF
C      PG. THE ARGUMENT OF THE SINC-FUNCTION IS THE Z-COORDINATE OF
C      POINT R IN THE COORDINATE SYSTEM OF PROJECTION PG.
C
C      THE TRANSFORM BETWEEN THE 2 SYSTEMS IS:
C
C       RG=[D]*R=[DGPSI]*[DGTHE]*[DGPHI]*R
C
C       THE MATRICES USED ARE (CONSISTEND WITH MRQBXS USED IN THE SIMPLE
C       BACKPROJECTION CALLED IN SPIDER BY BP XY):
C            
C           | COS(PSI(G)) SIN(PSI(G) 0| |COS(THETA(G)) 0 -SIN(THETA(G))| | COS(PHI(G)) SIN(PHI(G) 0|
C      [D]= |-SIN(PSI(G)) COS(PSI(G) 0|*|       0       1       0      |*|-SIN(PHI(G)) COS(PHI(G) 0|
C           |      0          0      1| |SIN(THETA(G)) 0  COS(THETA(G))| |      0          0      1|
C
C        NOTE THAT THE ZG-COORDINATE IS INDEPENDENT OF PSI.
C ************************************************************************

        SUBROUTINE GENW3D

        PARAMETER (IBUF=1024)

        COMMON /DEBUG/DFLAG
        COMMON /COMMUN/DOCF

        COMMON ADUM(80),
     $  PHIG(4000),THEG(4000),
     $  ANG(8),
     $  IARRY(4000),IARRY2(4000),PLIST(16),N(8),
     $  FX(4000),FY(4000),FZ(4000),RVOL(IBUF),BUF(IBUF),W(IBUF)
     
        INCLUDE 'CMBLOCK.INC'
        INCLUDE 'LABLOCK.INC'

        DOUBLE PRECISION 
     $  darg,dxa,pi,WADD,FX,FY,FZ,ST,UNIT,RAY,THD,PHD,DIST,GSIG
        CHARACTER*1 FA 
        LOGICAL INOPEN,WOPEN,DFLAG,DBFLAG,INVFLAG,GFLAG,SFLAG   
        CHARACTER * 81 DOCF,FILNAM,FILNAM1,FILNAM2
        CHARACTER * 81 FILPAT1
        CHARACTER * 1  NULL,CDUM,YN


        DATA PI/3.1415927/
        DATA LUN/15/,LUN1/16/,LUN2/17/,NDOC/18/
CNO_SAVE
	IRTFLG = -999
        NULL=CHAR(0)

         DFLAG=.FALSE.
         DBFLAG=.FALSE.
C
C USE GAUSSIAN OR SINC-FUNCTION
C
         CALL RDPRMC(YN,NA,.TRUE.,'(S)INC-FCT. OR (G)AUSS-FCT.',
     $   NULL,IRTFLG)
         IF(YN.EQ.'G' .OR. YN.EQ.'g')THEN
         GFLAG=.TRUE.
         SFLAG=.FALSE.
         WRITE(NOUT,*) 'GAUSSIAN'
         ELSE
         GFLAG=.FALSE.
         SFLAG=.TRUE.
         WRITE(NOUT,*) 'SINC-FUNCTION'
         ENDIF
C
C  CALCULATE TRANSFER OR WIGHTING FUNCTION?
C
         INVFLAG=.TRUE.
        CALL RDPRMC(YN,NA,.TRUE.,
     $  '(W)EIGHTING- OR (T)RANSFER-FUNCTION',NULL,IRTFLG)
        IF(YN.EQ.'T' .OR. YN.EQ.'t') THEN 
        INVFLAG=.FALSE.
        WRITE(NOUT,*) '** CALCULATING TRANSFER FUNCTION **'
        ELSE 
        WRITE(NOUT,*) '** CALCULATING WEIGHTING FUNCTION **'
        ENDIF
        CALL RDPRM(CRIT,NLOOP,ILOOP,'CRITICAL VALUE FOR W')
        IF(CRIT.EQ.0) CRIT=10.E-5

        CALL RDPRM(RRAY,NLOOP,ILOOP,'3D VOLUME DIAMETER')
        RAY=DBLE(RRAY)
C
C     GET FILE NAMES AND NUMBERS:
C
        NC=1
	IRTFLG = -999
        CALL RDPRMC(FA,NC,.TRUE.,
     $  'GENER.ANGLES IN (L)ABEL,(D)OCF,(E)XTERNAL',CDUM,IRTFLG)
        NC=1
C
C   FA='D' INDICATES THE ANGLES OF PG ARE CONTAINED IN A DOCUMENT FILE.
C   FA='E' INDICATES THE ANGLES OF PG ARE GIVEN EXTERNALLY.
C   OTHERWISE THEY ARE CONTAINED IN THE LABEL OF THE IMAGES.
C   THUS THE DEFAULT IS, THAT ALL ANGLES ARE CONTAINED IN THE IMAGE LABELS.
C 


C       NSW=1
C        WRITE(NOUT,101)
C101	FORMAT($,1X,')
        IHOW=4000
        IHI=4000
        ILOW=0
        CALL RDPRAI(IARRY,4000,IHOW,ILOW,IHI,
     $  '.FILE NUMBERS OF PROJECTION FILES:',CDUM,IRTFLG)
C
C     GET THE ANGLES FOR ALL FILES:
C
C*****************************************************************
C>>>
        IF(FA.EQ.'E'.OR.FA.EQ.'e') THEN 
C
C     ANGLES OF PG GIVEN EXTERNALLY:
C
        DO 1 I=1,IHOW
        WRITE(NOUT,110)
110     FORMAT(1H ,'INPUT DATA OF THE PROJECTION SET:')
        CALL RDPRM(GTHE,NLOOP,ILOOP,'THETA')
        CALL RDPRM(GPHI,NLOOP,ILOOP,'PHI')
        PHIG(I)=GPHI/180.*PI
        THEG(I)=GTHE/180.*PI
1       CONTINUE
        ENDIF
C>>>
C***************************************************************
C>>>
        IF(FA.EQ.'D'.OR.FA.EQ.'d') THEN 
C
C     ANGLES OF PG CONTAINED IN DOCUMENT FILE:
C
        CALL FILERD(DOCF,NLET,NULL,'DOC1',IRTFLG)
        NOPEN=0
        NLIST=4
        KK=0
        DO 2 K=1,IHOW
        CALL UNSAV(DOCF,NOPEN,NDOC,IARRY(K),PLIST,NLIST,LERR,1)
        NOPEN=1
        IF(LERR.NE.0) THEN 
        WRITE(NOUT,112) K
112     FORMAT(1X,'KEY ',I4,' NOT FOUND')
        GOTO 2
        ENDIF
        IF(PLIST(4).EQ.1) THEN 
        KK=KK+1
	PHIG(KK)=PLIST(1)/180.*PI
        THEG(KK)=PLIST(2)/180.*PI
        IARRY(KK)=IARRY(K)
        ENDIF
2       CONTINUE
        CLOSE(NDOC)
        IHOW=KK
        ENDIF
C>>>
C**********************************************************
C>>>
        IF(FA.EQ.'L'.OR.FA.EQ.'l') THEN 

C     ANGLES OF PG CONTAINED IN FILE TITLE OR LABEL
C

        II=0
	IRTFLG = -999
        CALL RDPRMC( FILPAT1,NCHAR1,.TRUE.,
     $  'TEMPLATE OF PROJECTION FILE SERIES',CDUM,IRTFLG)
        FILPAT1(NCHAR1+1:NCHAR1+1)=NULL

        DO 3 I=1,IHOW
        CALL FILGET(FILPAT1,FILNAM,NCHAR1,IARRY(I),IRTFLAG)
        NDUM=1
	CALL OPENFB(BUF,FILNAM,LUN,NSAM,NROW,'OLD',IER,NDUM)
        IF(IER.NE.2) THEN
        WRITE(NOUT,111) FILNAM
111     FORMAT(1X,A40,'*** NOT FOUND ***')
        GOTO 3
        ENDIF
C>>>
        IF(NSAM.GT.128.OR.NROW.GT.128) THEN 
        WRITE(NOUT,123)
123     FORMAT(1H ,'MAXIMUM IMAGE SIZE 128**2')
        CALL ERRT(6,'GENW',IE)
        CLOSE (LUN)
        RETURN
        ENDIF
C>>>
        II=II+1
        NN = 3
        CALL GETANG(LUN,NSAM,BUF,15,NN,ANG,IRTFLG)
        PHIG(II)=ANG(1)/180.*PI
        THEG(II)=ANG(2)/180.*PI
        IARRY(II)=IARRY(I)
C***************

        CLOSE(LUN)
3	CONTINUE
        IHOW=II
        ENDIF
C>>>
C***************************************************************
C GET NSAM AND NROW:
C
C        CALL FILGET(FILPAT1,FILNAM,NCHAR1,IARRY(1),IRTFLG)
C        NDUM=1
C	CALL OPENFB(BUF,FILNAM,LUN,NSAM,NROW,'OLD',IER,NDUM)
C        IF(IER.NE.2) CALL ERRT(4,' GENW',IE)
C>>>
C        IF(NSAM.GT.128.OR.NROW.GT.128) THEN 
C        WRITE(NOUT,123)
C        CALL ERRT(6,'GENW',IE)
C        ENDIF
C        CLOSE (LUN)
C********************************************************************
C     THE ANGULAR DATA OF THE GENERATING PROJECTIONS ARE KNOWN
C     NOW. 
C--------------------------------------------------------------------
C
C     NOW ALL ANGLES ARE KNOWN. PHIG,THEG FOR THE GENERATING PROJECTIONS
C
C     FIRST CALCULATE THE WEIGHTING FUNCTIONS (REAL)
C
C     BEGIN TO CALCULATE ALL MATRIX COEFFICIENTS NEEDED.
C     (LAST ROW TO CALCULATE THE Z' COORDINATE OF PROJECTION PG IN THE
C     3D FOURIER TRANSFORM
C
        DO 6 J=1,IHOW
        THD=DBLE(THEG(J))
        PHD=DBLE(PHIG(J))
        ST=DSIN(THD)
        FX(J)=DCOS(PHD)*ST
        FY(J)=DSIN(PHD)*ST
        FZ(J)=DCOS(THD)
6       CONTINUE


C PRINT OUT THE GIVEN ANGLES:

        WRITE(NOUT,299) 
299     FORMAT(1H ,'ANGLES USED TO GENERATE THE WEIGHTING FUNCTION')
        WRITE(NOUT,298) (K,IARRY(K),
     $  THEG(K)/PI*180.,
     $  PHIG(K)/PI*180.,K=1,IHOW) 
298     FORMAT(1H ,'PROJ.# ',I5,'IMAGE # ',I5,
     $  ' THETA:',F10.4,
     $  ' PHI:',F10.4)
C GET 3D INPUT ARRAY:
         CALL FILERD(FILNAM,NLET,NULL,'INPUT',IRTFLG)
         IF(FILNAM(1:1).EQ.'*') THEN
C           NO INPUT GIVEN, CALCULATE JUST THE WEIGHTING FUNCTION.
            INOPEN=.FALSE.
           ELSE
           INOPEN=.TRUE.
           CALL OPENFB(BUF,FILNAM,LUN,NSAM,NROW,'OLD',IER,NSLICE)
             IF(IFORM.NE.-9) THEN
             WRITE(NOUT,2003)
2003         FORMAT(1X,
     $ 'WRONG FILE FORMAT. NEEDS TO BE SIMPLE 3D FOURIER FORMAT')
             CLOSE (LUN)
             CALL ERRT(10,'GENW3D',NE)
             RETURN
           ENDIF
         ENDIF
C GET OUTPUT FILE FOR WEIGHTED RECONSTRUCTION:
         IF(INOPEN) THEN 
         IFORM=-9
         CALL FILERD(FILNAM1,NLET,NULL,'OUTPUT',IRTFLG)
         CALL OPENFB(BUF,FILNAM1,LUN1,NSAM,NROW,'UNKNOWN',IER,NSLICE)
         ENDIF
C GET OUTPUT FILE FOR WEIGHTING FUNCTION:
         CALL FILERD(FILNAM2,NLET,NULL,'WEIGHT',IRTFLG)
         IF(FILNAM2(1:1).EQ.'*') THEN
         WOPEN=.FALSE.
         ELSE
         WOPEN=.TRUE.
          IF(.NOT.INOPEN) THEN
           CALL RDPRMI(NSAM,NROW,NLOOP,ILOOP,'NSAM,NROW')
           CALL RDPRMI(NSLICE,NDUM,NLOOP,ILOOP,'NSLICE')
          ENDIF
           IFORM=-9
           CALL OPENFB(BUF,FILNAM2,LUN2,NSAM,NROW,'UNKNOWN',IER,NSLICE)
         ENDIF
         NS2=NSAM/2
         NR2=NROW/2
         NL2=NSLICE/2
         UNIT=DBLE(NSAM-2)
         GSIG=2*RAY*RAY
C
C       NOW GO OVER THE VOLUME
C
	DO  1001  K=1,NSLICE

C THIS WRITE STATEMENT WRITES TO THE .LOG FILE OF A BATCH JOB.
C DONE TO BE ABLE TO CHECK PROGRESS. 
        WRITE(6,88) K
88      FORMAT(1X,'PROCESSING SLICE # ',I4)

	IZ=K-NL2-1
	DO  1001  J=1,NROW
	IY=J-NR2-1
	NR=(K-1)*NROW+J
        DO 11 II=1,NSAM
           W(II)=0.
11      CONTINUE
C LOOP OVER NSAM:
        DO 1002 II=1,NSAM,2
          IX=II/2
          WADD=0.
          DO 100 LL=1,IHOW
          Z=FX(LL)*FLOAT(IX)+FY(LL)*FLOAT(IY)+FZ(LL)*FLOAT(IZ)
          FUNCT=1
          DIST=Z/UNIT
          dxa=pi*DIST
          darg=ray*dxa
          if(dxa.ne.0.0.AND.SFLAG) funct=sin(darg)/DARG
          IF(DIST.NE.0.0.AND.GFLAG) FUNCT=DEXP(-DIST*DIST*GSIG)
          WADD=WADD+FUNCT
100       CONTINUE
C COMMENTED OUT FOR DEBUG:
       IF(INVFLAG) THEN
          IF(WADD.GE.CRIT) THEN
          WADD=1/WADD
          ELSE
          WADD=CRIT
          ENDIF
       ENDIF
c for debug use:
c          W(I)=z
c---------------
          W(II)=WADD
1002       CONTINUE 
        IF(INOPEN) THEN
        CALL  REDLIN(LUN,RVOL,NSAM,NR)
        DO 222 I=1,NSAM,2
        RVOL(I)=RVOL(I)*W(I)
222     RVOL(I+1)=RVOL(I+1)*W(I)
        CALL WRTLIN(LUN1,RVOL,NSAM,NR)
        ENDIF
        IF(WOPEN) CALL WRTLIN(LUN2,W,NSAM,NR)
C        WRITE(NOUT,2002) NR
2002    FORMAT(1X,'RECORD WRITTEN:',I5)
1001    CONTINUE
        CLOSE(LUN)
        CLOSE(LUN1)
        CLOSE(LUN2)
        RETURN
        END
