
C++************************************************************************
C
C GETLAB.FOR -- CREATED NOV 87  BY ArDean Leith
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C  GETLAB(LUN,NSAM,BUF,IBUF1,NBUF,VALUE,IRTFLG)
C
C  PURPOSE:    THIS SUBROUTINE RETRIEVES VARIABLES FROM THE HEADER 
C              BY BUFFER POSITION NUMBER
C
C  PARAMETERS:
C             LUN          LOGICAL UNIT NUMBER OF FILE 
C             NSAM         NUMBER OF SAMPLES IN FILE
C             BUF          BUFFER FOR USE IN READING FILE
C             IBUF1        FIRST BUFFER POSITION TO BE RETRIEVED
C             NBUF         NUMBER OF BUFFER POSITIONS TO BE RETRIEVED
C             VALUES       ARRAY FOR BUFFER VALUES RETRIEVED
C             IRTFLG       ERROR FLAG
C 
C       NOTE:   THE HEADER RECORD(S) OF THE FILE CONTAINS THE FOLLOWING 
C               BUFFER POSITIONS WHICH CAN BE RETRIEVED (AMONG OTHERS)
C                   7  FMAXD = IMAGE MAXIMUM
C                   8  FMIND = IMAGE MINIMUM
C                   9  AVD   = IMAGE AVERAGE
C                  10  SIG   = STANDARD DEVIATION (SQ. ROOT OF VARIANCE)
C                  14  IANGLE= FLAG INDIACATING THAT THERE ARE TILT
C                              ANGLES ARE STORED
C                  15  PHI   = TILT ANGLE
C                  16  THETA = TILT ANGLE
C                  17  GAMMA = TILT ANGLE
C                  18  XOFF  = X OFFSET
C                  19  YOFF  = Y OFFSET
C                  20  ZOFF  = Z OFFSET
C                  21  SCALE = SCALE
C
C--*******************************************************************
  
        SUBROUTINE GETLAB(LUN,NSAM,BUF,IBUF1,NBUF,VALUES,IRTFLG)

        DIMENSION VALUES(*),BUF(1)   
c       BUF MUST BE >MAX(256,NSAM)

        COMMON /UNITS/  NDOC,NIN,NOUT

        ISTOP = IBUF1+NBUF-1
        IF (ISTOP .GT. 256) THEN
           WRITE(NOUT,*) 
     &           '*** MAXIMUM HEADER LOCATION MUST BE < 257: ',ISTOP
           CALL ERRT(100,'GETLAB',NE)
           IRTFLG = 1
           RETURN
        ENDIF

C       READ THE HEADER INTO BUF
        CALL REDHED(LUN,BUF,17000,NSAM,0,IRTFLG)
                 
        J = 0
        DO  I = IBUF1,ISTOP
           J         = J + 1
           VALUES(J) = BUF(I)
        ENDDO
        IRTFLG = 0

        RETURN
        END

