
C-************************************************************************
C
C    USED IN FUZZY CLASSIFICATION
C
C    CALLED BY:  GKTEST
C
C-************************************************************************
C
C     X : PATTERN MATRIX ( N PATTERNS, NF FEATURES )
C     U : MEMBERSHIP MATRIX (C , N CLUSTERS )
C     V : CLUSTER CENTRE MATRIX ( C * NF )
C     R : VOLUME CLUSTER MATRIX
C     SF,ISF   : WORKING AREA
C     MN,MP,MC : DIMENSIONS IN THE MAIN PROGRAM
C     MAXIT : MAXIMUM NUMBER OF ITERATIONS ALLOWED
C     EPSIL : ACCURACY PARAMETER (INPUT). IF EPSIL = 0. SOME
C             MATRIX IS SINGULAR (OUTPUT)  
C     ITER  : NUMBER OF ITERATIONS (OUTPUT)
C
C-************************************************************************

      SUBROUTINE GKSM (R,N,NF,C,AM,MAXIT,EPSIL,ITER) 
C
      PARAMETER (MN=999,MP=10,MC=10)
C
      INTEGER C,FLAG
      REAL X(MN,MP),U(MC,MN),V(MC,MP),R(MC),SF(MP,MP),ISF(MP,MP),
     +     RDUM(80)
      COMMON RDUM,X,U,V
C
      NN=2*MP
      ENF = 2./FLOAT(NF)
      EAM = 1./(AM-1.)
C     
C     INITIAL MEMBERSHIP MATRIX
C 
      L = MOD(ITER,C)+1
      DO  I = 1,N
         DO  J=1,C
          U(J,I)=0.
	 ENDDO
         U(L,I)=1.
         L=L+1
         IF (L.GT.C) L=1
      ENDDO
C
C     MAIN LOOP
C 
      DO  ITER=1,MAXIT
C
C     CLUSTER CENTRES ( UPDATE ) AND CHECK CONVERGENCE
C
         FLAG=0   
         DO  J=1,C
            SUM=0.
            DO  I=1,N
             SUM=SUM+U(J,I)
	    ENDDO
            DO  L=1,NF
               SCEN=0.
               DO  I=1,N
                SCEN=SCEN+U(J,I)*X(I,L)
	       ENDDO
               VTEM=V(J,L)
               V(J,L)=SCEN/SUM
               IF (ABS(VTEM-V(J,L)).GT.EPSIL) FLAG=1
	    ENDDO
	 ENDDO
C
         IF (FLAG.EQ.0) RETURN
C
C        MEMBERSHIP VALUES (UPDATE)
C
         DO  J=1,C
C
C        SCATTER FUZZY MATRIX
C
            DO  L=1,NF
            DO  K=1,L
               SF(L,K)=0.
               DO  I=1,N
                  SF(L,K)=SF(L,K)+(X(I,L)-V(J,L))
     &                   *(X(I,K)-V(J,K))*U(J,I)
	       ENDDO
	    ENDDO
	    ENDDO
	    
C
C           DETERMINANT AND INVERSE OF SF MATRIX
C
            CALL INVDET (SF,ISF,MP,NF,DET)
C       
C           SINGULAR MATRIX ?
C
            IF (DET.EQ.0.) THEN
               EPSIL=0.
               RETURN
            ENDIF
C
C           INDUCED NORM MATRIX
C
            DET=DET**ENF
            DO  L=1,NF
            DO  K=L,NF
               ISF(L,K)=R(J)*DET*ISF(L,K)
               ISF(K,L)=ISF(L,K) 
	    ENDDO
	    ENDDO
C
C           MAHALANOBIS DISTANCES
C
            DO  I=1,N
               U(J,I)=0.
               DO  L=1,NF
                  SUM=0.
                  DO  K=1,NF
                   SUM=SUM+(X(I,K)-V(J,K))*ISF(K,L)
		  ENDDO
               U(J,I)=U(J,I)+SUM*(X(I,L)-V(J,L))
	       ENDDO
               IF (U(J,I).LE.0.) U(J,I)=1.0E-05
            U(J,I)=1./(U(J,I)**EAM)
	    ENDDO
C
	 ENDDO
C
C        UPDATE MEMBERSHIP MATRIX 
C
C
         DO  I=1,N
            SUMDIS=0.
            DO  J=1,C
              SUMDIS=SUMDIS+U(J,I)
	    ENDDO
            DO  J=1,C
               U(J,I)=(U(J,I)/SUMDIS)**AM
	    ENDDO
	 ENDDO
      ENDDO
      ITER=ITER-1
C
C        THE CALCULATION OF THE U IS VERY OPTIMAL, BUT AT THIS POINT
C            THE U'S ARE NOT EXACTLY THE PERTENECE FUNCTIONS
C
      DO  J=1,N
         DO  I=1,C
            U(I,J)=U(I,J)**(1./AM)
	 ENDDO
      ENDDO
C

      RETURN
      END


