
C++***************************************************** 1/17/81  VAX
C
C GRAPH.F
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C   PURPOSE:  TO DISPLAY NSET ONE-DIMENSIONAL FUNCTIONS STORED IN AN 
C             ARRAY F(NPT) ON THE LINE PRINTER.
C
C   GRAPH(F,NPT,NSET)
C         F      FLOATING POINT ARRAY OF DIMENSION NSET*NPT CONTAINING
C                NSET FUNCTIONS
C         NPT    DIMENSION OF A FUNCTION
C         NSET   NUMBER OF FUNCTIONS TO BE DISPLAYED ON A SINGLE
C                GRAPH < OR = 10
C
C    DESCRIPTION:  THE RANGE BETWEEN MAXIMUM AND MINIMUM OF THE WHOLE
C                 ARRAY F IS MAPPED ONTO ONE PRINTER PAGE, WHILE THE
C                 NPT SAMPLING POINTS ARE SPREAD ACROSS THE FULL WIDTH 
C                 OF THE PAGE.  EACH OF THE NSET FUNCTIONS IS PRINTED
C                 WITH A DIFFERENT SYMBOL, STARTING WITH '0','1',ETC.
C
C    NOTE:        IPTRTT APPEARS TO BE FLAG FOR OUTPUT DEVICE
C                 < 10 APPEARS TO BE FOR PRINTER
C                 THE VALUE THEN IS A SCALING FACTOR??
C                 OTHER VALUES APPEAR TO BE FOR THE TERMINAL  
C
C--*******************************************************************

      SUBROUTINE GRAPH(F,NPT,NSET)

      COMMON /UNITS/LUN,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
      COMMON /IPRTT/IPRTT

      DIMENSION   F(1)
      INTEGER     ACROSS
	CHARACTER*1  ICHAR(10),IC1,IC2,IC3,ICH,IPLOT(128)

      DATA ICHAR/'0','1','2','3','4','5','6','7','8','9'/
      DATA IC1,IC2,IC3/' ','-','I'/
      DATA NPRI/128/,NTERM/70/
      DATA PRISC/50./,TERMSC/18./

	SCAL     = TERMSC
	FACT     = FLOAT(IPRTT) / 10.0
	ACROSS   = NTERM
	IF (IPRTT .LT. 10) THEN
           FACT   = FLOAT(IPRTT)
           SCAL   = PRISC
           ACROSS = NPRI
           WRITE(NDAT,*) ' '
        ENDIF
100	T     = F(3)
        B     = T
	ISCAL = SCAL + 0.5
        KK    = NSET * NPT

C       FIND MIN/MAX OF F(1...NSET*NPT)
        DO   I = 1,KK
          IF (F(I) .GT. T) T = F(I)
          IF (F(I) .LT. B) B = F(I)
        ENDDO

      IF (T .EQ. B) THEN
         WRITE(NOUT,51)
51       FORMAT(' *** ZERO Y RANGE. GRAPH ABANDONED')
         RETURN
      ENDIF

C     SCALE Y VALUES 
      S = FACT * SCAL / (T - B)
      DO  I = 1,KK
         F(I) = S * (F(I) - B)
      ENDDO

      DX   = FLOAT(ACROSS-1) / FLOAT(NPT-1)
      IY   = ISCAL

C     GO DOWN THE Y LINES ONE BY ONE   
4     ICH  = IC1
      IF (IY .EQ. ISCAL .OR. IY .EQ. 0) ICH = IC2
      DO   I = 1,ACROSS
         IPLOT(I) = ICH
      ENDDO
      IPLOT(1)      = IC3
      IPLOT(ACROSS) = IC3
      X = 1.0
      DO I = 1,NPT
         K = I
         DO  J = 1,NSET
            IF(IFIX(F(K)+.5).EQ.IY) IPLOT(IFIX(X+.5)) = ICHAR(J)
            K = K + NPT
         ENDDO
         X = X + DX
      ENDDO

C     OUTPUT THE LINE
      WRITE(NDAT,8) (IPLOT(K),K=1,ACROSS)
8     FORMAT('   ',128A1)
      IF (IY .NE. 0) THEN
C        NEXT LINE
         IY = IY - 1
         GOTO 4
      ENDIF


C     ALL LINES FINISHED

C     RESTORE F() VALUES
      S = 1.0 / S
      DO  I = 1,KK
         F(I) = F(I)*S + B
      ENDDO

      T1   = B + (T - B) / FACT
      WRITE(NDAT,13) B,T,B,T1
13    FORMAT(/' Y RANGE ',G12.3,' TO ',G12.3,' (ACTUAL ',G12.4,
     &     ' TO ',G12.3,')')

      RETURN
      END
