
C++*******************************************************************
C
C GRAPHO.F 
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C    GRAPHO(F,NPT,NSET,SCALE)
C
C    PURPOSE:  TO DISPLAY NSET ONE-DIMENSIONAL FUNCTIONS STORED IN AN 
C              ARRAY F(NPT) ON THE LINE PRINTER.
C
C         F      FLOATING POINT ARRAY OF DIMENSION NSET*NPT CONTAINING
C                NSET FUNCTIONS
C         NPT    DIMENSION OF A FUNCTION
C         NSET   NUMBER OF FUNCTIONS TO BE DISPLAYED ON A SINGLE
C                GRAPH < OR = 10
C	 SCALE	 SCALE FACTOR TO BE USED. IF SCALE>0. THIS VALUE
C		 IS BEING USED TO OVERRIDE THE AUTOMATIC SCALING.
C
C    DESCIPTION:  THE RANGE BETWEEN MAXIMUM AND MINIMUM OF THE WHOLE
C                 ARRAY F IS MAPPED ONTO ONE PRINTER PAGE, WHILE THE
C                 NPT SAMPLING POINTS ARE SPREAD ACROSS THE FULL WIDTH 
C                 OF THE PAGE.  EACH OF THE NSET FUNCTIONS IS PRINTED
C                 WITH A DIFFERENT SYMBOL, STARTING WITH '0','1',ETC.
C
C--*******************************************************************

      SUBROUTINE GRAPHO(F,NPT,NSET,SCALE)

      COMMON /UNITS/ LUN,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
      COMMON /CMAX/  T,B

      CHARACTER*1 IPLOT(128),ICHAR,ICH,IC1,IC2,IC3
      DIMENSION F(1),ICHAR(10)
      INTEGER   IO_STAT

      DATA ICHAR/'0','1','2','3','4','5','6','7','8','9'/
      DATA IC1,IC2,IC3/' ','-','I'/

      WRITE(NDAT,999)
999   FORMAT('1')

      T = F(1)
      B = T
      KK = NSET*NPT
      DO I = 1,KK
         IF(F(I) .GT. T) T=F(I)
         IF(F(I) .LT. B) B=F(I)
      ENDDO

      IF (SCALE.NE.0.) GOTO 505
      IF (T.NE.B) GOTO 52
      WRITE(NOUT,51)
51    FORMAT(' ** ZERO RANGE. GRAPH NOT DONE **')
      RETURN

505   S = SCALE
      GOTO 53

52    S = 50./(T-B)
53    DO  I = 1,KK
       F(I) = S*(F(I)-B)
      ENDDO
      T1 = -S*B + .5
      IF (T1.LT.0.) T1 = T1-1.
C     TIX = 10000.
C     IF (T1.LT.TIX) TIX = T1
C     IX = TIX
      IX = 0
      DX = 127./FLOAT(NPT-1)
      IY = 50

4     ICH = IC1
C     IF (IY.EQ.50.OR.IY.EQ.0.IY.EQ.IX) ICH = IC2
      IF (IY.EQ.50.OR.IY.EQ.0)ICH=IC2
      DO I = 1,128
         IPLOT(I) = ICH
      ENDDO

      IPLOT(1) = IC3
      IPLOT(128) = IC3
      X = 1.
      DO  I = 1,NPT
        K = I
        DO  J = 1,NSET
          IF (IFIX(F(K)+.5).EQ.IY) IPLOT(IFIX(X+.5)) = ICHAR(J)
          K = K+NPT
	ENDDO
      X = X+DX
      ENDDO

      WRITE(NDAT,8,IOSTAT=IO_STAT) IPLOT
8     FORMAT('   ',128A1)
      IF (IO_STAT .NE. 0) WRITE(NOUT,100) IO_STAT
100   FORMAT(' IOSTAT=',I8)
      IF (IY .EQ. 0) GOTO 9
      IY = IY -1
      GOTO 4

9     S = 1./S
      DO I = 1,KK
         F(I) = F(I)*S + B
      ENDDO

      WRITE(NDAT,13) B,T
13    FORMAT(/'   RANGE ',G12.4,' TO ',G12.4)

      RETURN
      END
