
C++*************************************************************************
C
C GRAPHS.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C       GRAPHS(F,NPT,NSET)
C
C       THIS SUBROUTINE DISPLAYS NSET ONE-DIMENSIONAL FUNCTIONS STORED
C       IN AN ARRAY F(NPT) ON THE LINE PRINTER
C
C         F       FLOATING POINT ARRAY OF DIMENSION NSET*NPT CONTAINING
C                 NSET FUNCTIONS
C         NPT     DIMENSION OF A FUNCTION
C         NSET    NUMBER OF FUNCTIONS TO BE DISPLAYED ON A SINGLE
C                 GRAPH < OR = 10
C       DESCRIPTION:  THE RANGE BETWEEN MAXIMUM AND MINIMUM OF THE WHOLE
C                     ARRAY F IS MAPPED ONTO ONE PRINTER PAGE, WHILE
C                     THE NPT SAMPLING POINTS ARE SPREAD ACROSS THE FULL
C                     WIDTH OF THE PAGE.  EACH OF THE NSET FUNCTIONS IS
C                     POINTED WITH A DIFFERENT SYMBOL, STARTING
C                     WITH '0', '1', ETC.
C
C--*******************************************************************

      SUBROUTINE GRAPHS(F,NPT,NSET)


      COMMON /UNITS/LUN,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
      COMMON /MASTER/NSAMX,NROWX,IREC,NLABEL,IFORM,IMAMI,
     1              FMAX,FMIN,AV,SIG,IHISTO

      CHARACTER*1 IPLOT(128),ICHAR(10),ICH,IC1,IC2,IC3,POINT,PERIOD,
     1          MINUS,PLUS
      DIMENSION F(1),ISTORE(5)

      DATA ICHAR/'0','1','2','3','4','5','6','7','8','9'/
      DATA IC1,IC2,IC3/' ','-','I'/
      DATA PERIOD,POINT,PLUS,MINUS/'.','"','+','-'/

      T  = F(1)
      B  = T
      KK = NSET*NPT
      DO  I = 1,KK
         IF(F(I).GT.T) T=F(I)
         IF(F(I).LT.B) B=F(I)
      ENDDO
      S = 50./(T-B)
      DO  I = 1,KK
       F(I) = S*(F(I)-B)
      ENDDO 
      T1 = -S*B + .5
      IF (T1.LT.0.) T1 = T1-1.
      TIX = 32000.
      IF (T1.LT.TIX) TIX = T1
      IX = TIX
      DX = 127./FLOAT(NPT-1)
      IY = 50
4     ICH = IC1
      IF (IY.EQ.50.OR.IY.EQ.0.OR.IY.EQ.IX) ICH = IC2
      DO  I = 1,128
       IPLOT(I)   = ICH
      ENDDO
      IPLOT(1)   = IC3
      IPLOT(128) = IC3
      X = 1.
      DO  I = 1,NPT
         K = I
         DO  J = 1,NSET
           IF(IFIX(F(K)+.5).EQ.IY) IPLOT(IFIX(X+.5)) = ICHAR(J)
         K = K+NPT
   	 ENDDO
      X = X+DX
      ENDDO

      WRITE(NDAT,8) IPLOT
8     FORMAT('   ',128A1)
      IF (IY .NE. 0) THEN
        IY = IY -1
        GOTO 4
      ENDIF

9	S=1./S
C      IF MAX=MIN DO NOT SCALE  
	IF(FMAX.LE.FMIN)GO TO 11
C*********SCALES HISTOGRAM************

	XO=-2.
	FMAXQ=FMAX/100.
	FMINQ=FMIN/100.
	DO  IXO=1,4
          IF(FMAXQ-FMINQ.GE.2.)GO TO 200
          XO=XO+1.
          FMAXQ=FMAXQ*10.
          FMINQ=FMINQ*10.
	ENDDO

200	FJ=127./(FMAXQ-FMINQ)
   	IMINH=FMINQ
	IF (FLOAT(IMINH).LT.FMINQ) IMINH=FMINQ+1.
	FMINV=FLOAT(IMINH)
	IMAXH=FMAXQ
	FMAXV=FLOAT(IMAXH)
230	STEP=(FMAXV-FMINV)/4.
	DO  I=1,128
  	IPLOT(I)=IC1
	ENDDO
	DO  I=0,4
          BT=FMINV+FLOAT(I)*STEP
          J=INT((BT-FMINQ)*FJ)+1.5
          IF(J.GT.128)J=128
          IF(J.LT.1)J=1
          IPLOT(J)=POINT
          ISTORE(I+1)=J
	ENDDO

	WRITE(NDAT,260)IPLOT
260	FORMAT('   ',128A1)
	FMAXV=FMAXV/(10.**XO)
	FMINV=FMINV/(10.**XO)+0.00001
	IF(FMINV.LT.0.)FMINV=FMINV-0.00002
	STEP=STEP/(10.**XO)
	DO  IA=1,5
          FVALUE=FMINV+FLOAT(IA-1)*STEP
          FVALUE=FVALUE/100.
          DO 280 IB=1,9
            IC=IB-5
            IF (ISTORE(IA).LE.4)IC=IB-ISTORE(IA)
            IF (ISTORE(IA).GT.124)IC=IC-ISTORE(IA)+124
            IF (IB.EQ.5)GO TO 290
            IF (IB.EQ.1)GO TO 295
            IVALUE=FVALUE
            DO ID=1,10
              IF (IABS(IVALUE).EQ.ID-1)IPLOT(ISTORE(IA)+IC)=ICHAR(ID)
            ENDDO
            FVALUE =FVALUE-FLOAT(IVALUE)
            FVALUE =FVALUE*10.
            GO TO 280

290         IPLOT(ISTORE(IA)+IC)=PERIOD
            GO TO 280

295         IPLOT(ISTORE(IA)+IC)=PLUS
            IF (FVALUE.LT.0)IPLOT(ISTORE(IA)+IC)=MINUS
280       CONTINUE
	ENDDO
	WRITE(NDAT,260)IPLOT

C***************************************
11    DO I=1,KK
         F(I) = F(I)*S + B
      ENDDO

      WRITE(NDAT,13) B,T
13    FORMAT(/'   RANGE ',G12.4,' TO ',G12.4)

      RETURN
      END

