C++*********************************************************************
C
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK        *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND     *
C *    RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201 *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS *
C *    OR USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN        *
C *    APPROVAL OF THE CENTER FOR LABORATORIES AND RESEARCH.           *
C **********************************************************************
C
C	PROGRAM TO COMPUTE A FOURIER MAP FROM GNL'S AS DEFINED BY KLUG,
C	CRICK AND WYCKOFF.
C	AUTHORS MONIQUE TIRION AND KENNETH HOLMES
C	MAX PLANCK INSTITUTE FOR MEDICAL RESEARCH, HEIDELBERG, GERMANY
C	E-MAIL: HOLMES@MPIMF-HEIDELBERG.MPG.DE
C	COPYRIGHT, MAX PLANCK GESELLSCHAFT. NOT TO BE USED FOR COMMERCIAL
C	PURPOSES WITHOUT PRIOR AGREEMENT.
C	INPUT GNL'S IN CAMBRIDGE FORMAT IN RECIPROCAL ANGSTROMS
C       OUTPUT SPIDER 3D IMAGE FORMAT
C
C       ADAPTED TO SPIDER 8/17/94 JF
C
C--*********************************************************************

        SUBROUTINE HF

	INCLUDE 'CMBLOCK.INC'
CNO_SAVE

	INTEGER*4 HEADER
	COMPLEX*8 GNL,SMALLG,GINPT
        COMMON BUF(512),GNL(120,31),SMALLG(0:200,31),GINPT(120,31),
     &         BIGR(120,31),HEADER(256),LS(31),NS(31)

	INTEGER*4    GMAX
	COMPLEX*8    GOB,ANG,THETA,G
	REAL         CELL(6)
	INTEGER*4    NABC(3,3)
	INTEGER      IGROUP
        CHARACTER*81 FILNAM, FILE1
	CHARACTER*1  ANS,NULL
	LOGICAL      LESS

	DATA TWOPI/6.28318/,DELSR/1.0/,C/357.5/,SCALE/0.005/

        NULL=CHAR(0)
        LUN1=20

C       SET IRTFLG TO SAVE FILE EXTENSION
        IRTFLG = 9
        CALL FILERD(FILE1,NLET,NULL,'GNL',IRTFLG)
        IF (IRTFLG .NE. 0) RETURN

        CALL RDPRM(C,NLOOP,ILOOP,'HELICAL REPEAT')

        CALL RDPRM(GINT,NLOOP,ILOOP,'GRID INTERVAL')

        CALL RDPRM2(XI,XE,NLOOP,ILOOP,'MAP RANGE (FROM, TO)')

        YI=XI
        YE=XE
        ZI=XI
        ZE=XE

        CALL RDPRMC(ANS,NCHAR,.TRUE.,'TURN MAP UPSIDE-DOWN? (Y/N)',
     &              NULL,IRTFLG)
        CALL RDPRM(PHIDEL,NLOOP,ILOOP,'PHI-ROTATION$')
        CALL RDPRM(DIS,NLOOP,ILOOP,'Z-TRANSLATION [A]')
	AN = PHIDEL/57.2958
        CALL RDPRM(RADCUT,NLOOP,ILOOP,'RADIAL CUTOFF (0=DEF)')
	LESS = .FALSE.
	IF (RADCUT.GT.0.0) LESS = .TRUE.
	RADCUT = ABS (RADCUT)
        CALL RDPRM(B,NLOOP,ILOOP,'B-FACTOR (0=DEF)')

	CALL README (FILE1,GINPT,NS,LS,GMAX,BIGR)

C       TRANSFORM ORIGIN OF GNL'S
	DO I = 1,GMAX
  	  ALPHA = - NS(I) * AN + TWOPI * LS(I) * DIS/C
	  THETA = CMPLX (0.0,ALPHA)
	  ANG = CEXP(THETA)
	  DO J = 1,120
	    IF(ANS.EQ.'Y'.OR.ANS.EQ.'Y') THEN
	      GOB = CONJG (GINPT(J,I))
	    ELSE
	      GOB = GINPT(J,I)
	    END IF
	    GNL(J,I) = GOB * ANG
	  END DO
	END DO

C       FOURIER-BESSEL TRANSFORM
	WRITE(NOUT,800) GMAX
800     FORMAT(' GMAX ',I12)

	DO I = 1,GMAX
	  DO J = 0,200
	    SMALLG(J,I) = (0.0,0.0)
	  END DO

	  ZSQ = (LS(I)/C) ** 2
	  DO J = 0,200
	    R = DELSR * J
	    DO K = 1,120
  	      IF (GNL(K,I).NE.(0.0,0.0))  THEN
	         SSQ = ZSQ + BIGR(K,I)**2
	         SMALLG (J,I) = SMALLG(J,I)
     1              + BIGR(K,I) * BESSL(NS(I),TWOPI*R* BIGR(K,I)) 
     2              * GNL(K,I) * EXP(-B*SSQ/4.)
	      END IF
	    END DO
	  END DO

	END DO
	WRITE (NOUT,*) 'GNLS CALCULATED'

C       HELICAL FOURIER

	NX=NINT((XE-XI)/GINT)
	NY=NINT((YE-YI)/GINT)
	NZ=NINT((ZE-ZI)/GINT)
        IFORM = 3
        MAXIM = 0
        CALL OPFIL(.TRUE.,FILNAM,LUN1,'U',NX,NY,NZ,
     &             MAXIM,'OUTPUT', .TRUE.,IRTFLG)
        IF (IRTFLG.NE.0) RETURN 

	NX0=NINT(XI/GINT)
	NY0=NINT(YI/GINT)
	NZ0=NINT(ZI/GINT)
	CELL(1)=NX*GINT
	CELL(2)=NY*GINT
	CELL(3)=NZ*GINT
	CELL(4)=90.
	CELL(5)=90.
	CELL(6)=90.

C       WRITE HEADER RECORD
	IGROUP=1
	HEADER(1)=IGROUP
	DO  J=1,6
          HEADER(J+1)=NINT(1000.0*CELL(J))
        END DO

C       SETS AXES IN STANDARD ORIENTATION
	HEADER(8)=1     
	NABC(1,1) =  NX0
	NABC(2,1) =  NX0 + NX - 1
	NABC(3,1) =  NX
	NABC(1,2) =  NY0
	NABC(2,2) =  NY0 + NY - 1
	NABC(3,2) =  NY
	NABC(1,3) =  NZ0
	NABC(2,3) =  NZ0 + NZ - 1
	NABC(3,3) =  NZ
	L=8
	DO  J=1,3
	   DO  K=1,3
	      L=L+1
              HEADER(L)=NABC(K,J)
	   ENDDO
	ENDDO
C	WRITE(20,REC=1)HEADER

	RHOMAX=0.0
	NP=0
	NPT=NX*NY*NZ
	DO K = 1,NZ
	  Z = (NZ0 + K - 1)*GINT
	  DO J = 1,NY
	    Y = (NY0 + J - 1)*GINT
	    DO I = 1,NX
	      X = (NX0 + I - 1)*GINT
	      R = SQRT(X**2 + Y**2)
	      IR = R/DELSR

C       INTERPOLATE IN GNL LIST
	      FRAC = R/DELSR - IR

C       GET PHI
	      PHI = 0.0
	      IF (X.NE.0.0.OR.Y.NE.0.0) PHI = ATAN2(Y,X)
	      SUM = 0.0

	      DO II = 1,GMAX
	        G = SMALLG(IR,II) + (SMALLG(IR+1,II)
     $                - SMALLG(IR,II))*FRAC
	        FAC = 2.0
	        IF (NS(II).EQ.0.AND.LS(II).EQ.0) FAC = 1.0
	        ARG =  ( NS(II) * PHI -  TWOPI * LS(II) * Z / C )
	        SUM = SUM
     $                  + SCALE * FAC * (REAL(G) * COS(ARG) 
     $                  - AIMAG(G) * SIN(ARG))
	      END DO 
	      IF(SUM.GT.RHOMAX) RHOMAX=SUM

	      IF (RADCUT.NE.0.0) THEN
	        IF (LESS) THEN
	          IF (R.LT.RADCUT) SUM = 0.0
	        ELSE
	          IF (R.GT.RADCUT) SUM = 0.0
	        END IF
	      END IF

C       RECORD IN SPIDER FILE...
C	IB = NP/LGT2
C	IP = NP - IB*LGT2 + 1
C	NP = NP + 1
C	LINE(IP) = SUM
C	IF((IP.EQ.LGT2).OR.(NP.EQ.NPT))

              BUF(I) = SUM
C        RECORD(I)=SUM
	    END DO 
C	END OF DO  I=  LOOP

C        WRITE (*,*) J,K,RECORD
            CALL WRTLIN(LUN1,BUF,NX,(K-1)*NY+J)
	  END DO  
C	  END OF DO J= LOOP

C       WRITE (8,REC=K) RHO
	END DO 
C	END OF DO K= LOOP

	WRITE(NOUT,110) RHOMAX
110     FORMAT(' RHOMAX = :',F10.0)

	CLOSE(LUN1)
        RETURN
	END

C++******************************************************
C
C	SUBROUTINE README.FOR
C
C--******************************************************

	SUBROUTINE README (FILE1,GNL,N,L,GMAX,BIGR)

C       TO READ CAMBRIDGE FORMAT DATA

	INCLUDE 'CMBLOCK.INC'

	INTEGER*4 GMAX
	COMPLEX*8 GNL(120,31),ARG
	INTEGER*4 N(1),L(1)
	CHARACTER TITLE*30,FILE1*60
	REAL*4    BIGR(120,31)

	OPEN (10,FILE=FILE1,STATUS='UNKNOWN',ACCESS='SEQUENTIAL')
	DO I = 1,90
	  READ (10,100,END=99) TITLE
100       FORMAT (A)
	  READ(10,*) SC, N(I), L(I)
	  WRITE (NOUT,*) I,TITLE,SC,N(I),L(I)

	  DO J = 1,120
	    GNL(J,I) = (0.0,0.0)
	    BIGR(J,I) = 0.0
	  END DO
	  DO J = 1,120
            READ (10,*) BIG,AMOD,PHASE
 	    IF (BIG+AMOD+PHASE.EQ.0.0) GO TO 1
	    ARG = CMPLX(0.0,(PHASE/57.2958))
	    GNL(J,I) = SC * AMOD * CEXP(ARG)
	    BIGR(J,I) = BIG
            WRITE (NOUT,*) BIG,AMOD,PHASE

C           CHANGES NM-1 TO A-1
            BIGR(J,I) = BIG * 0.1 
	  END DO
1         CONTINUE
	END DO

99      GMAX = I - 1

	RETURN
	END

C++*********************************************************************
C
C	FUNCTION BESSL (NN,X0)
C
C--*********************************************************************

       FUNCTION BESSL (NN,X0)

       INCLUDE 'CMBLOCK.INC'

       REAL*8    Y(50)
       INTEGER*4 NN,M
       REAL*4    X0,BESSL,BSL0,BSL1

       IF (NN.EQ.0) THEN
	  BESSL = BSL0(X0)
	  RETURN
       END IF
       IF (NN.EQ.1) THEN
	  BESSL = BSL1(X0)
	  RETURN
       END IF
       IF (NN.EQ.-1) THEN
	  BESSL = - BSL1(X0)
	  RETURN
       END IF
       M = ABS(NN) + 1
       IF (M.GT.50) THEN
	 WRITE (NOUT,*) '*** ORDER OF BESSEL FUNCTION TOO HIGH'
	 STOP
       END IF

       SIGN = 1.0
       IF (MOD(ABS(NN),2).NE.0.AND.NN.LT.0) SIGN = -1.0
       DO  I = 1,M
         Y(I) = 0.0
       END DO
       IF (X0.LT.0.0) THEN
          WRITE (NOUT,*) '*** NEGATIVE ARGUMENT PRESENTED TO BESSL'
          STOP
       END IF

       BESSL = 0.0
       IF (X0.LT.0.005) RETURN
       IF (M.GT.2.AND.X0.LE.0.7*(M-3.9)) RETURN
       Y(1) = BSL0(X0)
       Y(2) = BSL1(X0)
       DO  I = 1,M - 2
          Y(I+2) = (2.0/X0)*FLOAT(I)*Y(I+1)-Y(I)
       END DO
       BESSL = Y(M) * SIGN

       RETURN
       END

C++*********************************************************************
C
C	FUNCTION BSL0(X0)
C0123456789012345678901234567890123456789012345678901234567890123456789
C--*********************************************************************

       FUNCTION  BSL0(X0)

       REAL*4 X0,BSL0

       IF ((X0-4.0) .LT. 0) THEN
         X2 = (X0/4.0)**2
         BSL0 = ((((((-0.00050144*X2+0.00767719)*X2-0.07092536)*X2+
     $        0.44435843)*X2-1.7777561)*X2+3.9999973)*
     $        X2-3.9999999)*X2+1.0
       ELSE
         X1 = 4.0/X0
         X2 = X1**2
         P = ((((-0.00000370*X2+0.0001736)*X2-.48761E-04)
     $       *X2+0.00017343)*X2-0.00175306)*X2+0.39894228     
         Q = X1*(((((0.00000323*X2-0.00001421)*X2+0.00003425)*X2-
     $            0.00008698)*X2+0.00045643)*X2-0.01246694)

         BSL0 = SQRT(X1)*(P*COS(X0-.7853981)-Q*SIN(X0-.7853981))
       END IF

       RETURN
       END

  
C++*********************************************************************
C
C	FUNCTION BSL1(X0)
C
C--*********************************************************************

      FUNCTION BSL1(X0)

      REAL*4 X0,BSL1

      IF((X0-4.0) .LT. 0) THEN

         X1=X0/4.0
         X2 = X1**2
         BSL1 = X1*(((((((-0.0001290 *X2+0.0022069 )*X2-0.02366168)*X2+
     $    0.1777583) *X2-0.8888840 )*X2+2.6666661)*X2-4.0)*X2+2.0)
      ELSE
        X1=4.0/X0
        X2 = X1**2
        P = ((((0.0000042*X2-0.0000201)*X2+0.0000581)
     $      *X2-0.0002232)*X2+0.0029218)*X2+0.3989423
       
        Q = (X1)*(((((-0.0000037 *X2+0.0000162 )
     $      *X2-.398708E-04)*X2+0.0001065 )
     $      *X2-0.0006390 )*X2+0.0374008 ) 
        BSL1 = SQRT(X1)*(P*SIN(X0-.7853981)+Q*COS(X0-.7853981))
      END IF

      RETURN
      END
