
C++*************************************************************************
C
C HISMAP4  -- CREATED FEB 88
C             ADAPTED FROM HISMAP.FOR  ON FEB 25 1988 BY al
C
C **************************************************************************
C *  AUTHOR: A. LEITH                                                      *
C *  COPYRIGHT (C)1988 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C    HISMAP4(IDIM,NPTS,JX,JY,X,Y,ID,MOD,PEX,NOR,LENID,EXTEN)
C
C    PURPOSE:  PREPARES CONTINUOUS SCALE (NOT PRINTER LINE) PLOT METAFILE 
C
C    PARAMETERS:  IDIM     DIMENSION FOR ARRAYS
C                 NPTS     NO. OF POINTS ON MAP
C                 JX,JY    
C                 X,Y      COORDINATES OF POINTS
C                 ID       ID OF POINTS
C                 MOD      MODE FOR SYMBOLS OR LABELS
C                 PEX      STANDARD DEVIATIONS
C                 NOR
C                 LENID    LENGTH OF ID LABELS
C                 EXTEN    SPIDER DATA EXTENSION (IN CHAR. FORM)
C
C    NOTE:
C      COORDINATES X(*) FOR HORIZONTAL AXIS JX, Y(*) FOR VERTICAL AXIS JY 
C      LABELS ARE IN ID(*), FORMAT A1 IF MOD=1, FORMAT A4 IF MOD=4  
C      POINTS AT MORE THAN PEX STANDARD DEVIATIONS ARE POSITIONED ON THE
C      EDGES OF THE GRAPH (SUBROUTINE EPUR4).      
C      WARNING:  X(*), Y(*), ID(NPTS+1) ARE DESTROYED UPON RETURN
C      GRAPH IS ABORTED IF MORE THAN 264 POINTS ARE ON THE EDGES
C
C   CALLS:      EPUR4       BORNS   (IN HPLAN.FOR)
C               CNFILE      CSPUSH   CSPSTX     CSTRAP    CSSCALT
C
C   CALLED BY:  SGRAF  
C
C****************************************************************************

      SUBROUTINE HISMAP4(IDIM,NPTS,X,Y,ID,MOD,PEX,EXTEN)
C      SUBROUTINE HISMAP4(IDIM,NPTS,JX,JY,X,Y,ID,MOD,PEX,NOR,LENID,EXTEN)

      INCLUDE 'CONSTG.INC'
      COMMON /UNITS/ LUNDOC,NIN,NOUT
      COMMON /POLY/  HOLES,REVERS

      PARAMETER      (NSIZE=2000)
      COMMON /COMMUN/ DATA(3,NSIZE)
      PARAMETER      (NHID=200)
C ID(IDIM,LENID)   LENID=1, NOR=1		12/7/93		<ML
      DIMENSION      X(IDIM), Y(IDIM),KLIC(371)
      CHARACTER * 4  ID(IDIM)      
      LOGICAL        REVERS
      CHARACTER *(*) EXTEN
      CHARACTER * 81 CSFILE
      CHARACTER * 7  DISP
      CHARACTER * 4  CDATA,CID
      CHARACTER * 2  AXTYPE
      CHARACTER * 1  NULL,CCHAR
      EQUIVALENCE    (CDATA,FDATA)

      NULL=CHAR(0)

C     CHECK TO SEE THAT NUMBER OF POINTS IS NOT EXCESSIVE
      IF (NPTS .GT. IDIM) THEN
         CALL ERRT(101,' *** NPTS EXCEED ARRAY DIM. IN HISMAP4',IDUM)
         RETURN
      ENDIF         

C     DO NOT PRINT FILE SPECIFICS
      ICSPR  = -2

C     DO NOT ASK FOR FILE TITLE
      ITFLG  =  0

C     DON'T CHECK FOR COUNTER-CLOCKWISE CONTOURS
      REVERS = .FALSE.
      LABAH  = .TRUE.
      LUNCS  = 10

C     SET MAXIMUM NO. OF CONTOURS TO 4000 (I HOPE THIS IS ENOUGH)
      MAXCNT = 4000

C     SET RECORD LENGTH 
      IRECLN = 2048
      DISP   = 'UNKNOWN'

C     SET FILE TYPE TO 6 TO DENOTE PRESENCE OF TEXT CONTOURS
      IFRMCS = -6

C     GET MAP FILE NAME 
10    CALL CNFILE(CSFILE,NLETC,
     &    'ENTER FILE NAME FOR MAP',DISP,EXTEN,LUNCS,IRTFLG)
      IF (IRTFLG .EQ. -1) RETURN

C     GET TEXT SIZE
      ITSIZA = 24
      ITSIZD = 25
      CALL RDPRIS(ITSIZA,ITSIZD,NLOOP,ILOOP,
     &   'TEXT SIZE FOR AXIS AND DATA (USE <CR> FOR DEFAULT = 24)',
     &   IRTFLG)
      IF (IRTFLG .EQ. -1) THEN
         CLOSE(LUNCS)
         GOTO 10
      ENDIF

C     FIND POINTS ON BOUNDARY OF MAP
      CALL EPUR4(IDIM,NPTS, X,Y,ID,MOD,PEX,KP,KLIC,KODE,NDAT)
      IF (KODE .EQ. 1) THEN
         CALL ERRT(101,
     &     '*** MAP ABORTED, MORE THAN 264 POINTS ON FRAME',IDUM)
        RETURN     
      ENDIF

      IF (KP .NE. 0) WRITE(NDAT,140) KP, PEX                 
  140 FORMAT (6X,I4,' POINTS LOCATED BEYOND',F5.1,                        
     &  'STANDARD DEVIATIONS ARE POSITIONED ON THE EDGES OF THE MAP'/)

   20 CALL BORNS (NPTS,X,XMINT,XMAXT)
      CALL BORNS (NPTS,Y,YMINT,YMAXT)              
                  
C     SET PARAMETERS FOR AXIS
      ITANGL = 0
      ITSIZE = ITSIZA
      JUST   = 0
      INTEN  = 9
      IFON   = 0
      IPEN   = 0
      MARK   = 0
      LINTP  = 0
      ZCOO   = 0.0

C     ADD AXIS TO PLOT
      XORG   = 0.0
      YORG   = 0.0
      XEND   = 120.0
      YEND   = 100.0
      AXTYPE = 'XO'
21    CALL CSAXIS(AXTYPE,XMINT,XMAXT,XORG,YORG,XEND,YEND,XFACTR,
     &            LUNCS,IRTFLG)
      IF (IRTFLG .EQ. -1) GOTO 10
      AXTYPE = 'YO'
      CALL CSAXIS(AXTYPE,YMINT,YMAXT,XORG,YORG,XEND,YEND,YFACTR,
     &            LUNCS,IRTFLG)
      IF (IRTFLG .EQ. -1) GOTO 21
 
C     X = 0 ORIGIN LINE
      DATA(1,1) = XORG
      DATA(2,1) = (0.0 - YMINT) * YFACTR
      DATA(1,2) = XEND
      DATA(2,2) = DATA(2,1)
      LINTP     = 1
      NDATA     = 2
      CALL CSPUSH(LUNCS,DATA,NDATA,NF,0,ICNTR)        

C     Y ORIGIN LINE
      DATA(1,1) = (0.0 - XMINT) * XFACTR
      DATA(2,1) = YORG
      DATA(1,2) = DATA(1,1)
      DATA(2,2) = YEND
      CALL CSPUSH(LUNCS,DATA,NDATA,NF,1,ICNTR)        

C     PUT FILENAME AT BOTTEM
      MARK  = 0
      LINTP = 1
      YPOS  = -12.0
      XPOS  = 0.0
      JUST  = 0
      CALL CSPSTX(CSFILE,NLETC,LUNCS,NF,0,ITEXT)

C     PUT IN DUMMY CHAR AT LOWER LEFT CORNER TO PREVENT CLIPPING
C     OF Y AXIS RIGHT JUSTIFIED LABELS

      XPOS       = -20
      YPOS       = -14.0
      NCHAR      = 1
      CDATA(1:1) = ' '
      CALL CSPSTX(CDATA,NCHAR,LUNCS,NF,1,ITEXT)
      
C     SET PARAMETERS FOR TEXT CONTOURS
      ITANGL = 0
      ITSIZE = ITSIZD
      JUST   = 1
      INTEN  = 9
      IFON   = 0
      IPEN   = 0
      MARK   = -2

      NDATA  = 2
      IPUSH  = 0

      DO  IPT = 1,NPTS 

C       LOCATION OF ID ON MAP
        DATA(1,1) = (X(IPT) - XMINT) * XFACTR
        DATA(2,1) = (Y(IPT) - YMINT) * YFACTR

        CID   = ID(IPT)
        NCHAR = 0

        DO I = 1,4
          CCHAR = CID(I:I)
          IF (CCHAR .NE. ' ' .AND. CCHAR .NE. NULL) THEN
             NCHAR = NCHAR + 1
             CDATA(NCHAR:NCHAR) = CCHAR
          ENDIF
        ENDDO

        IF (NCHAR .LT. 4) THEN
          DO I = NCHAR+1,4
             CDATA(I:I) = NULL
          ENDDO
        ENDIF

C       NUMBER OF CHARACTERS IN ID STRING
        DATA(1,2) = NCHAR
        DATA(2,2) = FDATA
        IF (IPT .EQ. NPTS) IPUSH = 1

        CALL CSPUSH(LUNCS,DATA,NDATA,NF,IPUSH,ICNTR)        
      ENDDO


C     SCALE THE CONTOUR META-FILE, USING ONE PASS (ISERCH = 0)
      ISERCH = 0
      CALL CSSCALT(LUNCS,ISERCH)

C     CLOSE THE CONTOUR META-FILE 
      CALL CSTRAP(LUNCS)
      CLOSE(LUNCS)

      RETURN
      END                                                                     
