
C++************************************************************************
C
C HIST.F
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    HIST(LUN,NSAM,NROW,NSLICE,IDUST,LUN2,UNUSED)
C
C    PURPOSE:    COMPUTE 128 PLACE HISTOGRAM FROM IMAGE RECORDS, STORE 
C                HISTOGRAM IN THE HISTOGRAM RECORDS, DISPLAY HISTOGRAM ON 
C                LINE PRINTER AND, OPTIONALLY, REMOVE DATA THAT ARE OUT OF 
C                RANGE.
C
C    PARAMETERS:  LUN        IO UNIT NUMBER OF IMAGE FILE
C                 LUN2       IO UNIT NUMBER FOR DOCUMENT FILE
C                 NSAM,NROW  DIMENSIONS OF IMAGE
C                 IDUST      'DUST' OPTION      0  ONLY HISTOGRAM
C                                               1  REMOVE OUT OF RANGE DATA
C
C    VARIABLES:   IPRTT     FLAG FOR LINE PRINTER FORMATTED OUTPUT
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

      SUBROUTINE HIST(LUN,NSAM,NROW,NSLICE,IDUST,LUN2,UNUSED)

      INCLUDE 'CMBLOCK.INC'

C     GRAPH & GRAPHS USE IPLOT BUFFER
      COMMON IPLOT(128),BUF(1)

      CHARACTER     NULL,ANS
      DIMENSION     PLIST(4)
      
      NULL = CHAR(0)

      IF (IFORM .LE. 0) THEN
C        CAN NOT DO HISTOGRAM ON FOURIER FILES
         CALL ERRT(2,'HIST',NE)
         RETURN
      ENDIF

C     MAKE SURE STATISTICS ARE CURRENT
      IF (IMAMI .NE. 1) CALL NORM3(LUN,NSAM,NROW,NSLICE,FMAX,FMIN,AV)

C     CALCULATE HISTOGRAM HERE
C     ZERO THE HISTOGRAM
      DO  K = 1,128
         BUF(NSAM+K) = 0.0 
      ENDDO

C      MPI 3-MAR-80 ADDED TO AVOID "BEAT" PHENOMENON   al
C      APPEARS TO EPAND RANGE OF HIST. MIN/MAX FOR < 0...2 
C      HAS NO EFFECT ON CORRECTNESS OF HISTOGRAM   al

C      FOR MIN = 0.0 MAX=2.0 HMIN=0.0    HDIFF1=2.0
C      FOR MIN = 0.0 MAX=1.0 HMIN=0.0    HDIFF1=1.0
C      FOR MIN = 0.0 MAX=1.5 HMIN=-0.25  HDIFF1=2.0   al

25     HDIFF = FMAX - FMIN
       IF (HDIFF .LE. 2.0) THEN
           VAR = 2.0
           DO  L=1,16
             IF (HDIFF .GT. VAR) GOTO 23
             VAR = VAR / 2.0
          ENDDO
          CALL ERRT(5,'HIST  ',NE)
          RETURN

23        HDIFF1 = VAR * 2.0
          DIFF   = HDIFF1 -HDIFF
          HMIN   = FMIN - DIFF / 2.0
       ELSE
          HMIN   = FMIN
          HDIFF1 = HDIFF
       ENDIF

       FF = 127.0 / HDIFF1

C      BUF(NSAM+129) AND BUF(NSAM+130) COMMUNICATE NEW MINIMUM AND 
C      INCREMENT USED IN HISTOGRAM COMPUTATION.

       BUF(NSAM+129) = HMIN
       BUF(NSAM+130) = HDIFF1

C      MPI END

      NUMEL = 0
      I     = 0
      Y     = 0.0

      DO  IZ=1,NSLICE
         DO  IY=1,NROW
            I = I+1
            CALL REDLIN(LUN,BUF,NSAM,I)
            DO  K = 1,NSAM
               B = BUF(K)
               J = INT((B-FMIN)*FF) + 1.5
               IF (J.GE.1 .AND. J.LE.128)  THEN
                  S           = B - AV
                  Y           = Y + S*S
                  BUF(NSAM+J) = BUF(NSAM+J)+1.
                  NUMEL       = NUMEL+1
               ENDIF

C              IF (J .LT. 1) THEN
C                 OFF HISTOGRAM AT BOTTEM
C                 BUF(NSAM+1) = BUF(NSAM+1)+1.
C              ELSEIF (J .GT. 128) THEN
C                 OFF HISTOGRAM AT TOP
C                 BUF(NSAM+128) = BUF(NSAM+128) + 1.
C              ELSE
C                 BUF(NSAM+J)=BUF(NSAM+J)+1.
C              ENDIF

	    ENDDO
	ENDDO
      ENDDO

605   CONTINUE
      IF (FCHAR(4:4) .EQ. 'D' .OR. FCHAR(5:5) .EQ. 'D') THEN
C        OUTPUT TO DOC FILE
         DINC     = 1.0 / FF
         PLIST(2) = FMIN
         DO  I=1,128
            PLIST(1) = I
            PLIST(2) = PLIST(2) + DINC
            PLIST(3) = BUF(NSAM + I)
            CALL SAVD(LUN2,PLIST,3,IRTFLG)
         ENDDO
         CALL SAVDC
         CLOSE(LUN2)

      ELSEIF (FCHAR(4:4) .EQ. 'T' .OR. FCHAR(5:5) .EQ. 'T') THEN
C        OUTPUT ON TERMINAL
         NSAV  = NDAT
         NDAT  = NOUT
         IPRTT = 10
         WRITE(NOUT,*) ' '
         CALL GRAPH(BUF(NSAM+1),128,1)
         WRITE(NOUT,*) ' '
         NDAT  = NSAV

      ELSEIF (FCHAR(4:4) .NE. 'B') THEN
C        OUTPUT IN RESULTS FILE
         CALL GRAPHS(BUF(NSAM+1),128,1)
      ENDIF

C     FIND MAXIMUM FREQUENCY OCCURING IN HISTOGRAM (HISMAX) & LOCATION
      HISMAX = BUF(NSAM+1)
      J = 1
      DO  I = 2,128
         IF (BUF(NSAM+I) .GE. HISMAX) THEN
           HISMAX = BUF(NSAM+I)
           J  = I
         ENDIF
      ENDDO

C     CONVERT LOCATION J TO CORRESPONDING IMAGE INTENSITY (MODE)
      IF (J .EQ. 1 .OR. J .EQ. 128) THEN
         TMODE  = BUF(NSAM+J)
      ELSE
         YM1    = BUF(NSAM+J-1)
         Y1     = BUF(NSAM+J+1)
         TMODE  = FLOAT(J-1)+(YM1-Y1)*.5/ (YM1+Y1-2.*BUF(NSAM+J))
      ENDIF
      TMODE     = FMIN + TMODE / FF

      WRITE(NOUT,*) ' '

      HMAX = HMIN + HDIFF1
      BSIZ = HDIFF1 / 127.0
      WRITE(NOUT,91) FMIN,FMAX,HMIN,HMAX,AV,TMODE,SIG,BSIZ
91    FORMAT(' FILE RANGE:          ',G10.3,'   .........   ',G10.3,/,
     &       ' HISTOGRAM RANGE:     ',G10.3,'   .........   ',G10.3,/,
     &       ' FILE MEAN:           ',G10.3,'   MODE:       ',G10.3,/,
     &       ' S.D.:                ',G10.3,'   BIN SIZE:   ',G10.3)

      IF (NDAT .NE. NOUT) WRITE(NDAT,91) FMIN,FMAX,HMIN,HMAX,AV,
     &                    TMODE,SIG,BSIZ 


C     USUALLY RETURN HERE
      IF (IDUST .EQ. 0) RETURN


C     IDUST ONLY SECTION ---------------------------------------

      CALL RDPRM(FMULT,NLOOP,ILOOP,'STANDARD DEVIATION FACTOR')
      CALL RDPRMI(ISIDE,IDUM,NLOOP,ILOOP,
     &             '(1)BOTTOM (2)TOP (3)BOTH SIDES')
      IF (FCHAR(4:4) .EQ. 'V') 
     &     CALL RDPRM(VALUE,NLOOP,ILOOP,'VALUE TO BE SUBSTITUTED')

      BOT  = FMIN
      TOP  = FMAX
      R    = NUMEL
      SIGT = SQRT(Y/R)
      Y    = SIGT * FMULT
      IF (ISIDE .EQ .1 .OR. ISIDE .EQ. 3) BOT = TMODE - Y
      IF (ISIDE .EQ .2 .OR. ISIDE .EQ. 3) TOP = TMODE + Y

C     FOR 'DU' CASE
      TT  = TOP
      TB  = BOT

C      FOR 'DU V' CASE
      IF (FCHAR(4:4) .EQ. 'V') THEN
         TT = VALUE
         TB = VALUE
      ENDIF

      AV = 0.0
      DO  I=1,NROW*NSLICE
         CALL REDLIN(LUN,BUF,NSAM,I)
         DO  K=1,NSAM
            IF (BUF(K) .GT. TOP) BUF(K)=TT
            IF (BUF(K) .LT. BOT) BUF(K)=TB
            AV = AV+BUF(K)
         ENDDO
         CALL WRTLIN(LUN,BUF,NSAM,I)
      ENDDO


C     SET STATISTICS TO UNDETERMINED
      CALL SETPRMB(BUF,LUN,NSAM,NROW,0.0,0.0,AV,'U')

      RETURN
      END
