
C++*********************************************************************
C
C HISTM.FOR                                       ARDEAN LEITH 1 MAY 96
C
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK        *
C *  COPYRIGHT (C)1981, 1996 HEALTH RESEARCH INCORPORATED, ALBANY,     *
C *             NY 12201                                               *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH         *
C *    RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS             *
C *    OR USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN        *
C *    APPROVAL OF HEALTH RESEARCH INC.                                *
C **********************************************************************
C
C    HISTM(LUN,LUNMA,LUNDOC,NSAM,NROW,NSLICE)
C
C    PURPOSE:    COMPUTE 128 PLACE HISTOGRAM FROM IMAGE RECORDS, STORE 
C                HISTOGRAM IN THE HISTOGRAM RECORDS, DISPLAY HISTOGRAM ON 
C                LINE PRINTER AND, OPTIONALLY, REMOVE DATA THAT ARE OUT OF 
C                RANGE.
C
C    PARAMETERS:  LUN        IO UNIT NUMBER OF IMAGE FILE
C                 LUNMA      IO UNIT NUMBER FOR MASK FILE
C                 LUNDOC     IO UNIT NUMBER FOR DOCUMENT FILE
C                 NSAM,NROW  DIMENSIONS OF IMAGE
C
C    VARIABLES:   IPRTT     FLAG FOR LINE PRINTER FORMATTED OUTPUT
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

      SUBROUTINE HISTM(LUN,LUNMA,LUNDOC,NSAM,NROW,NSLICE)

      INCLUDE 'CMBLOCK.INC'

C     GRAPH & GRAPHS USE IPLOT BUFFER
      COMMON IPLOT(128),SAVE(130),BUF(4096),BUFMA(4096)

      CHARACTER        NULL,ANS
      DIMENSION        PLIST(4)
      DOUBLE PRECISION HAV, HAV2, HSIG

      NULL = CHAR(0)

      IF (NSAM .GT. 4096) THEN
C        INSUFFICENT BUFFER SIZE FOR BUF
         CALL ERRT(3,'HISTM',NE)
         RETURN
      ELSEIF (IFORM .LE. 0) THEN
C        CAN NOT DO HISTOGRAM ON FOURIER FILES
         CALL ERRT(2,'HISTM',NE)
         RETURN
      ENDIF

      IF (IMAMI .NE. 1) CALL NORM3(LUN,NSAM,NROW,NSLICE,FMAX,FMIN,AV)

C     ZERO THE HISTOGRAM
      DO  K = 1,130
         SAVE(K) = 0.0 
      ENDDO

      HMIN = FMAX
      HMAX = FMIN

C     DETERMINE RANGE UNDER MASK
      HAV  = 0.0
      HAV2 = 0.0
      NUMV = 0
      IREC = 0
      DO IZ=1,NSLICE
         DO IY=1,NROW
            IREC = IREC + 1
            CALL REDLIN(LUN,  BUF,  NSAM,IREC)
            CALL REDLIN(LUNMA,BUFMA,NSAM,IREC)

            DO  K = 1,NSAM
               IF (BUFMA(K) .GE. 0.5) THEN
C                 PIXEL HAS POSITIVE MASK VALUE
                  BVAL = BUF(K)
                  HAV  = HAV  + BVAL 
                  HAV2 = HAV2 + BVAL * DBLE(BVAL)
                  NUMV = NUMV + 1
                  IF (BVAL .LT. HMIN) HMIN = BVAL
                  IF (BVAL .GT. HMAX) HMAX = BVAL
               ENDIF
            ENDDO
         ENDDO
      ENDDO

      DTOP  = HAV2 - HAV * HAV / DBLE(NUMV)
      IF (DTOP .LT. 0.0) THEN
C        SQRT OF NEGATIVE NUMBER
         WRITE(NOUT,*) 
     &       '*** CAN NOT DETERMINE S.D., SQRT(',DTOP,') IMPOSSIBLE' 
         CALL ERRT(100,'HISTM',NE)
         GOTO 999

      ELSEIF (NUMV .EQ. 1) THEN
C        DIVISION BY ZERO
         WRITE(NOUT,*) '*** CAN NOT DETERMINE S.D. (ONLY ONE PIXEL) ' 
         CALL ERRT(100,'HISTM',NE)
         RETURN
      ENDIF

      HSIG   = DSQRT(DTOP / DBLE(NUMV - 1))
      HAV    = HAV  / DBLE(NUMV)
      HDIFF  = HMAX - HMIN
      FF     = 127.0 / HDIFF

C     SAVE(129) AND SAVE(130) CONTAIN NEW MINIMUM AND 
C     INCREMENT USED IN HISTOGRAM COMPUTATION.
      SAVE(129) = HMIN
      SAVE(130) = HDIFF

      NUMLOW = 0
      NUMHI  = 0
      NUMEL  = 0
      IREC   = 0
      DO IZ=1,NSLICE
         DO IY=1,NROW
            IREC = IREC + 1
            CALL REDLIN(LUN,  BUF,  NSAM,IREC)
            CALL REDLIN(LUNMA,BUFMA,NSAM,IREC)

            DO  K = 1,NSAM
               IF (BUFMA(K) .GE. 0.5) THEN
C                 PIXEL HAS POSITIVE MASK VALUE
                  BVAL = BUF(K)
                  J    = INT((BVAL-HMIN) * FF) + 1.5

                  IF (J.GE.1 .AND. J.LE.128)  THEN
                     SAVE(J) = SAVE(J) + 1.0
                     NUMEL   = NUMEL + 1

                  ELSEIF (J .LT. 1) THEN
C                    OFF HISTOGRAM AT BOTTEM
                     NUMLOW = NUMLOW + 1

                  ELSEIF (J .GT. 128) THEN
C                    OFF HISTOGRAM AT TOP
                     NUMHI = NUMHI + 1
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
      ENDDO

      CALL RDPRMC(ANS,NA,.TRUE.,
     &   'OUTPUT TO RESULTS FILE, DOC. FILE, OR TERMINAL? (R/D/T)',
     &     NULL,IRTFLG)
      IF (IRTFLG .EQ. -1) GOTO 999

      IF (ANS .EQ. 'D') THEN
C        OUTPUT TO DOC FILE
         DINC     = 1.0 / FF
         PLIST(2) = HMIN
         DO  I=1,128
            PLIST(1) = I
            PLIST(2) = PLIST(2) + DINC
            PLIST(3) = SAVE(I)
            CALL SAVD(LUNDOC,PLIST,3,IRTFLG)
         ENDDO
         CALL SAVDC

      ELSEIF (ANS .EQ. 'T') THEN
C        OUTPUT ON TERMINAL
         NSAV  = NDAT
         NDAT  = NOUT
         IPRTT = 10
         WRITE(NOUT,*) ' '
         CALL GRAPH(SAVE(1),128,1)
         WRITE(NOUT,*) ' '
         NDAT  = NSAV

      ELSEIF (ANS .NE. 'B') THEN
C        OUTPUT IN RESULTS FILE
         CALL GRAPHS(SAVE(1),128,1)
      ENDIF

C     FIND MAXIMUM FREQUENCY OCCURING IN HISTOGRAM (HISMAX) & LOCATION
      HISMAX = SAVE(1)
      J = 1
      DO  I = 2,128
         IF (SAVE(I) .GE. HISMAX) THEN
           HISMAX = SAVE(I)
           J  = I
         ENDIF
      ENDDO

C     CONVERT LOCATION J TO CORRESPONDING IMAGE INTENSITY (MODE)
      IF (J .EQ. 1 .OR. J .EQ. 128) THEN
         HMODE  = SAVE(J)
      ELSE
         YM1    = SAVE(J-1)
         Y1     = SAVE(J+1)
         HMODE  = FLOAT(J-1)+(YM1-Y1)*.5/ (YM1+Y1-2.*SAVE(J))
      ENDIF
      HMODE  = HMIN + HMODE / FF

      IF (NUMLOW .GT. 0 .OR. NUMHI .GT. 0) THEN
C        SCALING ERROR
         WRITE(NOUT,*) '*** SCALING ERROR IN HISTOGRAM'
         WRITE(NOUT,90) NUMLOW,NUMHI
90       FORMAT(' PIXELS BELOW MIN: ',I8,' PIXELS ABOVE MAX:',I8)
         IF (NDAT .NE. NOUT) WRITE(NDAT,90) NUMLOW,NUMHI
      ENDIF

      NTOT = NSAM * NROW * NSLICE
      BSIZ = HDIFF / 127.0

      WRITE(NOUT,91) NTOT,NUMEL,FMIN,FMAX,HMIN,HMAX,HAV,HMODE,HSIG,BSIZ
91    FORMAT(' TOTAL PIXELS:        ',I10,  '   INSIDE MASK:',I8   ,/,
     &       ' FILE RANGE:          ',G10.3,'   .........   ',G10.3,/,
     &       ' HISTOGRAM RANGE:     ',G10.3,'   .........   ',G10.3,/,
     &       ' HISTOGRAM MEAN:      ',G10.3,'   MODE:       ',G10.3,/,
     &       ' S.D.:                ',G10.3,'   BIN SIZE:   ',G10.3)

      IF (NDAT .NE. NOUT) WRITE(NDAT,91) NTOT,NUMEL,FMIN,FMAX,
     &                    HMIN,HMAX,HAV,HMODE,HSIG,BSIZ

      WRITE(NOUT,*) ' '

998   CLOSE(LUNDOC)
999   CLOSE(LUN)
      CLOSE(LUNMA)

      RETURN
      END
