
C **************************************************************************
C *  AUTHOR :                                                              *
C *  COPYRIGHT (C)1987 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C   MODIFIED BY MAHIEDDINE LADJADJ   9/1/93
C   LONG FILE NAMES FEB 89 AL
C   CRAY COMPATIBLE FORTRAN JAN 90 JF	
C   JULY 87 (RANDOM SHUFFLING)
C   12/21/87 (NEW IN - CORE) 12/30/87 I4
C   5/03/89 (CO - PROJECTION OF ARBITRARY DATA SET) JF
C
C   PURPOSE:  CREATE A SEQUENTIAL FILE CONTAINING IMAGES 
C             IN NON - TRANSPOSED FORMAT TO BE USED AS INPUT TO JP'S
C	      NEW CORAN PROGRAMS
C      MSK VERSION : USE MASK FILE TO DECIDE WHICH POINTS TO INCLUDE
C
C      JPMSK1(MAXDIMT)
C	  LUN		LOGICAL UNIT NUMBER OF  INPUT FILE
C		 -  -  LATER USED FOR IMAGE COORDINATE FILE
C	  LUNS	LOGICAL UNIT NUMBER OF SEQUENTIAL OUTPUT FILE CONTAINING
C		MASKED IMAGE DATA
C	  LUNM  LOGICAL UNIT NUMBER OF MASK FILE
C		 -  -  LATER USED FOR EIGENVALUE FILE
C	  LUNP  LOGICAL UNIT NUMBER OF PIXEL COORDINATE FILE
C	  MAXDIMT  MAXIMUM BUFFER LENGTH TO BE USED BY PROGRAM
C
C  COMMANDS SUPPORTED :
C
C     CA SI     -  INITIALIZE SEQUENTIAL FILE
C     CA SI +   -  AS ABOVE,  BUT ADD TO EXISTING SEQ. FILE
C                  THIS COMMAND HAS BEEN REMOVED. RUN CA SI ON THE
C		   WHOLE SET OF IMAGES.          12/15/93   ML
C     CA S      -  RUN CORAN USING PREVIOUSLY CREATED SEQUENTIAL FILE
C     CA SF     -  FETCH IMAGES, FROM SEQ. FILE, AS SEEN WITH THE MASK
C
C     MADAR  =  MASK ADDRESS ARRAY  -  -  CONTAINS LEXICOGRAPHIC
C               ADDRESSES OF MASK POINTS IN IMAGE FRAME
C     LUV  =    LOGICAL UTILITY VECTOR  -  -  CONTAINS ACTIVE/INACTIVE
C               SWITCH FOR EACH IMAGE IN SEQUENTIAL ORDER. 
C               LUV(I) = 0 MEANS INACTIVE; LUV(I) = 1 MEANS ACTIVE.
C
C-----------------------------------------------------------------------------

        SUBROUTINE JPMSK1(MAXDIMT)
        
        INCLUDE 'CMBLOCK.INC'
        
c      has to be multiple of 8.
        PARAMETER (MAXIM = 20000)
        
C       MAXIMUM NUMBER OF ELEMENTS IN MASK SET TO 25000
        PARAMETER (MAXMSK = 25000)
        
        COMMON /COMMUN/ FILNM1, FILENM, FILNMP, FILNAM, MSKNAM, 
     &                  FILNMC, FILNME, FILNMO, FILPAT1, 
     &                  FILPATC, FILPATE, FILPATP
        
C       ADUM IS USED BY OPENF!
        COMMON   ADUM(2048), IMAR(MAXIM), IMINAC(MAXIM), 
     &           MADAR(MAXMSK), LUV(MAXIM), BUF(1)
        
        CHARACTER  * 81   FILNM1, FILENM, FILNMP, FILNAM, MSKNAM, 
     &               FILNMC, FILNME, FILNMO, FILPAT1, FILPATC, FILPATE, 
     &               FILPATP
        CHARACTER  * 1    NULL, ANS
        CHARACTER * 3     ANS1
        
        DOUBLE PRECISION  DAV, VAR
        LOGICAL           EX, FLAGR
        INTEGER * 4       NUMIM,  NPIX,  NSAM,  NROW,  NMASK,  NEGTOT
        INTEGER * 4       JBUF, IMASK, IOFF1, L, NTOT, I1
        INTEGER * 4       NTIM(4)
        CHARACTER *16     CNTIM
        EQUIVALENCE       (NTIM,CNTIM)
        LOGICAL           PICK_PCA

        DATA        FMINT/ - 10.0E30/, FMAXT/10.0E30/
        DATA        LUN, LUNS, LUNM, LUNP/71, 72, 73, 74/
C       LSHFT 2 ** 16 (2 BYTES)
        INTEGER*4   LSHFT
        DATA        LSHFT/65536/

CNO_SAVE
       NULL    =  CHAR(0)
       FLAGR   =  .FALSE.
       MAXDIM  = MAXDIMT - 4 * MAXIM - 3 * MAXMSK - 2048

       FILPATP = 'PIX***'//NULL
       FILPATE = 'EIG***'//NULL
       FILPATC = 'IMC***'//NULL
       
C      SEED INTEGERS FOR RANDOM SHUFFLE
       CALL TIME(CNTIM)
       IST = NTIM(4)
       KST = NTIM(3)

C ---------------------------------------------------------
C       OPTION 'CA SI'  --  INITIALIZATION 
C ---------------------------------------------------------
 
        IF (FCHAR(5:5) .EQ. 'I') THEN

C         NONTRANSPOSED  SEQUENTIAL  FILE
C         READ IN IMAGE FILE NUMBERS SELECTED; STORED IN IMAR().

          NUMIMG  =  MAXIM  
          CALL FILSEQP(FILPAT1, NLET1, IMAR, MAXIM, NUMIMG, 
     &       'ENTER IMAGE FILE TEMPLATE', IRTFLG)
          IF (IRTFLG  .NE.  0) RETURN

C         NUMIMG = NUMBER OF IMAGES REQUESTED TO BE USED.
C         THE ACTUAL NUMBER NUMIM MAY BE SMALLER DUE TO GAPS IN 
C         THE FILE SERIES.
        
          CALL FILERD(MSKNAM, NLET, NULL, 'MASK', IRTFLG)
          IF (IRTFLG  .NE.  0) RETURN
        
          CALL FILERD(FILNAM, NLET,  NULL, 'SEQUENCE', IRTFLG)
          IF (IRTFLG  .NE.  0) RETURN
          
          CALL OPENF(MSKNAM, LUNM, NSAM, NROW, 'O', NF, NDUM)
          IF (NF .NE. 2) GOTO 9300
    
C         SAVE MASK DIMENSIONS FOR LATER COMPARISON
          NSAMM = NSAM
          NROWM = NROW
        
          CALL RDPRMC(ANS1, NCHAR, .TRUE., 
     &       'ALL FILES EXIST? (Y/N), (I)NV, (R)ANDOM', NULL, IRTFLG)
  
          IF (INDEX(ANS1, 'R') .GT. 0)  THEN
             FLAGR = .TRUE.
             CALL RDPRM(SH, NLOOP, ILOOP, 'SHUFFLE FACTOR')
             IF (SH .LE. 0.) SH = 1.
          ENDIF
        
          CALL RDPRM(ADA, NLOOP, ILOOP, 'ADDITIVE CONSTANT')
          IF (INDEX(ANS1, 'N') .GT. 0) THEN
            NUMIM = 0
            DO IM = 1, NUMIMG
              CALL FILGET(FILPAT1, FILNM1, NLET1,IMAR(IM), IRTFLG)
              IF (IRTFLG  .NE.  0) GOTO 9000
              CALL FILCAN(FILNMO, NFN, NULL, NULL,FILNM1,DATEXC,IER)
              INQUIRE(FILE = FILNMO, EXIST = EX)
              IF (EX) THEN
                 NUMIM = NUMIM + 1
                 IMAR(NUMIM) = IMAR(IM)
              ENDIF
            END DO         
          ELSE
             NUMIM = NUMIMG
          ENDIF
  
C         BUFFER ALLOCATIONS
          JMASK = NSAM * NROW + 10
          JBUF  = JMASK + JMASK 
  
C         READ IN MASK, DETERMINE NUMBER OF PIXELS, 
C         SET UP MASK ADDRESS ARRAY
          NMASK = 0
          IMASK = JMASK - NSAM

C         THE 2 D ARRAY IMAGE IS SQUEEZED INTO A 1 D ARRAY BUF().
C         MADAR()= (ROW -1) * LSHFT + K FOR EACH BUF(ROW,K) > .5
C	  SHIFT TO THE LEFT BY TWO BYTES.
C   ML.
          DO I = 1, NROW
            IMASK = IMASK + NSAM

C           READ A ROW FROM A ROW OF THE MASK INTO THE ARRAY BUF(). 
            CALL REDLIN(LUNM, BUF(IMASK), NSAM, I)
            DO K = 1, NSAM
              IF (BUF(IMASK + K-1) .GE. 0.5) THEN
                NMASK = NMASK + 1
                IF (NMASK.GT.MAXMSK) GOTO  9800
                MADAR(NMASK) = (I - 1) * LSHFT + K
              END IF
            END DO
          END DO
          CLOSE(LUNM)
          IF (NMASK .EQ. 0) THEN
	     WRITE(NOUT, *) ' *** MASK CONTAINS NO POINTS'
             CALL ERRT(100,'JPMSK1',NE)
             RETURN
          ENDIF

C         OPEN SEQUENTIAL FILE
	  NULL = CHAR(0)
          CALL FILCAN(FILENM, NFN, NULL, NULL, FILNAM, DATEXC, IER)
          OPEN(FILE = FILENM, UNIT = LUNS, STATUS = 'UNKNOWN', 
     &         FORM = 'UNFORMATTED')
                     
C         WRITE HEADER
          WRITE(LUNS) NUMIM, NMASK, NSAM, NROW, 
     1              (MADAR(K), K = 1, NMASK), 
     2              (IMAR(K), K = 1, NUMIM)
          IM1 = 1
          IM2 = NUMIM
  
C         LOOP OVER ALL IMAGES
          BUFNEG = 0.0
          NEGTOT = 0
    
C         APPLY SHUFFLE FACTOR TO COMPUTE TOTAL NUMBER OF SHUFFLES
C         IF 'R'  OPTION USED.
          NSH = SH * FLOAT(NMASK) + 0.5
          DO IM = IM1, IM2
            NEG = 0
            N1 = IMAR(IM)
          
C           RANDOM SHUFFLE OPTION
            IF (IM .NE. IM1 .AND. FLAGR) THEN
               CALL PERMUT(BUF, NMASK, NSH, IST, KST)
            ELSE
              CALL FILGET(FILPAT1, FILNM1, NLET1, N1, IRTFLG)
              IF (IRTFLG  .NE.  0) GOTO  9000
              CALL OPENF(FILNM1, LUN, NSAM, NROW, 'O', NF, NDUM)
              IF (IFORM .NE. 1 .AND. IFORM .NE. 3) GOTO  9100
              IF (NF .NE. 2) GOTO  9300
    
C             COMPARE MASK DIMENSIONS WITH IMAGE DIMENSIONS
              IF ((NSAMM .NE. NSAM) .OR. (NROWM .NE. NROW)) THEN
                WRITE(NOUT, 3194) NSAM, NROW, NSAMM, NROWM
3194            FORMAT('*** IMAGE DIMENSION (',I4,',',I4,
     &                 ') NOT SAME AS MASK DIMENSION (',I4,',',I4,')')
                CLOSE(LUNM)
                CLOSE(LUN)
                CALL ERRT(100,'JPMSK1',NE)
                RETURN
              ENDIF
    
C             READ IMAGE AREA WHERE MASK > 0.5 INTO CORE AND
C	      COMPUTE ITS MASK-RELATED,  PRECISE AVERAGE
              WEIGHT = 0.
              VAR    = 0.
              IF (INDEX(ANS1, 'I') .GT. 0) THEN
                 SIGN =  -1.
              ELSE
                 SIGN = 1.
              ENDIF
              NCOUNT = 0
              IMASK  = JMASK - NSAM
              DO I = 1, NROW
                IMASK = IMASK + NSAM
                CALL REDLIN(LUN, BUF(JBUF), NSAM, I)
                DO  K = 1, NSAM
                  IF (BUF(IMASK + K-1) .GE. 0.5) THEN
                    BUF(JBUF + K-1) = BUF(JBUF + K-1) + ADA
                    WEIGHT = WEIGHT + (BUF(JBUF + K-1) * SIGN)              
                                           
C                   TEST FOR NEGATIVE VALUES
                    IF ( (BUF(JBUF + K-1) * SIGN) .LT. 0.) THEN
                      IF(BUF(JBUF+K-1) .LT. BUFNEG)
     &                  BUFNEG = BUF(JBUF+K-1)
                      NEG = NEG + 1
                    END IF

                    VAR = VAR + BUF(JBUF + K-1)**2
                    NCOUNT = NCOUNT + 1
                    BUF(NCOUNT) = BUF(JBUF + K-1) * SIGN
                                           
                  ENDIF
                END DO
              END DO
              DAV = WEIGHT/FLOAT(NCOUNT)
              VAR = (VAR - DAV**2 * FLOAT(NCOUNT))/FLOAT(NCOUNT - 1)

C	      WEIGHT = SUM OF THE ELEMENTS(.gt. 0.5) IN A ROW.
C             VAR : IT IS NOT USED ANYWHERE IN THIS FILE.
C
              BUF(NCOUNT + 1) = WEIGHT
              BUF(NCOUNT + 2) = VAR
            END IF
  
C           N1 THE SPIDER FILE (IMAGE) NUMBER,  TO BE USED AS
C           IMAGE ID THROUGHOUT THE CORAN ANALYSIS
C           NOTE THAT FOR THE RANDOM SHUFFLE OPTION,  WEIGHT AND VAR
C           ARE UNCHANGED AND DON'T HAVE TO BE RE-COMPUTED

C           WRITE IMAGE (NUMBER N1) INTO SEQUENTIAL FILE
C           THE RECORD WILL BE :: BUF(1:NCOUNT), WEIGHT, VAR, N1

            CALL WRITT(LUNS, N1, BUF, NCOUNT + 2)          
            CLOSE(LUN)

            IF (NEG .NE. 0) THEN
               NEGTOT = NEGTOT + NEG
C              WRITE(NOUT, 91) NEG
C91            FORMAT('*** WARNING:',I6,' NEGATIVE VALUES ENCOUNTERED')
            ENDIF
          END DO  
              
          WRITE(NOUT, 301) NMASK
301    	  FORMAT(' NUMBER OF POINTS IN MASK: ',I6)

          IF (NEGTOT .NE. 0)  THEN
             WRITE(NOUT,*) ' '
	     WRITE(NOUT, 302) NEGTOT, BUFNEG
302          FORMAT(' *** WARNING: ',I8, 
     &              ' NEGATIVE VALUES ENCOUNTERED.'/
     &              '     SMALLEST NEGATIVE VALUE IS: ', 1PE10.3)
             WRITE(NOUT, 303) BUFNEG
303          FORMAT('     CORAN DOES NOT ACCEPT NEGATIVE VALUES ',
     &                    'IN IMAGES.',/,
     &              '     ADD A CONSTANT >= ', 1PE10.3,/,
     &              '     PCA ACCEPTS NEGATIVE VALUES',
     &              '     NO NEED TO ADD A CONSTANT.')
          ENDIF

C         END OF SEQUENTIAL FILE PREPARATION STEP
          CLOSE(LUNS)
          RETURN
        ENDIF
 
C-------------------------------------------------------------------
C       OPTION 'CA S' -- START OF CORAN ANALYSIS  
C-------------------------------------------------------------------
 
C       OPEN SEQUENTIAL FILE

        CALL FILERD(FILNAM, NLET, NULL, 'SEQUENCE', IRTFLG)
	IF (IRTFLG  .NE.  0) RETURN
 
        CALL FILCAN(FILENM, NFN, NULL, NULL, FILNAM, DATEXC, IER)
        OPEN(LUNS, FILE = FILENM, STATUS = 'OLD', 
     &         FORM = 'UNFORMATTED')
                    
C	NUMBER OF IMAGES, NUMBER OF PIXELS, NUMBER OR ROW AND COLUMN IN 
C	THE IMAGES.
C	MADAR : IMAGE OF THE MASK.
C	IMAR(I) : SELECTED IMAGE (IMAGE NUMBER STORED)

C       READ IN HEADER
        READ(LUNS) NUMIM, NPIX,  NSAM,  NROW, 
     $             (MADAR(K), K = 1, NPIX),
     $             (IMAR(K), K = 1, NUMIM)
                                    
        WRITE(NOUT, 119) NUMIM, NPIX
119	FORMAT('  NUMBER OF IMAGES: ', I4,'   NUMBER OF PIXELS: ', I6,/) 
      
C       OPTION 'SF' CHOSEN ?
        IF (FCHAR(5:5) .NE. 'F') THEN
          CALL RDPRMI(NFAC, IMNO, NLOOP, ILOOP, 
     &            'NUMBER OF FACTORS, COORDINATE FILE CODE')

          CALL RDPRMC(ANS, NCHAR, .TRUE., 
     &       'PRINCIPAL COMPONENT ANALYSIS (Y) OR CORAN(N) ', 
     &       NULL, IRTFLG)
          IF (ANS .EQ. 'Y') THEN
            PICK_PCA = .TRUE.
          ELSE
            PICK_PCA = .FALSE.
          ENDIF

C         IF YOU CHANGE PIX*** TO PIXEL***, NLET=8 FOR FILPATP
          NLET = 6
          CALL FILGET(FILPATC, FILNMC, NLET, IMNO, IRTFLG)
          CALL FILGET(FILPATP, FILNMP, NLET, IMNO, IRTFLG)
          CALL FILGET(FILPATE, FILNME, NLET, IMNO, IRTFLG)

C         INITIALIZE LOGICAL UTILITY VECTORS FOR IMAGES AND PIXELS
          DO  IM = 1, NUMIM
             LUV(IM) = 1
          END DO

           NSW = 1
           NUMIMF = MAXIM
           ninact = maxim
           CALL RDPRAI(IMINAC, MAXIM, NINACT, 0, NUMIMF, 
     &         'ENTER INACTIVE IMAGE NUMBERS (EG. 1-3,5,6-9 or 0 )',
     &          NULL, IRTFLG)

C          NACT = NUMBER OF ACTIVE IMAGES. = NUMBER OF IMAGES
           NACT = NUMIM 

           IF (IMINAC(1).GT.0) THEN
             DO I = 1, NINACT
               DO IM = 1, NUMIMF
                 IF (IMINAC(I) .EQ. IMAR(IM)) THEN

C                  PUT THIS IMAGE TO INACTIVE STATUS
                   LUV(IM) = 0
                   NACT = NACT - 1
                   GOTO  6200
                 ENDIF
               END DO
               WRITE(NOUT, 6151) IMINAC(I)
6151           FORMAT('*** IMAGE: ', I3, ' WAS NOT FOUND.',/,
     &                ' IMAGE  NOT SET TO INACTIVE STATUS   ')
6200         END DO
           ENDIF                         

C         NOW OPEN ALL FILES NEEDED IN SUBSEQUENT ROUTINES
C         OPEN SEQUENTIAL FILE
Cifdef SP_SUN4
C          OPEN(LUNS, FILE = FILENM, STATUS = 'OLD', 
C     &         FORM = 'UNFORMATTED')
Celse
C          OPEN(LUNS, FILE = FILENM, STATUS = 'OLD', 
C     &         FORM = 'UNFORMATTED', READONLY)
Cendif
        
C         OPEN IMAGE COORDINATE FILE
          CALL FILCAN(FILNAM, NFN, NULL, NULL, FILNMC, DATEXC, IER)
          OPEN( FILE = FILNAM, UNIT = LUN, STATUS = 'UNKNOWN', 
     &         FORM = 'UNFORMATTED')
        
C         OPEN PIXEL COORDINATE FILE
          CALL FILCAN(FILNAM, NFN, NULL, NULL, FILNMP, DATEXC, IER)
          OPEN( FILE = FILNAM, UNIT = LUNP, STATUS = 'UNKNOWN', 
     &         FORM = 'UNFORMATTED')
                           
C         OPEN EIGENVALUE FILE
          CALL FILCAN(FILNAM, NFN, NULL, NULL, FILNME, DATEXC, IER)
          OPEN(FILE = FILNAM, UNIT = LUNM, STATUS = 'UNKNOWN',
     &         FORM = 'UNFORMATTED')
                                          
C         LOGICAL UNIT NUMBERS:
C                   LUNS  =  SEQ FILE CONTAINING MASKED IMAGE DATA
C		    LUN   =  SEQ FILE RESERVED FOR IMAGE COORDINATES
C		    LUNP  =  SEQ FILE RESERVED FOR PIXEL COORDINATES
C		    LUNM  =  SEQ FILE RESERVED FOR EIGENVALUES

          IF (PICK_PCA) THEN
            CALL PCA_MAIN(NUMIM, NFAC, NSAM, NROW, NPIX, LUNS,
     &                   LUN, LUNP, LUNM, BUF, MAXDIM, LUV, 
     &                   IMAR, MADAR, NACT, NOUT)
          ELSE
            CALL SCORAN3(NUMIM, NFAC, NSAM, NROW, NPIX, LUNS,
     &                   LUN, LUNP, LUNM, BUF, MAXDIM, LUV, 
     &                   IMAR, MADAR, NACT, NOUT)
          ENDIF

C         CLOSE ALL FILES
750	  CLOSE(LUNS)
          CLOSE(LUN)
          CLOSE(LUNP)
          CLOSE(LUNM)
          RETURN
        ELSE

C-------------------------------------------------------------------
C         OPTION 'CA SF'  --  RETRIEVE IMAGES FROM SEQUENTIAL FILE
C-------------------------------------------------------------------

800       WRITE(NOUT,* ) ' FOR IMAGE SERIES:'
          CALL FILSEQP(FILPAT1, NLET1, IDUM, 0, IDUM,  
     $  'FILE PREFIX OR TEMPLATE (EG. PIC****)',IRTFLG)
     
          CALL RDPRMI(INC, ISIGN, NLOOP, ILOOP, 
     &                'ENTER INC,  SIGN( + 1/ - 1)')
          IF (ISIGN .EQ. 0) THEN
             SIGN =  + 1.
          ELSE
             SIGN = ISIGN
          ENDIF

          NSW    = 1
          IOFF   = NSAM
          NTOT   = NSAM * NROW
          IOFF1  = NSAM * NROW + 2 * NSAM
          NUMIMF = MAXIM
          CALL RDPRAI(IMAR, MAXIM, NUMIMF, IMAR(1),IMAR(NUMIM), 
     &                '.ENTER IMAGE NUMBERS', NULL, IRTFLG)
       
          DO  IM = 1, NUMIMF

C	    NOTE BY MAHIEDDINE LADJADJ
C		THE NEXT 4 LINES HAVE BEEN BADLY WRITEN.
C		WHAT HAPPENS IF IT IS NEVER IM1 .GE. IMAR(R)
C               OR IF READT READS AN EOF ?

            CALL READT(LUNS, IM1, BUF(IOFF + 1), NPIX + INC)
            DO WHILE (IM1 .LT. IMAR(IM)) 
               CALL READT(LUNS, IM1, BUF(IOFF + 1), NPIX + INC)
            END DO
            IF (IM1 .EQ. IMAR(IM)) THEN
              FMIN =  + 1.E8
              FMAX =  - FMIN
              DAV  = 0.0
              DAV2 = 0.0
              DO  L = 1, NPIX
                B = BUF(IOFF + L) * SIGN
                BUF(IOFF + L) = B
                IF (B.LT.FMIN) FMIN = B
                IF (B.GT.FMAX) FMAX = B
                DAV  = DAV + B
                DAV2 = DAV2 + B**2
              END DO
              AV2 = DAV2/FLOAT(NSAM * NROW)
              AV  = DAV/FLOAT(NSAM * NROW)
              SIG = (SQRT(AV2 - AV**2))
        
C             PUT FMIN INTO BACKGROUND
              B = FMIN
              DO L = 1, NTOT
                BUF(IOFF1 + L) = B
              END DO
              DO L = 1, NPIX
                CALL MSKDEC(L, MADAR, NSAM, K, I)
                I1 = (I - 1) * NSAM + K
                BUF(IOFF1 + I1) = BUF(IOFF + L)
              END DO
        
C             OPEN IMAGE FILE
              NUM = IMAR(IM)
              CALL FILGET(FILPAT1, FILNM1, NLET1, NUM, IRTFLG)
              NSLICE = 1
              IFORM = 1
              CALL OPENF(FILNM1, LUN, NSAM, NROW, 'UNKNOWN',
     &                   NF, NSLICE)
              CALL SETPRM(LUN, NSAM, NROW, FMAX, FMIN, AV, 'R')
              I1 = IOFF1 + 1
              DO I = 1, NROW
                CALL WRTLIN(LUN, BUF(I1), NSAM, I)
                I1 = I1 + NSAM                
              END DO
              CLOSE(LUN)
            ELSE
              WRITE(NOUT, 8051) IMAR(IM)
8051          FORMAT(' *** IMAGE: ', I4, ' NOT FOUND')
            ENDIF
          END DO
          CLOSE(LUNS)
          
          RETURN
        ENDIF
        

C       ERROR MESSAGES

9000	IER = 19
        GOTO  9900
9100	IER = 2
        GOTO  9900
9300	IER = 4
9900	CALL ERRT(IER, 'JPMSK1', IER)

C       CLOSE ALL FILES
	CLOSE(LUNS)
        CLOSE(LUN)
        CLOSE(LUNP)
        CLOSE(LUNM)
	RETURN

9800	WRITE(NOUT, 9801) MAXMSK
9801	FORMAT(' *** NUMBER OF POINTS IN MASK EXCEEDS:', I5)
9600	CALL ERRT(5100, 'JPMSK1', NE)
        END
