
C++************************************************************* 5/14/85   
C                                    LONG FILE NAMES FEB 89 al
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C PURPOSE:   RECONSTITUTE IMAGES USING SELECTED FACTORS
C		 OF A CORRESPONDENCE ANALYSIS STORED IN
C		 FILES IMC00#, PIX00#, AND EIG00#, WHERE #
C		 IS THE COORDINATE FILE CODE.
C
C	JPMSK2(LUN,LUNM,LUNP,MAXDI4)
C	  LUN	   LOGICAL UNIT NUMBER OF  INPUT FILE
C	  LUNM     LOGICAL UNIT NUMBER OF MASK FILE
C	  LUNP     LOGICAL UNIT NUMBER OF PIXEL COORDINATE FILE
C	  MAXDI4   MAXIMUM BUFFER LENGTH TO BE USED BY PROGRAM
C
C COMMANDS SUPPORTED :
C CA SRP -- RECONSTITUTE IMAGE FOR PRINCIPAL COMPONENT ANALYSIS (PCA)
C CA SR  -- RECONSTITUTE FULL IMAGE INCLUDING 0-FACTOR
C CA SRD -- RECONSTITUTE DIFFERENTIAL IMAGE, WITHOUT 0-FACTOR
C CA SRI -- RECONSTITUTE IMPORTANCE IMAGE (AS CA_SRD BUT
C           WITHOUT COORDINATE WEIGHTING)
C CA SR/C (ETC.) -- AS ABOVE, BUT COORDINATES ARE SUPPLIED BY A
C 	    CLUSTER FILE FROM A PREVIOUS RUN OF 'CA CLA'
C--*******************************************************************
C**MODIFIED 2/6/86 J.F. TO ACCESS CLUSTER CENTER COORDINATES IF NEEDED
C  (OPTIONS CA SR/C, CA SRD/C, AND CA SRI/C). CLEANED UP FOR I4 VERSION
C  6/16/88 J.F.
C
C--*******************************************************************

	SUBROUTINE JPMSK2(LUN,LUNM,LUNP,MAXBUF)

C       MAXIMUM CLASS NUMBER TO BE USED:
C	PARAMETER (MAXKLA=50)

C       MAXIMUM NUMBER OF FACTORS ALLOWED:
C        PARAMETER (MAXFAC=24)

 
	INCLUDE 'CMBLOCK.INC'  
	COMMON    BUF(1)
	EQUIVALENCE(IDUM(1),BUF(1))
        COMMON  /COMMUN/ FILNM1,FILNM2,FILNMP,FILNAM,
     &                   FILNMC,FILNME,FILPATC,FILPATP,
     &                   FILPATE,FILPAT1
        CHARACTER *81    FILNM1,FILNM2,FILNMP,FILNAM,
     &                   FILNMC,FILNME,FILPATC,FILPATP,
     &                   FILPATE,FILPAT1
        CHARACTER * 1    NULL, ADD_AVRG, PCA_AVRG
	DIMENSION    COO(1)
        POINTER      (IPQCOO,COO)
	DIMENSION    NUMFAC(1)
        POINTER      (IPQNUMFAC,NUMFAC)
c	REAL      COO(MAXFAC),CG(MAXKLA,MAXFAC)
c	INTEGER   NUMFAC(MAXFAC)
	INTEGER   IDUM(2000)
	INTEGER   IMAR(2000)
        DATA AMIN/-10E30/,AMAX/10E30/
        LOGICAL   PICK_PCA
        INTEGER   PCA_ERR, PTRFAC, PTRCOO, MAXFAC
CNO_SAVE        
        NULL    = CHAR(0)
       
        FILPATC = 'IMC***'//NULL
        FILPATP = 'PIX***'//NULL
        FILPATE = 'EIG***'//NULL

C	IMUSED IS THE NUMBER OF IMAGES WE WANT TO RECONSTRUCT.
C	FOR EXAMPLE: 6 (IF FILE NUMBERS: 2-3,5-8)
C	IMAR() ARRAY HOLDS  THOSE FILE NUMBERS.
C			SEE DO LOOP 100
C	ML.

        CALL FILSEQP(FILPAT1,NLET1,IMAR,2000,IMUSED,
     $       'FILE TEMPLATE (EG. PIC****)' ,IRTFLG)
     

        IF (FCHAR(4:6) .EQ. 'SRP') THEN
          PICK_PCA = .TRUE.
          ICLUR    = 0
          CALL RDPRMC(PCA_AVRG, NCHAR, .TRUE.,
     &       'SUBTRACT AVERAGE IN PCA RECONSTITUTION? (N/Y)' ,
     &       NULL,IRTFLG)
          IF (PCA_AVRG .NE. 'Y' .AND. PCA_AVRG .NE. 'y') THEN
             ADD_AVRG = 'Y'
          ELSE
C	     SUBTRACT AVERAGE IN THE RECONSTRUCTED DATA.
             ADD_AVRG = 'N'
          END IF

        ELSE
          PICK_PCA = .FALSE.
     
C         IDIFF= SWITCH DETERMINING WHETHER THE AVERAGE IMAGE 
C         SHOULD BE SUBTRACTED.
C         IDIFF=1 -- SUBTRACT. IDIFF=-1 -- SUBTRACT 
C         AVERAGE IMAGE, AND THEN DIVIDE RESULT BY AVERAGE IMAGE 
C         TO OBTAIN  SO-CALLED "IMPORTANCE IMAGE" USED IN EARLIER
C         LITERATURE.

   	  IDIFF=0
          IF (FCHAR(6:6)    .EQ.'D') THEN
            IDIFF=1 
	  ELSEIF (FCHAR(6:6).EQ.'I') THEN
            IDIFF=-1
          END IF
C
C         CLUSTER READING SWITCH ICLUR IS SET IF RECONSTITUTION IS
C         ACCORDING TO CLUSTER CENTER COORDINATES.

	  ICLUR=0
	  IF (FCHAR(7:7).EQ.'C'.OR.FCHAR(8:8).EQ.'C') THEN

C	     WE ARE MOVINF THE CLUSTER COMMAND TO CL *** INSTEAD 
C	     OF CA *** AND WE ARE CHANGING THE INPUT FILES FROM
C	     A BINARY FORMAT (PIX***, IMC***) TO DOCUMENT FORMAT
C	     (SD *** COMMAND TYPE)
	  
	     WRITE(NOUT,*) 'NO CLUSTER RECONSTRUCTION FOR NOW !!'
	     RETURN

C            ICLUR=1
C            CALL FILERD(FILNM2,NLET,NULL,'CLUSTER',IRTFLG)
C            IF (IRTFLG.NE.0) RETURN
C            CALL FILCAN(FILNAM,NFN,NULL,NULL,FILNM2,DATEXC,IER)
C            OPEN(UNIT = LUN, FILE = FILNAM, STATUS = 'OLD',
C     &           FORM = 'UNFORMATTED')
C            READ(LUN) NUMIM,NFAC,NKLA,NUSE,(NUMFAC(K),K=1,NUSE)
C            IF (NKLA.GT.MAXKLA) GOTO 9500
C            READ(LUN)(IDUM(I),I=1,NUMIM),(IDUM(I),I=1,NUMIM),
C     &        (BUF(I),I=1,NKLA),((CG(L,J),L=1,NKLA),J=1,NUSE)
C            CLOSE(LUN)

	  ENDIF
        END IF

C       IF AN IMAGE NUMBER IS NOT FOUND IN THE INDEX ARRAY OF
C       IMC00# THEN IT IS ASSUMED THAT A DUMMY IMAGE HAS TO BE
C       CREATED.  IN THAT CASE, COORDINATES FOR EACH FACTOR USED
C       ARE SOLICITED, UNLESS THEY ARE SUPPLIED BY A CLUSTER FILE.

	CALL RDPRMI(IMNO,NDUM,NLOOP,ILOOP,'COORDINATE FILE CODE$')
        NLET = 6
        CALL FILGET(FILPATC,FILNMC,NLET,IMNO,IRTFLG)
        CALL FILGET(FILPATP,FILNMP,NLET,IMNO,IRTFLG)
        CALL FILGET(FILPATE,FILNME,NLET,IMNO,IRTFLG)

C       OPEN IMAGE COORDINATE FILE
	CALL FILCAN(FILNAM,NFN,NULL,NULL,FILNMC,DATEXC,IER)
	OPEN(FILE=FILNAM,UNIT=LUN,STATUS='OLD',FORM='UNFORMATTED')
        READ(LUN) NUMIM, NFAC, NSAM, NROW
 
C       OPEN PIXEL COORDINATE FILE
	CALL FILCAN(FILNAM,NFN,NULL,NULL,FILNMP,DATEXC,IER)
	OPEN(FILE=FILNAM,UNIT=LUNP,STATUS='OLD',FORM='UNFORMATTED')


C       BUFFER ALLOCATIONS
C       JMASK = OFFSET OF MASK ARRAY
C       JPIC = OFFSET OF ARRAY CONTAINING RECONSTITUTED IMAGE

20      IM = NSAM*NROW
	JMASK = 2000
	JPIC  = JMASK + IM
	JBUF  = JPIC + IM
	JOFF  = JBUF + NSAM
 
C       BYPASS FACTOR SELECTION IF COORDINATES COME FROM 
C       CLASSIFICATION  CLUSTER FILE

	IF (ICLUR.NE.1) THEN
	  MAXFAC = NFAC
          NUSE = MAXFAC	 
	   
c	  ARRAY NUMFAC(MAXFAC)
          PTRFAC = JOFF
	  IPQNUMFAC = %LOC(BUF(PTRFAC)) 
	  
          CALL RDPRAI(NUMFAC,MAXFAC,NUSE,1,MAXFAC,
     &              'ENTER FACTORS TO BE USED',NULL,IRTFLG) 
     
c	  NUSE IS THE MAX SIZE OF ANSWERS ENTERED IN NUMFAC ARRAY. 
C	  ASSIGN SPACE FOR NUMFAC ARRAY
          JOFF = PTRFAC + NUSE
	   
c	  ARRAY COO(MAXFAC)
	  PTRCOO = JOFF
	  IPQCOO = %LOC(BUF(PTRCOO))
	  JOFF = PTRCOO + MAXFAC
        ENDIF
	

C       OPEN EIGENVALUE FILE

	CALL FILCAN(FILNAM,NFN,NULL,NULL,FILNME,DATEXC,IER)
	OPEN(FILE=FILNAM,UNIT=LUNM,STATUS='OLD',FORM='UNFORMATTED')

C       NOW INITIALIZE JP'S RECONSTITUTION ROUTINE

        PCA_ERR = 0 
        IF (PICK_PCA) THEN
          CALL PCA_SRIPE( NUMIM, NPIX, NFAC, LUN, LUNP, LUNM,
     $                  BUF, JOFF, NSAM, NROW, SUMP, LCI, LIDI, LCP,
     $                  LIDP, LD, LPJ, JIM, MAXBUF, PCA_ERR)
          IF (PCA_ERR .NE. 0) RETURN
        ELSE
          CALL SRIPE(NUMIM,NPIX,NFAC,LUN,LUNP,LUNM,
     &              BUF,JOFF,NSAM,NROW,SUMP,LCI,LWI,LIDI,LCP,
     &              LWP,LIDP,LD,JIM, MAXBUF, PCA_ERR) 
          IF (PCA_ERR .NE. 0) RETURN
	END IF 

C       NOTE THAT BUF(LIDP) ... BUF(LIDP+NPIX-1) CONTAINS THE
C       MASK ADDRESS ARRAY (CALLED MADAR IN JPMSK1)

	CLOSE(LUN)
	CLOSE(LUNP)

C NOW THAT THE NUMBER OF FACTORS USED IN THE INITIAL ANALYSIS IS
C KNOWN, SWAP FACTOR SELECTION ARRAY AND CHECK AGAINST THIS NUMBER

	IF (ICLUR.NE.1) THEN
          DO  I=1,NUSE
          IF (NUMFAC(I).GT.NFAC)GOTO 9800
	  ENDDO
        ENDIF

C-------------------------------------------------
                  
C LOOP OVER ALL IMAGES;   begin DO LOOP 100
 
        DO  IM=1,IMUSED
	  IMNO = IMAR(IM)

          IF (ICLUR.EQ.1) THEN
	     WRITE(NOUT,*) ' NO CLUSTER RECONSTRUCTION IN JPMSK2'
	     RETURN
CC            IF THIS IS A RECONSTITUTION FROM CLUSTER COOS, 
CC            MAKE SURE THAT THE IMAGE NUMBER SPECIFIED IS WITHIN 
CC	     RANGE OF CLUSTER NUMBERS
CC
C             IF (IMNO.GT.NKLA) THEN
C                WRITE(NOUT,225) IMNO,NKLA
C225             FORMAT(' *** IMAGE NUMBER ',I4,
C     &                 ' OUTSIDE RANGE (1...',I4,')')
C                GOTO 100
C             ENDIF
C
C             DO 23 I=1,NUSE
C23              BUF(I) = CG(IMNO,I)
C             GOTO 24
          ENDIF

          IMS = IMSEQ4(NUMIM,IMNO,BUF(LIDI))
          IF (IMS .EQ. 0) THEN
             WRITE(NOUT,9700) IMNO
 9700        FORMAT(' ** IMAGE NO ',I5,' DOES NOT HAVE COORDINATES')

             NVAL = NUSE
             CALL RDPRAF(BUF,NUSE,NVAL,AMIN,AMAX,
     &       'ENTER COORDINATES FOR EACH FACTOR SELECTED',NULL,IRTFLG)

24           DO  I=1,MAXFAC
	       COO(I)=0
	     ENDDO

             IMAX=0
             DO 28 K=1,NUSE
               DO  I=1,MAXFAC
                 IF (NUMFAC(K).EQ.I) THEN
                   COO(I) = BUF(K)
                   IF (I.GT.IMAX) IMAX=I
                   GOTO 28
                 ENDIF
	       ENDDO
28          CONTINUE

C           WRITE OUT COORDINATES TO BE USED
            WRITE(NOUT,281)(COO(I),I=1,IMAX)
281         FORMAT(1X,8F8.3)

C           NOW CALL RECONSTITUTION ROUTINE FOR DUMMY IMAGE
	
            IF(PICK_PCA) THEN
              CALL PCA_RECDUM(NUMIM, NPIX, NUSE, NUMFAC, 
     &              NFAC, AV,BUF(JIM), BUF(LCP),
     &              COO, BUF(LD), BUF(LPJ), ADD_AVRG)
	    ELSE
              CALL RECDUM(NUMIM,NPIX,NUSE,NUMFAC,NFAC,SUMP,
     &              FMAX,FMIN,AV,SIG,IDIFF,BUF(JIM),
     &              BUF(LCP),COO,BUF(LD),BUF(LWI),BUF(LWP))
            END IF
	  ELSE

C           NOW CALL RECONSTITUTION ROUTINE FOR CURRENT IMAGE

            IF(PICK_PCA) THEN
               CALL PCA_REMAKE(IMS, NUMIM, NPIX, NUSE, NUMFAC, 
     &              NFAC, AV,BUF(JIM), BUF(LCP), 
     &              BUF(LCI), BUF(LD), BUF(LPJ), ADD_AVRG)
            ELSE
               CALL RECONS(IMS,NUMIM,NPIX,NUSE,NUMFAC,NFAC,SUMP,
     &                  FMAX,FMIN,AV,IDIFF,BUF(JIM),BUF(LCP),
     &                  BUF(LCI),BUF(LD),BUF(LWI),BUF(LWP))
            END IF
          ENDIF

          CALL FILGET(FILPAT1,FILNM1,NLET1,IMNO,IRTFLG)
          IF (IRTFLG .NE. 0) GOTO 9000
          IFORM=1
          CALL OPENF(FILNM1,LUN,NSAM,NROW,'UNKNOWN',NF,1)
          IF (NF.NE.2)GOTO 9300

          IF(PICK_PCA) THEN
            FMIN = BUF(JIM)
            DO I = 1, NPIX - 1
              IF (FMIN .GT. BUF(JIM + I)) FMIN = BUF(JIM+I)
            END DO
          END IF

C	 PUT FMIN IN THE RECONSTITUTED IMAGE BUF(JPIC:-)

	  DO  I = JPIC, JPIC + NSAM*NROW - 1
            BUF(I) = FMIN
          END DO
                                                    

C         NOW REPLACE PIXELS IN MASK AREA BY THEIR TRUE VALUES
C	  COPY OUTPUT IMAGE BUF(J IM:-) TO RECONSTITUTED IMAGE
C	  BUF(JPIC:-) SUBJECT TO MASK IN BUF(LIDP:-)
C         CA SR.

          DO  L = 1, NPIX
            CALL MSKDEC(L, BUF(LIDP), NSAM, K, I)
            BUF(JPIC + (I - 1)*NSAM + K - 1) = BUF(L + JIM - 1)
	  ENDDO

C         WRITE IMAGE ONTO DISK
C
           I1 = JPIC
           DO  I = 1, NROW
             CALL WRTLIN(LUN, BUF(I1), NSAM, I)
             I1 = I1 + NSAM
	   ENDDO

          CLOSE(LUN)
	ENDDO 


C	END DO LOOP 100

	RETURN
9000	IER=19
	GOTO 9900
9300	IER=4
	GOTO 9900
 
9800	WRITE(NOUT,9801) NFAC
9801	FORMAT(' ** AXIS NUMBER OUT OF RANGE. MAXIMUM = ',I2)
	WRITE(NOUT,9802) NFAC, NUMFAC(I)
9802    format(' The number of factors (or eigenvectors) used to ',
     $         ' create your data files is: ',i4,
     $         ' you have asked for factor: ',i4)
	RETURN

9900	CALL ERRT(IER,'JPMSK2',IER)
	END

