
C++************************************************************************
C
C LHIST.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C  PURPOSE: LOCAL HISTOGRAM GENERATION
C
C  LHIST(BO,NSAM,NLOCAL,KCTR,NPTR,H,MODE)
C         BO
C         NSAM       NUMBER OF SAMPLES
C         NLOCAL     LOCAL AREA DIMENSION (X IS = TO Y)
C         KCTR       COLUMN POINTER
C         NPTR       ROW    POINTER
C         H          HISTOGRAM BUFFER
C         MODE       1 NON-INCREMENTAL OPERATION, NO INTEGRATION
C                    2 INCREMENTAL OPERATION, NO INTEGRATION
C                    3 NON-INCREMENTAL OPERATION WITH INTEGRATION
C                    4 INCREMENTAL OPERATION WITH INTEGRATION
C*****************************************************

      SUBROUTINE LHIST(B0,NSAM,NLOCAL,KCTR,NPTR,H,MODE)

      INCLUDE 'CMBLOCK.INC'

      REAL      H(1),B0(1)
      DIMENSION NPTR(1)

      SAVE FVAL

      IF (MODE .EQ. 0) THEN
        IF (FMAX .EQ. FMIN) THEN
          CALL ERRT(5,'LHIST ',NE)
          MODE = -1
          RETURN
        ENDIF

        FVAL = 127./ (FMAX-FMIN)
        FN   = 1.  / FLOAT(NLOCAL)**2
        RETURN
      ENDIF

      KCTR1 = KCTR-NLOCAL / 2
      GOTO (10,200,8,200),MODE
     
C     INITIALIZE HISTOGRAM
8     DO  K = 129,256
        H(K) = 0.
      ENDDO

10    DO  K = 1,128
        H(K) = 0.
      ENDDO

      DO  I = 1,NLOCAL
         II = (NSAM * (NPTR(I) - 1) + KCTR1) - 1
         DO  K = 1,NLOCAL
            J = INT((B0(II+K)-FMIN)*FVAL) + 1.5
            IF (J .LE. 1) THEN
              H(1) = H(1) + 1.0
            ELSEIF (J. GE. 128) THEN
              H(128) =H(128) + 1.0
            ELSE
              H(J) = H(J) + 1.0
            ENDIF
	 ENDDO
      ENDDO

      IF (MODE .EQ. 3) THEN
         H(129) = H(1)
         DO  K = 2,128
           H(128+K) = H(127+K) + H(K)
	 ENDDO
      ENDIF
      RETURN



C     INCREMENTAL UPDATING OF HISTOGRAM
200   DO  I = 1,NLOCAL
         NRUN =  1.0
         HADD = -1.0
C        FIRST COLUMN SUBTRACTED IN FIRST PASS, LAST COL. ADDED IN SECOND PASS
         II = NSAM*(NPTR(I)-1)+KCTR1

201      J = INT((B0(II)-FMIN)*FVAL)+1.5
         IF (J .LE. 1) THEN
           H(1) = H(1) + HADD
         ELSEIF (J .GE. 128) THEN
           H(128) = H(128) + HADD
         ELSE
           H(J) = H(J) + HADD
         ENDIF

         IF (NRUN .EQ. 1) THEN
            II = II + NLOCAL -1
            NRUN = 2
            HADD = +1.
            GOTO 201
         ENDIF
      ENDDO


      IF (MODE .EQ. 4) THEN
         H(129) = H(1)
         DO  K = 2,128
           H(128+K) = H(127+K)+H(K)
	 ENDDO
      ENDIF

      RETURN
      END
