
C**************************************************************************
C
C  MAPDIST.FOR  -- CREATED JAN 91
C **************************************************************************
C *  AUTHOR: ArDean Leith 
C *  COPYRIGHT (C)1991 , WADSWORTH CENTER FOR LABORATORIES AND RESEARCH    *
C *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C    MAPDIST(MAPIMAG,DOCRES,IRTFLG) 
C
C    PURPOSE:       READS A DOC. FILE CONTAINING CLUSTER NUMBER (KEY),
C                   NUMBER OF VOXELS (REG 1) IN THE CLUSTER, AND COM. 
C                   DETERMINES DISTANCE BETWEEN ALL COMS.  IF DISTANCE
C                   BETWEEN PRESENT CLUSTER AND TEST CLUSTER IS LESS
C                   THAN THRESHOLD, SETS TEST CLUSTER TO PRESENT CLUSTER.
C                   REMAPS IMAGE TO NEW CLUSTER NUMBERS.  
C
C      PARAMETERS:  MAPIMAG          MAP INTO NEW FILE  
C                   DOCRES           OUTPUT NEW DOC FILE
C                   IRTFLG           ERROR FLAG
C
C      CALLED BY:   UTIL6
C
C      CALLS:       MAKTAB   
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

       SUBROUTINE MAPDIST(MAPIMAG,DOCRES,IRTFLG)
		                                                                                        
       INCLUDE 'CMBLOCK.INC'

       PARAMETER  (NSAMAX=10000)
       PARAMETER  (MAXREG=6)
       PARAMETER  (MAXKEY=4000)
       PARAMETER  (MAXLUT=4000)
       PARAMETER  (NEQMAX=10000)
C      USED IN MAKTAB ONLY
       PARAMETER  (NOTMAX = 10000)

       COMMON BUF(NSAMAX),TABLE(MAXLUT),DLIST(MAXREG+1),IEQUIV(2,NEQMAX)
     &        ,NOTDONE(NOTMAX),NOTUSED(NOTMAX), ISTACK(1)


       COMMON /DOC_BUF/ DBUF(MAXREG,MAXKEY)
 
       CHARACTER *81 DOCNAM,IMFILE,OUTFILE
       CHARACTER *1  NULL
       LOGICAL       MAPIMAG,DEBUG,DOCRES
     
       DATA MININT/-32768/,MAXINT/32767/,FLTMIN/-10E30/,
     &       FLTZER/10E-30/,FLTMAX/10E30/

       DATA LUND,LUNIM,LUND2/20,21,22/
	NULL=CHAR(0)
       IRTFLG = 1
       DEBUG  = .FALSE.

       WRITE(NOUT,90)
90     FORMAT(/,
     &  ' This routine reads a document file from the mapstat output',/,
     &  'and merges all clusters having center of mass less than    ',/,
     &  'threshold distance away into a new overall cluster.'/)

10     CALL FILERD(DOCNAM,NLETD,NULL,'CENTER-OF-MASS DOC',IRTFLG)
       IF (IRTFLG .EQ. -1) RETURN

       NLIST = 2
       IKEY  = 1
       CALL FILCAN(DOCNAM,NLET,NULL,NULL,DOCNAM,DATEXC,IRTFLG)
       CALL UNSDAL(DOCNAM,ICALL,LUND,IKEY,PLIST,NLIST,DBUF,
     &             MAXKEY,MAXREG,NKEY,IERR)

       IF (NVAL .GT. MAXLUT) THEN
C         TABLE OVERFLOW
          MAXL = MAXLUT
          WRITE(NOUT,*) ' *** IN MAPDIST, TABLE LIMIT:',MAXL
          CALL ERRT(100,'MAPDIST',NE)
          RETURN
       ENDIF
       WRITE(NOUT,*) 'READ DATA FOR :',NVAL,' CLUSTERS.'

       CALL RDPRM2S(SEPMIN,MINCLU,NLOOP,ILOOP,
     &    'ENTER THRESHOLD AND STARTING CLUSTER',IRTFLG)
       IF (IRTFLG .EQ. -1) GOTO 10
        
C      MINIMUM SEPARATION FOR ANY CELL PAIR (IN UM) SQ
       SEPMINSQ = SEPMIN * SEPMIN
C      INITIALIZE NUMBER OF EQUIVAENT CLUSTERS
       NEQUIV   = 0

       DO 4 I = MINCLU,NVAL-1

C        FIND IDENTITY OF PRESENT CLUSTER
         ICLU1 = DBUF(1,I) 
         N1    = DBUF(2,I) 
         X1    = DBUF(3,I)
         Y1    = DBUF(4,I) 
         Z1    = DBUF(5,I) 

         IF (N1 .LT. 0) THEN
C           IMPOSSIBLE NUMBER OF VALUES IN THIS CLUSTER
            WRITE(NOUT,*) ' *** N1 :',N1,' CAN NOT BE < 0'
            GOTO 4

         ELSEIF (N1 .GT. NVAL) THEN
C           IMPOSSIBLE NUMBER OF VALUES IN CLUSTER
            WRITE(NOUT,*) ' *** N1 :',N1,' CAN NOT BE > ',NVAL
            GOTO 4
         ENDIF

         DO 5 J = I+1,NVAL

C           FIND LOCATION OF TEST CLUSTER

            ICLU2 = DBUF(1,J)
            N2    = DBUF(2,J)
            X2    = DBUF(3,J) 
            Y2    = DBUF(4,J) 
            Z2    = DBUF(5,J) 

            IF (N2 .LT. 0) THEN
C               IMPOSSIBLE NUMBER OF VALUES IN CLUSTER
                WRITE(NOUT,*) ' *** N2 :',N2,' CAN NOT BE < 0'
                GOTO 5
            ENDIF

C           FIND SEPARATION BETWEEN TWO ORIGINAL CLUSTERS
            DXSQ = (X1 - X2)**2      
            DYSQ = (Y1 - Y2)**2
            DZSQ = (Z1 - Z2)**2

            DSQ  = DXSQ + DYSQ + DZSQ

            IF (DSQ .LE. SEPMINSQ) THEN
C              PUT IN SAME OVERALL CLUSTER
               NEQUIV = NEQUIV + 1
               IEQUIV(1,NEQUIV) = ICLU1
               IEQUIV(2,NEQUIV) = ICLU2
            ENDIF

5        CONTINUE
4      CONTINUE

       WRITE(NOUT,94) NEQUIV
94     FORMAT(/,' NUMBER OF EQUIVALENCES: ',I8)

       CALL MAKTAB(IEQUIV,NEQUIV,TABLE,NVAL,NTAB,
     &              NOTDONE,NOTUSED,ISTACK,IRTFLG)

       IF (MAPIMAG) THEN
C         USE TABLE TO MAP OLD IMAGE TO NEW IMAGE VALUES

20        CALL OPFIL(.TRUE.,IMFILE,LUNIM,'O',NSAM,NROW,NSLICE,
     &       IDUM,'CLUSTER INPUT',.FALSE.,IRTFLG)
          IF (IRTFLG .EQ. -1) THEN
             CLOSE (LUND)
             GOTO 10
          ENDIF
          IF (IRTFLG .NE. 0) GOTO 999

30        CALL OPFIL(.TRUE.,OUTFILE,LUNOUT,'U',NSAM,NROW,NSLICE,
     &       IDUM,'CLUSTER OUTPUT',.FALSE.,IRTFLG)
          IF (IRTFLG .EQ. -1) THEN
             CLOSE (LUND)
             CLOSE (LUNIM)
             GOTO 20
          ENDIF
          IF (IRTFLG .NE. 0) GOTO 999

          NREC2 = NROW * NSLICE
          CALL MAPIM(LUNIM,LUNOUT,NSAM,1,NREC2,TABLE,NDUM,BUF,IRTFLG)

       ELSEIF (DOCRES) THEN
C         SAVE RESULTS IN A DOC FILE
          DO I = 1,NVAL
            IF (TABLE(I) .GT. 0) THEN
               DLIST(1) = I
               DLIST(2) = TABLE(I)
               DLIST(3) = DBUF(3,I)
               DLIST(4) = DBUF(4,I)
               DLIST(5) = DBUF(5,I)
               CALL SAVD(LUND2,DLIST,5,IRTFLG)

               IF (DEBUG) THEN
C                 WRITE STATISTICS ON STANDARD OUTPUT
                  WRITE(NOUT,901) I-1,INUM,(DLIST(J),J=3,5)
901               FORMAT(2I10,3G12.3)
               ENDIF
            ENDIF
         ENDDO
         CLOSE(LUND2)                 
       ENDIF
      
       IRTFLG = 0 

 999   CLOSE(LUNIM)
       CLOSE(LUNOUT)
       CLOSE(LUND)
         
       RETURN
       END

