
C++*************************************************************************
C
C MAPDOC.FOR -- CREATED NOV 90 
C **************************************************************************
C *  AUTHOR:  ArDean Leith
C *  COPYRIGHT (C)1990, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C    MAPDOC(IRTFL)
C
C    PURPOSE:       READS A DOC FILE FOR MAPPING AN IMAGE FILE INTO
C                   A NEW IMAGE FILE. DOC. FILE CONTAINS X,Y & Z 
C                   LOCATIONS OF A VOXEL IN THE IMAGE FILE.  ALL
C                   VOXELS IN THE FILE HAVING THE CLUSTER NUMBER
C                   FOUND AT THIS LOCATION ARE CARRIED INTO A NEW
C                   FILE.  ALL OTHER VOXELS ARE SET TO ZERO. 
C
C    PARAMETERS     IRTFLG       ERROR RETURN
C
C    CALLS:         OPENFB    FILERD     UNSDAL    MAPIM
C                   REDLIN      
C
C    CALLED BY:     
C
C        0         2         3         4         5         6         7     
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*********************************************************************

       SUBROUTINE MAPDOC(IRTFLG)
        INCLUDE 'CMBLOCK.INC'
       PARAMETER  (NSAMAX=16000)
       PARAMETER  (MAXREG=7)
       PARAMETER  (MAXKEY=4000)
       PARAMETER  (MAXLUT=4000)

       COMMON BUF(NSAMAX),TABLE(MAXLUT)

       COMMON /DOC_BUF/ DBUF(MAXREG,MAXKEY)
       DIMENSION     PLIST(7)
 
       CHARACTER *81 DOCNAM,IMFILE,OUTFILE
       CHARACTER *1  NULL

       DATA LUND/20/,LUNIM/21/,LUNOUT/22/
	NULL=CHAR(0)
       IRTFLG = 1

10     CALL FILERD(DOCNAM,NLETD,NULL,'LOCATION DOC',IRTFLG)
       IF (IRTFLG .EQ. -1) RETURN

       NLIST = 4
       IKEY  = 1
       CALL FILCAN(DOCNAM,NLET,NULL,NULL,DOCNAM,DATEXC,IRTFLG)
       CALL UNSDAL(DOCNAM,ICALL,LUND,IKEY,PLIST,NLIST,DBUF,
     &             MAXKEY,MAXREG,NKEY,IERR)
c***       IF (IERR .NE. 0) GOTO 10

20      CALL OPFIL(.TRUE.,IMFILE,LUNIM,'O',NSAM,NROW,NSLICE,
     &     IDUM,'CLUSTER INPUT',.FALSE.,IRTFLG)
        IF (IRTFLG .EQ. -1) THEN
           CLOSE (LUND)
           GOTO 10
        ENDIF
        IF (IRTFLG .NE. 0) GOTO 999

30      CALL OPFIL(.TRUE.,OUTFILE,LUNOUT,'U',NSAM,NROW,NSLICE,
     &     IDUM,'CLUSTER OUTPUT',.FALSE.,IRTFLG)
        IF (IRTFLG .EQ. -1) THEN
           CLOSE (LUND)
           CLOSE (LUNIM)
           GOTO 20
        ENDIF
        IF (IRTFLG .NE. 0) GOTO 999

C      ZERO THE LOOK-UP-TABLE
       DO N = 1,MAXLUT
          TABLE(N) = 0
       END DO

       IRECL   = 0
       NEWVAL  = 0
       MAPINGS = 0
       NOMAP   = 0
  
       DO 100 N = 1,NKEY

C        CHECK TO SEE IF THIS LINE OF DOC FILE IS IN USE
         IVAL   = DBUF(1,N) 
         IF (IVAL .LE. 0) GOTO 100

         MAPINGS = MAPINGS + 1

C        FIND LOCATION OF VOXEL TO BE MAPPED
         IX     = DBUF(2,N) + 1
         IY     = DBUF(3,N) + 1
         ISLICE = DBUF(4,N)

C        IGNORE ANY SLICES NOT IN THE FILE
         IF (ISLICE .GT. NSLICE) GOTO 100

C        FIND RECORD NUMBER FOR THIS VOXEL
         IREC = (ISLICE -1) * NROW + IY
         IF (IREC .NE. IRECL) THEN
C            MUST READ A NEW RECORD OF THE IMAGE FILE
             CALL REDLIN(LUNIM,BUF,NSAM,IREC)
             IRECL = IREC
         ENDIF

C        FIND VALUE OF THE VOXEL IN OLD FILE
         IGOT = BUF(IX)

         IF (IGOT .GT. MAXLUT) THEN
C           OVERFLOW OF LUT TABLE POSSIBLE
            WRITE(NOUT,*) ' *** TABLE OVERFLOW IN DOCMAP'
            GOTO 999

         ELSEIF (IGOT .EQ. 0) THEN
C           UNMAPPABLE CLUSTER 
            NOMAP = NOMAP + 1

         ELSEIF (TABLE(IGOT) .EQ. 0.0) THEN
C           THIS CLUSTER NOT YET MAPPED TO ANYTHING

            NEWVAL = NEWVAL + 1
            TABLE(IGOT) = NEWVAL
         
         ENDIF
100    CONTINUE

c****************debug
       DO I = 1,NEWVAL
         IF(TABLE(I) .NE. 0.0) THEN
           IT = TABLE(I)
           WRITE(NOUT,*) I,' ->',IT
         ENDIF
       END DO
C******************************

       WRITE(NOUT,96) MAPINGS,NEWVAL
96     FORMAT(' MAPS:',I7,' REMAPPED VALUES:',I7)

       WRITE(NOUT,97) NOMAP
97     FORMAT(' UNMAPPED VALUES:',I7)

       NREC2 = NROW * NSLICE
       CALL MAPIM(LUNIM,LUNOUT,NSAM,1,NREC2,TABLE,NDUM,BUF,IRTFLG)
      
       IRTFLG = 0 
       
999    CLOSE(LUNIM)
       CLOSE(LUNOUT)
       CLOSE(LUND)
         
       RETURN
       END



