
C++*************************************************************************
C
C MAPFILT.FOR -- CREATED NOV 90 
C **************************************************************************
C *  AUTHOR:  ArDean Leith
C *  COPYRIGHT (C)1990, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C    MAPFILT(IRTFL)
C
C    PURPOSE:       READS A DOC. FILE CONTAINING CLUSTER NUMBER +1 (KEY)
C                   AND NUMBER OF VOXELS (REG 1) IN THE CLUSTER.  IF 
C                   NUMBER OF VOXELS < MIN OR > NMAX THEN THE VOXELS
C                   BELONGING TO THIS CLUSTER ARE SET TO ZERO.
C
C    PARAMETERS     IRTFLG       ERROR RETURN
C
C    CALLS:         OPFIL       FILERD     UNSDAL    MAPIM    REDLIN      
C
C    CALLED BY:     UTIL6
C
C        0         2         3         4         5         6         7     
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*********************************************************************

       SUBROUTINE MAPFILT(IRTFLG)

       INCLUDE 'CMBLOCK.INC'
       PARAMETER  (NSAMAX=16000)
       PARAMETER  (MAXREG=7)
       PARAMETER  (MAXKEY=16000)
       PARAMETER  (MAXLUT=16000)

       COMMON BUF(NSAMAX),TABLE(MAXLUT)

       COMMON /DOC_BUF/ DBUF(MAXREG,MAXKEY)
       DIMENSION     PLIST(7)

       LOGICAL       USEMAX ,DEBUG
       CHARACTER *81 DOCNAM,IMFILE,OUTFILE
       CHARACTER *1  NULL

       DATA LUND/20/,LUNIM/21/,LUNOUT/22/
       NULL=CHAR(0)
       IRTFLG = 1
       DEBUG  = .FALSE.

10     CALL FILERD(DOCNAM,NLETD,NULL,'CLUSTER DOC',IRTFLG)
       IF (IRTFLG .EQ. -1) RETURN

       NLIST = 2
       IKEY  = 1
       CALL FILCAN(DOCNAM,NLET,NULL,NULL,DOCNAM,DATEXC,IRTFLG)
       CALL UNSDAL(DOCNAM,ICALL,LUND,IKEY,PLIST,NLIST,DBUF,
     &             MAXKEY,MAXREG,NKEY,IERR)

       WRITE(NOUT,*) ' KEYS FOUND:', NKEY
       IF (NKEY .GT. MAXLUT) THEN
          ITEMP = MAXLUT
          WRITE(NOUT,*) '*** TOO MANY KEYS FOR TABLE SIZE:',ITEMP
          CALL ERRT(100,'MAPFILT',NE)
          RETURN
       ENDIF

20     CALL OPFIL(.TRUE.,IMFILE,LUNIM,'O',NSAM,NROW,NSLICE,
     &     IDUM,'CLUSTER INPUT',.FALSE.,IRTFLG)
       IF (IRTFLG .EQ. -1) THEN
           CLOSE (LUND)
           GOTO 10
       ENDIF
       IF (IRTFLG .NE. 0) GOTO 999

30     CALL OPFIL(.TRUE.,OUTFILE,LUNOUT,'U',NSAM,NROW,NSLICE,
     &     IDUM,'CLUSTER OUTPUT',.FALSE.,IRTFLG)
       IF (IRTFLG .EQ. -1) THEN
           CLOSE (LUND)
           CLOSE (LUNIM)
           GOTO 20
       ENDIF
       IF (IRTFLG .NE. 0) GOTO 999

       USEMAX = .FALSE.
       NMAX   = 0
       CALL RDPRIS(NMIN,NMAX,NLOOP,ILOOP,
     &   'RANGE OF CLUSTER SIZES RETAINED:',IRTFLG)
       IF (IRTFLG .EQ. -1) THEN
          CLOSE(LUNOUT)
          GOTO 30
       ENDIF
       IF (NMAX .GT. NMIN) USEMAX = .TRUE.

C      SET THE LOOK-UP-TABLE SO THAT ALL VALUES ARE RETAINED (DEFAULT)
       DO ICLUS = 1,MAXLUT
          TABLE(ICLUS) = ICLUS 
       END DO

       NEWVAL  = 0
       MAPINGS = 0
       NOMAP   = 0
       NUNDER  = 0
       NOVER   = 0
 
       DO  100 ICLUSP1 = 1,NKEY

C        CHECK TO SEE IF THIS LINE OF DOC FILE IS IN USE
         ILIST   = DBUF(1,ICLUSP1) 
         IF (ILIST .LE. 0) GOTO 100

         MAPINGS = MAPINGS + 1

C        FIND NUMBER OF VOXELS IN CLUSTER
         INUM   = DBUF(2,ICLUSP1) 
         ICLUS  = ICLUSP1 - 1

         IF (INUM .LT. 0) THEN
C           IMPOSSIBLE NUMBER OF VALUES IN CLUSTER
            WRITE(NOUT,*) ' *** INUM :',INUM,' CAN NOT BE < 0'
            NOMAP = NOMAP + 1
            GOTO 999

         ELSEIF (INUM .LT. NMIN) THEN
C           UNDER MINIMUM VALUES NEEDED PER CLUSTER, DISCARD CLUSTER
            NUNDER = NUNDER + 1
            IF (ICLUS .GT. 0) TABLE(ICLUS) = 0.0

         ELSEIF (USEMAX .AND. INUM .GT. NMAX) THEN
C           OVER MAXIMUM VALUES ALLOWED PER CLUSTER, DISCARD CLUSTER
            NOVER = NOVER + 1
            IF (ICLUS .GT. 0) TABLE(ICLUS) = 0.0
         ENDIF
100    CONTINUE

c****************debug
       IF (DEBUG) THEN
          DO I = 1,NKEY
            IF( TABLE(I) .NE. 0.0) THEN
              IT = TABLE(I)
              WRITE(NOUT,*) I,' ->',IT
            ENDIF
          END DO
       ENDIF
C******************************

       WRITE(NOUT,96) MAPINGS
96     FORMAT(/,' MAPS:',I7)

       WRITE(NOUT,97) NMIN,NUNDER
97     FORMAT(' DISCARDED VALUES <',I7,' =',I7)

       IF (USEMAX) THEN
         WRITE(NOUT,98) NMAX,NOVER
98       FORMAT(' DISCARDED VALUES >',I7,' =',I7)
       ENDIF

       NREC2 = NROW * NSLICE
       CALL MAPIM(LUNIM,LUNOUT,NSAM,1,NREC2,TABLE,NDUM,BUF,IRTFLG)
      
       IRTFLG = 0 
999    CLOSE(LUNIM)
       CLOSE(LUNOUT)
       CLOSE(LUND)
         
       RETURN
       END



