
C++*************************************************************************
C
C MHELP_SUB.FOR        
C
C	SSHELP.FOR HAS BEEN CHANGED  3/31/93  MAHIEDDINE LADJADJ
C
C **************************************************************************
C    AUTHOR : MAHIEDDINE LADJADJ
C *  COPYRIGHT (C)1985 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C    MHELP_SUB(FILNM,LUNDOC,JPROMPT)
C
C    PURPOSE:  READ THE HELP DOCUMENTATION FILE AND DISPLAY IT ON THE 
C              TERMINAL. IF THE JPROMPT EXISTS IN THE HELP FILE,
C              IT WOULD BE ENTERED ON A LINE AS:
C                        .JPROMPT
C	       LOCATE IT AND DISPLAY FILE STARTING FROM THAT LINE.
C
C    PARAMETERS:   FILNM    DOCUMENT FILENAME WITH EXTENSION
C                  LUNDOC   UNIT TO BE USED FOR OPENING THE DOCUMENT FILE
C                  JPROMPT  SEARCH STRING USED IN FILE POSITIONING
C
C    CALLED BY:    MHELP
C
C    CALLS:        
C
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*******************************************************************

        SUBROUTINE MHELP_SUB(FILNM,LUNDOC,JPROMPT)
        
        COMMON /UNITS/ LUNC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
       
        CHARACTER *(*)  FILNM, JPROMPT
        CHARACTER * 80  LINE
        CHARACTER * 1   ANSW,FF, NULL
        INTEGER         NLIN, NBAC, NA, DEL_DOT
        LOGICAL         LOOP,EX

        NULL    = CHAR(0)
        FF      = CHAR(12)
        NLIN    = 22
        NBAC    = 2 * NLIN - 2
        DEL_DOT = 0


C       SEE IF HELP FILE EXISTS
        INQUIRE(FILE=FILNM,EXIST=EX)
        IF (.NOT. EX) THEN
           WRITE(NOUT,101) FILNM
  101      FORMAT(' *** NON-EXISTANT HELP FILE: ',A,/)
           RETURN
        ENDIF
                         
C       OPEN THE HELP FILE
        OPEN(UNIT=LUNDOC,FILE=FILNM,STATUS='OLD',IOSTAT=IOS)
        IF (IOS .NE. 0) THEN
C          REPORT ERROR AND RETURN
           WRITE(NOUT,1011) FILNM
 1011      FORMAT(' *** ERROR OPENING HELP FILE: ',A,/)
           RETURN
        ENDIF


C	THE HELP FILE EXISTS AT THIS POINT. LET US SEE IF
C	THE JPROMPT IS USED IN THE FILE.
C	READ UNTIL FOUND OR UNTIL END OF FILE.
                         
        NA = LEN(JPROMPT)
        IF (NA .GT. 10) NA = 10

C       DELETE TRAILING BLANKS FROM JPROMPT
        NA = lnblnk(JPROMPT) 
        IF (NA .GT. 10) NA = 10

        IF (NA .LE. 0) THEN
            WRITE(NOUT,*) '*** BAD PROMPT FOR HELP FILE'
            RETURN
        ENDIF

C       CHECK IF FIRST CHAR. IN JPROMPT IS A DOT '.'
        IF(JPROMPT(1:1) .EQ. '.') DEL_DOT = 1

 310    LOOP = .TRUE.                   
	DO WHILE (LOOP) 
          READ(LUNDOC,102,END=200) LINE

C	  ONE LINE READ, CHECK IF IT IS THE PROMPT ?

          IF (INDEX(LINE,JPROMPT(DEL_DOT:NA)) .GT. 0) THEN

C	      JPROMPT EXISTS IN HELP FILE. BACKSPACE TO THAT 
C             LINE AND DISPLAY A SCREEN FULL.

              BACKSPACE(UNIT=LUNDOC,ERR=5101)
              WRITE(NOUT,106) FF
             
              GOTO 5101

           ENDIF
  180   ENDDO

C	END OF FILE (NO JPROMPT FOUND)
  200   REWIND (LUNDOC)

        WRITE(NOUT,106) FF
  106   FORMAT(1X,A1)

        WRITE(NOUT,90)
  90    FORMAT(//' <<<<<<<<<<<<<<< START OF HELP FOR THIS SECTION ',
     &               ' >>>>>>>>>>>>>>>',/)

       
 5101   CONTINUE
C       READ  LINES FROM THE FILE AND DISPLAY ON THE TERMINAL
C       DISPLAY ONE SCREEN FULL OF THE FILE HELP.

        DO I = 1, NLIN
           READ(LUNDOC,102,END=6001) LINE
  102      FORMAT(A)

C          DELETE TRAILING BLANKS FROM LINE
           NA = lnblnk(LINE) 

           WRITE(NOUT,103) LINE(1:NA)
  103      FORMAT(1X,A)
        ENDDO

C       GET THE REPLY FROM USER
        WRITE(NOUT,*) ' '
        WRITE(NOUT,222)
 222    FORMAT('PRESS (B)ackup, (C)ontinue, (E)xit, (S)tart-over :')
        READ(NIN,223) ANSW
 223    FORMAT(A)
 

C       PROCESS ANSWER
        IF ((ANSW .EQ. 'C') .OR. (ANSW .EQ. 'c') .OR.
     &      (ANSW .EQ. NULL).OR. (ANSW .EQ. ' ')) THEN
             
           GOTO 5101

        ELSEIF (ANSW .EQ. 'B' .OR. ANSW .EQ. 'b') THEN

C          SHOW PREVIOUS SCREEN OF THE HELP FILE
C          BACKSPACE NBAC LINES

           DO  I = 1, NBAC
              BACKSPACE(UNIT=LUNDOC,ERR=5101)
           ENDDO
           GOTO 5101

        ELSEIF (ANSW .EQ. 'S' .OR. ANSW .EQ. 's') THEN
            GOTO 200

        ELSEIF (ANSW .EQ. 'E' .OR. ANSW .EQ. 'e' .OR. 
     &          ANSW .EQ. 'Q' .OR. ANSW .EQ. 'q') THEN
            CLOSE(LUNDOC)
            RETURN
        ELSE
            GOTO 5101
        ENDIF  

 6001   WRITE(NOUT,105)
  105   FORMAT(//' <<<<<<<<<<<<<<<<<< END OF HELP FOR THIS SECTION ',
     &               ' >>>>>>>>>>>>>>>',/)

C       GET THE REPLY FROM USER
        WRITE(NOUT,222)
        READ(NIN,223) ANSW
 
        IF ((ANSW .EQ. 'S')  .OR. (ANSW .EQ. 's')) THEN
            GOTO 200

        ELSEIF ((ANSW .EQ. 'B')  .OR. (ANSW .EQ. 'b')) THEN
            GOTO 5101
        ENDIF

        CLOSE(LUNDOC)
        RETURN
        END

