
C++********************************************************************
C
C MODEL.FOR                                      REWRITTEN APRIL 97 al
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C   MODEL(LUN1,NSAM,NROW)
C
C   PURPOSE:  PREPARES TEST PICTURES
C
C   PARAMETERS:
C        LUN1        LOGICAL UNIT NUMBER OF FILE
C        NSAM,NROW   DIMENSIONS OF FILE
C
C23456789012345678901234567890123456789012345678901234567890123456789012
C***********************************************************************

      SUBROUTINE MODEL(LUN1,NSAM,NROW)
#ifdef SP_NT
	use dfport
#endif
      INCLUDE 'CMBLOCK.INC'

      COMMON NX,NY,CX,BUF(4096),A0(2048)
CNO_SAVE
      INTEGER    IRTFLG,NVAL
      DIMENSION  NX(20),NY(20),CX(20),PH(20), FWA(4)

      DATA AMIN/-10E30/,AMAX/10E30/

C     NEEDS TO BE INTEGER*2 TO EXTRACT CORRECT PORTION OF TIME STRING FOR
C     RANDOM GENERATOR:
      INTEGER*2     NXT(4)
      CHARACTER*8  CNXT
      EQUIVALENCE  (NXT,CNXT)
      CHARACTER    ANS,NULL,GA
      LOGICAL      GFLAG

      DATA TWOPI/6.283184/, MAXSIN/20/

      NULL = CHAR(0)

      WRITE(NOUT,100)
 100  FORMAT(
     &      ' .MENU: (B)LANK    -- CONSTANT DENSITY IMAGE'/
     &      '        (C)IRCLE   -- FILLED CIRCLE'/
     &      '        (Q)UICK    -- RANDOM UNIFORM STATISTICS'/
     &      '        (R)ANDOM   -- RANDOM UNIFORM STATISTICS'/
     &      '        (S)INE     -- SET OF SINE WAVES'/
     &      '        (T)EST     -- 2D SINE WAVE'/
     &      '        (W)EDGE    -- DENSITY WEDGE'/)

1010  CALL RDPRMC(ANS,NC,.TRUE.,'B/C/Q/R/S/T/W',NULL,IRTFLG)
      IF (IRTFLG .NE. 0) RETURN

      IF (ANS .EQ. 'B') THEN
C       BLANK -------------------------------------------------- BLANK
        CALL RDPRM(BACK,NLOOP,ILOOP,'BACKGROUND CONSTANT')
        CALL BLANK(LUN1,NSAM,NROW,BACK)
        RETURN

      ELSEIF (ANS .EQ. 'C' .OR. ANS .EQ. 'W') THEN
C       CIRCLE OR WEDGE ------------------------------- CIRCLE OR WEDGE
        IF (ANS .EQ. 'C') THEN
           CALL RDPRM(RAD,NLOOP,ILOOP,'RADIUS (FLOATING POINT)')
           RAD2 = RAD**2
        ENDIF

        SCF = 2.0/FLOAT(NROW+NSAM)
	DO I=1,NROW
          IF (ANS .EQ. 'C') THEN
C           CIRCLE WANTED
            FI2 = (FLOAT(I-NROW/2-1))**2
            DO  J=1,NSAM
               A0(J) = 0.0
               IF (FI2+FLOAT(J-NSAM/2-1)**2.LT.RAD2) A0(J) = 1.0
            ENDDO 
          ELSE
C           WEDGE WANTED
            FI = FLOAT(I)*SCF
            DO  J = 1,NSAM
               A0(J) = FI+ SCF * FLOAT(J)
            ENDDO
          ENDIF
C         SEND RECORD TO FILE
          CALL WRTLIN(LUN1,A0,NSAM,I)
        ENDDO
        RETURN

      ELSEIF (ANS .EQ. 'Q') THEN
C       PUT RANDOM NUMBERS IN THE IMAGE ------------------------- QUICK
        CALL TIME(CNXT)
	IST   = NXT(3)
        KST   = NXT(4)


	CALL  SRAND(KST)


        GFLAG = .FALSE.
        CALL RDPRMC(GA,NC,.TRUE.,'GAUSSIAN DISTRIBUTION? (Y/N)',
     &             NULL,IRT)
        IF (IRT .NE. 0) GOTO 1010

        IF (GA .EQ. 'Y') THEN
          GFLAG = .TRUE.
          CALL  RDPRM2(XM,SD,NLOOP,ILOOP,
     &        'MEAN AND STANDARD DEVIATION OF GAUSSIAN DIST.')
        ENDIF
        DO I = 1,NROW
          DO  K = 1,NSAM
            IF (GFLAG) THEN
               A0(K) = RANN(XM,SD,KST)
            ELSE
               A0(K) = RAN(IST)
            ENDIF
          ENDDO
          CALL WRTLIN(LUN1,A0,NSAM,I)
        ENDDO
        RETURN

      ELSEIF (ANS .EQ.'R') THEN

C        RANDOM OPTION ----------------------------------------- RANDOM
         CALL TIME(CNXT)
         IST   = NXT(3)
         KST   = NXT(4)
         CALL RDPRMC(GA,NC,.TRUE.,'GAUSSIAN DISTRIBUTION? (Y/N)',
     &               NULL,IRT)
         IF (IRT .EQ. -1) GOTO 1010

         GFLAG = .TRUE.
         IF (GA .EQ. 'N') GFLAG =.FALSE.
         CALL MRAND(OUT,KST,GFLAG,NOUT)
 
C        WARMUP:
         DO  I = 1,2000
            CALL GETRAND(OUT)
         ENDDO

         DO I = 1,NROW
            DO  K = 1,NSAM
              CALL GETRAND(OUT) 
              A0(K) = OUT
            ENDDO
            CALL WRTLIN(LUN1,A0,NSAM,I)
         ENDDO
         RETURN

      ELSEIF (ANS .EQ. 'S' .OR. ANS .EQ. 'T') THEN

C       PUT SINE WAVES OF INTENSITY IN THE IMAGE ----------------- SINE
        IF (ANS .EQ. 'T') THEN

C         ONLY ONE WAVE IS WANTED
          NS    = 1
          CX(1) = 1.
          NX(1) = 2
          NY(1) = 2
          PH(1) = 0.0

        ELSEIF (ANS .EQ. 'S') THEN

C         FIND NUMBER OF SINE WAVES WANTED IN THE IMAGE ---------------

2         CALL RDPRMI(NS,IDUM,NLOOP,ILOOP,'NUMBER OF SINE WAVES')
          IF (NS .GT. MAXSIN) THEN
             WRITE(NOUT,251)MAXSIN
251          FORMAT(' ** BUFFER RESTRICTED TO',I3,' SINE WAVES')
             NS = MAXSIN
          ENDIF

          DO I = 1,NS
9006        IRTFLG = 0
            NVAL   = 4
            CALL RDPRAF(FWA,4,NVAL,AMIN,AMAX,
     $         ' AMPLITUDE, PHASE, SP. FREQUENCY(KX,KY)',NULL,IRTFLG)
            IF (IRTFLG .NE. 0) GOTO 9006

            CX(I) = FWA(1)
            PH(I) = FWA(2)
            ANX   = FWA(3)
            ANY   = FWA(4)
	    
            NX(I) = ANX+0.5
            IF (ANX .LT .0.0) NX(I) = ANX-0.5
            NY(I) = ANY+0.5
            IF (ANY .LT. 0.0) NY(I) = ANY-0.5
            WRITE(NECHO,9008)I,CX(I),PH(I),NX(I),NY(I)
9008        FORMAT(1X,I5,2F8.2,2I6)
            PH(I) = PH(I)*TWOPI/360.0
          ENDDO
        ENDIF

C       PLACE SINE WAVE(S) OF INTENSITY IN THE IMAGE
        DO I=1,NROW
           PHASE = FLOAT(I-1)*TWOPI/FLOAT(NROW)
           DO J=1,NSAM
              A0(J) = 0.0
              DO K = 1,NS
              A0(J)=CX(K)*SIN(FLOAT(J-1)*TWOPI*FLOAT(NX(K))/FLOAT(NSAM)
     1              +PHASE*FLOAT(NY(K))+PH(K))+A0(J)
              END DO
           ENDDO
           CALL WRTLIN(LUN1,A0,NSAM,I)
         ENDDO
      ENDIF
      RETURN

      END
