
C **********************************************************
C   MAIN SUBROUTINE FOR MARKER PROGRAM.
C
C   WARNING !
C   PARAMETERS LS AND LV ARE HARD-WIRED IN ALL THE SUBROUTINES.
C
C **********************************************************
C ATTEMPTS TO ALIGN A DATA SET USING FIDUCIARY MARKERS.
C
C     THE PROCEDURE IS TO FIRST ROUGHLY TRANSLATIONALLY
C     ALIGN THE IMAGES. USING THIS SET, RECONSTRUCT THE
C     COORDS OF THE POINTS IN THE 3D OBJECT. THEN PROJECT
C     THE POINTS BACK DOWN ONTO A PLANE AND COMPARE THEM
C     WITH THE POINTS FROM PREVIOUS TRIAL. CORRECT THE
C     POINTS AND ONCE AGAIN RECONSTRUCT THE 3D, ETC.
C
C     THE CORRECTIONS ACCUMULATE IN A FEW REGISTERS.
C     ALL COMPARISONS EXCEPT FOR THE ROUGH ALIGNMENT ARE
C     BETWEEN THE ORIGINAL PROJECTIONS AND THEIR PSUEDO-
C     PROJECTIONS.
C **********************************************************

	SUBROUTINE  MRK(MAXDIM)

      INCLUDE     'CMBLOCK.INC'

        PARAMETER (LV=300)
        PARAMETER (LS=100)
        PARAMETER (MAXREG=7)
        PARAMETER (MAXKEY=100)
        PARAMETER (MAXLOOP=19999)
        REAL MAXPER
        PARAMETER (MAXPER=1.0e-7)

      LOGICAL*1      FULL
	CHARACTER*1  ASKFULL,NULL
      LOGICAL*1   PTACTIVE(LS,LV),PARAMQ(4)
      COMMON /GENERAL/PTACTIVE,NUMPTS(LV),NTVW,NTPT,CIR(2),PARAMQ

      DIMENSION         XYPTS(2,LS,LV),PRJ(2,LS,LV)
      DIMENSION         ROT(3,3),ANGLE(3,LV),TPSI(LV)
      DIMENSION         P3D(3,LS),SCALE(LV)
      DIMENSION         NUMBER(LV),SCALEI(LV)
      DIMENSION         CVPT(3)
      DIMENSION         SHFT(2),SHIFT(2,LV),TSHIFT(2,LV)
      DIMENSION         ERVW(LV),ERPT(LS)

      LOGICAL*1         FIRST
      INTEGER           NTIMES
	DOUBLE PRECISION  TCH,ERTOT
      DATA PI/3.141592654/
CNO_SAVE
      NTIMES=0
         NULL=CHAR(0)

C     GET COORDS OF MARKERS IN EACH PROJECTION

      CALL MRGETINFO(XYPTS,IREF,NUMBER,ANGLE,SCALE,TSHIFT,FIRST,NTTVL)

      DO  IVIEW=1,NTVW
c        IF(FIRST.and.paramq(4)) then
cc
cC           ROUGH ALIGN THESE PROJECTIONS IF FIRST TIME RUNNING
cc
c      CALL MRCENTI(XYPTS(1,1,IVIEW),CVPT,ptactive(1,IVIEW),ls)
c          SHIFT(1,IVIEW)=-cvpt(1)
c          SHIFT(2,IVIEW)=-cvpt(2)
c        TSHIFT(1,IVIEW)=SHIFT(1,IVIEW)
c        TSHIFT(2,IVIEW)=SHIFT(2,IVIEW)
c	else
        TPSI(IVIEW)=ANGLE(1,IVIEW)
        SHIFT(1,IVIEW)=TSHIFT(1,IVIEW)
        SHIFT(2,IVIEW)=TSHIFT(2,IVIEW)
c	endif
      call MRALIGN(XYPTS(1,1,iview),xypts(1,1,iview),
     &	ANGLE(1,iview),SHIFT(1,iview),SCALE(iview),
     &	PTACTIVE(1,iview),ntpt)
      ENDDO
C
C           GET 3D VOLUME
      CALL MR2TO3D(P3D, XYPTS, ANGLE, ERRX, ERRY, ERRZ)
      CALL MRPROJ(P3D, PRJ, ANGLE)
      CALL MRCALERR(XYPTS,PRJ,ERTOT,ERVW,ERPT)
      WRITE(NOUT,*)'    INITIAL INFO   '
	FULL=.FALSE.
      CALL MRPUTINFO(XYPTS,PRJ,ANGLE,P3D,
     &		NUMBER,TSHIFT,SCALE,ERVW,ERPT,FULL)
C
      PERRX=ERRX/NTPT
      PERRY=ERRY/NTPT
      PERRZ=ERRZ/NTPT
      BOXERR=SQRT(PERRX**2+PERRY**2)
	TCH=ERTOT
      WRITE(NOUT,701)NTIMES,BOXERR,ERTOT
c      WRITE(NDAT,701)NTIMES,BOXERR,ERTOT
 701  FORMAT(' PASS #',I5,' BERR= ',G10.4,' TOTCHANGE= ',10('*'),
     &      ' ERTOT= ',G11.4)
 700  FORMAT(' PASS #',I5,' BERR= ',G10.4,' TOTCHANGE= ',G10.3,
     &      ' ERTOT= ',G11.4)
C
C  ** MAIN LOOP **
C
C           CREATE PSUEDO-PROJECTIONS AND COMPARE TO LAST RUN
 200  NTIMES=NTIMES+1
c
c  do 210 only if _in plane_ rotation or _scale_ corrections requested.
c
	if(paramq(1).or.paramq(3))  then
	SHFT(1)=0.0
	SHFT(2)=0.0
      DO 210 JVIEW=0,NTVW
c this strange trick is to make sure that first pass is over reference
c image, in subsequent iterations reference image is skipped.
        IVIEW=JVIEW
        IF(JVIEW.EQ.IREF) GOTO 210
        IF(JVIEW.EQ.0) IVIEW=IREF
C
C        ASSUMPTION IS THAT TILT OF PROJ AND TILT OF ORIG ARE EQUAL
C        THIS IS THE ONLY REASON WE CAN REALLY COMPARE THE TWO
C
C        CALL MRQUATER(RPT,VPT,IVIEW,ROT)
C        CALL MRROTATE(ROT,PHI2,THETA,PSI)
c
c  Find psi (in plane) angle
c
        IF(PARAMQ(1)) then
        CALL MRANG2(prj(1,1,iview),xypts(1,1,iview),IVIEW,PSI)
	else
	PSI=0.0
	endif
c
c  Find scale
c
        IF(PARAMQ(3)) THEN
         CALL MRSCALE(prj(1,1,iview),xypts(1,1,iview),IVIEW,SCALEI)
         SCAL=SCALEI(IVIEW)
         IF(IVIEW.EQ.IREF) SCAD=1/SCALEI(IVIEW)
         SCALEI(IVIEW)=SCALEI(IVIEW)*SCAD
        ELSE
	 SCAL=1.0
         SCALEI(IVIEW)=1.0
        ENDIF
C           UPDATE CORRECTION INFO IN REGISTERS, no shifts at this moment
        C=COS(PSI)
        S=SIN(PSI)
	qt=tshift(1,iview)
        TSHIFT(1,IVIEW)=SCALEI(IVIEW)*(qt*C+TSHIFT(2,IVIEW)*S)
        TSHIFT(2,IVIEW)=SCALEI(IVIEW)*(-qt*S+TSHIFT(2,IVIEW)*C)
        TPSI(IVIEW)=PSI+TPSI(IVIEW)
        SCALE(IVIEW)=SCALE(IVIEW)*SCALEI(IVIEW)
        ANGLE(1,IVIEW)=PSI
c  apply corrections to points with SHFT set to zero ...
      call MRALIGN(XYPTS(1,1,iview),XYPTS(1,1,iview),
     &	PSI,SHFT,SCALEI(iview),PTACTIVE(1,iview),ntpt)
 210  CONTINUE
c now goto 3D
	CALL MR2TO3D(P3D, XYPTS, ANGLE, ERRX, ERRY, ERRZ)
	endif
c
c  Find translation (shift)
c
	IF(PARAMQ(4)) then
c
c  find center of gravity of points in 3D
c  and apply any shifts found to these points
c
	CALL MRCG3D(P3D)
c goto 2D
	CALL MRPROJ(P3D, PRJ, ANGLE)
c find the shift between points and shift XYPTS
	CALL MRSHIFT(PRJ,XYPTS,SHIFT)
c update parameters
	do    iview=1,ntvw
        TSHIFT(1,IVIEW)=TSHIFT(1,IVIEW)+shift(1,iview)
        TSHIFT(2,IVIEW)=TSHIFT(2,IVIEW)+shift(2,iview)
	enddo
c
c goto 3D again
	CALL MR2TO3D(P3D, XYPTS, ANGLE, ERRX, ERRY, ERRZ)
	endif
c
c  Find theta (tilt) angle
c
        IF(PARAMQ(2)) THEN
c goto 2D
	CALL MRPROJ(P3D, PRJ, ANGLE)
      DO 212 JVIEW=0,NTVW
c this strange trick is to make sure that first pass is over reference
c image, in subsequent iterations reference image is skipped.
c won't modify XYPTS, only THETA
        IVIEW=JVIEW
        IF(JVIEW.EQ.IREF) GOTO 212
        IF(JVIEW.EQ.0) IVIEW=IREF
        THETA = ANGLE(2,IVIEW)
c	print  *,iview,theta
c set scale=1.0 and psi=0.0
        CALL MRTHETA(prj(1,1,iview),xypts(1,1,iview),IVIEW,P3D,THETA)
        IF(IVIEW.EQ.IREF) THAD = THETA
        THETA=THETA-THAD
cc correction of theta is found, so calculate new theta
c        ANGLE(2,IVIEW)=ANGLE(2,IVIEW)+THETA
        ANGLE(2,IVIEW)=THETA
212	continue
c
c goto 3D again
	CALL MR2TO3D(P3D, XYPTS, ANGLE, ERRX, ERRY, ERRZ)
        ENDIF
c
c  goto 2D and get the error ...
      CALL MRPROJ(P3D, PRJ, ANGLE)
      CALL MRCALERR(XYPTS,PRJ,ERTOT,ERVW,ERPT)
C
      PERRX=ERRX/NTPT
      PERRY=ERRY/NTPT
      PERRZ=ERRZ/NTPT
      BOXERR=SQRT(PERRX**2+PERRY**2)
	TTCH=SNGL((TCH-ERTOT)/TCH)
c      IF(MOD(NTIMES,10).EQ.0) THEN
c        WRITE(NOUT,700)NTIMES,BOXERR,TTCH,ERTOT
c        WRITE(NDAT,700)NTIMES,BOXERR,TTCH,ERTOT
c      ENDIF
	TCH=ERTOT
      IF(MOD(NTIMES,150).EQ.0)  then 
        WRITE(NOUT,700)NTIMES,BOXERR,TTCH,ERTOT
	FULL=.FALSE.
     	CALL MRPUTINFO(XYPTS,PRJ,ANGLE,P3D,
     &		NUMBER,TSHIFT,SCALE,ERVW,ERPT,FULL)
	endif
c
c  do the next iteration ?
c
c      IF(TTCH.GT.MAXPER
	IF(ABS(TTCH).GT.MAXPER
     &			.AND. NTIMES.LT.MAXLOOP) GOTO 200
c
      IF(NTIMES.GE.MAXLOOP) THEN
            WRITE(NOUT,*)'  EXCEEDED LOOP LIMIT, ENDING CYCLE'
c            WRITE(NDAT,*)'  EXCEEDED LOOP LIMIT, ENDING CYCLE'
      ENDIF
C
C           JUST DO SOME CLEANING UP BEFORE PUTTING OUT VALUES
	TILR=ANGLE(2,IREF)
c      XSH=TSHIFT(1,IREF)
c      YSH=TSHIFT(2,IREF)
      DO  IVIEW=1,NTVW
        ANGLE(1,IVIEW)=TPSI(IVIEW)
        ANGLE(2,IVIEW)=ANGLE(2,IVIEW)-TILR
c  this is done to put tilt axis at (0,0) of reference view
c        TSHIFT(1,IVIEW)=TSHIFT(1,IVIEW)-(XSH*COS(ANGLE(2,IVIEW)))
c        TSHIFT(2,IVIEW)=TSHIFT(2,IVIEW)-YSH
      ENDDO
c
	FULL=.TRUE.
        WRITE(NOUT,700)NTIMES,BOXERR,TTCH,ERTOT
	CALL  RDPRMC(ASKFULL,NUMC,.TRUE., 'Full output (Y/N)',NULL,IRT)
	IF(ASKFULL.EQ.'Y'.OR.ASKFULL.EQ.'y') THEN
	FULL=.TRUE.
	ELSE
	FULL=.FALSE.
	ENDIF
      CALL MRPUTINFO(XYPTS,PRJ,ANGLE,P3D,
     &		NUMBER,TSHIFT,SCALE,ERVW,ERPT,FULL)
      CALL MRDOC(SCALE,TSHIFT,ANGLE,NUMBER,P3D)
c
      END
