C++************************************************************************
C
C   MYMODS.F
C
C   A SHELL FOR USER SUPLIED SUBROUTINES, INCLUDES A COMMENTED SAMPLE
C 
C--************************************************************************

	SUBROUTINE MYMODS(MAXMEM)

C       INCLUDE COMMONS WITH SYSTEM INFORMATION      
	INCLUDE 'CMBLOCK.INC'
 
C       ACTUAL LENGTH OF UNLABELED COMMON (MAXMEM) IS SET IN INILUN.F
	COMMON          BUF(1) 

	DIMENSION       Q(1)
	POINTER         (IPQ,Q) 
	LOGICAL         RUNTIME 
	CHARACTER *81   FILNAM

	DATA            LUN1,LUN2/8,9/

C       NO_SAVE SIGNALS Makefile TO USE NON-STATIC COMPILATION OPTION
CNO_SAVE

C       FILE OPENING PART --------------------------------------------

C       OPEN THE INPUT FILE
C       NSAM - X-DIMENSION, NROW - Y-DIMENSION, NSLICE - Z-DIMENSION
C       'O' - INDICATES EXISTING FILE, .FALSE. IS FOURIER INPUT NOT OK

 	CALL OPFIL(.TRUE.,FILNAM,LUN1,'O',NSAM,NROW,NSLICE,IDUMMY,
     &               'INPUT',.FALSE.,IRTFLG)

C       CHECK WHETHER THE FILE OPENED OK, IRTFLG<>0 INDICATES IT WAS NOT
	IF (IRTFLG .NE. 0)  GOTO 999

C       OPEN THE OUTPUT FILE, SAME DIMENSIONS AS INPUT FILE
C       'UNKNOWN' TELLS THE PROGRAM TO OVERWRITE ANY EXISTING FILE

 	CALL OPFIL(.TRUE.,FILNAM,LUN2,'U',NSAM,NROW,NSLICE,IDUMMY,
     &               'OUTPUT',.FALSE.,IRTFLG)

C       CHECK WHETHER THE FILE OPENED OK, IRTFLG<>0 INDICATES IT WAS NOT
	IF (IRTFLG .NE. 0)  GOTO 999

C       MEMORY ALLOCATION PART ---------------------------------------

C       THIS OPERATION NEEDS TWO VOLUMES, EACH NSAM*NROW*NSLICE LARGE.
	K1      = 1
	K2      = K1 + NSAM * NROW * NSLICE
	MEMWANT = K2 + NSAM * NROW * NSLICE

C       CHECK TO SEE IF SUFFICIENT MEMORY AVAILABLE IN UNLABELED COMMON
C       IF SO RETURN ADDRESS OF BUF IN IPQ FOR ARRAY Q
C       IF NOT, USE RUNTIME ALLOCATION FOR ARRAY Q
	CALL CHKMEM(MEMWANT,BUF,MAXMEM, RUNTIME,IPQ,IRTFLG)
	IF (IRTFLG .NE. 0) THEN 
C          FAILED TO HAVE OR ALLOCATE SUFFICIENT MEMORY
	   CALL ERRT(6,'MYMODS',NE)
	   GOTO  999
	ENDIF

C       SPECIFIC PROCESSING PART -------------------------------------

C       SOLICIT FLOATING POINT NUMERICAL PARAMETERS USED BY USONE 
        CALL  RDPRM2(CW,SW,NLOOP,ILOOP,'CENTRAL WEIGHT, SIDE WEIGHT')

C       CALL THE SUBROUTINE THAT WILL DO THE ACTUAL PROCESSING
        CALL  USONE(Q(K1),LUN1,Q(K2),LUN2,NSAM,NROW,NSLICE,CW,SW)

C       CLEANUP ------------------------------------------------------

C       DEALLOCATE MEMORY
        IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'MYMODS',IRTFLG) 

999     CLOSE(LUN1)
        CLOSE(LUN2)
        RETURN

        RETURN
	END




C       ----------------------------------------------------------


	SUBROUTINE  USONE(BIN,LUN1,BOU,LUN2,NSAM,NROW,NSLICE,CW,SW)

	DIMENSION  BIN(NSAM,NROW,NSLICE),BOU(NSAM,NROW,NSLICE)
CNO_SAVE
 
C       THIS PROGRAM APPLIES 1D MOVING AVERAGE IN X-DIRECTION (NSAM)
C       THE RESULT IS "DIRECTIONAL" FILTRATION IN ONE DIMENTION

C       BOU(I,*,*) = (CW*BIN(I,*,*) + SW * (BIN(I-1,*,*) +
C                     BIN(I+1,*,*))) / (CW+SW+SW)

C       THIS PROGRAM WORKS FOR BOTH 2D AND 3D FILES.  THIS IS DUE TO 
C       THE WAY 3D FILES ARE ORGANIZED IN SPIDER - THEY HAVE NSLICE 
C       2D SLICES. A 2D FILE IS A 3D FILE WITH NSLICE=1.

C       READ INPUT VOLUME
        DO K=1,NSLICE
           DO J=1,NROW
              IREC=J+(K-1)*NROW
              CALL  REDLIN(LUN1,BIN(1,J,K),NSAM,IREC)
           ENDDO
        ENDDO


C       DEFINE CENTER OF THE VOLUME ACCORDING TO SPIDER CONVENTION, NOT
C       USED IN THIS PROGRAM, JUST AN EXAMPLE.
C	NXC = NSAM   / 2 + 1
C	NYC = NROW   / 2 + 1
C	NZC = NSLICE / 2 + 1

	Q = 1.0 / (CW + SW + SW)
C       DO THE PROCESSING

C       FOLLOWING LINE IS A COMPILER PARALLEL DIRECTIVE FOR SGI MP.
C$DOACROSS LOCAL(I,J,K)

	DO K=1,NSLICE
	   DO J=1,NROW
C            NEXT TWO LINES TAKE CARE OF BORDER EFFECT.  IF THE IMAGE 
C            IS NOT TO BE TREATED AS CIRCULARLY CLOSED, THEY SHOULD 
C            BE COMMENTED OUT.
	     BOU(1,J,K) = (CW * BIN(1,J,K) + SW * (BIN(NSAM,J,K)+
     &                     BIN(2,J,K))) * Q
	     BOU(NSAM,J,K) = (CW*BIN(NSAM,J,K) + SW * (BIN(NSAM-1,J,K)+
     &                       BIN(1,J,K)))*Q

C            HERE IT IS REQUIRED THAT NSAM IS AT LEAST 3.  IT IS 
C            NOWHERE CHECKED, BUT IT CAN BE EASILY ADDED.
	     DO I=2,NSAM-1
                BOU(I,J,K) = (CW * BIN(I,J,K) + SW * (BIN(I-1,J,K) +
     &                     BIN(I+1,J,K))) * Q
             ENDDO
          ENDDO
        ENDDO

C       WRITE OUTPUT VOLUME
        DO K=1,NSLICE
           DO J=1,NROW
              IREC = J + (K - 1) * NROW
              CALL  REDLIN(LUN2,BOU(1,J,K),NSAM,IREC)
           ENDDO
        ENDDO

C       FILES WILL BE CLOSED IN THE CALLING PROGRAM 
        RETURN
	END
