
C++*********************************************************************
C
C NEXTIMG.F   -- NEW 2 December 97 al

C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK        *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND     *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.  *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS *
C *    OR USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN        *
C *    APPROVAL OF THE CENTER FOR LABORATORIES AND RESEARCH            *
C **********************************************************************
C
C    NEXTIMG(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,NDUM,IMGNUM,ITYPE,
C            DISP,IRTFLG)
C
C    PURPOSE:       TO OPEN A SPECIFIED IMAGE WITHIN STACK FOR RANDOM 
C                   ACCESS READING/WRITING.
C
C    PARAMETERS:
C        BUF        WORKING BUFFER FOR READ/WRITE                 (SENT)
C        FILNAM     CHARACTER ARRAY, CONTAINING FILE NAME         (SENT)
C        LUN        LOGICAL UNIT NUMBER FOR FILNAM.               (SENT)
C        NSAM,NROW  DIMENSIONS OF FILE                       (SENT/RET.)
C        NSLICE     DIMENSIONS OF FILE                       (SENT/RET.)
C        IMGNUM     IMAGE NUMBER WANTED AND FOUND            (SENT/RET.) 
C        WANTNEXT   LOGICAL FLAG TO GET NEXT IMAGE >= IMGNUM      (SENT) 
C        ITYPE      FILE TYPE SPECIFIER (=IFORM)                  (SENT) 
C        DISP       SEE OPENFB FOR EXPLANATION                    (SENT) 
C        IRTFLG     ERROR RETURN FLAG.                       (SENT/RET.)
C                   IRTFLG =  0    NORMAL RETURN
C                   IRTFLG = >0    ERROR RETURN
C                   IRTFLG = -1    ERROR RETURN, IMAGE NOT IN USE
C
C     CALLS:
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

	SUBROUTINE NEXTIMG(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,
     &                     IMGNUM,WANTNEXT,ITYPE,DISP,IRTFLG)

        INCLUDE 'CMBLOCK.INC'
        INCLUDE 'LABLOCK.INC'
CNO_SAVE

        COMMON /LUNARA/  LUNARA(100),LUNSTK(100)

        DIMENSION        BUF(*)
        CHARACTER *(*)   FILNAM,DISP
        CHARACTER * 1    DSP
        LOGICAL          WANTNEXT,ISBARE

        DATA NBUFSIZ/17000/

C       IF (DISP == 'Z' AND REDHED FAILS) SHOULD NOT STOP 
        NOSTOP = 0
        IF (DISP(1:1) .EQ. 'Z') NOSTOP = 1

C       SET ERROR RETURN
        IRTFLG = 1

        IF (.NOT. ISBARE(FILNAM)) THEN
C          INPUT IMAGE IS SIMPLE OR A SPECIFIC STACKED IMAGE, RETURN
           IRTFLG = 0
           RETURN
        ENDIF

C       V---- ONLY FOR INLINE OR REGULAR IMAGE STACK NOW ------------V

C       MAKE SURE THE CURRENT STACK OFFSET IS ALWAYS ZEROED
        LUNSTK(LUN) = 0

C       RECOVER OVERALL HEADER FROM THE STACK FILE
        CALL REDHED(LUN,BUF,NBUFSIZ,NSAM,NOSTOP,IRTFLG)
        IF (IRTFLG .NE. 0) RETURN

C       ALL IMAGES HAVE SAME IRECF AS STACK HEADER.
C       GET IRECF & LABRECF FROM STACK FILE HEADER
        IRECF   = BUF(3)
        LABRECF = BUF(13)

	IF (DISP(1:1) .EQ. 'U' .OR. DISP(1:1) .EQ. 'N') THEN
C          WANT TO MAKE A NEW IMAGE WITHIN EXISTING STACK ----------NEW

C          CHECK TYPE OF FILES IN THIS STACK
           IFORMOLD = BUF(5)
           IF (ITYPE .NE. IFORMOLD) THEN
C             EXISTING STACK FILE FORMAT NOT SAME AS IMAGE FORMAT
              CALL ERRT(101,
     &            'EXISTING STACK FILE FORMAT NOT COMPATIBLE',NE)
              RETURN
           ENDIF

C          SET OFFSET FOR THIS IMAGE INTO LUNSTK
           LUNSTK(LUN) = (IMGNUM-1) * IRECF + LABRECF

C          SET IMAGE SPECIFIC LOCATIONS IN IMAGE HEADER
           IMAMI   = 0
           BUF(6)  = IMAMI
           IANGLE  = 0
           BUF(14) = IANGLE
           KANGLE  = 0
           BUF(30) = KANGLE
           BUF(24) = 2        
           IMUSED  = 1
           BUF(25) = IMUSED         

C          PUT CURRENT DATE AND TIME INTO BUF
           CALL DATTIMTIT(BUF,.FALSE.,.TRUE.,LENTIT,IRTFLG)

C          PLACE THIS IMAGE HEADER INTO PROPER STACK LOCATION
           CALL WRTHED(LUN,BUF,NSAM,IRTFLG)

           DSP = 'N'

	ELSEIF (DISP(1:1) .EQ. 'O' .OR. DISP(1:1) .EQ. 'K' .OR.
     &          DISP(1:1) .EQ. 'Z' .OR. 
     &          DISP(1:1) .EQ. 'E' .OR. DISP(1:1) .EQ. 'M') THEN
C          WANT AN EXISTING IMAGE FROM AN EXISTING STACK -----------OLD

C          SET OFFSET FOR THIS IMAGE INTO LUNSTK
8888       LUNSTK(LUN) = (IMGNUM-1) * IRECF + LABRECF

C          GET SPECIFIED IMAGE HEADER FROM STACK FILE LOCATION
           CALL REDHED(LUN,BUF,NBUFSIZ,NSAM,NOSTOP,IRTFLG)
           IF (IRTFLG .NE. 0) THEN
C             PROBABLY RAN OFF END OF STACK FILE
              IRTFLG = 2
              RETURN
           ENDIF

           CALL OPENGETHDR(BUF,NDUM,NDUM,NDUM,NDUM,NDUM,
     &                     NDUM,NDUM,NDUM,IMUSED,LENTIT)

           IF (IMUSED .EQ. 0) THEN
C             NO EXISTING IMAGE WITHIN STACK
              IF (WANTNEXT) THEN
C                INCREMENT IMGNUM AND TRY AGAIN
                 IMGNUM = IMGNUM + 1
                 GOTO 8888
              ENDIF

              IF (DISP .NE. 'Z') THEN
                 CALL ERRT(101,'THIS IMAGE NOT USED IN STACK',NE)
              ENDIF
              IRTFLG = -1
              RETURN
           ENDIF

           DSP = 'O'

        ELSE
           CALL ERRT(101,'PGM. ERROR: UNKNOWN DISP IN NEXTIMG',NE)
           RETURN
        ENDIF

C       WRITE OUT FILE OPENING INFO --------------------------------ALL
        NLET   = lnblnk(FILNAM)
        LABBYT = 0
        CALL OPENINFO(FILNAM,NLET,NSAM,NROW,NSLICE,IMGNUM,NSTACK,  
     &                LENTIT,DSP,LABBYT,IRTFLG)

C       SET FLAG FOR NORMAL RETURN	
        IRTFLG = 0
 
        RETURN
	END
