
C                                LONG FILE NAMES FEB 89 al
C
C     SUBROUTINE TO DO THE MINIMISATION BY THE STEEPEST DESCENT
C     ALGORITHM.
C
C     EPS      UPPER BOUND FOR REMAINING ERROR
C     HIDIM    MATRIX CONTAINING THE VECTORS IN THE HIDIMENSIONAL
C              VECTOR SPACE
C     XLO      MATRICES FOR STORING THE TWODIMENSIONAL 
C     XLOC              COORDINATES
C     DISTH    MATRIX FOR STORING THE HIGH-DIMENSIONAL DISTANCES
C     DISTL    MATRIX FOR STORING THE TWO-DIMENSIONAL DISTANCES
C     NHI      DIMENSION OF HIGHDIMENSIONAL SPACE
C     NIM      NUMBER OF IMAGES

C       M. RADERMACHER, 21-OCT-82
C       UPDATED TO I4  

      SUBROUTINE NONLMAP(EPS,HIDIM,XLO,XLOC,DISTH,DISTL,NHI,NIM,
     $      ZAUBF,IL1,IL2,ITMAX,IT,MOD,EZ,EXP,NAME,OUT,LUNS)

      IMPLICIT INTEGER (I,J,K,L,M,N)

      INCLUDE 'CMBLOCK.INC'

      COMMON /DIM1/MAXIM,MAXFAC,MAXLO,NDIST
      INTEGER MAXIM,MAXFAC,MAXLO,LUNS
      INTEGER NHI,NIM,IL1,IL2,ITMAX,MOD,IT
      DIMENSION DISTH(NDIST),DISTL(NDIST),XLO(MAXIM,MAXLO),
     1          HIDIM(MAXIM,MAXFAC),XLOC(MAXIM,MAXLO)
      CHARACTER*4 NAME(MAXIM)
      INTEGER   IST,IN,IOFF
      LOGICAL   PFLAG
      CHARACTER *(*) OUT

      IF (NIM.LE.10) PFLAG=.TRUE.
      WRITE(NDAT,50) EPS,NHI,NIM,ZAUBF,IL1,IL2,ITMAX,IT,MOD,EZ,EXP
50    FORMAT(' EPS,NHI,NIM,ZAUBF,IL1,IL2,ITMAX,IT,MOD,EZ,EXP',/
     $    1X,E8.2,2I4,F4.1,5I4,2E10.2)
      IF(PFLAG) WRITE(NDAT,51) ((HIDIM(K,L),L=1,NHI),K=1,NIM)
51    FORMAT(' NLM',10F10.2)


C     CALCULATE DISTANCES IN HIGH-DIMENSIONAL SPACE

      DISMIN=0.
      DISMAX=0.
      SUM=0.
      IOFF=0
      DO  K=1,NIM-1
      IN=(K-1)*NIM-IOFF
      IOFF=IOFF+K
      DO  L=K+1,NIM
      IN=IN+1
      DIS=0.
      DO  LL=1,NHI
      D=HIDIM(K,LL)-HIDIM(L,LL)
      D=D*D
      DIS=DIS+D
      IF(PFLAG)WRITE(NDAT,52) K,L,LL,D,DIS,IN,HIDIM(K,LL),HIDIM(L,LL)
52    FORMAT(' IM(',I3,')-IM(',I3,'); FACT:',I3,' D=',F12.4,
     $    ' DIS=',F12.4,' IN:',I4,3X,F12.4,'-',F12.4)
      ENDDO
      DIS=SQRT(DIS)
      IF(DISMAX.LT.DIS) DISMAX=DIS
      IF(DISMIN.GT.DIS) DISMIN=DIS
      DISTH(IN)=DIS
      SUM=SUM+DIS*DIS**(1.-EXP)
      IF (PFLAG)WRITE(NDAT,53) IN,DIS
53    FORMAT('  RESULT DISTH(',I4,' )=',F12.4)
      ENDDO
      ENDDO
      CALL DISTLST(DISTH,DISTL,NIM,NDAT)
C*12    FORMAT(1H ,<MIN(NIM,24)>E12.4)
 
C     CALCULATE INITIAL DISTANCES IN 2-D SPACE

      CALL DIST2D(XLO,NIM,DISTL)
      CALL DISTLST(DISTH,DISTL,NIM,NDAT)
      WRITE(NDAT,111)(XLO(K,1),K=1,NIM)
      WRITE(NDAT,112)(XLO(K,2),K=1,NIM)
  
C      START THE ITERATIONS

       EZ=1000.
       IT=0
1000   IT=IT+1
       CALL NLTFILE(OUT,LUNS,NAME,XLO,NIM)
C      WRITE(NDAT,111)(XLO(K,1),K=1,NIM)
C      WRITE(NDAT,112)(XLO(K,2),K=1,NIM)
       DO  K=1,NIM
       CALL MRDEV(DISTH,DISTL,XLO,NIM,K,SUM,DEL1,DEL2,EXP)
       IF(IT.GT.1.OR.K.GT.1) GOTO 13
       CALL MRERR(DISTH,DISTL,SUM,NIM,E,EXP)
       WRITE(NDAT,113) E
113    FORMAT(' INITIAL ERROR:',E12.4)
13     CONTINUE
       XLOC(K,1)=XLO(K,1)-ZAUBF*DEL1
       XLOC(K,2)=XLO(K,2)-ZAUBF*DEL2
       ENDDO

C      CHECK THE NEW DISTANCES

       CALL DIST2D(XLOC,NIM,DISTL)
       CALL DISTLST(DISTH,DISTL,NIM,NDAT)
       CALL MRERR(DISTH,DISTL,SUM,NIM,E,EXP)
       WRITE (NDAT,110) E0,E,IT
110    FORMAT(' E0,E,IT',2E12.4,I4)
       IF(E.LT.EPS) GOTO 9000
       IF(E.LT.E0)  GOTO 9000
       IF(IT.EQ.1) GOTO 9000
       WRITE(NDAT,100)
100    FORMAT(' ERRORSUM INCREASED')
C      GOTO 9100
9000   DO  K=1,NIM
       XLO(K,1)=XLOC(K,1)
       XLO(K,2)=XLOC(K,2)
       ENDDO
       IF(E.LT.EZ) EZ=E
       E0=E
C      WRITE(NDAT,114)(XLOC(K,1),K=1,NIM)
C      WRITE(NDAT,115)(XLOC(K,2),K=1,NIM)
111    FORMAT(' STRT X-COORDINATES:',10E10.2)
112    FORMAT(' STRT Y-COORDINATES:',10E10.2)
114    FORMAT(' X-COORDINATES:',10E10.2)
115    FORMAT(' Y-COORDINATES:',10E10.2)
       IF(MOD.EQ.3.AND.IT.GE.ITMAX) GOTO 9100
       IF(IT.GT.200) GOTO 9100
       IF(E.LE.EZ.AND.IT.GT.ITMAX) GOTO 9100
       IF(E.GT.EPS) GOTO 1000
9100   CONTINUE
       CALL DISTLST(DISTH,DISTL,NIM,NDAT)
       RETURN
       END




