
C++*******************************************************************
C
C    NORM3.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C    NORM3(LUN,NSAM,NROW,NSLICE,FMAX,FMIN,AV)
C
C    PURPOSE:   DETERMINES MAX, MIN, AV AND SIG FOR A SPIDER IMAGE
C
C    PARAMETERS:
C        LUN          LOGICAL UNIT NUMBER OF IMAGE
C        NSAM,NROW,NSLICE    DIMENSIONS OF VOLUME
C        FMAX         MAXIMUM OF VOLUME
C        FMIN         MINIMUM OF VOLUME
C        AV           AVERAGE OF VOLUME
C
C--*******************************************************************

      SUBROUTINE NORM3(LUN,NSAM,NROW,NSLICE,FMAX,FMIN,AV)

      COMMON BUF(1)
      COMMON /MASTER/NSAMC,NROWC,IREC,NLABEL,IFORM,IMAMI,FMAXC,FMINC,
     1               AVC,SIG,IHIST
      COMMON /UNITS/LUNT,NIN,NOUT

      DOUBLE PRECISION DAV,DAV2,DTOP,FNALL,DTEMP
      LOGICAL          SETMIN
CNO_SAVE
      FNALL = (NSAM * NROW * NSLICE)
      IF (FNALL .LE. 0) THEN
         CALL ERRT(1,'NORM3',NE)
         WRITE(NOUT,*) '*** NSAM,NROW,NSLICE:',NSAM,NROW,NSLICE
         RETURN
      ENDIF

      DAV    = 0.
      DAV2   = 0.
      SETMIN = .TRUE.

      DO IRECT = 1,NROW*NSLICE
          CALL REDLIN(LUN,BUF,NSAM,IRECT)
           IF (SETMIN)  THEN
              FMAX   = BUF(1)
              FMIN   = FMAX
              SETMIN = .FALSE.
           ENDIF
          DO K = 1,NSAM
            B    = BUF(K)
            FMAX = AMAX1(B,FMAX)
            FMIN = AMIN1(B,FMIN)
            DAV  = DAV  + B
            DAV2 = DAV2 + B * DBLE(B)
          ENDDO
      ENDDO

   
      AV    = DAV / FNALL
      AVC   = AV
      FMAXC = FMAX
      FMINC = FMIN

      DTOP  = DAV2 - DAV * DAV / FNALL
      IF (DTOP .LT. 0.0) THEN
C        SQRT OF NEGATIVE NUMBER
         WRITE(NOUT,*) '*** SQRT(',DTOP,') IMPOSSIBLE' 
         DTEMP = DAV * DAV / FNALL
         WRITE(NOUT,*) '*** DAV2: ',DAV2,'DAV*DAV/FNALL:',DTEMP 
         CALL ERRT(31,'NORM3',NE)
         RETURN

      ELSEIF (FNALL .EQ. 1.0) THEN
C        DIVISION BY ZERO
         WRITE(NOUT,*) '*** FNALL(',FNALL,') GIVES DIV. BY 0' 
         CALL ERRT(31,'NORM3',NE)
         RETURN
      ENDIF

      SIG   = DSQRT( DTOP / DBLE(FNALL-1.0))

      CALL SETPRM(LUN,NSAM,NROW,FMAX,FMIN,AV,'U')

      RETURN
      END
