
C++*********************************************************************
C
C OPENFIL.FOR  -- ADAPTED FROM OPEN3.FOR OCT 88 al
C                 LONG FILENAMES         OCT 88 al
C                 MERGED WITH OPENF      AUG 96 al
C                 -1, -3 & -7 FOURIER FORMATS REMOVED  AUG 96 al
C
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK        *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND     *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.  *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS *
C *    OR USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN        *
C *    APPROVAL OF THE CENTER FOR LABORATORIES AND RESEARCH            *
C **********************************************************************
C
C    OPENFIL(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,NSTACK,IMUSED,DISP,NF)
C
C    PURPOSE:       OPEN NEW OR EXISTING DATA FILE FOR RANDOM 
C                   ACCESS READING/WRITING.  THIS ROUTINE IS NOT
C                   CALLED FOR CREATING STACKED IMAGES WITHIN A
C                   STACK (ONLY FOR THE OVERALL STACK).  NEITHER
C                   IS IT USED FOR INLINE STACKS!
C
C    PARAMETERS:
C        BUF        WORKING BUFFER FOR READ/WRITE                 (SENT)
C        FILNAM     CHARACTER ARRAY, CONTAINING FILE NAME         (SENT)
C        LUN        LOGICAL UNIT NUMBER FOR FILNAM.               (SENT)
C        NSAM,NROW  DIMENSIONS OF FILE                (SENT OR RETURNED)
C        NSLICE     NUMBER OF SLICES IN IMAGE         (SENT OR RETURNED)
C        NSTACK     STACK INDICATOR                    (SENT / RETURNED)
C                   ON INPUT FOR NEW FILE -2 IS NOT A STACK, 0 IS A
C                   STACK, ON OUTPUT RETURNS -2 IF NOT A STACK,  
C                   OR MAX. NUMBER OF IMAGE IN STACK. 
C        IMUSED     USED IMAGE STACK INDICATOR                (RETURNED)
C        DISP       CHARACTER VARIABLE CONTAINING DISPOSITION     (SENT)
C                   SEE OPFIL.F FOR POSSIBLE VALUES            
C        NF         ERROR RETURN FLAG.                        (RETURNED)
C                   NF = 2    FOR NORMAL RETURN
C                   NF = 1    FOR ERROR RETURN
C
C    CODING:   BASED ON IMAGE PARAMETERS NSAM,NROW, & NSLICE,  A
C              NEW FILE IS OPENED WITH IREC RECORDS, EACH NSAM * 4 
C              BYTES LONG.  IREC ALLOWS SPACE FOR THE 3-D DISTRIBUTION,
C              PLUS NORMAL IMAGE HEADER.
C
C    VARIABLES: IFORM (TYPE)  FILE TYPE SPECIFIER. 
C               +1    R     2-D IMAGE
C               +3    R3    3-D VOLUME FILE
C               -9    FS    3-D SIMPLE FORMAT FOURIER (MICHAEL'S FORMAT)
C               -11   O2    2-D FOURIER TRANSFORM, MIXED RADIX ODD
C               -12   E2    2-D FOURIER TRANSFORM, MIXED RADIX EVEN
C               -21   O3    3-D FOURIER TRANSFORM, MIXED RADIX ODD
C               -22   E3    3-D FOURIER TRANSFORM, MIXED RADIX EVEN
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

	SUBROUTINE OPENFIL(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,NSTACK,
     &                     IMUSED,DISP,NF)

        INCLUDE 'CMBLOCK.INC'
        INCLUDE 'LABLOCK.INC'
CNO_SAVE

        COMMON /LUNARA/ LUNARA(100),LUNSTK(100)

#ifdef SP_SUN4 
        INTEGER * 4     LOCLUN
#else
#ifdef SP_NT
        INTEGER * 4     LOCLUN
#else
        INTEGER * 8     LOCLUN
#endif
#endif

        LOGICAL         USEINLN
        COMMON /LUNARB/ LOCLUN(100),NSAMLUN(100),NWORDLUN(100),USEINLN
        COMMON /IOERR/  IERR

        DIMENSION       BUF(*)
        
        CHARACTER *(*)  FILNAM
        CHARACTER * 81  FILNM
        CHARACTER * 1   NULL,DISP,DSP
	LOGICAL         EX,NEWDAT,NEEDERRT

	DATA NSAMAX/16384/

        NULL = CHAR(0)

C       TO KEEP OLD DATE
        NEWDAT = .TRUE.
        IF (NF .LT. 0) NEWDAT = .FALSE.

C       SET FLAG FOR ERRONEOUS RETURN
        NF     = 1
        IERR   = 0
       
        IF (LUN .LE. 0 .OR. LUN .GT. 100) THEN
           WRITE(NOUT,*)
     &         '*** PGM. ERROR: LUN MUST BE 1...100, SET TO 99'
           LUN = 99
        ENDIF

C	CHECK IF USER WANTS TO USE THE IN-LINE BUFFER.
        INLNED = 0
        IF (USEINLN .AND. FILNAM(1:1) .EQ. '_') THEN
           CALL INLNBUF(FILNAM,NLET,INLNED,IRTFLG)
           IF (IRTFLG .NE. 0)  RETURN

	   IF (DISP .EQ. 'U' .OR. DISP .EQ. 'N') THEN
	     EX  = .FALSE.
	   ELSE
	     EX  = .TRUE.
	   ENDIF
           FILNM = FILNAM(1:NLET)

        ELSE
C          USE REGULAR FILE
           INLNED      = 0
	   LUNC        = LUN
           LOCLUN(LUN) = 0

           IF (DATEXC .EQ. NULL) THEN
              FILNM = FILNAM
           ELSE
              CALL FILCAN(FILNM,NLET,NULL,NULL,FILNAM,DATEXC,IER)
	      IF (IER .NE.0) THEN
                 CALL ERRT(100,'OPENFIL',NE)
                 RETURN
              ENDIF
          ENDIF

C         SEE IF FILE EXISTS NOW
#ifdef SP_VMS
	  INQUIRE(FILE=FILNM,IOSTAT=IER,EXIST=EX,RECL=NSAMO)
C         RECL RETURNED IS IN BYTES ON VAX/VMS
          NSAMO = NSAMO / 4
#else
	  INQUIRE(FILE=FILNM,IOSTAT=IER,EXIST=EX)
#endif
          IF (IER .NE. 0) THEN
             WRITE(NOUT,*) '*** FILE INQUIRY ERROR: ',FILNM(1:NLET)
             CALL ERRT(100,'OPENFIL',NE)
             RETURN
          ENDIF
        ENDIF


10	IF (DISP .EQ. 'U' .OR. DISP .EQ. 'N') THEN
C         USER WANTS A NEW FILE TO WRITE INTO ----------------------NEW

          IF (EX) THEN
C           FILE IS IS OPENED WITH 'UNKNOWN' BUT ALREADY EXISTS; IT WILL
C           BE REPLACED.  OLD FILES ARE DELETED FIRST 

#ifdef SP_VMS
            OPEN(UNIT=LUN,FILE=FILNM,STATUS='OLD',DISPOSE='DELETE')
            CLOSE(UNIT=LUN,DISPOSE='DELETE',IOSTAT=IER)
#else
            OPEN(UNIT=LUN,FILE=FILNM,STATUS='OLD')
            CLOSE(UNIT=LUN,STATUS='DELETE',IOSTAT=IER)
#endif
	    IF (IER .NE. 0) THEN
               WRITE(NOUT,*) '*** ERROR DELETING FILE: ', FILNM(1:NLET)
               CALL ERRT(100,'OPENFIL',NE)
               RETURN
            ENDIF
          ENDIF

	  IF (NSAM .EQ. 0 .OR. NROW .EQ. 0) THEN
C           FOR NEW FILES, NON-ZERO NSAM, NROW NEED TO BE SUPPLIED
2060	    WRITE(NOUT,*) '*** RECORD INFO MISSING: ',FILNM(1:NLET)
            CALL ERRT(100,'OPENFIL',NE)
            RETURN

          ELSEIF (NSAM .GT. NSAMAX) THEN
C           EXCESSIVE ROW LENGTH 
            WRITE(NOUT,*) '*** ROWLENGTH MUST BE <=:',NSAMAX
            CALL ERRT(100,'OPENFIL',NE)
            RETURN

          ELSEIF (NSLICE .EQ. 0) THEN
C           NEW FILES NEED A NON ZERO SLICE NUMBER FOR FILE FORMAT FLAG
            NSLICE = 1

          ELSEIF (NSLICE .LT. 0) THEN
C           WANT TO CREATE OBSOLETE SHORT HEADER FILE
#ifdef SP_VMS
            CALL ERRT(100,
     &         'CAN NO LONGER CREATE OBSOLETE SHORT HEADER FILE',NE)
            RETURN
#else
            WRITE(NOUT,*) 'WARNING:  NO SHORT HEADER FILES ON UNIX.'
            NSLICE = -NSLICE
#endif
          ENDIF

          IF (IFORM .EQ. -1 .OR. IFORM .EQ. -3 .OR. IFORM .EQ. -7) THEN
C           WANT TO CREATE OBSOLETE FOURIER FILE
            CALL ERRT(101,
     &         'CAN NOT CREATE OBSOLETE FOURIER FORMAT FILE',NE)
            RETURN
          ENDIF

          DSP = 'N'

C         ADJUST NUMBER OF HEADER RECORDS TO HAVE >=256*4 BYTES IN HEADER

C         STANDARD SPIDER FLOATING POINT FILES
          LENBYT = NSAM * 4
          LABREC = 1024 / LENBYT
          IF (MOD(1024,LENBYT) .NE. 0) LABREC = LABREC + 1
          LABBYT = LABREC * LENBYT

          IREC = NROW * NSLICE + LABREC

	  IF (INLNED .EQ. 0) THEN
C	     REGULAR FILE
#ifdef SP_VMS
             CLOSE (LUN)
             NBLOCK = FLOAT(NSAM) / 512.0 * FLOAT(IREC) + 1.0

C            RECL IS IN UNITS OF FLOATS ON VAX
             LENREC = NSAM
	     OPEN(UNIT=LUN,FILE=FILNM,STATUS='NEW',RECORDTYPE='FIXED',
     &          ACCESS='DIRECT',FORM='UNFORMATTED',
     &          BLOCKSIZE=16384,BUFFERCOUNT=4, 
     &          RECL=LENREC,INITIALSIZE=NBLOCK,
     &          IOSTAT=IER)

C            ACTIVATE FOLLOWING LINE IF YOU NEED MORE INFO ON OPEN
C            ERROR. THE CALL TO LIB$SIGNAL STOPS THE PROGRAM AND 
C            PRINTS AN INFORMATIVE ERROR MESSAGE.
C            IF (IER .NE. 0) CALL LIB$SIGNAL(%VAL(IER))

#else
#ifdef __osf__
C            RECL IS IN UNITS OF FLOATS ON OSF 
             LENREC = NSAM
	     OPEN(LUN,FILE=FILNM,STATUS='NEW',FORM='UNFORMATTED',
     &            ACCESS='DIRECT',IOSTAT=IER,RECL=LENREC)
#else
C            RECL IS IN UNITS OF BYTES ON SGI AND SUN
             LENREC = NSAM * 4
	     OPEN(LUN,FILE=FILNM,STATUS='NEW',FORM='UNFORMATTED',
     &            ACCESS='DIRECT',IOSTAT=IER,RECL=LENREC)
#endif
#endif
             IF (IER .NE. 0) THEN        
                WRITE(NOUT,*) '*** ERROR OPENING FILE: ',FILNM(1:NLET)
                CALL ERRT(100,'OPENFIL',NE)
                RETURN
             ENDIF

          ELSE
C            SET UP INLINE BUFFER AND TIE IT TO LUN
             NWORDS   = IREC * LENBYT / 4
             NEEDERRT = .TRUE.
             CALL OPENINLN(LUN,INLNED,.TRUE.,NSAM,NWORDS,
     &                     NEEDERRT,IRTFLG)
             IF (IRTFLG .NE. 0)  RETURN
	  ENDIF

#ifdef SP_VMS
          BUF(1) = -NSLICE
#else
          BUF(1) = NSLICE
#endif
          BUF(2) = NROW
          BUF(3) = IREC
C         BUF(4) USED TO CONTAIN NHISTREC
          BUF(4) = 0
          BUF(5) = IFORM
          IMAMI  = 0
          BUF(6) = IMAMI
          
          DO K= 6,212
C            ZERO UNUSED BUF VARIABLES
             BUF(K)=0
          ENDDO

          BUF(12) = NSAM
          BUF(13) = LABREC
          BUF(22) = LABBYT 
          BUF(23) = LENBYT
C         BUF(24) IS A STACK INDICATOR
          IF (NSTACK .NE. -2) THEN
C            NEW IMAGE IS BARE STACK OR STACKED IMAGE
C            SET BUF(24) > 0 FOR STACK 
             BUF(24) = 2
          ELSE
C            NOT A STACK
             BUF(24) = 0
          ENDIF
C         SET USED IMAGE INDICATOR  (BUF(25)) TO FLAG FOR GOOD STACK
          BUF(25) = -1.0
          BUF(26) = 0

C         ADDED JULY 88 al
          IF (IANGLE .NE. 0) THEN
             BUF(14) = IANGLE
             BUF(15) = PHI
             BUF(16) = THETA
             BUF(17) = PSI
          ENDIF

          IF (KANGLE .NE.0) THEN
             BUF(30) = KANGLE
             BUF(31) = PHI2
             BUF(32) = THETA2
             BUF(33) = PSI2
             BUF(34) = PHI1
             BUF(35) = THETA1
             BUF(36) = PSI1
          ENDIF

C         COPY MOST RECENT HDR_VALS INTO LOCATIONS 37..101
          DO I = 1,64
             BUF(36+I) = HDR_VALS(I)
          ENDDO

C         SET CURRENT DATE AND TIME & TITLE
          CALL DATTIMTIT(BUF,.FALSE.,NEWDAT,LENTIT,IRTFLG)

C         WRITE HEADER RECORDS INTO FILE	    
          LUNARA(LUN) = 0

          ILOC = 1
          DO I = 1,LABREC
             CALL WRTLIN(LUN,BUF(ILOC),NSAM,I)
             ILOC = ILOC + NSAM
          ENDDO

C         SET FINAL LUNARA OFFSET VALUE FOR USE BY REDLIN/WRTLIN
          LUNARA(LUN) = LABREC

          GOTO 2000

C         --------------------------------------------------------- OLD
C         --------------------------------------------------------- OLD

	ELSEIF (DISP .EQ. 'O' .OR. DISP .EQ. 'K' .OR.
     &          DISP .EQ. 'Z' .OR. 
     &          DISP .EQ. 'E' .OR. DISP .EQ. 'M') THEN

C         FILE EXISTS, AND IS ACCESSED WITH 'OLD', OPEN THE FILE

	  IF (.NOT. EX) THEN
C            ERROR -- FILE DOES NOT EXIST, BUT BEING OPENED WITH 'OLD'
             WRITE(NOUT,*) '*** FILE NOT FOUND: ',FILNM(1:NLET)

C	     FOR DISP=Z, DO NOT STOP THE BATCH JOB BY CALLING ERRT
             IF (DISP .EQ. 'Z' .AND. 
     $          (IPROC .NE. 0 .OR. COPT  .EQ. 'B')) RETURN
     
             CALL ERRT(100,'OPENFIL',NE)
             RETURN
          ENDIF

          IF (INLNED .NE. 0) THEN
C            USE EXISTING INLINE BUFFER, TIE IT TO LUN & RETRIEVE NSAM
             NEEDERRT = DISP(1:1) .NE. 'Z'
             CALL OPENINLN(LUN,INLNED,.FALSE.,NSAM,0,NEEDERRT,IRTFLG)
             IF (IRTFLG .NE. 0)  RETURN
          ELSE
C            REGULAR FILE ACCESS

#ifdef SP_VMS
C            NSAM IS GOTTEN FROM INQUIRY ON VAX
             NSAM = NSAMO

C            OPEN FOR READ / WRITE
             OPEN(UNIT=LUN,FILE=FILNM,STATUS='OLD',
     &              RECORDTYPE='FIXED', ACCESS='DIRECT', IOSTAT=IER)
C            IF (IER .NE. 0) CALL LIB$SIGNAL(%VAL(IER))

             IF (IER .NE. 0) THEN         
                WRITE(NOUT,*) '*** ERROR OPENING FILE: ',FILNM(1:NLET)
                CALL ERRT(100,'OPENFIL',NE)
                RETURN
             ENDIF
#else
#if defined(__osf__) || defined(osf_ieee)
C            REGULAR FILE ACCESS, RECL IS IN 4 BYTE WORDS
             OPEN(LUN,FILE=FILNM,STATUS='OLD',ACCESS='DIRECT',
     &           FORM='UNFORMATTED',RECL=256,IOSTAT=IER)
#else
C            REGULAR FILE ACCESS, RECL IS IN BYTES
             OPEN(LUN,FILE=FILNM,STATUS='OLD',ACCESS='DIRECT',
     &           FORM='UNFORMATTED',RECL=1024,IOSTAT=IER)
#endif
             IF (IER .NE. 0) THEN        
                WRITE(NOUT,*) '*** ERROR OPENING FILE: ',FILNM(1:NLET)
                CALL ERRT(100,'OPENFIL',NE)
                RETURN
             ENDIF
#endif
     	  ENDIF

C         READ HEADER FROM FILE            	       
          LUNARA(LUN) = 0
          IERR        = 0

#ifdef SP_VMS
C         FIND NUMBER OF RECORDS IN HEADER
          LABREC = 256 / NSAM
          IF (MOD(256,NSAM) .NE. 0) LABREC = LABREC + 1

C         READ IN HEADER
          ILOC = 1
          DO I = 1,LABREC
             CALL REDLIN(LUN,BUF(ILOC),NSAM,I)
             ILOC = ILOC + NSAM
          ENDDO

          IF (BUF(1) .GT. 0) THEN
C            A POSITIVE VALUE FOR NSLICE ON VAX IS FLAG FOR OLD
C            OBSOLETE SHORT HEADER FORMAT
             WRITE(NOUT,*) 
     &           'WARNING, THIS MAY BE OBSOLETE SHORT HEADER FILE'
             WRITE(NOUT,*) 'USE "COPYFROM SHORT" TO CONVERT.'
             CALL ERRT(100,'OPENFIL',NE)
          ENDIF
#else
C         ALL OF USED HEADER CAN BE READ IN ON UNIX IN ONE READ
          CALL REDLIN(LUN,BUF,256,1)
          IF (IERR .NE. 0) THEN
             WRITE(NOUT,*) '*** ERROR READING HEADER OF: ',FILNM(:NLET)
             CALL ERRT(100,'OPENFIL',NE)
             RETURN
          ENDIF
#endif
          CALL OPENGETHDR(BUF,IDUM,NROW,NSLICE,NSTACK,IFORM,
     &                    IDUM,LABBYT,LENBYT,IMUSED,LENTIT)

          IF (IFORM .EQ. -1 .OR. IFORM .EQ. -3 .OR. IFORM .EQ. -7) THEN
C           WANT TO READ OBSOLETE FORMAT FOURIER FILE
            WRITE(NOUT,96)
96          FORMAT(' *** CAN NOT READ OBSOLETE FOURIER FORMAT',/,
     &          '*** CONVERT FOURIER FILE TO REAL FORMAT USING ',
     &          'ORIGINAL VERSION OF SPIDER.'/)
             CALL ERRT(100, 'OPENFIL',NE)
            IF (DISP .NE. 'E') RETURN
          ENDIF

#ifdef SP_VMS
C         CHECK TO SEE IF IREC IS REASONABLE
          IRECMIN = NROW * NSLICE + LABREC
          IF (IREC .LT. IRECMIN) IREC = IRECMIN
          IF (LENBYT .LE. 0) LENBYT = NSAM * 4
#else
C         ON UNIX MUST GET NSAM FROM HEADER LOCATION 12
          NSAM   = BUF(12)
          IF (LENBYT .LE. 0) LENBYT = NSAM * 4
          LABREC = BUF(13)

C         CORRECT UNREASONABLE LABREC (BAD VALUE ONCE)        
          LABRECT = 1024 / LENBYT
          IF (MOD(1024,LENBYT) .NE. 0) LABRECT = LABRECT + 1
          IF (LABRECT .LE. 0 .OR. LABREC .NE. LABRECT) THEN
C             UNREASONABLE LABREC NUMBER SO DEFAULT IT
              LABREC = LABRECT
          ENDIF

C         ON UNIX, REOPEN THE FILE WITH FINAL RECORD LENGTH
          IF (INLNED .EQ. 0) THEN
             CLOSE(LUN)
#ifdef __osf__
C            ALPHA OSF (UNIX) SPECIFIC STATEMENTS FOLLOW
C            RECL IS IN UNITS OF FLOATS ON OSF
             LENREC = NSAM 
	     OPEN(LUN,FILE=FILNM,STATUS='OLD',FORM='UNFORMATTED',
     &             ACCESS='DIRECT',IOSTAT=IER,RECL=LENREC)
#else
C            NON ALPHA OSF (UNIX) SPECIFIC STATEMENTS FOLLOW
C            RECL IS IN UNITS OF BYTES 
             LENREC = NSAM * 4
	     OPEN(LUN,FILE=FILNM,STATUS='OLD',FORM='UNFORMATTED',
     &             ACCESS='DIRECT',IOSTAT=IER,RECL=LENREC)
#endif
             IF (IER .NE. 0) THEN        
                WRITE(NOUT,*) '*** ERROR REOPENING FILE: ',FILNM(1:NLET)
                CALL ERRT(100,'OPENFIL',NE)
                RETURN
             ENDIF
          ENDIF
#endif

C         SET HEADER RECORD OFFSET IN LUNARA
          LUNARA(LUN) = LABREC

          DSP = 'O'

        ELSE
          WRITE(NOUT,*)'*** PGM. ERROR: UNKNOWN DISP IN OPENFIL: ',DISP
          CALL ERRT(100,'OPENFIL',NE)
          RETURN
        ENDIF

2000	CONTINUE
C       WRITE OUT FILE OPENING INFO, SET IMGNUM TO OVERALL FILE
        IMGNUM = 0
        CALL OPENINFO(FILNAM,NLET,NSAM,NROW,NSLICE,IMGNUM,NSTACK,
     &                LENTIT,DSP,LABBYT,IRTFLG)

C       SET FLAG FOR NORMAL RETURN	
        NF = 2
        RETURN

	END
