
C++*********************************************************************
C
C OPENINLN.F                                    AUTHOR: A. LEITH
C                                           
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK         *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND      *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.   *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR  *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR*
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 		       *
C **********************************************************************
C
C    OPENINLN()
C
C    PURPOSE:       TO OPEN A INLINE BUFFER FOR READING/WRITING 
C                   VIA REDLIN AND WRTLIN.
C
C    PARAMETERS:    LUN      EQUIVALENT LUN                    (SENT)
C                   NUMBUF   BUFFER NUMBER                     (SENT)
C                   NEEDNEW  LOGICAL FLAG THAT WANT NEW BUFFER (SENT)
C                   NSAM     LENGTH OF ROW USED IN BUFFER      (SENT)
C                   NWORDS   SIZE OF BUFFER                    (SENT)
C                   NEEDERRT NEED TO CALL ERRT AND END IF ERROR(SENT)
C                   IRTFLG   ERROR FLAG                        (RETURNED)
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

      SUBROUTINE OPENINLN(LUN,NUMBUF,NEEDNEW,NSAM,NWORDS,
     &                    NEEDERRT,IRTFLG)

      INCLUDE 'CMBLOCK.INC'

C     RUN-TIME ALLOCATION SPECIFIC CODE  FOLLOWS
      PARAMETER       (NUMINLN = 9)
#ifdef SP_NT
      INTEGER LOCLUN,LOCBUF
#else
      INTEGER * 8     LOCLUN,LOCBUF
#endif

C     RUN-TIME ALLOCATION SPECIFIC CODE  ENDS

      LOGICAL USEINLN
      COMMON /LUNARB/ LOCLUN(100),NSAMLUN(100),NWORDLUN(100),USEINLN
      COMMON /LUNARC/ LOCBUF(NUMINLN), NWORDBUF(NUMINLN), 
     &                NSAMBUF(NUMINLN),IRECBUF(NUMINLN),
     &                LABRECBUF(NUMINLN)

      LOGICAL         NEEDNEW,RUNTIME,NEEDERRT
      POINTER         (IPOINTER,DUMMY)

      IRTFLG = 1
      IF (LUN .LE. 0 .OR. LUN .GT. 100) THEN
C        NO SUCH LUN
         WRITE(NOUT,91) LUN
91       FORMAT(' *** PGM ERROR, BAD LUN IN OPENINLN: ',I3)
         CALL ERRT(100,'OPENINLN',NE)
         RETURN

      ELSEIF (NUMBUF .LE. 0 .OR. NUMBUF .GT. NUMINLN) THEN
C        NO SUCH BUFFER
         ITEMP = NUMINLN
         WRITE(NOUT,90) ITEMP,NUMBUF
90       FORMAT(' *** INLINE BUFFERS AVAILABLE:  1...',I3,
     &          ' BUT YOU WANTED: ',I4)
         CALL ERRT(100,'OPENINLN',NE)
         RETURN
      ENDIF

      IF (NEEDNEW .AND. (NSAM .EQ. 0 .OR. NWORDS .EQ. 0))THEN
C        WANT TO DEALLOCATE AND CLOSE AN OPEN INLINE BUFFER
         IPOINTER = LOCBUF(NUMBUF)
C        RUN-TIME ALLOCATION SPECIFIC CODE FOLLOWS
         RUNTIME = .TRUE.
         CALL ALLOCIT(0,RUNTIME,IPOINTER,'OPENINLN',IRTFLG)
         NWORDBUF(NUMBUF)  = 0
         LOCBUF(NUMBUF)    = 0 
         NSAMBUF(NUMBUF)   = 0
         IRECBUF(NUMBUF)   = 0
         LABRECBUF(NUMBUF) = 0
C        RUN-TIME ALLOCATION SPECIFIC CODE ENDS
         IRTFLG = 0
         RETURN
      ENDIF

C     RUN-TIME ALLOCATION SPECIFIC CODE FOLLOWS
      IF (NEEDNEW .AND. (LOCBUF(NUMBUF) .GT. 0) .AND.
     &    (NWORDBUF(NUMBUF) .NE. NWORDS)) THEN
C        FREE EXISTING SPACE ALLOCATION FOR THIS FILE
         IPOINTER = LOCBUF(NUMBUF)
         RUNTIME  = .TRUE.
         CALL ALLOCIT(0,RUNTIME,IPOINTER,'OPENINLN',IRTFLG)
         IF (IRTFLG .NE. 0) RETURN
         NWORDBUF(NUMBUF) = 0
         LOCBUF(NUMBUF)   = 0 
      ENDIF

      IF (NEEDNEW .AND. LOCBUF(NUMBUF) .EQ. 0) THEN
C        MUST ALLOCATE SPACE FOR THIS BUFFER
         MEMWANT = NWORDS 
         CALL ALLOCIT(MEMWANT,RUNTIME,IPOINTER,'OPENINLN',IRTFLG)
         IF (.NOT. RUNTIME .OR. (IRTFLG .NE. 0)) RETURN
         LOCBUF(NUMBUF)   = IPOINTER
         NWORDBUF(NUMBUF) = NWORDS
         NSAMBUF(NUMBUF)  = NSAM
      ENDIF

      IPOINTER    = LOCBUF(NUMBUF)
      LOCLUN(LUN) = IPOINTER
C     RUN-TIME ALLOCATION SPECIFIC CODE ENDS


      IF (.NOT. NEEDNEW) THEN
C        RETRIEVE RECORD LENGTH FROM PREVIOUSLY SET VALUE
         NSAM = NSAMBUF(NUMBUF)
         IF (NSAM .LE. 0) THEN
C           NO PREVIOUS RUN-TIME FILE AVAILABLE
            WRITE(NOUT,93) NUMBUF
93          FORMAT(' *** NO EXISTING INLINE BUFFER: ',I3)
            IF (NEEDERRT) CALL ERRT(100,'OPENINLN',NE)
            RETURN
         ENDIF
      ENDIF

C     SET LENGTH OF RECORD AND BUFFER FOR THIS LUN
      NSAMLUN(LUN)  = NSAM
      NWORDLUN(LUN) = NWORDBUF(NUMBUF)
      LOCLUN(LUN)   = LOCBUF(NUMBUF)

      IF (NWORDS .GT. NWORDLUN(LUN)) THEN
C        NOT ENOUGH SPACE IN BUFFER
         WRITE(NOUT,94) NUMBUF,NWORDLUN(LUN),NWORDS
94       FORMAT(' *** INLINE BUFFER: ',I4,
     &          '  LIMITED TO:  ',I8,' WORDS, WANTED: ',I8)
         CALL ERRT(100,'OPENINLN',NE)
         RETURN
      ENDIF

      IRTFLG = 0

      RETURN
      END
