
C++*********************************************************************
C
C OPENINSTK.F   -- NEW NOVEMBER 1996  AUTHOR: ARDEAN LEITH
C                                           
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.    AUTHOR: J.FRANK      *
C *  COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, ALBANY, NY. *
C * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH            *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED        *
C * FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF       *
C * HEALTH RESEARCH INC.                                               *
C **********************************************************************
C
C    OPENINSTK(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,NIMAGE,
C              ITYPE,DISP,NF)
C
C    PURPOSE:       TO OPEN A NEW OR EXISTING INLINE STACK.
C
C    PARAMETERS:
C        BUF        WORKING BUFFER FOR READ/WRITE               (SENT)
C        FILNAM     CHARACTER ARRAY, CONTAINING FILE NAME       (SENT)
C        LUN        LOGICAL UNIT NUMBER FOR FILNAM I/O.         (SENT)
C        NSAM,NROW  DIMENSIONS OF FILE                     (SENT/RET.)
C        NSLICE     NUMBER OF PLANES                       (SENT/RET.)
C        NIMAGE     NUMBER OF IMAGES ALLOWED IN NEW STACK      (SENT )
C        ITYPE      IFORM                                  (SENT/RET.)                    
C        DISP       FILE DISPOSITION, SEE OPENFB FOR VALUES    (SENT)
C        NF         ERROR RETURN FLAG.
C                   NF = 2    NORMAL RETURN
C                   NF = 1    ERROR RETURN
C
C  COMMON VARIABLES:
C        NSTACK    STACK INDICATOR                              (RET.)
C                  -2 IS NOT STACK
C                  -1 STACKED IMAGE
C                  >=0 IS WHOLE STACK AND VALUE IS MAX. NO. IMAGES 
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

#ifdef SP_VMS
      SUBROUTINE OPENINSTK(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,NIMAGE,
     &                     ITYPE,DISP,NF)
      INCLUDE 'CMBLOCK.INC'
      WRITE(NOUT,*) '*** INLINE STACKS ONLY AVAILABLE ON UNIX SPIDER.'
      CALL ERRT(100,'OPENINSTK',NE)  
      NF = 1
      RETURN
      END
#else


	SUBROUTINE OPENINSTK(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,NIMAGE,
     &                       ITYPE,DISP,NF)

CNO_SAVE

        INCLUDE 'CMBLOCK.INC'
        INCLUDE 'LABLOCK.INC'

        COMMON /STACKCOM/NSTACK

        COMMON /LUNARA/  LUNARA(100),LUNSTK(100)

C       DO NOT CHANGE NUMINLN WITHOUT SEARCH FOR OTHER OCCURANCES!
        PARAMETER        (NUMINLN = 9)
#ifdef SP_NT
        INTEGER LOCBUF
#else
        INTEGER * 8      LOCBUF
#endif
        COMMON /LUNARC/  LOCBUF(NUMINLN), NWORDBUF(NUMINLN), 
     &                   NSAMBUF(NUMINLN),IRECBUF(NUMINLN),
     &                   LABRECBUF(NUMINLN)

        DIMENSION        BUF(*)
        CHARACTER *(*)   FILNAM,DISP
        CHARACTER * 1    NULL,DSP
        CHARACTER * 2    TYPE
	LOGICAL          STACKOPN,ISDIGI

        DATA NBUFSIZ/17000/

        NULL = CHAR(0)

C       SET ERROR RETURN
        NF     = 1

C       MAKE SURE THE STACK OFFSET IS ALWAYS ZEROED
        LUNSTK(LUN) = 0

C       MAKE SURE USER WANTS TO USE A INLINE STACK      
        ILOC = INDEX(FILNAM,'@')
        IF (ILOC .LE. 2 .OR. FILNAM(1:1) .NE. '_') THEN
           WRITE(NOUT,*)'*** BAD SYNTAX FOR INLINE STACK:',FILNAM
           CALL ERRT(100,'OPENINSTK',NE)
           RETURN
        ENDIF

C       RETRIVE INLINE BUFFER NUMBER FROM FILE NAME
        CALL INLNBUF(FILNAM(1:ILOC-1),NLET,INLNED,IRTFLG)
        IF (IRTFLG .NE. 0)  RETURN

        IF (ISDIGI(FILNAM(ILOC + 1:ILOC + 1))) THEN
C          FIND IMAGE NUMBER WITHIN STACK FILE 
           READ(FILNAM(ILOC+1:),*,IOSTAT=IER) IMGNUM

           IF (IER .NE. 0) THEN
C             ERROR CONVERTING STACK NUMBER
              CALL ERRT(101,'UNABLE TO INTERPRET IMAGE NUMBER',NE)
              RETURN
           ELSEIF (IMGNUM .LE. 0) THEN
              CALL ERRT(101,'STACKS START WITH IMAGE: 1',NE)
              RETURN
           ENDIF
C          FOR SPECIFIC IMAGE RETURN NSTACK = -1
           NSTACK = -1
        ELSE
           IMGNUM   = 0
C          SET DEFAULT IMGNUM FOR BARESTACK 
C          (BARE STACKS RETURN NSTACK >= 0)
           NSTACK   = 0
        ENDIF

C       SEE IF INLINE STACK EXISTS NOW
        STACKOPN = (NSAMBUF(INLNED) .GT. 0)

C       SHOULD NOT STOP IF DISP == 'Z' AND REDHED FAILS
        NOSTOP = 0
        IF (DISP(1:1) .EQ. 'Z') NOSTOP = 1

C -------------------------------- NEW --------------------------------

10	IF (DISP(1:1) .EQ. 'U' .OR. DISP(1:1) .EQ. 'N') THEN
C         WANT TO MAKE A NEW STACK OR NEW IMAGE WITHIN EXISTING STACK
   
C         CHECK TO SEE IF NECESSARY INPUT INFO IS AVAILABLE & CORRECT
          CALL OPENCHK(NSAM,NROW,NSLICE,ITYPE,NBUFSIZ,IRTFLG)
          IF (IRTFLG .NE. 0) RETURN

          DSP    = 'N'

          IF (.NOT. STACKOPN .OR. IMGNUM .EQ. 0) THEN
C            INLINE STACK DOES NOT EXIST YET, CREATE NEW INLINE STACK 
             IF (NIMAGE .LE. 0) THEN
                CALL RDPRIS(NIMAGE,IDUM,NLOOP,ILOOP,
     &                    'NUMBER OF IMAGES ALLOWED IN STACK',IRTFLG)
                IF (IRTFLG .NE. 0) RETURN
             ENDIF

C            ADJUST NUMBER OF HEADER RECORDS TO HAVE >=256*4 BYTES IN HEADER
             LENBYT = NSAM * 4
             LABREC = 1024 / LENBYT
             IF (MOD(1024,LENBYT) .NE. 0) LABREC = LABREC + 1
             LABBYT = LABREC * LENBYT

C            CREATE OVERALL STACK HEADER
             BUF(1) = NSLICE
             BUF(2) = NROW
             IREC   = NROW * NSLICE + LABREC
             BUF(3) = IREC
C            BUF(4) UNUSED (USED TO CONTAIN NHISTREC)
             BUF(4) = 0
C            BUF(5)IS FOR IFORM (ITYPE)
             BUF(5) = ITYPE
             IMAMI  = 0
             BUF(6) = IMAMI
          
             DO K= 7,212
C               ZERO REMAINING HEADER VARIABLES
                BUF(K)=0
             ENDDO

             BUF(12) = NSAM
             BUF(13) = LABREC
             BUF(22) = LABBYT 
             BUF(23) = LENBYT

C            SET STACKED IMAGE INDICATOR
             BUF(24) = 2        

C            SET IMAGE USE IMAGE INDICATOR FOR NEW STACK OVERALL HEADER
             IMUSED  = -1
             BUF(25) = IMUSED

C            SET CURRENT DATE AND TIME & TITLE
             CALL DATTIMTIT(BUF,.FALSE.,NEWDAT,LENTIT,IRTFLG)
 
             IF (IMGNUM .EQ. 0) THEN
C	        ACCESS THE WHOLE STACK, NOT A PARTICULAR IMAGE BUT SINCE
C               DISP IS NEW AND STACK IS ALREADY OPEN THIS IS REALLY
C               ASKING TO CREATE A NEW STACK IN AN EXISTING STACK. 

C               CLOSE OLD STACK (THIS DESTROYS DATA IN IT!!)
                CALL OPENINLN(LUN,INLNED,.TRUE.,0,0,.FALSE.,IRTFLG)
             ENDIF

C            SET UP INLINE BUFFER AND TIE IT TO LUN
             NWORDS = (LABREC + NIMAGE  * IREC) * (LENBYT / 4)   
             CALL OPENINLN(LUN,INLNED,.TRUE.,NSAM,NWORDS,.TRUE.,IRTFLG)
             IF (IRTFLG .NE. 0)  RETURN

C            SET FINAL HEADER OFFSET VALUE FOR USE BY REDLIN/WRTLIN
             LUNARA(LUN) = LABREC
        
C            WRITE OVERALL HEADER RECORD(S) INTO INLINE STACK	    
             CALL WRTHED(LUN,BUF,NSAM,IRTFLG)

             IRECBUF(INLNED)   = IREC  
             LABRECBUF(INLNED) = LABREC  

             IF (IMGNUM .EQ. 0) THEN
C               DO NOT HAVE AN IMAGE TO PLACE IN THE STACK YET
C	        ACCESS THE WHOLE STACK NOT A PARTICULAR IMAGE
               GOTO 7777
             ENDIF

          ELSE
C            PUTTING IMAGE INTO EXISTING INLINE FILE 

C            USE EXISTING INLINE BUFFER, TIE IT TO LUN & GET NSAM
             CALL OPENINLN(LUN,INLNED,.FALSE.,NSAM,0,.TRUE.,IRTFLG)
             IF (IRTFLG .NE. 0)  RETURN
          ENDIF
       
C         RECOVER OVERALL HEADER FROM INLINE STACK FILE
          CALL REDHED(LUN,BUF,NBUFSIZ,NSAM,0,IRTFLG)
          IF (IRTFLG .NE. 0) RETURN

          NSTACKF = BUF(24)         
          ITYPEF  = BUF(5)
          NSAMF   = BUF(12)
          NROWF   = BUF(2)
          NSLICEF = BUF(1)
          LABRECF = BUF(13)

          IF (NSTACKF .EQ. 0) THEN
C             INLINE BUFFER DOES NOT CONTAIN A STACK
              CALL ERRT(101,'INLINE BUFFER DOES NOT CONTAIN A STACK',NE)
              RETURN

          ELSEIF (NSAMF   .NE. NSAM .OR. NROWF .NE. NROW .OR.
     &            NSLICEF .NE. NSLICE) THEN
C             EXISTING FILE HAS DIFFERING DIMENSIONS
              CALL ERRT(101,'IMAGE DIMENSIONS NOT SAME AS STACK',NE)
              RETURN

          ELSEIF (ITYPE .NE. ITYPEF) THEN
C            EXISTING STACK FILE FORMAT NOT SAME AS IMAGE FORMAT
             WRITE(NOUT,96) ITYPE,ITYPEF
96           FORMAT('IMAGE FORMAT: ',I5,
     &              ' NOT COMPATIBLE WITH EXISTING STACK FORMAT: ',I5)
             CALL ERRT(100,'OPENINSTK',NE)
             RETURN
          ENDIF

          IF (IMGNUM .GT. BUF(26)) THEN
C            UPDATE OVERALL HEADER WITH MAXIMUM IMAGE NUMBER IN USE
             BUF(26) = IMGNUM
             CALL WRTHED(LUN,BUF,NSAM,IRTFLG)
          ENDIF

C         GET IRECF FROM STACK FILE HEADER
          IRECF   = BUF(3)

C         SET IMAGE SPECIFIC LOCATIONS IN IMAGE HEADER
          IMAMI   = 0
          BUF(6)  = IMAMI
          IANGLE  = 0
          BUF(14) = IANGLE
          KANGLE  = 0
          BUF(30) = KANGLE
          IMUSED  = 1
          BUF(25) = IMUSED         

C         PUT CURRENT DATE AND TIME INTO BUF
          CALL DATTIMTIT(BUF,.FALSE.,.TRUE.,LENTIT,IRTFLG)

C         SET FINAL HEADER OFFSET VALUE FOR USE BY REDLIN/WRTLIN
          LUNARA(LUN) = LABRECF

C         ALL IMAGES HAVE SAME IREC AS STACK HEADER!
C         SET OFFSET INTO LUNSTK
          LUNSTK(LUN) = (IMGNUM - 1) * IRECF + LABRECF

C         PLACE THIS IMAGE HEADER INTO STACK LOCATION
          CALL WRTHED(LUN,BUF,NSAM,IRTFLG)


C -------------------------------- OLD --------------------------------
           
	ELSEIF (DISP(1:1) .EQ. 'O' .OR. DISP(1:1) .EQ. 'K' .OR.
     &          DISP(1:1) .EQ. 'Z' .OR. 
     &          DISP(1:1) .EQ. 'E' .OR. DISP(1:1) .EQ. 'M') THEN
C          WANT AN EXISTING IMAGE FROM EXISTING STACK OR AN
C          EXISTING BARE STACK

           DSP = 'O'

           IF (.NOT. STACKOPN) THEN
C             INLINE STACK DOES NOT EXIST YET, ERROR
              WRITE(NOUT,*) '*** INLINE STACK DOES NOT EXIST'

C	      FOR DISP=Z, DO NOT STOP THE BATCH JOB BY CALLING ERRT
              IF (DISP(1:1) .EQ. 'Z' .AND. 
     &           (IPROC .NE. 0 .OR. COPT  .EQ. 'B')) RETURN
              CALL ERRT(100,'OPENINSTK',NE)
              RETURN
           ENDIF

C          USE EXISTING INLINE BUFFER, TIE IT TO LUN & GET NSAM
           CALL OPENINLN(LUN,INLNED,.FALSE.,NSAM,0,.TRUE.,IRTFLG)
           IF (IRTFLG .NE. 0)  RETURN

C          ALL IMAGES HAVE SAME IREC AS HEADER
           IREC        = IRECBUF(INLNED)
           LABREC      = LABRECBUF(INLNED)
           LUNARA(LUN) = LABREC

C          SET OFFSET INTO LUNSTK TO POINT TO CURRENT IMAGE
	   IF (IMGNUM .EQ. 0) THEN
	      LUNSTK(LUN) = 0
           ELSE
              LUNSTK(LUN) = (IMGNUM - 1) * IREC + LABREC
	   ENDIF

C          GET SPECIFIED IMAGE HEADER (MAY BE OVERALL HEADER!)
           CALL REDHED(LUN,BUF,NBUFSIZ,NSAM,NOSTOP,IRTFLG)
           IF (IRTFLG .NE. 0) RETURN

           IF (IMGNUM .EQ. 0) THEN
C             FOR OVERALL STACK RETURN NSTACK = MAX IMAGE (BUF(26)
              NSTACK = BUF(26)
           ENDIF

C          RECOVER IMAGE PARAMETERS FROM SPECIFIC IMAGE HEADER
           CALL OPENGETHDR(BUF,NDUM,NROW,NSLICE,NDUM,IFORM,
     &                     NDUM,LABBYT,IDUM,IMUSED,LENTIT)

C          SEE IF THIS IMAGE IS USED IN THE STACK
           IF (IMGNUM .GT. 0 .AND. IMUSED .EQ. 0) THEN
C             NO EXISTING IMAGE WITHIN STACK
              IF (DISP(1:1) .NE. 'Z') THEN
                 CALL ERRT(101,'THIS IMAGE NOT USED IN STACK',NE)
              ENDIF
              RETURN
           ENDIF

        ELSE
          WRITE(NOUT,*)'*** PGM. ERROR: UNKNOWN DISP IN OPENINSTK:',DISP
          CALL ERRT(100,'OPENINSTK',NE)
          RETURN
        ENDIF

C ------------------------------- BOTH --------------------------------

C       WRITE OUT FILE OPENING INFO
7777    CALL OPENINFO(FILNAM,NLET,NSAM,NROW,NSLICE,IMGNUM,NSTACK,
     &                LENTIT,DSP,LABBYT,IRTFLG)

C       SET FLAG FOR NORMAL RETURN	
        NF = 2
        RETURN

	END

C ----------------------------- OPENCHK -----------------------------

      SUBROUTINE OPENCHK(NSAM,NROW,NSLICE,ITYPE,NSAMAX,IRTFLG)

      INCLUDE 'CMBLOCK.INC'
	
C     CHECK THAT INPUT PARAMETERS ARE OK

      IRTFLG = 0

      IF (NSAM .LE. 0 .OR. NROW .LE. 0) THEN
C        FOR NEW FILES, NON-ZERO NSAM, NROW NEED TO BE SUPPLIED
         CALL ERRT(101,'NSAM OR NROW NOT GIVEN',NE)
         RETURN

      ELSEIF (NSAM .GT. NSAMAX) THEN
C        EXCESSIVE ROW LENGTH 
         WRITE(NOUT,*) '*** ROWLENGTH MUST BE <=:',NSAMAX
         CALL ERRT(100,'OPENCHK',NE)
         RETURN

      ELSEIF (NSLICE .EQ. 0) THEN
C        NEW FILES NEED A NON ZERO SLICE NUMBER
         NSLICE = 1

      ELSEIF (NSLICE .LT. 0) THEN
C        WANT TO CREATE OBSOLETE SHORT HEADER FILE
#ifdef SP_VMS
         CALL ERRT(100,'SHORT HEADER FILES NO LONGER SUPPORTED',NE)
         RETURN
#else
         WRITE(NOUT,*) 
     &            'WARNING:  SHORT HEADER FILES NO LONGER SUPPORTED.'
         NSLICE = -NSLICE
#endif
      ENDIF

      IF (ITYPE .EQ. -1 .OR. ITYPE .EQ. -3 .OR. ITYPE .EQ. -7) THEN
C        WANT TO CREATE OBSOLETE FOURIER FILE
         CALL ERRT(101,
     &         'CAN NOT CREATE OBSOLETE FOURIER FORMAT FILE',NE)
         RETURN
      ENDIF

      IRTFLG = 0
      RETURN

      END

#endif

