
C++*********************************************************************
C
C OPENSTK.F   -- DEC. 96 al
C
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK        *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND     *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.  *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS *
C *    OR USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN        *
C *    APPROVAL OF THE CENTER FOR LABORATORIES AND RESEARCH            *
C **********************************************************************
C
C  OPENSTK(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,NIMAG,ITYPE,DISP,IRTFLG
C
C  PURPOSE:       TO OPEN A NEW OR EXISTING STACK FILE.
C
C  PARAMETERS:
C        BUF        WORKING BUFFER FOR READ/WRITE                 (SENT)
C        FILNAM     CHARACTER ARRAY, CONTAINING FILE NAME         (SENT)
C        LUN        LOGICAL UNIT NUMBER FOR FILNAM.               (SENT)
C        NSAM,NROW  DIMENSIONS OF FILE                       (SENT/RET.)
C        NSLICE     NUMBER OF PLANES                         (SENT/RET.)
C        NIMAGE     NUMBER OF IMAGES ALLOWED IN NEW STACK         (SENT)
C        ITYPE      IFORM                                    (SENT/RET.)                    
C        DISP       FILE DISPOSITION, SEE OPENFB FOR VALUES       (SENT)
C        IRTFLG     ERROR RETURN FLAG.                       (SENT/RET.)
C                   NF = 0    NORMAL RETURN
C
C  COMMON VARIABLES
C        NSTACK     FILE STACK SPECIFIER.                    (SENT/RET.)
C                     -2          NOT A STACK
C                     -1          STACKED IMAGE
C                      0          BARE STACK, CONTAINS NO IMAGE(S)
C                     >0          BARE STACK, VALUE IS MAX. IMAGE
C
C
C  CALL TREE:  SEE OPENFB
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C--*********************************************************************

        SUBROUTINE OPENSTK(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,NIMAG,
     &               ITYPE,DISP,IRTFLG)

        INCLUDE 'CMBLOCK.INC'
        INCLUDE 'LABLOCK.INC'
CNO_SAVE

        COMMON /LUNARA/  LUNARA(100),LUNSTK(100)

        COMMON /STACKCOM/NSTACK

        DIMENSION        BUF(*)
        CHARACTER *(*)   FILNAM,DISP
        CHARACTER * 81   FILN,FILNPE
        CHARACTER * 1    NULL,DISPF,DSP
	LOGICAL          EX,ISERROR,ISDIGI

        DATA NBUFSIZ/17000/

        NULL = CHAR(0)

C       ALLOW IT TO OPEN MALFORMED STACK IN COPY COMMAND ONLY
        ISERROR = .TRUE.
        IF (NF .EQ. 88) ISERROR = .FALSE.

C       SHOULD NOT STOP IF DISP == 'Z' AND REDHED FAILS
        NOSTOP = 0
        IF (DISP .EQ. 'Z') NOSTOP = 1

C       SET ERROR RETURN
        IRTFLG = 1

C       MAKE SURE THE STACK OFFSET IS ALWAYS ZEROED
        LUNSTK(LUN) = 0

        ILOC        = INDEX(FILNAM,'@')      

        IF (ISDIGI(FILNAM(ILOC + 1:ILOC + 1))) THEN
C          FIND IMAGE NUMBER WITHIN STACK FILE 
           READ(FILNAM(ILOC+1:),*,IOSTAT=IER) IMGNUM
        
           IF (IER .NE. 0) THEN
C             ERROR CONVERTING STACK NUMBER
              CALL ERRT(101,'UNABLE TO INTERPRET IMAGE NUMBER',NE)
              RETURN
           ELSEIF (IMGNUM .EQ. 0) THEN
              CALL ERRT(101,'STACKS START WITH IMAGE: 1',NE)
              RETURN
           ENDIF
        ELSE
C          SET IMGNUM FOR BARESTACK
           IMGNUM = 0
        ENDIF

C       GET FILENAME WITHOUT DATEXC
        FILN = FILNAM(1:ILOC-1) // CHAR(0) 

C       CREATE STACK FILE NAME WITHOUT '@' BUT WITH EXTENSION
        CALL FILCAN(FILNPE,NLET,NULL,NULL,FILN,DATEXC,IER)
	IF (IER .NE.0) THEN
            CALL ERRT(101,'ADDING EXTENSION TO FILENAME',NE)
            RETURN
        ENDIF

C       SEE IF STACK FILE ALREADY EXISTS NOW
        INQUIRE(FILE=FILNPE,IOSTAT=IER,EXIST=EX)
        IF (IER .NE. 0) THEN
           CALL ERRT(101,'FILE INQUIRY FAILED',NE)
           RETURN
        ENDIF

	IF (DISP(1:1) .EQ. 'U' .OR. DISP(1:1) .EQ. 'N') THEN
C          WANT TO MAKE A NEW STACK OR NEW IMAGE WITHIN EXISTING STACK
C -------------------------------- NEW --------------------------------

           IF (.NOT. EX .OR. IMGNUM .EQ. 0) THEN
C             STACK FILE DOES NOT EXIST YET, OR MUST BE REPLACED
C             CREATE A NEW STACK FILE, OPENFIL WILL RETURN NSTACK = 0
              NSTACK = 0
              IFORM  = ITYPE
	      CALL OPENFIL(BUF,FILN,LUN,
     &                     NSAM,NROW,NSLICE,NSTACK,IMUSED,DISP,NFT)
              IF (NFT .NE. 2) RETURN

           ELSE
C             OPEN ALREADY EXISTING STACK FILE
              ITYPEIN = ITYPE
              DISPF   = 'O'
              NSTACK  = 0
	      CALL OPENFIL(BUF,FILN,LUN,
     &                  NSAMF,NROWF,NSLICEF,NSTACK,IMUSED,DISPF,NFT)
              IF (NFT .NE. 2)  RETURN
              ITYPE = IFORM

C             OPENFIL WILL RETURN NUMBER OF IMAGES IN STACK, OR -1
C             IF THIS IS A SPECIFIC IMAGE WITHIN THE STACK, -2 IS
C             FOR NON-STACK IMAGE.

              IF (NSTACK .LE. -2) THEN
C                EXISTING FILE IS NOT A STACK
                 CALL ERRT(101,'EXISTING FILE IS NOT A STACK',NE)
                 RETURN

              ELSEIF (NSAMF   .NE. NSAM .OR. NROWF .NE. NROW .OR.
     &                NSLICEF .NE. NSLICE) THEN
C                EXISTING FILE HAS DIFFERING DIMENSIONS
                 CALL ERRT(101,'IMAGE DIMENSIONS NOT SAME AS STACK',NE)
                 RETURN

              ELSEIF (ITYPEIN .NE. ITYPE) THEN
C                EXISTING STACK FILE FORMAT NOT SAME AS IMAGE FORMAT
                 CALL ERRT(101,
     &              'IMAGES IN STACK MUST HAVE SAME FILE FORMAT',NE)
                 RETURN

              ELSEIF (IMUSED .NE. -1) THEN
C                 THIS IS AN OLDER STACK IT MAY BE MALFORMED. ERROR
C                 IF TRYING TO CREATE NEW IMAGE IN A MALFORMED STACK
                  CALL CHECKSTACK(LUN,NSAM,BUF,.TRUE.,IRTFLG)
                  IF (IRTFLG .NE. 0) RETURN
              ENDIF
           ENDIF

           IF (IMGNUM .LE. 0) THEN
C             JUST WANT TO OPEN NEW BARE STACK, OPENFIL HAS ALREADY
C             RECOVERED RELEVANT HEADER VALUES
              IRTFLG  = 0
              NSTACK  = 0
              RETURN
           ENDIF

C          RECOVER OVERALL HEADER FROM THE STACK FILE
           CALL REDHED(LUN,BUF,NBUFSIZ,NSAM,0,IRTFLG)
           IF (IRTFLG .NE. 0) RETURN

           IF (IMGNUM .GT. BUF(26)) THEN
C             UPDATE OVERALL HEADER WITH MAXIMUM IMAGE NUMBER
              BUF(26) = IMGNUM
              CALL WRTHED(LUN,BUF,NSAM,IRTFLG)
           ENDIF

C          SET IMAGE SPECIFIC LOCATIONS IN IMAGE HEADER  
           IMAMI   = 0
           BUF(6)  = IMAMI
           IANGLE  = 0
           BUF(14) = IANGLE
           KANGLE  = 0
           BUF(30) = KANGLE
           LABBYT  = BUF(22)

C          THIS IS A STACK AND IMAGE IS IN USE
C          RETURNS NSTACK = -1
           NSTACK  = -1
           BUF(24) = 2        
           IMUSED  = 1
           BUF(25) = IMUSED         
           BUF(26) = 0         

C          PUT CURRENT DATE AND TIME INTO BUF
           CALL DATTIMTIT(BUF,.FALSE.,.TRUE.,LENTIT,IRTFLG)

C          ALL IMAGES MUST HAVE SAME IREC AS STACK HEADER.
C          GET IRECF FROM STACK FILE HEADER
           IRECF       = BUF(3)

C          SET OFFSET INTO LUNSTK
           LUNSTK(LUN) = (IMGNUM-1) * IRECF + BUF(13)

C          PLACE THIS IMAGE HEADER INTO PROPER STACK LOCATION
           CALL WRTHED(LUN,BUF,NSAM,IRTFLG)

           DSP = 'N'

C -------------------------------- OLD --------------------------------
           
	ELSEIF (DISP(1:1) .EQ. 'O' .OR. DISP(1:1) .EQ. 'K' .OR.
     &          DISP(1:1) .EQ. 'Z' .OR. 
     &          DISP(1:1) .EQ. 'E' .OR. DISP(1:1) .EQ. 'M') THEN
C          WANT AN EXISTING IMAGE FROM AN EXISTING STACK

           IF (.NOT. EX) THEN
C             STACK FILE DOES NOT EXIST YET, ERROR
              WRITE(NOUT,*) '*** STACK FILE NOT FOUND: ',FILN

C	      FOR DISP=Z, DO NOT STOP THE BATCH JOB BY CALLING ERRT
              IF (DISP .EQ. 'Z' .AND. 
     &            (IPROC .NE. 0 .OR. COPT  .EQ. 'B')) RETURN
              CALL ERRT(100,'OPENFB',NE)
              RETURN
           ENDIF

C          OPEN THE EXISTING OVERALL STACK FILE
           DISPF  = 'O'
	   CALL OPENFIL(BUF,FILN,LUN,
     &                  NSAM,NROW,NSLICE,NSTACK,IMUSED,DISPF,NFT)
           IF (NFT .NE. 2)  RETURN
           IF (NSTACK .LE. -2) THEN
C             EXISTING FILE IS NOT A STACK
              CALL ERRT(101,'EXISTING FILE IS NOT A STACK',NE)
              RETURN
           ENDIF
           ITYPE = IFORM

C          RECOVER OVERALL HEADER FROM THE STACK FILE
           CALL REDHED(LUN,BUF,NBUFSIZ,NSAM,NOSTOP,IRTFLG)
           IF (IRTFLG .NE. 0) RETURN

           IF (BUF(25) .NE. -1) THEN
C             STOP IF MALFORMED STACK (EXCEPT IF IRTFLG = 88) 
              CALL CHECKSTACK(LUN,NSAM,BUF,ISERROR,IRTFLG)
              IF (IRTFLG .EQ. 88 .AND. ISERROR) RETURN
              IF (IRTFLG .EQ. 88) THEN
C                MALFORMED STACK FOR "CP FIX" USE ONLY
                 IRTFLG = 88
                 RETURN
              ENDIF
           ENDIF

           IF (IMGNUM .LE. 0) THEN
C             JUST WANT BARE STACK, RETURN NSTACK = MAX IMAGE IN STACK 
C             (FROM OVERALL HEADER LOC: 26)
              NSTACK = BUF(26)
              IRTFLG = 0
              RETURN
           ENDIF

C          ALL IMAGES HAVE SAME IREC AS HEADER VALUE
           IREC = BUF(3)

C          SET OFFSET INTO LUNSTK
           LUNSTK(LUN) = (IMGNUM-1) * IREC + BUF(13)

C          GET SPECIFIED IMAGE HEADER FROM STACK FILE LOCATION
           CALL REDHED(LUN,BUF,NBUFSIZ,NSAM,NOSTOP,IRTFLG)
           IF (IRTFLG .NE. 0) RETURN

C          RECOVER IMAGE PARAMETERS FROM SPECIFIC IMAGE HEADER
           CALL OPENGETHDR(BUF,NDUM,NDUM,NDUM,NDUM,NDUM,
     &                     NDUM,NDUM,LABBYT,IMUSED,LENTIT)

           IF (IMUSED .EQ. 0) THEN
C             NO EXISTING IMAGE WITHIN STACK
              IF (DISP .NE. 'Z') THEN
                 CALL ERRT(101,'THIS IMAGE NOT USED IN STACK',NE)
              ENDIF
              RETURN
           ENDIF

C          RETURN NSTACK = -1 (FOR STACKED IMAGE)
           NSTACK = -1
           DSP = 'O'

        ELSE
           CALL ERRT(101,'PGM. ERROR: UNKNOWN DISP IN OPENFB',NE)
           RETURN
        ENDIF

C ------------------------------- BOTH --------------------------------

C       SET FLAG FOR NORMAL RETURN	
        IRTFLG = 0

C       WRITE OUT FILE OPENING INFO
        NLET = 0
        CALL OPENINFO(FILNAM,NLET,NSAM,NROW,NSLICE,IMGNUM,NSTACK,
     &                LENTIT,DSP,LABBYT,IRTFLG)

        RETURN
	END


C       ------------------- CHECKSTACK --------------------------------

        SUBROUTINE CHECKSTACK(LUN,NSAM,BUF,ISERROR,IRTFLG)

        COMMON /LUNARA/ LUNARA(100),LUNSTK(100)

        PARAMETER  (NBUFSIZT = 17000)
        DIMENSION  BUF(NBUFSIZT)
        DIMENSION  BUFCHK(NBUFSIZT)
        LOGICAL    ISERROR

C       POINT LUNSTK TO RECORD FOLLOWING OVERALL HEADER
        LUNSTKT     = LUNSTK(LUN)
        LUNSTK(LUN) = LUNARA(LUN)
        NBUFSIZ     = NBUFSIZT

C       READ NEXT  RECORD AFTER OVERALL HEADER. IF THIS IS NOT A HEADER
C       THEN THIS IS A MALFORMED STACK
        CALL REDHED(LUN,BUFCHK,NBUFSIZ,NSAM,0,IRTFLG)
        IF (IRTFLG .NE. 0) RETURN

        IRTFLG = 0

C       CHECK NSLICE, NROW, NSAM, LABREC, LABBYT
        IF (BUF(1)  .NE. BUFCHK(1)  .OR. BUF(2)  .NE. BUFCHK(2) .OR.  
     &      BUF(12) .NE. BUFCHK(12) .OR. BUF(13) .NE. BUFCHK(13) .OR.  
     &      BUF(22) .NE. BUFCHK(22)) THEN
            IF (ISERROR) THEN
               WRITE(NOUT,*) '*** ERROR: STACK NEEDS REFORMING'
               CALL ERRT(101,'USE  -CP FIX-  TO REFORM STACK',NE)
            ELSE
               WRITE(NOUT,*) '*** WARNING: STACK NEEDS REFORMING'
            ENDIF
            IRTFLG = 88
        ENDIF

C       RESTORE LUNSTK VALUE
        LUNSTK(LUN) = LUNSTKT

        RETURN
        END
