
C++*************************************************************************
C
C OPFIL.F                                         AUTHOR: ARDEAN LEITH                        
C
C **********************************************************************
C * SPIDER - MODULAR IMAGE PROCESSING SYSTEM.      AUTHOR: J. FRANK    *
C * COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, RENSSELAER,  *
C * NY.  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH       *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED FOR    *
C * PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF HEALTH    *
C * RESEARCH INC.                                                      *
C **********************************************************************
C
C  OPFIL(ASKNAM,FILNAM,LUN,DISP,NSAM,NROW,NSLICE,MAXIM,PROMPT,FOUROK,
C        ,IRTFLG)
C
C  PURPOSE:         SOLICITS FILE NAME AND OPENS FILE
C  
C  PARAMETERS:
C 
C        ASKNAM     LOGICAL FLAG TO QUERY NAME                    (SENT) 
C        FILNAM     FILENAME (WITOUT EXTENSION)               (SENT/RET)
C        LUN        UNIT TO OPEN FILE ON                          (SENT)
C        DISP       CHARACTER CONTAINING ONE OF THE               (SENT) 
C                   FOLLOWING DISPOSITION SPECIFICATIONS:
C                     'O' -  FILE IS ASSUMED TO EXIST.  NSAM AND
C                            NROW AND HEADER INFO (IN COMMON) ARE 
C                            RETURNED TO THE CALLING PROGRAM. 
C                   'Z/E' -  THE FILE IS ASSUMED TO EXIST; NSAM AND
C                            NROW AND HEADER INFO (IN COMMON) ARE 
C                            RETURNED TO THE CALLING PROGRAM.
C                            IF FILE DOES NOT EXIST THEN BATCH DOES
C                            NOT STOP!!! (ONLY DIFFERENCE FROM 'O'). 
C                   'N'  -   WANT NEW FILE. NSAM, NROW, NSLICE, AND
C                            IFORM ARE SENT.
C                   'U'  -   IT IS NOT KNOWN IF THE FILE EXISTS.  
C                            NON-ZERO NSAM & NROW VALUES HAVE TO BE 
C                            SENT.  IF THE FILE ALREADY EXISTS, IT 
C                            WILL BE REPLACED.
C                   'K & M'-    NO LONGER USED.
C        NSAM       IMAGE SIZE                             (SENT OR RET)
C        NROW       IMAGE SIZE                             (SENT OR RET)
C        NSLICE     IMAGE SIZE                             (SENT OR RET)
C        MAXIM      STACK INDICATOR                           (SENT/RET)
C                     0 ON INPUT IS FOR SPECIFIC IMAGE            (SENT)
C                    >0 ON INPUT WITH DISP = "N" IS MAXIM         (SENT)
C                        (MAXIM NOT IMPLEMENTED YET)                              
C                    -1 ON INPUT ALLOWS WHOLE STACK OPERATION     (SENT)                         
C                    -2 RETURNED IS NON-STACK IMAGE               (RET.)
C                    -1 RETURNED IS STACKED IMAGE                 (RET.)
C                    >= 0 RETURNED IS BARE-STACK SIZE             (RET.) 
C        PROMPT     PROMPT FOR FILNAME                            (SENT)
C        FOUROK     CAN USE EXISTING FOURIER FILES?               (SENT)
C        IRTFLG     ERROR FLAG (0 IS NORMAL)                  (SENT/RET)
C                    -9 ON INPUT KEEPS OLD DATE/TIME
C                    88 ON INPUT ALLOWS MALFORMED STACKS
C
C    CODING:   BASED ON NEW OR OLD PICTURE PARAMETERS NSAM,NROW, A
C              NEW FILE IS OPENED WITH IREC RECORDS, EACH NSAM*4 
C              BYTES LONG.  IREC ALLOWS SPACE FOR THE 2-D OR 3-D 
C              IMAGE  PLUS HEADER.  A STACK FILE CONTAINS AN OVERALL
C              HEADER PLUS MAXIM * IREC RECORDS. EACH IMAGE IN THE
C              STACK HAS ITS OWN HEADER RECORD(S) WHOSE FORMAT IS THE
C              SAME AS THE OVERALL HEADER RECORDS
C
C    COMMON VARIABLES:
C              IFORM (TYPE)  FILE TYPE SPECIFIER.           (SENT/RET.)
C               +1    R     2-D IMAGE FILE
C               +3    R3    3-D IMAGE (VOLUME) FILE
C               -9    FS    3-D SIMPLE FORMAT FOURIER (MR'S FORMAT)
C               -11   FO    2-D FOURIER TRANSFORM, MIXED RADIX ODD
C               -12   FE    2-D FOURIER TRANSFORM, MIXED RADIX EVEN
C               -21   FE    3-D FOURIER TRANSFORM, MIXED RADIX ODD
C               -22   FE    3-D FOURIER TRANSFORM, MIXED RADIX EVEN
C
C     CALL TREE:
C
C         OPENF                                        
C          v                                         (regular)
C          |                    (not stack)            (file)
C         OPFIL ---> OPENALL  --------------> OPENFIL -------->
C          |            |                      |
C          ^            |                      `----------> INLNBUF
C         OPENFB        |                        (inline)   OPENINLN
C                       |                        (file  )
C                       |
C                       |
C                       |    (inline stack)
C                       |-------------------> OPNINSTK -->      
C                       |
C                       |    (file stack)
C                       ` ------------------>  OPENSTK --> OPENFIL -->
C
C        1         2         3         4         5         6         7
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*********************************************************************

        SUBROUTINE OPFIL(ASKNAM,FILNAM,LUN,DISP,NSAM,NROW,NSLICE,
     &                   MAXIM,PROMPT,FOUROK,IRTFLG)

C       NO NEED TO COMPILE WITH STATIC
CNO_SAVE

        INCLUDE 'CMBLOCK.INC'

        COMMON /STACKCOM/NSTACK

        CHARACTER *(*)   FILNAM,PROMPT,DISP
        CHARACTER        NULL
        LOGICAL          ASKNAM,FOUROK

        PARAMETER        (NBUFSIZ = 17008)
        COMMON /IOBUF/   BUF(NBUFSIZ)

	NULL = CHAR(0)

C       CAN PASS NEGATIVE -9 IRTFLG TO OPENFB TO KEEP EXISTING DATE/TIME
C       VALUES WHEN OPENING NEW FILE
C       CAN PASS POSITIVE 88 IRTFLG TO OPENFB TO OPEN A MALFORMED STACK
        NF = 0
        IF (IRTFLG .LT. 0 .OR. IRTFLG .EQ. 88) NF = IRTFLG

        IF (ASKNAM) THEN
C          SOLICIT FILE NAME
           CALL FILERD(FILNAM,NLETI,NULL,PROMPT,IRTFLG)
           IF (IRTFLG .EQ. -1) RETURN
        ENDIF

        IF (DISP(1:1) .NE. 'O' .AND. DISP(1:1) .NE. 'Z' .AND.
     &      DISP(1:1) .NE. 'E') THEN
C         NEED TO OPEN A NEW FILE

C         IF NSAM OR NROW IS NOT LOGICAL ASK FOR THEM
          IF (NSAM .LE. 0 .OR. NROW .LE. 0) THEN
C            NEED TO FIND NSAM & NROW FOR NEW FILE
             CALL RDPRMI(NSAM,NROW,NLOOP,ILOOP,
     &                   'ENTER DIMENSIONS (NSAM,NROW)')
          ENDIF
          IF (NROW .LE. 0) NROW = NSAM

          IF (IFORM .EQ. 3 .AND. NSLICE .LE. 0) THEN
C            NEED TO INQUIRE AS TO NSLICE
             CALL RDPRMI(NSLICE,NDUM,NLOOP,ILOOP,
     &                  'ENTER NUMBER OF SLICES (NSLICE)')
          ENDIF
        ENDIF

C       OPEN THE FILE, USES IOBUF BUF HERE NOW, NOT UNLABELED COMMON!
        CALL OPENALL(BUF,FILNAM,LUN,NSAM,NROW,NSLICE,DISP,NF)

        IF (NF .EQ. 88) THEN
C          OPENING MALFORMED STACK FOR USE IN "CP FIX" 
           IRTFLG = NF
           RETURN

        ELSEIF (NF .EQ. 1) THEN
C          ERROR OPENING FILE -- OPEN ERROR
           IRTFLG = NF
           RETURN

        ELSEIF (NF .EQ. 2) THEN
C          FILE OPENED OK, CHECK FOR EXISTING FOURIER OK

           IF ((DISP(1:1) .EQ. 'O' .OR. DISP(1:1) .EQ. 'Z' .OR. 
     &          DISP(1:1) .EQ. 'E') .AND.
     &          IFORM .LT. 0 .AND. .NOT. FOUROK) THEN
C             CAN NOT USE AN EXISTING FOURIER FILE
              CALL ERRT(2,'OPFIL ',NE)
              IRTFLG = 1
              RETURN  
           ENDIF

        ELSEIF (MAXIM .EQ. 0 .AND. NSTACK .GE. 0) THEN
C          THIS OPERATION DOES NOT ACCEPT WHOLE STACKS 
           CALL ERRT(101,'OPERATION DOES NOT WORK ON WHOLE STACKS',NE)
           IRTFLG = 1
           RETURN
        ENDIF

C       RETURN NSTACK
        MAXIM   = NSTACK
        IRTFLG  = 0

        RETURN
        END
