
C++*************************************************************************
C
C ORNTS                  FILENAMES LENGTHENED DEC 27 88 al
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981, DIVISION OF LABORATORIES AND                       *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C ORNTS(FILNAM,FILNM1,LUN1,LUN2,NSAM,NROW,MAXDIM)
C
C PURPOSE:     FINDS ORIENTATION BETWEEN TWO CENTERED IMAGES USING
C              ONE-DIMENSIONAL CROSS-CORRELATION ALONG INTERPOLATED
C              DATA ON CIRCLES AROUND (NSAM/2+1,NROW/2+1).
C
C PARAMETERS:
C
C        FILNAM    CHAR. VARIABLE CONTAINING NAME OF FIRST FILE
C        FILNM1    CHAR. VARIABLE CONTAINING NAME OF SECOND FILE
C        LUN1      LOGICAL UNIT NUMBER OF FIRST FILE
C        LUN2      LOGICAL UNIT NUMBER OF SECOND FILE
C        NSAM      NUMBER OF SAMPLES
C        NROW      NUMBER OF ROWS
C        MAXDIM    MAXIMUM BUFFER SPACE AVAILABLE
C
C--*********************************************************************

      SUBROUTINE ORNTS(FILNAM,FILNM1,LUN1,LUN2,NSAM,NROW,MAXDIM)

      INCLUDE 'CMBLOCK.INC'
CNO_SAVE

      COMMON         JLIST(2048),BUF(20000)

      INTEGER        A1,A2,A3
      REAL           MU1,MU2,W(20),RA(20),FWA(20)
      CHARACTER *(*) FILNAM,FILNM1
      CHARACTER *2   IGR
      CHARACTER      NULL,CDUM

      EQUIVALENCE    (NS1,NSEL(1))                    
      EQUIVALENCE    (NS2,NSEL(2))
      EQUIVALENCE    (NS3,NSEL(3))

      DATA PI/3.141593/
      DATA AMIN/-10E30/, AMAX/10E30/

      NULL = CHAR(0)

C     CATCH NON-POWER-OF-TWO DIMENSION
      IF (NSAM.NE.2**LOG2(NSAM))  THEN
         CALL  ERRT(10,'ORNTS',NE)
         RETURN
      ENDIF

      W(1)=.1
      DO  I=2,20
         W(I)=1.
      ENDDO

      FNORM = 1./(FLOAT(NSAM)*FLOAT(NROW)*1.E6)
      L   = NSAM*2
      IF (FCHAR(4:4) .EQ. 'F') L=L*2
      LP2 = L*2
      A1  = L*4
      A2  = A1+LP2
      A3  = A2+LP2
      IB1 = A3+LP2+1
      IB2 = IB1+LP2
      IB3 = IB2+LP2

C     PRELIMINARY FIX FOR FLOATING POINT OVERFLOW IN FOURIER 
C     MULTIPLICATION. LP2 IS TOO LARGE. M.R.
 
      DO  K = 1,IB3+LP2
        BUF(K) = 0.
      ENDDO

      RCP = 1./FLOAT(L)
      CALL RDPRMI(NRING,IWGT,NLOOP,ILOOP,
     &   'NO. OF RINGS, WEIGHT OVERRIDE (1/0)')
	IF (NRING.GT.20)NRING=20

      CALL RDPRMC(IGR,NCHAR,.TRUE.,
     &   'GRAPH F. EACH R.? (Y/N), (A)BSOL. SCALE',CDUM,IRTFLG)

      IF (IGR(2:2).EQ.'A') THEN
        CALL RDPRM(SCALE,NLOOP,ILOOP,'ENTER SCALE FACTOR')
      ENDIF
      IF (A3+L .GE. MAXDIM) THEN
         CALL ERRT(6,'ORNTS',NE)
         RETURN
      ENDIF

28    NALL = LP2*NRING
      DR = FLOAT(3*NSAM)/FLOAT(8*NRING)
      DO  K = 1,NRING
        RA(K) = DR*FLOAT(K)
      ENDDO
      IF (IWGT.NE.1) GOTO 29   

      NVAL = NRING
      IRTFLG = 0
      CALL RDPRAF(FWA,NRING,NVAL,AMIN,AMAX,'ENTER RADII',NULL,IRTFLG)

      DO K = 1, NRING
        RA(K) = FWA(K)
      END DO

      WRITE(NECHO,287)(RA(K),K=1,NRING)

      NVAL = NRING
      IRTFLG = 0
      CALL RDPRAF(FWA,NRING,NVAL,AMIN,AMAX,'ENTER WEIGHTS',NULL,IRTFLG)

      DO K = 1, NRING
         W(K) = FWA(K)
      END DO

      WRITE(NECHO,287)(W(K),K=1,NRING)
287   FORMAT(5X,10F7.2)

      WAV = 0.
      DO  K=1,NRING
         WAV = WAV+W(K)
      ENDDO 
      DO  K = 1,NRING
         W(K) = W(K)/WAV
      ENDDO
29    MU1 = 0
      MU2 = 0
      SIG1 = 0
      SIG2 = 0
      SW = 0
      DO  I = 1,L
        BUF(I) = 0.
      ENDDO

C     FULL CIRCLE OPTION 10/19/81
      DTH=PI/FLOAT(L)
      IF (FCHAR(4:4) .EQ. 'F') DTH=DTH*2
      IF (FCHAR(4:4) .EQ. 'F') RCP=RCP*2

      S = 0.
      IF (IGR(2:2) .EQ. 'A') S = SCALE
      DO NR = 1,NRING
         R = RA(NR)
         CALL CRCSEC(LUN1,BUF,NSAM,NROW,BUF(A1+1),L,R,DTH)
         CALL CRCSEC(LUN2,BUF,NSAM,NROW,BUF(A2+1),L,R,DTH)
         T1 = 0
         T2 = 0
C        WRITE(3,82) (BUF(K+A2),K=1,LP2)
         DO  I = 1,L
            T1 = T1+BUF(I+A1)**2*W(NR)**2
            T2 = T2+BUF(I+A2)**2*W(NR)**2
         ENDDO
         SIG1 = SIG1 + T1*R
         SIG2 = SIG2 + T2*R
         CALL FOUR2(BUF(A1+1),L,1,+1,0)
         CALL FOUR2(BUF(A2+1),L,1,+1,0)
C        ACTIVATED TO CHECK LP2
C        WRITE(3,82)(BUF(K+A1),K=1,LP2)
C        WRITE(3,82)(BUF(K+A2),K=1,LP2)
         MU1 = MU1 + BUF(A1+1)*R*W(NR)
         MU2 = MU2 + BUF(A2+1)*R*W(NR)
         DO  I = 1,LP2,2
            I1 = I
            I2 = I+1
            SH = (-1)**(I/2)
            T1 = (BUF(I1+A1)*BUF(I1+A2) + BUF(I2+A1)*BUF(I2+A2))*SH
            BUF(I2+A1) = 
     &         (BUF(I1+A2)*BUF(I2+A1)-BUF(I2+A2)*BUF(I1+A1))*SH
            BUF(I1+A1) = T1
         ENDDO
C        WRITE(3,82) ( BUF(K+A1),K=1,LP2)
         CALL FOUR2(BUF(A1+1),L,1,-1,-1)
         DO  K = 1,L
             BUF(A1+K) = BUF(A1+K)*R*W(NR)
         ENDDO
C        WRITE(3,82)(BUF(A1+K),K=1,L)
         DO  I = 1,L
           BUF(I+A3) = BUF(I+A3) + BUF(A1+I)
         ENDDO
         SW = SW + R
         IF (IGR(1:1) .NE. 'Y') GOTO 50
         CALL GRAPHO(BUF(A1+1),L,1,S)
         WRITE(NDAT,49) NR,R,W(NR),S
49       FORMAT(' RING NO.',I3,'  RADIUS = ',F5.1,' UNITS',
     1    ' REL. WEIGHT = ',F7.5,'  SCALE = ',G12.3)
C        WRITE(NDAT,82)(BUF(A3+K),K=1,L)
50       CONTINUE
         R = R + DR
      ENDDO
      FL = FLOAT(L)
      SW = SW*FL
      SIG1 = FL*SQRT((SIG1-MU1**2/SW) * (SIG2-MU2**2/SW))
      MU1 = FL*MU1*MU2/SW
      DO  I = 1,L
        BUF(I+A1) = (BUF(A3+I)-MU1)/SIG1
      ENDDO
C     WRITE(3,82)(BUF(K+A1),K=1,L)
      IF (FCHAR(4:4) .EQ. 'M' .OR. FCHAR(5:5) .EQ. 'M')THEN
         CALL ORPKS(A1,A2,L,BUF(IB1),JLIST,NP,BUF(IB2),BUF(IB3))
         RETURN
      ENDIF

      T1 = BUF(A1+1)
      DO I = 1,L
         T = BUF(A1+I)
         IF (T .GE. T1) THEN
            T1 = T
            J = I
         ENDIF
      ENDDO

      IF (J+3.LE.L.AND.J-3.GE.1)GOTO 75
C     CIRCULAR CONTINUATION. IT IS ASSUMED THAT THIS CONTINUATION IS
C     ONLY NECESSARY IN ACF PASS.
      DO  I=L+1,L+3
        BUF(A1+I)= BUF(A1+I-L)
      ENDDO
      DO  I=-2,0
        BUF(A1+I)=BUF(A1+I+L)
      ENDDO

C     PARABOLIC FIT ESTIMATION
75    CALL PARAB1(BUF(A1+J-3),7,POS1)
      POS=POS1+J-1
C     WRITE(NDAT,7599)(BUF(A1+J-4+K),K=1,7),POS1,POS
7599  FORMAT(9G12.4)
      THETA1= (POS*RCP-0.5)*PI
      IF (FCHAR(4:4) .EQ. 'F') THETA1=THETA1-PI/2.
      THETD1= THETA1*180./PI

      CALL GRAPHO(BUF(A1+1),L,1,0.)
      CALL PDATES(FILNAM,0)
      CALL PDATES(FILNM1,0)
      WRITE(NDAT,81) THETA1,THETD1
81    FORMAT(/'  ANGLE BETWEEN PICTURES ',F7.3,' RAD  = ',
     1       F8.1,' DEGREES'/)
      IF (NOUT.NE.NDAT) WRITE(NOUT,83) THETD1
83    FORMAT(' ** ROTATION ANGLE ',F8.1,' DEGREES')
      IF (NS1 .EQ. 0) RETURN
      PARAM(NS1) = THETD1
      IF (NS2.NE.0) PARAM(NS2)=BUF(A1+J-3)
82    FORMAT(1X,10(G10.4,1X))

      RETURN
      END
