
C++************************************************************************
C
C  PATCH.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C   PATCH(LUN2,LUN1,NSAM2,NROW2,NSAM1,NROW1,NSAMS,NROWS,IN,AV,OPT)
C
C   PURPOSE: PATCH, PAD OR INSERT IMAGE
C
C      THIS SUBROUTINE TAKES IMAGE STORED ON LUN1 AND ADDS IT ONTO OR
C      INSERTS IT INTO IMAGE STORED ON LUN2 AT A SPECIFIED LOCATION.
C
C   PARAMETERS:
C        LUN1          INPUT IMAGE
C        LUN2          SECOND INPUT IMAGE (= OUTPUT IMAGE)
C        NSAM1,NROW1   DIMENSIONS OF IMAGE 1
C        NSLIC1
C        NSAM2,NROW2   DIMENSIONS OF IMAGE 2 (NSAM2 < OR = NSAM1)
C        NSLIC2
C                                            (NROW2 < OR = NROW1)
C        NSAMS,NROWS   TOP LEFT COORDINATES OF PATCH WITH RESPECT TO
C        NSLIC2        BIG IMAGE
C        IN            0   PATCH
C                      1   INSERT
C                      2   PAD: CREATE IMAGE ON LUN1 WITH VALUE AV 
C                          AND IMAGE FROM LUN2 INSERTED.
C                          IT WORKS FOR 3-D FILES.
C			3  SAME AS FOR 2 BUT WITH CIRCULAR CONTINUATION
C        AV            OUTSIDE VALUE, 
C
C--*******************************************************************

      SUBROUTINE PATCH(LUN2,LUN1,NSAM2,NROW2,NSLIC2,
     1                 NSAM1,NROW1,NSLIC1,NSAMS,NROWS,NSLICS,IN,AV,OPT)

      
      COMMON BUF(1)
      COMMON/MASTER/NSAMC,NROWC,IREC,NLABEL,IFORM,IMAMI,FMAX,FMIN,
     1  AVC,SIG,IHIST

      COMMON /UNITS/LUNC,NIN,NOUT
      COMMON /PARAM/PARAM(103),NS1

      INTEGER*4   I4,KKKCOU
      CHARACTER   OPT

      NROW2H = NROW2/2
      NSAM2H = NSAM2/2
      NSAMP  = NSAM1+1

      NS     = NROWS
      NE     = NS + NROW2-1

C     SKIP DIRECTLY TO CIRCULAR PADDING IF DESIRED
      IF (IN .EQ. 3) GOTO 90

      IF (NS .LE. 0)     NS = 1
      IF (NE .GT. NROW1) NE = NROW1

      KS = NSAMS
      KE = KS + NSAM2 - 1
      IF (KS .LE. 0)     KS = 1
      IF (KE .GT. NSAM1) KE = NSAM1

      KKS = NSLICS        
      KKE = KKS + NSLIC2-1 
      IF (KKS .LE. 0)      KKS = 1
      IF (KKE .GT. NSLIC1) KKE = NSLIC1

      IF (IN .EQ. 0) THEN
C        FOR PATCH OPERATION
         DO  I = NS,NE
            CALL REDLIN(LUN1,BUF,NSAM1,I)
            CALL REDLIN(LUN2,BUF(NSAMP),NSAM2,I-NS+1)
            DO K = KS,KE
               BUF(K) = BUF(K)+BUF(NSAMP+K-KS)
            ENDDO
            CALL WRTLIN(LUN1,BUF,NSAM1,I)
         ENDDO
         RETURN

      ELSEIF (IN .EQ. 1) THEN
C       FOR INSERT OPERATION

60	IF (OPT .EQ. 'S') THEN 
	   FMINS = AV-3*SIG
	   IF (FMINS.LT.FMIN) FMINS=FMIN
	   FMAXS = AV+3*SIG
	   IF (FMAXS.GT.FMAX) FMAXS=FMAX
	   SCALE = 2./(FMAXS-FMINS)

        ELSEIF (OPT .EQ. 'M') THEN
	   FMINS=FMIN
	   FMAXS=FMAX
	   SCALE=2./(FMAXS-FMINS)
        ENDIF

        DO I = NS,NE
           CALL REDLIN(LUN1,BUF,NSAM1,I)
           CALL REDLIN(LUN2,BUF(NSAMP),NSAM2,I-NS+1)
           IF (OPT .NE. 'S' .AND. OPT .NE. 'M') THEN
              DO K = KS,KE
                 BUF(K) = BUF(NSAMP+K-KS)
              ENDDO
           ELSE
  	      DO K = KS,KE
  	         BUF(K) = (BUF(NSAMP+K-KS)-FMINS)*SCALE
  	         IF (BUF(K).LT.0.)BUF(K) = 0.0
  	         IF (BUF(K).GT.2.)BUF(K) = 2.0
              ENDDO
           ENDIF
           CALL WRTLIN(LUN1,BUF,NSAM1,I)
        ENDDO
        RETURN

      ENDIF

C     FOR NON-CIRCULAR PADDING -------------------------------------
C     MODIFIED FOR 3D PADDING (11/11/86. JMC)
C     FIRST FILL WITH AV ALL THE PLANES BETWEEN 1 AND (NSLICS-1)
C     AV MAY BE THE AVERAGE IN THE OUTER TWO-PLANES OF THE VOLUME

90    CONTINUE
      IF (AV .LT. 99999) GOTO 118 

C     IF NOT CALCULATE THE MEAN OF THE 2 OUTSIDE VOXELS OF THE FILE 
C     (IT IS DONE THIS WAY TO AVOID PASSING A NEW VARIABLE IN A 
C      PROGRAM THAT IS CALLED SOMEWHERE ELSE ALSO).

      KKKCOU=0
      RRRBOR=0
      IF (NSLIC2.NE.1) THEN
         DO  KKSLIC=1,2
            DO  KKROW=1,NROW2
               CALL REDLIN (LUN2,BUF,NSAM2,(KKSLIC-1)*NROW2+KKROW)
               CALL REDLIN (LUN2,BUF(NSAM2+1),NSAM2,(NSLIC2-(KKSLIC-1))*
     1                     NROW2+KKROW)
               DO  KKSAM=1,2*NSAM2
                  RRRBOR=RRRBOR+BUF(KKSAM)                    
                  KKKCOU=KKKCOU+1 
	       ENDDO
	    ENDDO
	 ENDDO
         DO  KKSLIC=3,NSLIC2-2
            DO  KKROW=1,NROW2
               CALL REDLIN (LUN2,BUF,NSAM2,(KKSLIC-1)*NROW2+KKROW) 
               RRRBOR=BUF(1)+BUF(2)+BUF(NSAM2-1)+BUF(NSAM2)+RRRBOR
               KKKCOU=KKKCOU+4
	    ENDDO
	 ENDDO
       ELSE
         DO  KKROW=1,2      
            CALL REDLIN (LUN2,BUF,NSAM2,KKROW)
            CALL REDLIN (LUN2,BUF(NSAM2+1),NSAM2,NROW2-(KKROW-1))
            DO  KKSAM=1,2*NSAM2
               RRRBOR=RRRBOR+BUF(KKSAM)
               KKKCOU=KKKCOU+1
	    ENDDO
	 ENDDO
         DO  KKROW=3,(NROW2-2) 
            CALL REDLIN (LUN2,BUF,NSAM2,KKROW)
            RRRBOR=RRRBOR+BUF(1)+BUF(2)+BUF(NSAM2)+BUF(NSAM2-1)
            KKKCOU=KKKCOU+4                      
	 ENDDO
       ENDIF
       AV=(RRRBOR/KKKCOU)

C      JUST TO KNOW THE OUTER AVERAGE
       WRITE (NOUT,3555) AV
3555   FORMAT(' OUTER AVERAGE=',F15.5)

C     WRITE INTO REGISTER THE OUTER AVERAGE 
      IF (NS1.NE.0) PARAM(NS1)=AV    
118   IF (IN.EQ.3) GOTO 300


      IF (KKS.EQ.1) GOTO 125
      KKS1=KKS-1
      DO  JJ=1,NSAM1
          BUF(JJ)=AV
      ENDDO
      DO  KK=1,KKS1
         DO  K=1,NROW1
            CALL WRTLIN (LUN1,BUF,NSAM1,K+(NROW1*(KK-1)))
	 ENDDO
      ENDDO
125   CONTINUE     

C  THE PLANES BETWEEN NSLICS AND NSLICS WILL BE REALLY PADED

      DO 130 KK=KKS, KKE

         IF (NS.EQ.1) GOTO 150
         NS1=NS-1
         DO  I=1,NS1
            DO  K=1,NSAM1
                BUF(K)=AV
	    ENDDO
            CALL WRTLIN (LUN1,BUF,NSAM1,I+(NROW1*(KK-1)))
	 ENDDO
150      CONTINUE

         DO  I = NS,NE
            IF (KS.EQ.1) GOTO 160
            KS1 = KS - 1
            DO  K = 1,KS1
              BUF(K)= AV
	    ENDDO
160         CALL REDLIN
     1         (LUN2,BUF(NSAMP),NSAM2,(I-NS+1)+(NROW2*(KK-KKS)))
            DO  K = KS,KE
               BUF(K) = BUF(NSAMP+K-KS)
	    ENDDO
            IF (KE.EQ.NSAM1) GOTO 179
            KE1 = KE + 1
            DO  K = KE1,NSAM1
               BUF(K) = AV
	    ENDDO
179         CALL WRTLIN(LUN1,BUF,NSAM1,I+(NROW1*(KK-1)))
	 ENDDO

         IF (NE.EQ.NROW1) GOTO 130
         IE1 = NE + 1
         DO  I = IE1,NROW1
            DO  K = 1,NSAM1
               BUF(K) = AV
	    ENDDO
         CALL WRTLIN(LUN1,BUF,NSAM1,I+(NROW1*(KK-1)))
	 ENDDO

130   CONTINUE

C     FILL THE REST OF THE PLANES BETWEEN NSLICS AND NSLIC1 WITH AV 

       IF (KKE.EQ.NSLIC1) GOTO 175

       KKE1=KKE+1
       DO  I=1,NSAM1
          BUF(I)=AV
       ENDDO

      DO  KK=KKE1,NSLIC1
         DO  K=1,NROW1
            CALL WRTLIN (LUN1,BUF,NSAM1,K+(NROW1*(KK-1)))
	 ENDDO
      ENDDO

175   CONTINUE     
      RETURN




C INDEX ASSIGNMENTS FOR CIRCULAR PADDING ---------------------------
C
C	     KS2        KE2              KS1        KE1
C	NS2  I----------------------------------------I
C	     I          .                .
C	     I          .                .
C	     I          .                .
C	NE2  I...........                .............
C	     I
C	     I
C	     I
C	NS1  I...........
C	     I          .
C	     I          .
C	NE1  I------------------------------------------I
C
C
300   NE2 = 0
      NS2 = 0
      IF (NS.LT.0) NS = NS+NROW1
      NS1 = NS
      NE1 = NS+NROW2-1
      NPASS = 1
      IF (NE1.LE.NROW1) GOTO 320
      NS2 = 1
      NE2 = NE1-NROW1
      NE1 = NROW1
320   KS = NSAMS
      KE2 = 1
      KS2 = 0
      IF (KS.LT.0) KS = KS+NSAM1
      KS1 = KS
      KE1 = KS+NSAM2-1
      IF (KE1.LE.NSAM1) GOTO 3205
      KS2 = 1
      KE2 = KE1 - NSAM1
      KE1 = NSAM1
3205  ISTART = NS2
      IEND = NE2
      IFILLS = NE2+1
      IFILLE = NS1-1
      IRECOF = NROW1-NROWS+1
      NPASS = 1
321   IF (ISTART.EQ.IEND) GOTO 350
      DO 340 I = ISTART,IEND
         IRECIN = I-ISTART+IRECOF+1
         CALL REDLIN(LUN2,BUF(NSAMP),NSAM2,IRECIN)
         IF (KS2.EQ.0) GOTO 330
         NOFFS = NSAMP+KE2-2*KS2
         DO  K = KS2,KE2
           BUF(K) = BUF(K+NOFFS+1)
	 ENDDO

         DO  K = KE2+1,KS1-1
           BUF(K) = AV
	 ENDDO
330      NOFFS = NSAMP-KS1
         DO  K = KS1,KE1
           BUF(K) = BUF(K+NOFFS)
	 ENDDO

         IF (KE1.EQ.NSAM1) GOTO 340
         DO  K = KE1+1,NSAM1
           BUF(K) = AV
	 ENDDO

340   CALL WRTLIN(LUN1,BUF,NSAM1,I)

350   IF (IFILLS.EQ.IFILLE) GOTO 390
      DO  I = IFILLS,IFILLE
         DO  K = 1,NSAM1
           BUF(K) = AV
	 ENDDO
      CALL WRTLIN(LUN1,BUF,NSAM1,I)
      ENDDO

390   GOTO (400,410),NPASS
400   NPASS  = 2
      ISTART = NS1
      IEND   = NE1
      IFILLS = NE1 +1
      IFILLE = NROW1
      IRECOF = 0
      GOTO 321

410   RETURN
      END
