
C++************************************************************************
C   
C
C **************************************************************************
C *  AUTHOR :                                                              *
C *  COPYRIGHT (C)1987 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C
C	AUTHOR: MAHIEDDINE LADJADJ          11/1/93
C
C	PURPOSE:
C	THIS ROUTINE DOES THE PRINCIPAL COMPONENT ANALYSIS.
C
C	NFAC 	NUMBER OF EIGENVECTORS REQUESTED
C       NUMIM	NUMBER OF IMAGES
C       NACT    NUMBER OF ACTIVE IMAGES
C	NPIX    NUMBER OF PIXELS PER IMAGE
C	LSAV	SEQUENTIAL IMAGE FILE (INPUT)
C	LIMA    IMAGE COORDINATE FILE (OUTPUT)
C	LPIX    PIXEL COORDINATE FILE (OUTPUT)
C	LEIG	EIGENSTUFF FILE (OUTPUT)
C	LTMP	SCRATCH FILE
C
C       ALL FILES MUST BE UNFORMATTED!
C	Q	ARRAY OF FOUR - BYTE WORDS WITH DIMENSION MEM
C	LUV	LOGICAL UTILITY VECTOR (1 = ACTIVE,  0 = INACTIVE)
C         
C--*********************************************************************

        SUBROUTINE PCA_MAIN(NUMIM, NFAC, NSAM, NROW, NPIX, LSAV, 
     &	                   LIMA, LPIX, LEIG, Q, MEM, LUV, 
     &                     IMAR, MADAR, NACT, NOUT)

        DIMENSION  LUV(*), IMAR(*), MADAR(*), Q(*)
        INTEGER K, LTRANSP , LTMP, NFAC0, ANS, SIZE1, SIZE2, TSIZE
        DIMENSION B(1)
        POINTER   (IPQ,B)
        LOGICAL   RUNTIME 
        DATA  LTMP, LTRANSP/78, 79/
CNO_SAVE           

        NINACT = NUMIM - NACT 
        NFAC0 = NFAC
        NFAC  =  MIN0(NFAC, NACT, NPIX) 
        IF (NFAC .NE. NFAC0) THEN
          WRITE(NOUT,12) NFAC
12        FORMAT(' WARNING: NUMBER OF FACTORS LIMITED TO: ',I4)
        END IF

C
C       CREATE HEADERS FOR  FILES IMC ***  AND PIXEL *** 
C
        CALL REW(LIMA, 0)
        WRITE(LIMA) NUMIM, NFAC, NSAM, NROW
        CALL REW(LPIX, 0)
        WRITE(LPIX) NPIX, NFAC, NSAM, NROW

c	Memory needed. In-core
        SIZE1 = NPIX * NPIX + 4 * NPIX +  NFAC

c	Memory needed. Transposed
c	in  pcaxpose: T(npix,nact) needed, LU() used with nact, then
c       npix dimension. LD() also.  so change next line
c       SIZE2 = NPIX + MAX0(NACT,NPIX)  + 2 * NACT + NACT * NACT + NFAC
        SIZE2 = 2 * NPIX + MAX0(NACT,NPIX) * (NACT + 2) + NFAC

C       TRY TO DO SOME CALCULATIONS TO CHOOSE THE QUICKEST METHOD
C       ACCORDING TO THE SIZE OF THE PROBLEM
	
        TSIZE = MIN0(SIZE1,SIZE2)
	WRITE(NOUT,7891)  SIZE1,SIZE2,MEM
7891	FORMAT('  MEMORY NEEDED. IN-CORE:',I8,'  TRANSPOSED:',I8,
     &     ' AVAILABLE:',I8)

C	DYNAMIC ALLOCATION OF MEMORY POSSIBLE ?

        IF(200*NFAC .LT. MIN0(NPIX,NACT)) THEN
c	   small number of factors: GOTO iterative method	
           GOTO 310
	ELSE
	   CALL CHKMEM(TSIZE,Q,MEM,RUNTIME,IPQ,IRTFLG)
	   IF(IRTFLG.EQ.0)  THEN  
	      IF(TSIZE.EQ.SIZE1) GOTO 110
	      GOTO 210
	   END IF 
	ENDIF
	GOTO 310


C 
C         I N  -  C O R E   S O L U T I O N
C 
C
C              MEMORY SEGMENTATION POINTERS
C
110     LD      =  1
        LS      =  LD  +  NPIX
        LU      =  LS  +  NPIX * NPIX
        LPJ     =  LU  +  NPIX
        LCO     =  LPJ +  NPIX
        LW      =  LCO +  NPIX
        LMAX    =  LW  +  NFAC

        WRITE(NOUT,*) ' IN - CORE SOLUTION CHOSEN'
  
C	B(LCO) IS TEMPORARELY USED TO STORE STANDARD DEVIATIONS
        CALL PCA_INCOR3(NUMIM, NPIX, NACT, NFAC, B(LS), B(LD), 
     &       B(LU), B(LPJ), TRACE, SOM, LSAV, LUV)

        WRITE(LEIG) NFAC, SOM, TRACE
                                                      
C       STORE THE EIGENVALUES                       
        WRITE(LEIG) (B(LD + K), K = 0, NFAC - 1)

C       STORE PIXEL AVERAGES
        WRITE(LEIG) (B(LPJ + K), K = 0, NPIX -1)
                
        CALL  PCA_COORX(NFAC, NPIX, NPIX, NUMIM, B(LS), B(LD), 
     &	     TRACE, B(LPJ), SOM, B(LCO), B(LU), B(LW),
     &		      LSAV, LIMA, LPIX, LUV, MADAR)
     
        IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'pca_main',IRTFLG)
	 

        RETURN
   
C
C       T R A N S P O S E D   S O L U T I O N
C
C       MEMORY SEGMENTATION POINTERS         
210     LPJ      =  1
        LCO      =  LPJ +  NPIX
        LD       =  LCO +  NPIX
        LS       =  LD  +  MAX0(NACT,NPIX)
        LU       =  LS  +  NACT * MAX0(NACT,NPIX)
        LW       =  LU  +  MAX0(NACT,NPIX)
        LMAX     =  LW  +  NFAC
        IF(NACT .EQ. NUMIM)  THEN
          WRITE(NOUT,*)
     &     ' IN - CORE SOLUTION USING TRANSPOSED DATA'

          OPEN(FILE = 'TRANSPOSED.FILE', UNIT = LTRANSP, 
     &			STATUS = 'UNKNOWN', FORM = 'UNFORMATTED')
 
C         FOR THIS CALL, B(LD) IS USED AS A TEMPORARY STORAGE
          CALL PCAXPOSE(B(LD),B(LPJ), NPIX, NACT,
     &                     NUMIM,LUV, LSAV,LTRANSP)
                                                   
          CALL PCAXPROD(NPIX,NACT,NPIX,NFAC,B(LS),B(LD), 
     &         B(LU), TRACE, SOM, LTRANSP, MADAR)

          WRITE(LEIG) NFAC, SOM, TRACE, NACT

C         STORE EIGENVALUES
          WRITE(LEIG) (B(LD + K), K = 0, NFAC - 1)

C         STORE PIXEL AVERAGES
          WRITE(LEIG) (B(LPJ + K), K = 0, NPIX - 1)
                         
          CALL  PCAXCOOR(NFAC, NACT, NACT, NPIX, B(LS), B(LD), 
     &             TRACE, B(LPJ), SOM, B(LCO), B(LU), B(LW),
     &             LTRANSP,LPIX, LIMA, MADAR, LUV)

          IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'pca_main',IRTFLG)
 
          CLOSE(LTRANSP, STATUS = 'DELETE')
          RETURN
        ELSE
          WRITE(NOUT,*)' MEMORY SUFFICIENT FOR IN - CORE SOLUTION '
          WRITE(NOUT,*)' USING TRANSPOSED DATA. UNFORTUNATELY '
          WRITE(NOUT,*)' CANNOT HANDLE INACTIVE IMAGES'
          WRITE(NOUT,*)' ***>>> TRYING ITERATIVE SOLUTION '
 
          IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'pca_main',IRTFLG)
 

        END IF
      
C 
C       ITERATIVE   SOLUTION
C 

310     JBASE  =  NFAC  +  3
        LBASE  =  JBASE
        KB     =  LBASE
C	
C	MEMORY SEGMENTATION POINTERS
C
        LS	 =  1
        LD	 =  LS   +  NPIX * LBASE
        LPJ	 =  LD   +  NPIX
        LCO	 =  LPJ  +  NPIX
        LU	 =  LCO  +  NPIX
        LV	 =  LU   +  NPIX
        LW	 =  LV   +  NPIX
        LBZ	 =  LW   +  NFAC
        LAD	 =  LBZ  +  NPIX * LBASE
        LINB     =  LAD  +  LBASE * LBASE
        LMAX     =  LINB +  LBASE

	CALL CHKMEM(LMAX,Q,MEM,RUNTIME,IPQ,IRTFLG)
	IF(IRTFLG.NE.0)  THEN  
          WRITE(NOUT,* )
     &       ' *** MEMORY INSUFFICIENT FOR ITERATIVE SOLUTION'
          WRITE(NOUT,*) '    NEEDED ', LMAX, ' AVAILABLE ', MEM
          CALL ERRT(100, 'PCA_MAIN', NE)
          RETURN
        ENDIF

3100    OPEN(LTMP, FILE = 'SCR999.DAT', STATUS = 'UNKNOWN', 
     &             FORM = 'UNFORMATTED')
     
        CALL PCA_STOCHA(JBASE,NPIX,NUMIM,NFAC,NACT,NITER,LUV,B(LPJ), 
     &	            B(LS), B(LU), TRACE, SOM, B(LV), B(LBZ),
     &	            B(LD), B(LAD), B(LINB), LSAV, LTMP)

        CLOSE(LTMP, STATUS = 'DELETE')

C       Put eigenvectors in decending order ...
        CALL  ORDERE(NPIX,NFAC,B(LS),B(LD))
  
        CALL PCASCOOR(NFAC, KB, NPIX, NUMIM, B(LS), B(LD), 
     &              TRACE, B(LPJ), SOM, B(LCO), B(LU), B(LW),
     &	            LSAV, LIMA, LPIX, LUV, MADAR)
        
        WRITE(LEIG) NFAC, SOM, TRACE
        WRITE(LEIG) (B(LD + K), K = 0, NFAC - 1)
        WRITE(LEIG) (B(LPJ + K), K = 0, NPIX- 1)
	
        IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'pca_main',IRTFLG)
 
        RETURN
        END
