
C***********************************************************************
C
C  POSTRT.FOR        CREATED AUG 88
C                    UPDATED JULY 92 al
C
C***********************************************************************
C      AUTHOR:   ARDEAN LEITH
C      COPYRIGHT (C) 1988, WADSWORTH CENTER FOR LABORATORIES AND 
C      RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY, 12201
C***********************************************************************
C
C  POSTRT
C
C       PURPOSE:   CONTAINS THE PLOTTING COMMANDS FOR POSTSCRIPT
C                  PLOTTER.  
C
C       ENTRY POINTS ARE:
C
C             POSTRT(IPLUN)
C                  START PLOT OUPTUT ON UNIT NO. IPLUN
C    
C             POMOVE(IPLUN,X,Y)
C                  MOVE PEN TO (X,Y) WITHOUT DRAWING
C
C             PODRAW(IPLUN,X,Y)
C                  DRAW A LINE FROM CURRENT PEN POSITION TO (X,Y)
C
C             POEND(IPLUN)
C                  TO END A PLOTTING SEQUENCE
C  
C             POFONT(IPLUN,IFONT)
C                  TO CHANGE CHARACTER FONTS
C
C             POLINE(IPLUN,LINTYP)
C                  TO CHANGE LINE TYPE, LINTYP IS LINE TYPE (1-10)
C                  USED FOR BOTH TEXT AND LINES
C
C             POARAY(IPLUN,IDATA,NDATA,CLOSD,FILLED)
C                  DRAW A POLYGON. (USE POPATH TO START IT)
C
C             POTEX(IPLUN,TXLINE,ICHAR,X,Y,ITSIZE,ITANGL,JUST)
C                  PRINTS TEXT AT (X,Y) WITH SIZE=ITSIZE, ANGLE=ITANGL,
C                  AND JUSTIFICATION CODE=JUST
C
C             POTEXT(IPLUN,TEXT)
C                  PRINTS TEXT
C
C             POSIZE(IPLUN,ITSIZE)
C                  SET TEXT SIZE,  DEFAULT SIZE IS 12 POINT
C
C             POWIND(IPLUN,WXMIN,WYMIN,WXMAX,WYMAX)
C                   SETS OUTPUT CLIPPING WINDOW
C
C             POPATH(IPLUN,X,Y)
C                   START NEW PATH. MOVE PEN TO (X,Y) 
C
C      VARIABLES:   LASSIZ IS LAST TEXT SIZE IN POINTS
C                   
C
C      CALLED BY:   SSPOST         CNPOST
C 
C      CALLS:       nil
C 
C      NOTE:        THIS DRIVER USES THE SAME SCALING AS USED FOR THE 
C                   RASTER TECH. TERMINAL. ORIGIN (0,0) IS CENTER OF SCREEN.
C                   X INCREASES TO RIGHT,  Y INCREASES UP THE SCREEN
C                   SCREEN IS 1280 X 1024 (-640..639,-512..511)
C               
C                   (-640,511)......................(639,511)
C                        :                              :
C                        :                              :
C                        :                              :
C                        :                              :
C                   (-640,-512)......................(639,-512)
C
C
C      NOTE:        SETGREY WILL NOT WORK IN POSTSCRIPT. IT MUST BE
C                   SPELLED SETGRAY!
C
C--*************************************************************************

        SUBROUTINE POSTRT(IPLUN)

C       SUBROUTINE TO INITIALIZE POSTSCRIPT DISPLAY

        CHARACTER * 80  TXLINE
        CHARACTER * 18  FONT(0:9)
        CHARACTER * 15  FMT
        CHARACTER * 7   LINT(0:9)
        CHARACTER * 1   NULL
        LOGICAL         CLOSD,FILLED

        PARAMETER       (NSIZE=2000)
        INTEGER         IDATA(2*NSIZE)
        INTEGER         LINWID(0:9)
          
        DATA FMT/'(3H  (,A  ,1H))'/
        DATA LINT/'[] 0   ','[2] 0  ','[4] 0  ','[4 1] 0','[6 2] 0',
     &            '[] 0   ','[2] 0  ','[] 0   ','[2] 0  ','[] 0   '/

        DATA LINWID/1,1,1,1,1,2,2,3,3,4/

        DATA FONT(0) /'/Times-Roman      '/
        DATA FONT(1) /'/Times-Bold       '/
        DATA FONT(2) /'/Times-Italic     '/
        DATA FONT(3) /'/Helvetica        '/
        DATA FONT(4) /'/Helvetica-Bold   '/
        DATA FONT(5) /'/Helvetica-Oblique'/
        DATA FONT(6) /'/Courier          '/
        DATA FONT(8) /'/Courier-Bold     '/
        DATA FONT(9) /'/Times-Roman      '/

        DATA SCALET / 0.333333 /

          LASANG = 0
          LASTYP = -1
          LASTEN = -1
          LASFON =  0

          WRITE(IPLUN,333) 
333       FORMAT(T1,'%!PS-Adobe-1.0')

C         CONVERT RASTER TECH UNITS TO POSTSCRIPT UNITS (1/72 INCH)
C         XCENT = 4.25 * 72  , YCENT = 5.5 * 72,  SCALE = 72.0 / 128.0

C         SET ORIGIN AT CENTER, LANDSCAPE ORIENTATION
          WRITE(IPLUN,*) ' 306 396 translate 90 rotate'
C         SCALE UNITS SO SAME AS RASTER TECH
          WRITE(IPLUN,*) ' .5625 .5625 scale'

C..       SETS DEFAULT CHARACTER FONTS TO SIZE 12 POINTS
          LASSIZ = 12
          LASFON = 3
          WRITE(IPLUN,98) FONT(LASFON),LASSIZ

C         INITIALIZE THE TEXT EXTENT RECORDING VARIABLES
          WRITE(IPLUN,*) '  /xmax -20000 def'
          WRITE(IPLUN,*) '  /ymax -20000 def'
          WRITE(IPLUN,*) '  /xmin  20000 def'
          WRITE(IPLUN,*) '  /ymin  20000 def'

          RETURN


        ENTRY POWIND(IPLUN,WXMIN,WYMIN,WXMAX,WYMAX)
C         SETS OUTPUT WINDOW
          WRITE(IPLUN,90) WXMIN,WYMIN,WXMIN,WYMAX,
     &                    WXMAX,WYMAX,WXMAX,WYMIN
90        FORMAT( '  initclip newpath ',
     &               F8.2,1X,F8.2,' moveto ',
     &               F8.2,1X,F8.2,' lineto ',/,
     &            2X,F8.2,1X,F8.2,' lineto ',
     &               F8.2,1X,F8.2,' lineto closepath clip')
        RETURN


        ENTRY POMOVE(IPLUN,X,Y)
C..       MOVE PEN TO (X,Y)
          WRITE(IPLUN,92)X,Y
92        FORMAT(2X,F8.2,1X,F8.2,' moveto')
        RETURN


        ENTRY POPATH(IPLUN,X,Y)
C..       START NEW PATH. MOVE PEN TO (X,Y) 
          WRITE(IPLUN,97) X,Y
97        FORMAT('  newpath ',F8.2,1X,F8.2,' moveto')
        RETURN


        ENTRY PODRAW(IPLUN,X,Y)
C..       DRAW LINE TO (X,Y)
          WRITE(IPLUN,91)X,Y
91        FORMAT(2X,F8.2,1X,F8.2,' lineto')
        RETURN


        ENTRY POSHOW(IPLUN,CLOSD,FILLED)
C         FINISH OFF A LINE AND DISPLAY IT
          IF (CLOSD .OR. FILLED) THEN
             WRITE(IPLUN,*) ' closepath'
             IF (FILLED) THEN
                 WRITE(IPLUN,*) ' gsave fill grestore 1.0 setgray'
                 LASTEN = -1
             ENDIF
          ENDIF
          WRITE(IPLUN,*) ' stroke'
        RETURN


        ENTRY POLINE(IPLUN,INTEN,IPEN,LINTYP)
C..       SELECT LINE TYPE AND WIDTH
          IF (LASTYP .NE. LINTYP) THEN

C******************888DEBUG
            IF (LINTYP .LT. 0 .OR. LINTYP .GT. 9) THEN
               WRITE(6,*) 'LINTYP:',LINTYP
               STOP
            ENDIF
C**************************

            WRITE(IPLUN,93) LINT(LINTYP)(1:7),LINWID(LINTYP)
93          FORMAT(2X,A7,' setdash ',I1,' setlinewidth ')
            LASTYP = LINTYP
          ENDIF
          IF (LASTEN .NE. INTEN) THEN
            IF (INTEN .GE. 0) THEN
              GRAY = 1.0 - ((INTEN + 1) / 10.0)
            ELSE
              GRAY = 1.0 - (-INTEN / 256.0)
            ENDIF
            GRAY = MIN(1.0,GRAY)
            WRITE(IPLUN,100) GRAY
100         FORMAT(2X,F5.2,' setgray')
            LASTEN = INTEN
          ENDIF
          RETURN


        ENTRY POARAY(IPLUN,IDATA,NDATA,CLOSD,FILLED)
C..       DRAW A POLYGON

          NDATA2 = NDATA * 2
          ISTOP = 1
7333      IGO   = ISTOP + 399 
          IF (IGO .GT. NDATA2) IGO = NDATA2
          NOWD2 = (IGO - ISTOP + 1) / 2
          WRITE(IPLUN,95) (IDATA(I),I= IGO,ISTOP,-1)
95        FORMAT(7(I5,1X,I5,1X))
          WRITE(IPLUN,94) NOWD2
94        FORMAT(2X,I4,' {lineto} repeat')
          IF (IGO .LT. NDATA2) THEN
             ISTOP = IGO + 1
             GOTO 7333
          ELSEIF (CLOSD .OR. FILLED) THEN
             WRITE(IPLUN,*) ' closepath'
             IF (FILLED) THEN
                 WRITE(IPLUN,*) ' gsave fill grestore 1.0 setgray'
                 LASTEN = -1
             ENDIF
          ENDIF
          WRITE(IPLUN,*) ' stroke'
          RETURN


        ENTRY POTEX(IPLUN,TXLINE,ICHAR,X,Y,ITSIZE,ITANGL,JUST)
C         PRINT TEXT AT (X,Y)

          IPOINT = MAX((ITSIZE / 2),1)

          IF (LASSIZ .EQ. 0) THEN
C            ADDING TO OLD FILE
C            SETS DEFAULT CHARACTER FONTS TO SIZE 12 POINTS
             LASSIZ = 12
             LASFON = 3
             WRITE(IPLUN,98) FONT(LASFON),IPOINT

          ELSEIF (IPOINT .NE. LASSIZ) THEN
            RELSIZ = FLOAT(IPOINT) / FLOAT(LASSIZ)
            LASSIZ = IPOINT
            WRITE(IPLUN,89) RELSIZ
89          FORMAT('  currentfont ',F7.3,' scalefont setfont')
          ENDIF

C..       MOVE TO UNJUSTIFIED LOCATION 
          WRITE(IPLUN,96) X,Y
96        FORMAT(2X,F8.2,1X,F8.2,' moveto')

          IF (ITANGL .NE. LASANG) THEN
C            SET TEXT ANGLE
             WRITE(IPLUN,88) ITANGL
88           FORMAT('  gsave ',I4,' rotate')
          ENDIF

C         PUT TEXT ON STACK
          WRITE(FMT(9:10),8000,ERR=1011) ICHAR
8000      FORMAT(I2)
          WRITE(IPLUN,FMT) TXLINE(1:ICHAR)

C         FIND RELATIVE LOCATION FOR JUSTIFICATION
          YR =  -IPOINT * SCALET

          IF (JUST .EQ. 0) THEN
C            LEFT JUSTIFIED TEXT CENTERED ON Y
             WRITE(IPLUN,103) YR
103          FORMAT('  0 ',F8.2,' rmoveto')

          ELSE IF (JUST .EQ. 1) THEN
C            CENTERED IN X AND Y TEXT
             WRITE(IPLUN,101) YR
101         FORMAT('  dup stringwidth pop 2 div neg ',F8.2,' rmoveto')

          ELSEIF (JUST .EQ. 2) THEN
C            RIGHT JUSTIFIED TEXT CENTERED ON Y
             WRITE(IPLUN,102) YR
102          FORMAT('  dup stringwidth pop neg ',F8.2,' rmoveto')

          ENDIF

c*******************new stuff
          WRITE(IPLUN,*) '  currentpoint exch pop dup dup '
          WRITE(IPLUN,*) '  ymin lt {dup /ymin exch def} if'
          WRITE(IPLUN,*) '  ymax gt {    /ymax exch def} {pop} ifelse'
          WRITE(IPLUN,*) '  currentpoint pop dup dup '
          WRITE(IPLUN,*) '  xmin lt {dup /xmin exch def} if'
          WRITE(IPLUN,*) '  xmax gt {    /xmax exch def} {pop} ifelse'
C*************************************

          WRITE(IPLUN,*) '  show'

          IF (ITANGL .NE. LASANG) THEN
             WRITE(IPLUN,*) '  grestore'
             LASANG = ITANGL
          ENDIF

1011      RETURN



        ENTRY POTEXT(IPLUN,TXLINE,ICHAR)
C         PUT TEXT ON STACK
          WRITE(FMT(9:10),8000,ERR=1011) ICHAR
          WRITE(IPLUN,FMT) TXLINE(1:ICHAR)
          WRITE(IPLUN,*) ' show'
          RETURN


        ENTRY POSIZE(IPLUN,ITSIZE)
C..       SET TEXT SIZE,  DEFAULT SIZE IS 12 POINT

          IPOINT = MAX((ITSIZE / 2),1)

          IF (LASSIZ .EQ. 0) THEN
C            ADDING TO OLD FILE
C            SETS DEFAULT CHARACTER FONTS TO SIZE 12 POINTS
             LASSIZ = 12
             LASFON = 3
             WRITE(IPLUN,98) FONT(LASFON),IPOINT

          ELSEIF (IPOINT .NE. LASSIZ) THEN
            RELSIZ = FLOAT(IPOINT) / FLOAT(LASSIZ)
            LASSIZ = IPOINT
            WRITE(IPLUN,89) RELSIZ
          ENDIF
          RETURN



        ENTRY POFONT(IPLUN,IFONT)
C..       SETS CHARACTER FONTS
          IF (LASSIZ .EQ. 0) THEN
C            ADDING TO OLD FILE
C            SETS DEFAULT CHARACTER FONTS TO SIZE 12 POINTS
             LASSIZ = 12
          ENDIF

          IPOINT = LASSIZ
          LASFON = IFONT
          WRITE(IPLUN,98) FONT(LASFON),IPOINT
98        FORMAT(2X,A13,' findfont ',I3,' scalefont setfont')
          RETURN


        ENTRY POEND(IPLUN)
C..       END PLOTTER USE
          WRITE(IPLUN,99) 
99        FORMAT('  showpage')
          RETURN

        ENTRY POZERO(IPLUN)
C         INITIALIZE THE TEXT EXTENT RECORDING VARIABLES
          WRITE(IPLUN,*) '  /xmax -20000 def'
          WRITE(IPLUN,*) '  /ymax -20000 def'
          WRITE(IPLUN,*) '  /xmin  20000 def'
          WRITE(IPLUN,*) '  /ymin  20000 def'
          RETURN

        ENTRY POUL(IPLUN)
C         GOTO UPPER LEFT OF TEXT EXTENTS
          WRITE(IPLUN,*) '  xmin ymax moveto'
          RETURN

        ENTRY POLL(IPLUN)
C         GOTO LOWER LEFT OF TEXT EXTENTS
          WRITE(IPLUN,*) '  xmin ymin moveto'
          RETURN


        END

