C++************************************************************************
C
C PR3D.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR       *
C *  LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR    *
C *  USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF     *
C *  THE CENTER FOR LABORATORIES AND RESEARCH                              *
C **************************************************************************
C
C TO CALCULATE THE 3-D PHASE RESIDUE OUTSIDE MISSING CONE, PR OF FOURIER
C RINGS(RADIUS, DIRECTION RELATIVE TO Z) AND OF FOURIER SHELLS(RADIUS) IS
C CALCULATED. 
C 
C23456789012345678901234567890123456789012345678901234567890123456789012
C--************************************************************************

	SUBROUTINE PR3D(MAXDIM)
	INCLUDE 'CMBLOCK.INC'
	PARAMETER  (NSCALE=20)
	COMMON       BUF(1)
	DIMENSION    Q(1)
	POINTER      (IPQ,Q) 
	LOGICAL      RUNTIME
	REAL            SSANG, WI
	CHARACTER *81   FILNAM1,FILNAM2
	CHARACTER*1       NULL,SER
	DIMENSION       DLIST(6)
	EQUIVALENCE (JDUM,DUM)
	DATA  LUN1,LUN2/21,22/, NDOC/88/
CNO_SAVE
	NULL=CHAR(0)
	PI=4.0*ATAN(1.00)
C   ----------------------INPUT FIRST IMAGE
	CALL FILERD (FILNAM1,NLET,NULL,'FIRST INPUT',IRTFLG)
	CALL OPENFB(BUF,FILNAM1,LUN1,NSAM,NROW,'OLD',NF,NSLICE)
	IF (NF.EQ.1) THEN
         CALL ERRT(4,'RF 3',NE)
         RETURN
	ENDIF
	IF(IFORM.NE.3)THEN
	CALL ERRT(2,'RF 3',NE)
	RETURN
	ENDIF
C   ----------------------INPUT SECOND IMAGE
	CALL FILERD (FILNAM2,NLET,NULL,'SECOND INPUT',IRTFLG)
	CALL OPENFB(BUF,FILNAM2,LUN2,NSAM2,NROW2,'OLD',NF,NSLICE2)
	IF (NF.EQ.1) THEN
	CALL ERRT(4,'RF 3',NE)
	RETURN
	ENDIF
C
	IF(IFORM.NE.3)THEN
	CALL ERRT(2,'RF 3',NE)
	RETURN
	ENDIF
      IF(NSAM.NE.NSAM2.OR.NROW.NE.NROW2.OR.NSLICE.NE.NSLICE2) THEN	
	 CALL ERRT(1,'RF 3',NE)
         CLOSE(LUN1)
         CLOSE(LUN2)
         RETURN
      ENDIF
	CALL RDPRM(WI, NLOOP, ILOOP, 'RING WIDTH')
C
	CALL RDPRM2(SCALE1,SCALE2,NLOOP,ILOOP,
     &  'SCALE FACTOR(LOWER,UPPER)')
C
	DSCALE=(SCALE2-SCALE1)/FLOAT(NSCALE-1)
      CALL RDPRMC(SER,NUMC,.TRUE.,'MISSING CONE/WEDGE ANGLE (C/W)',
     &  NULL,IRT)
	IF(SER.NE.'C'.AND.SER.NE.'W')	   SER='C'
	CALL RDPRM(SSANG,NLOOP,ILOOP,'MAXIMUM TILT ANGLE')
      CALL RDPRM(FACT,NLOOP,ILOOP,'FACTOR FOR NOISE COMPARISON')
C
	LSD=NSAM+2-MOD(NSAM,2)
	Y1=FLOAT(MAX0(NSAM,NROW,NSLICE))
	INC=INT(Y1/WI)/2+1
C
	K1=1
	K2=K1+LSD*NROW*NSLICE
	K3=K2+LSD*NROW*NSLICE
	K4=K3+INC*NSCALE
	K5=K4+INC*NSCALE
	K6=K5+INC
	K7=K6+INC
	K8=K7+INC
	K9=K8+INC
        MEMWANT=K9+INC*NSCALE
        CALL CHKMEM(MEMWANT,BUF,MAXDIM,RUNTIME,IPQ,IRTFLG)
          IF (IRTFLG.NE.0)  THEN
	   CLOSE(LUN1)
	   CLOSE(LUN2)
	   CALL ERRT(6,'RF 3',NE)
	   RETURN
	  ENDIF
        DO  J = 1, NSLICE
          DO  I = 1, NROW
             L = (J-1)*NROW+I
	CALL REDLIN(LUN1,Q(K1+((I-1)+(J-1)*NROW)*LSD),NSAM,L)
          ENDDO
       ENDDO
       CLOSE(LUN1)
        DO   J = 1, NSLICE
          DO  I = 1, NROW
            L = (J-1)*NROW+I
	CALL REDLIN(LUN2,Q(K2+((I-1)+(J-1)*NROW)*LSD),NSAM,L)
          ENDDO
        ENDDO
	CLOSE(LUN2)
	INV=1
	CALL FMRS_3(Q(K1),NSAM,NROW,NSLICE,INV)
	IF (INV.EQ.0) THEN
	   CALL ERRT(38,'RF 3 ',NE)
	   RETURN
	ENDIF
	INV=1
	CALL FMRS_3(Q(K2),NSAM,NROW,NSLICE,INV)
	IF (INV.EQ.0)THEN
	   CALL ERRT(38,'RF 3 ',NE)
	   RETURN
	ENDIF 
	CALL PR3DB(Q(K1),Q(K2),Q(K3),Q(K4),Q(K5),Q(K6),Q(K7),Q(K8),
     &  Q(K9),LSD,NSAM,NROW,NSLICE,DSCALE,NSCALE,SCALE1,
     &  SSANG,INC,Y1,WI,SER)
C   ---------------TO WRITE RESULT INTO DOC FILE AND RESULT FILE
      NLIST=6
      WRITE(NOUT,*) 'THIS PROGRAM IS DEVELOPED FOR 3D RESOLUTION  '
      WRITE(NOUT,*) 'ESTIMATION BY THE METHOD OF PHASE RESIDUES AND' 
      WRITE(NOUT,*) 'FOURIER SHELL CORRELATION'
      WRITE(NOUT,*) 'FOLLOWING IS THE RESULT OF PR AND FSC '
      WRITE(NOUT,*) '(ALSO IN DOC FILE)'
      WRITE(NOUT,5600) WI
5600  FORMAT(1X,'RING WIDTH = ',G12.5)
      WRITE(NOUT,5700)
5700  FORMAT(10X,'|NUMBER|','|RING RADIUS',
     $ '|DPH|','|FSC|','|FSCCRIT|','|VOXELS|')		
      WRITE(NOUT,5800)
5800  FORMAT(10X,'        ',' NORMALIZED|')		
	DO 6000 L=1,INC 
         DUM=Q(K6+L-1)
	IF(DUM.EQ.0.0)   GO TO 6000
	 DLIST(1)=L
         DLIST(2)=FLOAT(L-1)/FLOAT(INC-1)*0.5
         DLIST(5)=FACT/SQRT(FLOAT(JDUM))
	IF(DLIST(5).GT.1.0)   DLIST(5)=1.0
	DLIST(6)=JDUM
C
	NSCM=0
	RFMIN=Q(K9+(L-1)*NSCALE)/Q(K4+(L-1)*NSCALE)
	NSCM=1
	IFOUND=0
	LCUR=L
	DO  NSC=2,NSCALE
	RFM=Q(K9+(L-1)*NSCALE+(NSC-1))/Q(K4+(L-1)*NSCALE+(NSC-1))
	IF(RFM.LT.RFMIN) THEN
	NSCM=NSC
	RFMIN=RFM
	IFOUND=1
	ENDIF
	ENDDO
C
	IF(IFOUND.EQ.0)  THEN
	WRITE(NOUT,436)   LCUR
436	FORMAT(' *** NO MINIMUM IN RFACT CURVE FOUND FOR RING = ',I5)
	ENDIF
C
C NSCM IS THE NUMBER OF THE ELEMENT IN EACH ARRAY WITH THE CORRECT
C SCALING. SCALE IS THE CORRECT SCALING.
C
	BK1=Q(K4+(L-1)*NSCALE+NSCM)
	BK2=Q(K3+(L-1)*NSCALE+NSCM)
	IF(BK1.NE.0.)THEN
	DLIST(3)=SQRT(BK2/BK1)
	ENDIF
	BK3=Q(K8+L-1)
	BK4=Q(K5+L-1)
	BK5=Q(K7+L-1)
	IF(BK3.NE.0.AND.BK4.NE.0.)THEN
	DLIST(4)=BK5/SQRT(BK4*BK3)
	ENDIF
756	CALL SAVD(NDOC, DLIST, NLIST, IRTFLG)
	WRITE(NOUT,6100)L,(DLIST(K),K=2,5),JDUM
6100    FORMAT (1X,I4,4(2X,F12.5),4X,I6)
6000	CONTINUE
	CALL SAVDC
	CLOSE(NDOC)
	IF(RUNTIME)  CALL ALLOCIT(0,RUNTIME,IPQ,'RF 3',IRTFLG)
	END
