C++************************************************************************
C
C $$ PUTF3.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR       *
C *  LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR    *
C *  USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF     *
C *  THE CENTER FOR LABORATORIES AND RESEARCH   			   *
C **************************************************************************
C
C
C
C--************************************************************************
C
C $$ PUTF3.FOR
C
         SUBROUTINE PUTF3
     $   (LUN,FBUF,NDIM,NROW,NSLICE,IX,IY,IZ,QR,QI,IREADO,*)
C
C Subroutine to update a three-dimensional Fourier transform on disk.
C this program is adapted from PUT3_P. It is checked if the line needed
C is in the current Fourier buffer. If yes, only the value is updated
C inside the buffer, if not, the current buffer is written to disk, and the
C needed line is read from disk. For the first call (ireado=0) no line
C is written, but only a line is read. To finish updating a Fourier transform
C call PUTF3 with ireado=-1. The variable ireado has to be kept in the calling
C routine of this subroutine.                         5/13/91 M.Radermacher
C
C      LUN         LOGICAL UNIT NUMBER OF 3D FOURIER
C      FBUF        BUFFER FOR ONE LINE OF THE 3D FOURIER
C      NDIM        (=NSAM) X-DIMENSION OF FOURIER
C      NROW        Y-DIMENSION
C      NSLICE      Z-DIMENSION
C      IX,IY,IZ    FOURIER COORDINATES WHERE VALUE IS TO BE PUT
C      QR          REAL PART OF COMPLEX FOURIER COEFFICIENT
C      QI          IMAG. PART OF COMPLEX FOURIER COEFFICIENT
C      IREADO      VARIABLE TO SAVE NUMBER OF LAST PROCESSED RECORD
C

         INCLUDE 'CMBLOCK.INC'
         DIMENSION  FBUF(NDIM)
C        WRITE(NOUT,100) IX,IY,IZ,QR,QI
100      FORMAT(1X,' IX,IY,IZ:',3(I5),' QR=',G10.4,'QI=',G10.4)
C
         IF(IREADO.GE.0) THEN
         IF(IX.EQ.0) THEN
         IF(IY.EQ.0)  THEN
         IF(IZ.EQ.0)  THEN
         IREAD=1
         CALL RWF3(LUN,NDIM,FBUF,QR,1,IREAD,IREADO)
C       A(1,1,1)=REAL(Q)
         RETURN
         ELSEIF(IZ.EQ.NSLICE/2)  THEN
         IREAD=NROW+1
         CALL RWF3(LUN,NDIM,FBUF,QR,1,IREAD,IREADO)
C      A(1,1,2)=REAL(Q)
         RETURN
         ELSEIF(IZ.LT.0)  THEN
C       PRINT *,
C     *           ' OUT OF BOUNDARY, IX=IY=0, IZ<0'
         RETURN 1
         ELSE
         ISLICE=2*IZ+1
         IREAD=(ISLICE-1)*NROW+1
         CALL RWF3(LUN,NDIM,FBUF,QR,1,IREAD,IREADO)
C      A(1,1,ISLICE)=REAL(Q)
         IREAD=ISLICE*NROW+1
         CALL RWF3(LUN,NDIM,FBUF,QI,1,IREAD,IREADO)
C      A(1,1,ISLICE+1)=AIMAG(Q)
         ENDIF
         ELSEIF(IY.EQ.NROW/2)  THEN
         IF(IZ.EQ.0)  THEN
         IREAD=2
         CALL RWF3(LUN,NDIM,FBUF,QR,1,IREAD,IREADO)
C      A(1,2,1)=REAL(Q)
         RETURN
         ELSEIF(IZ.EQ.NSLICE/2)  THEN
         IREAD=NROW+2
         CALL RWF3(LUN,NDIM,FBUF,QR,1,IREAD,IREADO)
C      A(1,2,2)=REAL(Q)
         RETURN
         ELSEIF(IZ.LT.0)  THEN
C       PRINT *,
C     *           ' OUT OF BOUNDARY, IX=0, IY=NROW/2, IZ<0'
         RETURN 1
         ELSE
         ISLICE=2*IZ+1
         IREAD=(ISLICE-1)*NROW+2
         CALL RWF3(LUN,NDIM,FBUF,QR,1,IREAD,IREADO)
C      A(1,2,ISLICE)=REAL(Q)
         IREAD=ISLICE*NROW+2
         CALL RWF3(LUN,NDIM,FBUF,QI,1,IREAD,IREADO)
C      A(1,2,ISLICE+1)=AIMAG(Q)
         ENDIF
         ELSEIF(IY.LT.0)  THEN
C      PRINT *,' OUT OF BOUNDARY, IX=0, IY<0'
         RETURN 1
         ELSE
         IROW=2*IY+1
         ISLICE=IZ+1
         IF(IZ.LT.0)  ISLICE=ISLICE+NSLICE
         IREAD=(ISLICE-1)*NROW+IROW
         CALL RWF3(LUN,NDIM,FBUF,QR,1,IREAD,IREADO)
C     A(1,IROW,ISLICE)=REAL(Q)
         IREAD=(ISLICE-1)*NROW+IROW+1
         CALL RWF3(LUN,NDIM,FBUF,QI,1,IREAD,IREADO)
C     A(1,IROW+1,ISLICE)=AIMAG(Q)
         RETURN
         ENDIF
         ELSEIF(IX.EQ.NDIM/2)  THEN
         IF(IY.EQ.0)  THEN
         IF(IZ.EQ.0)  THEN
         IREAD=1
         CALL RWF3(LUN,NDIM,FBUF,QR,2,IREAD,IREADO)
C     A(2,1,1)=REAL(Q)
         RETURN
         ELSEIF(IZ.EQ.NSLICE/2)  THEN
         IREAD=NROW+1
         CALL RWF3(LUN,NDIM,FBUF,QR,2,IREAD,IREADO)
C     A(2,1,2)=REAL(Q)
         RETURN
         ELSEIF(IZ.LT.0)  THEN
C       PRINT *,
C     *           ' OUT OF BOUNDARY, IX=NDIM/2, IY=0, IZ<0'
         RETURN 1
         ELSE
         ISLICE=2*IZ+1
         IREAD=(ISLICE-1)*NROW+1
         CALL RWF3(LUN,NDIM,FBUF,QR,2,IREAD,IREADO)
C     A(2,1,ISLICE)=REAL(Q)
         IREAD=ISLICE*NROW+1
         CALL RWF3(LUN,NDIM,FBUF,QI,2,IREAD,IREADO)
C     A(2,1,ISLICE+1)=AIMAG(Q)
         ENDIF
         ELSEIF(IY.EQ.NROW/2)  THEN
         IF(IZ.EQ.0)  THEN
         IREAD=2
         CALL RWF3(LUN,NDIM,FBUF,QR,2,IREAD,IREADO)
C     A(2,2,1)=REAL(Q)
         RETURN
         ELSEIF(IZ.EQ.NSLICE/2)  THEN
         IREAD=NROW+2
         CALL RWF3(LUN,NDIM,FBUF,QR,2,IREAD,IREADO)
C     A(2,2,2)=REAL(Q)
         RETURN
         ELSEIF(IZ.LT.0)  THEN
C       PRINT *,
C     *           ' OUT OF BOUNDARY, IX=NDIM/2, IY=NROW/2, IZ<0'
         RETURN 1
         ELSE
         ISLICE=2*IZ+1
         IREAD=(ISLICE-1)*NROW+2
         CALL RWF3(LUN,NDIM,FBUF,QR,2,IREAD,IREADO)
C     A(2,2,ISLICE)=REAL(Q)
         IREAD=ISLICE*NROW+2
         CALL RWF3(LUN,NDIM,FBUF,QI,2,IREAD,IREADO)
C     A(2,2,ISLICE+1)=AIMAG(Q)
         ENDIF
         ELSEIF(IY.LT.0)  THEN
C      PRINT *,' OUT OF BOUNDARY, IX=NDIM/2, IY<0'
         RETURN 1
         ELSE
         IROW=2*IY+1
         ISLICE=IZ+1
         IF(IZ.LT.0)  ISLICE=ISLICE+NSLICE
         IREAD=(ISLICE-1)*NROW+IROW
         CALL RWF3(LUN,NDIM,FBUF,QR,2,IREAD,IREADO)
C     A(2,IROW,ISLICE)=REAL(Q)
         IREAD=(ISLICE-1)*NROW+IROW+1
         CALL RWF3(LUN,NDIM,FBUF,QI,2,IREAD,IREADO)
C     A(2,IROW+1,ISLICE)=AIMAG(Q)
         RETURN
         ENDIF
         ELSEIF(IX.LT.0)  THEN
C     PRINT *,' OUT OF BOUNDARY, IX<0'
         RETURN 1
         ELSE
         IDIM=2*IX+1
         IROW=IY+1
         IF(IY.LT.0) IROW=IROW+NROW
         ISLICE=IZ+1
         IF(IZ.LT.0) ISLICE=ISLICE+NSLICE
         IREAD=(ISLICE-1)*NROW+IROW
         CALL RWF3(LUN,NDIM,FBUF,QR,IDIM,IREAD,IREADO)
C     A(IDIM,IROW,ISLICE)=REAL(Q)
         CALL RWF3(LUN,NDIM,FBUF,QI,IDIM+1,IREAD,IREADO)
C     A(IDIM+1,IROW,ISLICE)=AIMAG(Q)
         ENDIF
         ELSE
         IREADO=-IREADO
         CALL RWF3(LUN,NDIM,FBUF,QR,IDIM,-1,IREADO)
         ENDIF
         RETURN
         END
