      FUNCTION  QUADRI_Q(XX, YY, LSD, NXDATA, NYDATA, FDATA)
C  Quadratic interpolation
C  XX,YY treated as circularly closed.
C  fdata - image 1..nxdata, 1..nydata
C
C
C                                  SPECIFICATIONS FOR ARGUMENTS
      DIMENSION  FDATA(LSD,NYDATA)
CNO_SAVE
C
C                    F3    FC       F0, F1, F2, F3 are the values
C                     +             at the grid points.  X is the
C                     + X           point at which the function
C              F2++++F0++++F1       is to be estimated. (It need
C                     +             not be in the First quadrant).
C                     +             FC - the outer corner point
C                    F4             nearest X.
C
C                                   F0 is the value of the FDATA at
C                                   FDATA(I,J), it is the interior mesh
C                                   point nearest  X.
C                                   The coordinates of F0 are (X0,Y0),
C                                   The coordinates of F1 are (XB,Y0),
C                                   The coordinates of F2 are (XA,Y0),
C                                   The coordinates of F3 are (X0,YB),
C                                   The coordinates of F4 are (X0,YA),
C                                   The coordinates of FC are (XC,YC),
C
	x=xx
	y=yy
	if(x.lt.1.0)  x=x+(1-ifix(x)/nxdata)*nxdata
	if(x.gt.float(nxdata)+0.5)  x=amod(x-1.0,float(nxdata))+1.0
	if(y.lt.1.0)  y=y+(1-ifix(y)/nydata)*nydata
	if(y.gt.float(nydata)+0.5)  y=amod(y-1.0,float(nydata))+1.0
	i=IFIX(x)
	j=IFIX(y)
C
      X0 = I
      Y0 = J
      XA = I-1
      XB = I+1
      YA = J-1
      YB = J+1
C
      DX0 = X - X0
      DY0 = Y - Y0
C
      DXB = X - XB
      DYB = Y - YB
C
	ip1=i+1
	im1=i-1
	jp1=j+1
	jm1=j-1
      F0 = FDATA(I,J)
	if(ip1.gt.nxdata) ip1=ip1-nxdata	
      F1 = FDATA(Ip1,J)
	if(im1.lt.1)  im1=im1+nxdata
      F2 = FDATA(Im1,J)
	if(jp1.gt.nydata) jp1=jp1-nydata	
      F3 = FDATA(I,Jp1)
	if(jm1.lt.1)  jm1=jm1+nydata
      F4 = FDATA(I,Jm1)
C
      IC = I + INT(SIGN(1.0,DX0))
      JC = J + INT(SIGN(1.0,DY0))
C
      XC = IC
      YC = JC
	if(ic.gt.nxdata) then
	ic=ic-nxdata	
	elseif(ic.lt.1)  then
	ic=ic+nxdata
	endif
	if(jc.gt.nydata)  then
	jc=jc-nydata
	elseif(jc.lt.1)  then
	jc=jc+nydata
	endif
      FC = FDATA(IC,JC)
C                       O               HXA, HXB are the mesh spacings
C                       +               in the X-direction to the left
C                      HYB              and right of the center point.
C                       +
C               O++HXA++O++HXB++O       HYB, HYA are the mesh spacings
C                       +               in the Y-direction.
C                      HYA
C                       +               HXC equals either  HXB  or  HXA
C                       O               depending on where the corner
C                                       point is located.
C
      HXA = X0 - XA
      HXB = XB - X0
      HYA = Y0 - YA
      HYB = YB - Y0
C
      HXC = XC - X0
      HYC = YC - Y0
C
      HXCB = XC - XB
      HYCB = YC - YB
C                                       Construct the interpolant
C                                       F = F0 + C1*(X-X0) +
C                                           C2*(X-X0)*(X-X1) +
C                                           C3*(Y-Y0) + C4*(Y-Y0)*(Y-Y1)
C                                           + C5*(X-X0)*(Y-Y0)
      C1 = (F1-F0)/HXB
      T1 = (F0-F2)/HXA
      C2 = (C1-T1)/(HXA+HXB)
      C3 = (F3-F0)/HYB
      T3 = (F0-F4)/HYA
      C4 = (C3-T3)/(HYA+HYB)
      C5 = (FC-F0-HXC*C1-HXC*HXCB*C2-HYC*C3-HYC*HYCB*C4)/(HXC*HYC)
C
      QUADRI_Q = F0 + DX0*(C1+DXB*C2+DY0*C5) + DY0*(C3+DYB*C4)
      END
