C++*************************************************************************
C
C RCTFS.FOR                        
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C  RCTFS(MAXMEM)
C
C  PURPOSE: CTF CORRECTION USING WIENER FILTERING APPROACH
C  
C           F(K)=SUM(H I(K)FI(K))/(SUM(HI(K)^2)+1/SNR), WHERE HI(K) IS 
C           CTF FUNCTION FOR THE ITH VOLUME; 
C           FI(K) IS THE ITH VOLUME; SNR IS SIGNAL-TO-NOISE RATION.
C
C        1         2         3         4         5         6         7
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*********************************************************************

        SUBROUTINE  RCTFS(MAXMEM)
	PARAMETER (LENBUF=1024)
        INCLUDE 'CMBLOCK.INC'

	COMMON	OPENBUF(LENBUF),BUF(1024)
	DIMENSION	B(1)
	POINTER	(IPQ,B) 
	LOGICAL	RUNTIME
	CHARACTER*81	FILNAM
        CHARACTER	NULL
	DATA	LUN1/10/
	LOGICAL	FIRST
CNO_SAVE

        NULL=CHAR(0)
	FIRST=.TRUE.
C       WORKING ADDRESS
C       KAD1=1   ADDRESS FOR IMAGE ONE
C       KAD2=KAD1+NTOT FOR OUTPUT
C       KAD3=KAD2+NTOT FOR SUM OF CTF**2
1	CALL OPFIL(.TRUE.,FILNAM,LUN1,'O',NSAM1,NROW1,NSLICE1,
     &		   IDUM,'INPUT',.TRUE.,IRTFLG)
	IF (IRTFLG.NE.0) THEN
	   CALL  RDPRM(SNR,NLOOP,ILOOP,'SNR')
C          DIVIDE BY THE SUM OF H^2+1/SNR
	   S=1.0/SNR
           CALL  DIVCR3(B(KAD2),B(KAD2),B(KAD3),S,LRCL1/2,NROW,NSLICE)
           IFORM=3
	   CALL OPFIL(.TRUE.,FILNAM,LUN1,'N',NSAM,NROW,NSLICE,
     &		       IDUM,'OUTPUT',.FALSE.,IRTFLG)
	   IF (IRTFLG.NE.0) RETURN
	   INV=-1
	   CALL FMRS_3(B(KAD2),NSAM,NROW,NSLICE,INV)
C          WRITE THE OUTPUT
	   DO    J=1,NROW*NSLICE
 	       CALL  WRTLIN(LUN1,B(KAD2+(J-1)*LS),NSAM,J)
	   ENDDO
	   CLOSE(LUN1)
C          END OF PROCEDURE
           IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'TF CTF',IRTFLG)
           RETURN
	ENDIF

C        PROCESS THE VOLUME
        IF (IFORM.EQ.-21)  THEN
	   LRCL=NSAM1
	   LS=NSAM1
	   NSAM1=NSAM1-1
	ELSEIF(IFORM.EQ.-22)  THEN
	   LRCL=NSAM1
	   LS=NSAM1
	   NSAM1=NSAM1-2
	ELSEIF(IFORM.EQ.3) THEN
	   LS=NSAM1+2-MOD(NSAM1,2)
	   LRCL=NSAM1
	ELSE
	   CLOSE(LUN1)
	   CALL  ERRT(2,'TF CTF',NE)	
	   RETURN
	ENDIF

C       GET THE DIMENSION INFORMATION FOR THE FIRST TIME
	IF (FIRST) THEN
	   NSAM=NSAM1
	   NROW=NROW1
	   NSLICE=NSLICE1
	   NTOT=LS*NROW*NSLICE
	   KAD1=1
	   KAD2=KAD1+NTOT
	   KAD3=KAD2+NTOT
	   KAD4=KAD3+NTOT
	   MEMWANT=KAD4+LS
	   CALL CHKMEM(MEMWANT,BUF,MAXMEM-LENBUF,RUNTIME,IPQ,IRTFLG)
	   IF (IRTFLG.NE.0.OR.LRCL.GT.LENBUF)  THEN
	      CALL ERRT(6,'TF CTF',NE)
	      CLOSE(LUN1)
	      RETURN
	   ENDIF
C          SET OUTPUT VOLUME AND CTF^2 TO ZERO
	   DO  I=0,NTOT-1
	      B(KAD2+I)=0.0
	      B(KAD3+I)=0.0
           ENDDO
           FIRST=.FALSE.
        ELSE
	   IF(NSAM.NE.NSAM1.OR.NROW.NE.NROW1.OR.NSLICE.NE.NSLICE1) THEN
	      CLOSE(LUN1)
	      CALL  ERRT(1,'TF CTF',NE)
	      RETURN
	   ENDIF
        ENDIF
C       READ INPUT FILE
	DO J=1,NROW*NSLICE
           CALL  REDLIN(LUN1,B(KAD1+(J-1)*LS),LRCL,J)
	ENDDO
	CLOSE(LUN1)
	IF (IFORM .EQ. 3) THEN
C	   FOURIER TRANSFORM
           INV=+1
	   CALL FMRS_3(B(KAD1),NSAM,NROW,NSLICE,INV)
	   IF (INV.EQ.0)  THEN
	      CALL  ERRT(38,'TF CTF',NE)
	      IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'TF CTF',IRTFLG)
C	      RETURN
	   ENDIF   
	ENDIF

C       GET CTF
	CALL OPFIL(.TRUE.,FILNAM,LUN1,'O',NSAM1,NROW1,NSLICE1,
     &		   IDUM,'CTF',.TRUE.,IRTFLG)
	IF (IRTFLG .NE. 0) RETURN
	LRCL1=NSAM1
C       ONLY 3D FOURIER	
        IF (IFORM.EQ.-21) THEN
	   NSAM1=NSAM1-1
        ELSEIF (IFORM.EQ.-22) THEN
	   NSAM1=NSAM1-2
	ELSE	
	   CLOSE(LUN1)
	   IF(RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'TF CTF',IRTFLG)
	   CALL  ERRT(2,'TF CTF',NE)	
	   RETURN
	ENDIF

	IF (NSAM.NE.NSAM1.OR.NROW.NE.NROW1.OR.NSLICE.NE.NSLICE1) THEN
	   CLOSE(LUN1)
	   IF(RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'TF CTF',IRTFLG)
	   CALL ERRT(1,'TF CTF',NE)
	   RETURN
	ENDIF

C       MUTLIPLY INPUT VOLUME STORED IN THE MEMORY KAD1, PLACE OUTPUT
C       IN KAD2 AND SQUARED CTF IN KAD3.
	CALL MULFC3(LUN1,B(KAD1),B(KAD4),B(KAD2),B(KAD3),
     &		LRCL1/2,NROW,NSLICE)
	CLOSE(LUN1)
	GOTO  1
	END
