
C++************************************************************************
C
C RDDOCA2Q.F
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C   RDDOCA2Q(DOCNAM,NDOC)
C
C   PURPOSE:  THIS ROUTINE DUPLICATES FUNCTION OF RDDOC (UNSAVE DOCUMENT)
C             BUT IT USES IN-CORE STORAGE, WHICH MAKES SUBSEQUENT READ 
C             ACCESSES VERY QUICK.  IT SOLICITS DOC FILE NAME.  THE
C             REGISTERS ARE SPECIFED ON THE COMMAND LINE
C   
C   PARAMETERS:     DOCNAM    FILE NAME                   (SENT)
C                   NDOC      LUN FOR DOC FILE            (SENT)
C
C   USAGE:    'UD IC' --  FIRST ACCESS: READ CONTENTS OF ENTIRE DOCUMENT 
C                         INTO CORE
C             'UD IC' --  SUBSEQUENT ACCESSES: READ DIRECTLY FROM CORE
C             'UD ICE' -- TERMINATE CORE ACCESS OF CURRENT DOCUMENT
C
C   CALLS:    UNSDAL
C
C--*******************************************************************

	SUBROUTINE RDDOCA2Q(DOCNAM,NDOC)

	INCLUDE 'CMBLOCK.INC' 
CNO_SAVE

C       MAXIMUM NUMBER OF COLUMNS PER KEY 
        PARAMETER (MAXCOL=7)
    
C       MAXIMUM NUMBER OF KEYS IN DOC FILE
        PARAMETER (MAXKEY=9999)

C       RUN-TIME ALLOCATION SPECIFIC CODE LINES FOLLOW
C       MAXIMUM NUMBER OF OPENED CONCURRENT INCORE DOCUMENT FILES 
        PARAMETER      (MAXICDOCS=10)    
#ifdef SP_NT
        INTEGER        LOCDOC(MAXICDOCS)
#else
        INTEGER * 8    LOCDOC(MAXICDOCS)
#endif
        POINTER (IPQ,QBUF)
        DIMENSION QBUF(1)

C       MAXIMUM NUMBER OF REGISTERS RETURNED
        PARAMETER (MAXLIST=20)

        CHARACTER *(*) DOCNAM
	REAL           PLIST(MAXLIST)
	INTEGER        ILIST(MAXLIST)
        INTEGER        NUMREGS(MAXICDOCS),NUMKEYS(MAXICDOCS)
	CHARACTER *81  OLDNAM(MAXICDOCS)
        CHARACTER      NULL,DINDEX
        LOGICAL        FIRST,ISDIGI,FOUNDIT,RUNTIME

C       NEEDED FOR FUTURE CALLS
        SAVE           IPQ, NUMREGS, NUMKEYS, OLDNAM, LOCDOC, ICLAST

	DATA           OLDNAM/MAXICDOCS*'*'/

	IF (FCHAR(6:6) .EQ. 'E') THEN
C          WANT TO CEASE USING A DOC FILE -----------------------------
C          DETERMINE WHICH OF THE OLD NAMES NEEDS TO BE OBLITERATED
           FOUNDIT = .FALSE.
           DO  ICORE=1,MAXICDOCS
	     IF  (DOCNAM .EQ. OLDNAM(ICORE)) THEN
C              CHANGE OLDNAM SO THAT IT CANNOT BE MATCHED WITH ANY FILE NAME.
	       OLDNAM(ICORE)  = '*' 
               FOUNDIT        = .TRUE.
C              DEALLOCATE THE CORRESPONDING RUNTIME ARRAY
               IPQ     = LOCDOC(ICORE)
               RUNTIME = .TRUE.
               CALL ALLOCIT(0,RUNTIME,IPQ,'RDDOCA2Q',IRTFLG)
               LOCDOC(ICORE) = 0
               GOTO 10
             ENDIF
           ENDDO

           IF (.NOT. FOUNDIT) THEN
              WRITE(NOUT,91) DOCNAM
91            FORMAT(' DOCUMENT FILE NOT IN-CORE: ',A)
           ENDIF

10         IGOT = 0
           DO I = 1,MAXICDOCS 
              IF (OLDNAM(I) .NE. '*') THEN
                 IGOT = IGOT + 1
                 IF (IGOT .EQ. 1) 
     &              WRITE(NOUT,*)'DOCUMENT FILES LEFT IN-CORE:'
                 ILEN = LNBLNK(OLDNAM(I))
                 WRITE(NOUT,92) OLDNAM(I)(1:ILEN)
92               FORMAT(5X,A)
              ENDIF
           ENDDO
           IF (IGOT .LE. 0)  
     &          WRITE(NOUT,*)'NO DOCUMENT FILES LEFT IN-CORE'
           WRITE(NOUT,*) ' '
           RETURN
	ENDIF	



C       WANT TO GET REGISTERS FROM A DOC FILE ------------------------

C       COMPARE WITH OLD NAMES
	IC     = 0
        FIRST  = .FALSE.

C       NAME IS MOST-LIKELY STILL THE SAME
        IF (ICLAST .GT. 0 .AND. ICLAST .LE. MAXICDOCS .AND.
     &      DOCNAM .EQ. OLDNAM(ICLAST)) THEN
C          DOCNAM IS ALREADY IN-CORE
           ICALL = 1
           IC    = ICLAST
           GOTO 1300
        ENDIF

C       WANT TO FILL IC = 1 FIRST NOW al
	DO ICORE = MAXICDOCS,1,-1
           IF (DOCNAM .EQ. OLDNAM(ICORE)) THEN
C             DOCNAM IS ALREADY IN-CORE
              ICALL = 1
              IC    = ICORE
              GOTO 1300
           ENDIF

C          REMEMBER WHICH LOCATIONS ARE EMPTY
           IF (OLDNAM(ICORE)  .EQ. '*') IC = ICORE
        ENDDO

C       DOCUMENT NAME NOT FOUND. IF A BUFFER SECTION IS FREE, THE 
C       DOCUMENT FILE CAN BE READ INTO CORE, TO BE AVAILABLE FOR
C       SUBSEQUENT ACCESSES. IF NONE IS FREE, GIVE ERROR MESSAGE.

        IF (IC .EQ. 0) THEN
C          ERROR -- NAME NOT FOUND, AND NO SPACE AVAILABLE
           WRITE(NOUT,90) DOCNAM
90         FORMAT(' *** NO IN-CORE SPACE AVAILABLE FOR: ',A,/)
           CALL ERRT(100,'RDDOCA2Q',IE)
           RETURN
        ELSE
C          NAME NOT FOUND, SPACE AVAILABLE
           FIRST      = .TRUE.
           ICALL      = 0
C          DELAY SETTING OLDNAM UNTIL EVERYTING IS OK ON RETRIEVAL
           OLDNAM(IC) = '*'
         ENDIF

C       DOCUMENT NAME FOUND OR BUFFER SPACE AVAILABLE ----------------
C       PROCEED TO PARSE REGISTER LINE, CHECK FOR ',' OR 'X'

1300	K1=7
        DO K = K1,NALPH
           KNOW = K
           IF (FCHAR(K:K) .EQ. ',' .OR. FCHAR(K:K) .EQ. 'X') GOTO 1414
        ENDDO
	CALL ERRT(26,'RDDOCA2Q',IE)
	RETURN

1414	IF (FCHAR(KNOW:KNOW) .EQ. ',') THEN

           KNOW = KNOW - 1

C          CHECK IF FIRST ENTRY IS A NUMBER OR AN INDEX OF A DO-LOOP

	   IF (ISDIGI(FCHAR(KNOW:KNOW))) THEN
c              ASSUME FIRST ENTRY IS A NUMBER
               CALL CHKNUMQ(FCHAR(K1:),KNOW-K1+1,'I',IKEY,FNUM,IER)
               IF (IER .NE. 0) THEN
                  CALL ERRT(26,'RDDOCA',IE)
                  RETURN
               ENDIF

           ELSE          
C              FIRST ENTRY IS A DO-LOOP INDEX. PUT ITS VALUE INTO ILIST.

               DINDEX = FCHAR(KNOW:KNOW)
               IDX    = ICHAR(DINDEX) -64
               IKEY   = IALPHA(IDX)+0.5
C              JUMP TO NEXT COMMA
	       KNOW   = KNOW + 1
           ENDIF

        ELSE
C          FIRST ENTRY IS A REGISTER

           KNOW = INDEX (FCHAR(K1:NALPH),',') + K1 - 1
           CALL CHKSTRQ(FCHAR(K1:),KNOW-K1,'I',KLIST,DUM,KNLIST,IER)
           IKEY = PARAM(KLIST) + 0.5
        ENDIF

        CALL CHKSTRQ(FCHAR(KNOW+1:),NALPH-KNOW,'I',ILIST,DUM,
     &               NLIST,IER)

C       REGISTER LIST OK -----------------------------------------
        IF (IKEY .LT. 0) THEN
C          DESIRE A COMMENTED KEY
            CALL DOCCOM(DOCNAM,NDOC,IKEY,PLIST,NLIST,IRTFLG)
           IF (IRTFLG .NE. 0) RETURN
           GOTO 333
        ELSEIF (NLIST .GT. MAXLIST) THEN
           MAXLISTT = MAXLIST
           WRITE(NOUT,*) 'REGISTER LIST LIMIT: ',MAXLISTT
           CALL ERRT(100,'RDDOCA2Q',NE)
           RETURN
        ELSEIF (FIRST) THEN
C          SET ARRAY DIMENSIONS 
           MAXX        = NLIST + 1
           NUMREGS(IC) = NLIST + 1
           MAXY        = MAXCOL * MAXKEY / MAXX
           NUMKEYS(IC) = MAXY
        ELSE
C          GET ARRAY SIZE SAVED WHEN FILLED
           MAXX = NUMREGS(IC)
           MAXY = NUMKEYS(IC)
  	   IF (NLIST .GT. (MAXX - 1)) THEN
              WRITE(NOUT,1447) NLIST -1
1447          FORMAT(' *** NUMBER OF REGISTERS REQUESTED: ',I2,
     &               ' NUMBER AVAILABLE IN-CORE: ',I2,/)
              NLIST = MAXX - 1
              CALL ERRT(100,'RDDOCA2Q',NE)
           ENDIF
        ENDIF              


        IF (FIRST) THEN
C          FIND MAXY BY READING DOC FILE TO GET MAX KEY NUMBER
           CALL DOCINFO(DOCNAM,NDOC,MAXY,MAXCOLS,.TRUE.,IRTFLG)
           IF (IRTFLG .NE. 0) THEN
              CALL ERRT(100,'RDDOCA2Q',NE)
              RETURN
           ENDIF

C          ALLOCATE THE RUNTIME ARRAY (ONLY NEED MAXY ROWS NOW)
           MEMWANT = MAXX * MAXY
           CALL ALLOCIT(MEMWANT,RUNTIME,IPQ,'RDDOCA2Q',IRTFLG)
           IF (IRTFLG .NE. 0) THEN
              CALL ERRT(100,'RDDOCA2Q',NE)
              RETURN
           ENDIF

C          KEEP POINTER IN LUNDOC
           LOCDOC(IC)  = IPQ
           NUMKEYS(IC) = MAXY

C          DO NOT PRINT OPEN MESSAGE
           IRTFLG = -9

        ELSE
C          RECOVER DOC FILE INFO FROM INCORE IMAGE 
           IPQ = LOCDOC(IC)
        ENDIF

C       RECOVER DOC FILE INFO
        CALL UNSDAL(DOCNAM,ICALL,NDOC,IKEY,PLIST,NLIST,QBUF,
     &              MAXY,MAXX,NKEY,IRTFLG)
        IF (IRTFLG .NE. 0) THEN
           CALL ERRT(100,'RDDOCA2Q',NE)
           RETURN
        ENDIF

C       SET REGISTER CONTENTS FROM PLIST --------------------------
333     DO  K=1,NLIST
          IP = ILIST(K)
          IF (IP .GT. 103) THEN
C            CAN ONLY HAVE 103 PARAMETER POSITIONS
             WRITE(NOUT,*) '*** PARAM ARRAY LIMIT < 104, GOT: ',IP
          ELSEIF (IP .GT. 0) THEN
C            PLACE RECOVERED REGISTERS IN PARAM ARRAY
             PARAM(IP) = PLIST(K)

C             WRITE(6,987) IP-1,PARAM(IP)
C987          FORMAT(' REGISTER (',I3,'): ',G11.3)
          ENDIF
        ENDDO

        IF (FIRST) THEN
C          DELAY SETTING NAME UNTIL EVERYTHING IS OK ON RETRIEVAL
           OLDNAM(IC) = DOCNAM
        ENDIF           
        ICLAST = IC

        IF (.NOT. SILENT) WRITE(NOUT,*) ' '
	RETURN
	END

