
C++************************************************************************
C
C RDPRAI.F  -- CREATED JAN 87
C              COMMENTS ALLOWED JAN 98 al
C *  AUTHOR:   ArDean Leith                                                *
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.    AUTHOR: J.FRANK      *
C *  COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, ALBANY, NY. *
C * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH            *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED        *
C * FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF       *
C * HEALTH RESEARCH INC.                                               *
C **********************************************************************
C
C    RDPRAI(ILIST,NMAX,NUMBER,ILOW,IHI,PROMPT,HELPFI,IRTFLG)
C
C    PURPOSE:  READ AN ALPHANUMERIC STRING, CHECK FOR ANY SPECIAL OPERATION,
C              RETURNS ARRAY OF INTEGERS, THE NUMBER OF INTEGERS, OR A
C              FLAG  TO INDICATE THAT ONE SHOULD RETURN TO PREVIOUS QUESTION.
C              CHECKS TO SEE THAT INTEGERS ARE WITHIN SPECIFIED RANGE.
C
C              ALLOWABLE STRINGS           NUMBERS ENTERED IN ARRAY
C                 1,2,8,66                  1, 2, 8, 66
C                 1 2 8 66                  1, 2, 8, 66
C                 1,2  8 66                 1, 2, 8, 66
C                 1,2-8 66                  1, 2, 3, 4, 5, 6, 7, 8, 66
C
C              (A - ONLY SIGNIFIES A RANGE IF THE VALUE FOR ILOW IS >= 0 !!,
C               OTHERWISE INPUT IS INTERPRETED AS A NEGATIVE NUMBER.)
C
C    PARAMETERS : ILIST     ARRAY FOR ANSWERS
C                 NMAX      MAX LENGTH OF ARRAY
C                 NUMBER    ON ENTRY IS MAX NUMBER OF ANSWERS TO BE RETURNED!!
C                            (<0 ON ENTRY IS FLAG TO ACCEPT NULL RESPONSE)
C                           ON RETURN IS NUMBER OF ANSWERS ACTUALLY RETURNED
C                 ILOW      BOTTEM OF RANGE (CAN'T BE BELOW THIS)
C                 IHI       TOP OF RANGE (CAN'T BE ABOVE THIS)
C                 PROMPT    SOLICITATION MESSAGE
C                           IF THE FIRST CHARACTER OF THE PROMPT IS A ~
C                           THEN THE REMAINING CHARACTERS OF THE PROMPT
C                           ARE CONSIDERED TO BE AN INPUT STRING FOR
C                           THE COMMAND AND IT DOES NOT ASK FOR INPUT
C                 HELPFI    NAME OF RELEVANT HELP FILE (OR NULL)
C                 IRTFLG    RETURN FLAG (0 IS NORMAL, -1 IS GOTO
C                              PREVIOUS QUESTION)
C      
C    CALLS:       RDPRMC
C
C    NOTE: CAN PERFORM REGISTER SUBSTITUTION NOW al.
C          IF THE LAST CHAR. ON A LINE OF INPUT IS A COMMA IT WILL ASK 
C          FOR ANOTHER LINE OF INPUT.
C
C          BE SURE NUMBER IS SET TO NMAX BEFORE CALLING THIS ROUTINE
C
C--************************************************************************

        SUBROUTINE RDPRAI(ILIST,NMAX,NUMBER,ILOW,IHI,PROMPT,
     &             HELPFI,IRTFLG)        

        COMMON /UNITS/ LUNDOC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
        COMMON /PARAM/ PARAM(103),NSEL(10)

        INTEGER         ILIST(NMAX)
        CHARACTER *(*)  PROMPT,HELPFI
        CHARACTER * 80  ANS
        CHARACTER * 11  POSDIG
	CHARACTER * 1   NULL,A
        LOGICAL         SKIP,RANGE,GORANG,REGSUB,FIRST 

        DATA POSDIG/'0123456789.'/

        NULL   = CHAR(0)

C       A NEGATIVE NUMBER INPUT IS A FLAG THAT NULL INPUT IS ACCEPTABLE
        IF (NUMBER .LT. 0) THEN
          NVAL = -NUMBER
          SKIP = .TRUE.
        ELSE
          SKIP = .FALSE.
          NVAL = NUMBER
        ENDIF

C       ERROR PROOF THE INPUT
        IF (NVAL .LE. 0 .OR. NVAL .GT. NMAX) NVAL = NMAX

        FIRST  = .TRUE.
        IRTFLG = 0
        NUMBER = 0

   20   CONTINUE
        I      = 0

        IF (PROMPT(:1) .EQ. '~' .AND. .NOT. FIRST) THEN
C         INPUT VIA PROMPT WAS BAD, DO NOT ASK FOR INTERACTIVE INPUT
          IRTFLG = 1
          RETURN
        ENDIF
        FIRST = .FALSE.

        IF (PROMPT(:1) .EQ. '~') THEN
C         USE PROMPT FOR INPUT LINE
          ANS   = PROMPT(2:)
          NCHAR = LEN(PROMPT) - 1

        ELSE
C..       GET INTERACTIVE INPUT LINE
          CALL RDPRMC(ANS,NCHAR,.TRUE.,PROMPT,HELPFI,IRTFLG)
          IF (IRTFLG .EQ. -1) RETURN
          IRTFLG = 0
        ENDIF

   50   I = I + 1
C       CHECK TO SEE IF WE HAVE RUN OUT OF INPUT CHAR.
        IF (I .GT. NCHAR .OR. A .EQ. ';' .OR. ANS(I:I) .EQ. ';') THEN

          IF (A .EQ. ',' .AND. PROMPT(:1) .NE. '~') THEN
C           GET A CONTINUATION LINE OF INPUT

            CALL RDPRMC(ANS,NCHAR,.TRUE.,
     &      'ENTER NEXT LINE OF INPUT',NULL,IRTFLG)
            IF (IRTFLG .EQ. -1) GOTO 20
            I = 0
            GOTO 50

          ELSEIF (NUMBER .EQ. 0 .AND. SKIP) THEN
C           GOT NULL INPUT WHERE IT IS ALLOWED AS AN INPUT
C           RETURN A FLAG THAT NULL ANSWER ACCEPTED
            IRTFLG = 3
            RETURN

          ELSE IF (NUMBER .EQ. 0) THEN
C           DIDN'T GET ANY NUMBERS IN INPUT
            WRITE(NOUT,94)
   94       FORMAT('*** NEEDS AT LEAST ONE INTEGER INPUT')
            GOTO 20
            
          ELSE
C           END OF INPUT

C**************************DEBUG
C        WRITE(6,9088)NUM1,NVAL,NUMBER,NCHAR,I1,I2,I,A,ANS(1:I)
C9088    FORMAT(' END NUM1,NVAL,NUMBER,NCHAR,I1,I2,I,A)',/,
C     &          7I5,' :',A1,' :',/)
C******************************************

            RETURN
          ENDIF
        ENDIF

        A = ANS(I:I)
C       CONTINUE CHECKING CHAR. UNTIL WE GET A NON BLANK CHAR.
        IF (A .EQ. ' ' .OR. A .EQ. ',' .OR. 
     &      A .EQ. '(' .OR. A .EQ. ')' ) GOTO 50

C       HAVE GOT FIRST CHAR IN NUMBER WHICH IS NON BLANK
        I1 = I
        RANGE  = .FALSE.
        REGSUB = .FALSE.


        IF (A .EQ. 'X' .OR. A .EQ. 'x') THEN
C          WANT A REGISTER SUBSTITUTION
           REGSUB = .TRUE.
           ANS(I:I) = ' '
               
 490       I = I + 1
           IF (I .GT. NCHAR) GOTO 7
           A = ANS(I:I)
C          IF HAVE A BLANK SPACE AFTER X. CONTINUE TO READ CHAR.
           IF (A .EQ. ' ') GOTO 490
C          IF NO MORE CHAR. OR NON DIGIT CHAR. HAVE INVALID INPUT
           IF( A .LT. '0' .OR. A .GT. '9') GOTO 7

        ELSEIF (A .EQ. '-' .OR. A .EQ. '+') THEN
C          A NUMBER HAS A SIGN OR A RANGE SPECIFIER
           IF (ILOW .GE. 0 .AND. A .EQ. '-' .AND. NUMBER .GT. 0)
     &        THEN
              RANGE = .TRUE.
              ANS(I:I) = ' '
           ENDIF
               
   49      I = I + 1
           IF (I .GT. NCHAR) GOTO 7
           A = ANS(I:I)
C          IF HAVE A BLANK SPACE AFTER SIGN. CONTINUE TO READ CHAR.
           IF (A .EQ. ' ') GOTO 49

C          IF NO MORE CHAR. OR NON DIGIT CHAR. HAVE INVALID INPUT
           IF( A .LT. '0' .OR. A .GT. '9' .AND. 
     &       (A .NE. 'X' .AND. A .NE. 'x')) GOTO 7
        ENDIF
     

   54   IF (A .LT. '0' .OR. A .GT. '9') THEN

C          INPUT CHAR. IS NOT A DIGIT. FIND WHAT IT IS.
           GORANG = .FALSE.

           IF (A .EQ. 'X' .OR. A .EQ. 'x') THEN
C            WANT A REGISTER SUBSTITUTION
             REGSUB = .TRUE.
             ANS(I:I) = ' '
	     
C	      COPY LINES TO AVOID AN INSIDE LOOP JUMP
C             GOTO 49
   79        I = I + 1
             IF (I .GT. NCHAR) GOTO 7
             A = ANS(I:I)
C            IF HAVE A BLANK SPACE AFTER SIGN. CONTINUE TO READ CHAR.
             IF (A .EQ. ' ') GOTO 79

C            IF NO MORE CHAR. OR NON DIGIT CHAR. HAVE INVALID INPUT
             IF( A .LT. '0' .OR. A .GT. '9' .AND. 
     &          (A .NE. 'X' .AND. A .NE. 'x')) GOTO 7
              GOTO 54    
	                 
           ELSEIF (A .EQ. '.') THEN
C             INPUT MUST BE INTEGER BUT IF IT HAS A DECIMAL POINT WITH
C             ZERO'S AFTER IT THEN WE SHOULD ACCEPT IT.
   51         I2 = I - 1

   56         I  =  I + 1
C             IF WE HAVE REACHED END OF NUMBER ACCEPT IT
              IF (I .GT. NCHAR) GOTO 55
              A  = ANS(I:I)
              IF (A .EQ. ' ' .OR. A .EQ. ',') THEN
C                END OF NUMBER, OK TO DECODE IT
                 GOTO 55

              ELSE IF(A .NE. '0') THEN
C               NON ZERO DIGIT FOLLOWING DECIMAL POINT
                WRITE(NOUT,95)
   95           FORMAT('*** INPUT MUST BE INTEGER')
                GOTO 20

              ELSE
C                DIGITS CONTINUE
                 GOTO 56
              ENDIF

           ELSE IF (A .EQ. ' ' .OR. A .EQ. ',' .OR. A .EQ. ')' .OR.
     &              A .EQ. ';') THEN
C             END OF THIS NUMBER
              GOTO 53

           ELSE IF (A .EQ. '-' .AND. ILOW .GE. 0) THEN
C             RANGE SPECIFIER              
              GORANG =.TRUE.
              GOTO 53

           ELSE
C             INVALID CHAR. IN NUMBER
              GOTO 7
           ENDIF

        ELSE
C          HAVE VALID DIGIT
           I = I + 1
C          CHECK TO SEE IF WE HAVE REACHED END OF INPUT
           IF (I .GT. NCHAR) GOTO 53
           A = ANS(I:I)
           GOTO 54
        ENDIF
 
C       FIND NUMBER OF CHAR. IN NUMBER
   53   I2 = I - 1
   55   N  = I2 - I1 + 1

C       CONVERT NUMBER TO INTEGER FORMAT
        NUMBER = NUMBER + 1
        IF (N .LE. 0) GOTO 7

        READ(ANS(I1:I2),82,ERR=70)NUM1
   82   FORMAT(I12)

        IF (REGSUB) THEN
C          FIND REGISTER CONTENTS
           IF (NUM1 .LT. 1 .OR. NUM1 .GT. 100) THEN
C             IMPROPER REGISTER NUMBER              
              WRITE(NOUT,920) NUM1
920           FORMAT('*** IMPROPER REGISTER: ',I8)
              GOTO 20
           ENDIF
           NUM1 = PARAM(NUM1 + 1)
        ENDIF

        IF (NUM1 .LT. ILOW .OR. NUM1 .GT. IHI) THEN
           WRITE(NOUT,92) NUMBER,ILOW,IHI
   92      FORMAT('*** INPUT ',I4,' OUT OF RANGE (',
     &            I6,'...',I6,')')
           GOTO 20
        ENDIF
        IF (RANGE) THEN
C          PRESENTLY IN A RANGE LOOP
           INCR = 1
           IF (ILIST(NUMBER-1) .GT. NUM1) INCR = -1
           DO  IT = (ILIST(NUMBER-1)+INCR),NUM1,INCR
              IF (NUMBER .GT. NVAL) THEN
                 WRITE(NOUT,8109) NVAL
 8109            FORMAT('*** LIST TRUNCATED, ONLY HOLDS ',I5,' VALUES')
                 NUMBER = NUMBER - 1
                 RETURN
              ENDIF
              ILIST(NUMBER) = IT
              NUMBER = NUMBER + 1
   	   ENDDO
           NUMBER = NUMBER -1
        ELSE 
           IF (GORANG) I = I - 1
           ILIST(NUMBER) = NUM1
        ENDIF

C       GET ANY REMAINING NUMBERS FROM THE INPUT STRING
        IF (NUMBER .LT. NVAL) GOTO 50

C**************************DEBUG
C        WRITE(6,9089)NUM1,NVAL,NUMBER,NCHAR,I1,I2,I,A
C9089    FORMAT(' END NUM1,NVAL,NUMBER,NCHAR,I1,I2,I,A)',/,
C     &          7I5,' :',A1,' :',/)
C******************************************
       
      RETURN

    7   CONTINUE
C         UNKNOWN ERROR SOMEWHERE IN INPUT
          WRITE(NOUT,91) ANS(1:I)
   91     FORMAT('*** SYNTAX ERROR: ',A40)
          GOTO 20

   70   CONTINUE
C         ERROR SOMEWHERE IN DECODING INPUT NUMBER
          WRITE(NOUT,90) ANS(I1:I2)
   90     FORMAT('*** SYNTAX ERROR; INTEGER TOO LARGE? ',A24)
          GOTO 20

       END
       
