
C++*********************************************************************
C   REDLIN.F                                 - USED IOSTAT 12/8/97 al
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.    AUTHOR: J.FRANK      *
C *  COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, ALBANY, NY. *
C * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH            *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED        *
C * FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF       *
C * HEALTH RESEARCH INC.                                               *
C **********************************************************************
C
C     REDLIN(LUN,BUF,NB,NREC)
C
C     PURPOSE:  READ A LINE OF FLOATING POINT NUMBERS FROM FILE
C
C     PARAMETERS:
C        LUN    LOGICAL UNIT NUMBER OF FILE BEING READ       (SENT)
C        BUF    BUFFER WHERE RECORD IS READ IN               (SENT/RET.)
C        NB     NUMBER OF VALUES IN RECORD TO BE READ        (SENT)
C        NREC   RECORD TO BE READ                            (SENT)
C
C     VARIABLES:
C        IERR   ERROR CODE > 0 IS RETURNED IN CASE OF ERROR. (RETURNED)
C               IERR IS PASSED IN COMMON /IOERR/IERR
C 
C--*******************************************************************

      SUBROUTINE REDLIN(LUN,BUF,NB,NREC)

      REAL            BUF(NB)
      COMMON /IOERR/  IERR
      COMMON /UNITS/  LUNC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
      COMMON /LUNARA/ LUNARA(100),LUNSTK(100)
      LOGICAL         USEINLN
      COMMON /LUNARB/ LOCLUN(100),NSAMLUN(100),NWORDLUN(100),USEINLN

CNO_SAVE

C     RUN-TIME ALLOCATION SPECIFIC CODE LINE FOLLOWS
#ifdef SP_NT
      INTEGER         LOCLUN
#else
      INTEGER * 8     LOCLUN
#endif

      IF (LOCLUN(LUN) .GT. 0) THEN
C        USER HAS ASKED FOR INLINED BUFFER
         CALL INLN_REDLIN(LUN,BUF,NB,NREC)
         RETURN
      ENDIF

C     ADD LUNARA(LUN)(FOR LABEL REC) AND LUNSTK( FOR STACK OFFSET)
C     TO NREC TO GET THE CORRECT RECORD NUMBER
      I = NREC + LUNARA(LUN) + LUNSTK(LUN)

C     USE OF IOSTAT NOW MEANS THAT IERR IS RESET TO ZERO ON EACH
C     SUCCESSFUL READ.  DEC 97 al
      READ(LUN,REC=I,IOSTAT=IERR) BUF

CC    IF (IERR .NE. 0) THEN
CC       WRITE(NOUT,*) '*** ERROR (',IERR,') READING RECORD: ',I, 
CC   &                 ' ON I/O UNIT: ',LUN
CC    ENDIF

      RETURN
      END

C++*********************************************************************
C   inln_REDLIN.FOR                                 MAHIEDDINE LADJADJ
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.    AUTHOR: J.FRANK      *
C *  COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, ALBANY, NY. *
C * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH            *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED        *
C * FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF       *
C * HEALTH RESEARCH INC.                                               *
C **********************************************************************
C
C     INLN_REDLIN(LUN,BUF,NB,NREC)
C
C     PURPOSE:    READ A LINE OF FLOATING POINT NUMBERS FROM BUFFER
C
C     PARAMETERS:
C        LUN    LOGICAL UNIT NUMBER OF FILE BEING READ
C        BUF    BUFFER WHERE RECORD IS READ IN
C        NB     NUMBER OF VALUES IN RECORD TO BE READ
C        NREC   RECORD TO BE READ
C
C        IERR   ERROR CODE 1 IS RETURNED IN CASE OF ERROR
C               IERR IS DEFINED IN COMMON /IOERR/IERR
C 
C       MANY COMMANDS IN SPIDER READ A FILE, MANIPULATE THE DATA, AND
C       WRITE THE RESULTS INTO A FILE, ONLY TO HAVE THE NEXT COMMAND
C       READ THE DATA BACK AGAIN IN CORE TO MANIPULATE IT AND STORE
C       IT BACK TO A FILE THEN START THE CYCLE AGAIN...
C       TO SPEED UP THE PROCESS, THE USER HAS THE ABILITY TO HAVE THAT 
C       DATA KEPT IN INCORE MEMORY UNTIL HE DECIDES THAT HE WANTS IT IN
C       A FILE. FOR THAT, SPIDER OFFERS AN INCORE MEMORY IN A COMMON BLOCK
C       BUFFER:
C
C       THIS BUFFER CAN BE SUBDIVIDED INTO TWO PART TO HOLD TWO FILES, 
C       CALLED INFILES.
C       BY DEFAULT, THE INFILES ARE CALLED ___1 AND ___2. 
C
C       DATA FROM A FILE IS STORED EXACTLY THE SAME WAY AS IT IS IN 
C       DISK FILE (I.E: A HEADER FOLLOWED BY NROWS OF DATA OF LENGTH NSAMS
C       REPEATED NSLICE TIMES).
C 
C--*******************************************************************

      SUBROUTINE INLN_REDLIN(LUN,BUF,NB,NREC)

      REAL            BUF(NB)
      COMMON /IOERR/  IERR
      COMMON /UNITS/  LUNC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
      COMMON /LUNARA/ LUNARA(100),LUNSTK(100)
      LOGICAL         USEINLN
      COMMON /LUNARB/ LOCLUN(100),NSAMLUN(100),NWORDLUN(100),USEINLN

CNO_SAVE

C     RUN-TIME ALLOCATION SPECIFIC CODE LINE FOLLOWS
      INTEGER * 8  LOCLUN

      POINTER      (IPT,Q)
      DIMENSION    Q(1)

      IPT  = LOCLUN(LUN)
      ILOC = 0
C     RUN-TIME ALLOCATION SPECIFIC CODE LINE ENDS


C     GET THE CORRECT OFFSET INTO THE BUFFER.
      IGO = (LUNARA(LUN) + LUNSTK(LUN) + NREC - 1) * NSAMLUN(LUN) + ILOC 

      IF ((IGO + NB - 1 - ILOC) .GT. NWORDLUN(LUN)) THEN
C        GOES OUT OF RESERVED BUFFER!
         WRITE(NOUT,*) '*** INLN_REDLIN ARRAY OVERFLOW, LUN: ',LUN,
     &       'LENGTH:' ,IGO + NB
         IERR = 1
         RETURN
      ENDIF

C     READ THE BUFFER (THERE MAY BE A FASTER WAY TO DO THIS!!)
      DO I= 1,NB
         BUF(I) = Q(IGO + I)
      ENDDO
      IERR = 0

      RETURN
      END

