
C++************************************************************************
C
C REFORM.FOR -- CREATED JAN 91
C
C **************************************************************************
C *  AUTHOR:  ArDean Leith                                                 *
C *  PART OF SPIDER - MODULAR IMAGE PROCESSING SYSTEM.                     *
C *  COPYRIGHT (C)1987 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C    REFORM
C
C    PURPOSE:         REFORM AN IMAGE STAGE BY ROTATION OF 90, 180 OR 270 
C                     DEGREES AROUND  THE X,Y, OR Z AXIS.  CENTER OF ROTATION
C                     IS ALWAYS CENTER OF THE RELEVANT SLICE
C                     ROTATIONS ARE CLOCKWISE WHEN LOOKING ALONG AXES.  THE
C                     X AXIS + TO RIGHT,  Y AXIS + UP THE SCREEN, Z AXIS + OUT
C                     OF THE SCREEN!
C
C    VARIABLES:       
C                     MAXDIM     COMMON BUFFER SIZE
C
C    CALLED BY:       REFORM0
C
C    CALLS:           REDLIN       WRTLIN        ERRT      
C
C        0         2         3         4         5         6         7     
C23456789012345678901234567890123456789012345678901234567890123456789012
C***********************************************************************

        SUBROUTINE REFORM(LUNIN,LUNOUT,NSAM,NSAM1,NSAM2,NSAMS,
     &           NROW,NROW1,NROW2,NROWS,NSLICE,NSLICE1,NSLICE2,NSLICES,
     &           AXIS,IANG,MAXDIM,IRTFLG)

        INCLUDE 'CMBLOCK.INC'

        COMMON           BUF(16000),BUF2(1)

        CHARACTER   * 1  AXIS

C       MAX. NO. OF VOXELS IN OUTPUT I*1 BUFFER
        MAXVOX = MAXDIM - 16000

C       SET IMAGE SIZE AND WINDOW
        NSAM3   = NSAM2 - NSAM1 + 1
        NROW3   = NROW2 - NROW1 + 1
        NSLICE3 = NSLICE2 - NSLICE1 + 1

        IF (IANG .EQ. 0 .OR. IANG .EQ. 180) THEN
           NSAMS   = NSAM3
           NROWS   = NROW3
           NSLICES = NSLICE3
       
        ELSEIF (AXIS .EQ. 'Z') THEN
C           3-D FILE WITH Z SLICE
            NSAMS   = NROW3
            NROWS   = NSAM3
            NSLICES = NSLICE3

        ELSEIF (AXIS .EQ. 'Y') THEN
C           3-D FILE WITH Y SLICE 
            NSAMS   = NSLICE3
            NSLICES = NSAM3
            NROWS   = NROW3

        ELSEIF (AXIS .EQ. 'X') THEN
C           3-D FILE WITH X SLICE
            NROWS   = NSLICE3
            NSLICES = NROW3
            NSAMS   = NSAM3
        ENDIF

        IF (AXIS .EQ. 'Z') THEN
C          ROTATE AROUND Z AXIS

           WRITE(NOUT,*) ' WARNING THIS IS A SLOW ROTATION'

           IF (IANG .EQ. 90 .OR. IANG .EQ. 270) THEN
C             ROTATE 90 OR -90 AROUND Z AXIS

              IT5 = NSAMS * (NROWS + NSAM1 - 1)

              DO ISLICE1 = NSLICE1, NSLICE2
                ISLICE2  = ISLICE1 - NSLICE1 + 1

                DO IROW1 = NROW1,NROW2
                  IREC1  = (ISLICE1 - 1) * NROW + IROW1
                  CALL REDLIN(LUNIN,BUF,NSAM,IREC1)

                  IF (IANG .EQ. 270) THEN
                     ISAM2 = NSAMS - (IROW1 - NROW1)
                     IT    = ISAM2 - (NSAM1 * NSAMS)
                  ELSE
                     ISAM2 = IROW1 - NROW1 + 1
                     IT3   = IT5 + ISAM2
                  ENDIF                  

                  DO ISAM1 = NSAM1,NSAM2
                     IF (IANG .EQ. 270) THEN
C                      IROW2  = ISAM1 - NSAM1 + 1
C                      IPT    = (IROW2 - 1) * NSAMS + ISAM2
C                      IPT    = (ISAM1 - NSAM1) * NSAMS + ISAM2
C                      IPT    = ISAM1 * NSAMS - NSAM1 * NSAMS + ISAM2
                       IPT    = ISAM1 * NSAMS + IT
                    ELSE
C                      IROW2  = NROWS - (ISAM1 - NSAM1)
C                      IPT    = (IROW2 - 1) * NSAMS + ISAM2
C                      IPT    = (NROWS - ISAM1 + NSAM1 - 1) * NSAMS + ISAM2
C                      IPT    = (NROWS + NSAM1 - ISAM1 - 1) * NSAMS + ISAM2
C                      IPT    = NROWS*NSAMS + NSAM1*NSAMS - ISAM1*NSAMS - NSAMS +
C                               ISAM2
C                      IT3    = NROWS*NSAMS + NSAM1*NSAMS - NSAMS + ISAM2
C                      IT5    = NSAMS * (NROWS + NSAM1 - 1)
C                      IT3    = IT5 + ISAM2
                       IPT    = IT3 - ISAM1 * NSAMS
                    ENDIF
                    BUF2(IPT) = BUF(ISAM1)
                  ENDDO
                ENDDO

C               OUTPUT THE REFORMED SLICE
                DO IROW2 = 1,NROWS
                  IPT    = (IROW2 - 1)   * NSAMS + 1
                  IREC2  = (ISLICE2 - 1) * NROWS + IROW2
                  CALL WRTLIN(LUNOUT,BUF2(IPT),NSAMS,IREC2)
                ENDDO
              ENDDO

           ELSEIF (IANG .EQ. 180) THEN
C             ROTATE 180 DEGREES AROUND Z AXIS  

              IT7        = NSAMS + NSAM1
              DO ISLICE1 = NSLICE1,NSLICE2
                ISLICE2  = ISLICE1 - NSLICE1 + 1
                DO IROW1 = NROW1,NROW2
                  IREC1  = (ISLICE1 -1) * NROW + IROW1
                  IROW2  = NROWS - (IROW1 - NROW1)
                  IREC2  = (ISLICE2 -1) * NROWS + IROW2
                  CALL REDLIN(LUNIN,BUF,NSAM,IREC1)

C                 INVERT THE ROW
                  DO ISAM1 = NSAM1,NSAM2
C*                   ISAM2 = NSAMS - (ISAM1 - NSAM1)
C*                   ISAM2 = NSAMS - ISAM1 + NSAM1
C*                   IT7   = NSAMS + NSAM1
C*                   ISAM2 = IT7   - ISAM1
C*                   BUF2(ISAM2) = BUF(ISAM1)
                     BUF2(IT7 - ISAM1) = BUF(ISAM1)
                  ENDDO
                  CALL WRTLIN(LUNOUT,BUF2,NSAMS,IREC2)
                ENDDO
              ENDDO                    
           ENDIF 

        ELSEIF (AXIS .EQ. 'X') THEN

           WRITE(NOUT,*) ' WARNING THIS IS A SLOW ROTATION'

           DO ISLICE1 = NSLICE1,NSLICE2
             DO IROW1 = NROW1,NROW2
               IREC1 = (ISLICE1-1) * NROW + IROW1
               CALL REDLIN(LUNIN,BUF,NSAM,IREC1)

               IF (IANG .EQ. 90) THEN
                 ISLICE2 = IROW1 - NROW1 + 1
                 IROW2   = NROWS - (ISLICE1 - NSLICE1)

               ELSEIF (IANG .EQ. 180) THEN
                 ISLICE2 = NSLICES - (ISLICE1 - NSLICE1)
                 IROW2   = NROWS   - (IROW1   - NROW1)

               ELSEIF (IANG .EQ. 270) THEN
                 ISLICE2 = NSLICES - (IROW1   - NROW1)
                 IROW2   = ISLICE1 - NSLICE1  + 1
               ENDIF

               IREC2 = (ISLICE2 - 1) * NROWS + IROW2
               CALL WRTLIN(LUNOUT,BUF(NSAM1),NSAMS,IREC2)

             ENDDO
           ENDDO

        ELSE IF (AXIS .EQ. 'Y') THEN

           IF (IANG .EQ. 180) THEN

            IT = NSAMS + NSAM1
            DO ISLICE1 = NSLICE1,NSLICE2
C             INVERT THE SLICES
              ISLICE2  = NSLICES - (ISLICE1 - NSLICE1)

              DO IROW1 = NROW1,NROW2
                IROW2  = IROW1 - NROW1 + 1
                IREC1  = (ISLICE1 - 1) * NROW + IROW1
                CALL REDLIN(LUNIN,BUF,NSAM,IREC1)

C               MIRROR THE COLUMS ALONG THE ROW
                DO ISAM1 = NSAM1,NSAM2
C*                 ISAM2 = NSAMS - (ISAM1 - NSAM1)
C*                 ISAM2 = IT - ISAM1
                   BUF2(IT - ISAM1) = BUF(ISAM1)
                ENDDO

                IREC2  = (ISLICE2 - 1) * NROWS + IROW2
                CALL WRTLIN(LUNOUT,BUF2(NSAM1),NSAMS,IREC2)
              ENDDO
            ENDDO

          ELSEIF (IANG .EQ. 90 .OR. IANG .EQ. 270) THEN
          
C          ROTATE AROUND Y AXIS, RELATIVELY SLOW!!

           WRITE(NOUT,*) ' WARNING THIS IS A SLOW ROTATION'

           IT4 = (NSLICES + NSAM1 - 1) * NSLICES 
           IT7 = - NSAM1 * NSLICES 

           DO IROW1  = NROW1,NROW2
             IROW2   = IROW1 - NROW1 + 1

             DO ISLICE1 = NSLICE1,NSLICE2
               IREC1    = (ISLICE1 - 1) * NROW + IROW1
               CALL REDLIN(LUNIN,BUF,NSAM,IREC1)

               IF (IANG .EQ. 90) THEN
                 ISAM2 = ISLICE1 - NSLICE1 + 1
               ELSE
                 ISAM2 = NSAMS   - (ISLICE1 - NSLICE1)
               ENDIF

               IF (IANG .EQ. 90) THEN
C*               ISLICE2 = NSLICES - (ISAM1 - NSAM1)
C*               ISLICE2 = NSLICES + NSAM1 - ISAM1
C*               ISLICE2 = IT - ISAM1
C*               IPT = (ISLICE2 - 1) * NSLICES + ISAM2
C*               IPT = (NSLICES + NSAM1 - ISAM1 - 1) * NSLICES + ISAM2
C*               IPT = (NSLICES + NSAM1 - ISAM1 - 1) * NSLICES + ISAM2
C*               IPT = (NSLICES + NSAM1 - 1) * NSLICES - ISAM1 * NSLICES + ISAM2
C*               IT4 = (NSLICES + NSAM1 - 1) * NSLICES 
C*               IPT = IT4 - ISAM1 * NSLICES + ISAM2
                 IT5 = IT4 + ISAM2
C*               IPT = IT5 - ISAM1 * NSLICES

                 DO ISAM1 = NSAM1,NSAM2
                    BUF2(IT5 - ISAM1 * NSLICES) = BUF(ISAM1)
                 ENDDO

               ELSE
C*               ISLICE2 = ISAM1 - NSAM1 + 1
C*               IPT = (ISLICE2 - 1) * NSLICES + ISAM2
C*               IPT = (ISAM1 - NSAM1 + 1 - 1) * NSLICES + ISAM2
C*               IPT =  ISAM1 * NSLICES  - NSAM1 * NSLICES + ISAM2
C*               IT7 =                   - NSAM1 * NSLICES 
                 IT9 =  ISAM2 + IT7               
C*               IPT =  ISAM1 * NSLICES + IT9

                 DO ISAM1 = NSAM1,NSAM2
                   BUF2(ISAM1 * NSLICES + IT9) = BUF(ISAM1)
                 ENDDO
               ENDIF
             ENDDO

C            WRITE OUT THE REFORMED SLICE
             IPT    = 1 - NSAMS
             IREC2  = IROW2 - NSLICES
             DO ISLICE2 = 1,NSLICES
C*              IREC2 = (ISLICE2 - 1) * NSLICES + IROW2
                IREC2 = IREC2 + NSLICES

                IPT   = IPT + NSAMS
                CALL WRTLIN(LUNOUT,BUF2(IPT),NSAMS,IREC2)
             ENDDO
           ENDDO
          ENDIF
        ENDIF
                 
99      CLOSE(LUNIN)
        CLOSE(LUNOUT)

	RETURN
	END

