
C++************************************************************************
C
C REFORM0.FOR -- CREATED JAN 91
C
C **************************************************************************
C *  AUTHOR:  ArDean Leith                                                 *
C *  PART OF SPIDER - MODULAR IMAGE PROCESSING SYSTEM.                     *
C *  COPYRIGHT (C)1987 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C    REFORM0
C
C    PURPOSE:         REFORM AN IMAGE STAGE BY ROTATION OF 90, 180 OR 270 
C                     DEGREES AROUND  THE X,Y, OR Z AXIS.  CENTER OF 
C                     ROTATION IS ALWAYS CENTER OF THE RELEVANT SLICE.
C
C    PARAMETERS:       
C                     MAXDIM     COMMON BUFFER SIZE
C
C    CALLED BY:       
C
C    CALLS:           REFORM
C
C        0         2         3         4         5         6         7     
C23456789012345678901234567890123456789012345678901234567890123456789012
C***********************************************************************

        SUBROUTINE REFORM0(LUNIN,LUNOUT,NSAM,NROW,NSLICE,MAXDIM,IRTFLG)

        INCLUDE 'CMBLOCK.INC'

        COMMON          BUF(16000),BUF2(1)
        COMMON /COMMUN/ FILNAM

        CHARACTER   *81  FILNAM
        CHARACTER   * 1  NULL,AXIS
        LOGICAL          ERRI2

        NULL       = CHAR(0)

C       INPUT FILE OPENED IN UTIL3, FIND OUTPUT FILE NAME
 2      CALL FILERD(FILNAM,NLET,NULL,'OUTPUT',IRTFLG)
        IF (IRTFLG .NE. 0) RETURN

        WRITE(NOUT,900)
900     FORMAT('  FOLLOWING THREE QUESTIONS REFER TO ORIGINAL ',
     &         'VOLUME DIMENSIONS',/)
3       NSAM1 = 1
        NSAM2 = NSAM
        CALL RDPRIS(NSAM1,NSAM2,NLOOP,ILOOP,
     &  'ENTER FIRST AND LAST X COLUMN NUMBER (OR <CR> FOR ALL)',IRTFLG)
        IF (IRTFLG .EQ. -1) GOTO 2
        IF (ERRI2(NSAM1,NSAM2,2,1,NSAM,NSAM1,NSAM)) GOTO 3
        NSAM3 = NSAM2 - NSAM1 + 1

4       NROW1 = 1
        NROW2 = NROW
        CALL RDPRIS(NROW1,NROW2,NLOOP,ILOOP,
     &  'ENTER FIRST AND LAST Y ROW NUMBER (OR <CR> FOR ALL)',IRTFLG)
        IF (IRTFLG .EQ. -1) GOTO 3
        IF (ERRI2(NROW1,NROW2,2,1,NROW,NROW1,NROW)) GOTO 4
        NROW3 = NROW2 - NROW1 + 1

5       NSLICE1 = 1
        NSLICE2 = NSLICE
        CALL RDPRIS(NSLICE1,NSLICE2,NLOOP,ILOOP,
     &  'ENTER FIRST AND LAST Z SLICE NUMBER (OR <CR> FOR ALL)',IRTFLG)
        IF (IRTFLG .EQ. -1) GOTO 4
        IF (ERRI2(NSLICE1,NSLICE2,2,1,NSLICE,NSLICE1,NSLICE)) GOTO 5
        NSLICE3 = NSLICE2 - NSLICE1 + 1

6       CALL RDPRMC(AXIS,NLET,.TRUE.,
     &  'ENTER ROTATION AXIS (X,Y,Z) (<CR> IS Z)',NULL,IRTFLG)
         IF (IRTFLG .EQ. -1) GOTO 5
        IF (NLET .EQ. 0) AXIS = 'Z'

7       WRITE(NOUT,*) ' ROTATIONS ARE CLOCKWISE WHEN FACING ALONG AXIS.'
        WRITE(NOUT,*) ' X AXIS POINTS TO RIGHT, Y AXIS POINTS DOWN THE '
        WRITE(NOUT,*) ' SCREEN , Z AXIS POINTS OUT OF THE SCREEN.'
        IANG = 90
        CALL RDPRIS(IANG,IDUM,NLOOP,ILOOP,
     &  'ENTER CLOCKWISE ROTATION ANGLE (90,180, OR 270)',IRTFLG)
        IF (IRTFLG .EQ. -1) GOTO 6
        IF (IANG .LT. 0) IANG = -IANG
        IF (IANG .NE. 90 .AND. IANG .NE. 180 .AND. IANG .NE. 270) THEN
           CALL ERRT(31,'REFORM',NE)
           GOTO 7
        ENDIF


C       SET IMAGE SIZE AND WINDOW
        NSAM3   = NSAM2 - NSAM1 + 1
        NROW3   = NROW2 - NROW1 + 1
        NSLICE3 = NSLICE2 - NSLICE1 + 1

        IF (AXIS .EQ. 'Z') THEN
C           3-D FILE WITH Z SLICE
            IF (IANG .EQ. 0 .OR. IANG .EQ. 180) THEN
               NSAMS   = NSAM3
               NROWS   = NROW3
            ELSE
               NSAMS   = NROW3
               NROWS   = NSAM3
            ENDIF
            NSLICES = NSLICE3
            NUMVOX  = MAX(NSAM,NSAMS)

        ELSEIF (AXIS .EQ. 'Y') THEN
C           3-D FILE WITH Y SLICE 

            IF (IANG .EQ. 0 .OR. IANG .EQ. 180) THEN
               NSAMS   = NSAM3
               NSLICES = NSLICE3
            ELSE
               NSAMS   = NSLICE3
               NSLICES = NSAM3
            ENDIF
            NROWS   = NROW3
            NUMVOX  = MAX(NSAM,NSAMS)

        ELSEIF (AXIS .EQ. 'X') THEN
C           3-D FILE WITH X SLICE

            IF (IANG .EQ. 0 .OR. IANG .EQ. 180) THEN
        
               NROWS   = NROW3
               NSLICES = NSLICE3
            ELSE
               NROWS   = NSLICE3
               NSLICES = NROW3
            ENDIF
            NSAMS   = NSAM3
            NUMVOX  = NSAMS * NROWS

        ENDIF

C       MAX. NO. OF VOXELS ALLOWED 
        MAXVOX  = MAXDIM - 16000

        IF (NUMVOX .GT. MAXVOX) THEN
           CALL ERRT(9,'REFORM0',NE)
           RETURN
        ENDIF
     
C       OPEN OUTPUT FILE (HAD TO WAIT TO CALUCLATE SIZE FROM AXIS INFO)
        CALL OPENF(FILNAM,LUNOUT,NSAMS,NROWS,'NEW',NF,NSLICES)
        IF (NF .NE. 2) RETURN

C       ROTATE THE VOLUME
        CALL REFORM(LUNIN,LUNOUT,NSAM,NSAM1,NSAM2,NSAMS,
     &              NROW,NROW1,NROW2,NROWS,
     &       NSLICE,NSLICE1,NSLICE2,NSLICES,AXIS,IANG,MAXDIM,IRTFLG)

	RETURN
	END

