C++*************************************************************************
C
C    RFACTSDO.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981, 1989 DIVISION OF LABORATORIES AND		   *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE DIVISION OF    *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE DIVISION OF LABORATORIES AND RESEARCH			   *
C **************************************************************************
C
C    RFACTSDO: DIFFERENTIAL R-FACTOR AND PHASE RESIDUAL COMPARISON OF 
C              TWO FOURIER TRANSFORMS ON A SERIES OF RINGS.  FOLLOWS
C	       PHILOSOPHY OF FRANK ET AL. SCIENCE 214 (1981) 1353-1355. 
C	       IN THE CURRENT ROUTINE, ALL RINGS ARE COMPUTED AT ONCE,
C 	       BUT SCALE SEARCH IS DONE ON EACH RING SEPARATELY.  THUS THE
C	       RESULT IS EQUIVALENT TO THE RESULT OF APPLYING A SERIES
C	       OF CALLS TO "RF S" WITH SUCCESSIVE RINGS.
C NOTE THAT THIS APPROACH WILL LEAD TO UNREASONABLE PHASE RESIDUAL RESULTS 
C IF THE TWO FOURIER TRANSFORMS HAVE STRONGLY DIFFERENT RADIAL BEHAVIORS.
C
C-************************************************************************

	SUBROUTINE RFACTSDO(MAXDIM)
        INCLUDE 'CMBLOCK.INC'
        PARAMETER(NSCALE=20)
	COMMON       BUF(1)
	DIMENSION    Q(1)
	POINTER      (IPQ,Q) 
	LOGICAL      RUNTIME
	CHARACTER *81   FILNAM1,FILNAM2
	CHARACTER*1       NULL
	DIMENSION       DLIST(6)
	EQUIVALENCE (JDUM,DUM)
	DATA  LUN1,LUN2/21,22/,NDOC/88/
CNO_SAVE
	NULL=CHAR(0)
	PI=4.0*ATAN(1.00)
C   ----------------------INPUT FIRST IMAGE
	CALL FILERD (FILNAM1,NLET,NULL,'FIRST INPUT',IRTFLG)
	CALL OPENFB(BUF,FILNAM1,LUN1,NSAM,NROW,'O',NF,NSLICE)
	IF (NF.EQ.1) THEN
         CALL ERRT(4,'RF',NE)
         RETURN
	ENDIF
	IF(IFORM.NE.1)THEN
         CLOSE(LUN1)
	 CALL ERRT(2,'RF',NE)
	 RETURN
	ENDIF
C   ----------------------INPUT SECOND IMAGE
	CALL FILERD (FILNAM2,NLET,NULL,'SECOND INPUT',IRTFLG)
	CALL OPENFB(BUF,FILNAM2,LUN2,NSAM2,NROW2,'O',NF,NSLICE)
	IF (NF.EQ.1) THEN
 	 CALL ERRT(4,'RF',NE)
	 RETURN
	ENDIF
	IF(IFORM.NE.1)THEN
	 CALL ERRT(2,'RF',NE)
	 RETURN
	ENDIF
C
	IF (NSAM.NE.NSAM2.OR.NROW.NE.NROW2) THEN	
	 CALL ERRT(1,'RF',NE)
	 CLOSE(LUN2)
	 RETURN
	ENDIF
C
	CALL RDPRM(WI,NLOOP,ILOOP,'RING WIDTH')
C
	CALL RDPRM2(SCALE1,SCALE2,NLOOP,ILOOP,
     &	  'SCALE FACTOR(LOWER,UPPER)')
C
	DSCALE=(SCALE2-SCALE1)/FLOAT(NSCALE-1)
	K1=1
	K2=K1+NSAM*NROW
	K3=K2+NSAM*NROW
	Y1=FLOAT(MAX0(NSAM,NROW))
	INC=INT(Y1/WI)/2+1
	K4=K3+INC*NSCALE
	K5=K4+INC*NSCALE
	K6=K5+INC*NSCALE
	K7=K6+INC
	K8=K7+INC*NSCALE
	K9=K8+INC*NSCALE

        MEMWANT=MAX0(K9+INC*NSCALE,K3+2*MAX0(NSAM,NROW))
        CALL CHKMEM(MEMWANT,BUF,MAXDIM,RUNTIME,IPQ,IRTFLG)
	IF(IRTFLG.NE.0)  THEN
	   CLOSE(LUN1)
	   CLOSE(LUN2)
	   CALL ERRT(6,'RF',NE)
	   RETURN
	ENDIF
          DO  I = 1,NROW
             CALL REDLIN(LUN1,Q(K1+(I-1)*NSAM),NSAM,I)
          ENDDO
          CLOSE(LUN1)
          DO  I = 1,NROW
             CALL REDLIN(LUN2,Q(K2+(I-1)*NSAM),NSAM,I)
          ENDDO
	  CLOSE(LUN2)
	INV=1
	CALL FMR_2(Q(K1),NSAM,NROW,Q(K3),INV)
	IF (INV.EQ.0) THEN
	   CALL ERRT(38,'RF ',NE)
	   RETURN
	ENDIF
	INV=1
	CALL FMR_2(Q(K2),NSAM,NROW,Q(K3),INV)
	IF (INV.EQ.0)THEN
	   CALL ERRT(38,'RF ',NE)
	   RETURN
	ENDIF 
	CALL RFACTSD2(Q(K1),Q(K2),Q(K3),Q(K4),Q(K5),Q(K6),
     &  Q(K7),Q(K8),Q(K9),NSAM,NROW,DSCALE,NSCALE,SCALE1,
     &  INC,Y1,WI)
C   ---------------TO WRITE RESULT INTO DOC FILE AND RESULT FILE
      NLIST=6
      WRITE(NOUT,*) 'THIS PROGRAM IS DEVELOPED FOR 2D RESOLUTION  '
      WRITE(NOUT,*) 'ESTIMATION BY THE METHOD OF PHASE RESIDUES AND' 
      WRITE(NOUT,*) 'FOURIER RING CORRELATION'
      WRITE(NOUT,*) 'FOLLOWING IS THE RESULT OF PR AND FRC '
      WRITE(NOUT,*) '(ALSO IN DOC FILE)'
      WRITE(NOUT,5600) WI
5600  FORMAT(1X,'RING WIDTH = ',G12.5)
      WRITE(NOUT,5700)
5700  FORMAT(10X,'|NUMBER|','|RING RADIUS',
     $ '|DPH|','|FRC|','|FRCCRIT|','|PIXELS|')		
      WRITE(NOUT,5800)
5800  FORMAT(10X,'        ',' NORMALIZED|')		
C
	DO 6000 L=1,INC 
	DUM=Q(K6+L-1)
	IF(DUM.EQ.0.0) GO TO 6000
	DLIST(1)= L
	DLIST(2)=FLOAT(L-1)/FLOAT(INC-1)*0.5
	DLIST(5)=3./SQRT(PI*WI*(FLOAT(L)-0.5))
	DLIST(6)=JDUM
C
	NSCM=0
	RFMIN=Q(K9+(L-1)*NSCALE)/Q(K4+(L-1)*NSCALE)
	NSCM=1
	LCUR=L
	IFOUND=0
	DO  NSC=1,NSCALE
	RFM=Q(K9+(L-1)*NSCALE+(NSC-1))/Q(K4+(L-1)*NSCALE+(NSC-1))
	IF(RFM.LT.RFMIN) THEN
	NSCM=NSC
	RFMIN=RFM
	IFOUND=1
	ENDIF
	ENDDO
C
	IF(IFOUND.EQ.0)   WRITE(NOUT,436)LCUR
436	FORMAT(' *** NO MINIMUM IN RFACT CURVE FOUND FOR RING = ',I5)
C
C NSCM IS THE NUMBER OF THE ELEMENT IN EACH ARRAY WITH THE CORRECT
C SCALING. SCALE IS THE CORRECT SCALING.
C
	BK1=Q(K4+(L-1)*NSCALE+NSCM)
	BK2=Q(K3+(L-1)*NSCALE+NSCM)
	IF(BK1.NE.0.) DLIST(3)=SQRT(BK2/BK1)
	BK3=Q(K8+(L-1)*NSCALE+NSCM)
	BK4=Q(K5+(L-1)*NSCALE+NSCM)
	BK5=Q(K7+(L-1)*NSCALE+NSCM)
	IF( BK3.NE.0.OR.BK4.NE.0.) DLIST(4)=BK5/SQRT(BK4*BK3)
756     CALL SAVD(NDOC, DLIST, NLIST, IRTFLG)
	WRITE(NOUT,6100)L,(DLIST(K),K=2,5),JDUM
6100    FORMAT (1X,I4,4(2X,F12.5),4X,I6)
6000	CONTINUE

	CALL SAVDC
	CLOSE(NDOC)
	IF(RUNTIME)  CALL ALLOCIT(0,RUNTIME,IPQ,'RF',IRTFLG)
	END
