C++*******************************************************************
C
C ROT32.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C
C    ROT32:    ROTATE AN IMAGE
C
C     THIS SUBROUTINE WILL ROTATE AN IMAGE BY AN ARBITRARY ANGLE OF
C     DEGREE BETWEEN 0 AND 360.  IT IS CALLED WHEN THE IMAGE IS SMALL
C     ENOUGH TO FIT IN THE BUFFER.
C
C    ROT32(LUNI,LUNO,NSAM,NROWS,NROWE,NROWSK,NBUF,THETA,AV,SHX,SHY)
C       LUNI         LOGICAL UNIT NUMBER OF INPUT IMAGE
C       LUNO         LOGICAL UNIT NUMBER OF OUTPUT IMAGE
C	NSAM	     ROW LENGTH
C	NROWS,NROWE  STARTING AND ENDING ROW
C                    (A NEGATIVE NROWE MEANS THAT THE BUFFER IS NOT
C                     GOING TO BE WRITTING IN DISK BUT LEFT IN BUFFER)
C       NROWSK       SKIPPING FACTOR FOR ROWS
C                    (A NEGATIVE VALUE MEANS NONSEQUENTIAL INPUT AND OUTPUT) 
C       NBUF         LENGTH OF BUFFER ARRAY AVAILABLE
C       THETA        ROTATION ANGLE IN RADIANS
C       AV           AVERAGE OF INPUT IMAGE
C       SHX,SHY      ORIGIN SHIFT
C
C              POSITIVE THETA: COUNTER-CLOCKWISE ROTATION
C                              (MATHEMATICALLY POSITIVE DIRECTION)
C
C		AN IMAGE CAN BE A SLICE OF A THREE-DIMENSIONAL DENSITY
C		DISTRIBUTION. FOR THIS REASON, A STARTING ROW UNEQUAL
C		TO 1 AND AN ENDING ROW UNEQUAL TO NROW IS PERMITTED.
C
C--*******************************************************************

	SUBROUTINE ROT32
     1  (LUNI,LUNO,NSAM,NROWS,NROWE,NROWSK,NBUF,THETA,AV,SHX,SHY)

	COMMON ADUM (80), BUF(1)
	COMMON /UNITS/LUN,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
	INTEGER   NBUF,NADDR,L,I4  
	DATA PI/3.14159/
	I4(I)=I

        IFLAG1=0
        IFLAG2=0
        IF (NROWSK.LT.0) IFLAG1=1
        IF (NROWSK.LT.0) NROWSK=-NROWSK
        IF (NROWE.LT.0)  IFLAG2=1
        IF (NROWE.LT.0)  NROWE=-NROWE	
        IF ((THETA.NE.0.).OR.(IFLAG2.EQ.1)) GOTO 4
C	CALL COPY(LUNI,LUNO,NSAM,NROW)
        II=0
	DO  I=NROWS,NROWE,NROWSK
          IF (IFLAG1.EQ.0) II=II+1
          IF (IFLAG1.EQ.1) II=I
          CALL REDLIN(LUNI,BUF,NSAM,I)
	  CALL WRTLIN(LUNO,BUF,NSAM,II)
	ENDDO
	RETURN
4       CONTINUE
C
 	NSAMH = NSAM/2
	NROW=((NROWE-NROWS)/NROWSK)+1
	NROWH = NROW/2
	KCENT = NSAMH+1
	ICENT = NROWH+1
C
	NOFF1 = NSAM
	IF (NOFF1+I4(NSAM)*I4(NROW).LE.NBUF) GOTO 10
	CALL ERRT(34,'ROT32 ',NE)
	RETURN

C HERE INSERT CORRECT ERROR NUMBER. DIDN'T HAVE IT HANDY.
10	IF(THETA.GT.PI) THETA = -2.*PI+THETA
	IF(THETA.LT.-PI) THETA = 2.*PI+THETA
	COD = COS(THETA)
	SID = SIN(THETA)
C
C READ IN INPUT MATRIX
        J=0
	DO  I = NROWS,NROWE,NROWSK
        J=J+1
        L = I4(J-1)*I4(NSAM)+1
        CALL REDLIN(LUNI,BUF(NOFF1+L),NSAM,I)        
	ENDDO
C
C
C NOW GO THROUGH OUTPUT COO SYSTEM; COMPUTE, FOR EACH POINT IN
C ROW, THE POSITION IN THE OLD COO SYSTEM.
C THEN CALCULATE POINT FROM FOUR SURROUNDING POINTS USING BILINEAR
C INTERPOLATION. WRITE OUT EACH LINE AS YOU GO ALONG.
C
C  JUST TO ALLOW A CHANGE IN THE ROTATIONAL CENTER
C   (RELATIVE TO THE CENTRAL PIXEL)
C
        RICENT=ICENT+SHY
        RKCENT=KCENT+SHX  
C
        NOFF2=NOFF1+NSAM*NROW
        JJ=0
	DO  I = 1,NROW
        JJ=JJ+1
        IF (IFLAG1.EQ.1) II=NROWS+(I-1)*NROWSK
        IF (IFLAG1.EQ.0) II=I
	Y = I-RICENT
	YCOD = Y*COD + RICENT
	YSID = -Y*SID + RKCENT
	DO 80 K = 1,NSAM
	BUF(K) = AV
	X = K-RKCENT
	XOLD = X*COD + YSID
	YOLD = X*SID + YCOD
	IYOLD = YOLD
	IF(IYOLD.LT.1.OR.IYOLD.GT.NROW-1) GOTO 80
	YDIF = YOLD -IYOLD
	YREM = 1.-YDIF
	IXOLD = XOLD
	IF(IXOLD.LT.1.OR.IXOLD.GT.NSAM-1) GOTO 80
	XDIF = XOLD - IXOLD
	XREM = 1.-XDIF
	NADDR = I4(IYOLD-1)*I4(NSAM) + IXOLD + NOFF1 
        IF (IFLAG2.EQ.0) THEN 
         	BUF(K) = YDIF*(BUF(NADDR+NSAM)*XREM
     1                   +BUF(NADDR+NSAM+1)*XDIF)
     2                   +YREM*(BUF(NADDR)*XREM + BUF(NADDR+1)*XDIF)
     
                    ELSE IF (IFLAG2.EQ.1) THEN
	        BUF(NOFF2+(JJ-1)*NROW+K) = YDIF*(BUF(NADDR+NSAM)*XREM
     1                   +BUF(NADDR+NSAM+1)*XDIF)
     2                   +YREM*(BUF(NADDR)*XREM + BUF(NADDR+1)*XDIF)
        END IF
80	CONTINUE
        IF (IFLAG2.EQ.0) CALL WRTLIN (LUNO,BUF,NSAM,II)
	ENDDO
	RETURN
	END
