
C++************************************************************************
C
C SAVDOCQ.F                                       REWRITTEN SEPT 96 al
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C  SAVDOCQ(DOCNAM,NLET,IRTFLG)
C
C  PURPOSE:    SUBROUTINE TO SAVE PARTICULAR REGISTERS IN A 
C              DOCUMENT FILE.  CALLED FROM COMMAND LINE.
C
C  PARAMETERS:
C      DOCNAME   FILE NAME                       (SENT)
C      NLET      NUMBER OF CHAR IN DOCNAME       (SENT)
C      IRTFLG    ERROR FLAG                      (RETURNED)
C
C23456789012345678901234567890123456789012345678901234567890123456789012
C--*********************************************************************

	SUBROUTINE SAVDOCQ(DOCNAM,NLET,IRTFLG)

        INCLUDE 'CMBLOCK.INC' 
CNO_SAVE

C	MAX. NUMBER OF SIMULTANEOUS DOCUMENT FILES ALLOWED
	PARAMETER      (MAXICDOCS = 20) 
        CHARACTER *81  DCFILE(MAXICDOCS)

        CHARACTER *(*) DOCNAM

        CHARACTER *9   CDATT
        CHARACTER *8   CTIMT
        CHARACTER      DINDEX,CTEMP

C       MAXIMUM NUMBER OF REGISTERS SAVED
        PARAMETER      (MAXLIST=20)
	DIMENSION      DLIST(MAXLIST),ILIST(MAXLIST)
	LOGICAL        NEWFILE

C       NEEDED FOR FUTURE CALLS
        SAVE           DCFILE, ICLAST

        DATA           DCFILE/MAXICDOCS*'*'/
        DATA           ICLAST/0/

C       SET ERROR RETURN
        IRTFLG = 1

        IF (FCHAR(4:4) .EQ. 'E') THEN
C          WANT TO CEASE USING A DOC FILE -----------------------------
C          DETERMINE WHICH OF THE OLD NAMES NEEDS TO BE CLOSED
           DO  IC=1,MAXICDOCS
             IF  (DOCNAM .EQ. DCFILE(IC)) THEN
C              CHANGE DCFILE SO THAT IT CANNOT BE MATCHED WITH ANY FILE NAME.
               DCFILE(IC)  = '*' 
               CLOSE(200+IC)
               IRTFLG = 0
               RETURN
             ENDIF
           ENDDO

           WRITE(NOUT,*) ' DOCUMENT FILE NOT OPEN NOW '
          RETURN
        ENDIF



C       COMPARE NEW DOCUMNET FILE WITH OLD NAMES ---------------------

        IC     = 0

C       NAME IS MOST-LIKELY STILL THE SAME
        IF (ICLAST .GT. 0 .AND. ICLAST .LE. MAXICDOCS .AND.
     &      DOCNAM .EQ. DCFILE(ICLAST)) THEN
C          DOCNAM IS ALREADY IN-CORE
           IC    = ICLAST
           NDOC  = 200 + IC
           GOTO 14
        ENDIF

C       FILL FROM FRONT TO BACK
        DO ICT = 1,MAXICDOCS
           IF (DOCNAM .EQ. DCFILE(ICT)) THEN
C             DOCNAM IS ALREADY OPEN
              IC    = ICT
              NDOC = 200 + IC
              GOTO 14
           ENDIF

C          REMEMBER FIRST EMPTY LOCATION
           IF (IC .EQ. 0 .AND. DCFILE(ICT)  .EQ. '*') IC = ICT
        ENDDO

C       DOCUMENT NAME NOT FOUND IN LIST. 
        IF (IC .EQ. 0) THEN
C          LIST FULL, CLOSE SOMETHING & OPEN THIS
           IC = ICLAST + 1
           IF (IC .GT. MAXICDOCS) IC = 1
           DCFILE(IC) = '*'
           CLOSE(200 + IC)
        ENDIF

C       OPEN DOC FILE ON UNIT IC + 200 
        NDOC = IC + 200       
        CALL OPENDOC(DOCNAM,.FALSE.,NLET,NDOC,.FALSE.,
     &               .FALSE.,.TRUE.,.FALSE.,NEWFILE,IRTFLG)
        IF (IRTFLG .NE. 0) RETURN


C       FILE PROBABLY HASN'T BEEN ACCESSED YET DURING THIS RUN.
C       PUT NEW TITLE IN FILE
        CALL DATE(CDATT)
        CALL TIME(CTIMT)
	WRITE(NDOC,93)PRJEXC(1:3),DATEXC(1:3), CDATT,CTIMT,DOCNAM(:NLET)
93      FORMAT(' ; ',A,'/',A,3X,A,'  AT ',A,3X,A)

C       PUT NAME IN LIST OF OPENED FILES
        DCFILE(IC)= DOCNAM

C       PARSE REGISTER LINE, CHECK FOR ',' OR 'X' ---------------------

C       CHECK IF FIRST ENTRY IS A REGISTER, INTEGER, OR COMMENT
14 	DO K = 4,80
          CTEMP = FCHAR(K:K)
	  IF (CTEMP .EQ. ',' .OR. CTEMP .EQ. 'X' .OR.
     &        CTEMP .EQ. '/') GOTO 15
        ENDDO
	CALL ERRT(16,'SAVDOCQ',NE)
	RETURN

15      ICLAST = IC

        IF (CTEMP .EQ. '/') THEN
C          JUST WANT TO PUT COMMENT IN THE DOC FILE.
           WRITE(NDOC,94) FCHAR(1:80)
94         FORMAT(' ; ',A)
           IRTFLG = 0
           RETURN

	ELSEIF (CTEMP .EQ. 'X') THEN
C         FIRST ENTRY IS A REGISTER. PUT REGISTER CONTENTS IN DLIST.
          K = INDEX(FCHAR(4:80),',') + 3
	  IF (K .LE. 5) THEN
             CALL ERRT(16,'SAVDOCQ',NE)
             RETURN

          ELSE
            CALL CHKSTRQ(FCHAR(4:),K-4,'I',KLIST,DUM,NLIST,IER)
            IF (IER .NE. 0) THEN
               CALL ERRT(16,'SAVDOCQ',NE)
               RETURN
            ENDIF
	    DLIST(1) = PARAM(KLIST)
          ENDIF
       
	ELSEIF (FCHAR(K-1:K-1) .GE. 'A' .AND. 
     &          FCHAR(K-1:K-1) .LE. 'Z' .AND. 
     &          FCHAR(K-1:K-1) .NE. 'X') THEN
C          FIRST ENTRY IS A DO LOOP INDEX, PUT ITS VALUE INTO DLIST(1).
           DINDEX = FCHAR(K-1:K-1)
           IDX    = ICHAR(DINDEX) - 64
           DLIST(1) = IALPHA(IDX)

        ELSE
C          FIRST ENTRY IS A NUMBER. PUT NUMBER IN DLIST
           CALL CHKNUMQ(FCHAR(4:),K-4,'I',NUM,FNUM,IER)
           IF (IER.NE.0) THEN
              CALL ERRT(16,'SAVDOCQ',NE)
              RETURN
           ENDIF
           DLIST(1) = NUM
        ENDIF

        CALL CHKSTRQ(FCHAR(K+1:),79-K,'I',ILIST,DUM,NLIST,IER)
        IF (IER .NE. 0) THEN
           CALL ERRT(16,'SAVDOCQ',NE)
           RETURN
        ENDIF


C       TITLE HAS BEEN ADDED FOR THIS RUN, WRITE DATA ----------------
	DO  K=1,NLIST
          K1 = ILIST(K)
          DLIST(K+1) = PARAM(K1)
        ENDDO

	NLIST = NLIST+1

C       ICOUNT IS THE NUMBER OF VALUES IN THAT LINE.
        IGO    = 2
        LEFT   = NLIST - IGO + 1
	ICOUNT = MIN0(LEFT,6)

        IF (DLIST(1) .LT. 0.0) THEN
C         THIS IS A COMMENT KEY
          IKEY  = ABS(DLIST(1))
          IF (IKEY .GT. 999) THEN
            WRITE(NOUT,95) IKEY
95          FORMAT(' *** COMMENT KEY:',I5,' MUST BE < 1000'/)
            CALL ERRT(100,'SAVDOCQ',NE)
            RETURN
          ENDIF

          WRITE(NDOC,96) IKEY,ICOUNT,(DLIST(K),K=2,ICOUNT+1)
96        FORMAT(' ;',I3,' ',I1,6G12.5)
C         CONTINUATION LINES NOT SUPPORTED FOR COMMENT KEYS

        ELSE
          IKEY = DLIST(1) + 0.5
          IF (IKEY .GT. 99999) THEN
            WRITE(NOUT,97) IKEY
97          FORMAT(' *** KEY:',I7,' MUST BE < 100000'/)
            CALL ERRT(100,'SAVDOCQ',NE)
            RETURN
          ENDIF

          WRITE(NDOC,1462) IKEY,ICOUNT,(DLIST(K),K=2,ICOUNT+1)
1462	  FORMAT(I5,' ',I1,6G12.5)

20        IGO    = IGO + ICOUNT
          LEFT   = NLIST - IGO + 1
	  ICOUNT = MIN0(LEFT,6)

          IF (LEFT .GT. 0) THEN
C           CONTINUATION LINE OF REGISTERS
            WRITE(NDOC,98) ICOUNT,(DLIST(K),K=IGO,IGO+ICOUNT-1)
98          FORMAT('  -99 ',I1,6G12.5)
            GOTO 20
          ENDIF
        ENDIF


C       LEAVE FILE OPEN
        IRTFLG = 0

	RETURN
	END

