
C***************************************************************************
C        
C **************************************************************************
C *  COPYRIGHT (C)1987 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C	NFAC 	NUMBER OF EIGENVECTORS REQUESTED
C       NUMIM	NUMBER OF IMAGES
C       NACT    NUMBER OF ACTIVE IMAGES
C	NPIX    NUMBER OF PIXELS PER IMAGE
C	LSAV	SEQUENTIAL IMAGE FILE (INPUT)
C	LIMA    IMAGE COORDINATE FILE (OUTPUT)
C	LPIX    PIXEL COORDINATE FILE (OUTPUT)
C	LEIG	EIGENSTUFF FILE (OUTPUT)
C	LTMP	SCRATCH FILE
C
C       ALL FILES MUST BE UNFORMATTED!
C	Q	ARRAY OF FOUR - BYTE WORDS WITH DIMENSION MEM
C	LUV	LOGICAL UTILITY VECTOR (1 = ACTIVE,  0 = INACTIVE)
C
C***************************************************************************

        SUBROUTINE SCORAN3(NUMIM, NFAC, NSAM, NROW, NPIX, LSAV, 
     &	                   LIMA, LPIX, LEIG, Q, MEM, LUV, 
     &                     IMAR, MADAR, NACT, NOUT)

        DIMENSION  LUV(*), IMAR(*), MADAR(*),Q(*)
        INTEGER    K, LTRANSP, LTMP, ANS,SIZE1,SIZE2,TSIZE 
        DIMENSION  B(1)
        POINTER    (IPQ,B) 
        LOGICAL    RUNTIME 

        DATA     LTMP, LTRANSP/78, 79/
CNO_SAVE          

        NINACT = NUMIM - NACT 
        NFAC   =  MIN0(NFAC + 1, NACT, NPIX) - 1

C       CREATE HEADERS FOR COO FILES IMC ***  AND PIX *** 
        CALL REW(LIMA, 0)
        WRITE(LIMA) NUMIM, NFAC, NSAM, NROW, NINACT
        CALL REW(LPIX, 0)
        WRITE(LPIX) NPIX, NFAC, NSAM, NROW, NINACT

C       TRY TO DO SOME CALCULATIONS TO CHOOSE THE QUICKEST METHOD
C       ACCORDING TO THE SIZE OF THE PROBLEM

        SIZE1 = NPIX * NPIX + 4 * NPIX + NFAC 
        TSIZE = NACT * NACT + 4 * NACT + NFAC 
	SIZE2 = MAX0( (NPIX*NACT), TSIZE)
        TSIZE = MIN0(SIZE1,SIZE2)
	WRITE(NOUT,7891)  SIZE1,SIZE2,MEM
7891	FORMAT(' Memory needed. In-core:',I8,'  Transposed:',I8,
     &     ' Available:',I10)

        IF (200*NFAC.LT. MIN0(NPIX,NACT)) THEN
c	   SMALL NUMBER OF FACTORS: GOTO ITERATIVE METHOD	
           GOTO 310
	ELSE
           CALL CHKMEM(TSIZE,Q,MEM,RUNTIME,IPQ,IRTFLG)
           IF (IRTFLG .EQ. 0)  THEN  
	      IF (TSIZE .EQ. SIZE1) GOTO 110
	      GOTO 210
	   END IF
	END IF
	GOTO 310


C       IN  -  CORE SOLUTION ----------------------------------------
C       MEMORY SEGMENTATION POINTERS

110     LD    =  1
        LS    =  LD  +  NPIX
        LU    =  LS  +  NPIX * NPIX
        LPJ   =  LU  +  NPIX
        LCO   =  LPJ +  NPIX
        LW    =  LCO +  NPIX 
        LMAX  =  LW  +  NFAC
        WRITE(NOUT,*) ' --- IN - CORE SOLUTION'
 
        CALL INCOR3(NUMIM, NPIX, NACT, NFAC, B(LS), B(LD), 
     &        B(LU), B(LPJ), TRACE, SOM, LSAV, LUV)

        CALL  GETCOO(NFAC, NPIX, NPIX, NUMIM, B(LS), B(LD), 
     &		     TRACE, B(LPJ), SOM, B(LCO), B(LU), B(LW),
     &		     LSAV, LIMA, LPIX, LUV, MADAR)

        WRITE(LEIG) NFAC, SOM, TRACE
        WRITE(LEIG) (B(LD + K), K = 0, NFAC - 1)

        IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'scoran3',IRTFLG) 
        RETURN


C       TRANSPOSED  SOLUTION ----------------------------------------
C       MEMORY SEGMENTATION POINTERS  
C       NOTE: TRANSPOSE USES T(NPIX,NACT); IMPORTANT FOR B ARRAY ALLOCATION
     
210     LD    =  1
        LS    =  LD  +  NACT
        LU    =  LS  +  NACT * NACT
        LPJ   =  LU  +  NACT
        LCO   =  LPJ +  NACT
        LW    =  LCO +  NACT
        LMAX  =  LW  +  NFAC

        IF (NACT .EQ. NUMIM)  THEN
          WRITE(NOUT,*)
     &         ' --- IN - CORE SOLUTION USING TRANSPOSED DATA'

          OPEN(FILE = 'TRANSPOSED.FILE', UNIT = LTRANSP, 
     &			STATUS = 'UNKNOWN', FORM = 'UNFORMATTED')
 
          CALL TRANSPOSE(B, NPIX, NACT, NUMIM, LUV, LSAV,
     &                      LTRANSP)

          CALL INCOR3(NPIX, NACT, NPIX, NFAC, B(LS), B(LD), 
     &          B(LU), B(LPJ), TRACE, SOM, LTRANSP, MADAR)

          CALL  GETCOOT(NFAC, NACT, NACT, NPIX, B(LS), B(LD), 
     &                      TRACE, B(LPJ), SOM, B(LCO), B(LU), B(LW),
     &                      LTRANSP,LPIX, LIMA, MADAR, LUV)

          CLOSE(LTRANSP, STATUS = 'DELETE')
          WRITE(LEIG) NFAC, SOM, TRACE
          WRITE(LEIG) (B(LD + K), K = 0, NFAC - 1)
	  
          IF(RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'scoran3',IRTFLG) 
	  
          RETURN
        ELSE
          WRITE(NOUT,*)'MEMORY SUFFICIENT FOR IN - CORE SOLUTION '
          WRITE(NOUT,*)'USING TRANSPOSED DATA. UNFORTUNATELY '
          WRITE(NOUT,*)'CANNOT HANDLE INACTIVE IMAGES'
          WRITE(NOUT,*)'***>>> TRYING ITERATIVE SOLUTION '
	  
          IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'scoran3',IRTFLG)
        END IF
      
 
C       ITERATIVE SOLUTION --------------------------------------------
C	MEMORY SEGMENTATION POINTERS

310     JBASE  =  NFAC  +  3
        LBASE  =  JBASE
        KB     =  LBASE
        LS	 =  1
        LD	 =  LS   +  NPIX * JBASE
        LPJ	 =  LD   +  NPIX
        LCO	 =  LPJ  +  NPIX
        LU	 =  LCO  +  NPIX
        LV	 =  LU   +  NPIX
        LW	 =  LV   +  NPIX
        LBZ	 =  LW   +  NFAC
        LAD	 =  LBZ  +  NPIX * JBASE
        LINB     =  LAD  +  JBASE * JBASE
        LMAX     =  LINB +  JBASE
	
	CALL CHKMEM(LMAX,Q,MEM,RUNTIME,IPQ,IRTFLG)
        IF (IRTFLG .NE. 0)  THEN 
           WRITE(NOUT,* )
     &       ' *** MEMORY INSUFFICIENT FOR ITERATIVE SOLUTION'
           WRITE(NOUT,*)'    NEEDED ', LMAX, ' AVAILABLE: ', MEM
           CALL ERRT(100, 'SCORAN3', NE)
           RETURN
        ENDIF

        OPEN(LTMP, FILE = 'SCR999.DAT', STATUS = 'UNKNOWN', 
     &             FORM = 'UNFORMATTED')
 
        CALL STOCHA(JBASE, NPIX, NUMIM, NFAC, NITER, LUV, B(LPJ), 
     &	            B(LS), B(LU), TRACE, SOM, B(LV), B(LBZ), B(LD), 
     &	            B(LAD), B(LINB), LSAV, LTMP)

        CLOSE(LTMP, STATUS = 'DELETE')

C       PUT EIGENVECTORS IN DECENDING ORDER ...
        CALL ORDERE(NPIX,NFAC,B(LS),B(LD))

        CALL GETCOO(NFAC, KB, NPIX, NUMIM, B(LS), B(LD), 
     &              TRACE, B(LPJ), SOM, B(LCO), B(LU), B(LW),
     &	            LSAV, LIMA, LPIX, LUV, MADAR)
        
        WRITE(LEIG) NFAC, SOM, TRACE
        WRITE(LEIG) (B(LD + K), K = 0, NFAC - 1)

        IF (RUNTIME) CALL ALLOCIT(0,RUNTIME,IPQ,'SCORAN3',IRTFLG) 

        RETURN
        END

