
C++************************************************************************
C
C SETLAB.FOR -- CREATED NOV 87  BY ArDean Leith
C
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **********************************************************************
C
C    SETLAB(LUN,LENREC,BUF,IBUF1,NBUF,VALUE,TYPE,IRTFLG)
C
C    PURPOSE:    THIS SUBROUTINE SETS HEADER PARAMETERS BY BUFFER NUMBER
C                WRITES THE HEADER INTO THE FILE.
C
C    PARAMETERS:
C         
C             LUN          LOGICAL UNIT NUMBER OF FILE 
C             BUF          WORK SPACE FOR READ/WRITE BUFFER
C             LENREC       NUMBER OF SAMPLES IN THE FILE (PIXELS)
C             IBUF1        FIRST BUFFER POSITION TO BE SET
C             NBUF         NUMBER OF BUFFER POSITIONS TO BE SET
C             VALUES       ARRAY FOR BUFFER VALUES TO BE SET
C             TYPE         CHARACTER VARIABLE CONTAINING FLAG FOR IFORM
C			   TYPE SYMBOL	DATA TYPE	IFORM
C                           R		 2-D REAL	 +1
C                           R3		 3-D REAL	 +3
C                           P		 2-D POLAR	 +2
C                           D		 NON-IMAGE DATA   0
C                           F		 2-D FOURIER	 -1
C                           F3		 3-D FOURIER  	 -3
C                           U		 UNCHANGED	 UNCHANGED	
C                           Q2           2-D FOURIER     -5 PAUL'S FORMAT
C                           Q3           3-D FOURIER     -7     "   "
C               -7    FQ    3-D FOURIER TRANSFORM, PAUL'S FORMAT
C               -9    FS    3-D SIMPLE FORMAT FOURIER (MICHAEL'S FORMAT)
C               -11   O2    2-D FOURIER TRANSFORM, MIXED RADIX ODD
C               -12   E2    2-D FOURIER TRANSFORM, MIXED RADIX EVEN
C               -21   O3    3-D FOURIER TRANSFORM, MIXED RADIX ODD
C               -22   E3    3-D FOURIER TRANSFORM, MIXED RADIX EVEN
C
C             IRTFLG       ERROR FLAG  (-1 ON ENTRY SUPRESSES PRINT-OUT)
C
C 
C       NOTE:   THE HEADER RECORD(S) OF THE FILE CONTAIN THE FOLLOWING 
C               BUFFER POSITIONS WHICH CAN BE ALTERED (AMONG OTHERS):
C               POSITION   1  NSLICE = NUMBER OF SLICES (PLANES) IN VOLUME
C                                (=1 FOR AN IMAGE)  ON VAX  LONG HEADER
C                                FORMAT FILES THE VALUE OF NSLICE STORED IN
C                                THE FILE IS NEGATIVE.
C                          2  NROW   = NUMBER OF ROWS PER SLICE
C                          3  IREC   = (UNUSED)
C                          4  NHISTREC = (UNUSED)
C                          5  FLAG INDICATING DATA TYPE (=IFORM)
C                          6  IMAMI = FLAG INDICATING IF THE IMAGE HAS 
C				      BEEN SEARCHED FOR MAX AND MIN. 
C				      IMAMI IS SET TO +1 IF SEARCHED.
C                          7  FMAXD = IMAGE MAXIMUM
C                          8  FMIND = IMAGE MINIMUM
C			   9  AVD   = IMAGE AVERAGE
C                         10  SIG   = STANDARD DEVIATION (SQ. ROOT OF VARIANCE)
C                         11  IHIST = UNUSED
C                         13  LABLN = NUMBER OF FLOATING POINT HEADER VARIABLES
C                         14  IANGLE= ANGLE FILL FLAG
C                         15  PHI   = TILT ANGLE
C                         16  THETA = TILT ANGLE
C                         17  PSI   = TILT ANGLE
C                         18  XOFF  = X TRANSLATION
C                         19  YOFF  = Y TRANSLATION
C                         20  ZOFF  = Z TRANSLATION
C
C--*********************************************************************
  
        SUBROUTINE SETLAB(LUN,NSAM,BUF,IBUF1,NBUF,VALUES,TYPE,IRTFLG)
  
        DIMENSION          BUF(*),VALUES(*)
        CHARACTER   *(*)   TYPE
        LOGICAL            PRNT

        COMMON /UNITS/ LUNC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT

        PRNT = .TRUE.
        IF (IRTFLG .EQ. -1) PRNT = .FALSE.
        IRTFLG = 1

C       READ THE HEADER INTO BUF
        CALL REDHED(LUN,BUF,17000,NSAM,0,IRTFLG)
                 
C       UPDATE THE HEADER BUFFER
        IF (TYPE(1:1) .NE. 'U') THEN
           LENC = LEN(TYPE)
           IF (LENC .EQ. 2 .AND. TYPE(1:2) .EQ. 'R3') THEN 
                BUF(5) = 3
           ELSEIF (LENC .EQ. 2 .AND. TYPE(1:2) .EQ. 'F3') THEN
                BUF(5) = -3
           ELSEIF (LENC .EQ. 2 .AND. TYPE(1:2) .EQ. 'Q2') THEN
                BUF(5) = -5
           ELSEIF (LENC .EQ. 2 .AND. TYPE(1:2) .EQ. 'Q3') THEN
                BUF(5) = -7
           ELSEIF (LENC .EQ. 2 .AND. TYPE(1:2) .EQ. 'O2') THEN
                BUF(5) = -11
           ELSEIF (LENC .EQ. 2 .AND. TYPE(1:2) .EQ. 'E2') THEN
                BUF(5) = -12
           ELSEIF (LENC .EQ. 2 .AND. TYPE(1:2) .EQ. 'O3') THEN
                BUF(5) = -21
           ELSEIF (LENC .EQ. 2 .AND. TYPE(1:2) .EQ. 'E3') THEN
                BUF(5) = -22
           ELSEIF (TYPE(1:1) .EQ. 'F') THEN
                BUF(5) = -1
           ELSEIF (TYPE(1:1) .EQ. 'P') THEN
                BUF(5) = 2
           ELSEIF (TYPE(1:1) .EQ. 'D') THEN
                BUF(5) = 0
           ELSE
                BUF(5) = 1
           ENDIF
           IBUF5 = BUF(5)
           IF (PRNT) WRITE(NOUT,9993) IBUF5,TYPE
9993       FORMAT(' NEW IFORM:',I5,' TYPE:',2A2)
        ENDIF

        J     = 0
        ISTOP = MIN(256,IBUF1+NBUF-1)
        DO I = IBUF1,ISTOP
           J = J + 1
           BUFT   = BUF(I)
           BUF(I) = VALUES(J)
           IF (PRNT) WRITE(NOUT,9999) I,BUFT,VALUES(J)
9999       FORMAT(' HEADER LOCATION: ',I3,' CHANGED FROM: ',
     &              G10.3,' TO: ',G10.3)
        ENDDO


C       WRITE THE HEADER INTO FILE
        CALL WRTHED(LUN,BUF,NSAM,IRTFLG)

        IRTFLG = 0

        RETURN
        END

