
C++********************************************************************
C
C SETPRMb.FOR                       DATE LAST MODIFIED: 4/22/96 BY AL
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C     SETPRMB:   SET LABEL PARAMETERS
C
C    THIS SUBROUTINE WILL SET LABEL PARAMETERS IDENTIFYING TYPE,
C    NORMALIZATION STATUS OF FILE, AND TO WRITE LABEL INTO FILE.
C
C    CALL SETPRMB(A,LUN,NSAM,UNUSED,FMAXD,FMIND,AVD,TYPE)
C      A              WORKING BUFFER
C      LUN            LOGICAL UNIT NUMBER OF FILE TO BE LABELED
C      NSAM           NUMBER OF SAMPLES
C      UNUSED
C      FMIND,FMAXD    IF AVAILABLE, THE MINIMUM AND MAXIMUM OF THE
C                     IMAGE STORED IN THE FILE
C      AVD            AVERAGE VALUE
C      TYPE           CHAR. VARIABLE CONTAINING 1 OR TWO CHARACTERS 
C			 SYMBOLIZING THE DATA TYPE:
C			TYPE SYMBOL	DATA TYPE	IFORM
C			  R		 2-D REAL	 +1
C			  R3		 3-D REAL	 +3
C			  P		 2-D POLAR	 +2
C			  D		 NON-IMAGE DATA   0
C			  F		 2-D FOURIER	 -1
C			  F3		 3-D FOURIER  	 -3
C			  U		 UNCHANGED	 UNCHANGED	
C
C
C    CODING:  THE FIRST RECORD OF THE FILE CONTAINS THE FOLLOWING INFO:
C             ELEMENT # 5  FLAG INDICATING DATA TYPE (=IFORM)
C                       6  IMAMI = FLAG INDICATING IF THE IMAGE HAS 
C				BEEN SEARCHED FOR MAX AND MIN. 
C				IF FMAXD=FMIND IN SETPRM ARGUMENTS THEN
C				IMAMI IS SET TO -1, OTHERWISE, TO +1.
C			   7  FMAXD = IMAGE MAXIMUM
C		           8  FMIND = IMAGE MINIMUM
C			   9  AVD   = IMAGE AVERAGE
C                         10  SIG  = STANDARD DEVIATION (SQ. ROOT OF VARIANCE)
C                         11  IHIST= FLAG INDICATING IF THE HISTOGRAM HAS
C                                   BEEN COMPUTED,AND STORED IN RECORDS
C                                   1+NROW+1 ETC.
C
C       SUBROUTINE CALLS:    REDLIN, WRTLIN
C
C       NOTE: THIS ROUTINE REPLACES SETPRM.  SETPRM SHOULD NO LONGER
C             BE USED!
C
C--*******************************************************************

      SUBROUTINE SETPRMB(A,LUN,NSAM,UNUSED,FMAXD,FMIND,AVD,TYPE)

C     ACTUAL DIMENSION MUST BE AT LEAST LABREC * NSAM 
      DIMENSION   A(1)

      INCLUDE 'CMBLOCK.INC'
 
      CHARACTER *2  TYPE

C     READ THE HEADER FROM FILE  
      CALL REDHED(LUN,A,17000,NSAM,0,IRTFLG)

C     UPDATE THE NEW HEADER VALUES

      A(7)  = FMAXD
      A(8)  = FMIND
      A(9)  = AVD
      A(10) = SIG
      A(11) = IHIST

C     SET FLAG SHOWING WHETHER FMAXD,FMIND HAVE BEEN COMPUTED OR NOT

      A(6) = 1.0
      IF (FMAXD .EQ. FMIND) THEN
C        SET IMAMI TO UNDETERMINED (ZERO)
         A(6)  = 0.0
      ENDIF

C     SET FLAG INDICATING DATA TYPE 

      IF (TYPE(1:1) .NE. 'U') THEN
         A(5) = 1
         IF(TYPE(2:2) .EQ. '3') A(5) = 3.0
         IF(TYPE(1:1) .EQ. 'F') THEN
	    A(5) = -1.0
            IF (TYPE(2:2) .EQ. '3') A(5) = -3.0
         ENDIF
         IF (TYPE(1:1) .EQ. 'P') A(5) = 2.0
         IF (TYPE(1:1) .EQ. 'D') A(5) = 0.0
      ENDIF

C     WRITE THE HEADER BACK IN THE FILE
      CALL WRTHED(LUN,A,NSAM,IRTFLG) 
      
      RETURN
      END
                           
