
C++*************************************************************************
C
C  SLICE.FOR                                    LONGFILENAMES JAN 89 al
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C      SLICE(MAXDIM,LUN1,LUN2,LUNO)
C
C      THIS SUBROUTINE MAKES A PERPENDICULAR SLICE THROUGH NUMT
C      FILES AT THE SPECIFIED ROW, NREC.
C 
C      PARAMETERS:
C        MAXDIM    MAXIMUM BUFFER SPACE
C        LUN1      LOGICAL UNIT NUMBER OF THE OLDER FILE
C        LUN2      LOGICAL UNIT NUMBER OF THE FILE MOST RECENTLY OPENED
C        LUNO      LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C      CALLED BY: UTIL3
C
C--*******************************************************************

	SUBROUTINE SLICE(MAXDIM,LUN1,LUN2,LUNO)

        PARAMETER     (NUMTOT = 256)

	COMMON BUF(1)
        COMMON/COMMUN/FILNAM,FILOUT,FILPAT
        COMMON/UNITS/LUNC,NIN,NOUT
	COMMON/MASTER/NSAM1,NROW1,IREC,NLABEL,IFORM,IMAMI,FMAX,FMIN,
     1	              AV,SIG,IHIST

	DIMENSION     LUNA(2),NREC1(NUMTOT)
        CHARACTER *81 FILNAM(NUMTOT),FILOUT,FILPAT
        CHARACTER *1  NULL,F

	DATA          F/'R'/
CNO_SLICE
        NULL = CHAR(0)

	LUNA(1) = LUN1
	LUNA(2) = LUN2
	NMOD    = 2
	AVG     = 0.0
	NS      = 1
	NUMT    = 0

        WRITE(NOUT,*) ' SLICING A STACK OF 2D FILES'

        CALL FILSEQP(FILPAT,NLET,NREC1,NUMTOT,NUMT, 
     $      'FILE PREFIX OR TEMPLATE (EG. PIC****)', IRTFLG)
     
        IF (IRTFLG .EQ. -1) RETURN

200	DO  IFIL =1,NUMT
           CALL FILGET(FILPAT,FILNAM(IFIL),NLET,NREC1(IFIL),IRTFLG)
           IF (IRTFLG .NE. 0) THEN
15           CALL ERRT(19,'SLICE',NE)
             RETURN
           ENDIF
	ENDDO

        CALL FILERD(FILOUT,NLETO,NULL,'OUTPUT',IRTFLG)
        IF (IRTFLG .NE. 0) THEN
           CALL ERRT(19,'SLICE',NE)
           RETURN
        ENDIF
	
	CALL RDPRMI(NREC,NDUM,NLOOP,ILOOP,'ROW NUMBER OF SLICE')
	CALL OPENF(FILNAM(1),LUNA(1),NSAM,NROW,'OLD',NF,NSL1)
	CALL NORM3(LUNA(1),NSAM,NROW,NSL1,FMAX,FMIN,AV1)
	AVREF = AV1

C       THE EXTRA NSAM COMES FROM LABEL INFORMATION IN OPEN.
	NOFFS1 = 1+NSAM
	NOFFS2 = NOFFS1+3*NSAM
	NSAM2 = NSAM*2
	NSAM3 = NSAM*3
	IF (NF .EQ. 1) THEN
          CALL ERRT(4,'SLICE',NE)
          RETURN
        ENDIF

12  	IF (NREC.EQ.1 .OR. NREC+1.GT.NROW .OR. NSAM*8.GT.MAXDIM) THEN
           CALL ERRT(7,'SLICE',NE)
           RETURN
        ENDIF

C       NA1 CORRESPONDS TO THE 1ST OF 3 LINES OF ONE FILE.
C       NB1 CORRESPONDS TO THE 1ST OF 3 LINES OF THE SECOND FILE.
C       NAOUT CORRESPONDS TO 1 LINE CONTAINING INTERPOLATION BETWEEN THE
C       TWO FILES DESCRIBED ABOVE.

30	NA1 = NOFFS1
	NB1 = NOFFS2
	NAOUT = NB1+3*NSAM
	CALL OPENF(FILOUT,LUNO,NSAM,2*NUMT-1,'UNKNOWN',NF,1)
	IRECSV = IREC
	IF (NF .EQ. 1) THEN
	   CALL ERRT(4,'SLICE',NE)
	   CLOSE(LUNA(1))
	   RETURN
        ENDIF

C       READ IN THE THREE LINES TO BE USED FOR INTERPOLATION.

    	CALL REDLIN(LUNA(1),BUF(NA1),NSAM,NREC-1)
	CALL REDLIN(LUNA(1),BUF(NA1+NSAM),NSAM,NREC)
	B    = BUF(NA1+NSAM)
	AMAX = B
	AMIN = B
	AVG  = 0.0
	DO  KL1 = 1,NSAM
          B   = BUF(NA1+NSAM+KL1-1)
          AVG = AVG+B
          IF (B.GT.AMAX) AMAX=B
          IF (B.LT.AMIN) AMIN=B
	ENDDO
	CALL WRTLIN(LUNO,BUF(NA1+NSAM),NSAM,1)
	CALL REDLIN(LUNA(1),BUF(NA1+2*NSAM),NSAM,NREC+1)

C       FOR EFFICIENCY, DO THE FOLLOWING CALCULATIONS OUTSIDE LOOP.

	CONS1 = 1./SQRT(2.)
	CONS2 = 2.+6./SQRT(2.)
	CONS3 = 2.+8./SQRT(2.)

C       LOOP TO FORM AN INTERPOLATED LINE BETWEEN EACH OF THE NUMT LINES.
	DO  J = 2,NUMT
	   CALL OPENF(FILNAM(J),LUNA(NMOD),NSAM,NROW,'OLD',NF,NSL2)
  	   IF (NF.EQ.1 .OR. NF.EQ.2) THEN
	      IF (NMOD .NE. 1) CLOSE(LUNA(1))
  	      CLOSE(LUNA(2))
	      CLOSE(LUNO)
	      CALL ERRT(4,'SLICE',NE)
	      RETURN
           ENDIF
           IF (IMAMI.EQ.0)
     &         CALL NORM3(LUNA(NMOD),NSAM,NROW,NSL2,FMAX,FMIN,AV)
	   AV2 = AV

C          READ IN THE THREE LINES TO BE USED FOR INTERPOLATION.

    	   CALL REDLIN(LUNA(NMOD),BUF(NOFFS2),NSAM,NREC-1)
	   CALL REDLIN(LUNA(NMOD),BUF(NOFFS2+NSAM),NSAM,NREC)
	   CALL REDLIN(LUNA(NMOD),BUF(NOFFS2+NSAM2),NSAM,NREC+1)

C          INTERPOLATE BETWEEN EACH OF NSAM POINTS.

C          NORMALIZE THE FILES
	   DIFF1 = AVREF-AV1
	   DIFF2 = AVREF-AV2
	   DIFF  = (2*DIFF1)+(2*DIFF2)
C	   WRITE(4,7777)AVREF,AV1,AV2,DIFF1,DIFF2
C7777	   FORMAT(1X,5F6.3)

C          NORMALIZE LINE STARTING AT NOFFS2+NSAM SO THAT WHEN WE COPY
C          IT INTO THE OUTPUT FILE, IT IS NORMALIZED ALREADY.

	   DO  K = 1,NSAM
              KN = NOFFS2+NSAM+K-1
              BUF(KN) = BUF(KN)+DIFF2
	   ENDDO

C          FOR FIRST POINT,K=1, INTERPOLATE USING ONLY 3 POINTS.
C          NORMALIZE THE VALUES OF NREC-1 AND NREC+1 ONLY.

	   BUF(NAOUT) = (BUF(NOFFS1+NSAM)+BUF(NOFFS2+NSAM)+
     &        CONS1*(BUF(NOFFS1)+BUF(NOFFS1+NSAM+1)+BUF(NOFFS1+
     &        NSAM2)+BUF(NOFFS2)+BUF(NOFFS2+NSAM+1)+
     &        BUF(NOFFS2+NSAM2)+DIFF))/CONS2
	   B   = BUF(NAOUT)
	   AVG = AVG+B
	   IF (B .GT. AMAX) AMAX=B
	   IF (B .LT. AMIN) AMIN=B

C          LOOP TO INTERPOLATE FOR ALL BUT THE ENDPOINTS.

	   DO  K = 2,NSAM-1
              BUF(NAOUT+K-1) = (BUF(NOFFS1+NSAM+K-1)+BUF(NOFFS2+NSAM+
     &        K-1)+CONS1*(BUF(NOFFS1+K-1)+BUF(NOFFS1+NSAM+K-2)+
     &        BUF(NOFFS1+NSAM+K)+BUF(NOFFS1+NSAM2+K-1)+
     &        BUF(NOFFS2+K-1)+BUF(NOFFS2+NSAM+K-2)+BUF(NOFFS2+
     &        NSAM+K)+BUF(NOFFS2+NSAM2+K-1)+DIFF))/CONS3
	      B   = BUF(NAOUT+K-1)
              AVG = AVG+B
              IF (B.GT.AMAX) AMAX=B
              IF (B.LT.AMIN) AMIN=B
	   ENDDO

C          INTERPOLATE THE ENDPOINT.

	   BUF(NAOUT+NSAM-1) = (BUF(NOFFS1+NSAM2-1)+BUF(NOFFS2+
     &        NSAM2-1)+CONS1*(BUF(NOFFS1+NSAM-1)+BUF(NOFFS1+
     &        NSAM2-2)+BUF(NOFFS1+NSAM3-1)+BUF(NOFFS2+NSAM-1)+
     &        BUF(NOFFS2+NSAM2-2)+BUF(NOFFS2+NSAM3-1)+DIFF))/CONS2
	   B   = BUF(NAOUT+NSAM-1)
	   AVG = AVG+B
	   IF (B .GT. AMAX) AMAX=B
	   IF (B .LT. AMIN) AMIN=B
	   CALL WRTLIN(LUNO,BUF(NAOUT),NSAM,J*2-2)
 	   CALL WRTLIN(LUNO,BUF(NOFFS2+NSAM),NSAM,J*2-1)
	   DO  KL2 = 1,NSAM
              B   = BUF(NOFFS2+NSAM+KL2-1)
              AVG = AVG+B
              IF (B.GT.AMAX) AMAX=B
              IF (B.LT.AMIN) AMIN=B
	   ENDDO

C          CHANGE POINTER, THEN CLOSE OLDEST FILE.

	   IF (NMOD.EQ.1) GO TO 90
	   NOFFS1 = NB1
	   NOFFS2 = NA1
	   AV1    = AV2
	   NMOD   = 1
	   GO TO 95

90	   NOFFS1 = NA1
	   NOFFS2 = NB1
	   AV1    = AV2
	   NMOD   = 2
95	   CLOSE(LUNA(NMOD))
	ENDDO

	FMIN = AMIN
	FMAX = AMAX
	AVE  = AVG/(FLOAT(NSAM)*FLOAT(NUMT*2-1))
 	IREC = IRECSV
	CALL SETPRM(LUNO,NSAM,NUMT*2-1,FMAX,FMIN,AVE,F)

	RETURN
	END

