
C******************************************************
C MATRIX SOLUTION SUBROUTINE SOLVE(A,NFU,NDM)
C
C THIS ROUTINE SOLVES A SET OF SIMULTANEOUS EQUATIONS OF THE
C FORM: AX=B; WHERE:
C A = MATRIX OF COEFFICIENTS
C X = VECTOR OF SOLUTIONS
C B = VECTOR OF KNOWNS
C THE ROUTINE IS USED AS FOLLOWS:
C	CALL SOLVE(A,NFU,NDM)
C WHERE A IS THE AUGMENTED MATRIX AND NFU IS THE NUMBER OF EQUATIONS.
C A MUST BE DIMENSIONED (NDM,NDM+1) IN TNE MAIN PROGRAM
C THE SOLUTION IS RETURNED IN (A(I,NFU+1),I=1,NFU).  THE MATRIX IS
C DESTROYED AND NO CHECK IS MADE TO SEE IF THE MATRIX IS SINGULAR.
C A SINGULAR MATRIX WILL PROBABLY RESULT IN A FORTRAN ERROR MESSAGE.
C THE METHOD OF GAUSSIAN ELIMINATION WITH MAXIMUM PIVOT IS USED.
C
C**************************************************************

	SUBROUTINE SOLVE(A,NFU,NDM)

	REAL A(NDM,1)
	COMMON/UNITS/LUN,NIN,NOUT
CNO_SAVE
	N=NFU
	N1=N+1
	DO 10 I=1,N

C         FIND THE LARGEST PIVOT

	  F=A(I,I)
C	  WRITE(NOUT,333)F
C333	  FORMAT( ' F:  ',F8.3)
	  AF=ABS(F)
	  JJ=I
	  DO 110 J=I,N
	    IF (ABS(A(J,I)).LE.AF) GOTO 110
	    F=A(J,I)
C	    WRITE(NOUT,333)F
	    AF=ABS(F)
	    JJ=J
110	    CONTINUE

C IF THE LARGEST PIVOT IS IN A DIFFERENT ROW,
C THEN SWITCH ALL ELEMENTS OF THESE ROWS

	  IF (JJ.EQ.I) GOTO 120
	  DO  J=1,N1
	    G=A(I,J)
	    A(I,J)=A(JJ,J)
	    A(JJ,J)=G
	  ENDDO

C         DIVIDE THE REST OF THE ROW BY THE PIVOT

120	  DO  J=I,N1
C           WRITE(NOUT,334)F
C334        FORMAT( ' F IN DENOMINATOR :  ',F8.3)
	    A(I,J)=A(I,J)/F
	  ENDDO

C         SUBTRACT THIS ROW (MULTIPLIED BY A(K,I)) FROM ALL REMAINING ROWS

	  IF (I.EQ.N) GOTO 10
	  I1=I+1
	  DO  K=I1,N
	    F=A(K,I)
	    DO  J=I,N1
	      A(K,J)=A(K,J)-A(I,J)*F
	    ENDDO
	  ENDDO
10	  CONTINUE

C       SUBTRACT UPWARDS

	NM1=N-1
	DO  KM=1,NM1
	  K=N1-KM
	  KM1=K-1
	  F=A(K,N1)
	  DO  I=1,KM1
	    A(I,N1)=A(I,N1)-F*A(I,K)
	  ENDDO
	ENDDO

	END
