C++************************************************************************
C
C $$ SPEAK.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR       *
C *  LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR    *
C *  USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF     *
C *  THE CENTER FOR LABORATORIES AND RESEARCH   			   *
C **************************************************************************
C
C   PURPOSE:    SEARCH FOR HIGHEST PEAKS
C
C       THIS SUBROUTINE SEARCHES FOR THE ML HIGHEST PEAKS IN THE (REAL)
C       IMAGE FILNAM AND PRINTS OUT POSITIONS AND VALUES OF THESE PEAKS.
C
C       CALL SPEAK(FILNAM,LUN,NSAM,NROW,MAXDIM,OPT,NDOC)
C         FILNAM       FILE NAME
C         LUN          LOGICAL UNIT NUMBER OF IMAGE
C         NSAM,NROW    DIMENSIONS OF IMAGE
C         MAXDIM      MAXIMUM BUFFER SPACE AVAILABLE
C        OPT  OUTPUT OPTION
C        OPT=' ' DEFAULT:NO DOCUMENT OUTPUT
C        OPT='X' (I.E.,REGISTER LIST FOLLOWING):NO DOCUMENT OUTPUT
C                 BUT OUTPUT OF POSITION & VALUE OF PEAK IN REGISTERS)
C        OPT='D'   DOCUMENT OUTPUT:NUMBER,POSITION, AND VALUE
C        OF PEAK ARE WRITTEN INTO A DOCUMENT FILE
C        NDOC  LOGICAL UNIT NUMBER FOR DOCUMENT FILE
C
C23456789012345678901234567890123456789012345678901234567890123456789012
C--************************************************************************

         SUBROUTINE SPEAK(FILNAM,LUN,NSAM,NROW,MAXDIM,OPT,NDOC)

         COMMON /UNITS/ LUNC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
         COMMON /PARAM/PARAM(103),NS1,NS2,NS3,NS4,NS5,NS6,NS7
         COMMON  BUF(1)

         COMMON    /BATCH/IBCNT,NLOOP,IABSLP,ILOOP
         INTEGER          IBCNT,NLOOP,IABSLP,ILOOP

         PARAMETER      (MLM=4000)
         CHARACTER *(*) FILNAM
         CHARACTER      OPT
         DIMENSION  ILIST(MLM),KLIST(MLM),KXLIST(MLM),IXLIST(MLM)
     &              ,ALIST(MLM),RLIST(MLM),DLIST(5)
         DIMENSION  NSI(7)
         EQUIVALENCE  (NSI(1),NS1),(NSI(2),NS2),(NSI(3),NS3),
     &     (NSI(4),NS4),(NSI(5),NS5),(NSI(6),NS6),(NSI(7),NS7)
CNO_SAVE
C        REGISTER POSITIONS 1= INTEGER X-SHIFT
C          2= INTEGER Y-SHIFT
C          3= ABSOLUTE PEAK HEIGHT
C
C          5= FLOATING PT. X-SHIFT
C          6= FLOATING PT. Y-SHIFT
C          7= VALUE OF EXTREMUM OF PARABOLOID


         CALL RDPRMI(ML,NOR,NLOOP,ILOOP,
     1      'ENTER # OF PEAKS,CENTER ORIGIN OVERRIDE(0/1)')
         IF (ML .LT. 1)   ML = 1
         IF (ML .GT. MLM) ML = MLM
         IF (MAXDIM .LT. 3*NSAM+200) THEN
            CALL ERRT(6,'SPEAK ',NE)
            RETURN
         ENDIF

         DO K = 1,ML
            ILIST(K) = 0
            KLIST(K) = 0
            RLIST(K) = 0.0
            ALIST(K) = -1.0E20
         ENDDO

         NTAB     = 1
         NXCTR    = NSAM/2+1
         NYCTR    = NROW/2+1
         IF  (NOR .NE. 0) THEN
            CALL RDPRMI(NXCTR,NYCTR,NLOOP,ILOOP,
     $            'ENTER ORIGIN COORDINATES')
            CALL RDPRMI(NTAB,NDUM,NLOOP,ILOOP,
     $             'ENTER PEAK NUMBER FOR RATIO')
            IF (NTAB .GT. ML) THEN
               CALL ERRT(25,'SPEAK',NE)
               RETURN
            ENDIF
         ENDIF

30       THRESH = -1.0E20
         NMAX   = 0
         NSAM1  = NSAM-1
         NROW1  = NROW -1
         I1     = 0
         I2     = 1
         I3     = 2
         CALL REDLIN(LUN,BUF,NSAM,1)
         CALL REDLIN(LUN,BUF(1+NSAM),NSAM,2)

         DO  I = 3,NROW
            I1  = MOD(I1,3)+1
            I2  = MOD(I2,3)+1
            I3  = MOD(I3,3)+1
            I1A = (I1-1)*NSAM
            I2A = (I2-1)*NSAM
            I3A = (I3-1)*NSAM
            CALL REDLIN(LUN,BUF(1+I3A),NSAM,I)
            DO 150  K = 2,NSAM1
               A = BUF(K+I2A)
               IF (A.LE.THRESH) GOTO 150
               IF (A.LE.BUF(K-1+I2A)) GOTO 150
               IF (A.LE.BUF(K-1+I1A)) GOTO 150
               IF (A.LE.BUF(K+I1A))   GOTO 150
               IF (A.LE.BUF(K+1+I1A)) GOTO 150
               IF (A.LE.BUF(K+1+I2A)) GOTO 150
               IF (A.LE.BUF(K+1+I3A)) GOTO 150
               IF (A.LE.BUF(K+I3A))   GOTO 150
               IF (A.LE.BUF(K-1+I3A)) GOTO 150
               NMAX = NMAX + 1
               DO 100 L = 1,ML
                  IF (A .LE. ALIST(L)) GOTO 100
                  IF (L .EQ. ML)GOTO 90
                  L1 = L+1
                  DO J = L1,ML
                     JO = ML-J+L1
                     JN = ML-J+L1-1
                     ALIST(JO) = ALIST(JN)
                     ILIST(JO) = ILIST(JN)
                     KLIST(JO) = KLIST(JN)
                  ENDDO
90                ALIST(L) = A
                  ILIST(L) = I-1
                  KLIST(L) = K
                  IF (NMAX .GT. ML) THRESH = ALIST(ML)
                  GOTO 150
100            CONTINUE
150         CONTINUE
	 ENDDO

         CALL PDATES(FILNAM,0)
         WRITE(NDAT,299)
         IF (NMAX .EQ. 0)  THEN
            WRITE(NDAT,*)' NO PEAK FOUND'
            DO I=1,7
               IF (NSI(I) .NE. 0) PARAM(NSI(I)) = 0.0
            ENDDO
            RETURN
         ENDIF
295      IF (NDAT .NE. NOUT) WRITE(NOUT,302)
         IF (NDAT .NE. NOUT) WRITE(NOUT,299)
299      FORMAT('    NO   NSAM  NROW    X     Y    VALUE   RATIO'/)
         MLIST = MIN0(ML,NMAX)
         DO L = 1,MLIST
            KXLIST(L) = KLIST(L)-NXCTR
            IXLIST(L) = ILIST(L)-NYCTR
            RLIST(L)  = ALIST(L)/ALIST(NTAB)
C           IF (L .EQ. 1 .AND. NDAT .NE. NOUT)
            IF (NDAT .NE. NOUT)
     1         WRITE(NOUT,301)L,KLIST(L),ILIST(L),KXLIST(L),IXLIST(L),
     2              ALIST(L),RLIST(L)
301         FORMAT(5I6,G16.7,2X,F8.5)

            IF (OPT .EQ. 'D') THEN
               DLIST(1) = L
               DLIST(2) = KXLIST(L)
               DLIST(3) = IXLIST(L)
               DLIST(4) = ALIST(L)
               DLIST(5) = RLIST(L)
               CALL SAVD(NDOC,DLIST,5,IRTFLG)
            ENDIF
            WRITE(NDAT,301) L,KLIST(L),ILIST(L),KXLIST(L),IXLIST(L),
     1         ALIST(L),RLIST(L)
         ENDDO

         CALL SAVDC
         CLOSE(NDOC)
302      FORMAT(' LARGEST PEAK:'/)
         IF (NS1.NE.0) PARAM(NS1) = KXLIST(1)
         IF (NS2.NE.0) PARAM(NS2) = IXLIST(1)
         IF (NS3.NE.0) PARAM(NS3) = ALIST(1)
         IF (NS4.NE.0) PARAM(NS4) = RLIST(1)

C 9/25/81 PARABOLIC FIT TO THE 3X3 NEIGHBORHOOD OF THE PEAK POINT
C PROGRAM SECTION SENT BY M.VAN HEEL, MODIFIED FOR SPIDER USE. JF

         KL=KLIST(1)
         DO I=1,3
            IROW=ILIST(1)+I-2
            IF(IROW.LT.1) IROW=IROW+NROW
            IF(IROW.GT.NROW) IROW = IROW-NROW
            CALL REDLIN(LUN,BUF,NSAM,IROW)
            I1 = (I-1)*3
            DO K=1,3
               ISAM=KL+K-2
               IF (ISAM.LT.1)ISAM=ISAM+NSAM
               IF (ISAM.GT.NSAM) ISAM=ISAM-NSAM
C              USE RLIST BUFFER WHICH IS NO LONGER NEEDED
               RLIST(I1+K)= BUF(ISAM)
            ENDDO
         ENDDO

         CALL PARABL(RLIST,XSH,YSH,PEAKV)
         IF (NOUT.NE.NDAT) WRITE(NOUT,351)XSH,YSH,PEAKV
         WRITE(NDAT,351)XSH,YSH,PEAKV
351      FORMAT('  + (',F5.2,', ',F5.2,')  PEAK VALUE = ',G15.7)

         IF (NS5 .NE. 0) PARAM(NS5) = XSH+KXLIST(1)
         IF (NS6 .NE. 0) PARAM(NS6) = YSH+IXLIST(1)
         IF (NS7 .NE. 0) PARAM(NS7) = PEAKV

         END
