
c++************************************************************************
C
C SPEAK3D.FOR                           FILENAMES LENGTHENED DEC 88 al
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981, DIVISION OF LABORATORIES AND                       *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C    SPEAK3D:    SEARCH FOR HIGHEST PEAKS IN 3-D DENSITY DISTRIBUTION
C
C       THIS SUBROUTINE SEARCHES FOR THE ML HIGHEST PEAKS IN THE (REAL)
C       VOLUME FILNAM AND PRINTS OUT POSITIONS AND VALUES OF THESE PEAKS.
C
C       REWRITTEN FROM SPEAK3.FOR                         M.RADERMACHER
C       ASSUMES VOLUME TO BE UNIT CELL IN PERIODIC 3D. FINDS ALSO PEAKS
C       AT THE EDGE OF THE VOLUME.  
C
C       SPEAK3D(FILNAM,LUN,NSAM,NROW,NSLICE,MAXDIM,OPT,NDOC)
C         FILNAM       LOGICAL*1 ARRAY, CONTAINING FILE NAME
C         LUN          LOGICAL UNIT NUMBER OF IMAGE
C         NSAM,NROW,NSLICE    DIMENSIONS OF VOLUME
C	  MAXDIM	     MAXIMUM BUFFER SPACE AVAILABLE
C
C       IN SPEAK3D NOT ALL OPTIONS ARE ACTIVE.
C
C	  OPT		OUTPUT OPTION
C	  OPT=' '	DEFAULT:NO DOCUMENT OUTPUT
C	  OPT='X' (I.E.,REGISTER LIST FOLLOWING):NO DOCUMENT OUTPUT
C	            BUT OUTPUT OF POSITION & VALUE OF PEAK IN REGISTERS)
C	  OPT='D'  	DOCUMENT OUTPUT:NUMBER,POSITION, AND VALUE
C			OF PEAKS ARE WRITTEN INTO A DOCUMENT FILE
C	  NDOC		LOGICAL UNIT NUMBER FOR DOCUMENT FILE
C
C--*******************************************************************

      SUBROUTINE SPEAK3D(FILNAM,LUN,NSAM,NROW,NSLICE,MAXDIM,OPT,NDOC)

      PARAMETER (NPEAK=500)

      INCLUDE 'CMBLOCK.INC'

      COMMON DUMMY(512),ILIST(NPEAK),KLIST(NPEAK),KXLIST(NPEAK),
     $ IXLIST(NPEAK),
     $ LLIST(NPEAK),LXLIST(NPEAK),
     $ ALIST(NPEAK),RLIST(NPEAK),DLIST(9),BUF(100000)
       REAL MAGX,MAGY,MAGZ

C	REGISTER POSITIONS 1= INTEGER X-SHIFT
C			   2= INTEGER Y-SHIFT
C                          3= INTEGER Z-SHIFT
C                          4= FLOATING X-SHIFT (CENTER OF GRAVITY)
C                          5= FLOATING Y-SHIFT (CENTER OF GRAVITY)
C                          6= FLOATING Z-SHIFT (CENTER OF GRAVITY)  
C			   7= ABSOLUTE PEAK HEIGHT
C                          8= RATIO 

      CHARACTER *(*) FILNAM
      CHARACTER      OPT,NULL,OPTB,OPTC
      LOGICAL        CGR,WR

      NULL=CHAR(0)

        WR=.FALSE.
        IF(NSAM.LE.8.AND.NROW.LE.8.AND.NSLICE.LE.8) WR=.TRUE.
	SIGN=+1.
	MLM=NPEAK
	CALL RDPRMC(OPTB,NC,.TRUE.,'MAXIMA(+) OR MINIMA(-)?',NULL,IRT)
	IF (OPTB .EQ. '-') SIGN=-1.
	CALL RDPRMI(ML,NOR,NLOOP,ILOOP,
     &      'ENTER # OF PEAKS,CENTER ORIGIN OVERRIDE(0/1)')
        CGR = .FALSE.

C       OPTC INITIALIZED TO ' ' SINCE IT IS NOT INITIALIZED AND
C       I DO NOT KNOW WHAT IT WAS MEANT TO BE JUNE 93 al
        OPTC = ' '
        IF (OPTC .EQ. 'Y') CGR=.TRUE.
        MAGX=1
        MAGY=1
        MAGZ=1
	IF (ML.GT.MLM) ML=MLM
        IF (MAXDIM.GE.3*NROW*NSAM+6*NPEAK) GOTO 10
        CALL ERRT(6,'SPEAK3',NE)
        RETURN

10	THRESH=-1.E8*SIGN
	DO  K = 1,ML
        ILIST(K) = 0
        KLIST(K) = 0
	LLIST(K)=0
        RLIST(K) = 0.
        ALIST(K) = THRESH*SIGN
	ENDDO
	NTAB=1
	NXCTR=NSAM/2+1
	NYCTR=NROW/2+1
	NZCTR=NSLICE/2+1
        NSAMIC=NSAM+2
        NROWIC=NROW+2
        NSLICEIC=NSLICE+2
	NSL1=1
	NSL2=NSLICEIC
	NRO1=1
	NRO2=NROWIC
	NSA1=1
	NSA2=NSAMIC
	IF(NOR.EQ.0)GO TO 30
	CALL RDPRMI(NXCTR,NYCTR,NLOOP,ILOOP,'X,Y ORIGIN COORDINATES')
	CALL RDPRMI(NZCTR,NDUM,NLOOP,ILOOP,'Z ORIGIN COORDINATE')
	CALL RDPRMI(NTAB,NDUM,NLOOP,ILOOP,
     $          'ENTER PEAK NUMBER FOR RATIO')
	IF(NTAB.LE.ML)GO TO 30
	CALL ERRT(25,'SPEAK3',NE)
	RETURN

30	CONTINUE
C       CALL RDPRMC(OPTB,NC,.TRUE.,'BOX SELECTION?(Y/N)',NULL,IRT)
	IF (OPTB .NE. 'Y') GOTO 35
C	CALL RDPRMI(NSL1,NSL2,NLOOP,ILOOP ,'LOWER, UPPER SLICE')
C	CALL RDPRMI(NRO1,NRO2, NLOOP,ILOOP,'LOWER, UPPER ROW')
C	CALL RDPRMI(NSA1,NSA2,NLOOP,ILOOP ,'LOWER, UPPER SAMPLE')
35    NMAX = 0
      NSAM1 = NSA2-1
      NROW1 = NRO2-1
      I1 = 0
      I2 = 1
      I3 = 2
	NRS=(NSL1-1)*NROW
	NTOTSL=NSAMIC*NROWIC
C READ IN FIRST 2 SLICES


C AS SLICE 1 READ LAST SLICE OF VOLUME.
        DO  I=1,NROW
        LINE=(NSLICE-1)*NROW+I
        NROFF=I*NSAMIC+2
        CALL REDLIN(LUN,BUF(NROFF),NSAM,LINE)
C PUT COPY OF LAST POINT IN LINE BEFORE FIRST
        BUF(NROFF-1)=BUF(NROFF+NSAM-1)
C PUT COPY OF FIRST POINT IN LINE BEHIND LAST
        BUF(NROFF+NSAM)=BUF(NROFF)
        IF(WR) WRITE(NOUT,93) LINE,(BUF(KK),KK=NROFF-1,NROFF+NSAM)
93      FORMAT(1X,I5,10F10.2)
	ENDDO

C COPY LAST LINE TO LINE 1 AND FIRST LINE TO NROWIC
        NRLAST=(NROW)*NSAMIC
        NRVLAST=(NROW+1)*NSAMIC
        NRFIRST=NSAMIC
        DO  I=1,NSAMIC
        BUF(I)=BUF(NRLAST+I)
        BUF(NRVLAST+I)=BUF(NRFIRST+I)
	ENDDO
        IF(WR) WRITE(NOUT,*)

C AS SLICE 2 READ FIRST SLICE O VOLUME.
        DO  I=1,NROW
        NROFF=I*NSAMIC+2+NTOTSL
        LINE=I
        CALL REDLIN(LUN,BUF(NROFF),NSAM,LINE)
C PUT COPY OF LAST POINT IN LINE BEFORE FIRST
        BUF(NROFF-1)=BUF(NROFF+NSAM-1)
C PUT COPY OF FIRST POINT IN LINE BEHIND LAST
        BUF(NROFF+NSAM)=BUF(NROFF)
        IF(WR) WRITE(NOUT,93) LINE,(BUF(KK),KK=NROFF-1,NROFF+NSAM)
	ENDDO
        IF(WR) WRITE(NOUT,*)
C COPY LAST LINE TO LINE NROWIC+1 AND FIRST LINE TO 2*NROWIC
        NRLAST=(NROW)*NSAMIC+NTOTSL
        NRVLAST=(NROW+1)*NSAMIC+NTOTSL
        NRFIRST=NSAMIC+NTOTSL
        DO  I=1,NSAMIC
        BUF(I+NTOTSL)=BUF(NRLAST+I)
        BUF(NRVLAST+I)=BUF(NRFIRST+I)
	ENDDO

	DO  ISLL=2,NSLICEIC-1
        ISL=ISLL
        IF(ISL.EQ.NSLICEIC-1) ISL=1
	I1=MOD(I1,3)+1
	I2=MOD(I2,3)+1
	I3=MOD(I3,3)+1
	I1Z=(I1-1)*NTOTSL
	I2Z=(I2-1)*NTOTSL
	I3Z=(I3-1)*NTOTSL
C THREE BUFFER PLANES ARE ROTATED
	NROWS=(ISL-1)*NROW+1
	NROWE=ISL*NROW
C READ IN MOST RECENT ('BOTTOM') PLANE
	DO  I=NROWS,NROWE
  	NROFF=(I-NROWS+1)*NSAMIC+2
	CALL REDLIN(LUN,BUF(NROFF+I3Z),NSAM,I)
        BUF(NROFF+I3Z-1)=BUF(NROFF+NSAM+I3Z-1)
        BUF(NROFF+I3Z+NSAM)=BUF(NROFF+I3Z)
        IF(WR) WRITE(NOUT,93) I,(BUF(KK),KK=NROFF-1+I3Z,NROFF+NSAM+I3Z)
	ENDDO

C COPY LAST LINE TO LINE NROWIC+1 AND FIRST LINE TO 2*NROWIC
        NRLAST=(NROW)*NSAMIC+I3Z
        NRVLAST=(NROW+1)*NSAMIC+I3Z
        NRFIRST=NSAMIC+I3Z
        DO  I=1,NSAMIC
        BUF(I+I3Z)=BUF(NRLAST+I)
        BUF(NRVLAST+I)=BUF(NRFIRST+I)
	ENDDO
        IF(WR) WRITE(NOUT,91)  ISLL,ISL,I1Z,I2Z,I3Z
91      FORMAT(1X,'SLICE:',I4,' REAL SLICE:',I4,' I1Z:',I5,' I2Z:',I5,
     $  ' I3Z:',I5)
        IF(WR) WRITE(NOUT,92) (BUF(KK),KK=1,3*NTOTSL)
        IF(WR) WRITE(NOUT,*)
92      FORMAT(1X,10F10.2)


C TOP SLICE IS IN BUFFER SECTION I1Z+1...I1Z+NTOTSL
C CENTER SLICE IS IN     SECTION I2Z+1...I2Z+NTOTSL
C BOTTOM SLICE IS IN     SECTION I3Z+1...I3Z+NTOTSL

C NOW GO THROUGH STACK OF THREE, ROW BY ROW

       IF(WR) WRITE(NOUT,*) 
     $ 'I1A,I2A,I3A,I1B,I2B,I3B,I1T,I2T,I3T,I1Z,I2Z,I3Z'
	DO  I=NRO1+2,NRO2
C COMPUTE BUFFER OFFSETS FOR ALL NINE ROWS
C CENTER ROWS:
	I1A=I2Z+(I-3)*NSAMIC
	I2A=I2Z+(I-2)*NSAMIC
	I3A=I2Z+(I-1)*NSAMIC
C TOP ROWS:
	I1T=I1Z+(I-3)*NSAMIC
	I2T=I1Z+(I-2)*NSAMIC
	I3T=I1Z+(I-1)*NSAMIC
C BOTTOM ROWS:
	I1B=I3Z+(I-3)*NSAMIC
	I2B=I3Z+(I-2)*NSAMIC
	I3B=I3Z+(I-1)*NSAMIC

       IF(WR) WRITE(NOUT,999) 
     $ I1A,I2A,I3A,I1B,I2B,I3B,I1T,I2T,I3T,I1Z,I2Z,I3Z
999   FORMAT(12(1X,I5))
      DO 150  K = NSA1+1,NSA2-1
      A = BUF(K+I2A)*SIGN
      IF(SIGN*A.LT.SIGN*THRESH) GOTO 150
      IF(A.LT.BUF(K-1+I2A)*SIGN) GOTO 150
	IF(A.LT.BUF(K-1+I2T)*SIGN)GOTO 150
	IF(A.LT.BUF(K-1+I2B)*SIGN)GOTO 150
      IF(A.LT.BUF(K-1+I1A)*SIGN) GOTO 150
	IF(A.LT.BUF(K-1+I1T)*SIGN)GOTO 150
	IF(A.LT.BUF(K-1+I1B)*SIGN)GOTO 150
      IF(A.LT.BUF(K+I1A)*SIGN) GOTO 150
	IF(A.LT.BUF(K+I1T)*SIGN)GOTO 150
	IF(A.LT.BUF(K+I1B)*SIGN)GOTO 150
      IF(A.LT.BUF(K+1+I1A)*SIGN) GOTO 150
	IF(A.LT.BUF(K+1+I1T)*SIGN)GOTO 150
	IF(A.LT.BUF(K+1+I1B)*SIGN)GOTO 150
      IF(A.LT.BUF(K+1+I2A)*SIGN) GOTO 150
	IF(A.LT.BUF(K+1+I2T)*SIGN)GOTO 150
	IF(A.LT.BUF(K+1+I2B)*SIGN)GOTO 150
      IF(A.LT.BUF(K+1+I3A)*SIGN) GOTO 150
	IF(A.LT.BUF(K+1+I3T)*SIGN)GOTO 150
	IF(A.LT.BUF(K+1+I3B)*SIGN)GOTO 150
      IF(A.LT.BUF(K+I3A)*SIGN) GOTO 150
	IF(A.LT.BUF(K+I3T)*SIGN)GOTO 150
	IF(A.LT.BUF(K+I3B)*SIGN)GOTO 150
      IF(A.LT.BUF(K-1+I3A)*SIGN) GOTO 150
	IF(A.LT.BUF(K-1+I3T)*SIGN)GOTO 150
	IF(A.LT.BUF(K-1+I3B)*SIGN)GOTO 150
	IF(A.LT.BUF(K+I2T)*SIGN)GOTO 150
	IF(A.LT.BUF(K+I2B)*SIGN)GOTO 150
      NMAX = NMAX + 1
      DO 100 L = 1,ML
      IF(A.LT.ALIST(L)) GOTO 100
      IF(L.EQ.ML)GOTO 90
      L1 = L+1
      DO  J = L1,ML
      JO = ML-J+L1
      JN = ML-J+L1-1
      ALIST(JO) = ALIST(JN)
      ILIST(JO) = ILIST(JN)
      KLIST(JO) = KLIST(JN)
      LLIST(JO)=LLIST(JN)
      ENDDO
90    ALIST(L) = A
      ILIST(L) = I-2
      KLIST(L) = K-1
      LLIST(L)=ISLL-1
      GOTO 150
100   CONTINUE
150   CONTINUE
      ENDDO
      ENDDO


C NO CENTER OF GRAVITY CALCULATION:
      IF(.NOT.CGR) GOTO 295

295   IF(NDAT.NE.NOUT) WRITE(NOUT,302)
      IF(NDAT.NE.NOUT) WRITE(NOUT,299)
299   FORMAT(' NO   NSAM  NROW   NSLICE    X    Y    Z',
     $'    VALUE    RATIO'/)
      MLIST = MIN0(ML,NMAX)
      DO 300 L = 1,MLIST
      KXLIST(L) = KLIST(L)-NXCTR
      IXLIST(L) = ILIST(L)-NYCTR
      LXLIST(L)=LLIST(L)-NZCTR
      IF(ALIST(NTAB).NE.0.) RLIST(L) = ALIST(L)/ALIST(NTAB)

      IF (L.EQ.1.AND.NDAT.NE.NOUT) THEN
      WRITE(NOUT,301)L,KLIST(L),ILIST(L),LLIST(L),KXLIST(L),IXLIST(L),
     2   LXLIST(L),SIGN*ALIST(L),RLIST(L)

      ENDIF
        IF (OPT .NE. 'D') GOTO 300
	DLIST(1) = L
	DLIST(2) = KXLIST(L)
	DLIST(3) = IXLIST(L)
	DLIST(4) = LXLIST(L)
	DLIST(5) = ALIST(L)*SIGN
	DLIST(6) = RLIST(L)
	CALL SAVD(NDOC,DLIST,6,IRTFLG)
300   WRITE(NDAT,301) L,KLIST(L),ILIST(L),LLIST(L),KXLIST(L),IXLIST(L),
     1 LXLIST(L),SIGN*ALIST(L),RLIST(L),FLOAT(KXLIST(L))*MAGX,
     2 FLOAT(IXLIST(L))*MAGY,FLOAT(LXLIST(L))*MAGZ
301   FORMAT(4I5,3X,2I5,I4,G11.3,F12.3,3F12.2)
302   FORMAT(' ** LARGEST PEAK:'/)
      IF(NSEL(1).EQ.0) GOTO 310

      PARAM(NSEL(1)) = KXLIST(1)
      PARAM(NSEL(2)) = IXLIST(1)
      PARAM(NSEL(3)) = LXLIST(1)
      IF(NSEL(4).EQ.0)GOTO 310
      PARAM(NSEL(4)) = ALIST(1)*SIGN

310   IF(NSEL(5).EQ.0)GOTO 311
      PARAM(NSEL(5)) = RLIST(1)

311   CONTINUE
380   IF (OPT .EQ. 'D') THEN
          CALL SAVDC
          CLOSE(NDOC)
      ENDIF

400   RETURN
      END
