C++************************************************************************
C
C $$ SPEAKC.FOR
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *  THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR       *
C *  LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR    *
C *  USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF     *
C *  THE CENTER FOR LABORATORIES AND RESEARCH   			   *
C **************************************************************************
C
C
C
C23456789012345678901234567890123456789012345678901234567890123456789012
C--************************************************************************
C
C $$ SPEAKC.FOR
C
         SUBROUTINE SPEAKC(FILNAM,LUN,NSAM,NROW,MAXDIM,OPTO,NDOC)
C $$ SPEAK:    SEARCH FOR HIGHEST PEAKS
C
C       THIS SUBROUTINE SEARCHES FOR THE ML HIGHEST PEAKS IN THE (REAL)
C       IMAGE FILNAM AND PRINTS OUT POSITIONS AND VALUES OF THESE PEAKS.
C
C       CALL SPEAK(FILNAM,LUN,NSAM,NROW,MAXDIM,OPTO,NDOC)
C         FILNAM       CHAR. ARRAY, CONTAINING FILE NAME
C         LUN          LOGICAL UNIT NUMBER OF IMAGE
C         NSAM,NROW    DIMENSIONS OF IMAGE
C         MAXDIM       MAXIMUM BUFFER SPACE AVAILABLE
C       OPTO        OUTPUT OPTION
C         OPTO=' '      DEFAULT:NO DOCUMENT OUTPUT
C         OPTO='X' (I.E.,REGISTER LIST FOLLOWING):NO DOCUMENT OUTPUT
C                  BUT OUTPUT OF POSITION & VALUE OF PEAK IN REGISTERS)
C       OPTO='D'   DOCUMENT OUTPUT:NUMBER,POSITION, AND VALUE
C       OF PEAK ARE WRITTEN INTO A DOCUMENT FILE
C       NDOC  LOGICAL UNIT NUMBER FOR DOCUMENT FILE
C
C REVISED 5/21/86 MR
C LAST MODIFIED:
C 8/18/87 NOTE: THE I2-VERSION OF THIS PROGRAM HAD THE FILE NAME SPEAKG.FOR
C                                                                     M.R.
C     REGISTER POSITIONS 1= INTEGER X-SHIFT
C          2= INTEGER Y-SHIFT
C          3= ABSOLUTE PEAK HEIGHT
C
C          5= FLOATING PT. X-SHIFT
C          6= FLOATING PT. Y-SHIFT
C          7= VALUE OF EXTREMUM OF PARABOLOID
C
         INCLUDE 'CMBLOCK.INC'

         COMMON DUMMY(80),ILIST,KLIST,KXLIST,IXLIST,NPTR,IPOS,
     $   ALIST,RLIST,DLIST,BUF(32000)

         PARAMETER (MLM=4000)

         REAL ILIST(MLM),KLIST(MLM),KXLIST(MLM),IXLIST(MLM)
         DIMENSION NPTR(64),IPOS(MLM)
         REAL ALIST(MLM),RLIST(MLM),DLIST(6)
         CHARACTER *(*) FILNAM
         CHARACTER      OPTO,POSE,NULL

         NULL=CHAR(0)
CNO_SAVE
         CALL RDPRMI(ML,NOR,NLOOP,ILOOP,
     1      'ENTER # OF PEAKS,CENTER ORIGIN OVERRIDE(0/1)')
         IF (ML.GT.MLM) THEN
         ML=MLM
         WRITE(NOUT,9) ML
9        FORMAT(' *** WARNING: NUMBER OF PEAKS REDUCED TO ',I3)
         ENDIF
         IF(MAXDIM.GE.3*NSAM+200) GOTO 10
         CALL ERRT(6,'SPEAKG',NE)
         RETURN

10       DO  K = 1,ML
         IPOS(K)=0.
         ILIST(K) = 0
         KLIST(K) = 0
         RLIST(K) = 0.
         ALIST(K) = 0.
	 ENDDO

         NTAB=1
         NXCTR=NSAM/2+1
         NYCTR=NROW/2+1
         IF(NOR.EQ.0)GO TO 30
         CALL RDPRMI(NXCTR,NYCTR,NLOOP,ILOOP,'NEW ORIGIN COORDINATES$')
      CALL RDPRMI(NTAB,NDUM,NLOOP,ILOOP,'ENTER PEAK NUMBER FOR RATIO$')
         IF(NTAB.LE.ML)GO TO 30
         CALL ERRT(25,'SPEAK',NE)
         RETURN
30       CALL RDPRM2(CG,CG2,NLOOP,ILOOP,
     $   'ELLIPSE AXES (X,Y) FOR CGR CALCULATION$')
         IF(CG2.EQ.0) CG2=CG
         CG2SQ=CG2*CG2
         CGSQ=CG*CG
         CALL RDPRMC(POSE,NC,.TRUE.,'POSITIVITY ENFORCED? (Y/N)$',
     &   NULL,IRTFLG)
         CALL RDPRM(FNEIGH,NLOOP,ILOOP,'NEIGHB.DIST.$')
         CALL RDPRMI(IEWX,IEWY,NLOOP,ILOOP,'EDGE EXCL. WIDTH X,Y$')
         IF(IEWY.EQ.0) IEWY=IEWX
C
C  MAKE SURE THAT THE EDGE EXCLUSION IS AT LEAST AS LARGE AS THE
C  CENTER OF GRAVITY AXES:
         IF(IEWX.LT.CG) IEWX=CG
         IF(IEWY.LT.CG2) IEWY=CG2
         WRITE(NOUT,2001) IEWX,IEWY
2001     FORMAT(' EDGE EXCLUSION WIDTH USED: X: ',I4,'  Y: ',I4)
C
         IESAM=NSAM-IEWX
         IEROW=NROW-IEWY
C
C CALCULATE HOW MANY RECORDS HAVE TO BE READ IN TO COVER THE COMPLETE
C FIELD USED IN THE CENTER OF GRAVITY CALCULATIONS:
         NREC=CG2+0.5
C MAKE DIAMETER AN ODD NUMBER (=TOTAL NUMBER OF RECORDS TO
C BE AVAILABLE AT ANY TIME)
         NREC=NREC*2+1
         IF(CG.EQ.0.0)NREC=3
         NMAX = 0

C INITIALIZE MREAD
         CALL MREAD(0,BUF,NSAM,NREC,NPTR)
C
         NREC2=NREC/2+1
         NRECH=NREC/2
C
         DO  I = NREC2,NROW-NREC2+1
         CALL MREAD(LUN,BUF,NSAM,NREC,NPTR)
C
C GET BEGINNING LOCATION OF THREE LINES
C
         I1A=(NPTR(NREC2-1)-1)*NSAM
         I2A=(NPTR(NREC2)-1)*NSAM
         I3A=(NPTR(NREC2+1)-1)*NSAM
         DO 150  K = NREC2,NSAM-NREC2+1
         A = BUF(K+I2A)
C CHECK IF A IS SMALLER THAN THE 8 SURROUNDING POINTS:
         IF(A.LE.BUF(K-1+I2A)) GOTO 150
         IF(A.LE.BUF(K-1+I1A)) GOTO 150
         IF(A.LE.BUF(K+I1A)) GOTO 150
         IF(A.LE.BUF(K+1+I1A)) GOTO 150
         IF(A.LE.BUF(K+1+I2A)) GOTO 150
         IF(A.LE.BUF(K+1+I3A)) GOTO 150
         IF(A.LE.BUF(K+I3A)) GOTO 150
         IF(A.LE.BUF(K-1+I3A)) GOTO 150
C MAKE SURE THAT PEAK LIES NOT IN THE RANGE OF THE EDGE (DEFINED BY IEW)
         IF(K.LE.IEWX.OR.K.GE.IESAM) GOTO 150
         IF(I.LE.IEWY.OR.I.GE.IEROW) GOTO 150
C NEW LOKAL PEAK FOUND:
         NMAX = NMAX + 1
C CHECK IF PEAK IS LARGER THAN THE ML PEAKS FOUND PREVIOUSLY
C AND PUT IT INTO THE CORRECT RANK POSITION
         DO 100 L = 1,ML
         IF(A.LE.ALIST(L)) GOTO 100
         IF(L.EQ.ML)GOTO 90
         L1 = L+1
         DO  J = L1,ML
         JO = ML-J+L1
         JN = ML-J+L1-1
         ALIST(JO) = ALIST(JN)
         ILIST(JO) = ILIST(JN)
         KLIST(JO) = KLIST(JN)
         IPOS(JO)=IPOS(JN)
	 ENDDO
C
C STORE PEAK HEIGHT AND COORDINATES
90       ALIST(L) = A
         IPOS(L)=0
         ILIST(L) = I
         KLIST(L) = K
         IF(L.EQ.1)IPAR=I
         IF(L.EQ.1)KPAR=K
         IF(NMAX.GT.ML) THRESH = ALIST(ML)
C
C CENTER OF GRAVITY SEARCH
         IF(CG.EQ.0.0)GOTO 150
         CNY=0
         CNX=0
         CGR=0
         PCORR=0.
         IF (POSE.NE.'Y') GOTO 991
C IF POSITIVITY OF AREA IS ENFORCED DETERMINE THE AREA'S MINIMUM:
         AMIN=1.E10
         DO  ICG=1,NREC
         IND1=(NPTR(ICG)-1)*NSAM
         YSQ=(ICG-NREC2)**2
         DO 198 KCG=K-NRECH,K+NRECH
         XSQ=(KCG-K)**2
         CRIT=XSQ/CGSQ+YSQ/CG2SQ
         IF(CRIT.GT.1) GOTO 198
         IND2=IND1+KCG
         XXX=BUF(IND2)
         IF(AMIN.GT.XXX) AMIN=XXX
198      CONTINUE
	 ENDDO
         PCORR=AMIN
C CALCULATE THE CENTER OF GRAVITY:
991      DO  ICG=1,NREC
         IND1=(NPTR(ICG)-1)*NSAM
         YSQ=(ICG-NREC2)**2
C
         DO 98 KCG=K-NRECH,K+NRECH
         XSQ=(KCG-K)**2
         CRIT=XSQ/CGSQ+YSQ/CG2SQ
C
         IF(CRIT.GT.1) GOTO 98
C POINT INSIDE ELLIPTICAL AREA FOR CGR DETERMINATION:
         IND2=IND1+KCG
C
C       WRITE(NOUT,1998) ICG,KCG,CRIT,BUF(IND2)
C     IF((KCG-K)**2+(ICG-NREC2)**2.GT.CG2)GOTO 98
1998     FORMAT(' ICG,KCG,CRIT,BUF',2I5,2F12.4)
C
         ADD=BUF(IND2)-PCORR
         IF(ADD.GE.0) GOTO 2000
         IPOS(L)=1
1999     FORMAT(' *** NEGATIVE VALUE ',G12.4,
     $   '  FOUND AT PIXEL ',2I5,'  CGR MEANINGLESS')
2000     CGR=CGR+ADD
         CNY=CNY+FLOAT(ICG-NREC2)*ADD
         CNX=CNX+FLOAT(KCG-K)*ADD
98       CONTINUE
	 ENDDO
         ILIST(L)=CNY/CGR+ILIST(L)
         KLIST(L)=CNX/CGR+KLIST(L)
         GOTO 150
100      CONTINUE
150      CONTINUE
	 ENDDO
         CALL PDATES(FILNAM,0)
         WRITE(NDAT,299)
         IF(NMAX.EQ.0)  THEN
         WRITE(NDAT,1234)
1234     FORMAT(1X,'*** NO PEAK FOUND')
         DO  I=1,7
         IF(NSEL(I).NE.0) PARAM(NSEL(I)) = 0.0
         ENDDO
         RETURN
         ENDIF
295      IF(NDAT.NE.NOUT) WRITE(NOUT,302)
         IF(NDAT.NE.NOUT) WRITE(NOUT,299)
         IF(NDAT.EQ.NOUT) WRITE(NDAT,299)
299      FORMAT('    NO    NSAM    NROW     X       Y      VALUE',
     1   '     RATIO',5X,' 1 IF NEGATIVE'/5X,58X,'VALUES IN CGR AREA')
         MLIST = MIN0(ML,NMAX)
C FOR EACH PEAK, MAKE SURE IT DOES NOT FALL WITHIN RADIUS FNEIGH
C OF A HIGHER-RANKING ONE
         FN2=FNEIGH*FNEIGH
C CANGED 2/8/88 MR
C     DO 2950 L=MLIST,3,-1
         DO  L=MLIST,2,-1
         FK=KLIST(L)
         FI=ILIST(L)
         DO 2910 LI=1,L-1
         IF((FK-KLIST(LI))**2+(FI-ILIST(LI))**2.GT.FN2)GOTO 2910
         ALIST(L)=0.0
2910     CONTINUE
	 ENDDO
         NPKCNT=0
C
C NOW SELECT PEAKS THAT COMPLY WITH ALL CONDITIONS AND ENTER IN DOC-FILE
         DO 300 L=1,MLIST
         IF(ALIST(L).NE.0.0)GOTO 309
         IF(L.NE.NTAB) GOTO 300
         NTAB=NTAB+1
         WRITE(NOUT,319)
319     FORMAT(' NUMBER OF REFERENCE PEAK INCREASED BY 1. NEW VAL.:'
     $   ,I5)
         GOTO 300
309      NPKCNT=NPKCNT+1
         KXLIST(L) = KLIST(L)-NXCTR
         IXLIST(L) = ILIST(L)-NYCTR
         RLIST(L) = ALIST(L)/ALIST(NTAB)
         IF(L.GT.1.OR.NDAT.EQ.NOUT) GOTO 303
         IF(IPOS(L).EQ.1) WRITE(NOUT,304)
304      FORMAT(' *** WARNING: PIXELS WITH NEGATIVE VALUES FOUND '/
     1   ,'DURING CALCULATION OF CENTER OF GRAVITY')
         WRITE(NOUT,301) NPKCNT,KLIST(L),ILIST(L),KXLIST(L),IXLIST(L),
     2   ALIST(L),RLIST(L),IPOS(L)
         IF(NDAT.EQ.NOUT) WRITE(NDAT,301) NPKCNT,KLIST(L),ILIST(L),
     1   KXLIST(L),IXLIST(L),ALIST(L),RLIST(L),IPOS(L)
303      IF (OPTO.NE.'D') GOTO 2990
         DLIST(1) = NPKCNT
         DLIST(2) = KXLIST(L)
         DLIST(3) = IXLIST(L)
         DLIST(4) = ALIST(L)
         DLIST(5) = RLIST(L)
         DLIST(6) = IPOS(L)
         CALL SAVD(NDOC,DLIST,6,IRTFLG)
2990     WRITE(NDAT,301) NPKCNT,KLIST(L),ILIST(L),
     1   KXLIST(L),IXLIST(L),ALIST(L),RLIST(L),IPOS(L)
300      CONTINUE
         CALL SAVDC
         CLOSE(NDOC)
301      FORMAT(I6,4F8.2,G12.3,F10.5,5X,I2)
302      FORMAT(' LARGEST PEAK:'/)
C
C STORE VALUES IN REGISTERS IF REQUIRED:
         IF(NSEL(1).NE.0) PARAM(NSEL(1)) = KXLIST(NTAB)
         IF(NSEL(2).NE.0) PARAM(NSEL(2)) = IXLIST(NTAB)
         IF(NSEL(3).NE.0) PARAM(NSEL(3)) = ALIST(NTAB)
         IF(NSEL(4).NE.0) PARAM(NSEL(4)) = RLIST(NTAB)
C
C 9/25/81 PARABOLIC FIT TO THE 3X3 NEIGHBORHOOD OF THE PEAK POINT
C PROGRAM SECTION SENT BY M.VAN HEEL, MODIFIED FOR SPIDER USE. JF
C
C APPLIED ONLY TO HIGHEST RANKING PEAK
C
         KL=KPAR
         DO  I=1,3
         IROW=IPAR+I-2
         IF(IROW.LT.1) IROW=IROW+NROW
         IF(IROW.GT.NROW) IROW = IROW-NROW
         CALL REDLIN(LUN,BUF,NSAM,IROW)
         I1=(I-1)*3
         DO  K=1,3
         ISAM=KL+K-2
         IF(ISAM.LT.1)ISAM=ISAM+NSAM
         IF(ISAM.GT.NSAM) ISAM=ISAM-NSAM
C USE RLIST BUFFER WHICH IS NO LONGER NEEDED
         RLIST(I1+K)= BUF(ISAM)
	 ENDDO
C
C DEBUGGING ONLY
C     WRITE(NOUT,341)(RLIST(I1+K),K=1,3)
C341  FORMAT(1X,3G12.5)
C
	 ENDDO
         CALL PARABL(RLIST,XSH,YSH,PEAKV)
         IF(NOUT.NE.NDAT)   WRITE(NOUT,351)XSH,YSH,PEAKV
         WRITE(NDAT,351)XSH,YSH,PEAKV
351      FORMAT('  + (',F5.2,', ',F5.2,')  PEAK VALUE = ',G12.5)
         IF(NSEL(5).NE.0) PARAM(NSEL(5))= XSH+KPAR-NXCTR
         IF(NSEL(6).NE.0) PARAM(NSEL(6))= YSH+IPAR-NYCTR
         IF(NSEL(7).NE.0) PARAM(NSEL(7))= PEAKV
         IDIFF=ML-NPKCNT
         WRITE(NOUT,401) ML,NPKCNT,IDIFF
401      FORMAT(' **',I5,' PEAKS SPECIFIED ',I5,' PEAKS PASSED ',
     $   I5,' PEAKS EXCLUDED (NEIGB.DIST. EDGE EXCL.)')
         END
