
C++*******************************************************************
C
C  SSDISP.FOR  -- CREATED JAN 85
C                 REWRITTEN SEPT 86 al
C **************************************************************************
C *  AUTHOR: R. BANERJEE                                                   *
C *  COPYRIGHT (C)1985 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **************************************************************************
C
C $$ SSDISP (DATA, NDATA,ITRM,CLEAR)
C
C    PURPOSE:  TO DRAW THE CONTOUR ON THE VT125 TERMINAL.
C              FINDS A SCALING  FACTOR AND PLOTS  ON A DIGITAL VT125 TERMINAL.
C
C    PARAMETERS:
C      DATA    ARRAY CONTAINING THE DATA POINTS
C      NDATA   NO. OF DATA POINTS TO BE DRAWN ON THE SCREEN
C      ITRM    TERMINAL TYPE
C      CLEAR   FLAG FOR CLEARING SCREEN FIRST
C
C    CALLS:      TMSTRT    TMEND   TMCLR   TMMOVE   TMDRAW  (IN VT125.FOR)
C
C    CALLED BY:  SSIND0    SSINP0  SSDRV1  SSSHOW   SSDELT  ?
C
C--********************************************************************

        SUBROUTINE SSDISP(DATA,NDATA,ITRM,CLEAR)

        COMMON /UNITS/LUNDOC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT

        PARAMETER     (NSIZE=2000)
        DIMENSION     DATA(3,NSIZE)
        LOGICAL       CLEAR

        IF (NDATA .LT. 2) RETURN

C..     FIND THE VALUE OF THE SCALING FACTOR 
        XMAXD = DATA(1,1)
        XMIND = XMAXD
        YMAXD = DATA(2,1)
        YMIND = YMAXD

        DO  I = 1, NDATA
           X = DATA(1,I)
           Y = DATA(2,I)
           IF (XMAXD .LT. X) XMAXD = X
           IF (XMIND .GT. X) XMIND = X
           IF (YMAXD .LT. Y) YMAXD = Y
           IF (YMIND .GT. Y) YMIND = Y
 	ENDDO

        XDIFF = XMAXD - XMIND
        YDIFF = YMAXD - YMIND
        SCAL = YDIFF
        IF (XDIFF .GT. SCAL) SCAL = XDIFF

C..      ASSIGN DEFAULT VIEWPORT SIZE 
C..      (THIS VALUE WILL CHANGE FOR DIFFERENT HARDWARES)

         IF (ITRM .EQ. 0) THEN
C..         FOR VT-125 TERMINAL
            IVPTSZ = 450
            YEXTRA = 0.0
         ELSE IF (ITRM .EQ. 1) THEN
C..         FOR VT-240 TERMINAL
            IVPTSZ = 350
            YEXTRA = 130.0
         ENDIF 

         IF (SCAL .EQ. 0.0) THEN
            WRITE(NOUT,111)
  111       FORMAT(1X,'UNKNOWN ERROR IN SCALE, RETURNING TO MENU')
            RETURN
         ELSE
            PLSCL = IVPTSZ / SCAL
         ENDIF

C..      ASSIGN UNIT NO. TO THE PLOTTING HARDWARE
         IPLUN = NOUT

C..      INITIALIZE THE PLOTTER
         CALL TMSTRT(IPLUN)

C..      CLEAR THE SCREEN
         IF (CLEAR) CALL TMCLR(IPLUN)

C..      FIND THE CONTOUR DATA AND PLOT THEM
C..      INITIALIZE DATA

               XPR = (DATA(1,1) - XMIND) * PLSCL
               YPR = (DATA(2,1) - YMIND) * PLSCL + YEXTRA
               CALL TMMOVE(IPLUN,XPR,YPR) 

               DO  I2 = 2, NDATA
                  X2 = PLSCL * (DATA(1,I2) - XMIND)
                  Y2 = PLSCL * (DATA(2,I2) - YMIND) + YEXTRA
                  CALL TMDRAW(IPLUN,X2,Y2)
 	       ENDDO

C..      TURN OFF THE PLOTTER
         CALL TMEND(IPLUN)

      RETURN
      END 

