
C++*************************************************************************
C
C TILTFD.FOR
C                                       
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C TILTFD(ARX,ARY,BRX,BRY,ATX,ATY,BTX,BTY,GAMR,GAMT,PHI,THA,GAMT2,IER)
C
C  COMPUTES THE TILT ANGLES CORRESPONDING TO A RECIPROCAL SPACE
C  PLANE INTERCEPTING THE SPIKES OF THE FOURIER TRANSFORM OF A 
C  PLANE REAL SPACE LATTICE.  TILTFD IS USED TO FIND THE IMAGE
C  PROJECTION DIRECTION.
C
C   INPUT:
C	ARX,ARY,BRX,BRY	: THE UNIT VECTORS OF THE AXIAL VIEW XTAL SPOTS
C	ATX,ATY,BTX,BTY : THE UNIT VECTORS OF THE XTAL PROJECTION WHO'S
C		PROJECTION ANGLES ARE TO BE FOUND
C   OUTPUT:
C	GAMR: ANGLE BETWEEN THE REFERENCE INPUT VECTORS, A TO B
C	GAMT: ANGLE BETWEEN THE TILT PROJECTION'S VECTORS
C	PHI: ASMUTHAL ANGLE BETWEEN THE AR VECTOR AND PROJECTION PLANE
C	THA: TILT ANGLE, BETWEEN AXIS AND NORMAL TO THE PROJECTION PLANE
C	GAMT2: COMPUTED ANGLE BETWEEN AT AND BT USING THA AND PHI
C		DETERMINED FROM VECTOR LENGTHS
C  ... NOTE THAT
C	THE COMPUTATION ASSUMES THE TWO SETS OF UNIT VECTORS ARE
C	TO THE SAME PAIR OF RECIPROCAL SPACE SPIKES. THAT IS,THE 
C	INDEXING INPUT TO (****) IS CONSISTENT. THE COMPUTATION OF
C	GAMT2 IS USE TO CHECK THIS CONSISTENCY.
C	  	TILTFD ALSO ASSUMES THE ANGLE BETWEEN THE REFERENCE
C	AXIAL PROJECTION VECTORS IS 90. DEGREES. THIS IS CHECKED WITH
C	GAMR.
C
C	IER = 0 NO ERROR
C	IER = 1 ERROR MESSAGE FROM ARCTRG CALL
C
C--*******************************************************************

	SUBROUTINE TILTFD(ARX,ARY,BRX,BRY,ATX,ATY,BTX,BTY,
     1                    GAMR,GAMT,PHI,THA,GAMT2,IER)

C       COMPUTE VECTOR LENGTHS AND THE ANGLES GAMT AND GAMR

	REAL LRA,LRB,LTA,LTB
	COMMON/UNITS/LUN,NIN,NOUT
	DATA PI/3.14159/
	IER = 0
	LRA=SQRT(ARX*ARX+ARY*ARY)
	LRB=SQRT(BRX*BRX+BRY*BRY)
	LTA=SQRT(ATX*ATX+ATY*ATY)
	LTB=SQRT(BTX*BTX+BTY*BTY)
	COSGT=(ATX*BTX+ATY*BTY)/(LTA*LTB)
	COSGR=(ARX*BRX+ARY*BRY)/(LRA*LRB)
C       CALL ARCTRG(COSGR,DUM,GAMR,IER) !REPLACED BY ACOS BY NAIK 10/2/86
        GAMR = ACOS(COSGR)
C	IF(IER.NE.0)RETURN
C       CALL ARCTRG(COSGT,DUM,GAMT,IER) !      NAIK 10/2/86
        GAMT = ACOS(COSGT)
C	IF(IER.NE.0)RETURN

	WRITE(5,1)COSGR,GAMR,180*GAMR/PI,COSGT,GAMT,180*GAMT/PI
1	FORMAT(6F10.2)

	RA=(ATX*ATX+ATY*ATY)/(ARX*ARX+ARY*ARY)
	RB=(BTX*BTX+BTY*BTY)/(BRX*BRX+BRY*BRY)

C       COMPUTE PHI AND THA

	SNSPHI=(RA-1.)/(RA+RB-2.)
	D=1.
	IF(GAMR.GT.GAMT)D=-1.
	S=D*SQRT(SNSPHI)
C       CALL ARCTRG(S,PHI,DUM,IER)       ! NAIK 10/2/86
C	IF(IER.NE.0)RETURN
        PHI = ASIN(S)

	CSSTHA=(RB-1.)/(RB*RB+RB*RA-2.*RB-RA+1.)
	S=SQRT(CSSTHA)
C       CALL ARCTRG(S,DUM,THA,IER)      ! NAIK 10/2/86
C	IF(IER.NE.0)RETURN
        THA = ACOS(S)

	DN=0.5/SQRT(RA*RB)
	S=DN*SIN(2*PHI)*(1.-1./CSSTHA)
	IF(ABS(S).LT.1.)GOTO 110
	WRITE(NOUT,109)RA,RB,PHI,CSSTHA,S
109	FORMAT(' ** TILTFD ERROR, COS(GAMAT2)>1.'/
     1         ,'RA,RB,PHI,CSSTHA,S'/'   ',5(F10.3))
C110	CALL ARCTRG(S,DUM,GAMT2,IER)      ! NAIK 10/2/86
  110   GAMT2 = ACOS(S)
	RETURN
	END
