
C *******************************************************************
C
C	PROGRAM TO CALCULATE THE TRANSFER FUNCTION IN THREE DIMENSIONS
C       IN FOURIER SPACE. THE TRANSFER FUNCTION IS THE SUM OF THE SINC-
C       FUNCTIONS ALONG THE Z-DIRECTIONS IN THE COORDINATE SYSTEMS OF 
C       EACH PROJECTION PG.
C       
C      LET R BE THE POINT IN THE THREE-DIMENSIONAL WEIGHTING
C      FUNCTION. PG THE PROJECTION GENERATING THE SINC-FUNCTION.  LET
C      RG BE THE SAME VECTOR REPRESENTED IN THE COORDINATE SYSTEM OF
C      PG. THE ARGUMENT OF THE SINC-FUNCTION IS THE Z-COORDINATE OF
C      POINT R IN THE COORDINATE SYSTEM OF PROJECTION PG.
C
C      THE TRANSFORM BETWEEN THE 2 SYSTEMS IS:
C
C       RG=[D]*R=[DGPSI]*[DGTHE]*[DGPHI]*R
C
C       THE MATRICES USED ARE (CONSISTEND WITH MRQBXS USED IN THE SIMPLE
C       BACKPROJECTION CALLED IN SPIDER BY BP XY):
C            
C           | COS(PSI(G)) SIN(PSI(G) 0| |COS(THETA(G)) 0 -SIN(THETA(G))| | COS(PHI(G)) SIN(PHI(G) 0|
C      [D]= |-SIN(PSI(G)) COS(PSI(G) 0|*|       0       1       0      |*|-SIN(PHI(G)) COS(PHI(G) 0|
C           |      0          0      1| |SIN(THETA(G)) 0  COS(THETA(G))| |      0          0      1|
C
C        NOTE THAT THE ZG-COORDINATE IS INDEPENDENT OF PSI.
C ************************************************************************

        SUBROUTINE TRANS3D
     $ (LUN1,NSAM,NROW,NSLICE,PHIG,THEG,GFLAG,
     $ SFLAG,mflag,ggray,RRAY,invflag,IHOW,crit,fx,fy,fz,w)

        PARAMETER (IBUF=1024)

        COMMON /DEBUG/DFLAG
c dimensions were 4000 for phig,theg,fx,fy,fz and 1024 for w
        dimension PHIG(*),THEG(*),W(*)

        double precision  FX(*),FY(*),FZ(*)

#ifdef SP_UNIX
        INCLUDE 'CMBLOCK.INC'
#else
        INCLUDE 'COMMON1:CMBLOCK.INC'
#endif
#ifdef SP_UNIX
        INCLUDE 'LABLOCK.INC'
#else
        INCLUDE 'COMMON1:LABLOCK.INC'
#endif

        DOUBLE PRECISION 
     $  darg,dxa,pi,WADD,ST,UNIT,RAY,THD,PHD,DIST,GSIG,fus,fug  

        LOGICAL DFLAG,DBFLAG,INVFLAG,GFLAG,SFLAG,mflag   

        DATA PI/3.1415927/

         DFLAG=.FALSE.
         DBFLAG=.FALSE.
C
C USE GAUSSIAN OR SINC-FUNCTION
C
C
C  CALCULATE TRANSFER OR WIGHTING FUNCTION?
C
        WRITE(NOUT,*) '** CALCULATING TRANSFER FUNCTION **'

        RAY=DBLE(RRAY)
        gray=dble(ggray)
        DO 6 J=1,IHOW
        THD=DBLE(THEG(J))
        PHD=DBLE(PHIG(J))
        ST=DSIN(THD)
        FX(J)=DCOS(PHD)*ST
        FY(J)=DSIN(PHD)*ST
        FZ(J)=DCOS(THD)
6       CONTINUE


C PRINT OUT THE GIVEN ANGLES:

        WRITE(NOUT,299) 
299     FORMAT(1H ,'ANGLES USED TO GENERATE THE WEIGHTING FUNCTION')
        WRITE(NOUT,298) (K,
     $  THEG(K)/PI*180.,
     $  PHIG(K)/PI*180.,FX(K),FY(K),FZ(K),K=1,IHOW) 
298     FORMAT(1H ,'PROJ.# ',I5,
     $  ' THETA:',F10.4,
     $  ' PHI:',F10.4,' D=',F8.4,';',F8.4,';',F8.4,';')
         NS2=NSAM/2
         NR2=NROW/2
         NL2=NSLICE/2
         UNIT=DBLE(NSAM-2)
         if(.not.gflag) GSIG=2*RAY*RAY
         if(gflag) gsig=2*gray*gray
C
C       NOW GO OVER THE VOLUME
C
	DO  1001  K=1,NSLICE

C THIS WRITE STATEMENT WRITES TO THE .LOG FILE OF A BATCH JOB.
C DONE TO BE ABLE TO CHECK PROGRESS. 
        WRITE(6,88) K
88      FORMAT(1X,'PROCESSING SLICE # ',I4)

	IZ=K-NL2-1
	DO  1001  J=1,NROW
	IY=J-NR2-1
	NR=(K-1)*NROW+J
        DO 11 II=1,NSAM
           W(II)=0.
11      CONTINUE
C LOOP OVER NSAM:
        DO 1002 II=1,NSAM,2
          IX=II/2
          WADD=0.
          DO 100 LL=1,IHOW
          Z=FX(LL)*FLOAT(IX)+FY(LL)*FLOAT(IY)+FZ(LL)*FLOAT(IZ)
          FUs=1
          fug=1
          DIST=Z/UNIT
          dxa=pi*DIST
          darg=ray*dxa
          if(dxa.ne.0.0.AND.(SFLAG.or.mflag)) fus=dsin(darg)/DARG
          IF(DIST.NE.0.0.AND.(GFLAG.or.mflag)) FUg=DEXP(-DIST*DIST*GSIG)
          wadd=wadd+fus*fug
100       CONTINUE
C COMMENTED OUT FOR DEBUG:
       IF(INVFLAG) THEN
          IF(WADD.GE.CRIT) THEN
          WADD=1./WADD
          ELSE
          WADD=CRIT
          ENDIF
       ENDIF
          W(II)=WADD
1002       CONTINUE 
         CALL WRTLIN(LUN1,W,NSAM,NR)
C        WRITE(NOUT,2002) NR
2002    FORMAT(' RECORD WRITTEN:',I5)
1001    CONTINUE
        CLOSE(LUN1)
        RETURN
        END
