
C++*************************************************************************
C
C UNSDAL.FOR                 ADAPTED FOR EXTENDED FILE NAMES NOV 88 al
C                            ALTERED FOR RUNTIME USE SEPT 96 al
C
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.    AUTHOR: J.FRANK      *
C *  COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, ALBANY, NY. *
C * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH            *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED        *
C * FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF       *
C * HEALTH RESEARCH INC.                                               *
C **********************************************************************
C
C  UNSDAL(DOCNAM,ICALL,NDOC,IKEY,PLIST,NLIST,DBUF,MAXKEY,MAXREG,
C          NKEY,IRTFLG)
C
C  PURPOSE:  RECOVERS ARRAY OF REGISTERS FROM DOCUMENT FILES.  IF IKEY
C            IS <0 RECOVERS THE LINE OF REGISTERS FROM THE COMMENT
C            LINE HAVING THAT COMMENT KEY ONLY. 
C
C  PARAMETERS:
C         DOCNAM        DOC. FILE NAME INCLUDING EXTENSION        (SENT)
C         ICALL         FLAG SHOWING DOC FILE IS IN DBUF NOW (SENT/RET.)                 
C                       ICALL = 1 (GET LINE FROM OPEN ARRAY)
C                       ICALL = 0 (MUST OPEN & READ FILE) 
C         NDOC          LOGICAL UNIT FOR DOC FILE                 (SENT)
C         IKEY          KEY FOR LINE OF REG. RETURNED IN PLIST    (SENT)
C         PLIST         ARRAY TO HOLD OUTPUT FROM KEY IKEY    (RETURNED)
C         NLIST         NUMBER OF REGISTERS RETRIEVED ON A        (SENT)
C                       LINE (1ST VALUE RETRIEVED ON EACH
C                       LINE IS KEY NO. NOT A REGISTER)          
C         DBUF          BUFFER TO RETRIEVE ARRAY FROM 
C                           DOC. FILE                         (RETURNED)
C         MAXKEY        NUMBER OF HIGHEST LINE THAT CAN BE        (SENT) 
C                       RETRIEVED IN ARRAY DBUF 
C         MAXREG        ONE PLUS MAX. NUMBER OF REGISTERS PER     (SENT)
C                       LINE THAT CAN BE RETRIEVED IN DBUF.             (SENT)
C         NKEY          NUMBER OF HIGHEST KEY FOUND IN FILE   (RETURNED)
C         IRTFLG        ERROR FLAG (O IS NORMAL RETURN)      (SENT/RET.)
C
C     
C  TYPICAL DOC FILE LINES:
C        KEY #REGS/LINE    VALUES ........
C  COL: 123456789 123456789 123456789 123456789 123456789 1234565789
C          1 4   20.070000   17.379999   17.379999   17.379999
C       999994   20.070000   17.379999   17.379999   17.379999
C         -1 5   21.070000   12.379999   12.379999   16.379999
C        ; COMMENT LINE (PREVIOUS LINE IS A CONTINUATION FOR KEY 99999)
C        : 1 4   20.070000   17.379999   17.379999   17.379999
C        ; COMMENT LINE (PREVIOUS LINE IS A COMMENT KEY LINE)
C
C  WHEN RETRIEVED THE LINE FOR KEY 1 IS PLACED IN LINE 1  OF DBUF AND
C  THE FIRST VALUE ON LINE ONE IS THE KEY NUMBER: 1 AND THE SECOND VALUE
C  OF LINE ONE IN DBUF IS: 20.07
C
C  THE LINE OF DBUF CORRESPONDING TO IKEY IS RETURNED IN PLIST
C  IF IKEY IS NOT FOUND AN ERROR MESSAGE IS PRINTED BUT DBUF
C  IS STILL RETURNED OK.
C
C  NOTE: AS OF 6/22/96, WE ARE ALLOWING KEYS UP TO 99999. I.E USING THE
C	  FIRST COLUMN.     ML
C--*******************************************************************

        SUBROUTINE UNSDAL(DOCNAM,ICALL,NDOC,IKEY,PLIST,NLIST,
     &                    DBUF,MAXKEY,MAXREG,NKEY,IRTFLG)

	INCLUDE 'CMBLOCK.INC' 
CNO_SAVE

	DIMENSION       PLIST(*)
        DIMENSION       DBUF(MAXREG,MAXKEY)

	CHARACTER *(*)  DOCNAM
        CHARACTER *80   RECLIN
	LOGICAL         NEWFILE,GOTCOM
        INTEGER WARNIT
       
C       NEGATIVE VALUE OF IRTFLG SUPPRESSES TERMINAL OUTPUT OF TITLE
	NPR  = IRTFLG

C       SET ERROR RETURN FLAG
	IRTFLG = 1

	IF (NLIST .LE. 0) THEN
	   WRITE(NOUT,*) '*** NUMBER OF REGISTERS NOT SPECIFIED'
	   RETURN

        ELSEIF (NLIST .GT. MAXREG) THEN
           WRITE(NOUT,101) (MAXREG-1)
101        FORMAT(' *** UNABLE TO UNSAVE DOCUMENT FILE'/
     $            ' *** CURRENTLY ONLY ',I3,' REGISTERS ALLOWED')
           RETURN
        ENDIF

        IF (IKEY .LT. 0) THEN
C          DESIRE A COMMENTED KEY
           CALL DOCCOM(DOCNAM,NDOC,IKEY,PLIST,NLIST,NPR)
           IRTFLG = NPR
           RETURN
        ENDIF

C       ICALL=0 MEANS THAT CURRENTLY, NO CORE IMAGE OF DOCUMENT EXISTS.
C       OPEN FILE, DUMP DOCUMENT INTO CORE, PICK REGISTERS SELECTED,
C       SWITCH ICALL TO 1, CLOSE DOCUMENT FILE, AND RETURN.
C       ICALL=1 MEANS THAT CORE IMAGE IS AVAILABLE, AND ACCESS IS
C       AUTHORIZED BY CALLING ROUTINE WHICH DID THE COMPARISON BETWEEN
C       OLD FILE AND NEW FILE NAME.

        IF (ICALL .GT. 0) GOTO 591


C       -----------------------------------------------------------

C       OPEN THE DOC FILE USING EXTENSION FROM DOCNAM
        CALL OPENDOC(DOCNAM,.FALSE.,NLET,NDOC,.FALSE.,
     &               .TRUE.,.TRUE.,.TRUE.,NEWFILE,NPR)
        IF (NPR .NE. 0) RETURN
 
C       CLEAR DBUF RETURNED ANSWER BUFFER
        DO  I=1, MAXREG
           DO J = 1,MAXKEY
              DBUF(I,J) = 0.0
           ENDDO
        ENDDO

        NKEY   = 0
        GOTCOM = .FALSE.

C -----------------------------------------------------------------

C       READ NEXT LINE FROM DOC FILE
510	READ (NDOC,518,END=595) RECLIN
518	FORMAT(A80)

        IF (RECLIN(2:2) .EQ. ';') THEN
C         COMMENT LINE FOUND
          IF (.NOT. GOTCOM) THEN
C           FIRST COMMENT ENCOUNTERED, SO WRITE OUT THIS HEADING
            IF (NPR .GE. 0)  WRITE(NOUT,*) RECLIN
            GOTCOM = .TRUE.
          ENDIF
            
        ELSEIF (RECLIN(3:3) .EQ. '-') THEN
C         CONTINUATION LINE OBTAINED
          READ(RECLIN,620,ERR=510) KEYDUM,NREGPL
620       FORMAT(I5,I2)
          NGO  = NEND + 1
          NEND = MIN(NREGPL+NGO-1,MAXREG)
          READ(RECLIN,621,ERR=510) (DBUF(J,KEYGOT),J=NGO,NEND)
621       FORMAT(7X,6F12.6)
       
        ELSE
C         REGULAR REGISTER LINE
          READ(RECLIN,620,ERR=510) KEYGOT,NREGPL

          IF (KEYGOT .LE. MAXKEY .AND. KEYGOT .GT. 0) THEN
C           KEY FITS IN DBUF
            DBUF(1,KEYGOT) = KEYGOT
            NGO  = 2
            NEND = MIN(NREGPL+1,MAXREG)
            READ(RECLIN,621,ERR=510) (DBUF(J,KEYGOT),J=NGO,NEND)
            IF (NKEY .LT. KEYGOT) NKEY = KEYGOT

          ELSE
C           KEY THAT WILL NOT FIT IN DBUF SENDS ERROR MSG.
            IF(WARNIT.EQ.1) THEN 
            WRITE(NOUT,9901) MAXKEY
9901        FORMAT(' ALL KEYS GREATER THAN ',I7,' WILL BE IGNORED')
            WRITE(NOUT,9900) KEYGOT
9900        FORMAT(' *** ILLEGAL KEY: ',I7,' DISCARDED')
            WARNIT = 0
            END IF
          ENDIF
        ENDIF

C       READ NEXT LINE OF DOC FILE
	GOTO 510

C ---------------------------------------------------------------

C       END OF DOCUMENT FILE FOUND. CLOSE FILE AND SWITCH ICALL TO 1.
595  	CLOSE(NDOC)
	ICALL = 1

C ---------------------------------------------------------------

591     IF (IKEY .GT. MAXKEY .OR. INT(DBUF(1,IKEY) + 0.5) .EQ. 0) THEN
	  WRITE(NOUT,8889) IKEY
8889	  FORMAT(' *** KEY:',I6,'  NOT FOUND')
          CLOSE(NDOC)
	  RETURN
	ENDIF

	DO  K = 1,NLIST
           PLIST(K) = DBUF(K+1,IKEY)
        ENDDO

        IRTFLG = 0
  	CLOSE(NDOC)

	RETURN
	END


