#ifdef SP_F90

       SUBROUTINE vaxtounix

       COMMON /UNITS/LUNC,NIN,NOUT

       WRITE(NOUT,*) 'DUMMY CALL: vaxtounix'
       RETURN
       END
#else
 
 
#ifdef SP_VMS 
 
C   THIS ROUTINE SPECIFIC TO UNIX SPIDER AND IS NOT 
C       USED AT VAX SITES
 
       SUBROUTINE vaxtounix
 
       COMMON /UNITS/LUNC,NIN,NOUT
 
       WRITE(NOUT,*) 'DUMMY CALL: vaxtounix'
       RETURN
       END
 
#else
#ifdef SP_SUN4 
 
C   THIS ROUTINE NOT AVAILABLE ON SUN
 
       SUBROUTINE vaxtounix
 
       COMMON /UNITS/LUNC,NIN,NOUT
 
       WRITE(NOUT,*) 'DUMMY CALL: vaxtounix'
       RETURN
       END
#else  

C++************************************************************************
C
C  VAXTOUNIX.FOR  -- ADAPTED FROM GETVAX
C                    CONVERTED TO RUN ON UNIX, JULY 93 al
C **************************************************************************
C *  COPYRIGHT (C)1989, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH 			   *
C **************************************************************************
C
C    VAXTOUNIX(FILOLD,FILNEW,LUNO,LUNN,IRTFLG)
C
C    PURPOSE:       TO CONVERT A VAX SPIDER IMAGE FILE TO UNIX FORMAT
C                   RUNNING ON A UNIX MACHINE
C
C    PARAMETERS:
C        FILOLD     VAX FORMAT FILE NAME 
C        LUNO       LOGICAL UNIT NUMBER TO BE ASSIGNED TO FILOLD.
C        FILNEW     UNIX  FORMAT FILE NAME 
C        LUNN       LOGICAL UNIT NUMBER TO BE ASSIGNED TO FILNEW
C        IRTFLG     ERROR RETURN FLAG. (0 IS NORMAL)
C
C    VARIABLES:   IFORM  = FILE TYPE SPECIFIER. 
C                        = +3    3-D IMAGE
C	                 = +1    2-D IMAGE
C                        = -1    2-D FOURIER TRANSFORM
C                        = -3    3-D FOURIER TRANSFORM
C                        = +8    8 BIT BLACK AND WHITE IMAGE
C                        = 11    8 BIT COLOR IMAGE
C
C--********************************************************************

	SUBROUTINE VAXTOUNIX(FILOLD,FILNEW,LUNO,LUNN,IRTFLG)

CNO_SAVE
        INCLUDE 'CMBLOCK.INC'

        COMMON /LUNARA/ LUNARA(100),LASLAB
        COMMON /IOERR/  IERR

	COMMON BUFO(17000)
               
        CHARACTER *(*) FILOLD,FILNEW
        CHARACTER *81  FILO,FILN

        CHARACTER * 1  NULL
	LOGICAL        EX

	DATA LENLAB/256/

        NULL = CHAR(0)

10      IF (FILOLD .EQ. NULL) THEN
C           FILOLD NOT SET BY CALLER
            CALL FILERD(FILOLD,NLET,NULL,'VAX SPIDER INPUT',IRTFLG)
            IF (IRTFLG .NE. 0) RETURN

            CALL FILCAN(FILO,NLET,NULL,NULL,FILOLD,DATEXC,IRTFLG)
            IF (IRTFLG .NE. 0) RETURN
        ELSE
C           FILE NAME SENT BY CALLER
            CALL FILCAN(FILO,NLET,NULL,NULL,FILOLD,DATEXC,IRTFLG)
            IF (IRTFLG .NE. 0) RETURN
        ENDIF

C       SET FLAG FOR ERRONEOUS RETURN
        IRTFLG = 1
        IERR   = 0
      
	INQUIRE(FILE=FILO ,IOSTAT=IOERR,EXIST=EX)

	IF (IOERR .OR. .NOT. EX) THEN
C          ERROR -- FILE DOES NOT EXIST, BUT IS BEING OPENED WITH 'OLD'
           CALL ERRT(18,'GETVAXU',NE)
           RETURN
        ENDIF

C       RETRIEVE ALL LABEL INFO (LABEL CONTAINS >=256 * 4 BYTES)
        LENBYT = 1024
#ifdef SP_VMS 
        LENRECT = LENBYT / 4
#else
        LENRECT = LENBYT
#endif

        OPEN(LUNO,FILE=FILO ,ACCESS='DIRECT',IOSTAT=IFAIL,
     &       STATUS='OLD',RECORDTYPE='FIXED',FORM='UNFORMATTED',
     &       RECL=LENRECT)

        IF (IFAIL) THEN
           CALL ERRT(101,'*** ERROR OPENING INPUT FILE (RECL=1024)',NE)
           RETURN
        ENDIF

        LUNARA(LUNO) = 1

C       READ FIRST 1024 BYTES  OF VAX INPUT FILE
3       CALL REDLIN8(LUNO,BUFO,LENBYT,0,IRTFLG)
        IF (IRTFLG .NE. 0) THEN
           CALL ERRT(101,'*** CAN NOT READ VAX HEADER',NE)
           RETURN
        ENDIF

C       CONVERT TO UNIX FLOATING POINT NUMBERS           
        DO I=1,211
           CALL VAX32U(BUFO(I))
        ENDDO

        NSLICE = BUFO(1)
        IF (NSLICE .EQ. 0) THEN
C          FIX UP OLD LABELS NSLICE
           BUFO(1) = -1
           NSLICE  = 1

        ELSEIF (NSLICE .GT. 0) THEN
C           OLD SPIDER SHORT LABEL FILE
            WRITE(NOUT,*) '*** SHORT LABEL CONVERSION NOT IMPLEMENTED'
            CALL ERRT(2,'VAXTOUNIX',NE)
            RETURN
        ELSE
            NSLICE = -NSLICE
        ENDIF

C       PIXELS IN EACH RECORD IS IN BUFO(12) = NSAM
        NROW     = BUFO(2)
        NHISTREC = BUFO(4) + 0.5
        IFORM    = BUFO(5)
        IF (BUFO(5) .LT. 0.0) IFORM = BUFO(5) - 0.5
        IHIST    = BUFO(11) + 0.5

        NSAM     = BUFO(12)

       IF (NSAM .LE. 0) THEN
           WRITE(NOUT,*) '*** NSAM NOT PRESENT IN HEADER LOCATION 12'
           CALL ERRT(2,'VAXTOUNIX',NE)
           RETURN
        ENDIF


C       FIND NUMBER OF FLOATING POINT VALUES IN EACH RECORD
        IF (IFORM .EQ. 8 .OR. IFORM .EQ. 11) THEN
C          8 BIT SPIDER FILES
           CALL ERRT(101,
     &          'VAXTOUNIX CAN NOT CONVERT 8 BIT VAX SPIDER FILES',NE)
           RETURN

        ELSEIF(IFORM .NE. 1 .AND. IFORM .NE. 3) THEN
          WRITE(NOUT,*)
     &         'WARNING: VAXTOUNIX NOT TESTED FOR THIS FORMAT:',IFORM
        ENDIF

C       FIND NUMBER OF RECORDS IN THE HEADER LABEL
        LABRECS  = 256 / NSAM
        IF (MOD(256,NSAM) .NE. 0) LABRECS = LABRECS + 1
        BUFO(13) = LABRECS

        LABBYT   = LABRECS * NSAM * 4
        BUFO(22) = LABBYT
        LENBYT   = NSAM * 4

C       CLOSE THE VAX FILE AND REOPEN WITH CORRECT RECORD LENGTH
        CLOSE(LUNO)
#ifdef SP_VMS 
        LENRECT = LENBYT / 4
#else
        LENRECT = LENBYT
#endif
        OPEN(LUNO,FILE=FILO ,ACCESS='DIRECT',IOSTAT=IFAIL,
     &       STATUS='OLD',RECORDTYPE='FIXED',FORM='UNFORMATTED',
     &       RECL=LENRECT)
        
        IF (IFAIL) THEN
           CALL ERRT(101,'CAN NOT REOPEN INPUT FILE',NE)
           RETURN
        ENDIF


        IF (FILNEW .EQ. NULL) THEN
C          FILNEW NOT SENT BY CALLER
           CALL FILERD(FILNEW,NLET,NULL,'UNIX SPIDER OUTPUT',IRTFLG)
           IF (IRTFLG .EQ. -1) THEN
              CLOSE(LUNO)
              GOTO 10
           ENDIF
           IF (IRTFLG .NE. 0) RETURN

           CALL FILCAN(FILN ,NLET,NULL,NULL,FILNEW,DATEXC,IRTFLG)
           IF (IRTFLG .NE. 0) RETURN

        ELSE
           CALL FILCAN(FILN ,NLET,NULL,NULL,FILNEW,DATEXC,IRTFLG)
           IF (IRTFLG .NE. 0) RETURN

        ENDIF

C       OPEN NEW FILE FOR UNIX VERSION
#ifdef SP_VMS 
        LENRECT = LENBYT / 4
#else
        LENRECT = LENBYT
#endif
        OPEN(LUNN,FILE=FILN,ACCESS='DIRECT',RECL=LENRECT,IOSTAT=IFAIL,
     &       STATUS='UNKNOWN',RECORDTYPE='FIXED',FORM='UNFORMATTED')

        IF (IFAIL) THEN
           CALL ERRT(101,'CAN NOT OPEN UNIX OUTPUT FILE',NE)
           RETURN
        ENDIF

C       PLACE HEADER IN NEW UNIX FILE
        LUNARA(LUNN) = 1

        J = 1
        DO I = 0,LABRECS-1
          CALL WRTLIN(LUNN,BUFO(J),NSAM,I)
          J = J + NSAM
        ENDDO

C       RESET LUNARA FOR IMAGE PORTION OF FILE IO
        LUNARA(LUNO) = LABRECS
        LUNARA(LUNN) = LABRECS

C       FIND NUMBER OF IMAGE RECORDS EXPECTED IN FILE (IEND)
        CALL TOTREC(NSAM,NROW,NSLICE,IEND)

        DO  I = 1,IEND

C           READ EACH RECORD OF VAX FILE
            CALL REDLIN(LUNO,BUFO,NSAM,I)

            IF (IERR .NE. 0) THEN
              CALL ERRT(12,'VAXTOUNIX',NE)
              RETURN
            ENDIF

C           CONVERT FLOATING POINT NUMBERS TO UNIX FORMAT
            DO J=1,NSAM
              CALL VAX32U(BUFO(J))
            ENDDO

C           WRITE RECORD TO UNIX FILE
            CALL WRTLIN(LUNN,BUFO,NSAM,I)

        ENDDO

        WRITE(NOUT,*) ' '
        NCHAR1 = lnblnk(FILN)
        WRITE(NOUT,*) FILN(1:NCHAR1)
        
        IF (NSLICE .GT. 1) THEN
           WRITE(NOUT,2099) NSAM,NROW,NSLICE,LABBYT
2099       FORMAT(' UNIX IMAGE SIZE: (',I4,',',I4,',',I4,')',
     &            '    LABEL BYTES:',I6,/)
        ELSE
           WRITE(NOUT,2098) NSAM,NROW,LABBYT
2098       FORMAT(' UNIX IMAGE SIZE: (',I4,',',I4,')',
     &            '    LABEL BYTES:',I6,/)
        ENDIF

C       SET FLAG FOR NORMAL RETURN	
        IRTFLG = 0

        RETURN
        END
 
#endif
#endif
 
#endif
