
C++*******************************************************************
C   WRTLIN.FOR                       IOSTAT ADDED DEC. 97 al 
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.    AUTHOR: J.FRANK      *
C *  COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, ALBANY, NY. *
C * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH            *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED        *
C * FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF       *
C * HEALTH RESEARCH INC.                                               *
C **********************************************************************
C
C     WRTLIN(LUN,BUF,NB,NREC)
C
C     PURPOSE:  WRITE A LINE OF FLOATING POINT NUMBERS INTO FILE
C
C     PARAMETERS:
C        LUN    LOGICAL UNIT NUMBER OF FILE BEING WRITTEN
C        BUF    BUFFER WHERE RECORD IS READ IN
C        NB     NUMBER OF VALUES IN RECORD TO BE READ
C        NREC   RECORD TO BE READ
C
C        IERR   ERROR CODE 1 IS RETURNED IN CASE OF ERROR
C               IERR IS DEFINED IN COMMON /IOERR/IERR
C 
C--*******************************************************************

      SUBROUTINE WRTLIN(LUN,BUF,NB,NREC)

      REAL            BUF(NB)
      COMMON /IOERR/  IERR
      COMMON /UNITS/  LUNC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
      COMMON /LUNARA/ LUNARA(100),LUNSTK(100)
      LOGICAL         USEINLN
      COMMON /LUNARB/ LOCLUN(100),NSAMLUN(100),NWORDLUN(100),USEINLN

CNO_SAVE

C     RUN-TIME ALLOCATION SPECIFIC CODE LINE FOLLOWS
#ifdef SP_NT
      INTEGER         LOCLUN
#else
      INTEGER * 8     LOCLUN
#endif

      IF (LOCLUN(LUN) .GT. 0) THEN
C        USER HAS ASKED FOR INLINED BUFFER
         CALL INLN_WRTLIN(LUN,BUF,NB,NREC)
         RETURN
      ENDIF

C     ADD LUNARA(LUN) (FOR LABEL REC) AND LUNSTK (FOR STACK OFFSET)
C     TO NREC TO GET THE CORRECT RECORD NUMBER
      I = NREC + LUNARA(LUN) + LUNSTK(LUN)
      WRITE(LUN,REC=I,IOSTAT=IERR) BUF
     
      IF (IERR .NE. 0) THEN
         WRITE(NOUT,*) '*** ERROR (',IERR,') WRITING RECORD: ',I, 
     &                 ' LENGTH: ',NB,' ON UNIT: ',LUN
      ENDIF

      RETURN
      END


C++********************************************************************
C   INLN_WRTLIN.F                                
C **********************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.    AUTHOR: J.FRANK      *
C *  COPYRIGHT (C)1985, 1996 HEALTH RESEARCH INCORPORATED, ALBANY, NY. *
C * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO HEALTH            *
C * RESEARCH INC. AND ARE NOT TO BE DISCLOSED TO OTHERS OR USED        *
C * FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF       *
C * HEALTH RESEARCH INC.                                               *
C **********************************************************************
C
C     INLN_WRTLIN(LUN,BUF,NB,NREC)
C
C     PURPOSE:    READ A LINE OF FLOATING POINT NUMBERS FROM BUFFER
C
C     PARAMETERS:
C        LUN    LOGICAL UNIT NUMBER OF FILE BEING READ
C        BUF    BUFFER WHERE RECORD IS READ IN
C        NB     NUMBER OF VALUES IN RECORD TO BE READ
C        NREC   RECORD TO BE READ
C 
C        IERR   ERROR CODE 1 IS RETURNED IN CASE OF ERROR
C               IERR IS DEFINED IN COMMON /IOERR/IERR
C 
C       MANY COMMANDS IN SPIDER READ A FILE, MANIPULATE THE DATA, AND
C       WRITE THE RESULTS INTO A FILE, ONLY TO HAVE THE NEXT COMMAND
C       READ THE DATA BACK AGAIN IN CORE TO MANIPULATE IT AND STORE
C       IT BACK TO A FILE THEN START THE CYCLE AGAIN...
C       TO SPEED UP THE PROCESS, THE USER HAS THE ABILITY TO HAVE THAT 
C       DATA KEPT IN INCORE MEMORY UNTIL HE DECIDES THAT HE WANTS IT IN
C       A FILE. FOR THAT, SPIDER OFFERS AN INCORE MEMORY IN A COMMON BLOCK
C       BUFFER:
C
C       ON VAX SYSTEMS, SPIDER OFFERS AN INCORE MEMORY IN A COMMON 
C       BLOCK.  THIS BUFFER CAN BE SUBDIVIDED INTO TWO PARTS TO HOLD 
C       AT MOST TWO FILES, FILES ARE CALLED _1 AND _2. 
C
C       DATA FROM A FILE IS STORED EXACTLY THE SAME WAY AS IT IS IN A
C       DISK FILE (I.E: A HEADER FOLLOWED BY NROWS OF DATA OF LENGTH NSAMS
C       REPEATED NSLICE TIMES).
C 
C--*******************************************************************

      SUBROUTINE INLN_WRTLIN(LUN,BUF,NB,NREC)

      REAL   BUF(NB)

      COMMON /UNITS/  LUNC,NIN,NOUT,NECHO,IFOUND,NPROC,NDAT
      COMMON /LUNARA/ LUNARA(100),LUNSTK(100)
      LOGICAL         USEINLN
      COMMON /LUNARB/ LOCLUN(100),NSAMLUN(100),NWORDLUN(100),USEINLN

CNO_SAVE

C     RUN-TIME ALLOCATION SPECIFIC CODE LINE FOLLOWS
      INTEGER * 8     LOCLUN

      POINTER (IPT,Q)
      DIMENSION Q(1)

      IPT  = LOCLUN(LUN)
      ILOC = 0
C     RUN-TIME ALLOCATION SPECIFIC CODE LINE ENDS

C     GET THE CORRECT OFFSET INTO THE BUFFER.
      IGO = (LUNARA(LUN) + LUNSTK(LUN) + NREC - 1) * NSAMLUN(LUN) + ILOC 

      IF ((IGO + NB - 1 - ILOC) .GT. NWORDLUN(LUN)) THEN
C        GOES OUT OF RESERVED BUFFER!
         WRITE(NOUT,*) '*** INLN_WRTLIN ARRAY OVERFLOW, LUN: ',LUN,
     &      '   LENGTH:',IGO + NB,'   NREC: ',NREC,
     &      '   LUNARA(LUN) :',LUNARA(LUN),'  LUNSTK(LUN):',LUNSTK(LUN),  
     &      '   NSAMLUN(LUN) :',NSAMLUN(LUN)  
         CLOSE(NDAT)
         CALL ERRT(100,'WRTLIN',NE)
         IERR = 1
         RETURN

      ELSEIF (IGO .LT. ILOC) THEN
C        BEFORE RESERVED BUFFER!
         WRITE(NOUT,*) '*** INLN_WRTLIN ARRAY UNDERFLOW, LUN: ',LUN,
     &      '   IGO:',IGO,'   NREC: ',NREC,
     &      '   LUNARA(LUN) :',LUNARA(LUN),'  LUNSTK(LUN):',LUNSTK(LUN),  
     &      '   NSAMLUN(LUN) :',NSAMLUN(LUN)  
         CLOSE(NDAT)
         CALL ERRT(100,'WRTLIN',NE)
         IERR = 1
         RETURN
      ENDIF

C     WRITE THE BUFFER (THERE MAY BE A FASTER WAY TO DO THIS!!)
      DO I= 1,NB
         Q(IGO + I) = BUF(I) 
      ENDDO
      IERR = 0

      RETURN
      END

