
/*$Header: /net/ithaca/usr1/web/src/RCS/backmen.c,v 1.23 1996/03/25 14:43:17 leith Exp $*/

/*C++*************************************************************************
C
C backmen.c             Mar. 94
C                                                                  
C **************************************************************************
C *  AUTHOR: A. LEITH                                                      *
C *  COPYRIGHT (C)1994, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    PARAMETERS:   
C
C    PURPOSE:      SETS IMAGE CONTAST NOT USING LUTS
C
C    CALLED BY:    fitdocmen -> backmen  -> showimage --> redvol
C                                             |           wipic
C                                             |-------------'
C                                           pickdisp
C
C
C              
C                  backmen_buta -> showimage --> redvol
C                                      |         wipic
C                                      |----------'
C                                  pickdisp
C
C  Note:
C  THis routine could be speeded up by caching the floating point 
C       data from the file
C
C--********************************************************************
*/

#include "common.h"
#include "routines.h"

#include <Xm/Scale.h>

 /* internal function prototypes */
 void          backmen_buta(Widget iw_temp, XtPointer, XtPointer);
 void          backmen_butm(Widget iw_temp, XtPointer, XtPointer);
 
 /* externally defined global variables */
 extern XImage     * imager;
 extern XImage     * imagel;
 extern FILEDATA   * filedatal;
 extern FILEDATA   * filedatar;
 extern char       * cptrduml; /* data used by image right */
 extern char       * cptrdumr; /* and left */

 /* internal file variables */
 static Widget iw_backmen = (Widget) 0, iw_ngo, iw_nend;
 static int    ngo, nend;
 static int    leftside;

 /************************   backmen   ********************************/

 void backmen(int leftsidet)

 { 
 static Widget   iw_rowcolh;
 Widget          iw_rowcolv;
 Widget          iw_dums, iw_pushc, iw_pusha;
 float*          fptrdum = NULL;

 ngo      = imagego;
 nend     = imageend;

 if (iw_backmen == (Widget)0)
    {   /* create background enhancing menu widget first */

    iw_backmen  = wid_dialog(iw_win, 0, "Contrast options", -1,-1);
    iw_rowcolv  = wid_rowcol(iw_backmen, 'v', -1,-1);

    /* create label for scales */
    wid_labelg(iw_rowcolv, 0,
               "Lower and upper end of ramp:", -1,-1);

    /* create a horizontal row-col widget for scales */
    iw_rowcolh = wid_rowcol(iw_rowcolv, 'h', -1,-1);

    /* create scale widget for starting value ------------------- ngo */
    iw_ngo  = wid_scale(iw_rowcolh, 0, "", imagego,imageend,  ngo, 
                        180,50, -1,-1);

    /* create scale widget for ending value -------------------- nend */
    iw_nend = wid_scale(iw_rowcolh, 0, "", imagego,imageend,  nend, 
                       180,50, -1,-1);

    XtAddCallback(iw_ngo, XmNvalueChangedCallback, 
                           (XtCallbackProc)backmen_butm,"0");
    XtAddCallback(iw_nend,XmNvalueChangedCallback, 
                           (XtCallbackProc)backmen_butm,"0");

    /* create box for apply ----------------------------------- apply */
    iw_dums = wid_stdbut(iw_rowcolv, iw_backmen, 
                        &iw_dums, &iw_pushc, &iw_pusha,  "CA",
                        fin_cb,fin_cb ,backmen_buta, NULL);
    }
 else
    {   /* contrast menu widget already exists, update it */

    /* alter scale widget for starting value -------------------- ngo */
    iw_ngo = wid_scale(iw_rowcolh, iw_ngo, "", imagego,imageend,  ngo, 
                       180,50, -1,-1);

    /* alter scale widget for ending value  -------------------- nend */
    iw_nend = wid_scale(iw_rowcolh, iw_nend, "", imagego,imageend,  nend, 
                       180,50, -1,-1);
    }

 /* set wait cursor */
 waitcursor(iwin, TRUE, 79);

 if (leftsidet)
    {
    /* free up ximage buffer, before creating a new one */
    if (imagel) 
       {  XFree((void *)imagel); imagel =  NULL; } 

    /* show left image using original ramp */                 
    nsam2 = nsaml; nrow2 = nrowl;
    showimage(filedatal, TRUE,  &cptrduml, 
                         TRUE,  &imagel,
                         FALSE, &fptrdum, FALSE,
                         filedatal->fmin, filedatal->fmax,
                         imagego, 127);    

    /* shift left image to shifted location & display */
    pickdisp(TRUE, FALSE);
    }
 else
    {
    /* show right image using original ramp */                       
    nsam2 = nsamr; nrow2 = nrowr;

    /* free up ximage buffer, before creating a new one */
    if (imager) 
       { XFree((void *)imager); imager = NULL; } 

    showimage(filedatar, TRUE,  &cptrdumr, 
                         TRUE,  &imager,
                         FALSE, &fptrdum, FALSE,
                         filedatar->fmin, filedatar->fmax,   
                         imagego, 127);    

    /* shift right image to shifted location and display */      
    pickdisp(FALSE, TRUE);
    }

 leftside = leftsidet;

 /* cancel waitcursor */
 waitcursor(iwin, FALSE, 79);

 XtManageChild(iw_backmen);

 }


 /*********** movescale  button callback *****************************/

 void backmen_butm(Widget iw_temp, XtPointer data, XtPointer call_data)

 {
 int             nindex, ival, iflags, indx;  
 float           fval,ft;

 /* get new ngo & nend */
 XmScaleGetValue(iw_ngo,  &ngo);
 XmScaleGetValue(iw_nend, &nend);

 if (nend <= ngo)
    {
    spout("*** Upper value must be > lower value");
    return;
    }
 else if (ngo < imagego)
    {
    spout("*** Lower value outside of image ramp");
    return;
    }
 else if (nend > imageend)
    {
    spout("*** Upper value outside of image ramp");
    return;
    }

 iflags = DoRed | DoGreen | DoBlue; 

 /* set the colors in the image portion of the colormap, only */ 
 indx = 0;
 ft = 65535.0 / ((float) (nend - ngo));
 for (nindex = mapgo; nindex <= imageend; nindex++)
    {
    colort[indx].pixel = nindex;
    colort[indx].flags = iflags;

    if (nindex <= ngo) 
       {
       /* set this color to black */
       ival = 0;
       }
    else if (nindex < nend)
       {
       /* interpolate shade */
       fval = (float)(nindex - ngo) * ft;
       ival = fval;
       }
    else 
       {
       /* set this color to white */
       ival = 65535;
       }

    colort[indx].red     = ival;
    colort[indx].green   = ival;
    colort[indx++].blue  = ival;
    }
 
 /* put the colors in the desired colormap */
 XStoreColors(idispl, map, colort, indx);

 XFlush(idispl);
 return;
 }


 /*********** accept  button callback *****************************/

 void backmen_buta(Widget iw_temp, XtPointer data , XtPointer calldata)

 {

 float      fmint, fmaxt;
 float      *fptrdum = NULL;

 /* remove menu */
 XtUnmanageChild(iw_backmen);

 /* set wait cursor */
 waitcursor(iwin, TRUE, 79);

 /* set location for images */ 
 placed = TRUE;
 if (leftside)
    {
    ixuln  = ixull;  iyuln = iyull;
    nsam2 = nsaml; nrow2 = nrowl;

    /* find new displayed levels of fmin and fmax for left image */
    fmint = (((float)(ngo - imagego) / (float)(imageend - imagego))) * 
            (filedatal->fmax - filedatal->fmin) + filedatal->fmin;

    fmaxt = (((float)(nend - imagego) / (float)(imageend - imagego))) * 
            (filedatal->fmax - filedatal->fmin)+ filedatal->fmin;
 
    /* free up ximage buffer, before creating a new one */
    if (imagel) 
       { XFree((void *)imagel); imagel =  NULL;  } 

    /* show left image using new ramp */
    showimage(filedatal, TRUE,  &cptrduml, 
                         TRUE,  &imagel,
                         FALSE, &fptrdum, FALSE,
                         fmint, fmaxt, imagego, 127);

    /* shift left image to shifted location and display */
    pickdisp(TRUE, FALSE);
    }
 else
    {
    /* set location for right images */ 
    placed = TRUE;
    ixuln  = ixulr;  iyuln = iyulr;
    nsam2 = nsamr; nrow2 = nrowr;

    /* find new displayed levels of fmin and fmax for right image */
    fmint = (((float)(ngo - imagego) / (float)(imageend - imagego))) * 
            (filedatar->fmax - filedatar->fmin) + filedatar->fmin;

    fmaxt = (((float)(nend - imagego) / (float)(imageend - imagego))) * 
            (filedatar->fmax - filedatar->fmin) + filedatar->fmin;

    /* free up ximage buffer, before creating a new one */
    if (imager) 
       {  XFree((void *)imager); imager = NULL;  } 
   
    /* show right image using new ramp */
    showimage(filedatar, TRUE,  &cptrdumr, 
                         TRUE,  &imager, 
                         FALSE, &fptrdum, FALSE,
                         fmint, fmaxt, imagego, 127);

    /* shift right image to shifted location and display */      
    pickdisp(FALSE, TRUE);
    }

 /* apply normal uncontrasted ramp to whole image */
 ngo  = imagego;
 nend = imageend;
 XmScaleSetValue(iw_ngo,ngo);
 XmScaleSetValue(iw_nend,nend);
 backmen_butm(NULL,NULL,NULL);

 /* cancel waitcursor */
 waitcursor(iwin, FALSE, 79);

 return; 
 }

