/* 
C++************************************************************************
C
C cald.c	Converted from Fortran program in Nov 95
C
C **************************************************************************
C *  COPYRIGHT (C)1992, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH	        		   *
C **************************************************************************
C
C
C
C23456789012345678901234567890123456789012345678901234567890123456789012
C--************************************************************************
C
*/
#include "common.h"
#include "routines.h"
#include "x.h"
#include "std.h"
#include <math.h>

/* constants */
#define	DEPS	1.0e-7

/******************************** cald ***********************/
float* cald(float *a1, float *a2)
{
    float	r1[9], r2[9], r3[9];		/* the rotation matrix */
    float	psi, theta, phi;		/* Eular angles */
    int		i, j, k;				/* loop variables */
    float	*atmp1, *atmp2;

    /* change angle to quatum defination */
    atmp1 = a1;
    atmp2 = a2;
    for (i=0; i<3; i++) {
	*atmp1 = antiang(*atmp1);
	*atmp2 = antiang(*atmp2);
	atmp1++;  atmp2++;
	}

    /* clear the buffers */
    for (i=0; i<9; i++) {
	r1[i] = 0.0;
	r2[i] = 0.0;
	r3[i] = 0.0;
	}

    /* build up the matrix from a1 and a2 */
    bldr(a2[0], a2[1], a2[2], &r2[0]);
    bldr(a1[0], a1[1], a1[2], &r1[0]);
  
    /* get the new matrix */
    for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
	    for (k=0; k<3; k++) 
	    	r3[i*3+j] = r3[i*3+j] + r2[i*3+k] * r1[k*3+j];
    	    }
	}

    /* formalize the numbers */
    for (i=0; i<9; i++) {
	if (fabs(r3[i]) < DEPS) 	r3[i] = 0.0;
	if (r3[i]-1.0  > -DEPS) r3[i] = 1.0;
	if (r3[i]+1.0  < DEPS)  r3[i] = -1.0;
	}

    /* calculate the angles */
    if (r3[8] == 1.0) {
	theta = 0.0;
	psi   = 0.0;

	if (r3[0] == 0.0)	phi = angle(asin(r3[1]));
	else { 
	  if (r3[0]>=0.0 && r3[1]>=0.0)	
	    phi = angle(atan(r3[1]/r3[0]));
	  else if (r3[0]<0.0)
	    phi = 180.0 + angle(atan(r3[1]/r3[0]));
	  else		
	    phi = 360.0 + angle(atan(r3[1]/r3[0]));
	     }
 	}

    else if (r3[8] == -1.0) {
	theta = 180.0;
	psi   = 0.0;

	if (r3[0] == 0.0)	phi = angle(asin(-r3[1]));
	else {
 	  if (r3[0]<=0.0 && r3[1]<=0.0)	
	    phi = angle(atan(r3[1]/r3[0]));
	  else if (r3[0]>0.0)
	    phi = 180.0 + angle(atan(r3[1]/r3[0]));
	  else		
	    phi = 360.0 + angle(atan(r3[1]/r3[0]));
	     }
	}

    else {
	    theta = angle(acos(r3[8]));

	if (r3[6] == 0.0) {
	    if (theta*r3[7] <= 0.0)	psi = 270.0;
	    else			psi = 90.0;
	    }
	else {
	  if (r3[6]>=0.0 && r3[7]>=0.0)
	    psi = angle(atan(r3[7]/r3[6]));
	  else if (r3[6]<0.0)
	    psi = 180.0 + angle(atan(r3[7]/r3[6]));
	  else  
	    psi = 360.0 + angle(atan(r3[7]/r3[6]));
	    }

	if (r3[2] == 0.0) {
	    if (theta*r3[5] <= 0.0)	phi = 270.0;
	    else			phi = 90.0;
	    }
	else {
	  if (-r3[2]>=0.0 && r3[5]>=0.0) 
	    phi = angle(atan(r3[5]/-r3[2]));
	  else if (-r3[2]<0.0)
	    phi = 180 + angle(atan(r3[5]/-r3[2]));
	  else 
	    phi = 360.0 + angle(atan(r3[5]/-r3[2]));
	    }
	}

    a1[0] = roundang(phi);
    a1[1] = roundang(theta);
    a1[2] = roundang(psi);

    for (i=0; i<3; i++) {
	if ( a1[i] == 360.0 )
	    a1[i] = 0.0;
	}

    return a1;
}


	
