
/*$Header: /net/ithaca/usr1/web/src/RCS/cntplot.c,v 1.8 1995/05/26 16:21:42 leith Exp $*/

/*
C++*************************************************************************
C
C cntplot.c          Mar 94  
C                                                                  
C **************************************************************************
C *  AUTHOR: A. LEITH                                                      *
C *  COPYRIGHT (C)1994, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH	        		   *
C **************************************************************************
C
C    cntplot
C
C    PARAMETERS:   fileptrcnt      pointer to cnt data structure
C
C    PURPOSE:      displays contours
C
C    CALLED BY:    cntmen
C
C--********************************************************************
*/

/* common variables */
#include "common.h"
#include "filescnt.h"
#include "x.h"
#include "routines.h"

 /* externally defined variables */
 extern int            cntlastcnt, cntcenter, cntclear, cntselect;
 extern int            cntnumlis, cntmirror, cntcurcol, cnttext;
 extern float          cntplscl;

/******************************  cntplot  ***************************/

 int cntplot( FILEDATACNT * fileptrcnt)

 {
 DATACNT    * cnt;
 int          icntr, inten, ivptsize, ivptd2, icntrt, ix1, iy1, nchar;
 int          j, ipts, i;
 float      * data;
 char       * textdata = NULL;
 int          ndata;
 XGCValues    gcval;
 GC           icontxt;
 float        xsize, ysize, plsclt, xdiff, ydiff, xmaxd, ymaxd, plx, ply;
 XPoint     * points;

 union  DUALC
        {
        float fdata;
        char  cdata[5];
        } txtcnt;

 /* create contour data structure */
 if ((cnt = (DATACNT *) calloc(1, sizeof(DATACNT))) == NULL)
   {spout("*** Unable to calloc cnt in cntplot.c"); return FALSE;}

 /* set scaling size for plot */
 xsize    = iwidex;
 ysize    = ihighx;
 ivptsize = MYMIN(iwidex,ihighx);
 ivptd2   = ivptsize / 2;

 if (cntcenter) 
    {
    /* find plot size */
    xmaxd = (fileptrcnt -> xmax - fileptrcnt -> xmin) / fileptrcnt -> scale;
    xdiff = MYMAX(xmaxd,1.0);
    ymaxd = (fileptrcnt -> ymax - fileptrcnt -> ymin) / fileptrcnt -> scale;
    ydiff = MYMAX(ymaxd,1.0);
    }
 else
    {
    xdiff = 1.0;
    ydiff = 1.0;
    }

 /* find offset constants, y-origin at upper left increases downward */
 plx = 0.5 * (xsize - ivptsize) + ivptd2 - (0.5 * cntplscl * xdiff);
 ply = ysize - (0.5 * (ysize - ivptsize)) -
              ivptd2 + (0.5 * cntplscl * ydiff);

 if (cntmirror) 
    { /* mirror around x axis */
    ply    = ply - cntplscl;
    plsclt = - cntplscl;
    }
 else
    plsclt = cntplscl;

/******************
printf(" plx,ply: %f %f,   plscl,plsclt: %f %f --------\n",
         plx,ply,cntplscl,plsclt);
*****************/

 /* set number of contours to be displayed */
 cntlastcnt = fileptrcnt -> ncntrs;
 if (cntselect) cntlastcnt = cntnumlis;

 /* clear screen, if desired */
 if (cntclear) webclear(NULL,NULL,NULL);
 
 if (!cntcurcol)
    {  /* create temporary graphics context for use here */
   icontxt = XCreateGC(idispl,iwin, 0, &gcval);
   XCopyGC(idispl, icontx, 0, icontxt);
    }
 else
    icontxt = icontx;

 /* allocate memory for cnt data array in cnt structure */
 cnt -> nsize = 4000;

 if ((data = (float *) malloc(cnt->nsize * 3 * sizeof(float))) == NULL) 
   {spout("*** Unable to malloc data in cntplot.c"); return FALSE;}
 cnt -> data = &data[0];

 if ((points = (XPoint *) malloc(cnt -> nsize * sizeof(XPoint))) == NULL)
   {spout("*** Unable to malloc points in cntplot.c"); return FALSE;}
   

 for (icntr= 1; icntr <= cntlastcnt; icntr++)
    { 
    /* get contour data from the file for icntr */

    icntrt = icntr;
    if (cntselect) icntrt = cntnumlis;

    if (csget(fileptrcnt, cnt, icntrt, data, &ndata)) 
       {          /* have retrieved a contour */
       inten = cnt -> inten;
       /*  if (cnt -> ipen == 0 && cnt -> inten == 9) inten = 0; */
       if (cntcurcol) inten = -1;
   
       if (fileptrcnt -> ifrmcs == -6 && cnt -> mark == -2)
          {   /* new style text contour */
          ix1   = data[0] * cntplscl + plx;
          iy1   = ply - data[1] * plsclt;
          nchar = data[2];
          txtcnt.fdata        = data[3];
          txtcnt.cdata[nchar] = '\0';

          witext(icontxt, (char *) &txtcnt.fdata, ix1, iy1, 
               cnt -> itsize, cnt -> just, inten , cnt -> ipen, FALSE);

          }
       else
          {  /* standard graphics contour */

          j = 0;
          for (ipts = 0; ipts < ndata; ipts++)
             {
             points[ipts].x = data[j++] * cntplscl + plx;
             points[ipts].y = ply - data[j++] * plsclt;
             if (cnt -> threed) j++;
             }
 
          wilines(icontxt, points, ndata, inten, cnt -> ipen,
                  cnt -> lintp);
/************************
          printf(" (%f,%f) -->(%d,%d)-- %d %d\n",
data[0],data[1],
points[0].x,points[0].y,ndata,cnt->ndata);
**************************/
          }
       }
    }
 
 if (cnttext) 
    {
    /* allocate memory for cnt text array in cnt structure */
    if ((textdata = (char *) malloc(cnt->nsize*sizeof(char))) == NULL) 
       {spout("*** Unable to malloc textdata in cntplot.c"); 
        return FALSE;}

    for (i=1; i <= fileptrcnt -> lsttxt; i++)
       {
       if (csgettext(fileptrcnt, cnt, i, textdata) > 0)
          {
          /* got text back */

          /* set position for text */
          ix1   = cnt-> data[1] * cntplscl + plx;
          iy1   = ply - cnt -> data[2] * plsclt;

          /* set color for text */
          inten = cnt -> inten;
          if (cnt -> ipen == 0 && cnt -> inten == 9) inten = 0;
          if (cntcurcol) inten = -1;

          witext(icontxt, textdata, ix1, iy1, 
             cnt -> itsize, cnt -> just, inten , cnt -> ipen, FALSE);
          }
       }

    }


 if (!cntcurcol)  XFreeGC(idispl,icontxt);
 if (data)     free(data);
/****************debug  
  printf("  freeing data %d %d\n",textdata,&textdata);
*****************/
 if (cnttext && textdata) free(textdata);
 if (points)   free(points);
 if (cnt)      free(cnt);
/****************debug 
 printf(" finished freeing data %d %d\n",textdata,&textdata);
*****************/

 return TRUE ;
 }


