
/*
***************************************************************************
** comap.c 			7/28/93
** WID_COMAP.FOR (VAX) transported to UNIX
**
** **************************************************************************
** *  AUTHOR:  Mahieddine Ladjadj                                           *
** *  COPYRIGHT (C)1993, WADSWORTH CENTER FOR LABORATORIES AND              *
** *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
** *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
** *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
** *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
** *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
** **************************************************************************
**
**    WID_COMAP
**
**    PURPOSE:        DISPLAYS CORRELATION MAP, USER CAN OUTLINE VARIABLE
**                    SHAPED WINDOW FROM MAP,  STORE THE IMAGE IDS THAT
**                    ARE LOCATED INSIDE WINDOW IN A DOC FILE, FIND AND
**                    DISPLAY AVERAGE OF IMAGES LOCATED WITHIN WINDOW
**
*****************************************************************************
*/

#include "common.h"
#include "files.h"
#include "std.h"
#include "routines.h"

/* external variables */
  extern    char      docxnam[81];
  extern    int       sayno, itrad, key1, key2, itxreg, ityreg;
  extern    float     stndrt;
  double              factxa, factxb,factya, factyb;
  float               *bufx, *bufy;
  int                 *nkey;

  int    nline, ixmar, iymar, ixsize, iysize, gotbin, gotav, gotavim,
         nowbin;

/******************************** comap () ********************************/

void comap(void)
{
  register int k;
  int     maxkeys, icolort,
          maxreg, ix, iy, numtot, numcor, lastkey,
          doc_read, pkey;
  float   *plist, *ptr, *dbuf;
  double  davx, davx2, davy, davy2, bx, by, xmin, xmax, ymin, ymax, xr, yr,
          sigx, sigy, xrange, yrange, xdev, ydev, tlx, tly;
  char    outstr[81];
     
  /* set wait cursor */
  waitcursor(iwin, TRUE, 79);                                  
  
  /* recover all the lines from the doc file and store in the dbuf array.
  ** column 1 is the key, following columns are registers. correspondence
  ** analysis may have up to 24 registers. */

  doc_read = TRUE;
  pkey     = 0;
  maxkeys  = MYMAX(key1,key2) + 1;
  maxreg   = MYMAX(itxreg,ityreg) + 1;
  dbuf     = (float *) NULL;
  if (unsdal(docxnam, &doc_read, pkey, plist, maxreg, &dbuf, &lastkey,
                   maxkeys,maxreg) > 0)
      {
      /* problem retrieving doc file, assume it does not exist */
      if (dbuf) free(dbuf);
      sprintf(outstr," Could not read document file: %s",docxnam);
      spout(outstr);

      /* unset wait cursor */
      waitcursor(iwin, FALSE, 0);                                  
      return;
      }
  
  if (lastkey < MYMIN(key1,key2))
      {
      if (dbuf) free(dbuf);
      sprintf(outstr," both keys needed %d & %d > lastkey = %d",
         key1,key2,lastkey);
      spout(outstr);

      /* unset wait cursor */
      waitcursor(iwin, FALSE, 0);                                  
      return;
      } 
      
  /* determine maximun/minimum x and y coordinates, avg, etc; */
  xmin = xmax = ymin = ymax = 0;
  davx = davx2 = davy = davy2 = 0; 

  /* run time allocate of memory space for nkey[], bufx, bufy */
  if ( (nkey = (int *) calloc((key1 * key2), sizeof(float)) )
             == (int *) NULL)                                       {
     spout(" *** Could not allocate memory");
     waitcursor(iwin, FALSE, 0);  return;                           }
  if ( (bufx = (float *) calloc((key1 * key2), sizeof(float)) )
             == (float *) NULL)                                     {
     spout(" *** Could not allocate memory");
     waitcursor(iwin, FALSE, 0);  return;                           }
  if ( (bufy = (float *) calloc((key1 * key2), sizeof(float)) )
             == (float *) NULL)                                     {
     spout(" ***Could not allocate memory");
     waitcursor(iwin, FALSE, 0);  return;                           }

  ptr = dbuf + (key1-1) * maxreg;
  nline = 0;

  for (k = key1; k <= key2; k++)
      {
      /* check that the key has some registers. */
      if ( (*ptr) > 0.0) 
          {
          nkey[nline] = k;          /* image number     */
          bx = *(ptr + itxreg);      /*  dbuf[k][itxreg]  */
          by = *(ptr + ityreg);      /*  dbuf[k][ityreg]  */

          /* record bx & by */
          bufx[nline] = bx;
          bufy[nline] = by;
          nline++;
    
          /* record max and min for bx & by   */
          xmin = MYMIN(bx,xmin);
          xmax = MYMAX(bx,xmax);
          ymin = MYMIN(by,ymin);
          ymax = MYMAX(by,ymax);

          /* record average and standard deviation for bx & by */
          davx += bx;
          davx2 += (double)(bx * bx);
          davy += by;
          davy2 += (double)( by * by);
          }
      ptr += maxreg;
      }        /*   for (pkey = key1; pkey <= key2; pkey++)  */

  xr = xmax - xmin;
  yr = ymax - ymin;

  /* calculate sigma and average */
  davx /= (float) nline;
  davy /= (float) nline;

  sigx = sqrt((davx2 - (davx * davx / (float) nline)) / (double) (nline - 1));
  sigy = sqrt((davy2 - (davy * davy / (float) nline)) / (double) (nline - 1));

  xrange = MYMIN(2 * stndrt * sigx, xr);
  yrange = MYMIN(2 * stndrt * sigy, yr);

  xdev = xrange / 2;
  ydev = yrange / 2;

  tlx = davx - xdev;
  tly = davy - ydev;

  /* set plot size */
  ixsize = iwidex - 2;
  iysize = ihighx - 2;
  ixmar = iymar = 1;
  numtot = numcor = 0;

  /* remember current color in icontx */
  icolort = icolor;

  /* set current color to image almost white */
  wicolor(icontx,-126);

  /* clear the screen */
  clearover(NULL,NULL,NULL);

  /* find scale factors */
  factxa =  (float) ixsize / xrange;
  factxb = - (tlx * factxa) + ixmar + 0.5;
  factya = (float) iysize / yrange;
  factyb = - (tly * factya) + iymar + 0.5;

  for (k = 0; k < nline; k++)
    {
      ix = bufx[k] * factxa + factxb;
      iy = bufy[k] * factya + factyb;
/**
      if (ix >= ixmar && ix <= (ixsize + ixmar) && 
          iy >= iymar && iy <= (iysize + iymar))
**/
      if (ix > ixmar && ix < (ixsize ) && 
          iy > iymar && iy < (iysize ))
          {
          /* the image is inside the plot extent; find the label number */

          if (sayno)
              {  /* write image number at this location on plot */
              sprintf(outstr,"%d",nkey[k]);
              witext(icontx,outstr,ix,iy,0,0,-9,2,FALSE);
              }
          else
              { 
              /* draw a permanent circle at this position */
              xorc(iwin,    icontx, TRUE, ix, iy, itrad);
              xorc(imagsav, icontx, TRUE, ix, iy, itrad);
              }

          /* increment total number of images within plot extent */
          numcor += 1;
          }
      else
        {
          /* image is outside of plot extent */
          nkey[k] = - nkey[k];
        }

      /* increment total number of images */
      numtot += 1;
    }
  sprintf(outstr," Images: %d images within plot: %d",numtot,numcor);
  spout(outstr);
     
  /* replace current color in icontx */
  wicolor(icontx,icolort);

  /* initialize bin counter for this map */
  nowbin = 0;

  /* set flag for each bin mask */
  gotbin = gotavim = gotav = FALSE;

  /* cancel wait cursor */
  waitcursor(iwin, FALSE, 0);

  /* start masking of correspondence map */
  corr_mask(0);
}
