/*
*****************************************************************************
** comapsav.c		8/93
**
** Purpose: Analyses outlined comap, saves averages, etc...
**
** called by:  comapmen5.c comapmen3.c comapmen2.c
**
** WID_COMAPSAV.FOR transported to UNIX
**
** **************************************************************************
** *  AUTHOR:  Mahieddine Ladjadj                                          *
** *  COPYRIGHT (C)1990 , WADSWORTH CENTER FOR LABORATORIES AND             *
** *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
** *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
** *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
** *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
** *    THE CENTER FOR LABORATORIES AND RESEARCH			    *
** **************************************************************************
*****************************************************************************
*/

#include "common.h"
#include "std.h"
#include "x.h"
#include <Xm/Text.h>
#include <stdlib.h>
#include "routines.h"


  extern char   docnammo[81], filnamm[81], imnamm[81];  
  extern int    nline, ixmar, iymar, ixsize, iysize, gotbin, gotav,
                nowbin, gotavim;
  extern float  *bufx, *bufy;
  extern int    *nkey, ovrlap;
  extern double factxa, factxb,factya, factyb;
  extern int    itxrego, ityrego, labrego;
  extern float  rangel, rangeu;
  extern Pixmap msksav;

/********************** comapsav ***********************************/

void comapsav(int itag)
{
  register int  i, j1;
  XImage        *imagemask;
  FILEDATA      *ptrdoc;
  FILE          *fp;
  char          outstr[81], filnam[81];
  int           nlist, nrun, ixs, iys, iform, ixulw, iyulw;
  float         *plist, b, *ipoc;
  float         pmin, pmax, pmean, thup, thdown, pnormal, scal, fmsc;
  double        psig, davd, davd2;
  int           nlet, numfil, iloc, irec, nnmax = 127;   /* limited display */
  int           xxsiz, yysiz;
  static int    icountav, icountin, *ibin;
  static char   *lbuf;
  static float  *avims, *avim;
  static int    ibmaxx, ibmaxy, ibminx, ibminy;

  if (itag == 4 || itag == 5)
      {  /* clear and restart masking */
      gotav = gotbin  = gotavim = FALSE;

      /* free up the memory space */
      if(ibin) 
        { free(ibin); ibin = (int *) NULL; }
      if(avims)
        { free(avims); avims = (float *) NULL; }
      if(avim)
        { free(avim); avim = (float *) NULL; }
      if(lbuf)
        { free(lbuf); lbuf = (char *) NULL;  }
      if(msksav) 
        { XFreePixmap( idispl, msksav ); msksav = (Pixmap) NULL; }
 
     if (itag == 4)
         {/* blank markers but not image, by clearing the overlays */
         clearover(NULL,NULL,NULL);
         corr_mask(1);
         }
      else 
         corr_mask(2);

      return;
      }

  /* set wait cursor */
  waitcursor(iwin, TRUE, 79);
 
  /* find which images are within the polygon */
  if (!gotbin)
      {  /* recover the overlay bitmap into an image structure */
      imagemask = (XImage *) 0;
      if ( (imagemask = XGetImage(idispl,msksav,ixmar,iymar,
                               (unsigned int) ixsize, (unsigned int)iysize,
                               (unsigned long)128, (int) XYPixmap) )
                   == (XImage *) 0)                {
           spout("*** Image formation failed ");
           waitcursor(iwin, FALSE, 0);  return;    }

      nowbin++;
      if (!gotbin)
          { /* run time allocate of memory space for ibin[] */
          if ( (ibin = (int *) calloc(nline, sizeof(int)))
                     == (int *) NULL)                                {
               spout(" *** WARNING: returning from comapsav.c ");
               spout(" CALLOC() CAN'T ALLOCATE HEAP MEMORY SPACE for ibin");
               waitcursor(iwin, FALSE, 0);   return;                 }
          }
      icountin = 0;
      ibmaxx = ibmaxy = -10000;
      ibminx = ibminy =  10000;

      for (i = 0; i < nline; i++)
          {
          if (nkey[i] > 0)
              {    /* image inside map, find scaled location */
              ixs = (int) (bufx[i] * factxa + factxb);
              iys = (int) (bufy[i] * factya + factyb);

             if ( XGetPixel(imagemask,ixs,iys) > 0)
                  {
                  ibin[i] = nowbin;
                  icountin++;
                  ibmaxx = MYMAX(ibmaxx,ixs);
                  ibminx = MYMIN(ibminx,ixs);
                  ibmaxy = MYMAX(ibmaxy,iys);
                  ibminy = MYMIN(ibminy,iys);
                  }
              }
          else
              ibin[i] = -1;
          }
      
      /* free-up image storage in Xserver */  
      XDestroyImage(imagemask);
      gotbin = TRUE;
      }                     /************    if (!gotbin) */

  if (itag == 1) 
      {    /* save bin in doc file now. */
      nrun = 1;
      fp = (FILE *) NULL;
      nlist = MYMAX(MYMAX(itxrego,ityrego),labrego) + 2;

      /* find the name of the document file */
      nlet = strlen(docnammo);
      if (nlet==0)
          { sprintf(outstr,"filename is of lenght 0 (empty) !!! ");
            spout(outstr); waitcursor(iwin, FALSE, 0); return; }
      strcpy(filnam,docnammo);
      if(filnam[nlet] != '.')
                strcat(strcat(filnam,"."),datexc);

      j1 = 0;
      for (i = 0; i < nline; i++)
          {  /* only binned images inside map will be put in doc file */
          if (ibin[i] > 0 && nkey[i] >= 0)
              {  /* run time allocate of memory space for plist[]*/
              if ( (plist = (float *) calloc(nlist, sizeof(float)) )
                          == (float *) NULL )                           {
                    spout(" *** WARNING: return from comapsav.c ");
                    spout("CAN'T ALLOCATE HEAP MEMORY SPACE for plist");
                    waitcursor(iwin, FALSE, 0);   return;               }
 
              plist[0]        = (float) ++j1;
              plist[1]        = (float) nkey[i];
              plist[itxrego+1] = bufx[i];
              plist[ityrego+1] = bufy[i];
              plist[labrego+1] = ibin[i];
              fp = savdn1(filnam,datexc,&fp,plist,nlist,&nrun,0,TRUE);
              nrun = 0;
              free(plist);    /* free memory space */
              }
          }

      /* set next filename for doc file */
      filnam[nlet] = (char) NULL;
      strcpy(docnammo,filnam);

      /* increment the NEXT filename */
      if(filinc(docnammo,filnam,&nlet,&numfil) != 2)
           {
           sprintf(outstr,"could not increment filename %s",docnammo);
           spout(outstr); waitcursor(iwin, FALSE, 0); return;
           }
      filnam[nlet] = (char) NULL;
      strcpy(docnammo,filnam);     
  
      if(fp) 
          {
          fclose(fp);
          sprintf(outstr,"images within mask: %d. total images: %d",
                  icountin,nline);
          }
      else
          {
          sprintf(outstr," No Doc. saved. No images within mask ?");
          }
      spout(outstr);

      /* reset cursor and put up menu again */
      waitcursor(iwin, FALSE, 0);

      comapmen2();
      return;
      }       /*********************************************  itag = 1  */

  if ( (!gotav) && ((itag == 2) || (itag == 3)))
      {  /* calculate average image */
      icountav = 0;

      for (i = 0; i < nline; i++)
          {
          if (ibin[i] == nowbin)
              { /* skip images outside map boundary */
              if ( nkey[i] >= 0) 
                  {   /* construct file name */
                  filget(filnamm,filnam,81,nkey[i]);
                  strcat(strcat(filnam,"."),datexc);

                  /* open 'old' image file */
                  if ((ptrdoc = openold(filnam,&nsam,&nrow,&nslice,&iform,"o")) 
                             != (FILEDATA *) NULL )
                       {
                       if(nslice > 1) 
                           {
                           sprintf(outstr,"NOT 2D image. nslice= %d\n",nslice);
                           spout(outstr);
                           }
 
                       /* add image from new file to new running total */
                       icountav++;
                       if (icountav == 1) 
                           {
                           /* allocate memory space for average image */
                           if ( (avim = (float *) calloc((nsam * nrow),
                                 sizeof(float)) ) == (float *) NULL)           {
                           spout(" *** WARNING: return from comapsav.c ");
                           spout("CAN'T ALLOCATE MEMORY SPACE for avim");
                           waitcursor(iwin, FALSE, 0); return;                 }
                           }
                       ipoc = avim ;
                       for (irec = 1; irec <= nrow; irec++)
                           {
                           if (redlin(ptrdoc,ptrdoc->buf,nsam,irec) == TRUE)
                               {
                               for (j1 = 0; j1 < nsam; j1++)
                                   {
                                   (*avim++) += ptrdoc->buf[j1];
                                   }
                               }
                           else
                               {
                               spout("*** could not read buf for average");
                               spout("one value of AVIM[] lost:: comapsav.c");
                               }
                           }
                       avim = ipoc;
                       closefile(ptrdoc);
                       }
                  else
                       {
                       sprintf(outstr,"Bad filename: %s in comapsav.c",filnam);
                       spout(outstr);
                       }
                  }                    /*  if (nkey[i] > = 0)  */
              }                        /* if ibin[i]==nowbin  */
          }                            /* for (i = 0; i < nline; i++) */

      }     /******************    (!gotav && (itag == 2 || itag == 3)) */

  if ( (itag == 2) || ((itag == 3) && (!gotav)) )
      {
      if (icountav > 0) 
          {  /* at least one file in sum, scale the summed page */
          davd = davd2 = 0.0;
          pmin = 10e30;    pmax = -10e30;

          /* find min, max, avg, & standard dev. for average image */
          for (i = 0; i < nrow * nsam; i++)
              {
              pmax = MYMAX(avim[i],pmax);
              pmin = MYMIN(avim[i],pmin);
              davd += avim[i];
              davd2 += (double)(avim[i]) * (double)(avim[i]);
              }
          if ((pmax-pmin) < FLTZER)                      {
              spout("*** Blank average image skipped ");
              waitcursor(iwin, FALSE, 0); return;        }

          /* standard deviation */
          psig   = sqrt( (davd2 - (davd * davd) / (nsam * nrow) ) 
                      / (double)(nsam * nsam - 1.0) );
          pmean  = davd / (nsam * nrow);
          thup   = pmean + rangeu * psig;
          thdown = pmean - rangel * psig;
          if (pmax > thup)   pmax = thup;
          if (pmin < thdown) pmin = thdown;

          /* normalization for averaging */
          pnormal = 1.0 / ((float) icountav);

          /* conversion factor from floating point to logical *1 range */
          scal = ((float)(nnmax - imagego)) / (pmax - pmin);
          fmsc = - pmin * scal + imagego + 0.5;
                                                                  
          /* run time allocation of memory space for 8 bit image */
          if ( (lbuf = (char *) calloc((nsam * nrow),sizeof(char)) ) 
                     == (char *) NULL)                               {
              spout(" *** WARNING: return from comapsav.c ");
              spout(" MALLOC() CAN'T ALLOCATE HEAP MEMORY SPACE");
              waitcursor(iwin, FALSE, 0);    return;                 }                         
                            
          /* run time allocation of memory space for saved image */
          if ( (avims = (float *) calloc((nsam * nrow),sizeof(float)) ) 
                      == (float *) NULL)                              {
              spout(" *** WARNING: return from comapsav.c ");
              spout(" MALLOC() CAN'T ALLOCATE HEAP MEMORY SPACE");
              waitcursor(iwin, FALSE, 0);    return;                  } 
          if (nnmax < 128)
              {
              for (i = 0; i < nrow * nsam; i++)
                  {
                  b = avim[i];
                  if (b > thup)   b = thup;
                  if (b < thdown) b = thdown;
                  avims[i] = b * pnormal;
                  lbuf[i]  =  (b * scal) + fmsc;
                  }
              }
          else
              {
              for (i = 0; i < nrow * nsam; i++)
                  {
                  b = avim[i];
                  if (b > thup)   b = thup;
                  if (b < thdown) b = thdown;
                  avims[i] = b * pnormal;

                  /* convert 0..255 integers to -128..127 logical *1 */
                  lbuf[i] = (((b * scal) + fmsc) < 128   ? 
                            (b * scal) + fmsc            :
                            (b * scal) + fmsc - 256.0 );
                  }
              }
         
          gotav = TRUE;
          }                                /* (icountav > 0)  */

      if (itag == 2 && icountav > 0)      /* display the average image */
          {  /* find the location for image on screen */
          ixulw = MYMAX( 0,
                (((ibmaxx + ibminx) /2.0) - (float) nsam /2.0)) + ixmar;
          if ((ixulw + nsam) > iwidex) ixulw = iwidex - nsam;
          iyulw = MYMAX( 0, 
                (((ibmaxy + ibminy) /2.0) - (float) nrow /2.0)) + iymar;
          if ((iyulw + nrow) > ihighx) iyulw = ihighx - nrow;

          /* no overlaping of average image and mask ? */
          if(!ovrlap)
              {
              /* shift by xxsiz and yysiz */
              xxsiz = (int) ((ibmaxx - ibminx + nsam) / 2.0) + 10;
              yysiz = (int) ((ibmaxy - ibminy + nrow) / 2.0) + 10;

              /* try to find a position left, top, bottom, then right */
              if((ixulw - xxsiz) > 0)
                  ixulw -= xxsiz;                   /* left */
              else if((iyulw - yysiz) > 0)
                  iyulw -= yysiz;                   /* top */
              else if((iyulw + yysiz) < ihighx)
                  iyulw += yysiz;                   /* bottom */
              else if((ixulw + xxsiz) < iwidex)
                  ixulw += xxsiz;                   /* right */
              else
                  spout("can not move average image in any  direction !!");

              spout(" *** NOTE: shift is proportional to size of MASK ");
              }

          /* create image for  the screen in window */
          imagemask  = XCreateImage(idispl,visual,
                                 DefaultDepthOfScreen(iscreen),
                                 ZPixmap, 0, (char *) lbuf,
                                 (unsigned int) nsam,
                                 (unsigned int) nrow, 8, nsam);

          /* put image to the screen */
          XPutImage(idispl,iwin,icontx,imagemask,0,0,ixulw,iyulw,nsam,nrow); 

          /* make average image permanent */
          XPutImage(idispl,imagsav,icontx,imagemask,0,0,ixulw,iyulw,nsam,nrow); 

          /* make changes visible */
          XFlush(idispl);

          /* free up memory space (lbuf space has to be saved */
          imagemask->data = (char *) NULL; 
          XDestroyImage(imagemask);
          }

      sprintf(outstr,"images averaged: %d. total images: %d",
         icountav,nline);  spout(outstr);
      }        /*********    (itag == 2 || (itag == 3 && (!gotav)) ) */

  if (itag == 3 && icountav > 0)
      {/*****  save the averaged image       *********** itag=3      */
      /* find the filename for averaged image */
      nlet = strlen(imnamm);
      if (nlet==0)
         { sprintf(outstr,"filename is of lenght 0 (empty) !!! ");
           spout(outstr); waitcursor(iwin, FALSE, 0); return;        }
   
      strcpy(filnam,imnamm);
      if(filnam[nlet] != '.')
               strcat(strcat(filnam,"."),datexc);

      iform = 1;      /* 2D image */    
      if ((ptrdoc = opennew(filnam,nsam,nrow,nslice,iform,"n"))
                 != (FILEDATA *)NULL)
            {
            iloc = 0;
            for (i = 1; i <= nrow; i++)
                {
                wrtlin(ptrdoc,&avims[iloc],nsam,i);
                iloc += nsam;
                }
            closefile(ptrdoc);

            /* increment the NEXT filename */
            filnam[nlet] = (char) NULL;
            strcpy(imnamm,filnam);
            if(filinc(imnamm,filnam,&nlet,&numfil) != 2)
              { sprintf(outstr,"could not increment filename %s",imnamm);
                spout(outstr); waitcursor(iwin, FALSE, 0); return;       }

            filnam[nlet] = (char) NULL;
            strcpy(imnamm,filnam); 
            }
      else
            {
            spout("can not save averaged image or masked images");
            spout("can not open the file as a new file.");
            sprintf(outstr,"filename: %s already exist.",imnamm);
            spout(outstr);
            }
      }          /*********   if (itag == 3 && icountav > 0) */
 
  /* unset cursor */
  waitcursor(iwin, FALSE, 0);

  /* put up menu again */
  comapmen2();

  return;
  }               
 /****** end of comapsav function ******/

