
/*$Header: /net/ithaca/usr1/web/src/RCS/contrastmen.c,v 1.10 1996/01/22 20:18:27 leith Exp $*/

/*C++*************************************************************************
C
C contrastmen.c             DEC 92 
C                                                                  
C **************************************************************************
C *  AUTHOR: A. LEITH                                                      *
C *  COPYRIGHT (C)1991, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    contrastmen
C
C    PARAMETERS:   
C
C    PURPOSE:      SETS IMAGE CONTAST USING LUTS
C
C    CALLED BY:    web_com
C
C    NOTE:         maybe this should work over imagego...imagend 
C                  instead of mapgo... mapend??
C
C--********************************************************************
*/

#include "common.h"
#include "routines.h"

#include <Xm/Scale.h>
#include <Xm/ToggleBG.h>

 /* internal function prototypes */
 void          contrastmen_buta(Widget iw_temp, caddr_t, caddr_t);

 /* internal common variables */
 static Widget iw_contrastmen= (Widget)0, iw_red, iw_blue, iw_green;
 static Widget iw_ngo, iw_nend;
 static int    ngo, nend, redit, greenit, blueit;

 /**********************   contrastmen   ******************************/

 void contrastmen(Widget iw_caller, caddr_t data, caddr_t call_data)

 { 
 Widget iw_lab1, iw_rowcolv, iw_rowcolh;
 Widget iw_dums, iw_pushc, iw_pusha;

  if (iw_contrastmen <= (Widget)0)
    {   /* create contrast menu widget first */

    iw_contrastmen  = wid_dialog(iw_win, 0, "Contrast options", -1,-1);
    iw_rowcolv      = wid_rowcol(iw_contrastmen, 'v', -1,-1);

    /* create label for scales */
    iw_lab1 = wid_labelg(iw_rowcolv, 0,
                         "Lower and upper end of ramp:", -1,-1);

    /* create a horizontal row-col widget for scales */
    iw_rowcolh = wid_rowcol(iw_rowcolv, 'h', -1,-1);

    /* create scale widget for starting value ------------------- ngo */
    ngo     = mapgo;
    iw_ngo  = wid_scale(iw_rowcolh, 0, "", mapgo,mapend,  ngo, 
                        180,50, -1,-1);

    /* create scale widget for ending value -------------------- nend */
    nend    = mapend;
    iw_nend = wid_scale(iw_rowcolh, 0, "", mapgo,mapend,  nend, 
                       180,50, -1,-1);

    XtAddCallback(iw_ngo,XmNvalueChangedCallback,
                           (XtCallbackProc) contrastmen_buta,"0");
    XtAddCallback(iw_nend,XmNvalueChangedCallback,
                           (XtCallbackProc) contrastmen_buta,"0");

    /* create toggle boxes for red, greeen & blue colors */
    redit   = TRUE;
    greenit = TRUE;
    blueit  = TRUE;

    iw_red   = wid_togmen(iw_rowcolv, 0, "Red  ", redit,   -1,-1);
    iw_green = wid_togmen(iw_rowcolv, 0, "Green", greenit, -1,-1);
    iw_blue  = wid_togmen(iw_rowcolv, 0, "Blue ", blueit,  -1,-1);

    /* create box for cancel --------------------------------- cancel */
    iw_dums = wid_stdbut(iw_rowcolv, iw_contrastmen, 
                        &iw_dums, &iw_pushc, &iw_pusha,  "C",
                        fin_cb,fin_cb ,fin_cb, NULL);
    }
 else
    {   /* contrast menu widget already exists, update it */

    /* alter scale widget for starting value -------------------- ngo */
    iw_ngo = wid_scale(iw_rowcolh, iw_ngo, "", mapgo,mapend,  ngo, 
                       180,50, -1,-1);

    /* alter scale widget for ending value  -------------------- nend */
    iw_nend = wid_scale(iw_rowcolh, iw_nend, "", mapgo,mapend,  nend, 
                       180,50, -1,-1);
    }

 XtManageChild(iw_contrastmen);

 }


 /************* accept button callback ********************************/

 void contrastmen_buta(Widget iw_temp, caddr_t data, caddr_t call_data)

 {
 int             nindex, ival, iflags, indx;  
 float           fval;

 /* get ngo & nend */
 XmScaleGetValue(iw_ngo, &ngo);
 XmScaleGetValue(iw_nend,&nend);
 if (nend < ngo)
    {
    spout("*** Upper value must be > lower value");
    return;
    }
 else if (ngo < mapgo)
    {
    spout("*** Lower value outside of image ramp");
    return;
    }
 else if (nend > mapend)
    {
    spout("*** Upper value outside of image ramp");
    return;
    }


 redit   = XmToggleButtonGadgetGetState(iw_red);
 blueit  = XmToggleButtonGadgetGetState(iw_green);
 greenit = XmToggleButtonGadgetGetState(iw_blue);

 iflags = 0;
 if (redit)   iflags = DoRed; 
 if (greenit) iflags = iflags | DoGreen ;
 if (blueit)  iflags = iflags | DoBlue;
                                                                                 

 /* set the colors in the image portion of the colormap, only */ 
 indx = 0;
 for (nindex = mapgo; nindex <= mapend; nindex++)
    {
    colort[indx].pixel = nindex;
    colort[indx].flags   = iflags;

    if (nindex <= ngo) 
       {
       /* set this color to black */
       ival = 0;
       }
    else if (nindex < nend)
       {
       /* interpolate shade */
       fval = 65535.0 * (float)(nindex - ngo) / (float) (nend - ngo);
       ival = fval;
       }
    else 
       {
       /* set this color to white */
       ival = 65535;
       }

    colort[indx].red   = ival;
    colort[indx].green = ival;
    colort[indx++].blue  = ival;
    }
 
 /* put the colors in the desired colormap */
 XStoreColors(idispl,map,colort,indx);

 XFlush(idispl);
 return;
 }




   
  
