/*
C***************************************************************************
C  corr_mask.c                   8/93
c
C  WID_MASK.FOR (VAX) transported to UNIX
C  made into mask.c, & corr_mask.c
C                                                              
C **************************************************************************
C *  AUTHOR: Mahieddine Ladjadj                                            *
C *  COPYRIGHT (C)1990, WADWSWORTH CENTER FOR LABORATORIES AND RESEARCH,   *
C *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    MASK
C
C    PURPOSE:      CREATES A MASK BY DRAWING AROUND AREAS WITH MOUSE
C
C    PARAMETERS:   
C
C    CALLED BY:    MASKMEN
C
C--*********************************************************************
*/

#include "common.h"
#include "routines.h"

 void  corr_mask_pop    (Widget, XEvent *, String *, Cardinal *);
 extern Pixmap        msksav;

 /* internal common variables */
 GC            icontmx;
 static int    masks, gotpt, filler, ipts=0;

/************************ corr_mask   *******************************/

void corr_mask(int itag)
  {
  /* set number of masked polygons made so far to zero */
  if (itag == 0 || itag == 1)
      masks  = 0;
  /* else if (itag==2) continue showing all masks */

  /* set number of points on current polygon of this mask */
  gotpt = FALSE;
  filler = TRUE;

  /* interactive correspondance map masking */
  showbutx("Starts / sets line segment.",
           "Close Polygon / Call Menu.",
           "Stop routine.",FALSE);

  actions(iw_win, "corr_mask_pop", corr_mask_pop, "M123");


  /* update the screen */
  XFlush(idispl);
  expos_cb(NULL,NULL,NULL); 

  /* set new lut for all correct use with x-or */
  overlut();

  /* set mask so image in first 7 planes and mask in overlay plane 8 */
  wisetplanes(idispl, icontx, 128);
 
  /* set up a xor graphics context */
  icontmx = setxor(icontx);
  }


/****************** corr_mask_pop  ************************************/

void corr_mask_pop(Widget iw_t, XEvent *event, String *params,
                   Cardinal *num_params)
  {
  static XPoint *points;
  static int    icoloro, ix2, iy2, ix1, iy1, ix0, iy0, nsize = 10;

  if (!(strcmp(*params,"M")))                 /* cursor moved */
      {  /* get mouse position ix2,iy2  */
      getloc(event,'M',&ix2,&iy2);

      if (gotpt)
         /* already have line, mouse moved, so erase and draw new line */
         xorline(iwin,icontmx,FALSE,ix1,iy1,ix2,iy2);
      }
  else if (!(strcmp(*params,"1")))            /* button 1 pushed */ 
      { /* get mouse position ix2,iy2  */
      getloc(event,'M',&ix2,&iy2);

      /* left button, end a line segment of the polygon */
      if (!gotpt)
          {
          gotpt = TRUE;
          ix0     = ix2;
          iy0     = iy2;
          ipts    = 0;
          }
      else
          {   /* draw permanent line to this location */

          /* set to draw in overlay planes */
          wisetplanes(idispl,icontx,128);

          /* draw line and save line in backing store */
          xorline(iwin,   icontx,TRUE,ix1,iy1,ix2,iy2);
          xorline(imagsav,icontx,TRUE,ix1,iy1,ix2,iy2);

          /* set to draw in all planes */
          wisetplanes(idispl,icontx,-1);
          }
      if (ipts == 0)
          {
          if ((points = (XPoint *) calloc(nsize, sizeof(XPoint)))
                      ==(XPoint *) NULL )
              { spout("***  CALLOC() CAN'T ALLOCATE HEAP MEMORY SPACE");
              return; }
          }
      else if (ipts >= nsize)
          {
          nsize += 10;
          if((points = (XPoint *) realloc(points, nsize * sizeof(XPoint)))
                     ==(XPoint *) NULL )                                  {
              spout("*** Unable to realloc points array in corr_mask.c.");
              ipts = nsize - 2;                                           }
          }
      ix1 = ix2;
      iy1 = iy2;
      points[ipts].x   = ix2;
      points[ipts++].y = iy2;
      }
 
  else if (!(strcmp(*params,"2")))          /* button 2 pushed */ 
      { /*   finished with this mask polygon */

      /* clear any stuff in overlay planes (any previous masks) */
      if (masks == 0) 
           { clearover(NULL,NULL,NULL);  XFlush(idispl); }

      if (ipts > 3)
          {
          /* set to draw in the overlay */
          wisetplanes(idispl,icontx,128);

          /* close the mask */
          points[ipts].x   = ix0;
          points[ipts++].y = iy0;

          icoloro = icolor;
          if (filler) 
              /*  set fill color to white (depends on overlut) */
              wicolor(icontx,colorgo+9);
          else
              /* set fill color to black */
              wicolor(icontx,-1);
 
          /* draw the filled mask on the image and
             save it in backing store  */
          XFillPolygon(idispl,iwin,icontx,points,ipts,
                       Complex,CoordModeOrigin);
          XFillPolygon(idispl,imagsav,icontx,points,ipts,
                       Complex,CoordModeOrigin);
 
          /* we may have multiple masks on the screen. but we are only
             interested in the Last one */
          /* create a pixmap for the new mask*/
          msksav = XCreatePixmap(idispl,iwin,iwidex,ihighx,idepth);
          if (msksav <= 0)  
             { spout(" *** WARNING: can not create a pixmap for new mask");
               spout(" the average image will  be wrong !!!!!");
             } 

	  /* clear the mask buffer with black--0, then draw polygon in it */
          wicolor(icontx,-1);
	  XFillRectangle(idispl,msksav,icontx, 0,0,iwidex,ihighx);
          wicolor(icontx,colorgo+9);
          XFillPolygon(idispl,msksav,icontx,points,ipts,
                       Complex,CoordModeOrigin);
 
          /* reset current color */
          wicolor(icontx,icoloro);

          masks++;
          if (points) free(points);
          ipts    = 0;
          points =  (XPoint *) NULL;
          gotpt = FALSE;
          nsize = 10;
          }
      else
          {
          spout(" Not enough points for a MASK !.");
          }

      /* cancel button assignments */  
      XtUninstallTranslations(iw_t);

      /* remove instructional message */
      showbutx("","","",TRUE); 
 
      /* set to draw in all planes */
      wisetplanes(idispl,icontx,-1);

      /* restore default lut */
      /* defluts(TRUE); */   
      setamap(map);

      XFlush(idispl);
   
      /* halt correspondence map masking, get 2nd map menu
         for storing data or averaging images from map */
      comapmen2();          
      }
  else if (!(strcmp(*params,"3")))          /* button 3 pushed */        
      { /* stop the correspondance map */
      /* cancel button assignments */  
      XtUninstallTranslations(iw_t);

      /* remove instructional message */
      showbutx("","","",TRUE);

      /* enable all planes for drawing */
      wisetplanes(idispl,icontx,-1);

      /* restore default lut */
      /* defluts(TRUE); */   
      setamap(map);
      }
  return;
  }
