/*$Header*/

/*
C++*********************************************************************
C
C    CSGET.FOR  -- CREATED DEC 84
C                  REWRITTEN SEPT 86 al
C                  3-D ADAPTED FEB 86 al
C                  ADDED NEW TEXT CONTOURS FEB 88 al
C                  MARKER = 6 ADDED JUNE 88 al
C                  LIMFIL ADDED AUG 88 al
C                  PORTED TO C 1993 al
C **********************************************************************
C *  AUTHOR: A. LEITH                                                  *
C *  COPYRIGHT (C)1985, WADSWORTH CENTER, NEW YORK STATE DEPARTMENT    *
C *    OF HEALTH, ALBANY, NY 12201.                                    *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS *
C *    OR USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN        *
C      APPROVAL OF WADSWORTH CENTER.  			               *
C **********************************************************************
C
C    csget(FILEDATACNT * fileptrcnt, DATACNT * cnt, int icntr, 
C           float * data, int * ndata)
C
C    PURPOSE:  TO READ CONTOUR DATA FROM A CONTOUR STORAGE FILE 
C
C    PARAMETERS:
C        ICNTR          CONTOUR NO. SPECIFIED BY THE CALLER
C        CNT            STRUCTURE CONTAINING CO-ORDINATES OF CONTOURS
C        NDATA          NO. OF POINTS ON THE  CONTOUR 
C
C    VARIABLES:
C        MPT            ISHAP,MARKER, PENTYPE AND LINE TYPE 
C        ZCOO           Z- COORDINATE FOR THE CONTOUR 
C        IRECLN         RECORD LENGTH
C
C   CALLS:              REDLIN
C
C--********************************************************************
*/

#include "filescnt.h"
#include "routines.h"

 /*************************** csget ***********************************/

 int csget(FILEDATACNT * fileptrcnt, DATACNT * cnt, int icntr, 
           float * data, int * ndata)

 {
 union  DUAL 
        {
        float fsjifp;
        int   isjifp;
        } sjifp;

 int           istrec, minrec, istsuf, lstrec, lstsuf, ksj, irep;
 int           icurec, j, irecln, maxsuf, ival; 
 FILEDATA     *fileptrimg;
 float *       buf;
 float *       contrl;

 /* get file related variables for later use */
 fileptrimg = fileptrcnt -> image;
 buf        = fileptrimg -> buf;
 irecln     = fileptrcnt -> irecln;
 contrl     = fileptrcnt -> contrl;

 /* save the contour number and data pointer in cnt structure */
 cnt  -> icntr  = icntr;
 cnt  -> data   = data;
 cnt  -> ndata  = 0;
  
 if (icntr < 1)
    {spout("*** Invalid contour requested"); return FALSE;}

 minrec       = 5 + fileptrcnt -> ndtctl;

 istrec       = contrl[6*(icntr-1) + 0];
 istsuf       = contrl[6*(icntr-1) + 1];
 lstrec       = contrl[6*(icntr-1) + 2];
 lstsuf       = contrl[6*(icntr-1) + 3];
 cnt ->zcoo   = contrl[6*(icntr-1) + 5];
 sjifp.fsjifp = contrl[6*(icntr-1) + 4];

 if (istrec < minrec)
    { spout("*** Deleted contour requested"); return FALSE;}
 
 if ((fileptrcnt -> ifrmcs == -6) && (sjifp.isjifp < 0))
    {    /* new style text contour */
    ksj        = - (sjifp.isjifp);
    /* compute text angle, size, etc */
    cnt -> ipen   = ksj % 10; ksj = ksj / 10;
    cnt -> ifon   = ksj % 10; ksj = ksj / 10;
    cnt -> inten  = ksj % 10; ksj = ksj / 10;
    cnt -> just   = ksj % 10; ksj = ksj / 10;
    cnt -> itsize = ksj % 10; ksj = ksj / 10;
    cnt -> itangl = ksj % 10; 
    cnt -> mark   = -2;
    }
 else
   { /* regular contour */
   ksj           = sjifp.isjifp;
   cnt -> inten  = ksj % 10; ksj = ksj / 10;
   cnt -> lintp  = ksj % 10; ksj = ksj / 10;
   cnt -> ipen   = ksj % 10; ksj = ksj / 10;
   cnt -> mark   = ksj % 10; ksj = ksj / 10;
   cnt -> ishap  = ksj % 10; ksj = ksj / 10;
   cnt -> limfil = ksj % 10;

/*****
printf("inten: %d, lintp: %d ipen: %d, ksj: %f \n", 
cnt->inten, cnt->lintp, cnt->ipen,sjifp.fsjifp);
******/
   }

 cnt -> threed = 
   (fileptrcnt -> ifrmcs  == 4 && (cnt -> mark == 1 || cnt -> mark ==6)); 

 icurec = istrec;
 if (icurec != fileptrcnt -> lsrecd)
    {   /* read the istrec, if not in the buffer */
    if (redlin(fileptrimg,&buf[0],irecln,icurec) != TRUE) 
       {fileptrcnt -> lsrecd = -1; return FALSE; }
    fileptrcnt -> lsrecd = icurec;
    } 

 maxsuf        = irecln;
#ifdef VMS
  /* set used file record length to 1080 (buggy cnt files on VMS)*/
  maxsuf = maxsuf / 4;
#endif

 j             = istsuf-1;
 * ndata       = 0;
 cnt -> ndata  = 0;
 ival          = 0;

 do 
    {
    for (irep = 0; cnt->threed ? irep < 3 : irep < 2; irep++)
       {
       /* read data points from file */
       if (j >= maxsuf)
          {
          /* file record length exceeded, read next file record */
          if (redlin(fileptrimg,&buf[0],irecln,++icurec) != TRUE) 
              {fileptrcnt -> lsrecd = -1; return FALSE; }
          fileptrcnt -> lsrecd = icurec;
          j = 0;
          }

       /* put x data into array */
       if (ival >= cnt -> nsize) 
          {spout("*** Cnt array full, contour truncated"); return TRUE;}

       data[ival++] = buf[j++];
       }
    (cnt -> ndata)++;

    }  while ((j < lstsuf) || (icurec < lstrec));

 * ndata = cnt -> ndata;
 return TRUE;
 }

