
/*
C++*********************************************************************
C
C    csgettext.c
C
C **********************************************************************
C *  AUTHOR:A. LEITH                                                   *
C *  COPYRIGHT (C)1985 , WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.  *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHER  *
C *    OR USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN        *
C *    APPROVAL OF THE CENTER FOR LABORATORIES AND RESEARCH            *
C **********************************************************************
C
C    PURPOSE:  TO READ TEXT FROM A CONTOUR STORAGE FILE 
C
C    PARAMETERS:
C        FILEPTRCNT STRUCTURE FOR CNT FILE          (SENT)
C        CNT       STRUCTURE FOR A CONTOUR          (SENT/ALTERED)
C        ITEXT     THE NO. OF THE TEXT TO BE READ   (SENT)
C        DATA      CHAR. STRING                     (RETURNED)
C
C    RETURNS:    NDATA     NO. OF CHAR. IN STRING         
C
C    VARIABLES:
C        ISJIFP    ANGLE, SIZE, JUSTIFICATION, INTENSITY, FONT, 
C                      AND PEN TYPE
C        CBUF      BUFFER FOR READING TEXT
C        IRECLN    RECORD LENGTH.
C        
C    CALLED BY:    
C
C--********************************************************************
*/

#include "filescnt.h"
#include "routines.h"

 /*********************** csgettext **********************************/

 int csgettext(FILEDATACNT * fileptrcnt, DATACNT * cnt, int itext, 
           char * data)

 {
 union  DUAL 
        {
        float fsjifp;
        int   isjifp;
        } sjifp;

 int           istsuf, lstsuf, ksj, kjust, loc, nsize, ndata;
 int           icurec, j, irecln; 
 FILEDATA     * fileptrimg;
 char  *       cbuf;
 char  *       dataptr;
 float *       txtctr;

 /* get file related variables for later use */
 fileptrimg = fileptrcnt -> image;
 cbuf       = (char *) fileptrimg -> buf;
 irecln     = fileptrcnt -> irecln;
 txtctr     = fileptrcnt -> txtctr;

 /* save the contour number and data pointer in cnt structure */
 cnt  -> itext  = itext;
 cnt  -> ndata  = 0;
  
 if (itext < 1)
    {spout("*** Invalid text requested"); 
    return -1;}

 loc            = 5 * (itext - 1);
 istsuf         = txtctr[loc + 0];
 lstsuf         = txtctr[loc + 1];
 sjifp.fsjifp   = txtctr[loc + 2];
 cnt -> data[1] = txtctr[loc + 3];
 cnt -> data[2] = txtctr[loc + 4];

 if (istsuf <= 0)
    {spout("*** Non existent text requested");
    printf("itext: %d, loc: %d, istsuf: %d, lstsuf: %d", 
            itext,loc,istsuf,lstsuf);
    printf("        txtctr0: %f, txtctr1: %f, txtctr2: %f",
           txtctr[loc+0], txtctr[loc+1], txtctr[loc+2]);
    return -1;
    }
 
 ksj           = sjifp.isjifp;
 /* compute text angle, size, etc */
 cnt -> itangl = ksj /1000000;
 ksj           = ksj - cnt -> itangl * 1000000;
 cnt -> itsize = ksj / 10000;
 ksj           = ksj - cnt -> itsize * 10000;
 cnt -> just   = ksj / 1000;
 /* guard against integer overflow */
 kjust         = cnt -> just;
 kjust         = kjust * 1000;
 ksj           = ksj - kjust;
 cnt -> inten  = ksj / 100;
 ksj           = ksj - cnt -> inten * 100;
 cnt -> ifon   = ksj / 10;
 cnt -> ipen   = ksj - cnt -> ifon * 10;

 /* text is in record four, records are all 4*nsam bytes long! */
 icurec = 4;
 if (icurec  != fileptrcnt -> lsrecd)
    {   /* read the istrec, if not in the buffer */
    if (redlin8(fileptrimg,(unsigned char *) &cbuf[0],irecln*4,icurec)
               != TRUE) 
       {fileptrcnt -> lsrecd = -1; return -1; }
    fileptrcnt -> lsrecd = icurec;
    } 

 /* j is next record position to be read */
 j       = istsuf - 1;
 nsize   = cnt -> nsize;
 /* dataptr points to next position for char data */
 dataptr = data;
 /* ndata is number of characters in text string so far */
 ndata   = 0;


 do 
    {
    /* read data points from file */
    if (j >= (irecln * 4))
       {
       /* file record length exceeded, bad txtctr!! */
       spout("*** Cnt text control bad!");
       return -1;
       }

    if (ndata >= (nsize -1)) 
       {   /* char. array overflow */
       spout("*** Text array full, text truncated"); 
       *dataptr = '\0'; 
       cnt -> ndata  = ndata;
       return ndata;
       }

    /* put char. into text array */
    if (cbuf[j] != '\0')
      {  /* skip enbedded nulls in text strings */
      *dataptr = cbuf[j];
      dataptr++;
      ndata++;
      }
    j++;

    }  while ((j < lstsuf));


    /* set number of char in string */
    cnt -> ndata  = ndata;
    *dataptr      = '\0';

 return ndata;
 }



