
/*
C**********************************************************************
C
C  dendromen.c -- ADAPTED FROM DENDRO2.FOR 27 JUNE 94 Lekan Odesanya
C
C**********************************************************************
C FORMS A TRUNCATED SCREEN PLOT FOR THE DENDROGRAM
C
C BASED ON ARBRE.FOR,    A DENDROGRAM PGM BY - JEAN-PIERRE BRETAUDIERE 
C THE UNIVERSITY OF TEXAS HEALTH SCIENCE CENTER AT HOUSTON                      
C MEDICAL SCHOOL - DEPARTMENT OF PATHOLOGY AND LABORATORY MEDICINE              
C P.O. BOX 20708, HOUSTON, TX 77225.                                            
C*--------------------------------------------------------------------*         
*/      

/* common variables */
#include "x.h"
#include "common.h"
#include "routines.h"
#include <Xm/Text.h>
#include <Xm/ToggleBG.h>

 /* external global variables */
 extern int    maxpart;

 /* internal function prototypes */
 void      dendromen_buta(Widget iw_temp, caddr_t, caddr_t);
 void      dendro_pop(Widget, XEvent *, String *, Cardinal *);
 void      dendromen_d(Widget, caddr_t, caddr_t);

 /* internal common variables  */
 int          * NUM,  newdoc, wantshow = FALSE;
 float        * V, * YT, *buf, vmin, vmax;
 char         * fname = NULL, * cbuf, filpat[81];
 static Widget  iw_dendromen= (Widget)0, iw_thlev= (Widget)0, 
                iw_patt= (Widget)0, iw_lab= (Widget)0, iw_rc= (Widget)0,
		iw_rowcolv= (Widget)0, iw_show= (Widget)0;

void dendromen( char * dfil)
{
 char     cval[50], * cdum;
 int      maxreg, maxkeys;
 int      k, doc_read;
 float  * dbuf = NULL, *ptr;
 float    plist[8], val;
 Widget   iw_dums, iw_pusha, iw_pushc;

/* set image display range */
imagego  = mapgo;
imageend = 127;
  
 /* set size for array pointed to by dbuf */
 maxreg    = 2+1;
 maxkeys   = 9999;

 /* tell it to open the doc files each time */
 doc_read = TRUE;

 if (unsdal(dfil, &doc_read, 0, plist, 2, &dbuf, &maxpart,
           maxkeys, maxreg) > 0) 
    {   /* problem retrieving doc file, assume it does not exist */
    if (dbuf) free(dbuf); dbuf = NULL;
    return;
    }

 /* allocate space for  arrays */
 if (((V = (float *) realloc((void *) V, maxpart * sizeof(float))) == 
               (float *) NULL) ||
     ((NUM = (int *) realloc((void *) NUM, maxpart * sizeof(int))) == 
               (int *) NULL) ||
     ((YT = (float *) realloc((void *) YT, maxpart * sizeof(float))) == 
               (float *) NULL))
   { 
     spout("*** Unable to reallocate memory in dendromen."); 
     return; 
   }   

 newdoc = TRUE;
 ptr = dbuf;
 
 NUM[0]  = (int) *(ptr+1);
 V[0]    = vmin = vmax = val = *(ptr+2);
 ptr  += maxreg; 
 
 for (k=1; k<maxpart; k++)
    {
    NUM[k]  = (int) *(ptr+1);
    V[k]    = val = *(ptr+2);
      
    if( val < vmin ) vmin = val;
    if( val > vmax ) vmax = val;
     
    ptr  += maxreg; 
    }

/* strip pathname from filename  */
#ifdef VMS
     fname = strrchr(dfil,']');
#else
     fname = strrchr(dfil,'/');
#endif

 /* free dbuf */
 if (dbuf) free(dbuf); dbuf = NULL;

 if (iw_dendromen <= (Widget)0)
    {   /* create surface menu widget first */

    iw_dendromen  = wid_dialog(iw_win, 0, "Dendrogram Threshold", -1, -1);
    iw_rowcolv  = wid_rowcol(iw_dendromen, 'v', -1, -1);

    /* create label box for threshold range ---------------------  range  */
    sprintf(cval,"Thresh Range:  %f ...%f",vmin,vmax);
    iw_lab  = wid_labelg(iw_rowcolv, 0, cval, -1, -1);

    /* create horizontal rowcol for threshold and scaling ---------------  */
    iw_rc  = wid_rowcol(iw_rowcolv, 'h', -1, -1);

    /* create text box  for threshold --------------------  threshold */
    /* SET DEFAULT THRESHOLD TO 50% OF RANGE */
    sprintf(cval,"%f",(vmin+vmax)/2);
    iw_thlev = wid_textboxb(iw_rc,0,"Threshold:",cval,10);

   /* create toggle box for wantshow ---------------------------- wantshow */
    iw_show = wid_toggleg(iw_rowcolv, iw_show, "Show Averaged Images",
           wantshow,dendromen_d,cdum,-1,-1);

   /* create text box for file pattern input  -------------- filpat */
    iw_patt = wid_textboxb(iw_rowcolv,iw_patt,"Template name of images:",filpat,12);
    if (!wantshow) XtUnmanageChild(XtParent(iw_patt));

    /* create box for apply ----------------------------------- apply */
    iw_dums = wid_stdbut(iw_rowcolv, iw_dendromen, 
                        &iw_dums, &iw_pushc, &iw_pusha,  "CA",
                        fin_cb, fin_cb, dendromen_buta, NULL);
   }
 else
   {   /* dendrogram menu widget already exists, update it */

    /* create label box for thresh range ---------------------  range  */
    sprintf(cval,"Range:     %f ...%f",vmin,vmax);
    iw_lab  = wid_labelg(iw_rowcolv, iw_lab, cval, -1, -1);

    }

 XtManageChild(iw_dendromen);

}

 /************** dendromen_d (wantshow toggle callback) *************/

 void dendromen_d(Widget iw_temp, caddr_t data, caddr_t call_data)
  { 
  int showing;

  showing  = wantshow;
  wantshow = XmToggleButtonGadgetGetState(iw_show);

  if (wantshow && !showing)
     XtManageChild(XtParent(iw_patt));

  else if (!wantshow && showing)
     XtUnmanageChild(XtParent(iw_patt));  
  } 

 /*********** accept button callback **********************************/

 void dendromen_buta(Widget iw_temp, caddr_t data, caddr_t call_data)
 {
 XImage  *image;
 char    *string, str[5], filpatt[30];
 int      npix, ival, irect, nsr;
 float    fmin, fmax, scal, fm;
 float    pmin, xdis, yscale, vdiv;
 float    ypos, ylabel, ydiv, buffer[256];
 int      ii, x0, y0, x1, y1, x2, y2, nkla, obj;
 int      i, j, k, yti, ytj, ht, wt, jgo, iform;

 /* find threshold */
 string = XmTextGetString(iw_thlev);
 sscanf(string,"%f",&pmin);
 if (pmin > vmax) pmin = vmax;
 if (pmin < vmin) pmin = vmin;
 if (string) free(string);

 if (wantshow) 
    {
    strcpy(filpat,XmTextGetString(iw_patt));
    if (strlen(filpat) == 0) wantshow = FALSE;
    } 

 XtUnmanageChild(iw_dendromen);

 /* clear the screen be4 next display */
 webclear( NULL, NULL, NULL); 

 /* make clock cursor (#79) */
 waitcursor(iwin,TRUE,79);
  
if (wantshow)
 {
  strcpy(filpatt,filpat);
  filget(filpat,filpat,81,NUM[0]);
  /*  Add datexc (name extension ) to filpat if not already there  */
  if (strstr(filpat,datexc) == 0)
      {
      if ((strlen(filpat) + strlen(datexc) + 1) > 80)
         { spout("*** File name too long:"); spout(filpat); 
         return; }  
      strcat(strcat(filpat,"."),datexc);
      }
  /* open the first one to get the dimensions of the images */
  filedata = openold(filpat,&nsam,&nrow,&nslice,&iform,"o"); 
  if (filedata == (void *)FALSE)
     {
     spout("*** Unable to open image file");
     waitcursor(iwin,FALSE,0);
     return;
     }
  closefile(filedata);

  nsr =  nsam * nrow;
 /* allocate space for image arrays */
  if(newdoc)
   {
     newdoc = FALSE;
     if (((buf = (float *) realloc((void *) buf, nsr * sizeof(float))) == 
               (float *) NULL) ||
      ((cbuf = (char *) realloc((void *) cbuf, nsr * sizeof(char))) == 
               (char *) NULL))
     { 
      spout("*** Unable to reallocate memory in dendromen_buta."); 
      waitcursor(iwin,FALSE,0);
      return; 
     }   
   }
   /* initialize buf to zero  */
   for(i = 0; i < nsr; i++)
     *(buf+i) = 0.0;
 }

  y0 = ht = ihighx - 150; wt = iwidex - 90; 
  /* find y-scale for tree  */
  yscale = (float) ht / (vmax - pmin); 

  /* draw and label y-axis lines   */
  XDrawLine(idispl, iwin, icontx, 5, y0+20, 5, 20);
  XDrawLine(idispl, imagsav, icontx, 5, y0+20, 5, 20);

  ylabel = pmin;  ypos = y0;
  vdiv = (vmax-pmin)/5.0;  ydiv = (0.0 - y0)/5.0;
  for (i = 0; i <= 5; i++)
   {
     XDrawLine(idispl, iwin, icontx, 5, ypos+20, 10, ypos+20);
     XDrawLine(idispl, imagsav, icontx, 5, ypos+20, 10, ypos+20);
     sprintf(str,"%4.2f",ylabel);
     witext(icontx, str, 10, ypos+23, 0, 0, -9, 2, FALSE);
     ylabel += vdiv;   ypos += ydiv;
   }

  nkla = 0;
  for( j = 0; j < maxpart; j++)
    {
      if (V[j] < pmin) continue;              /* do not plot truncated lines */
      YT[nkla++]   = (V[j] - pmin) * yscale;  /* find node height (YT)  */
    }

  /* find horizontal distance between leaves  */
  xdis = (float) wt / (nkla - 1.0);

   /* put filename at the top */
   if( fname );
     witext(icontx, fname + 1, iwidex/2 - 15, 15, 0, 0, -9, 2, FALSE);
      
  obj = 1; jgo = 0;

  /* start i sweep  */
  for( i = 0; i < nkla; i++)
   {
     yti     = YT[i];
     x0 = x1 =  i * xdis + 70;  y1 = ht - yti + 20;
     /* draw vertical line to (X(i),yti)  */
     XDrawLine(idispl, iwin, icontx, x0, y0+20, x1, y1);
     XDrawLine(idispl, imagsav, icontx, x0, y0+20, x1, y1);

     /* determine and write #. of objects for the tree node   */
     for(j = jgo; j < maxpart; j++)
      {
        if (V[j] < pmin ) 
            obj++;
        else
         {
            if( obj <= 9) 
              sprintf(str,"%1d",obj);
            else
             {
              if( obj > 99)
                sprintf(str,"%3d",obj);
              else
                sprintf(str,"%2d",obj);
             }
            witext(icontx, str, x0-15, y0+35, 0, 0, -9, 2, FALSE);
            obj = 1; 
 
            if ( wantshow )
              {              
              for(k = jgo; k <= j; k++) /* add the images */
                 {
                 strcpy(filpat,filpatt);
                 filget(filpat,filpat,81,NUM[k]);
 
                 /*  Add datexc (name extension ) to filpat if needed  */
                 if (strstr(filpat,datexc) == 0)
                   {
                   if ((strlen(filpat) + strlen(datexc) + 1) > 80)
                    { spout("*** File name too long:"); spout(filpat); 
                      return; }  
                   strcat(strcat(filpat,"."),datexc);
                   }

           /* try to open the file */
                filedata = openold(filpat,&nsam,&nrow,&nslice,&iform,"o"); 
                if (filedata == (void *)FALSE)
                 {
                  spout("*** Unable to open image file");
                  waitcursor(iwin,FALSE,0);
                  return;
                 }

                npix = 0;  
                for (irect = 1; irect <= nrow; irect++)
                 {
                  if (redlin(filedata,buffer,nsam,irect) != TRUE) 
                   { free(cbuf); free(buf); 
                      spout("redlin returned FALSE");
                     waitcursor(iwin,FALSE,0); return; }
                   for(ii = 0; ii < nsam; ii++)
                    buf[npix++] += buffer[ii];
                 }

                 closefile(filedata);
               }  /*  k  */


              /* determine max and min of all images */
              fmin = fmax = buf[0] ;
              for(npix = 1; npix < nsr; npix++)
                 {
                 if(buf[npix] < fmin) fmin = buf[npix];
		 if(buf[npix] > fmax) fmax = buf[npix]; 
                 }
		 
              /* CONVERSION FACTOR FROM FLOATING POINT TO LOGICAL * 1 RANGE */
              scal = (float) (imageend - imagego) / (fmax - fmin);
              fm   = -fmin * scal + imagego + 0.5;

              for(npix = 0; npix < nsr; npix++)
               {
                if (imageend < 128) 
                 {   /* no need to convert numbers > 128 */   
                    ival = (*(buf + npix) * scal) + fm;
                    *(cbuf + npix) = ival;
                 }
                else
                 {   /* need to convert numbers > 128 */   
                     ival = (*(buf + npix) * scal) + fm;
                   /* convert 0..255  integers to -128...127 */
                     if (ival > 127) ival = ival - 256;
                    *(cbuf + npix) = ival;
                 }
              }

               /* display the image */
               wipic(icontx,cbuf,x0-nsam+15,y0+40,nsam,nrow,
                     TRUE,FALSE,&image);

               /* initialize buf to zero  */
               for(ii = 0; ii < nsr; ii++)
                *(buf+ii) = 0.0;

              } /* wantshow  */
             jgo = j + 1;  break;
         } /* else  */
      }  /*  for(j = jgo  */                               

     if (i == (nkla-1) ) break;  

   /* find length of horizontal line   */
   for( j = i+1; j < nkla; j++)
    {   
      ytj   = YT[j];
      if (ytj >= yti) 
       {
         /* draw horizontal line to (X(j),yti)  */
         x2  =  j * xdis + 70;  y2 = ht - yti + 20;
         XDrawLine(idispl, iwin, icontx, x1, y1, x2, y2);
         XDrawLine(idispl, imagsav, icontx, x1, y1, x2, y2);
         break;
       }
    } /* for(j = i+1 */

  }  /*  i  */

 /*  remove clock cursor */
 waitcursor(iwin,FALSE,0);

 /* display menu */
 showbutx("Recalls Menu.", 
          "Recalls Menu.",
          "Stops dendrogram.",FALSE); 

 /* define buttons for menu retrieval or stop */
 actions(iw_win, "dendro_pop", dendro_pop, "123");

 }

 /*********** dendro_pop **********************************/

 void dendro_pop(Widget iw_t, XEvent *event, String *params,
                   Cardinal *num_params)
 {

 if (!(strcmp(*params,"3")))            /* button 3 pushed */ 
    {

    /* uninstall translations */
    XtUninstallTranslations(iw_t);
   /* unmanage information box */
    showbutx("","","",TRUE);
    }

 else                                  /* button 1 or 2 pushed */        
    {                                       
     XtManageChild(iw_dendromen);
    }

 }
