
/*$Header: /net/ithaca/usr1/web/src/RCS/filtere.c,v 1.7 1995/03/15 15:31:26 liy Exp $*/

/***********************************************************************
 *
 * filtere.c
 *
 ***********************************************************************
 * author : ArDean Leith
 * COPYRIGHT (C)1993, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 ***********************************************************************
 *
 * filtere()
 * 
 * PURPOSE:    convolves an floating point image with a kernal 
 *             performing an erosion or dilation at specified
 *             connectivity and threshold.
 *
 * RETURNS:    true or false
 *
 * PARAMETERS: 
 *
 ***********************************************************************
*/

#include "std.h"
#ifndef VMS
#include "float.h"
#endif

 /******************************  filtere **************************/

 int filtere(float * oldimage, int nsamt, int nrowt, 
             int erode, int coeff, float threshold, 
             float * newimage, float * fmint, float * fmaxt)
 {
 int     ix, iy,   nrowkd2, nsamkd2, it;
 float   oldval;
 int     icon3, icon4;
 int     ip, hit;
 float * endimage;
 float   fmino,fmaxo;

 nrowkd2 = 1;
 nsamkd2 = 1;
 fmino   = *fmint;
 fmaxo   = *fmaxt;

 *fmint = FLT_MAX;
 *fmaxt = FLT_MIN;

 /* convolve over all positions in the new image except border */

 for (iy = 1; iy < nrowt - 1; iy++)
    {
    icon3 = iy * nsamt;

    for (ix = 1; ix < nsamt - 1; ix++)
       {
       icon4 = icon3 + ix;
       oldval = oldimage[icon4];

       if (erode  && (oldval < threshold)) 
          {
          newimage[icon4] = fmino;
          *fmaxt          = MYMAX(*fmaxt,fmino);
          *fmint          = MYMIN(*fmint,fmino);
          }
  
       else if (!erode && (oldval >= threshold))
          {
          newimage[icon4] = oldval;
          *fmaxt          = MYMAX(*fmaxt,oldval);
          *fmint          = MYMIN(*fmint,oldval);
          }

       else
          {
          hit   = 0;
          /* convolve over all positions in the kernal */
          if (oldimage[icon4 - 1        ] >= threshold) hit++;
          if (oldimage[icon4 - nsamt - 1] >= threshold) hit++;
          if (oldimage[icon4 - nsamt    ] >= threshold) hit++;
          if (oldimage[icon4 - nsamt + 1] >= threshold) hit++;
          if (oldimage[icon4 + 1        ] >= threshold) hit++;
          if (oldimage[icon4 + nsamt + 1] >= threshold) hit++;
          if (oldimage[icon4 + nsamt    ] >= threshold) hit++;
          if (oldimage[icon4 + nsamt - 1] >= threshold) hit++;

          if (erode  && (hit > coeff))
             {
             newimage[icon4] = oldval;
             *fmaxt          = MYMAX(*fmaxt,oldval);
             *fmint          = MYMIN(*fmint,oldval);
             }
          else if (!erode && (hit > coeff))
             {
             newimage[icon4] = fmaxo;
             *fmaxt          = MYMAX(*fmaxt,fmaxo);
             *fmint          = MYMIN(*fmint,fmaxo);
             }
          else
             {
             newimage[icon4] = fmino;
             *fmaxt          = MYMAX(*fmaxt,fmino);
             *fmint          = MYMIN(*fmint,fmino);
             }

          }
        }
     }

  /* fill in border with minimum value */

  endimage = newimage + nsamt * nrowt - 1;

  /* fill top and bottem borders */
  for(ip = 0; ip < nsamt * nrowkd2; ip++)
     {
     *(newimage + ip) = fmino;
     *(endimage - ip) = fmino;
     }

  /* fill left and right borders */
   for(it = 0; it < nsamkd2; it++)
     {    
     for(ip = it; ip < nsamt*nrowt; ip=ip+nsamt)
        {
        *(newimage + ip) = fmino; 
        *(endimage - ip) = fmino;
        }
     }

  return TRUE;
}


