
/*$Header: /net/ithaca/usr1/web/src/RCS/filterfill.c,v 1.4 1996/07/31 19:51:18 leith Exp $*/

/***********************************************************************
 *
 * filterfill.c
 *
 ***********************************************************************
 * author : ArDean Leith
 * COPYRIGHT (C)1993, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 ***********************************************************************
 *
 * filterfill()
 * 
 * PURPOSE:    seed fill
 *
 * RETURNS:    true or false
 *
 * PARAMETERS: 
 *
 ***********************************************************************
*/

#include "std.h"
#ifndef VMS
#include "float.h"
#endif
#include "routines.h"
 
 void flood (int);

 static int     nsamtt, connect8;
 static float   thresht, fvalt;
 static float * newimaget;
 static float * oldimaget;
 static int   * visited;

 /**************************  filterfill *****************************/

 int filterfill(float * oldimage, int nsamt, int nrowt, 
             int ixseed, int iyseed, float thresh, float fval, 
             float * newimage, float * fmint, float * fmaxt,
             int connectivity)
 {
 int     nrowkd2, nsamkd2;
 float * endimage;
 int   * endimagei;
 float * fptr;
 int     ip, it, iloc;

 /* set border size */
 nrowkd2 = 1;
 nsamkd2 = 1;

 thresht   = thresh;
 fvalt     = fval;
 newimaget = newimage;
 oldimaget = oldimage;
 nsamtt    = nsamt;
 connect8  = TRUE;
 if (connectivity == 4) connect8 = FALSE;

 /* copy oldimage to newimage */
 memcpy((void *) newimage,(void *) oldimage, nsamt*nrowt*4);
 
 /* allocate visited space */
 if ((visited = (int *) calloc(nsamt * nrowt ,
                      sizeof(int))) == (int *) 0)
    {spout("*** Unable to allocate visited in filterfill."); 
     return FALSE;}

 /* fill top and bottem borders of visited to prevent going out */
 /* borders of the image */

 endimagei = visited + nsamt * nrowt - 1;

 for(ip = 0; ip < nsamt * nrowkd2; ip++)
     {
     *(visited   + ip) = TRUE;
     *(endimagei - ip) = TRUE;
     }

 /* fill left and right borders */
 for(it = 0; it < nsamkd2; it++)
     {    
     for(ip = it; ip < nsamt*nrowt; ip=ip+nsamt)
        {
        *(visited   + ip) = TRUE; 
        *(endimagei - ip) = TRUE;
        }
     }

 /* do a flood fill around seed location */

 iloc = (iyseed-1) * nrowt + ixseed - 1;
 flood(iloc);

 /* fill in border with fval value */
 endimage = newimage + nsamt * nrowt - 1;

 /* fill top and bottem borders */
 for(ip = 0; ip < nsamt * nrowkd2; ip++)
     {
     *(newimage + ip) = fvalt;
     *(endimage - ip) = fvalt;
     }

 /* fill left and right borders */
 for(it = 0; it < nsamkd2; it++)
     {    
     for(ip = it; ip < nsamt*nrowt; ip=ip+nsamt)
        {
        *(newimage + ip) = fvalt; 
        *(endimage - ip) = fvalt;
        }
     }

 /* check min and max on final image */
 for (fptr= newimage; fptr < newimage + nsamt * nrowt; fptr++)
      {
      if (*fptr < *fmint) *fmint = *fptr;
      if (*fptr > *fmaxt) *fmaxt = *fptr;
      }

 if (visited) 
     {free(visited); visited = NULL; }

 return TRUE;
 }

 /**************************  flood *****************************/

 void flood (int iloc)
 {

 if (!visited[iloc] && oldimaget[iloc] < thresht)
    {
    /* set pixel to visited */
    visited[iloc] = TRUE;

    /* set this pixel to fval */
    newimaget[iloc]  = fvalt;

    /* call flood on all 8 bordering pixels */
    flood(iloc - 1     );
    flood(iloc - nsamtt);
    flood(iloc + 1     );
    flood(iloc + nsamtt);

    if (connect8)
       {   /* use 8 fold connectivity */
       flood(iloc - nsamtt - 1);
       flood(iloc - nsamtt + 1);
       flood(iloc + nsamtt + 1);
       flood(iloc + nsamtt - 1);
       }
    }
 else
    {
    /* set pixel to visited */
    visited[iloc] = TRUE;
    }
 return;
 }


