
/*$Header: /net/ithaca/usr1/web/src/RCS/filterv.c,v 1.5 1995/03/15 15:31:30 liy Exp $*/

/***********************************************************************
 *
 * filterv.c
 *
 ***********************************************************************
 * author : ArDean Leith
 * COPYRIGHT (C)1993, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 ***********************************************************************
 *
 * filterv()
 * 
 * PURPOSE:    convolves an floating point image with a kernal.
 *
 * RETURNS:    true or false
 *
 * PARAMETERS: 
 *
 ***********************************************************************
*/

#include "std.h"
#ifndef VMS
#include "float.h"
#endif

 /* external function prototypes */
 extern void     spout       (char *);

 /* external common variables */
 

 /*************************  filterv   *************************/

 int filterv(float * oldimage, int nsamt, int nrowt, 
            float * kernal,   int nsamk, int nrowk, 
            float * newimage, float * fmint, float * fmaxt)
 {
 int     ix, iy,  ixk, iyk, nrowkd2, nsamkd2, it;
 float   val, ksum, oldval, valt;
 int     icon1, icon2, icon3, icon4, icon5;
 int     ip;
 float * pf;
 float * endimage;

 if (nsamk % 2 == 0 || nrowk % 2 == 0)
    {
    spout("*** Filter kernal size must be odd!");
    return FALSE;
    }

 nrowkd2 = nrowk / 2;
 nsamkd2 = nsamk / 2;

 /* sum up kernal factors */
 ksum = 0.0;
 for (pf = kernal; pf < kernal + nsamk * nrowk; pf++)
    ksum += *pf;

/*******************
 printf("ksum: %f \n",ksum);
********/

 *fmint = FLT_MAX;
 *fmaxt = FLT_MIN;

 /* convolve over all positions in the new image except border */

 for (iy = nrowkd2; iy < nrowt - nrowkd2; iy++)
    {
    icon5 = iy * nsamt;
    icon3 = iy - nrowkd2;

    for (ix = nsamkd2; ix < nsamt - nsamkd2; ix++)
       {
       icon2  = ix - nsamkd2;
       oldval = oldimage[icon5 + ix];
       val    = 0.0;

       /* convolve over all positions in the kernal that are wanted */
       for (iyk = 0; iyk < nrowk; iyk++)
          {
          icon1    = iyk * nrowk;
          /* iyo   = icon3 + iyk; */
          /* icon4 = icon2 + iyo * nrowt; */
          icon4    = icon2 + (icon3 + iyk) * nsamt;

          for (ixk = 0; ixk < nsamk; ixk++)
             {
             /* get current kernal value     */
             /* valk = kernal[icon1 + ixk];  */
             /* ixo = ix - nsamkd2 + ixk;    */
             /* get current image value      */
             /* vali = oldimage[icon4 + ixo];*/

             if (kernal[icon1 + ixk] >= 1.0)
                { 
                valt = oldimage[icon4 + ixk] - oldval;
                val += valt * valt;
                }
             }
          }
       newimage[icon5 + ix] = val;

       *fmaxt = MYMAX(*fmaxt,val);
       *fmint = MYMIN(*fmint,val);
       }
     }

  /* fill in border with minimum value */

  endimage = newimage + nsamt * nrowt - 1;

  /* fill top and bottem borders */
  for(ip = 0; ip < nsamt * nrowkd2; ip++)
     {
     *(newimage + ip) = *fmint;
     *(endimage - ip) = *fmint;
     }

  /* fill left and right borders */
   for(it = 0; it < nsamkd2; it++)
     {    
     for(ip = it; ip < nsamt*nrowt; ip=ip+nsamt)
        {
        *(newimage + ip) = *fmint; 
        *(endimage - ip) = *fmint;
        }
     }

  return TRUE;
}


