
/*$Header: /alwyn/usr1/web/RCS/his.c,v 1.1 93/11/10 10:11:39 leith Exp $*/

/*
C++************************************************************************
C
C  WID_HIS.FOR 
C  his.c                    NOV 92 al
C
C **************************************************************************
C *  SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK            *
C *  COPYRIGHT (C)1981, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    HIS(filedata,jhist)
C
C    PURPOSE:    COMPUTE NHIST PLACE HISTOGRAM FROM IMAGE RECORDS, STORE 
C                HISTOGRAM IN THE HISTOGRAM RECORDS
C
C    PARAMETERS:  filedata      file info
C                 jhist         histogram data
C
C--*******************************************************************
*/

#include "files.h"
#include "common.h"

#define  NHIST 128

 extern int   redlin     (FILEDATA *, float *, int, int);
 extern void  spout      (char *);
 extern int   closefile  (FILEDATA *);
 extern int   setprm     (FILEDATA *, char *);
 extern int   rhist      (FILEDATA *, int *);
 extern int   whist      (FILEDATA *, int *);

/**************************  his   ***************************/

 int his(FILEDATA *filedata, int * jhist)

 {
 float *   buf;
 char      outstr[80];
 float     hdiff, ff, b, av, hmin, y, sig, tmode, binsiz;
 int       i, k, j, jhismax, nhist, irec;
 int   *   iptr;

 buf = filedata -> buf;

 if (filedata -> iform <= 0 || filedata -> iform > 3) 
    {
    /* can not do histogram on fourier files */
    spout("*** can not do histogram on this file format");
    return FALSE;
    }

 else if (filedata -> ihist == 1) 
    {
    /* can get histogram from histogram records for the image */
    if (! rhist(filedata, jhist)) return FALSE;

    hdiff = filedata -> fmax - filedata -> fmin;
    hmin  = filedata -> fmin;

    sig   = filedata -> sig;
    av    = filedata -> av;
     }
 else
    {
    /*  must calculate histogram from image data */

    hdiff = filedata -> fmax - filedata -> fmin;
    hmin  = filedata -> fmin;
    ff    = (NHIST - 1) / hdiff;

    y     = 0.0;
    av    = 0.0;

    /*  zero the histogram */
    for ( iptr = jhist; iptr < jhist + NHIST; iptr++)
       *iptr = 0;

    for (irec = 1; irec <= nrow * nslice; irec++)
       {
       redlin(filedata, buf, nsam, irec);

       for (k = 0; k < nsam; k++)
          {
          b  = buf[k];

          av = av + b;
          y  = y + b * b;

          /* j is histogram bin */
          j = (b - hmin) * ff + 1.5;

          if (j < 0) 
             /*  off histogram at bottem */
             (*jhist)++;

          else if (j >= NHIST) 
             /* off histogram at top */
             (*(jhist + NHIST-1))++;

          else
             (*(jhist+j))++;
          }
       }

    /* update image statistics in the header of the file */
    av  = av / (float) (nsam * nrow * nslice);
    sig = sqrt((y - av * av) / (float) (nsam * nrow * nslice));

    filedata -> sig   = sig;
    filedata -> av    = av;
    filedata -> ihist = 1;

    /* write histogram records into image file and set ihist to 1 */
    whist(filedata, jhist);

    setprm(filedata, "u");
    }

 /* find maximum frequency occuring in histogram (jhismax) and 
    location of maximum frequency (j) */

 jhismax = 0;
 j       = 0;
 for (iptr = jhist; iptr < jhist + NHIST; iptr++)
    {
    if (*iptr >= jhismax) 
      {
      jhismax = *iptr;
      j  = (int) (iptr - jhist);
      }
    }


 /* convert j to corresponding image intensity */
 binsiz = hdiff / (NHIST - 1);
 nhist  = NHIST;
 tmode  = hmin + j * binsiz;

 sprintf(outstr,"range: %f ... %f    bins: %d,   binsize: %f",
                      fmin,   fmax,     nhist,       binsiz);
 spout(outstr);

 sprintf(outstr,"mean: %f   mode: %f    sd: %f", av, tmode, sig);
 spout(outstr);

 closefile(filedata);
 return TRUE;
 }

