
/*$Header: /net/ithaca/usr1/web/src/RCS/imagemen.c,v 1.63 1998/02/05 22:20:48 liy Exp $*/

/*
C++*************************************************************************
C
C imagemen.c          Nov  92                                                                    
C **************************************************************************
C *  AUTHOR: A. LEITH                                                      *
C *  COPYRIGHT (C)1992, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH	        		   *
C **************************************************************************
C
C    IMAGEMEN
C
C    PARAMETERS:   iw_temp, data, call_data (usual callback data)
C
C    PURPOSE:      MAKES A FILE WIDGET, INTERROGATES FILE WIDGET AND
C                  CALLS IMAGE DISPLAY ROUTINE
C
C    CALLED BY:    web_com
C
C--********************************************************************
*/

/* common variables */
#include <Xm/PushBG.h>
#include <Xm/FileSB.h>

#ifdef VMS
#include <unixlib.h>
#include <unixio.h>
#else
#include <unistd.h>
#endif

#include "common.h"
#include "routines.h"

 extern void dendromen( char * );

 /* internal function  prototypes */
 void       imagemen_cb(Widget, XtPointer, XtPointer);
 void       imagemennol_cb(Widget, XtPointer, XtPointer);
 void       imagemen_cb_com(void);

 /* externally defined common variables */
 extern char    outstr[80];
 extern float   erodethresh;  /* used in filtermen, erodemen */

 /* common variables used elsewhere */
 FILEDATA*  filedatal = NULL; /* used by backmen, pickback, pickmen */
 FILEDATA*  filedatar = NULL; /* used by backmen, pickback, pickmen */
 float*     fimage = NULL;    /* used by filtermen also */
 char       filcolvol[81];    /* name for color vol. file */

 char*       cptrduml = NULL; /* data used by image right */
 char*       cptrdumr = NULL; /* and left */
 int         icattyp;

 /* file scope variables used here */
 static char   filnew[128];
 static int    imnum = 1;
 static char   ops[] = "123456789abcdefghijk";
 static char * prompts[] = 
      {
      "Image",                      "First image",   
      "Volume",                     "Histogram",
      "Image for Marker Picking",   "Image for Masking",
      "Image for Profiling",        "Untilted Image",
      "Image for Annotating",       "Image for Convolution",
      "Image for Categorizing",     "Image with Reflections",
      "Recorded image",             "Dendrogram Doc. File",
      "Volume", 		    "Movie montage",
      "Volume",                     " ",
      "Drill-n-fill Volume",        "Montage Doc. File"
      };
 /* Note: i is shift which does not get a prompt */


/*************************  imagemen  ********************************/

 void imagemen(Widget iw_temp, XtPointer data, XtPointer call_data)
 { 

 char   *oper;
 char   pattern[86];
 char   *filename;
 char   ctemp[3];

 oper = data;

 if (filwant[0] != '\0')
    { /* use new file pattern */
    strcpy(&pattern[0],&filwant[0]);

    if (datexc[0] != '\0')
       {
       /* set default file filter */
       strcat(pattern,".");
       strcat(pattern,&datexc[0]);
       }
    filwant[0] = '\0';
    }
 else
    pattern[0] = '\0';

 /* convert first oper character to integer */
 ioper = (int) (strchr(ops,*oper) - ops) + 1;
 if (ioper == 11 && strlen(oper) > 1)
    { /* categorize has two types */ 
    strncpy(ctemp, (char*)data, 2); ctemp[2]= '\0';
    icattyp = atoi(&ctemp[1]);
    }

 /* set name for file */
 filename = &filnow[0];

 if (ioper == 8 && imnum == 2)
    {      /*************************** interactive particle picking */
    if (!listfi) 
       {
       iw_filesnol = wid_text(iw_win, iw_filesnol, "Tilted image",
                    filename, *imagemennol_cb, NULL, -1, -1);
       }
    else
       {
       /* create a file selector box */
       iw_files =  wid_file(iw_win, iw_files , "Tilted image",
               filename, &pattern[0], *imagemen_cb, -1, -1);
       }
    } 

 else if (ioper == 17  && imnum == 2)
    {        /**************************** surface color texture map */
    filename = &filcolvol[0];
    if (!listfi) 
       {
       iw_filesnol = wid_text(iw_win, iw_filesnol, "Color volume",
                    filename, *imagemennol_cb, NULL, -1, -1);
       }
    else
       {
       /* create a file selector box */
       iw_files =  wid_file(iw_win, iw_files , "Color volume",
               filename, &pattern[0], *imagemen_cb, -1, -1);
       }
    }

 else if (ioper == 18)
    {	 /********************************************* Shift image */
    if (imagel)
	shift(IMAGE);
    else if (islice > 1) 
	spout("*** Shift doesn't work for 3D image.");
    else 
	spout("*** Current Image does not exist.");
    } 

 else if (ioper == 11 && icattyp == 1)
    {   /********************************* categorize from doc file */
    if (!listfi) 
       {
       iw_filesnol = wid_text(iw_win, iw_filesnol, prompts[19],
                    filename, *imagemennol_cb, NULL, -1, -1);
       }
    else
       {
       /* create a file selector box */
       iw_files =  wid_file(iw_win, iw_files , prompts[19],
               filename, &pattern[0], *imagemen_cb, -1, -1);
       }
    }
 else
    {   /************************************* all other operations */
    if (!listfi) 
       {
       iw_filesnol = wid_text(iw_win, iw_filesnol, prompts[ioper-1],
                    filename, *imagemennol_cb, NULL, -1, -1);
       }
    else
       {
       /* create a file selector box */
       iw_files =  wid_file(iw_win, iw_files , prompts[ioper-1],
               filename, &pattern[0], *imagemen_cb, -1, -1);
       }
    }
 }



/******************** imagemen_cb ************************************/

 void imagemen_cb(Widget iw_temp, XtPointer data,  
                                  XtPointer call_data)
  { 
  char       dirnowt[128];
  char  *    string;

  XmFileSelectionBoxCallbackStruct *cbs = 
    (XmFileSelectionBoxCallbackStruct *) call_data;

  /* get name of file returned from the file widget */
  string = extractstr(cbs->value);
  strcpy(filnew,string);
  XtFree(string);

  /* get name of directory used in the file widget */
  string = extractstr(cbs->dir);
  strcpy(dirnowt,string);
  XtFree(string);

  if (strcmp(dirnow,dirnowt) != 0)
     {    /* new directory wanted */
     strcpy(dirnow,dirnowt);
     chdir(dirnow);
     }

  imagemen_cb_com();
  return;
  }

/******************** imagemennol_cb ************************************/
/* used if !listfi (when do not want file selction widget */

 void imagemennol_cb(Widget iw_temp, XtPointer data,  
                                  XtPointer call_data)
  { 
  char  *           string;

  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;

  /* get name of file returned from the file widget */
  string = extractstr(cbs -> value); 
  strcpy(filnew,string);
  XtFree(string);

  imagemen_cb_com();

  return;
  }

/******************** imagemen_cb_com *************************/

 void imagemen_cb_com()
  { 
  int               sayrange;

  if (!listfi)
     XtUnmanageChild(iw_filesnol);
  else
     XtUnmanageChild(iw_files);

  expos_cb(NULL,NULL,NULL);

  sayrange = TRUE;
  if (ioper == 4) sayrange = FALSE;

  if ( ioper == 20 || (ioper == 11 && icattyp == 1))
    {   /* display montage or categorize from doc. file menu */
    docmontmen(filnew);
    return;
    }
  else if ( ioper == 14)
    {   /* display dendrogram menu */
    dendromen(filnew);
    return;
    }

  else if (ioper == 17 && imnum == 2)
     {                /************** display colored surface from 3-d volume */ 
     imnum = 1;

     strcpy(filcolvol,filnew);
     /* printf("filcolvol: %s, filnewt: %s \n",filcolvol,filnew); */

     /* get menu for displaying the colored surface */   
     surfmen(ioper == 15, FALSE, ioper == 17);
     return;
     }

  imagemen1(filnew,sayrange);

  return;
  }

 /******************************** imagemen1 *************************/

 int imagemen1(char * filnewt, int sayrange)
 { 

  int         iform;
  char*       cptrdum = NULL;
  float*      fptrdum = NULL;
  XRectangle  rect;
  int         nsams, nrows;        /*the size of the image on the screen*/

  /* set waitcursor */
  waitcursor(iwin, TRUE, 79);

  /* open the new file */
  filedata = openold(&filnewt[0],&nsam,&nrow,&nslice,&iform,"o");

  if (filedata == (FILEDATA *)FALSE)
     {
     spout("*** Unable to open image file");
     waitcursor(iwin, FALSE, 79);
     if (ioper == 8)
         if (filedatal) closefile(filedatal);
     return FALSE;
     }

  if ((filedata->imami != 1) && (norm(filedata) == FALSE))
     {    
     spout("*** Normalization failed.");
     closefile(filedata);
     waitcursor(iwin, FALSE, 79);
     return FALSE;
     }

  fmin = filedata->fmin;
  fmax = filedata->fmax; 

  if (sayrange)
     {   /* list image range */
     sprintf(&outstr[0],"Range: %f ... %f",fmin,fmax);
     spout(&outstr[0]);
     }

  if ((fmax - fmin) < 10e-10)
     {     /* this is a blank image */
     spout("*** BLANK IMAGE SKIPPED");
     waitcursor(iwin, FALSE, 79);
     closefile(filedata);
     return FALSE;
     }

  /* set flag if present image file same as previous file */
  if (strcmp(&filnow[0],&filnewt[0]) == 0) 
     samefile = TRUE;
  else
     {
     samefile = FALSE;
     gotsurf  = FALSE;
     }

  /* set name of present displayed image */
  strcpy(&filnow[0],&filnewt[0]);

  /* set default display area to full slice */
  ixullmin = marlef;      iyullmin = martop;
  ixlrlmax = iwidex;      iylrlmax = ihighx;
  ixulrmin = iwidex;      iyulrmin = martop;
  ixlrrmax = iwidex;      iylrrmax = ihighx; 
  if (ioper == 8)
     {
     ixlrlmax = iwidex/2;
     ixulrmin = iwidex/2+1;
     }

  nsam1    = 1;    nrow1   = 1;   
  nslice1  = 1;    nslice2 = 1;

  if (iskip > 1)         /* shrink image by factor of iskip */ 
    { 
     if( (nsam/iskip) < (ixlrlmax-ixullmin+1) )   /* try to display  */
       nsam2 = nsam;                              /* as much as the  */
     else                                         /* window can view */
       nsam2 = (ixlrlmax-ixullmin+1)*iskip;
     if( (nrow/iskip) < (iylrlmax-iyullmin+1) ) 
       nrow2 = nrow;
     else 
        nrow2 = (iylrlmax-iyullmin+1)*iskip;
    }
  else 
    {  
     nsam2   = MYMIN(nsam,ixlrlmax-ixullmin+1);
     nrow2   = MYMIN(nrow,iylrlmax-iyullmin+1);
    }

  /* reset the image size parameters */
  dispsize( nsam1, nsam2, nrow1, nrow2, 1,1,iskip, &nsams, &nrows, &islice);

  /* set image display range */
  imagego  = mapgo;
  imageend = 127;

  if (ioper == 1 && nstack > 0) {
			/*********************** display a stack file */
      /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     stackmen();      
  }
  else if (ioper == 1  && nslice == 1 )
     {                /************************* display single image */

     /* set the full image size for image display */
     nsam2 = nsam;  nrow2 = nrow;  islice = 1;      
     dispsize( nsam1, nsam2, nrow1, nrow2, 1,1,iskip, &nsams, &nrows, &islice);

     imagedis(filedata,TRUE,&imagel,TRUE,FALSE,&fptrdum,
              imagego,imageend);

     /* set the variables for image shifting */
     ixullmin = ixul;	ixlrlmax=MYMIN(ixul+nsams,ixlrlmax);
     iyullmin = iyul;	iylrlmax=MYMIN(iyul+nrows,iylrlmax);

     nsam2 = nsams;  nrow2 = nrows;  islice = 1;

     /* record left image location and size */
     nsaml  = nsams;   nrowl  = nrows;
     nsam1l = nsam1;  nrow1l = nrow1;
     nsam2l = MYMIN(nsams,ixlrlmax-ixullmin+1);  
     nrow2l = MYMIN(nrows,iylrlmax-iyullmin+1);
     ixull  = ixul;  iyull  = iyul;
     ixulli = ixul;  iyulli = iyul;
     ixuln = ixul;   iyuln = iyul;

     /* free up the file descriptor */
     closefile(filedata);

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);     }

  else if (ioper == 1 || (ioper == 2 && nslice > 1))
     {                /************* display montage from 3-d volume */     

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     montmen3();     
     }

  else if (ioper ==  2 || (ioper == 11 && icattyp == 0))
     {                 /************ display montage from 2-d volume */     
                       /* also used by categorize from seq. file     */     
     montmen2();

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);
     }

  else if (ioper == 17 && imnum == 1)
     {                /***************** get color volume name first */
     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);
     imnum = 2; 
     imagemen(NULL,"h",NULL);
     }

  else if (ioper ==  3  || ioper == 15)
     {                /************* display surface from 3-d volume */ 
     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);
     imnum = 1;

     if (nslice <=1)
        {
        spout("*** Can not show surfaces of 2-D images.");
        closefile(filedata);
        return FALSE;
        } 
     /* display the surfaces or sird image of surface */   
     surfmen(ioper == 15, FALSE, ioper == 17);
     }

  else if (ioper ==  4)
     {            /*************************************** histogram */ 
     hist(filedata);
     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);
     /* filedata is closed in hist-->his */
     }

  else if (ioper == 5)
     {   /************************************************** markers */
     /* locate markers from this image */ 
     /* display single image */
     marker = 1;

     /* set the full image size for image display */
     nsam2 = nsam;  nrow2 = nrow;  islice = 1;      
     dispsize( nsam1, nsam2, nrow1, nrow2, 1,1,iskip, &nsams, &nrows, &islice);

     imagedis(filedata,TRUE, &imagel,TRUE,FALSE,&fptrdum,
              imagego,imageend);

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     /* record left image location and size */
     ixullmin = ixul;	ixlrlmax=ixul+nsams;
     iyullmin = iyul;	iylrlmax=iyul+nrows;

     nsaml  = nsams;   nrowl  = nrows;
     nsam1l = nsam1;  nrow1l = nrow1;
     nsam2l = nsams;  
     nrow2l = nrows;
     ixull  = ixul;  iyull  = iyul;
     ixulli = ixul;  iyulli = iyul;

     gold();

     /* free up the file descriptor */
     closefilep(&filedata);
     }

  else if (ioper == 6)
     {             /****************************************** mask  */
     islice = 1;
     imagedis(filedata,FALSE, &imagel,FALSE,FALSE,&fptrdum,
              imagego,imageend);

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     /* mask this image */ 
     mask();

     /* free up the file descriptor */
     closefile(filedata);
     }

  else if (ioper == 7)
     {         /******************************************** profile */
     islice = 1;

     /* display single image first*/
     imagedis(filedata,FALSE,&imagel,FALSE,FALSE,&fptrdum,
              imagego,imageend);

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     /* profile this image */
     profile();

     /* filedata closed in profilemen_buts */
     }

  else if (ioper == 8)
     {           /********************************* particle picking */

     /* set upper left corner */
     placed = TRUE;
     islice = 1;

     if (imnum == 1)  
        {          /* display  left image */

        /* set left image visible display position */
        nsam2 = nsam;  nrow2 = nrow;  islice = 1;

        ixuln  = ixullmin;  iyuln  = iyullmin;  
        rect.x = ixuln; rect.y = iyuln;
        rect.width = ixlrlmax-ixullmin;
        rect.height = iylrlmax-iyullmin;

        XSetClipRectangles(idispl,icontx,0,0,&rect,1,Unsorted);

        /* do not save the floating point buffer for future use */
        showimage(filedata, TRUE,  &cptrduml, 
                            TRUE,  &imagel, 
                            FALSE, &fptrdum, TRUE,
                            filedata->fmin, filedata->fmax,
                            imagego, imageend);

        /* save pointer to left filedata */
        filedatal = filedata;

        /* record left image location and size */
        nsaml  = nsam;   nrowl  = nrow;
        nsam1l = nsam1;  nrow1l = nrow1;
        nsam2l = MYMIN(nsam,ixlrlmax-ixullmin+1);  
        nrow2l = MYMIN(nrow,iylrlmax-iyullmin+1);
        ixull  = ixuln;  iyull  = iyuln;
        ixulli = ixuln;  iyulli = iyuln;

        rect.width = ixlrrmax-ixullmin;
        rect.height = iylrrmax-iyullmin;

        XSetClipRectangles(idispl,icontx,0,0,&rect,1,Unsorted);

        /* cancel waitcursor */
        waitcursor(iwin, FALSE, 79);
        imnum = 2;

        /* display second image */
        imagemen(NULL,"8",NULL);
        }
     else
        {
        /* set right image visible display position */
        ixuln  = ixulrmin; iyuln  = iyulrmin; 
        nsam1  = 1;   nsam2   = nsam;
        nrow1  = 1;   nrow2   = nrow;

        islice = 1;

        /* do not save the floating point buffer for future use */
        showimage(filedata, TRUE,  &cptrdumr, 
                            TRUE,  &imager, 
                            FALSE, &fptrdum, TRUE,
                            filedata->fmin, filedata->fmax,
                            imagego, imageend);
                   
        /* save pointer to right filedata */
        filedatar = filedata;

        nsamr  = nsam;   nrowr  = nrow;
        nsam1r = nsam1;  nrow1r = nrow1;
        nsam2r = MYMIN(nsam,ixlrrmax-ixulrmin+1);  
        nrow2r = MYMIN(nrow,iylrrmax-iyulrmin+1);
        ixulr  = ixuln;  iyulr  = iyuln;
        ixulri = ixuln;  iyulri = iyuln;

        /* cancel waitcursor */
        waitcursor(iwin, FALSE, 79);
        imnum = 1;

        /* retrieve existing doc files for this set of data */
        fitdocmen(NULL,NULL,NULL);

        /* do not free up the image file descriptors */
        }
     }
 else if (ioper == 9)
     {           /***************************************** annotate */

     islice = 1;
     /* display single image first*/
     imagedis(filedata,FALSE,&imagel,FALSE,FALSE,&fptrdum,
              imagego,imageend);

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     /* annotate this image */
     annotemen();

     }

 else if (ioper == 10 || ioper == -10)
     {          /******************************************** filter */
     islice = 1;

     /* display single image from file first, do not keep image */
     /* save the floating point buffer for future use           */

     showimage(filedata, FALSE, &cptrdum,
                         FALSE, &imagel, 
                         TRUE,  &fimage, TRUE,
                         filedata->fmin, filedata->fmax,
                         imagego, imageend);
          

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     if (ioper == 10)
        {
        erodethresh = filedata->fmax;
        /* display filter selection menu */
        filtermen();
        }
     else
        {
        /* use previous filter type during repeat */
        }
     }

  else if (ioper == 12)
     {   /****************************************** pick reflections */ 
     /* display single image */
     islice = 1;
     imagedis(filedata,FALSE, &imagel,FALSE,FALSE,&fptrdum,
              imagego,imageend);

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     reflmen();

     /* free up the file descriptor */
     closefile(filedata);
     }                  
 
  else if (ioper == 13)
     {   /****************************************** Show recording */ 

     /* display single recorded image */
     islice = 1;
     imagedis(filedata,FALSE, &imagel,FALSE,FALSE,&fptrdum,
           (int) filedata ->fmin, (int)filedata ->fmax);

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     /* free up the file descriptor */
     closefile(filedata);
     }                  
   else if (ioper ==  16)
     {   /****************************************** Movie montage */
     /* want to start again at upper left origin */
     ixul   = marlef;
     iyul   = martop;
     nbot   = -1;
     if (nslice > 1)
        {         /************* display montage from 3-d volume */    
        montmen3();
        }
     else
        {         /************* display montage from 2-d volume */     
        montmen2();
        }
     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);
     }
  else if (ioper ==  19)
     {                /************* drill-n-fill from 3-d volume */ 
     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);

     if (nslice <= 1)
        {
        spout("*** Can not drill or fill 2-D images.");
        closefile(filedata);
        return FALSE;
        }
 
     /* drill or fill a  volume */   
     dentmen();
     } 

  else if (ioper ==  20 || (ioper == 11 && icattyp == 1))
     {                 /*** use doc file to display montage from 2-d volume */     
                       /*** also for categorize from doc file. list */
     mont2(imagego, imageend);

     /* cancel waitcursor */
     waitcursor(iwin, FALSE, 79);
     }

 
  return TRUE;
  }

