
/*$Header: /alwyn/usr1/web//RCS/lut.c,v 1.4 94/03/25 15:57:43 leith Exp $*/
/*
C++*************************************************************************
C
C WID_LUT.FOR -- CREATED DEC 89
C lut.c                  DEC 92
C                                                           
C **************************************************************************
C    AUTHOR:  ArDean Leith
C *  COPYRIGHT (C)1989, WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    LUT(FILLUT,MAP,NMIN,NMAX)
C
C    PARAMETERS:    FILNAM        LUT FILE NAME                  INPUT
C                   MAP           COLORMAP POINTER               INPUT
C                   NMIN,NMAX     RANGE OF INDICES TO BE SET     INPUT
C
C    CALLED BY:     LUTMEN
C
C--*********************************************************************
*/

#include "common.h"
#include "routines.h"

 /*************************** lut ************************************/

 int lut(char * filnam, Colormap mapt, int nmint, int nmaxt)
 {
 int             nindex, neg, nchar, indx, ipixel, ival;
 int             ir, ig, ib;
 float           vmax, scalin, facsum, factin, fact, dx;
 FILE            *luncol;
 char            output[80];
 char            ctemp[81];

 /* file name sent from caller */
        
 if (strcmp(filnam,"bwth") == 0) 
    {
    spout("*** Use contrast option to do this!"); return FALSE;
    }
 else if ((strcmp(filnam,"flip") == 0)  || (strcmp(filnam,"colo") ==0)) 
    {
    spout("*** Command not implemented yet!");  return FALSE;
    }

 neg = FALSE;
 if (*filnam == '-') 
    {  
    neg = TRUE;
    *filnam = ' ';
    }
 else if (*filnam == '+')
    *filnam = ' ';

 if ((luncol= fopen(filnam,"r")) ==  NULL)
      {    /* file does not exist */
      sprintf(output," *** Unable to open look-up-table: %s",filnam);
      spout(output);  return FALSE;
      } 

 /* find out whether the lut spicifies the indicies or resets */
 /* all of the indices one-by-one */
 /* if the indicies are specified there are 4 integers per line */
 /* otherwise only 3 integers per line */

 vmax   = 65535.0;
 fact   = vmax / 255.0;

 scalin = vmax / (nmaxt - nmint + 1);
 factin = (float) (nmaxt - nmint + 1) / 256.0;
 facsum = nmint;

 nchar = strlen(fgets(ctemp, 80, luncol));
 rewind(luncol);

    /*   printf("nchar: %d \n",nchar); */


 if (nchar <= 16 || nchar >= 79) 
    {
    /* load color lookup table, indicies not specified */
    indx = 0;
    for (nindex = 1; nindex <=256; nindex++)
       {
       /* read a line of the lut file */
       if (fscanf(luncol,"%d %d %d",&ir,&ig,&ib) == EOF)
          break;

      /* determine indx  for this line.  only indices: nmint...nmaxt  */
      /* (usually 0..127) are to be used. so I map lines 1..256 to    */
      /* this range.  this may discard some lines from the lut file!   */

      dx     = (nindex - 1) * factin + facsum;
      ipixel = dx;
      if (neg) ipixel = nmaxt - ipixel;

      /* make sure ipixel is in range:  nmint...nmaxt */
      ipixel = MYMAX(MYMIN(ipixel,nmaxt),nmint);

      colort[indx].pixel   = ipixel;
      colort[indx].red     = MYMIN(MYMAX((ir*fact),0),vmax);
      colort[indx].green   = MYMIN(MYMAX((ig*fact),0),vmax);
      colort[indx].blue    = MYMIN(MYMAX((ib*fact),0),vmax);
      colort[indx++].flags   = DoRed|DoGreen|DoBlue;

/***********
      printf(" %d : %d : %d , %d , %d \n",indx,ipixel,colort[indx-1].red,
                                 colort[indx-1].green,colort[indx-1].blue);
*********/

      }
    }
 else
    {
    /* load color lut  at indicies given in file  */
    indx = 0;
    for (nindex = 0; nindex < 256; nindex++)
       {
       /* read a line of the lut file */
       if (fscanf(luncol,"%d %d %d %d",&ir,&ig,&ib,&ival) == EOF)
          break;
       /* make sure index is within nmint....nmaxt range */
       if (ival >= nmint && ival <= nmaxt) 
          {
          /* minus sign is for negative vs positive */
          if (neg) ival = nmaxt - ival;

          colort[indx].pixel   = ival;
          colort[indx].flags   = DoRed|DoGreen|DoBlue;

          colort[indx].red     = ir * fact;
          colort[indx].green   = ig * fact;
          colort[indx++].blue  = ib * fact;
          }
       }
    }

 /* put the colors in the desired colormap */
 XStoreColors(idispl,mapt,colort,indx);

 XFlush(idispl);
 fclose(luncol);
 return TRUE;
 }

