
/*$Header: /net/ithaca/usr1/web/src/RCS/mask.c,v 1.4 1996/04/01 17:25:07 leith Exp $*/
/*
C***************************************************************************
C
C  WID_MASK.FOR  -- CREATED FEB 90
C  mask.c                   DEC 92
C                                                              
C **************************************************************************
C *  AUTHOR: ArDean Leith                                                  *
C *  COPYRIGHT (C)1990, WADWSWORTH CENTER FOR LABORATORIES AND RESEARCH,   *
C *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    MASK
C
C    PURPOSE:      CREATES A MASK BY DRAWING AROUND AREAS WITH MOUSE
C
C    PARAMETERS:   
C
C    CALLED BY:    MASKMEN
C
C--*********************************************************************
*/

#include "common.h"
#include "routines.h"

 /* internal subroutine prototypes */
 void          mask_pop     (Widget, XEvent *, String *, Cardinal *);

 /* global variable used by maskmen also */
 int                  filler = TRUE;
 GC                   icontxx;

 /* internal common variables */
 static int           masks, firstpt, ix0,iy0, ipts;
 static XPoint *      points;
 static int           nsize;
 static int           ix2,iy2,ix1,iy1;

 /************************  mask   *******************************/

 void mask(void)

 {
 /* set new lut for all correct use with x-or */
 overlut();

 /* refresh screen */
 expos_cb(NULL,NULL,NULL);

 /* set number of masked polygons made so far to zero */
 masks  = 0;
 nsize  = 300;

 if ((points = (XPoint *) malloc(nsize * sizeof(XPoint))) == (int) 0)
      { 
      spout("*** Unable to malloc point array in mask.c.");
      return;
      }

 /*  set flag for new polygon */
 firstpt = TRUE;      

 showbutx("Starts / sets line segment.",
          "Menu.",
          "Close this polygon.",FALSE);

/* set up a xor graphics context */
icontxx = setxor(icontx);

 actions(iw_win, "mask_pop", mask_pop, "M123");
 }


 /****************** mask_pop  ************************************/

 void mask_pop(Widget iw_t, XEvent *event, String *params,
                   Cardinal *num_params)
 {
 int icoloro;

 if (!(strcmp(*params,"M")))                 /* cursor moved */
    {
    if (!firstpt)
       {
       /* get mouse position ix2,iy2  */
       getloc(event,'M',&ix2,&iy2);

       /* already have line, mouse moved, so erase and draw new line */
       xorline(iwin,icontxx,FALSE,ix1,iy1,ix2,iy2);
       }
    }

 else if (!(strcmp(*params,"1")))            /* button 1 pushed */ 
    {
    /* get mouse position ix2,iy2  */
    getloc(event,'M',&ix2,&iy2);

    if (firstpt)
       {
       firstpt = FALSE;
       ix0     = ix2;
       iy0     = iy2;
       ipts    = 0;
       }
    else
       {   /* draw permanent line to this location */

       /* set to draw in overlay plane */
       wisetplanes(idispl,icontx,128);

       xorline(iwin,   icontx,TRUE,ix1,iy1,ix2,iy2);
       /* save line in backing store */
       xorline(imagsav,icontx,TRUE,ix1,iy1,ix2,iy2);

       /* set to draw in all planes */
       wisetplanes(idispl,icontx,-1);
       }
    ix1 = ix2;
    iy1 = iy2;
    if (ipts >= nsize)
       {
       nsize  = nsize + 50;
       if ((points = (XPoint *) realloc(points, nsize * sizeof(XPoint))) == (int) 0)
          { 
          spout("*** Unable to realloc point array in mask.c.");
          ipts = nsize - 2;
          }
       }
    points[ipts].x   = ix1;
    points[ipts++].y = iy1;
    }
 
 else if (!(strcmp(*params,"2")))          /* button 2 pushed */ 
    {
    /* menu callup */
    maskmen();
    }

 else if (!(strcmp(*params,"3")))          /* button 3 pushed */        
    {  
    /*   finished with this mask polygon */

    if (ipts > 0)
       {   
       /* close the last polygon */

       /* set to draw in overlay planes */
       wisetplanes(idispl,icontx,128);

       xorline(iwin,   icontx,TRUE,ix1,iy1,ix0,iy0);
       xorline(imagsav,icontx,TRUE,ix1,iy1,ix0,iy0);
       points[ipts].x   = ix0;
       points[ipts++].y = iy0;

       icoloro = icolor;
       if (filler) 
          /*  set fill color to overlay color (depends on overlut) */
          wicolor(icontx,colorgo +9);

       else
          /* set fill color to black */
          wicolor(icontx,-1);
 
       /* draw the mask on the image */
       XFillPolygon(idispl,iwin,icontx,points,ipts,
                     Complex,CoordModeOrigin);

       /* draw the mask on the backing store */
       XFillPolygon(idispl,imagsav,icontx,points,ipts,
                    Complex,CoordModeOrigin);

       /* set to draw in all planes */
       wisetplanes(idispl,icontx,-1);

       /* reset current color */
       wicolor(icontx,icoloro);

       masks++;
       ipts    = 0;
       firstpt = TRUE;
       }
   }
 return;
 }

