
/*$Header: /net/ithaca/usr1/web/src/RCS/monn.c,v 1.9 1995/09/18 21:15:07 mladjadj Exp $*/
/*
C++************************************************************************
C
C MONN -- CREATED JAN 90 FROM WKMONN.FOR
C             SPEEDED UP APR 90
C             CONVERTED TO C IN JULY 92 al
C
C **************************************************************************
C *  AUTHOR:  ArDean Leith                                                 *
C *  COPYRIGHT (C)1992,  WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C    MONN(ICONTX,LUNF,IXUL,IYUL,AXIS,IWANT,NSAM,NSAM1,NSAM2,NSAMS,
C            NROW,NROW1,NROW2,NROWS,NSLICE,NSLICE1,NSLICE2,NSLICES,
C            NMAX,MAXDIM,IRTFLG)
C
C    PURPOSE:         DIPLAYING MONTAGE FROM 3-D FILE SERIES. NORMALIZES
C                     EACH SLICE OF THE MONTAGE SEPARATELY. USE WIZTOXYN 
C                     FOR INCORE NORMALIZATION IF POSSIBLE.
C
C    PARAMETERS:      
C
C    CALLED BY:       WID_MONT2    MONT3
C
C****************************************************************************
*/

#include "common.h"
#include "routines.h"

#define  FLTZER 10e-10

/****************************  monn **********************************/ 

 int monn(GC icontxt, FILEDATA *fileptr,  int iwant,
          int nsami, int nrowi, int nslices, int nmint, int nmaxt)

 {   
 float         *buf, *rbuf;
 char          *cbuf;
 float         fmint, fmaxt, fval, fn, fncon;
 int           iloc, i, m, npixpr;
 int           isam, irow;
 int           iconz, irect, nsamu, nrowu, nsliceu, miskip, miskipm1;
 int           irec1;

 buf = filedata -> buf;

 /* allocate space for buffers */
 if ((rbuf = (float *) malloc(nsami * nrowi * sizeof(float))) == 
             (float *) 0)
    {spout(" *** Can not allocate rbuf memory in monn."); return FALSE;}

 /* zero slice min and maximum values */
 fmint =  10e20;
 fmaxt = -10e-20;

 iloc  = -1;

 if (axis == 'z')
    {  /*   Z SLICE WANTED   (FASTEST SLICING AXIS) */

    islice = iwant;
    iconz  = (islice - 1) * nrow;

    if (iskip == 1)
       {    /* display z slice at full size */
       for (irow=nrow1; irow <= nrow2; irow++)
          {
          irect = irow + iconz;
 
          if (redlin(fileptr,buf,nsam,irect) == FALSE) 
             {free(rbuf); return FALSE;}

          for (isam=nsam1-1; isam < nsam2; isam++)
             {
             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             rbuf[++iloc] = fval;
             }
          }
       }

    else if (iskip > 1)
       {    /* display z slice at reduced size */

       /* find upper end of col. and row series */
       nsamu = nsam2 - ((nsam2-nsam1+1) % iskip) - iskip + 1;
       nrowu = nrow2 - ((nrow2-nrow1+1) % iskip) - iskip + 1;

       for (irow = nrow1; irow <= nrowu; irow=irow+iskip)
          {
          irect = irow + iconz;
          if (redlin(fileptr,buf,nsam,irect) == FALSE) 
             {free(rbuf); return FALSE; }

          for (isam=nsam1-1; isam < nsamu; isam=isam+iskip)
             {
             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             rbuf[++iloc] = fval;
             }
          }
       }
    
    else if (iskip < -1)
       {  /* display z slice at magnified size */
       miskip   = -iskip;
       miskipm1 = miskip - 1;
       npixpr   = nsami * miskip;

       for (irow = nrow1; irow <= nrow2; irow++)
          { 
          irect = irow + iconz;
          if (redlin(fileptr,buf,nsam,irect) == FALSE) 
             {free(rbuf); return FALSE;}

          iloc = (irow -1) * npixpr;

          for (isam=nsam1-1; isam < nsam2; isam=isam++)
             {     /* check all pixels for min and max values */
             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             for (i = 0; i < miskip; i++)
                {
                for (m = 0; m < miskip; m++)
                   rbuf[iloc + m * nsami] = fval;
                iloc++;
                }
             }
          }
       }
    }

 else if (axis == 'y')
    {   /* y axis slicing */

    irow = iwant;

    if (iskip == 1)
       {   /* display y slice at normal size */
       for (islice=nslice1; islice<=nslice2; islice++)
          {
          irect = (islice - 1) * nrow + irow;

          if (redlin(fileptr,buf,nsam,irect) == FALSE) 
             {free(rbuf); return FALSE; }

          for (isam=nsam1-1; isam < nsam2; isam++)
             {
             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             rbuf[++iloc] = fval;
             }
          }
       }

    else if (iskip > 1)
       {   /* display y slice at reduced size */

       /* find upper end of col. and slice series */
       nsamu   = nsam2   - ((nsam2  -nsam1  +1) % iskip) - iskip + 1;
       nsliceu = nslice2 - ((nslice2-nslice1+1) % iskip) - iskip + 1;

       for (islice = nslice1; islice <= nsliceu; islice=islice+iskip)
          {
          irect = (islice -1) * nrow + irow;
          if (redlin(fileptr,buf,nsam,irect) == FALSE) 
             {free(rbuf); return FALSE; }

          for (isam=nsam1-1; isam < nsamu; isam=isam+iskip)
             {
             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             rbuf[++iloc] = fval;
             }
          }
       }

    else if (iskip < -1)
       {  /* display y slice at magnified size */
       miskip   = -iskip;
       miskipm1 = miskip - 1;

       for (islice = nslice1; islice <= nslice2; islice++)
          { 
          irect = (islice - 1) * nrow + irow;
          if (redlin(fileptr,buf,nsam,irect) == FALSE) 
             {free(rbuf); return FALSE; }

          iloc = (islice - 1) * nsami * miskip;

          for (isam=nsam1-1; isam < nsam1; isam=isam++)
             {   /* check all pixels for min and max values */
             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             for (i = 0; i < miskip; i++)
                {
                for (m = 0; m < miskip; m++)
                   rbuf[iloc + m * nsami] = fval;
                iloc++;
                }
             }
          }
       }
    }

 else if (axis == 'x')
    {   /* WANT X SLICE  (SLOW) */

    isam = iwant -1;

    if (iskip == 1)
       {   /* display x slice at normal size */

       for (islice=nslice1; islice<=nslice2; islice++)
          {
          irec1 = (islice - 1) * nrow;
          for (irow=nrow1; irow <=nrow2; irow++)
             {
             irect = irec1 + irow;
             if (redlin(fileptr,buf,nsam,irect) == FALSE) 
                {free(rbuf); return FALSE; }
             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             rbuf[++iloc] = fval;
             }
          }
       }

    else if (iskip > 1)
       {   /* display x slice at reduced size */

       /* find upper end of col. and slice series */
       nrowu   = nrow2   - ((nrow2  -nrow1  +1) % iskip) - iskip + 1;
       nsliceu = nslice2 - ((nslice2-nslice1+1) % iskip) - iskip + 1;

       for (islice = nslice1; islice <= nsliceu; islice=islice+iskip)
          {
          irec1 = (islice -1) * nrow;
          for (irow=nrow1; irow<=nrowu; irow=irow+iskip)
             {
             irect = irec1 + irow;
 
             if (redlin(fileptr,buf,nsam,irect) == FALSE) 
                {free(rbuf); return FALSE; }

             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             rbuf[++iloc] = fval;
             }
          }       
       }

    else if (iskip < -1)
       {  /* display x slice at magnified size */
       miskip   = -iskip;
       miskipm1 = miskip - 1;

       for (islice = nslice1; islice <= nslice2; islice++)
          { 
          irec1 = (islice - 1) * nrow;

          iloc  = (islice - 1) * nsami * miskip;

          for (irow=nrow1; irow<=nrow2; irow++)
             {
             irect = irec1 + irow;
             if (redlin(fileptr,buf,nsam,irect) == FALSE) 
                {free(rbuf); return FALSE; }

             fval  = buf[isam];
             fmaxt = MYMAX(fmaxt,fval);
             fmint = MYMIN(fmint,fval);
             for (i = 0; i < miskip; i++)
                {
                for (m = 0; m < miskip; m++)
                   rbuf[iloc + m * nsami] = fval;
                iloc++;
                }
             }
          }
       }
    }
 

 /* allocate space for cbuf */
 if ((cbuf = (char *) malloc(nsami * nrowi * 
                 sizeof(char))) == (char *) 0)
     {      
     spout(" *** Unable to allocate cbuf memory in monn.c");
     if (rbuf) free(rbuf); return FALSE;
     }

 /* CHECK TO SEE IF THIS SLICE CAN BE NORMALIZED OK */
 if (fabs(fmaxt - fmint) < FLTZER)
    {    /* a blank slice */
    fn    = 0.0;
    fncon = nmint;
    }
 else
    {
    fn    = (float) (nmaxt - nmint) / (fmaxt - fmint);
    fncon = -fmint * fn + nmint + 0.5;
    }


 /* normalize data and convert to 8 bits */
 for (i=0; i < iloc; i++)
    cbuf[i] = rbuf[i] * fn + fncon;

 /* DISPLAY THIS IMAGE SLICE */
 wipic(icontxt,cbuf, ixul, iyul, nsami, nrowi, TRUE, FALSE, &imagel);

 if (rbuf) free(rbuf); rbuf = NULL;
 if (cbuf) free(cbuf); cbuf = NULL;

 return TRUE;
 }
