
/*$Header: /net/ithaca/usr1/web/src/RCS/opennew.c,v 1.13 1996/04/01 21:39:37 mladjadj Exp $*/
/*
 * opennew.c
 *
 *
 *************************************************************************
 * SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK 
 * COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 **************************************************************************
 *
 * opennew(FILNAM,NSAM,NROW,NSLICE,IFORM,DISP)
 *
 * PURPOSE:     TO CREATE A NEW SPIDER DATA FILE FOR RANDOM ACCESS io
 *
 * PARAMETERS:     FILNAM     CHARACTER ARRAY, CONTAINING FILE NAME 
 *      LUN        LOGICAL UNIT NUMBER TO BE ASSIGNED TO FILNAM.
 *      NSAM,NROW  DIMENSIONS OF FILE - INPUT OR OUTPUT PARAMETERS
 *                 OF OPEN, DEPENDING ON DISP.
 *      DISP       CHARACTER CONTAINING ONE OF THE FOLLOWING 
 *                 DISPOSITION SPECIFICATIONS:
 *
 *                  U -         IT IS NOT KNOWN IF THE FILE EXISTS.  
 *                              IF IT DOES NOT EXIST, NON-ZERO NSAM AND
 *                              NROW VALUES HAVE TO BE AVAILABLE; THE
 *                              FILE IS CREATED WITH THESE DIMENSIONS.
 *                              IF THE FILE ALREADY EXISTS, IT WILL BE 
 *                              OVERWRITTEN. 
 *
 *        NF                    ERROR RETURN FLAG.
 *                              NF = 0    NORMAL RETURN 
 *                              NF = 1    ERROR RETURN
 *
 *	 NSLICE	                NUMBER OF PLANES, EACH CONTAINING NSAM*NROW 
 *                              ELEMENTS. 
 *
 *    VARIABLES:   IFORM  = FILE TYPE SPECIFIER. 
 *                        = +3    3-D IMAGE
 *	                  = +1    2-D IMAGE
 *                        = -1    2-D FOURIER TRANSFORM
 *                        = -3    3-D FOURIER TRANSFORM
 *                        = +8    8 BIT BLACK AND WHITE IMAGE
 *                        = 11    8 BIT COLOR IMAGE
 *
 *   NOTE:         opennew does not set a title in the file nor does
 *                 it set angles in the file (unlike fortran version)
 *
 *********************************************************************/

#include "files.h"
#ifdef VMS
#include <unixio.h>
#endif

 /* external common variables */
 extern   float  fmin,fmax;
 extern   float  param[102];

 /* external file prototypes */
 extern   int    wrtlin(FILEDATA *, float *, int, int);
 extern   int    closefile(FILEDATA *);
 extern   void   spout(char *);

#ifdef P_SGIOSF
unsigned int *pt;
#define CONVERT_4( A, B )                        \
             (unsigned int)(A) =                 \
            ((unsigned int)(B) >> 24) |          \
            ((unsigned int)(B) << 24) |          \
           (((unsigned int)(B) >> 8) & 0xff00) | \
           (((unsigned int)(B) << 8) & 0xff0000)

#endif

/*************************  opennew  ****************************/


 FILEDATA *opennew(char * filnam, int nsam, int nrow, int nslice, 
                    int iform, char disp[]) 
 { 

 /* header union for equivalencing floating point and character buffers */
 union  BUFFER 
        {
        float hbuf[256];
        char  cbuf[1024];
        } header;


 FILEDATA * fileptr;
 time_t     now;
 char     * datentime ,type[3], output[80];
 float    * buf;
 FILE     * fp;
 int        nlabel,reclen,headrec,headbyt;
 int        i,k,imami,iangle,ot,k1,tsam, ptr_ok;
 float    * ptr, * ptrf;
 
 if (strcmp(disp,"u") != 0 && strcmp(disp,"n") != 0) 
    { spout(" *** Invalid disp in opennew"); return 0; }

 if (strcmp(disp,"n") == 0 && (fp = fopen(filnam,"rb")) != NULL)
    {spout(" *** New file already exists"); fclose(fp); return 0;}

 /* open the file */
#ifdef VMS
 /* mrs=n maximum record size , rfm= fixed length record format */
 sprintf(output,"mrs=%d",nsam*sizeof(float));
 if (( fp = fopen(filnam,"w+b",output,"rfm=fix")) == NULL)
    { spout(" *** Unable to open new file"); return 0; } 
#else
 if ((fp = fopen(filnam,"w+b")) == NULL) 
    { spout(" *** Unable to open new file"); return 0; } 
#endif

 if (nsam == 0 || nrow == 0) 
     /*   for new files, non-zero nsam, nrow need to be supplied  */
     {spout(" ***  Need size info to open file."); return 0; }     

 else if (nslice == 0) 
     {
     /*   new files need a non zero slice number for file format flag */
     spout(" Warning:  nslice set to 1.");
     nslice = 1;
     }

 else if (nslice < 0) 
     {
     /*   want to create an short label format file  */
     spout(" Warning:  short label files no longer  supported.");
     nslice = -nslice;
     }
       
 /*adjust number of header records to have 256*4 bytes in header */

 if (iform < 6)
     /*   standard spider floating point files */
     reclen = nsam * 4;

 else if (iform == 8)
     /*   for 8 bit files     */
     reclen = nsam ;

 else if (iform == 10)
     /*   pixel by pixel 8 bit color image file  */
     reclen = nsam * 3;

 else if (iform == 11)
     /*   color by color 8 bit color image file  */
     reclen = nsam * 3;

 else if (iform == 12)
     /*   8 bit runlength color file */
     reclen = nsam * 4;

 else if(iform == 16)
     /*   for 16 bit integer files */
     reclen = nsam * 2;

 else 
     { spout(" *** UNKNOWN IFORM IN OPENNEW"); return 0; }

 /* allocate space for a spider file structure */
 fileptr = (FILEDATA *) malloc(sizeof(FILEDATA));

 if (fileptr == (FILEDATA *) 0) 
     { spout(" *** Unable to allocate memory in opennew."); 
        printf(" *** Unable to allocate memory in opennew.\n");
        return 0; }

 fileptr -> fp = fp;
 
 /* set number of header records and length of header */
 headrec = 1024 / reclen;
 if ((1024 % reclen) != 0) headrec = headrec + 1;
 headbyt = headrec * reclen;

 header.hbuf[0] = -nslice;
 header.hbuf[1] = nrow;
 header.hbuf[2] = headbyt;
 nlabel         = 0;
 header.hbuf[3] = nlabel;
 header.hbuf[4] = iform;
 imami          = 0;
 header.hbuf[5] = imami;

 /*     zero unused buf variables */
 for (k = 5; k < 212 ;k++)
          header.hbuf[k] = 0.0;

 header.hbuf[11] = nsam;
 header.hbuf[12] = headrec;

 /* angles are not set in opennew */
 iangle = 0;
 header.hbuf[13] = iangle;
 header.hbuf[14] = 0.0;
 header.hbuf[15] = 0.0;
 header.hbuf[16] = 0.0;

 header.hbuf[21] = headbyt;
 header.hbuf[22] = reclen;


 fileptr -> offset  = 0;
 fileptr -> nsam    = nsam;
 fileptr -> nrow    = nrow;
 fileptr -> nslice  = nslice;
 fileptr -> nlabel  = nlabel;
 fileptr -> iform   = iform;
 fileptr -> imami   = imami;
 fileptr -> fmin    = 0.0;
 fileptr -> fmax    = 0.0;
 fileptr -> av      = 0.0;
 fileptr -> sig     = 0.0;
 fileptr -> ihist   = 0;
 fileptr -> headrec = headrec;
 fileptr -> headbyt = headbyt;
 fileptr -> reclen  = reclen;
 fileptr -> iangle  = iangle;

 /* find current date and time */
 now       = time(0);
 datentime = ctime(&now);

 /* set date for file header */
 strncpy(&header.cbuf[844],datentime + 8,  2);
 header.cbuf[846] = '-';
 strncpy(&header.cbuf[847],datentime + 4,  3);
 header.cbuf[850] = '-';
 strncpy(&header.cbuf[851],datentime + 20, 4);
 header.cbuf[855] = ' ';

 /* set date in current file information */
 strncpy(fileptr -> date, &header.cbuf[844],11);
 fileptr -> date[11] = '\0';

 /* set time in file header */
 strncpy(&header.cbuf[856],datentime + 11, 8);
 header.cbuf[864] = '\0';

 /* set time in current file information */
 strncpy(fileptr -> time, &header.cbuf[856],9);

 /* initialize ximage pointer */
 fileptr -> ximage = NULL;

#ifdef NEVER
 /* title is not set in opennew **************************/
 if ((lentitle = strlen(ctitle)) > 0)
    {    /* title in use */
    strncpy(&header.cbuf[864], ctitle, lentitle);
    strncpy(fileptr -> title,  ctitle, lentitle);
    }
#endif

/*
          DEC OSF/1 with files written in IEEE standard        
          to allow portability to other unix machines  
*/ 
#ifdef P_SGIOSF
   pt = (unsigned int *) &header.hbuf[211];
   
   /* flip the bytes for date and time in header */
   for (k1 = 211; k1 <= 216 ; k1++, pt++)
      { CONVERT_4(*pt,*pt);  }
#endif 

 /* write header to open file */
#ifdef VMS
   /* VAX VAX VAX VAX VAX VAX VAX VAX VAX VAX VAX VAX VAX VAX VAX VAX */
   /* we need to make sure that the complete header is written. however
      wrtlin accepts only records of size nsam*sizeof(float) in which we
      have opened the file fp.  so copy the record into a new memory
      having a multiple number the size of nsam*sizeof(float) that will
      hold the complete header 
   */
   ptrf = header.hbuf;  ptr_ok = 0;
   if ( (headrec * nsam) > 256) 
      {
      /* the last record written will have a number of items less than
         nsam so we will try to write from beyond the end of header.hbuf.
      */
      if ( (ptrf = (float *) malloc(headrec * nsam * sizeof(float))) == (float *) NULL )
         {
	  printf(" can not malloc memory in openenw.c \n");
	  closefile(fileptr); return 0; 
	 }
       ptr_ok = 1;
       memcpy((void *) ptrf,(void *)header.hbuf, 256 * sizeof(float)); 	 
      }
   
   for (k1=1; k1 <= headrec ; k1++)
       {
       if (wrtlin(fileptr, ptrf, nsam, k1) != TRUE)
          { closefile(fileptr); return 0; }
       ptrf += nsam;
       }
       
   if(ptr_ok) free(ptrf);
    
/* VAX VAX VAX VAX VAX VAX VAX VAX VAX  VAX VAX VAX VAX VAX VAX VAX */

#else
   if (wrtlin(fileptr,header.hbuf,256,1) != TRUE)
    { closefile(fileptr); return 0; }
#endif

 /* set offset for reading image data from file in future */
 fileptr -> offset = headbyt;

 if ((fileptr -> buf = (float *) malloc(nsam * sizeof(float))) == 
     (float *) 0) 
     {     
     spout(" *** Unable to allocate read/write buffer in opennew.");
     closefile(fileptr); return 0;
     }
 
 /* print file opening information */
 if (iform == -1)
     strcpy(type,"f ");
 else if (iform == -2)
     strcpy(type,"p ");
 else if (iform == -3)
     strcpy(type,"f3");
 else if (iform == -7)
     strcpy(type,"fq");
 else if (iform == -9)
     strcpy(type,"fs");
 else if (iform == -11)
     strcpy(type,"fo");
 else if (iform == -12)
     strcpy(type,"fe");
 else if (iform == 0)
     strcpy(type,"d ");
 else if (iform == 3)
     strcpy(type,"r3");
 else if (iform == 8)
     strcpy(type,"8 ");
 else if (iform == 10)
     strcpy(type,"cp");
 else if (iform == 11)
     strcpy(type,"cc");
 else if (iform == 12)
     strcpy(type,"cr");
 else if (iform == 16)
     strcpy(type,"16");
 else
     strcpy(type,"r ");

 

 /* print filename */
 sprintf(output," %s",filnam);
 spout(output);
     
 /* print file parameters */
 if (nslice > 1)
     {
     sprintf(output," (%s) %d %d %d  New: %s At %s  Header bytes: %d",
           type, nsam, nrow, nslice, fileptr -> date, fileptr -> time, headbyt);
     spout(output);
     }
 else
     {
     sprintf(output," (%s) %d %d     New: %s At %s  Header bytes: %d",
           type, nsam, nrow, fileptr -> date, fileptr -> time, headbyt);
     spout(output);
     }


 param[2] = nsam;
 param[3] = nrow;
 param[4] = 0.0;
 param[5] = 0.0;
 param[6] = 0.0;
 param[7] = 0.0;
 param[8] = nslice;

 return fileptr;

}
