
/*$Header: /net/ithaca/usr1/web/src/RCS/openold.c,v 1.30 1998/02/05 22:20:40 liy Exp $*/

/***********************************************************************
 *
 * openold.c
 *
 ***********************************************************************
 * SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK 
 * COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 ***********************************************************************
 *
 *  openold(filnam, nsamptr, nrowptr, nsliceptr, iformptr, disp)
 *
 * PURPOSE:     TO OPEN AN EXISTING DATA FILE FOR RANDOM ACCESS
 *              READING-WRITING.
 *
 * PARAMETERS:         
 *      FILNAM          FILE NAME                  (SENT)
 *      NSAMPTR,NROWPTR DIMENSIONS OF FILE         (RETURNED)
 *      NSLICEPTR       NO. OF SLICES IN FILE      (RETURNED)
 *      IFORMPTR        FILE TYPE SPECIFIER.       (RETURNED)
 *                        = +3    3-D IMAGE
 *	                  = +1    2-D IMAGE
 *                        = -1    2-D FOURIER TRANSFORM
 *                        = -3    3-D FOURIER TRANSFORM
 *                        = +8    8 BIT BLACK AND WHITE IMAGE
 *                        = 11    8 BIT COLOR IMAGE
 *      DISP       CHARACTER SPECIFYING READONLY OR NOT  (SENT)
 *                        = 'o' - READ/WRITE. 
 *                        = 'q' - READ/WRITE, no error message if
 *                                does not exist 
 *                        = 'r' - FILE EXISTS, OPEN READONLY.
 *
 * RETURNS:    FILEDATA * OR NULL IF ERROR READING FILE
 *
 *********************************************************************/

#include "files.h"
#include "routines.h"

/*       #ifdef P_SGIOSF          */
/*       #include <cvt.h>        */
/*        int cvt_ftof( void *, int, void *, int, int);        */
/*       #endif        */


#ifdef P_SGIOSF
unsigned int *pt;
#define CONVERT_4( A, B )                        \
             (unsigned int)(A) =                 \
            ((unsigned int)(B) >> 24) |          \
            ((unsigned int)(B) << 24) |          \
           (((unsigned int)(B) >> 8) & 0xff00) | \
           (((unsigned int)(B) << 8) & 0xff0000)

#endif

 extern float param[102];
 extern int   nsam8, nrow8, nslice8;
 extern int   nstack, inuse, maxim;

 /* force it to run image size widget for raw images on first call */
 int needrawsize = TRUE;

/*********************** openold ************************************/

 FILEDATA *openold(char filnam[], int *nsamptr, int *nrowptr, 
                   int *nsliceptr, int *iformptr, char disp[])
  
 { 
 union  BUFFER 
        {
        float hbuf[256];
        char  cbuf[1024];
        } header;

 FILEDATA  *fileptr;

 char      type[3], output[160];
 FILE      *fp;
 int       nsam,nrow,nslice,iform,headbyt,headrec;
 int       lentitle, k1, i;
 char *    cptr;

 if (strcmp(disp,"r") == 0) 
    fp = fopen(filnam,"rb");

 else if (strcmp(disp,"o") == 0 || strcmp(disp,"q") == 0)
    fp = fopen(filnam,"r+b");

 else 
    { spout("*** Unknown disp in openold"); return FALSE; }

 if (fp ==  NULL)
     {    /* file does not exist, or is write protected try readonly */
     fp = fopen(filnam,"rb");
     if (fp ==  NULL)
        {  /* does not exist, etc. */

        if (strcmp(disp,"q") != 0)
           {  /* give error output (used in montage increment) */
           sprintf(output,"*** Unable to open: %s",filnam);
           spout(output);  
           }
        return FALSE;
        }
     else
        {  /* opened readonly */
        spout("Opening file as read only!!:");
        }
     } 

 /* allocate space for a spider file structure */
 if ((fileptr = (FILEDATA *) malloc(sizeof(FILEDATA))) ==
    (FILEDATA *) 0) 
    {spout("*** Unable to malloc filedata in openold."); return FALSE; }


  /* set offset for reading header from file */
  fileptr -> offset = 0;
  fileptr -> fp     = fp;

  /* read header of image file */
  if (redlin(fileptr,&header.hbuf[0],256,1) != TRUE)
     { /* assume file is small 8 bit file */
     output[0] = 'N'; 
     }
  else
     { sprintf(output,"%e",header.hbuf[1]);}

/*        DEC OSF/1 with files written in IEEE standard        
**        to allow portability to other unix machines  
*/ 
/*    #ifdef P_SGIOSF         */
/*     for (k1 = 211; k1 <= 216 ; k1++)        */
/*           {            */
             /* need 180:4 floats for title,date,time */
/*           ot = cvt_ftof(&header.hbuf[k1], CVT_BIG_ENDIAN_IEEE_S,        */
/*                         &header.hbuf[k1], CVT_IEEE_S, 0);        */
/*           if (ot)         */
/*                 {         */
                     /* ot=0 success */
/*        printf(" Conversion cvt_ftof of float %d has failed !!\n",k1);    */
/*                 }        */
/*           }        */
/*    #endif          */
#ifdef P_SGIOSF
   pt = (unsigned int *) &header.hbuf[211];
   /* flip the bytes for date and time in header */
   for (k1 = 211; k1 <= 216 ; k1++, pt++)
      { CONVERT_4(*pt,*pt);  }
#endif
    

  if ((output[0] == 'N') ||
      header.hbuf[11] < 1.0   || header.hbuf[11] > 50000.0 ||
      header.hbuf[1]  < 1.0   || header.hbuf[1]  > 50000.0 ||
      header.hbuf[5]  < -.5   || header.hbuf[5]  >     2.0 ||
      header.hbuf[4]  < -20.0 || header.hbuf[4] ==  0.0    ||
      header.hbuf[4]  > 32  )
      {    /* this is not a spider image, treat as 8 bit raw */

      if (nsam8 <= 0 || nrow8 <= 0 || nslice8 <= 0)
         {   /* inquire as to correct raw image size */
         spout("*** Set raw image size and try again");
         get_imsiz();
         closefile(fileptr);
         return FALSE;
         }

      nsam   = nsam8;
      nrow   = nrow8;
      nslice = MYMAX(nslice8,1);

      fileptr -> nslice  = nslice;
      fileptr -> nrow    = nrow;

      fileptr -> nlabel  = 0;
      fileptr -> iform   = 8;
      fileptr -> imami   = 1;
      fileptr -> fmax    = 255.0;
      fileptr -> fmin    = 0.0;
      fileptr -> av      = 0.0;
      fileptr -> sig     = 0.0;
      fileptr -> ihist   = 0;

      fileptr -> nsam    = nsam;
      fileptr -> headrec = 0.0;
      fileptr -> iangle  = 0.0;
      fileptr -> phi     = 0.0;
      fileptr -> theta   = 0.0;
      fileptr -> gamma   = 0.0;

      fileptr -> headbyt = 0;
      fileptr -> reclen  = nsam;

      /* set date, time title to NULL in current file information */
      fileptr -> date[0] =  '\0';
      fileptr -> time[0] =  '\0';
      fileptr -> title[0] = '\0';

      lentitle = 0;
      }
  else
      {

      nslice = header.hbuf[0];
      if (nslice < 0) nslice = - nslice;
      fileptr -> nslice  = nslice;

      fileptr -> nrow    = header.hbuf[1];

      fileptr -> nlabel  = header.hbuf[3];
      fileptr -> iform   = header.hbuf[4];
      fileptr -> imami   = header.hbuf[5];
      fileptr -> fmax    = header.hbuf[6];
      fileptr -> fmin    = header.hbuf[7];
      fileptr -> av      = header.hbuf[8];
      fileptr -> sig     = header.hbuf[9];
      fileptr -> ihist   = header.hbuf[10];

      fileptr -> nsam    = header.hbuf[11];
      fileptr -> headrec = header.hbuf[12];
      fileptr -> iangle  = header.hbuf[13];
      fileptr -> phi     = header.hbuf[14];
      fileptr -> theta   = header.hbuf[15];
      fileptr -> gamma   = header.hbuf[16];

      fileptr -> headbyt = header.hbuf[21];
      fileptr -> reclen  = header.hbuf[22];
      nstack = header.hbuf[23];
      inuse  = header.hbuf[24];
      maxim  = header.hbuf[25];	

      /* set error correction for bad headbyt and reclen spider files */
      iform = fileptr -> iform;
      if (iform < 4 && 
         (fileptr -> reclen < (fileptr -> nsam) * 4)) 
         {
         fileptr -> reclen = (fileptr -> nsam) * 4;
         } 

      headrec = 1024 / (fileptr -> reclen);
      if (fileptr->reclen < 1024 &&
         (1024 % (fileptr -> reclen)) != 0) headrec++;
      headbyt = headrec * (fileptr -> reclen);

      if (iform < 4 && ((fileptr -> headbyt) < headbyt))
         {
         fileptr -> headbyt = headbyt;
         }

      /* set date in current file information */
      strncpy(&fileptr -> date[0], &header.cbuf[844],11);
      fileptr -> date[11] = '\0';

      /* set time in current file information */
      strncpy(&fileptr -> time[0], &header.cbuf[856],9);
      fileptr -> time[8] = '\0';

      /* set title in current file information */
      strncpy(&fileptr -> title[0], &header.cbuf[864], 160);
      fileptr -> title[159] = '\0';

      lentitle = strlen(fileptr -> title);
      }

  /* set offset for reading image data from file in future */
  fileptr -> offset = fileptr -> headbyt;

  *nsamptr   = fileptr -> nsam;
  *nrowptr   = fileptr -> nrow;
  *nsliceptr = fileptr -> nslice;

  if ((fileptr -> buf = (float *) malloc(*nsamptr * sizeof(float))) == 
      (float *) 0) 
     {spout("*** Unable to malloc buf in openold."); return FALSE; }
 
 /* initialize ximage pointer */
 fileptr -> ximage = NULL;

  /*        print file opening information */
  iform = fileptr -> iform;
  if (iform == -1)
     strcpy(type,"f ");
  else if (iform == -2)
     strcpy(type,"p ");
  else if (iform == -3)
     strcpy(type,"f3");
  else if (iform == -7)
     strcpy(type,"fq");
  else if (iform == -9)
     strcpy(type,"fs");
  else if (iform == -11)
     strcpy(type,"fo");
  else if (iform == -12)
     strcpy(type,"fe");
  else if (iform == 0)
     strcpy(type,"d ");
  else if (iform == 3)
     strcpy(type,"r3");
  else if (iform == 8)
     strcpy(type,"8 ");
  else if (iform == 10)
     strcpy(type,"cp");
  else if (iform == 11)
     strcpy(type,"cc");
  else if (iform == 12)
     strcpy(type,"cr");
  else if (iform == 16)
     strcpy(type,"16");
  else if (iform == 88)
     strcpy(type,"ra");
  else
     strcpy(type,"r ");

  if ((lentitle > 0) && (lentitle + strlen(filnam)) < 70)
     /* print filename and title on one line */
     {
     sprintf(output," %s %s", filnam,fileptr -> title);
     spout(output);
     }
  else
     /* print filename */
     {
     sprintf(output," %s",filnam);
     spout(output);
     }

  if (iform == 8 && fileptr -> headbyt <= 0)
      {  /* opened a raw 8 bit image (no header) */
      if (nslice > 1)
         sprintf(output," (raw) %d %d %d   Header bytes: 0",
           *nsamptr, *nrowptr, *nsliceptr);
      else
         sprintf(output," (raw) %d %d    Header bytes: 0",
           *nsamptr, *nrowptr);
      spout(output);
      }

  else
      {
     /* print file parameters */
     if (nslice > 1)
        sprintf(output," (%s) %d %d %d  Old: %s At %s  Header bytes: %d",
           type, *nsamptr, *nrowptr, *nsliceptr, fileptr -> date, 
           fileptr -> time, fileptr -> headbyt);
     else
        sprintf(output," (%s) %d %d     Old: %s At %s  Header bytes: %d",
           type, *nsamptr, *nrowptr, fileptr -> date, fileptr -> time, 
           fileptr -> headbyt);
     spout(output);

     if ((lentitle > 0) && (lentitle + strlen(filnam)) >= 70)
        {    /* must print rest of title on separate lines */
        strncpy(&output[0],&fileptr -> title[0],80);
        output[80] = '\0';
        spout(output);
        if (lentitle > 80)
           {
           cptr = fileptr -> title + 80;
           strncpy(output,cptr,80);
           spout(output);
           }
        }
     }

  param[2] = fileptr -> nsam;
  param[3] = fileptr -> nrow;
  param[4] = fileptr -> fmax;
  param[5] = fileptr -> fmin;
  param[6] = fileptr -> av;
  param[7] = fileptr -> sig;
  param[8] = fileptr -> nslice;
 
  return fileptr;
}


#if BUGGY
c__________________________________________________________
  printf(" *** in open: fp: %d file: %s \n",fp,filnam);
  printf(" *** in open: date: %s \n",fileptr -> date);
  printf(" *** in open: time: %s \n",fileptr -> time);
  printf(" *** in open: header.cbuf: %s \n",&header.cbuf[844]);
  printf(" *** in open: title: %s \n",fileptr -> title);
  printf(" *** in open: lentitle: %d \n",lentitle);
  printf(" *** in open: *nsam: %d \n",*nsamptr);
  printf(" *** in open: fileptr: %d \n",fileptr);
c---------------------------------------------------
#endif


