

/*$Header: /net/alwyn/usr1/web/src/RCS/openoldcnt.c,v 1.7 94/09/07 12:00:36 leith Exp $*/
/*
 ***********************************************************************
 *
 * openoldcnt.c
 *
 *************************************************************************
 * SPIDER - MODULAR IMAGE PROCESSING SYSTEM.  AUTHOR: J.FRANK 
 * COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 **************************************************************************
 *
 * OPENOLDCNT(FILNAM,DISP)
 *
 * PURPOSE:     TO OPEN AN EXISTING CNT DATA FILE FOR RANDOM ACCESS
 *              READING-WRITING.
 *
 * PARAMETERS:     FILNAM     CHARACTER ARRAY, CONTAINING FILE NAME 
 *      DISP       CHARACTER CONTAINING ONE OF THE FOLLOWING 
 *                 DISPOSITION SPECIFICATIONS:
 *
 *                 'O' -      existing file open read/write
 *                 'R' -      existing file open read only
 *
 * RETURNS:       ERROR RETURN FLAG.
 *
 *********************************************************************/

#include "filescnt.h"
#include "routines.h"

 static char  outstr[120];

/*************************  openoldcnt   *****************************/

 FILEDATACNT *openoldcnt(char filnam[], char disp[])
  
 { 

 FILEDATACNT  *fileptrcnt;
 FILEDATA     *fileptrimg;

 char  *   cbuf;
 float *   buf;
 float *   contrl, * txtctr;

 FILE      *fp;
 int       i, j, maxsuf, irecln, irec;

  if (strcmp(disp,"r") == 0) 
     fp = fopen(filnam,"rb");

  else if (strcmp(disp,"o") == 0)
     fp = fopen(filnam,"r+b");

  else 
     { spout("*** Unknown disp in openoldcnt"); return FALSE; }

  if (fp ==  NULL)
     {    /* file does not exist */
     sprintf(outstr,"*** Unable to open: %s",filnam); 
     spout(outstr);  return FALSE;
     } 

#ifdef VMS
  /* set file record length to 1080 * 4 (buggy cnt files on VMS)*/
  irecln = 1080 * 4;
#else
  /* set file record length to 1080 */
  irecln = 1080;
#endif

  /* allocate space for a cnt file structure */
  if ((fileptrcnt = (FILEDATACNT *) calloc(1, sizeof(FILEDATACNT))) ==
      (FILEDATACNT *) 0) 
     {spout("*** Unable to malloc filedata in openoldcnt"); return FALSE;}

  /* allocate space for a spider file structure within FILEDATACNT */
  if ((fileptrcnt -> image = (FILEDATA *) malloc(sizeof(FILEDATA))) ==
      (FILEDATA *) 0) 
     {spout("*** Unable to malloc image in openoldcnt."); return FALSE; }
  fileptrimg = fileptrcnt -> image;

  if ((fileptrimg -> buf = (float *) malloc(irecln * sizeof(float))) == 
      (float *) 0) 
     {spout("*** Unable to malloc buf in openoldcnt."); return FALSE; }
  buf = fileptrimg -> buf;

  /* set offset for reading header from file */
  fileptrimg -> offset = 0;
  fileptrimg -> fp     = fp;

  fileptrcnt -> irecln = irecln;


  /* read first record of cnt file */
  if (redlin(fileptrimg,&buf[0],30,1) != TRUE)
     { closefilecnt(fileptrcnt); return FALSE; }

  fileptrcnt -> maxcnt  =  buf[0];
  fileptrcnt -> maxtxt  =  buf[1];
  fileptrcnt -> maxrec  =  buf[2];

  fileptrcnt -> ndtsuf  =  buf[3];
  fileptrcnt -> ntxsuf  =  buf[4];
  fileptrcnt -> ifrmcs  =  buf[5];
  fileptrcnt -> scale   =  buf[6];
  fileptrcnt -> xmax    =  buf[7];
  fileptrcnt -> xmin    =  buf[8];
  fileptrcnt -> ymax    =  buf[9];
  fileptrcnt -> ymin    =  buf[10];

  fileptrcnt -> icomp   =  buf[11];
  fileptrcnt -> ndtctl  =  buf[12];
  fileptrcnt -> ncntrs  =  buf[13];
  fileptrcnt -> istcon  =  buf[14];
  fileptrcnt -> lstcon  =  buf[15];
  fileptrcnt -> isttxt  =  buf[16];
  fileptrcnt -> lsttxt  =  buf[17];
  fileptrcnt -> ntxt    =  buf[18];
  fileptrcnt -> pixpcm  =  buf[19];
  fileptrcnt -> lencor  =  buf[20];
  fileptrcnt -> icntfmt =  buf[21];


#ifdef __osf__
printf("maxcnt: %d, lencor: %d  \n", 
fileptrcnt -> maxcnt, fileptrcnt -> lencor);
#endif


  /* read second record of cnt file */

  if ((cbuf = (char *) malloc(irecln * 4 * sizeof(char))) == (char *) 0) 
     {spout("*** Unable to malloc cbuf in openoldcnt."); return FALSE; }

  if (redlin8(fileptrimg,(unsigned char *) &cbuf[0],irecln*4,2) != TRUE)
     { closefilecnt(fileptrcnt); return FALSE; }
 
  /* set date in current file information */
  strncpy(&fileptrcnt -> date[0], &cbuf[0],11);
  fileptrcnt -> date[11] = '\0';

  /* set time in current file information */
  strncpy(&fileptrcnt -> time[0], &cbuf[10],8);
  fileptrcnt -> time[8] = '\0';

  /* set title in current file information */
  strncpy(&fileptrcnt -> title[0], &cbuf[18], 40);
  fileptrcnt -> title[40] = '\0';
  if (cbuf) free(cbuf);

  /* read third record of cnt file for textctr */
  if (redlin(fileptrimg, &buf[0], irecln, 3)  != TRUE)
     { closefilecnt(fileptrcnt); return FALSE; }

  /* allocate space for a txtctr */
  if ((txtctr = (float *) malloc(sizeof(float) * fileptrcnt -> maxtxt * 5)) ==
      (float *) 0) 
     {spout("*** Unable to malloc txtctr in openoldcnt"); return FALSE; }
  fileptrcnt -> txtctr = txtctr;

  for (i = 0; i < fileptrcnt -> maxtxt * 5; i++)
     txtctr[i] = buf[i];

  /* READ RECORD NOS. 5 THRU NDTCTL + 4 AND INITIALIZE ARRAY CONTRL
     FIRST CALCULATE MAXIMUM VALID NUMBERS MAXSUF STORED IN ONE 
     RECORD THEN READ ONE RECORD AT A TIME AND COPY MAXSUF NUMBERS TO 
     THE CONTROL ARRAY AND THEN READ THE NEXT RECORD. */

  /* allocate space for contrl */
  if ((contrl = (float *) malloc(sizeof(float) * fileptrcnt -> ncntrs * 6)) ==
      (float *) 0) 
     {spout("*** Unable to malloc contrl in openoldcnt"); return FALSE; }
  fileptrcnt -> contrl = contrl;

  irec   = 4;
  maxsuf = irecln / 6;
  maxsuf = maxsuf * 6;

  j = maxsuf;
  for (i = 0; i < fileptrcnt -> ncntrs * 6; i++)
     {
     if (j >= maxsuf)
        {
        j = 0;
        if (redlin(fileptrimg,&buf[0],irecln,++irec) != TRUE)
           { closefilecnt(fileptrcnt); return FALSE; }
        }
     contrl[i] = buf[j++];

#ifdef __osf__
printf("contrl[%d]: %f   \n", i,contrl[i]);
#endif
     }
 

  /* print filename and title on one line */
  sprintf(outstr,"Contour metafile: %s %s", filnam,fileptrcnt -> title);
  spout(outstr);

  /******************* print scale factor 
  sprintf(outstr,"Scale: %f", fileptrcnt -> scale);
  spout(outstr);
  ****************/

  return fileptrcnt;
}





   

  


