
/***************************************************************************
* powermen.c  created from wid_powbut.for by
*             Mahieddine Ladjadj          5/24/94
*
* WID_POWBUT.FOR  -- FEB 93
*
* **********************************************************************
* *  AUTHOR: ArDean Leith                                                  *
* *  COPYRIGHT (C)1993, WADWSWORTH CENTER FOR LABORATORIES AND RESEARCH,   *
* *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
* *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
* *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
* *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
* *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
* **********************************************************************
*
*    powermen
*
*    PURPOSE:      MOVES A POWER SPECTRUM WINDOW AROUND IN AN IMAGE
*
*    CALLED BY:    winsel
*
************************************************************************
*/

#include "common.h"
#include "routines.h"

void  powerbut_pop (Widget, XEvent *, String *, Cardinal *);

static int first, erase, chksiz;
static GC  icontxxt;

/****************************  powermen  ******************************/

void powermen(void)
  {
  /* xor graphics context */
  icontxxt = setxor(icontx);

  /* set default color in temporary GC for box */
  wicolor(icontxxt,icolorov);

  first = chksiz = TRUE;
  erase = FALSE;

  showbutx("Set temporary power spectrum.",
           "Set permanent power spectrum.",
           "Stop this routine.",FALSE);

  /* setup movement on pointer motion, button press etc... */
  actions(iw_win,"powerbut_pop",powerbut_pop,"M123");

  XFlush(idispl);
  }

/***************************  powerbut_pop  ********************************/
void powerbut_pop(Widget iw_t, XEvent *event, String *params,
               Cardinal *num_params)
 {
 static int ixp, iyp, ixb, iyb;
 int ixs, iys;

 if (!(strcmp(*params, "M")))
     {  /**************************************** mouse movement only */
     getloc(event,'M',&ixs,&iys);
 
     /* box moved, erase old box (if necessary) --- but none */
     xorbox(iwin,icontxxt, FALSE, ixs, iys, nsamw, nroww);
     first = FALSE;
     }

 else if (!(strcmp(*params, "1")))
     {                          /******************   button 1 pushed */
     getloc(event,'B',&ixs,&iys);

     /* show temporary power spectrum  */
     /* erase old box first and put a new one */
     xorbox(iwin,icontxxt, TRUE, ixs, iys, nsamw, nroww);

     /* replace last spectrum area before continuing */
     if (erase)
         XCopyArea(idispl,imagsav,iwin,icontx,
                   ixp,iyp,nsamw,nroww,ixp,iyp);

     /* show power spectrum */
     ixulw = ixs; iyulw = iys;
     wid_power(FALSE,&chksiz); 

     /* remember last spectrum position */
     ixp = ixulw;  iyp = iyulw;
     erase = first = TRUE;
     }

  else if (!(strcmp(*params, "2")))
     {  /***  Make a permanent power spectrum.        button 2 pushed */
     getloc(event,'B',&ixs,&iys);
   
     /* show temporary power spectrum  */
     /* erase old box first and put a new one */
     xorbox(iwin,icontxxt, TRUE, ixs, iys, nsamw, nroww);

     /* replace last spectrum area before continuing */
     if (erase)
        XCopyArea(idispl,imagsav,iwin,icontx,
                   ixp,iyp,nsamw,nroww,ixp,iyp);

     /* show power spectrum */
     ixulw = ixs;  iyulw = iys;

     wid_power(TRUE,&chksiz);

     /* remember last spectrum position */
     ixp = ixulw;  iyp = iyulw;
     erase = FALSE;
     first = TRUE;
     }

  else if (!(strcmp(*params, "3")))
     {  /***  stop this routine.                     button 3 pushed */
     /* erase old box first */
     xorbox(iwin,icontxxt, TRUE, ixb, iyb, nsamw, nroww);

     /* replace last spectrum area before continuing */
     if (erase)
        XCopyArea(idispl,imagsav,iwin,icontx,
                   ixp,iyp,nsamw,nroww,ixp,iyp);  

     XFlush(idispl);
     defluts(TRUE);

     /* uninstall translations and free GC */
     XtUninstallTranslations(iw_t);
     XFreeGC(idispl,icontxxt);

     /* unmanage information box */
     showbutx("","","",TRUE);
     }
  }
