/*$Header: /net/ithaca/usr1/web/src/RCS/profile.c,v 1.10 1995/11/14 16:14:00 liy Exp $*/
/*
*****************************************************************************
*  profile.c         6/10/93
*
*  wid_profile.for (in VAX) ported to UNIX
*
* **************************************************************************
* *  AUTHOR:  Mahieddine Ladjadj                                           *
* *  COPYRIGHT (C)1990 , WADSWORTH CENTER FOR LABORATORIES AND             *
* *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
* *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
* *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
* *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
* *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
* **************************************************************************
*
*    PROFILE()
*
*    PURPOSE:        PLOTS A PROFILE THROUGH ROW OR COL OF AN IMAGE FILE
*
*    PARAMETERS:     NONE
*
*
*****************************************************************************
*/

#include "common.h"
#include "routines.h"

/* internal subroutine prototypes */
 void  profile_pop     (Widget, XEvent *, String *, Cardinal *);
 void  profile_plot    (float *);
 void  profilemen      (void);
 void  profilemen_buts (int, int, int);
 void  profile_line    (int);

 /* internal common variables */
 int   first , horizbar , leavebar, normbar, permbar, doxyaxis, permxyaxis ;
 int   spline;		/* the special line flag */
 int   linenum;		/* the line number of the special line */
 float *buft;
 int LS0, ls;           /* line style of the icontxm GC */
 int LW0, lw;           /* line width of the icontxm GC */

 /* external common variables */
 GC     icontxm;

 /***************************  profile  *******************************/

 void profile(void)
 {
   first = horizbar = normbar = TRUE;
   leavebar = permbar = spline = FALSE;
   linenum = 0;

   /* avoid drawing X and Y axis more than once */
   doxyaxis = 2; permxyaxis =2;

   /* run-time allocation of memory space */
   buft = (float *) calloc(nrow, sizeof(float));
   if (buft == (float *) NULL)
       {  spout("*** Calloc failed in profile"); return; }

   /* set up a xor graphics context */
   icontxm = setxor(icontx);

   /* open a message window with the following strings */
   showbutx("Image Profile.",
            "Display menu.",
            "Stop this routine.", FALSE);

   /* set the actions for right, left, and center buttons */
   actions(iw_win, "profile_pop",profile_pop,"M123");

   /* set color for the overlay */
   wicolor(icontxm,icolorov);

   /* refresh screen */
   expos_cb(NULL,NULL,NULL);
 }

 /********************** profile_pop  *******************/

 void profile_pop(Widget iw_t, XEvent *event, String *params,
                   Cardinal *num_params)
   {
   XGCValues     gcv;
   int           ixc, iyc, ixi, iyi, ix2, iy2, ix3, iy3;
   static int    ix0, iy0, ix1, iy1;
   int           irec, irow, idum;
   float         *buf;
   char          outstr[80];

   /* get current line style and width */
   /* line_style is LineSolid(0), LineOnOffDash(1), ... */
   /* line_width is 0,1,...,5                        */
   if(first)
      { /* save the line width and style of the GC */
      XGetGCValues(idispl, icontx, GCLineWidth | GCLineStyle, &gcv);
      LW0 = gcv.line_width;
      LS0 = gcv.line_style;
      ls = LS0; lw = LW0 - 1;
      }

   if (!(strcmp(*params,"M")))                 /* cursor moved */
       {
       /* get mouse position ixc,iyc  */
       getloc(event,'M',&ixc,&iyc);

       /*  find location relative to image */
       ixi = ixc - ixul + 1;
       iyi = iyc - iyul + 1;

       if ( (ixi < 1 || ixi > nsam ) || (iyi < 1 || iyi > nrow) )
           { /*  outside of image. */
           ;
           }
       else if (horizbar)
           {
	   /* if draw the special line */
	   if (spline) {
	      iyi = linenum;
	      iyc = linenum + iyul - 1;
	      }

           /* horizontal bar inside image, report location. */
           sprintf(outstr,"Row inside image: %d$", iyi); 
	   spout(outstr);

           /* find bar ends */
           ix2 = ixul;             iy2 = iyc;
           ix3 = ixul + nsam -1;   iy3 = iyc;
           if(!first)
              xorline(iwin,icontxm,FALSE, ix2,iy2, ix3,iy3);
           else
              {
              xorline(iwin,icontxm,FALSE, ix2,iy2, ix3,iy3);
              first = FALSE;
              }
	   }
       else
	   {
	   /* if draw the special line */
	   if (spline) {
	      ixi = linenum;
	      ixc = linenum + ixul - 1;
	      }

           /* vertical bar inside image, report location. */
           sprintf(outstr,"Col. inside image: %d$", ixi);
           spout(outstr);

           /* find bar ends */
           ix2 = ixc;    iy2 = iyul;
           ix3 = ixc;    iy3 = iyul + nrow -1;
           if(!first)
              xorline(iwin,icontxm,FALSE, ix2,iy2, ix3,iy3);
           else
              {
              xorline(iwin,icontxm,FALSE, ix2,iy2, ix3,iy3);
              first = FALSE;
              }
           }
       }

   else if (!(strcmp(*params,"1")))            /* button 1 pushed */
       {
       /* get mouse position ixc,iyc  */
       getloc(event,'M',&ixc,&iyc);

       /*  find location relative to image */
       if (!spline) {
           ixi = ixc - ixul + 1;  
	   iyi = iyc - iyul + 1;
	   }
       else {
	   if (horizbar) {
	      ixi = nsam;
	      iyi = linenum;
	      }
	   else {
	      ixi = linenum;
	      iyi = nrow;
	      }
	   }

       if ( (ixi < 1 || ixi > nsam) || (iyi < 1 || iyi > nrow) )
	   {  /*  outside of image. can not select this location */
           ;
	   }
       else if (horizbar)
	   {
           /* horizontal bar inside image, report location. */
           sprintf(outstr,"Profiled row: %d", iyi);  spout(outstr);

           /* find bar ends */
           ix2 = ixul;              iy2 = iyc;
           ix3 = ixul + nsam -1;    iy3 = iyc;

           if (leavebar)
	      {
              xorline(iwin,icontxm,TRUE, ix2, iy2, ix3, iy3);

              /* save bar in backing store */
              xorline(imagsav,icontxm,TRUE, ix2, iy2, ix3,iy3);
	      }
           else
              {
              if(!first)
                xorline(iwin,icontxm,FALSE, ix2,iy2, ix3,iy3);
              else
                 {
                 xorline(iwin,icontxm,FALSE, ix2,iy2, ix3,iy3);
                 first = FALSE;
                 }
              }

           /* read row from image file */
	   if (!spline)
               irec = (islice - 1) * nrow + iyi;
	   else 	/* special line */
               irec = (islice - 1) * nrow + linenum;

           buf  = filedata -> buf;
           if (redlin(filedata,buf,nsam,irec))
               profile_plot(buf);

           /* plotting the profile has changed the static x0,y0 & x1,y1
           ** in xorline. delete the last bar */
           if (!leavebar)
              xorline(iwin,icontxm,TRUE, ix2, iy2, ix3, iy3);

	   }
       else
	   {
           /* vertical bar inside image, report location. */
           sprintf(outstr,"Profiled col.: %d", ixi); spout(outstr);

           /* find bar ends */
           ix2 = ixc;     iy2 = iyul;
           ix3 = ixc;     iy3 = iyul + nrow -1;

           if (leavebar)
	      {
              xorline(iwin,icontxm,TRUE, ix2,iy2, ix3,iy3);

              /* save bar in backing store */
              xorline(imagsav,icontxm,TRUE, ix2,iy2, ix3,iy3);
	      }
           else
              { 
              if(!first)
                 xorline(iwin,icontxm,FALSE, ix2,iy2, ix3,iy3);
              else
                 {
                 xorline(iwin,icontxm,FALSE, ix2,iy2, ix3,iy3);
                 first = FALSE;
                 }
              }

           /* read col from image file */
           buf = filedata -> buf;
           for (irow = 0; irow < nrow; irow++)
	      {
              irec = (islice - 1) * nrow + irow;
              redlin(filedata,buf,nsam,irec);
              buft[irow] = buf[ixi];
	      }
           profile_plot(buft);

           /* plotting the profile has changed the static x0,y0 & x1,y1
           ** in xorline. delete the last bar */
           if (!leavebar)
              xorline(iwin,icontxm,TRUE, ix2, iy2, ix3, iy3);

	   }

       }                                       

   else if (!(strcmp(*params,"2")))           /* button 2 pushed */
       { /* get mouse position ixc,iyc  */
       getloc(event,'M',&ixc,&iyc);

       if(horizbar) 
           {
           ix2 = ixul;              iy2 = iyc;
           ix3 = ixul + nsam -1;    iy3 = iyc;
           }
       else
           {
           ix2 = ixc;     iy2 = iyul;
           ix3 = ixc;     iy3 = iyul + nrow -1;
           }

       if(!first)
          xorline(iwin,icontxm,TRUE, ix2,iy2, ix3,iy3);
       else
          {
          xorline(iwin,icontxm,TRUE, ix2,iy2, ix3,iy3);
          first = FALSE;
          }
 
       /* menu  callup */
       profilemen();

       if (!permbar) { permxyaxis=2; doxyaxis=2; }
       }

   else if (!(strcmp(*params,"3")))           /* button 3 pushed */
       {
       /* free the heap memory space */
       if(buft) free(buft);

       /* stop this routine */
       profilemen_buts(idum,idum,idum);
       }

 return;
 }

/*********************  profile_plot  *****************************/

 void  profile_plot(float *buf)
 {
   int    xline, yline, i, xpsize, ypsize;
   int    ixend, ixori, iyori, ix, itick;
   int    iytop, iybot, ixgo, iygo, iy;
   float  bufmin,bufmax,vrange,vscale,hscale;

   xline = TRUE;
   yline = TRUE;

   if (normbar)
       {
       /* find the max y value in the buffer */
       bufmax = buf[0];
       bufmin = buf[0];
       for (i = 1; i < nsam; i++)
	  {
          bufmax = MYMAX(bufmax, buf[i]);
          bufmin = MYMIN(bufmin, buf[i]);
	  }
       }
   else
       {
       /* use fmin and fmax for scaling */
       bufmax = fmax;
       bufmin = fmin;
       }

   vrange = bufmax - bufmin;
   if (vrange < FLTZER)
      {spout("*** NULL PROFILE");  return; }

   /* set plot size */
   xpsize = iwidex - 100;
   ypsize = ihighx - 100;

   /* find x and y scaling facters */
   hscale = xpsize / nsam;
   vscale = ypsize / vrange;

   /* find origin of plot */
   ixori = ((iwidex - xpsize) / 2) + hscale;
   iyori = ihighx - (ihighx - ypsize) / 2;

   if (xline)
       {
       /* draw horizontal X axis */
       ixend = nsam * hscale + ixori - hscale;
       if(doxyaxis > 0) 
          {
          xorline(iwin,icontxm,TRUE, ixori, iyori, ixend, iyori);
          doxyaxis--;
          }
       if (permxyaxis > 0)
          {
          if (permbar)
              xorline(imagsav,icontxm,TRUE, ixori, iyori,
                                            ixend, iyori);
          permxyaxis--;
          }

       /* add tick marks */
       itick = 50;
       if (nsam < 20 )
          itick = 1;
       else if (nsam < 300)
          itick = 10;

       for (i = 0; i < nsam ; i += itick)
	   {
           ix = i * hscale + ixori - hscale;
           xorline(iwin,icontxm,TRUE, ix, iyori, ix, iyori + 4);
           if (permbar)
             xorline(imagsav,icontxm,TRUE, ix, iyori, ix, iyori + 4);
	   }
       }
     
   if (yline)
       {
       /* draw vertical Y axis */
       iytop = iyori - (vrange * vscale);
       iybot = iyori;

       /* draw Y axis */
       if (doxyaxis > 0)
          {
          xorline(iwin,icontxm,TRUE, ixori, iybot, ixori, iytop);
          doxyaxis--;
          }

       if (permxyaxis > 0)
          {
          if (permbar)
             xorline(imagsav,icontxm,TRUE, ixori, iybot,
                                           ixori, iytop);
          permxyaxis--;
          }

       /* put tick at bottom of Y axis */
       xorline(iwin,icontxm,TRUE, ixori-4, iybot, ixori, iybot);
       if (permbar)
         xorline(imagsav,icontxm,TRUE, ixori-4, iybot, ixori, iybot);

       /* put tick at top of Y axis */
       xorline(iwin,icontxm,TRUE, ixori-4, iytop, ixori, iytop);
       if (permbar)
         xorline(imagsav,icontxm,TRUE, ixori-4, iytop, ixori, iytop);
       }

     /* plot the profile */
     ixgo = ixori;
     iygo = iyori;

     /* select a line style and width for icontxm GC */

     profile_line(1);
     for (i = 0; i < nsam; i++)
        {
        ix = i * hscale + ixori;
        iy = iyori - ((buf[i] - bufmin) * vscale);
        xorline(iwin,icontxm,TRUE, ixgo, iygo, ix, iy);
        if (permbar)
           xorline(imagsav,icontxm,TRUE, ixgo, iygo, ix, iy);
        ixgo = ix;
        iygo = iy;
        }


     /* reset a line style and width for icontxm GC */
     profile_line(0);

 return;
 }

/*********************  profile_line  *****************************/

 void  profile_line(int doit)
   {
   XGCValues     gcv;
   int ls1, lw1;

   if (doit)
       {  /* change line style and width if icontxm GC */
       if (lw >= 5) 
           { /* reset line width and change line style */
           lw = 1;
           if (ls >= 2 ) 
             ls = 0;
           else
             ls++;
           }
       else
           {/* next line width */
           lw++;
           }
       ls1 = ls; lw1 = lw;
       }
   else
       {  /* reset line style and width if icontxm GC */
       ls1 = LS0; lw1 = LW0;
       }

   gcv.line_width = lw1;
   switch( ls1 )
       {
         case 0:   gcv.line_style = LineSolid;        break;
         case 1:   gcv.line_style = LineOnOffDash;    break;
         case 2:   gcv.line_style = LineDoubleDash;   break;
       }

   XChangeGC( idispl, icontxm, GCLineWidth | GCLineStyle, &gcv);


   }
