/*$Header: /net/ithaca/usr1/web/src/RCS/redlin.c,v 1.8 1995/05/08 15:12:24 leith Exp $*/

/*
 *************************************************************************
 *
 * redlin.c
 *
 *************************************************************************
 * AUTHOR: A. LEITH 
 * COPYRIGHT (C)1981,1992, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 **************************************************************************
 *
 * redlin(fileptr,buf,nsam,irec)
 * 
 * PURPOSE:     
 *
 * RETURNS:      TRUE IF OK, FALSE OTHERWISE
 * 
 * PARAMETERS:   fileptr
 *               buf
 *               nsam
 *               irec    
 *
 **********************************************************************/

#include "files.h"

#ifdef P_SGIOSF
unsigned int *pt;
#define CONVERT_4( A, B )                        \
             (unsigned int)(A) =                 \
            ((unsigned int)(B) >> 24) |          \
            ((unsigned int)(B) << 24) |          \
           (((unsigned int)(B) >> 8) & 0xff00) | \
           (((unsigned int)(B) << 8) & 0xff0000)

#endif

 int redlin8f(FILEDATA *fileptr, float *buf, int nsam, int irec);

 /* *************************** redlin *******************************/

 int redlin(FILEDATA *fileptr, float *buf, int nsam, int irec)
 {

 FILE      *fp;
 int       ioff,it,k1,ot;
 size_t    igot;

 if (fileptr -> iform == 8)
    {    /* 8-bit raw file */
    return(redlin8f(fileptr, buf, nsam, irec));
    }

 /* find number of header and record bytes to be skipped */
 if (irec != 0) 
    ioff  = fileptr -> offset + (irec -1) * nsam * 4;
 else
    ioff  = 0;


 /* find file pointer for this file */
 fp    = fileptr -> fp;


 if ((igot = fseek(fp,ioff,SEEK_SET)) != 0) 
     {       /* offset seek failed */
     printf("*** seek failed in redlin: %d \n",igot);
     printf(" igot: % d, ioff: %d ,nsam: %d \n",igot,ioff,nsam);
     return FALSE;
     }

 /* offset seek was successful, read record number nsam */
 igot = fread(buf, sizeof(float), nsam, fp);
 
#ifdef  P_SGIOSF
   pt = (unsigned int *) buf;
   /* flip the bytes for date and time in header */
   for (k1 = 0; k1 < nsam; k1++, pt++)
      { CONVERT_4(*pt,*pt);  }
#endif

 if (feof(fp))
      {
      printf("*** End of file in redlin \n");
      printf("igot: %d, ioff: %d ,nsam: %d irec: %d\n",
              igot,ioff,nsam,irec);
      printf(" fp->nrow: %d, fp->offset: %d \n",
                fileptr->nrow,fileptr->offset);
      return FALSE;
      }          

 if (ferror(fp))
      {
      printf("*** Error reading input file in redlin \n");
      return FALSE;
      }

 return TRUE;
 }

