
/*$Header: /net/ithaca/usr1/web/src/RCS/redvol.c,v 1.6 1996/01/22 21:20:29 leith Exp $*/

/*
C***********************************************************************
C
C   REDVOL.C  -- CREATED NOV 13 1993
C             
C *****************************************************************************
C *  AUTHOR: ArDean Leith                                                  *
C *  COPYRIGHT (C)1993, WADSWORTH CENTER FOR LABORATORIES AND RESEARCH     *
C *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH  			   *
C **********************************************************************
C
C  REDVOL(FILEDATA,  WANT8, CPTR, WANT32, FPTR, NSAM1, NSAM2,   
C         NROW1, NROW2,  NSLICE1, NSLICE2,  NMINT, NMAXT,  
C         FMINT FMAXT, SCAL, OFFSET)
C           
C
C  PURPOSE:     READS SPIDER IMAGE INTO 8 / OR 32 BIT ARRAY NORMALIZED 
                FROM NMIN TO NMAX.  RETURNS NORMALIZTION CONSTANTS ALSO.
C
C  PARAMETERS:  FILEDATAT
C               WANT8, WANT32      WANT 8 BIT BUFFER, WANT 32 BIT BUFFER
C               NSAM1,NSAM2        SAMPLE RANGE               (SENT)
C               NROW1,NROW2        ROW RANGE                  (SENT)
C               NSLICE1,NSLICE2    SLICE RANGE                (SENT)
C               NMINT,NMAXT,       TARGET NORMALIZATION RANGE (SENT)
C               FMIN,FMAX,         DESIRED FILE RANGE         (SENT)
C               SCAL, OFFSET       NORMALIZATION PAREMETERS   (RETURNED)
C               ISKIPT             SKIP (MAG. ) FACTOR        (SENT)
C
C  RETURNS:     TRUE OR FALSE 
C
C  NOTE:        for char variables char = 128 is same as char = -128
C               so no manipulation is necessary.
C               The old if(char > 127) char = char - 256 is NOT needed.
C
C--*********************************************************************
*/

#include "files.h"
#include "routines.h"

 /* externally defined variables */
 extern int imagego, imageend;        /* set norm. range for image */

 /* static variables for use here */

 /* common variables used here */
 int nsams, nrows, nslices;

 /*************************   redvol   ****************************/

 int redvol(FILEDATA *filedata, 
            int want8,    char  ** cptr,
            int want32,   float ** fptr,
            int nsam1,    int nsam2,   
            int nrow1,    int nrow2, 
            int nslice1,  int nslice2, 
            int nmint,    int nmaxt, 
            float fmint,  float fmaxt, 
            float * scal, float * offset,
            int iskipt)

 {
 int         nsam, nrow, irec, j, m, npixlin, miskip, npixcol;
 int         islice, irow, isam, ival, newmin, noff;
 float       *buf,  *fptrt,  *fptrtt, *fptrttt;
 char        *cptrt, *cptrtt, *cptrttt;
 float       buft;

 nsam    = filedata -> nsam;
 nrow    = filedata -> nrow;

 /* find nsams, nrows in displayed image (do not expand nslices)*/
 dispsize(nsam1,nsam2,nrow1,nrow2,nslice1,nslice2,iskipt,
         &nsams, &nrows, &nslices);

 /* find buf for redlin */
 buf    = filedata -> buf;

 newmin = (fmint != filedata-> fmin || fmaxt != filedata->fmax) ;

 if (want8)
    {
    /* find normalization constants for 8 bit output data */
    *scal    = (nmaxt - nmint) / (fmaxt - fmint);
    *offset  = -fmint  * *scal + nmint + 0.5;

    if (!(*cptr)) 
       {
        /* allocate cbuf space first */
        if ((*cptr = (char *) malloc(nsams * nrows * nslices *
                      sizeof(char))) ==  (char *) 0)
            { 
            spout(" *** Unable to allocate memory in redvol.");
            return FALSE;
            }
       }
    }

 if (want32 && !(*fptr)) 
    {
    /* allocate float space first */
    if ((*fptr = (float *) malloc(nsams * nrows * nslices *
                      sizeof(float))) == (float *) 0)
        { 
        spout(" *** Unable to allocate memory in redvol.");
        return FALSE;
        }
    }

 /* set temporary data pointers */
 fptrt    = *fptr;
 cptrt    = *cptr;
 irec     = 0;


 /* fill cptr volume with 8 bit data and fptr vol with 32 bit data */

 if (iskipt > -2 && iskipt < 2)
    {    /*************************************************full size */
    for (islice = nslice1; islice <= nslice2; islice++)
       {
       for (irow = nrow1; irow <= nrow2; irow++)
          {
          irec = (islice - 1) * nrow + irow;
          if (!redlin(filedata,buf,nsam,irec)) return FALSE;
        
          if (newmin)
             { /* want to alter min & max  definition*/
             if (want8 && want32)
                {  
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   ival     = (char) (buf[isam] * *scal + *offset);
                   *cptrt++ = MYMIN(MYMAX(ival,imagego),imageend);
                   *fptrt++ = buf[isam];
                   }  /* for isam           */
                }     /* if want8 && want32 */

             else 
                {  /* want8 (can not have want32 only and newmin) */
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   ival = (char) (buf[isam] * *scal + *offset);
                   *cptrt++ = MYMIN(MYMAX(ival,imagego),imageend);
                   }  /* for isam      */
                }     /* else  (want8) */
             }        /* if newmin     */

          else
             {  /* not newmin */
             if (want8 && want32)
                {  
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   *cptrt++ = (char) (buf[isam] * *scal + *offset);
                   *fptrt++ = buf[isam];
                   }  /* for isam   */
                }     /* if want8 && want32 */

             else if (want8)
                {  
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   *cptrt++ = (char) (buf[isam] * *scal + *offset);
                   }  /* for isam   */
                }     /* if want8   */

             else
                {  /* want32 */
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   *fptrt++ = buf[isam];
                   }  /* for isam   */
                }     /* if want32  */
             }        /* not newmin */
          }           /* for irow   */
       }              /* for islice */
    }                 /* if iskipt  */


 else if (iskipt > 1)
    {    /************************************** reduced size image */
    for (islice = nslice1; islice <= nslice2; islice++)
        {
       for (irow = nrow1; irow <= nrow2; irow += iskipt)
          {
          irec = (islice - 1) * nrow + irow;
          if (!redlin(filedata,buf,nsam,irec)) return FALSE;
          
          if (newmin)
             { /* want to alter min & max  definition*/
             if (want8 && want32)
                {  
                for (isam =nsam1-1; isam < nsam2; isam += iskipt)
                   {
                   ival = (char) (buf[isam] * *scal + *offset);
                   ival = MYMIN(MYMAX(ival,imagego),imageend);
                   *cptrt++ = ival;
                   *fptrt++ = buf[isam];
                   }  /* for isam           */
                }     /* if want8 && want32 */

             else if (want8)
                {  
                for (isam =nsam1-1; isam < nsam2; isam += iskipt)
                   {
                   ival = (char) (buf[isam] * *scal + *offset);
                   ival = MYMIN(MYMAX(ival,imagego),imageend);
                   *cptrt++ = ival;
                   }  /* for isam   */
                }     /* if want8   */
             }        /* if newmin  */
          else
             {  /* not newmin */
             if (want8 && want32)
                {  
                for (isam =nsam1-1; isam < nsam2; isam += iskipt)
                   {
                   *cptrt++ = (char) (buf[isam] * *scal + *offset);
                   *fptrt++ = buf[isam];
                   }  /* for isam   */
                }     /* if want8 && want32 */

             else if (want8)
                {  
                for (isam =nsam1-1; isam < nsam2; isam += iskipt)
                   {
                   *cptrt++ = (char) (buf[isam] * *scal + *offset);
                   }  /* for isam      */
                }     /* if want8      */

             else
                { /* want32 only */ 
                for (isam =nsam1-1; isam < nsam2; isam += iskipt)
                   {
                   *fptrt++ = buf[isam];
                   }  /* for isam      */
                }     /* else (want32) */
             }        /* newmin        */
          }           /* for irow      */
       }              /* for islice    */
    }                 /* if iskipt > 1 */
   

 else if (iskipt < -1)
    {    /************************************* magnified size image */
    miskip  = -iskipt;
    npixlin = 0;

    for (islice = nslice1; islice <= nslice2; islice++)
       {
       for (irow = nrow1; irow <= nrow2; irow++)
          {
          irec = (islice - 1) * nrow + irow;
          if (!redlin(filedata,buf,nsam,irec)) return FALSE;
          cptrtt  = cptrt + npixlin;
          fptrtt  = fptrt + npixlin;

          /* npixlin & npixcol used to set corner of active image area*/
          npixlin += miskip * nsams;
          npixcol = 0; 

          if (newmin)
             { /* want to alter min & max  definition*/
             if (want8 && want32)
                { 
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   buft    = buf[isam];
                   ival    = (char) (buft * *scal + *offset);
                   ival    = MYMIN(MYMAX(ival,imagego),imageend);
                   cptrttt = cptrtt + npixcol - 1;
                   fptrttt = fptrtt + npixcol - 1;
                   npixcol += miskip;

                   for (j=0; j < miskip; j++)
                      {
                      noff = 0;
                      cptrttt++;
                      fptrttt++;
                      for (m=0; m < miskip; m++)
                         {
                         *(cptrttt + noff) = ival;
                         *(fptrttt + noff) = buft;
                         noff += nsams;
                         }  /* for m              */
                      }     /* for j              */
                   }        /* for isam           */
                }           /* if want8 && want32 */

             else if (want8)
                {  
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   ival    = (char) (buf[isam] * *scal + *offset);
                   ival    = MYMIN(MYMAX(ival,imagego),imageend);
                   cptrttt = cptrtt + npixcol - 1;
                   npixcol += miskip;

                   for (j=0; j < miskip; j++)
                      {
                      noff = 0;
                      cptrttt++;
                      for (m=0; m < miskip; m++)
                         {
                         *(cptrttt + noff) = ival;
                         noff += nsams;
                         }  /* for m      */
                      }     /* for j      */
                   }        /* for isam   */
                }           /* for want8  */
             }              /* if newmin  */

          else
             { /*not newmin*/
             if (want8 && want32)
                {  
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   buft    = buf[isam];
                   ival    = (char) (buft * *scal + *offset);
                   cptrttt = cptrtt + npixcol - 1;
                   fptrttt = fptrtt + npixcol - 1;
                   npixcol += miskip;

                   for (j=0; j < miskip; j++)
                      {
                      noff = 0;
                      cptrttt++;
                      fptrttt++;
                      for (m=0; m < miskip; m++)
                         {
                         *(cptrttt + noff) = ival;
                         *(fptrttt + noff) = buft;
                         noff += nsams;
                         }  /* for m              */
                      }     /* for j              */
                   }        /* for isam           */
                }           /* if want8 && want32 */

             else if (want8)
                {  
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   ival    = (char) (buf[isam] * *scal + *offset);
                   cptrttt = cptrtt + npixcol - 1;
                   npixcol += miskip;

                   for (j=0; j < miskip; j++)
                      {
                      noff = 0;
                      cptrttt++;
                      for (m=0; m < miskip; m++)
                         {
                         *(cptrttt + noff) = ival;
                         noff += nsams;
                         }  /* for m      */
                      }     /* for j      */
                   }        /* for isam   */
                }           /* for want8  */

             else
                {     /* want32 only */
                for (isam =nsam1-1; isam < nsam2; isam++)
                   {
                   buft    = buf[isam];
                   fptrttt = fptrtt + npixcol -1;
                   npixcol += miskip;

                   for (j=0; j < miskip; j++)
                      {
                      noff = 0;
                      fptrttt++;
                      for (m=0; m < miskip; m++)
                         {
                         *(fptrttt + noff) = buft;
                         noff += nsams;
                         }  /* for m          */
                      }     /* for j          */
                   }        /* for isam       */
                }           /* else (want32)  */ 
             }              /* not newmin     */
          }                 /* for irow       */
       }                    /* for islice     */
    }                       /* if iskipt      */

 return TRUE;
 }




