/*$Header: /net/ithaca/usr1/web/src/RCS/savdn1.c,v 1.15 1996/05/13 15:17:02 leith Exp $*/

/*
C++*************************************************************************
C
C savdn1.c          NOV 92 al
C
C **************************************************************************
C *  COPYRIGHT (C)1981,1987, WADSWORTH CENTER FOR LABORATORIES AND         *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH                            *
C **************************************************************************
C
C    SAVDN1(DOCNAM,DATEXC,FILEPTR,DLIST,NLIST,OPENIT,APPEND)
C
C    PURPOSE:    SUBROUTINE TO SAVE PARAMETERS IN DOCUMENT FILE ALREADY OPENED
C
C    PARAMETERS:   DOCNAM  NAME FOR DOC FILE (WITHOUT EXTENSION!)
C                  DATEXC  FILE EXTENSION
C                  FILEPTR FILEPOINTER
C                  DLIST   ARRAY CONTAINING FLOATING PT. NUMBERS TO BE SAVED.
C                  NLIST   NUMBER OF ELEMENTS IN ARRAY DLIST
C                          FIRST ELEMENT IS KEY, OTHERS ARE REGISTERS
C                  OPENIT  TRUE = FIRST CALL (OPENS FILE)
C                  APPEND  TRUE = OPEN/APPEND, FALSE = OPEN/DISCARD OLD
C
C    RETURNS:      FILEPTR OR NULL
C
C--*******************************************************************
*/

#include "std.h"

extern void  spout(char *);
extern void  spouts(char *);

/*************************  savdn1  ****************************/


 FILE * savdn1(char * docnam, char * datexct, FILE ** fpt,
               float * dlist, int nlist, int * openit, int append,
               int dateit)
 {
 FILE     * fp;
 time_t     now;
 char     * datentime, * docname;
 int        icount, ikey, i;
 size_t     slen;

 if (*openit) 
    {
    /* first call of this routine */

    /* add datexct to docnam if not already there */
    slen = strlen(docnam) + strlen(datexct) + (size_t) 2;

    if (!(docname = (char *) malloc(slen * sizeof(char))))
       {
       spout("*** Unable to allocate memory in savdn1!");
       return (FILE *) 0;
       }  

    strcpy(docname,docnam);
    if (strstr(docname,datexct) != 
               (docname + strlen(docname) - strlen(datexct)))
       {   /* add extension now */
       strcat(docname,".");  strcat(docname,datexct);
       }

    if (append )
       {fp = fopen(docname,"a+");}
    else
       {
#ifdef VMS
       /* first test if file exists */
       if ((fp = fopen(docname,"r")) != NULL ) 
          {fp = freopen(docname,"r+", fp);}
       else    
          {fp = fopen(docname,"w+");}   
#else
       fp = fopen(docname,"w+"); 
#endif
       }


    if (fp ==  (FILE *) NULL)
       {    /* file can not be opened */
       spouts("*** Unable to open: "); spout(docname);  
       free(docname); return fp;
       }

    if (dateit) 
       {  /* put date, etc in file now */
       /* find current date and time */
       now       = time(NULL);
       datentime = ctime(&now);
       /* should these pointers be freed later ? */

       fprintf(fp," ; %3.3s/%3.3s  %s   %s",
                datexct,datexct, docname, datentime);
       }

    *openit = FALSE;
    if (docname) free(docname);
    *fpt = fp;
    }
 else if (*fpt)
    {
    /* use file pointer sent from caller */
    fp = *fpt;
    }
 else
    {
    spout("*** Pgm error: NULL fp sent to savdn1 without openit");
    return (FILE *) 0;
    }


 ikey = dlist[0] + 0.5;
 if (nlist > 0 && ikey > 9999) 
    {
    spout("*** Document file keys must be < 10000");
    return fp;
    }

 if (nlist < 0)
    {
    spout("*** Continuation lines not supported by savdn1.c");
    return (FILE *) 0;
    }

 else if (nlist > 0 && nlist <= 7) 
    {
    icount = nlist - 1;

    /* ikey is the key number which identifies each line       */
    /* icount is the number of values (registers) in that line */

    fprintf(fp," %4.4d %1.1d",ikey, icount);

    for (i = 1; i <= icount; i++)
       { fprintf(fp,"%12.6f",dlist[i]);  }
    fprintf(fp,"\n");
    }

 fflush(fp);
 return fp;
 }

