
/*$Header: /net/ithaca/usr1/web/src/RCS/scaleimg.c,v 1.3 1995/09/14 18:31:56 mladjadj Exp $*/

/***********************************************************************
 *
 * scaleimg.c
 *
 ***********************************************************************
 * author : ArDean Leith
 * COPYRIGHT (C)1993, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 ***********************************************************************
 *
 * scaleimg()
 * 
 * PURPOSE:  scales a floating point into 8 bit over range nmint..nmaxt.
 *
 * RETURNS:  true or false
 *
 * PARAMETERS: 
 *
 ***********************************************************************
*/

#include "std.h"
#include "routines.h"

 /****************************  scaleimg  *****************************/
 
 int scaleimg(float * oldimage, int nsam, int nsam1, int nsam2, 
                                int nrow, int nrow1, int nrow2,
                                float fmin, float fmax, int iskip,
                                int   nmint, int   nmaxt,
                                char ** cptr)
 { 

 char   *cbuf;
 int    nsams, nrows, npix, ival, miskip;
 int    j, m, ix,iy, nrowu, idum=0;
 float  scal, fm;

 if (fabs(fmax - fmin) < FLTZER)
    { spout("*** BLANK IMAGE SKIPPED"); return FALSE; }


 /* find number of rows in displayed image */
 dispsize(nsam1,nsam2, nrow1,nrow2, idum,idum, iskip,
         &nsams, &nrows, &idum);

 if (!(*cptr))
    {  /* must allocate space for cbuf */
    if ((*cptr = (char *) malloc(nsams * nrows * sizeof(char))) 
               == (char *) 0)
       { 
       spout(" *** Unable to allocate memory in scaleimg.c");
       return FALSE;
       }
    }
 /* set temporary data pointers */
 cbuf = *cptr;

 /* CONVERSION FACTOR FROM FLOATING POINT TO LOGICAL * 1 RANGE */
 scal = (float) (nmaxt - nmint) / (fmax - fmin);
 fm   = -fmin * scal + nmint + 0.5;

 npix = 0;

 if (iskip > -2 && iskip < 2)
    {    /* display normal full size */

    for (iy = nrow1-1; iy < nrow2; iy++)
       {
       if (nmaxt < 128) 
           {   /* no need to convert numbers > 128 */   
           for (ix = nsam1 - 1; ix < nsam2; ix++)
              {
              ival = (*(oldimage + nsam * iy + ix) * scal) + fm;
              *(cbuf + npix++) = ival;
              }
           }
       else
           {   /* need to convert numbers > 128 */   
           for (ix = nsam1 - 1; ix < nsam2; ix++)
              {
              ival = (*(oldimage + nsam * iy + ix) * scal) + fm;
              /* convert 0..255  integers to -128...127 */
              if (ival > 127) ival = ival - 256;
              *(cbuf + npix++) = ival;
              }
           }
       }
    }

 else if (iskip > 1)
    {    /* display reduced size image */

    nrowu = nrow2 - (nrow2-nrow1+1) % iskip - iskip + 1;

    for (iy = nrow1-1; iy <= nrowu; iy=iy + iskip)
       {

       if (nmaxt < 128) 
           {   /* no need to convert numbers > 128 */   
           for (iy = nsam1 - 1; iy < nsam2; iy = iy + iskip)
              {
              ival = (*(oldimage + nsam * iy + ix) * scal) + fm;
              *(cbuf + npix++) = ival;
              }
           }
       else
           {   /* no need to convert numbers > 128 */   
           for (ix = nsam1 - 1; ix < nsam2; ix++)
              {
              ival = (*(oldimage + nsam * iy + ix) * scal) + fm;
              /* convert 0..255  integers to -128...127 */
              if (ival > 127) ival = ival - 256;
              *(cbuf + npix++) = ival;
              }
           }
       }
    }


 else if (iskip < -1)
    {    /* display magnified size image */
    miskip = -iskip;

    for (iy = nrow1-1; iy < nrow2; iy++)
       {
       if (nmaxt < 128) 
           {   /* no need to convert numbers > 128 */   
           for (ix = nsam1 - 1; ix < nsam2; ix = ix++)
              {
              ival = (*(oldimage + nsam * iy + ix) * scal) + fm;

              for (j = 0; j < miskip ; j++)
                 {
                 for (m = 0; m < miskip; m++)
                   *(cbuf + npix + m * nsams + j) = ival;
                 }
              npix = npix + miskip;
              }
           }
       else
           {   /* need to convert numbers > 128 */   
           for (ix = nsam1 - 1; ix < nsam2; ix++)
              {
              ival = (*(oldimage + nsam * iy + ix) * scal) + fm;
              /* convert 0..255  integers to -128...127 */
              if (ival > 127) ival = ival - 256;

              for (j = 0; j < miskip ; j++)
                 {
                 for (m = 0; m < miskip; m++)
                   *(cbuf + npix + m * nsams + j) = ival;
                 }
              npix = npix + miskip;
              }
           }
        npix = (iy - nrow1 + 1) * miskip * nsams - 1;
        }
    }

 return TRUE;
}

