
/*$Header: /net/ithaca/usr1/web/src/RCS/spout.c,v 1.7 1996/05/13 14:39:47 leith Exp $*/

/*
 ***********************************************************************
 *
 * spout.c
 *
 ***********************************************************************
 * COPYRIGHT (C)1992, WADSWORTH CENTER FOR LABORATORIES AND
 * RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      
 * THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     
 * LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  
 * USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF 
 * THE CENTER FOR LABORATORIES AND RESEARCH 			  
 ***********************************************************************
 *
 * spout(message)
 * 
 * PURPOSE:   prints out a message in message area of web
 *
 * VARIATIONS:
 *            spout :   next message goes on new line
 *            spouts:   next message does not start new line 
 *            spouto:   next message overwrites this line 
 *            spouti:   put out integer, next message does not 
 *                      start new line 
 *
 * RETURNS:   void 
 * 
 * PARAMETERS:     
 *
 ***********************************************************************
 */

#include "std.h"
#include "x.h"
#include "routines.h"
#include <Xm/Text.h>

 /* externally defined global variables */
 extern FILE      * resfp;           /* web_results file */
 extern Widget    iw_scroll;         /* scroll window widget */
 extern int       trace_web;

 /* internal static variables */
 static XmTextPosition igo  = 0; 
 static XmTextPosition iend = 0;

 /**************************** spout **********************************/

 void spout (char *message)
   {
   /* add the text after the last position */

   if (strcspn(message,"$")  == strlen(message) -1)
      {   /*  $ in last position is flag for overwrite with next line */

      /* put line in scrolling window */
      /* to insert text the values for from and to should be equal */
      XmTextReplace(iw_scroll, igo, iend, message);

      /* remember position of end of this line */
      iend = XmTextGetLastPosition(iw_scroll);

      /* remove the $ */
      XmTextReplace(iw_scroll, iend-1, iend, " ");

      XmTextSetInsertionPosition(iw_scroll, igo);
      /* next line start position same as start position of this one */
      }

   else
      {
      /* put line in scrolling window */

      XmTextReplace(iw_scroll, igo, iend, message);

      /* add a newline to the string */
      XmTextInsert(iw_scroll, XmTextGetLastPosition(iw_scroll), "\n");

      /* move cursor so XmNmotionVerifyCallback gets called */
      XmTextSetInsertionPosition(iw_scroll, XmTextGetLastPosition
                                        (iw_scroll)-1);

      /* update next line starting position */ 
      igo = XmTextGetLastPosition(iw_scroll);

      /* remember position of end of this line */
      iend = igo;

      if (trace_web) 
         {
         /* print message in file */
         fprintf(resfp,"%s\n",message);
         fflush(resfp);
         }
      }
  }

 /**************************** spouts *********************************/

 void spouts (char *message)
   {
   /* put line in scrolling window, do not add newline!!! */
   XmTextReplace(iw_scroll, igo, iend, message);

   /* move cursor so XmNmotionVerifyCallback gets called */
   XmTextSetInsertionPosition(iw_scroll, XmTextGetLastPosition
                                        (iw_scroll)-1);

   /* update next line starting position */ 
   igo = XmTextGetLastPosition(iw_scroll);

   /* remember position of end of this line */
   iend = igo;

   if (trace_web) 
      {
      /* print message in file */
      fprintf(resfp,"%s",message);
      /* no flush as line is not finished yet */
      }
  }
 /**************************** spouti *********************************/

 void spouti (int nval)
   {
   char * cval;

   /* put integer in scrolling window */

   /* convert nval to char string */
   if (!(cval = itoa(nval))) return;

   XmTextReplace(iw_scroll, igo, iend, cval);

   /* move cursor so XmNmotionVerifyCallback gets called */
   XmTextSetInsertionPosition(iw_scroll, XmTextGetLastPosition
                                        (iw_scroll)-1);

   /* update starting position */ 
   igo = XmTextGetLastPosition(iw_scroll);

   /* remember position of end of this line */
   iend = igo;

   if (trace_web) 
      {
      /* print message in file */
      fprintf(resfp,"%s",cval);
      /* no flush as line is not finished yet */
      }

   if (cval) free(cval);
   }

 /**************************** spouto *********************************/

 void spouto (char *message)
   {
   /* overwrite this text line with next incoming text line*/

   /* put line in scrolling window */
   /* to insert text the values for from and to should be equal */
   XmTextReplace(iw_scroll, igo, iend, message);

   /* remember position of end of this line */
   iend = XmTextGetLastPosition(iw_scroll);

   /* next line start position same as start position of this line */
   XmTextSetInsertionPosition(iw_scroll, igo);
   }


 
