

/*
C++************************************************************************
C
C SURFMEN  -- CREATED AS WID_SURFMEN.FOR
C             CONVERTED TO C -- OCT 92 al
C
C **************************************************************************
C *  AUTHOR:  ArDean Leith                                                 *
C *  COPYRIGHT (C)1991, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C    SURFMEN
C
C    PURPOSE:         DISPLAY SURFACE MENU
C
C    CALLED BY:       IMAGEMEN
C
C        v----------------imnum=1---------------------------^
C        v                                                  ^
C    imagemen --> wid_files --> imagemen_cb ---> imagemen1 -^
C                                  v
C                                  v (imnum=2)
C                                  v
C                                surfmen --> surfmen_buta --> surfmovie
C                                  ^           v
C                                  ^           v
C                                  ^         surf ---------> surfcolor
C                                  ^           v                v
C                                  ^---------surf_pop        redvol
C                                              v
C                                             (end)   
C
C***********************************************************************
*/

#include <Xm/Scale.h>
#include <Xm/Text.h>
#include <Xm/ToggleBG.h>

#include "common.h"
#include "routines.h"

#define MAX_ARGS 3
#define EULER	 1
#define XYZ	 0


 /* function prototypes */
 extern void   	surfmovie    (int);

 /* internal function prototypes */
 void          	surfmen_buta(Widget iw_temp, caddr_t, caddr_t);
 void          	surfmen_butc(Widget iw_temp, caddr_t, caddr_t);
 void          	angle_cb(Widget iw_temp, caddr_t, caddr_t);
 void          	surf_pop(Widget, XEvent *, String *, Cardinal *);
 void	       	setangles(int, float, float, float, float, float, float);
 void          	savesurf_cb(Widget iw_temp, caddr_t, caddr_t);
 
 /* external common variables */
 float    phi, theta, psi, thlev;
 int      canrotate;
 int      nvolcolors = 3;
 extern unsigned char  * refmap;

 /* internal common variables */
 static Widget   iw_scales[6],iw_angs[6],iw_angsxyz[6],iw_rc,iw_mov;
 static Widget   iw_thlev, iw_refl, iw_fct, iw_fast, iw_colors;
 static Widget   iw_pushc, iw_pusha, iw_sc, iw_angle;
 static Widget   iw_surfmen = (Widget) 0;
 static Widget   iw_lab, iw_rowcolh1, iw_rowcolh2, iw_rowcolv;
 static Widget	 iw_euler, iw_xyz;
 static Widget	 iw_text;		/* filename widget */

 static int      sirdit, distit, colorit;
 static float	 ang_x, ang_y, ang_z;
 static int	first=TRUE;

 /***************************   surfmen   ****************************/

 void surfmen(int sirditt, int distitt, int coloritt)

 { 
 int    i, n, ifct, irefl, iscale;
 Widget iw_lab1, iw_labb1, iw_labb2,  iw_dum;
 Widget iw_dums;
 char   cval[50];
 Arg    args[2];

 /* set flags for sirds or distance display */
 sirdit = sirditt;   distit = distitt; colorit = coloritt;

 canrotate = TRUE;
 if (nsam * nrow * nslice > 256 * 256 * 256)
    {   /* probably too big to allocate rotation memory */
    canrotate = FALSE;
    }

 if (iw_surfmen <= (Widget)0)
    {   /* create surface menu widget first */

    iw_surfmen  = wid_dialog(iw_win, 0, "Surface options", -1, -1);
    iw_rowcolv  = wid_rowcol(iw_surfmen, 'v', -1, -1);

    phi      = 0.0;
    theta    = 0.0;
    psi      = 0.0;
    gotsurf  = FALSE;

    /* create label box for view dir.  ---------------------  label  */
    sprintf(cval," Viewed along rows of volume (nrow x nslice plane)");
    iw_labb1  = wid_labelg(iw_rowcolv, 0, cval, -1, -1);

    /* create horizontal rowcol for range and fast -----------------  */
    iw_rowcolh1  = wid_rowcol(iw_rowcolv, 'h', -1, -1);

    /* create label box for image range ---------------------  range  */
    sprintf(cval,"Range:     %f ...%f",filedata->fmin,filedata->fmax);
    iw_lab  = wid_labelg(iw_rowcolh1, 0, cval, -1, -1);

    /* create label box for spacing -----------------------  spacing  */
    wid_labelg(iw_rowcolh1, 0,"   ", -1, -1);

    /* create toggle box for movie ---------------------------- movie */
    iw_mov = wid_togmen (iw_rowcolh1,0,"MOVIE",False,0,0);

    /* create label box for spacing -----------------------  spacing  */
    iw_dum = wid_labelg(iw_rowcolh1, 0,"   ", -1, -1);

    /* create toggle box for angle convertion--------- angle conversion */
    iw_angle = wid_toggleg (iw_rowcolh1,0,"Euler",False,angle_cb,"E",0,0);

    /* create horizontal rowcol for threshold and scaling ------------  */
    iw_rc  = wid_rowcol(iw_rowcolv, 'h', -1, -1);

    /* create text box  for threshold --------------------  threshold */
    /* SET DEFAULT THRESHOLD TO 50% OF RANGE */
    thlev = filedata -> fmin + 
                ((filedata->fmax - filedata->fmin) / 2.0);

    sprintf(cval,"%f",thlev);
    iw_thlev = wid_textboxb(iw_rc,0,"Threshold:",cval,10);

    /* create scale widget for scaling ---------------------- scaling */
    iscale = scaleval * 10;
    iw_sc = wid_scale(iw_rc, 0, "Scale: ",10,200,iscale,250,50,-1,-1); 

    XtSetArg( args[0], XmNdecimalPoints, 1);
    XtSetValues(iw_sc, args, 1);

    /* create text box  for colors    -----------------------  colors */
    sprintf(cval,"%d",nvolcolors);
    iw_colors = wid_textboxb(iw_rowcolv,0,"Number of colors:",cval,10);

    /* create horizontal rowcol for refl and fct ------------------   */
    iw_rowcolh2  = wid_rowcol(iw_rowcolv, 'h', -1, -1);

    /* create scale widget for % reflection -------------------- refl */
    irefl = 100 * refl;   
    iw_refl = wid_scale(iw_rowcolh2, 0, "% Reflection ", 0,100,  irefl, 
                       180,50, -1,-1);

    /* create scale widget for fct  ----------------------------- fct */
    ifct = 100.0 * fct;
    iw_fct  = wid_scale(iw_rowcolh2, 0, "% Attenuation", 0,100,  ifct, 
                       180,50, -1,-1);

    /* create scale widgets for windowing volume ------------- scales */
    for (i=0; i < 6; iw_scales[i++]  = (Widget) 0);
    iw_dums = wid_win(iw_rowcolv, &iw_scales[0], nsam, nrow, nslice);

    if (canrotate)
       {
       /* create scale widgets for angles -------------------- angles */
       for (i=0; i < 6; iw_angs[i++]  = (Widget) 0);
       iw_euler = wid_angles(iw_rowcolv, &iw_angs[0], EULER,
                         (int) phi, (int) theta, (int) psi, -1,-1);

       /* create a sub-widget of XYZ angles for later use */
       for (i=0; i < 6; iw_angsxyz[i++]  = (Widget) 0);
       iw_xyz = wid_angles(iw_rowcolv, &iw_angsxyz[0],XYZ, 
                         (int) ang_x, (int) ang_y, (int) ang_z, -1,-1);
       XtUnmanageChild(iw_xyz);
       }
    else
       {  /* create non-rotate label */
       for (i=0; i < 3; i++)
          XtUnmanageChild(iw_angs[i]);
       iw_lab1  = wid_labelg(iw_rowcolv, 0, "Too large to rotate", -1, -1);
       }

    /* create box for apply ----------------------------------- apply */
    iw_dums = wid_stdbut(iw_rowcolv, iw_surfmen, 
                        &iw_dums, &iw_pushc, &iw_pusha,  "CA",
                        fin_cb, surfmen_butc, surfmen_buta, NULL);
    }   /* (iw_surfmen <= (Widget)0)  */



 else
    {   /* surface menu widget already exists, update it */

    /* create label box for image range ---------------------  range  */
    sprintf(cval,"Range:     %f ...%f",filedata->fmin,filedata->fmax);
    iw_lab  = wid_labelg(iw_rowcolh1, iw_lab, cval, -1, -1);

    /* create text box for threshold ---------------------  threshold */
    if (!gotsurf)
       {
       /* must reset default threshold */
       thlev = filedata->fmin + 
                ((filedata->fmax - filedata->fmin) / 2.0);

       sprintf(cval,"%f",thlev);
       iw_thlev = wid_textboxb(iw_rc,iw_thlev,"Threshold:",cval,10);
       }

    /* create scale widget for scaling -------------------- scaling */
    iscale = scaleval * 10;
    iw_sc = wid_scale(iw_rc,iw_sc,"Scale: ",10,200,iscale,250,50,-1,-1); 

    /* create scale widget for % reflection -------------------- refl */
    irefl   = 100.0 * refl;
    iw_refl = wid_scale(iw_rowcolh2, iw_refl, "% Reflection ", 0,100, 
                        irefl, 180, 50, -1, -1);

    /* create scale widget for fct  ----------------------------- fct */
    ifct   = 100.0 * fct;
    iw_fct = wid_scale(iw_rowcolh2, iw_fct,  "% Attenuation", 0,100, 
                        ifct, 225, 50, -1, -1);

    /* create scale widgets for windowing volume ------------- scales */
    iw_dums = wid_win(iw_rowcolv, &iw_scales[0], nsam, nrow, nslice);

    /* create text box  for colors    -----------------------  colors */
    sprintf(cval,"%d",nvolcolors);
    iw_colors = wid_textboxb(iw_rowcolv,iw_colors,"Number of colors:",cval,10);

    if (canrotate)
       {
       /* create scale widgets for angles -------------------- angles */
       if (XmToggleButtonGadgetGetState(iw_angle)) 
	   XtManageChild(iw_xyz);
	else
	   XtManageChild(iw_euler);       
       }
    else
       {  /* create non-rotate label */
       for (i=0; i < 3; i++)
          XtUnmanageChild(iw_angs[i]);
       iw_lab1  = wid_labelg(iw_rowcolv, iw_lab1, 
                             "Too large to rotate", -1, -1);
       }
    }   /* else */

 if (colorit)
    {  /* show the color number widget */
    XtManageChild(iw_colors); 
    }
 else 
    {     /* remove unused colors widget */
    XtUnmanageChild(iw_colors); 
    } 
 XtManageChild(iw_surfmen);
 }

 /*********** cancel button callback **********************************/

 void surfmen_butc(Widget iw_temp, caddr_t data, caddr_t call_data)
   {
	if (first) {
	    if( filedata && filedata->fp ) 
		closefile(filedata);
	}
   XtUnmanageChild(iw_surfmen);
   }

 /*********** accept button callback **********************************/

 void surfmen_buta(Widget iw_temp, caddr_t data, caddr_t call_data)
 {
 char *string;
 int  iphi, itheta, ipsi, irefl, ifct, iscale;
 int  iang_x,iang_y,iang_z; 		/* xyz angles */
 int    i, iang[6];
 float  ang1[3], ang2[3], ang3[3], *angtmp, fiang[6];

 /* set the first open flag*/
 first = FALSE;
 
 /* get nsam1 & nsam2 */
 XmScaleGetValue(iw_scales[0],&nsam1);
 XmScaleGetValue(iw_scales[1],&nsam2);
 if (nsam2 < nsam1)
    { spout("*** ENDING COL MUST BE > STARTING COL"); return; }

 /* get nrow1 & nrow2 */
 XmScaleGetValue(iw_scales[2],&nrow1);
 XmScaleGetValue(iw_scales[3],&nrow2);
 if (nrow1 > nrow2)
    { spout("*** ENDING ROW MUST BE > STARTING ROW"); return; }

 /* get nslice1 & nslice2 */
 XmScaleGetValue(iw_scales[4],&nslice1);
 XmScaleGetValue(iw_scales[5],&nslice2);
 if (nslice1 > nslice2)
    { spout("*** ENDING SLICE MUST BE > STARTING SLICE"); return; }

 if (canrotate)
    {
    /* get rotation angles */
   
    if (XmToggleButtonGadgetGetState(iw_angle))
        {    /* XYZ angles */
        for (i=0; i<6; i++) 
    	    XmScaleGetValue(iw_angsxyz[i],&iang[i]);
	    
        for (i=0; i<3; i++) {
	    ang1[i] = (float)iang[i];
	    ang2[i] = (float)iang[i+3];
	    }

    	xyz_to_eul(&ang1[0], &ang1[1], &ang1[2]);
    	xyz_to_eul(&ang2[0], &ang2[1], &ang2[2]);
        angtmp = cald(ang2, ang1);
	for (i=0; i<3; i++) {
	    ang3[i] = *angtmp;
	    angtmp++;
	    }

	phi = ang3[0]; theta = ang3[1]; psi = ang3[2];
	
    	eul_to_xyz(&ang3[0], &ang3[1], &ang3[2]);
	ang_x = ang3[0]; ang_y = ang3[1]; ang_z = ang3[2];

        /* reset the angle values */
        setangles(XYZ, ang_x, ang_y, ang_z, 0, 0, 0);
    	}
    else
        {    /* euler angles */
        for (i=0; i<6; i++)
    	    XmScaleGetValue(iw_angs[i],&iang[i]);
	    
        for (i=0; i<3; i++) {
	    ang1[i] = (float)iang[i];
	    ang2[i] = (float)iang[i+3];
	    }

	if (ang2[0]==0 && ang2[1]==0 && ang2[2]==0) {	/* increment is 0 */
	    phi = ang1[0]; theta = ang1[1]; psi = ang1[2];
	    }
	else {
    	    xyz_to_eul(&ang2[0], &ang2[1], &ang2[2]);
            angtmp = cald(ang2, ang1);

	    for (i=0; i<3; i++) {
	    	ang3[i] = *angtmp;
	    	angtmp++;
	        }
	phi = ang3[0]; theta = ang3[1]; psi = ang3[2];
	}

        /* reset the angle values */
        setangles(EULER, phi, theta, psi, 0, 0, 0);
	}
    }

 /* get scaling value */
 XmScaleGetValue(iw_sc,&iscale);
 scaleval = iscale/10.0;

 /* get reflection percentage */
 XmScaleGetValue(iw_refl,&irefl);
 refl = (float) irefl / 100.0;

 /* get fct percentage */
 XmScaleGetValue(iw_fct,&ifct);
 fct = (float) ifct / 100.0; 

 /* find threshold */
 string = XmTextGetString(iw_thlev);
 sscanf(string,"%f",&thlev);
 if (thlev > filedata -> fmax) thlev = filedata -> fmax;
 if (thlev < filedata -> fmin) thlev = filedata -> fmin;
 if (string) free(string);

 if (colorit) 
    {   /* get number of colors to be used on surface */
    string = XmTextGetString(iw_colors);
    sscanf(string,"%d",&nvolcolors);
    if (nvolcolors < 1)
       { spout("*** NUMBER OF SURFACE COLORS MUST BE > 0"); return; }
    if (nvolcolors > 99)
       { spout("*** NUMBER OF SURFACE COLORS MUST BE < 100"); return; }
    if (nvolcolors > 7)
       { spout("*** SURFACE COLORMAP ONLY SUPPORTS 7 COLORS!!");}
    }

 XtUnmanageChild(iw_surfmen);

 /* refresh screen */
 expos_cb(NULL,NULL,NULL); 
 XFlush(idispl);

 XmUpdateDisplay(iw_win);

 /* display surfaces */
 if (XmToggleButtonGadgetGetState(iw_mov))
     surfmovie(canrotate);
 else
   {
   surf(canrotate,sirdit,distit,colorit);

   /* display menu */
   showbutx("Recalls Menu.", 
          "Save the current image.",
          "Stop surfacing.",FALSE); 

   /* define buttons for menu retrieval or stop */
   actions(iw_win, "surf_pop", surf_pop, "123");
   }
 }

 /*************************** surf_pop **********************************/

 void surf_pop(Widget iw_t, XEvent *event, String *params,
                   Cardinal *num_params)
 {

 if (!(strcmp(*params,"1")))            /* button 1 pushed */ 
    {                                       
    surfmen(sirdit, distit, colorit);
    }

 else if (!(strcmp(*params,"2")))            /* button 2 pushed */ 
    {
    /* input the filename*/
    iw_text = wid_text(iw_win, (Widget)0, "Enter the filename:", docnam,
                    savesurf_cb, NULL, ixulw+nsamw, iyulw+nroww);

    }
                                       
 else if (!(strcmp(*params,"3")))	     /* button 3 pushed */ 
    {
    /* uninstall translations */
    XtUninstallTranslations(iw_t);
 
    if( filedata && filedata->fp!=NULL)	{
	closefile(filedata);	   
	first = TRUE;
    }
    /* return to narmal colormap */
    setamap(map);

    /* unmanage information box */
    showbutx("","","",TRUE);
    }
 }

 /*************************** savesurf_cb **********************************/
 void savesurf_cb(Widget iw_temp, caddr_t data, caddr_t call_data)
 {
   float        jslope, kslope;
   int		nsams,nrows;
   float        *bufs;
   int          i, j, k, kk, nlet, numfil, irec;
   FILEDATA     *filedatat;
   char		output[100];

   XmSelectionBoxCallbackStruct * cbs;

   cbs = (XmSelectionBoxCallbackStruct *) call_data;

   /* check the input name */
   strcpy(docnam,extractstr(cbs -> value));

   /* check input filename */
   if (strlen(docnam) == 0) 
       {
       spout("*** Must specify name for the file.");
       return;
       }

   /* add datexc to docnam if not already there */
   if (strstr(docnam,datexc) == 0)
       {
        strcat(docnam,".");
        strcat(docnam,datexc);
       }                      
         
  XtUnmanageChild( iw_text );

  /* find reflection image size */
  jslope = (1.0 - nrow*scaleval)/(1.0 - nrow);
  kslope = (1.0 - nslice*scaleval)/(1.0 - nslice);
  nsams  = (nrow2 - nrow1)*jslope + 1;
  nrows  = (nslice2 - nslice1)*kslope + 1;

  /* locate the space for the saving image */
  if((bufs = (float *) malloc(sizeof(float)*nsams)) == NULL)
  {
    spout("*** Allocation failed for writing to file in surfmovie!");
    return;
  }

  /* open new output file */
  filedatat = opennew(docnam, nsams, nrows, 1, 1, "u");  

  /* put newimage in output file */
  kk = 0;
  for (irec = 1; irec <= nrows; irec++)
     {
     for(k = 0; k < nsams; k++) bufs[k] = (float) refmap[kk++];

     if (!wrtlin(filedatat, bufs, nsams, irec)) 
        {
	sprintf(output,"*** Error writing to file in %s!", docnam);
	spout(output);
        closefile(filedatat); free(bufs); return;
        }
     }

  closefile(filedatat);     
  free(bufs);
}

/************** angle_cb ****************************
 convert the angles between Euler angles or XYZ angles
 ****************************************************
 */

void angle_cb(Widget iw_temp, caddr_t data, caddr_t callback)
{
    Arg        	args[MAX_ARGS];   	/* arg list */
    int		iphi,itheta,ipsi; 	/* euler angles */
    int		iang_x,iang_y,iang_z; 	/* xyz angles */
    int		icang[3],i; 		/* increased angles */
    float	fcang[3]; 		/* increased angles float*/

    /* the using XYZ angle */
    if (XmToggleButtonGadgetGetState(iw_angle))
        {
	/* set label for toggle button */
 	XtSetArg(args[0], XmNlabelString, XmStringCreate("XYZ",
                              XmSTRING_DEFAULT_CHARSET));
	XtSetValues(iw_angle, args, 1);
	
	/* calculate the xyz angle */
    	XmScaleGetValue(iw_angs[0],&iphi);
    	XmScaleGetValue(iw_angs[1],&itheta);
    	XmScaleGetValue(iw_angs[2],&ipsi);
    	phi   = (float) iphi;		ang_x = phi;
    	theta = (float) itheta;		ang_y = theta;
    	psi   = (float) ipsi; 		ang_z = psi;

	eul_to_xyz(&ang_x, &ang_y, &ang_z);

	/* get the increased angles */
	for (i=0; i<3; i++) {
    	    XmScaleGetValue(iw_angs[i+3],&icang[i]);
	    fcang[i] = (float) icang[i];
	    }

	eul_to_xyz(&fcang[0], &fcang[1], &fcang[2]);

	setangles(XYZ, ang_x,ang_y,ang_z, fcang[0], fcang[1], fcang[2]);

	XtUnmanageChild(iw_euler);
	XtManageChild(iw_xyz);
	}
    else
        {
 	XtSetArg(args[0], XmNlabelString, XmStringCreate("Euler",
                              XmSTRING_DEFAULT_CHARSET));
	XtSetValues(iw_angle, args, 1);

	/* calculate the xyz angle */
    	XmScaleGetValue(iw_angsxyz[0],&iang_x);
    	XmScaleGetValue(iw_angsxyz[1],&iang_y);
    	XmScaleGetValue(iw_angsxyz[2],&iang_z);
    	ang_x = (float)iang_x;	phi   = ang_x;
    	ang_y = (float)iang_y;	theta = ang_y;
    	ang_z = (float)iang_z;  psi   = ang_z;

	xyz_to_eul(&phi, &theta, &psi);

	/* get the increased angles */
	for (i=0; i<3; i++) {
    	    XmScaleGetValue(iw_angs[i+3],&icang[i]);
	    fcang[i] = (float) icang[i];
	    }

	xyz_to_eul(&fcang[0], &fcang[1], &fcang[2]);

	setangles(XYZ, ang_x,ang_y,ang_z, fcang[0], fcang[1], fcang[2]);

	XtUnmanageChild(iw_xyz);
	XtManageChild(iw_euler);
	}
}

void setangles(int angtype, float t1,float t2,float t3, 
	     		    float t4,float t5,float t6)
{
    Arg        	args[MAX_ARGS];   	/* arg list */
    float	angles[6];
    int 	i;

    angles [0] = t1;
    angles [1] = t2;
    angles [2] = t3;
    angles [3] = t4;
    angles [4] = t5;
    angles [5] = t6;

    for (i=0; i<6; i++) {
	
	XtSetArg(args[0], XmNvalue, angles[i]);

    	if (angtype) 	 /* elular angtype */
	    XtSetValues(iw_angs[i], args, 1);

    	else 		/* xyz angtype */
	    XtSetValues(iw_angsxyz[i], args, 1);
	}
}
	
