  
/*
 C++********************************************************************
 C
 C    web.c -- CREATED May 92 (C language version)
 C
 C    Author:  ArDean Leith
 C
 C *********************************************************************
 C *  COPYRIGHT (C)1992,1996  HEALTH RESEARCH INC, EMPIRE STATE PLAZA, *
 C *    ALBANY, NY 12201.  THE CONTENTS OF THIS DOCUMENT ARE           *
 C *    PROPRIETARY TO HEALTH RESEARCH INC., AND ARE NOT TO BE         *
 C *    DISCLOSED TO OTHERS OR USED FOR PURPOSES OTHER THAN INTENDED   *
 C *    WITHOUT WRITTEN APPROVAL OF HEALTH RESEARCH INC.               *
 C *********************************************************************
 C
 C     PURPOSE: MAIN ROUTINE FOR WEB
 C              INITIALIZES X WINDOWS AND STARTS WEB
 C
 C     PARAMETERS: ARGV 
 C                 ARGC   
 C
 C--********************************************************************
*/

#include "std.h"
#include "x.h"
#include "files.h"

#define MAXCOL 256
#define XtNfontPath "fontPath"
#define XtCFontPath "FontPath"

 /* external function prototypes */
 extern Widget        web_main(Widget);
 
 /* command line option recovery structure */
 static XrmOptionDescRec options[] = {
    {"-reduction", "*imageReduction",         XrmoptionSepArg, NULL},
    {"-listfiles", "*listFiles",              XrmoptionNoArg, "FALSE"},
    {"+names",     "*listNames",              XrmoptionNoArg, "TRUE"},
    {"-nsam8",     "*nsam8",                  XrmoptionSepArg, NULL},
    {"-nrow8",     "*nrow8",                  XrmoptionSepArg, NULL},
    {"-nslice8",   "*nslice8",                XrmoptionSepArg, NULL},
    {"-cursor",    "*cursor",                 XrmoptionSepArg, NULL},
    {"-wfg",       "*webForeground",          XrmoptionSepArg, NULL},
    {"-wbg",       "*webBackground",          XrmoptionSepArg, NULL},
    {"-marx",      "*imageXMargin",           XrmoptionSepArg, NULL},
    {"-mary",      "*imageYMargin",           XrmoptionSepArg, NULL},
    {"-marlef",    "*imageLeftMargin",        XrmoptionSepArg, NULL},
    {"-martop",    "*imageTopMargin",         XrmoptionSepArg, NULL},
    {"-cfg",       "*cursorForeground",       XrmoptionSepArg, NULL},
    {"-cbg",       "*cursorBackground",       XrmoptionSepArg, NULL},
    {"-lutdir",    "*lutDirectory",           XrmoptionSepArg, NULL},
    {"-ext",       "*dataExt",                XrmoptionSepArg, NULL},
    {"-trace",     "*traceWeb",               XrmoptionNoArg, "TRUE"},
 };    



 /* resource recovery structure */
 typedef struct {
    char  *fontpath;
    char  *geom;
    int   iskip;
    int   marx;    
    int   mary;    
    int   marlef;    
    int   martop;   
    Boolean   listfi;
    Boolean   names;
    Boolean   trace_web;
    int   magsizx;
    int   magsizy;
    int   magfac;
    int   nsam8;
    int   nrow8;
    int   nslice8;
    int   iradius;
    float scaleval;
    float refl;
    float fct;
    int   icursor;
    int   icolorcf;
    int   icolorcb;
    int   icolorov;
    int   icolor;
    int   icolorb;
    char  *lutdir;
    char  *datexc;
    } ApplicationData, *ApplicationDataPtr;

 

 /* resource recovery structure */
 static XtResource resources[] = {
    {XtNfontPath, XtCFontPath, XtRString, sizeof(char *),
     XtOffset(ApplicationDataPtr, fontpath),
              XtRString, "/usr/lib/X11/fonts"},

    {"lutDirectory", "LutDirectory", XtRString, sizeof(char *),
     XtOffset(ApplicationDataPtr, lutdir), XtRString, "none"},

    {"dataExt", "DataExt", XtRString, sizeof(char *),
     XtOffset(ApplicationDataPtr, datexc), XtRString, "~"},

    {XtNgeometry,XtCGeometry , XmRString, sizeof(char *),
     XtOffset(ApplicationDataPtr, geom), XmRString, "1280x1024+0+0"},

    {"imageReduction","ImageReduction" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, iskip), XmRImmediate, (XtPointer) 1 },

    {"imageXMargin","ImageXMargin" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, marx), XmRImmediate, (XtPointer) 1 },

    {"imageYMargin","ImageYMargin" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, mary), XmRImmediate, (XtPointer) 1 },

    {"imageLeftMargin","ImageLeftMargin" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, marlef), XmRImmediate, (XtPointer) 1 },

    {"imageTopMargin","ImageTopMargin" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, martop), XmRImmediate, (XtPointer) 1 },

    {"listFiles","ListFiles" , XtRBoolean, sizeof(Boolean),
     XtOffset(ApplicationDataPtr, listfi), XmRImmediate, (XtPointer) True },

    {"traceWeb","TraceWeb" , XtRBoolean, sizeof(Boolean),
     XtOffset(ApplicationDataPtr, trace_web), XmRImmediate, (XtPointer) False },

    {"listNames","ListNames" , XtRBoolean, sizeof(Boolean),
     XtOffset(ApplicationDataPtr, names), XmRImmediate, (XtPointer) False },
 
    {"magSizeX","magSizeX" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, magsizx), XmRImmediate, (XtPointer) 64 },

    {"magSizeY","magSizeY" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, magsizy), XmRImmediate, (XtPointer) 64 },

    {"nsam8","Nsam8" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, nsam8), XmRImmediate, (XtPointer) 940 },

    {"nrow8","Nrow8" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, nrow8), XmRImmediate, (XtPointer) 940 },

    {"nslice8","Nslice8" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, nslice8), XmRImmediate, (XtPointer) 1 },

    {"markerRadius","MarkerRadius" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, iradius), XmRImmediate, (XtPointer) 5 },

    {"scaleVal","ScaleVal" , XtRFloat, sizeof(float),
     XtOffset(ApplicationDataPtr, scaleval), XmRString, "1.0" },

    {"reflectionFactor","ReflectionFactor" , XtRFloat, sizeof(float),
     XtOffset(ApplicationDataPtr, refl), XmRString, " 0.5" },

    {"fct","Fct" , XtRFloat, sizeof(float),
     XtOffset(ApplicationDataPtr, fct),  XmRString, "1.0" },

    {"cursor","Cursor" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, icursor), XmRImmediate, (XtPointer) 22 },

    {"cursorForeground","CursorForeground" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, icolorcf), XmRImmediate, (XtPointer) 147 },

    {"cursorBackground","CursorBackground" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, icolorcb), XmRImmediate, (XtPointer) 137 },

    {"overlayColor","OverlayColor" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, icolorov), XmRImmediate, (XtPointer) 157 },

    {"webForeground","WebForeground" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, icolor), XmRImmediate, (XtPointer) 137 },

    {"webBackground","WebBackground" , XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, icolorb), XmRImmediate, (XtPointer) 0 },
    };



 /* to use these variables include common.h in your source code    */ 

 Display        *idispl;           /* current display               */
 Screen         *iscreen;          /* current screen                */

 Widget         iw_top;            /* top widget                    */
 Widget         iw_main ;          /* main box widget               */
 Widget         iw_menub;          /* menu bar widget               */
 Widget         iw_win;            /* image display widget          */
 Widget         iw_files  = 0;     /* file selection widget         */
 Widget         iw_filesnol = 0;   /* file (no list) widget         */
 Widget         iw_scroll;         /* scrolled output widget        */
 Widget         iw_scrollpar;      /* parent scrolled output widget */

 Window         iwin, iroot, iwtop; /* image, root & top windows    */
 Pixmap         imagsav;           /* image backing store           */
 int            idepth;            /* display window depth          */
 unsigned int   ihighx, iwidex;    /* display window x & Y dimen.   */
 int            ixulx, iyulx;      /* display window location       */
 GC             icontx;            /* usual graphics context        */
 GC             icontxb  = 0;      /* blanking graphics context     */
 Colormap       map;               /* current colormap              */
 Visual         *visual;           /* visual                        */
 FILEDATA       *filedata;         /* current image file info       */
 Colormap       mapover;           /* overlay colormap              */
 XColor         colort[MAXCOL];    /* temporary color setting array */
 int            nsam,nrow,nslice;  /* last image size               */
 int            islice;            /* current slice                 */
 int            nsam1,nsam2;       /* col. range                    */
 int            nrow1,nrow2;       /* row range                     */
 int            nslice1,nslice2;   /* slice range                   */
 double         fmin,fmax;         /* last image range              */
 int            ixulw,iyulw;       /* window position               */
 int            nsamw,nroww;       /* window size                   */
 int            ixul,iyul;         /* upper left corner last image  */
 int            ixuln,iyuln;       /* corner of next image          */
 int            nprow = 1;         /* pictures per row              */
 int            nbot,nrit;         /* bottem & rit image position   */
 char           axis;              /* axis of slicing               */
 XImage         *imagel = NULL;    /* left image pointer            */
 XImage         *imager = NULL;    /* right image pointer           */
 int            nsaml,nrowl;       /* left image size               */
 int            nsamr,nrowr;       /* right image size              */
 int            nsam1l,nrow1l;     /* left image lower extent       */
 int            nsam2l,nrow2l;     /* left image upper extent       */
 int            nsam1r,nrow1r;     /* right image lower extent      */
 int            nsam2r,nrow2r;     /* right image upper extent      */
 int            ixull, iyull;      /* left image position           */
 int            ixulr, iyulr;      /* left image position           */
 int            ixulli,iyulli;     /* right image position          */
 int            ixulri,iyulri;     /* right image position          */
 int            ixullmin,ixlrlmax; /* left image possible extent    */
 int            iyullmin,iylrlmax; /* left image possible extent    */
 int            ixulrmin,ixlrrmax; /* right image possible extent   */
 int            iyulrmin,iylrrmax; /* right image possible extent   */

 int            ispicol[MAXCOL];   /* web colormap values           */
 Cursor         icursorx;          /* current x cursor number       */
 XColor         colorcursf;        /* cursor foreground color       */
 XColor         colorcursb;        /* cursor background color       */
 float          param[102];        /* Web registers (unused)        */ 
 int            ioper    = 0;      /* current operation number      */
 char           filnow[81] = "\0"; /* name of last image file       */ 
 char           winnam[81] = "\0"; /* name of last window file      */ 
 char           docnam[81] = "\0"; /* name of last doc    file      */ 
 char           filnext[81]= "\0"; /* name of next image file       */ 
 char           dirnow[81] = "\0"; /* name of current directory     */ 
 int		ntimg     = FALSE; /* start next image (marker)	    */
 int            newline   = FALSE; /* new line flag                 */
 int            nuscrn    = FALSE; /* new screen flag               */
 int            gotsurf   = FALSE; /* surface in memory now         */
 int            samefile  = FALSE; /* filnow == filnex              */
 int            placed    = FALSE; /* image position sent flag      */
 int            drawing   = FALSE; /* flag for xor drawing          */
 int            force     = FALSE; /* force images / row flag       */
 int            ntop      = 0;     /* top image position            */
 int            imagego   =   0;   /* (=0) colormap normalization range  */
 int            imageend  = 127;   /* colormap normalization range  */
 int	        nstack = 0;	   /* stack indicator */
 int		inuse = 0;	   /* stack file inused indicator */
 int		maxim = 0;	   /* max num of files in stack */

 /* following values do not seem to be used as initialized          */
#ifdef VMS
 int            mapgo     = 20;    /* image start in colormap       */
 int            colorend  = 238;   /* colors end in colormap        */
#else
 int            mapgo     = 48;    /* image start in colormap       */
 int            colorend  = 256;   /* colors end in colormap        */
#endif
 int            colorgo   = 128;   /* colors start in colormap      */
 int            mapend    = 127;   /* image  end in colormap        */



 /* following values appear no longer used!!!!!!!!                  */
 /* int            nmax      = 127; */  /* colormap normalization range  */
 int            lastskip  = 1;     /* last image mag. factor        */
 int            ncolors   = 128;   /* number of defined colors      */

 /* following values accessible from defaults file                  */
 int            iskip     = 1;     /* image mag. factor             */
 int            marx      = 1;     /* x image margin                */
 int            mary      = 1;     /* y image margin                */
 int            marlef    = 0;     /* left window margin            */
 int            martop    = 0;     /* top window margin             */
 int            trace_web = FALSE; /* does not make results file    */
 int            listfi    = TRUE;  /* list files in selector flag   */
 int            names     = FALSE; /* print image names flag        */
 int            names_height = 0;  /* pixel height of image filename */
 int            magsizx   = 64;    /* magnification size (x)        */
 int            magsizy   = 64;    /* magnification size (y)        */
 int            magfac    = 4;     /* magnification factor          */
 int            nsam8     = 940;   /* size for raw files            */
 int            nrow8     = 940;   /* size for raw files            */
 int            nslice8   = 1;     /* size for raw files            */
 int            iradius   = 5;     /* marker location radius        */
 float          scaleval  = 1.0;   /* surface magnification         */
 float          refl      = 0.5;   /* surface rendering reflection  */
 float          fct       = 1.0;   /* surface rendering attenuation */
 int            icursor   =  22;   /* web cursor number             */
 int            icolorcf  = 147;   /* cursor foreground             */
 int            icolorcb  = 137;   /* cursor background             */
 int            icolorov  = 157;   /* overlay color                 */
 int            icolor    = 137;   /* current web color             */
 int            icolorx   = 137;   /* current x color               */
 int            icolorb   =   0;   /* current background            */
 int            icolorxb  =   0;   /* current x background color    */
 char           datexc[]  = "DAT"; /* file extension                */ 
#ifdef VMS
 char           lutdir[81]  = "RGB_LOC:*.RGB"; /* lut directory     */ 
#else
 char           lutdir[81]  = "\0"; 
#endif

 
 /* following values should be made accessible from defaults file?  */
 char           filwant[81] = "*"; /* new file search pattern       */
 int            movie    = FALSE;  /* image movie flag              */
 int            keepxy   = FALSE;  /* retain values of x & y        */
 int            normit   = FALSE;  /* normalize each slice          */
 int            xaxis    = FALSE;  /* X axis selected               */
 int            yaxis    = FALSE;  /* Y axis selected               */
 int            zaxis    = TRUE;   /* Z axis selected               */
 int	        show_num = TRUE;  /* set show marker number flag   */ 
 int            leavit   = FALSE;  /* leave window at position      */
 int            sayit    = TRUE ;  /* say window position           */
 int            inimag   = FALSE;  /* inside image data only        */
 int            getscreen = FALSE; /* get screen pixel value        */
 int            getfile  = FALSE;  /* get file pixel value          */
 int            docit    = FALSE;  /* store data in doc file        */
 int            ikey     = 1;      /* doc. file key                 */
 int            ikeyc    = 1;      /* doc. file key for categorizing */
 int            marker   = 0;      /* set marker flag               */

/* variables used here */ 
 static ApplicationData  AppData;

 /* also used by actions.c */
 XtAppContext     app_context;


/********************* main  ***********************/

 int main(unsigned int argc, char **argv)
 
 { 
 int i, nn;

 if (argc > 1 && strcmp(argv[1],"help") == 0)
   { 
   printf(" Purpose: A spider image viewer and analyzer \n");
   printf(" Usage:   web [options] file-extension \n");
   printf(" Sample:  web DAT \n");
   exit(1);
   }
#ifdef NEVER
  if (argc == 1)
     {
     printf("Enter file extension:");
     scanf("%s",datexc);
     }

   /* vms has only uppercase filenames */
   nn = 0;
   while (datexc[nn])
       { /* is char in the range a-z ? */
       if (toascii((int) datexc[nn]) >= 97 && 
           toascii((int) datexc[nn]) <= 122 ) 
	      datexc[nn] = (char) toupper((int) datexc[nn]);
       nn++;
       }
#endif
 /************ debug **************  (removed) ************************
 printf("initial argc: %d\n",argc);
 for (i=0; i<argc; i++)
    printf(" %s",argv[i]);
    printf(" \n");
    printf("names: %d listfi: %d \n",names,listfi);
 *********** debug ****************/

#if defined(__osf__) || defined (VMS) || defined(SP_FIVE2 )
 iw_top = XtAppInitialize(&app_context, "Web", 
                          options,XtNumber(options),
                         (int *)&argc, argv,  
                          NULL,NULL, 0);

#else

   /* XtOpenApplication calls XtToolkitInitialize,
   followed by XtCreateApplicationContext, then calls
   XtOpenDisplay with display_string NULL and application_name
   NULL, and finally calls XtAppCreateShell with application_name
   NULL, widget_class applicationShellWidgetClass, and the
   specified args and num_args and returns the created shell.

   The modified argc and argv returned by XtDisplayInitialize
   are returned in argc_in_out and argv_in_out. If
   app_context_return is not NULL, the created application
   context is also returned. If the display specified by the
   command line cannot be opened, an error message is issued
   and XtOpenApplication terminates the application. If
   fallback_resources is non-NULL, XtAppSetFallbackResources is
   called with the value prior to calling XtOpenDisplay.  

   Widget XtOpenApplication(app_context_return,
          application_class, options, num_options, argc_in_out,
          argv_in_out, fallback_resources, widget_class, args,
          num_args)
                XtAppContext app_context_return;
                String application_class;
                XrmOptionDescRec* options;
                Cardinal num_options;
                int* argc_in_out;
                String* argv_in_out;
                String* fallback_resources;
                WidgetClass widget_class;
                ArgList args;
                Cardinal num_args;
   */

 iw_top = XtOpenApplication(&app_context, "Web", options,
           XtNumber(options), (int *) &argc, argv, NULL, 
           applicationShellWidgetClass, NULL, 0);
#endif


 /* cache the display pointer */ 
 idispl = XtDisplay(iw_top);


 /* check the application resources -------------------------------*/
 XtGetApplicationResources(iw_top, &AppData, resources,
             XtNumber(resources), NULL, 0);

#ifdef NEVER
 /************ debug ********* (active) *****/
 if (argc > 1)
    {
    printf(" after resources, argc: %d :",argc);
    for (i=0; i<argc; i++)
       printf(" %s",argv[i]);
    printf(" \n");
    /* printf("iskip: %d\n",AppData.iskip);    */
    /* printf("geometry: %s\n",AppData.geom);  */
    /* printf("appdata.names: %d listfi: %d \n", */
    /*    AppData.names,AppData.listfi);         */
    }
 /**********************************************/
#endif

 /* check iskip */
 if (AppData.iskip == 0)
   {
   printf("In resources, Image_reduction: %d must be non-zero\n",
          AppData.iskip);
   AppData.iskip = iskip;
   }
 iskip =  AppData.iskip;


 /* check marx, mary */
 if (AppData.marx < 0 || AppData.mary < 0)
   {
   printf("In resources, imageXMargin:%d &  ImageYMargin: %d must be > zero\n",
          AppData.marx, AppData.mary);
   AppData.marx = marx;
   AppData.mary = mary;
   }
 marx =  AppData.marx;
 mary =  AppData.mary;

 /* check marlef, martop */
 if (AppData.marlef < 0 || AppData.martop < 0)
   {
   printf("In resources, ImageLeftMargin:%d &  ImageTopMargin: %d must be > zero\n",
          AppData.marlef, AppData.martop);
   AppData.marlef = marlef;
   AppData.martop = martop;
   }
 marlef =  AppData.marlef;
 martop =  AppData.martop;

 /* check magsizx, magsizy */
 if (AppData.magsizx <= 0 || AppData.magsizy <= 0)
   {
   printf("In resources, magSizeX:%d &  magSizeY: %d must > zero\n",
          AppData.magsizx, AppData.magsizy);
   AppData.magsizx = marlef;
   AppData.magsizy = magsizy;
   }
 magsizx =  AppData.magsizx;
 magsizy =  AppData.magsizy;

 /* check nsam8, nrow8 nslice8*/
 if (AppData.nsam8 <= 0 || AppData.nrow8 <= 0 || AppData.nslice8 <= 0 )
   {
   printf("In resources, nsam8:%d , nrow8: %d & nslice8: %d must be > zero\n",
          AppData.nsam8, AppData.nrow8,AppData.nslice8);
   AppData.nsam8   = nsam8;
   AppData.nrow8   = nrow8;
   AppData.nslice8 = nslice8;
   }
 nsam8   =  AppData.nsam8;
 nrow8   =  AppData.nrow8;
 nslice8 =  AppData.nslice8;

 /* check iradius */
 if (AppData.iradius <= 0)
   {
   printf("In resources, MarkerRadius: %d must be > zero\n",
          AppData.iradius);
   AppData.iradius = iradius;
   }
 iradius =  AppData.iradius;

 /* check scaleval */
 if (AppData.scaleval <= 0.0)
   {
   printf("In resources, ScaleVal: %d must be > zero\n",
          AppData.scaleval);
   AppData.scaleval = scaleval;
   }
 scaleval =  AppData.scaleval;

 /* check scaleval */
 if (AppData.scaleval <= 0.0)
   {
   printf("In resources, ScaleVal: %d must be > zero\n",
          AppData.scaleval);
   AppData.scaleval = scaleval;
   }
 scaleval =  AppData.scaleval;

 /* check refl */
 if (AppData.refl < 0.0)
   {
   printf("In resources, ReflectionFactor: %d must be >= zero.\n",
          AppData.refl);
   AppData.refl = refl;
   }
 refl =  AppData.refl;

 /* check fct */
 if (AppData.fct < 0.0)
   {
   printf("In resources, Fct: %d must be >= zero.\n",
          AppData.fct);
   AppData.fct = fct;
   }
 fct =  AppData.fct;

 /* check icursor */
 icursor =  AppData.icursor;

 /* check icolorcf, magicolorcbsizy */
 if (AppData.icolorcf < 0 || AppData.icolorcb < 0)
   {
   printf("In resources, cursorForeground:%d &  cursorBackground: %d must >= zero\n",
          AppData.icolorcf, AppData.icolorcb);
   AppData.icolorcf = icolorcf;
   AppData.icolorcb = icolorcb;
   }
 icolorcf =  AppData.icolorcf;
 icolorcb =  AppData.icolorcb;

 /* check icolor, icolorb */
 if (AppData.icolor < 0 || AppData.icolorcb < 0)
   {
   printf("In resources, webcolor:%d &  webBackground: %d must >= zero\n",
          AppData.icolor, AppData.icolorb);
   AppData.icolor = icolor;
   AppData.icolorb = icolorb;
   }
 icolor   = AppData.icolor;
 icolorb  = AppData.icolorb;
 icolorx  = icolor;   
 icolorxb = icolorb;   

 /* check icolorov */
 if (AppData.icolorov < 0.0)
   {
   printf("In resources, overlayColor: %d must be >= zero.\n",
          AppData.icolorov);
   AppData.icolorov = icolorov;
   }
 icolorov =  AppData.icolorov;
 /* check iradius */

 if (*AppData.lutdir != 'n')
   {   /* lutdir is provided by resouces or command line */
   strncpy(lutdir,AppData.lutdir,80);
   }

 if (*(AppData.datexc) != '~')
   {   /* datexc is provided by resources or command line */
   strncpy(datexc,AppData.datexc,3);
   }

 if (argc > 1 && *argv[argc-1] != '-')
   {  /* last arguement is datexc if does not begin with - */
   strncpy(datexc,argv[argc-1],3);
   }

 printf(" %s --> Data file extension: %s\n", AppData.datexc, datexc);
 /* printf(" geometry: %s: %d \n", AppData.geom,iskip); */

 listfi    = AppData.listfi;
 names     = AppData.names;
 trace_web = AppData.trace_web;

 /* start the application ------------------------------------ */
 web_main(iw_top);

 /* loop to get events until exit is requested */
 XtAppMainLoop(app_context);

 return TRUE;

 }

