/*
***************************************************************************
*  translated to wid_power.c by mahieddine ladjadj   5/24/94
*C WID_POWER.FOR  -- CREATED FEB 93
*C
*C **************************************************************************
*C *  AUTHOR: ArDean Leith                                                  *
*C *  COPYRIGHT (C)1993, WADSWORTH CENTER FOR LABORATORIES AND RESEARCH,    *
*C *    NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.              *
*C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
*C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
*C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
*C *    THE CENTER FOR LABORATORIES AND RESEARCH 			    *
*C **************************************************************************
*C
*C  int wid_power(perm,chktsiz)
*C
*C  PURPOSE:   READSBACK WINDOW, STORES IN A IMAGE FILE
*C
*C  PARAMETERS:   PERM        PERMANENT OR TEMPORARY SPECTRUM
*C                chktsiz      checks size of window.
*C
*C  CALLED BY:    powerbut_pop()
*C
*C  CALLS:        POWER   X-WINDOWS ROUTINES
*
*
******************************************************************************
*/
#include "common.h"
#include "routines.h"


/********************************* wid_power()   *****************************/
int wid_power(int perm, int *chktsiz)
   {
   register int j, i, k;
   char	outstr[80];
   static int nsamwt, nrowwt;
   int ik, jt, nsamok, nrowok, it, irtflg, ipad;
   XImage        *imagep;
   char   *lbuf, pbuf;
   float *bufin, fmin, fmax, fmsc, scal, b;
   /* KLUDGE TO GET AROUND TOO MANY CONTINUATIONS */
   int nmax = 127, GOODMAX = 46;
   int good[] = {
	   8,  16,  20,  24,  32,  36,  40,  48,  60,  64,
	  72,  80,  96, 100, 108, 120, 128, 144, 160, 180, 
	 192, 200, 216, 240, 256, 288, 300, 320, 360, 384, 
	 400, 432, 480, 512, 576, 600, 640, 700, 720, 768,
	 800, 864, 900, 960, 1000, 1024
      };

   /* set waitcursor */
   waitcursor(iwin, TRUE, 79);

   if (*chktsiz)
      {
      nsamwt = nsamw;
      nrowwt = nroww;
      if (nsamwt > 1024)
       { spout(" Window col. dimensions reduced to 1024 "); nsamwt = 1024; }
      if (nrowwt > 1024)
       { spout(" Window row dimensions reduced to 1024 "); nrowwt = 1024; }

      nsamok = nrowok = FALSE;
/*----------------------------------------------------- changed by YH */
      for (i = GOODMAX-1; i >0 ; i--)
          {
          it = good[i-1];	/* floor */
	  ik = good[i];		/* ceiling */
          if (!nsamok)
             { /* check column size */
             if (it <= nsamw)	/* get the correct column size */ 
                {
		nsamwt = ( (nsamw-it)>abs(ik-nsamw)? ik : it );
		nsamok = TRUE;
		if (nsamwt != nsamw) {
		    sprintf(outstr, " *** column size is reset from %d to %d.",nsamw,nsamwt);
		    spout(outstr);
		    }
                }
	     }
          if (!nrowok)
             { /* check row size */
             if (it <= nroww) /* get the correct row size */
                {
		nrowwt = ( (nroww-it)>abs(ik-nroww)? ik : it );
		nrowok = TRUE;
		if (nrowwt != nroww) {
		    sprintf(outstr, " *** row size is reset from %d to %d.",nroww,nrowwt);
		    spout(outstr);
		    }
                }
             }
          if (nsamok && nrowok) {
	     nsamw = nsamwt;
	     nroww = nrowwt;
	     break ;    /* get out of loop */
	     }
          }       /****      for (i = 0; i < GOODMAX; i++)  **/
/*----------------------------------------------------- changed by YH */

      if (!nrowok && !nsamok)
          { spout(" *** Spectrum sizing failed ");
            waitcursor(iwin,FALSE,79); return 1; } 

      *chktsiz = FALSE;
      }                  /* if (*chktsiz)  */

   /* read an 8 bit Z-Pixmap from window  */
   imagep = (XImage *) NULL;
   if ( (imagep = XGetImage(idispl, imagsav, ixulw, iyulw,
                     (unsigned int) nsamwt, (unsigned int) nrowwt,
                     (unsigned long) 255, (int) ZPixmap) )
                == (XImage *) NULL)                {
           spout("***  failed XGetImage operation ");
           waitcursor(iwin,FALSE,79);
           return 2;                               }

   /* images are padded to units of four bytes by XGetImage  */ 
   /* see nsamwt .vs. imagep->bytes_per_line                   */
   ipad = (nsamwt % 4) ? 4 - (nsamwt % 4) : 0;
                                                   
   /* run time allocation of memory space for 8 bit image */
   if ( (bufin = (float *) malloc(nsamwt * nrowwt * sizeof(float)) ) 
               == (float *) NULL)                                   {
              spout(" *** WARNING: return from comapsav.c ");
              spout(" MALLOC() CAN'T ALLOCATE HEAP MEMORY SPACE");
              waitcursor(iwin, FALSE, 79); XDestroyImage(imagep);
              return 5;                                             }

   /* convert image data (char) to floating point for power computations */
   /* skip the padding for each scan line                                 */
   k = 0;
   for (i = 0; i < nrowwt; i++)
       {
       for (j = 0 ; j < nsamwt; j++, k += ipad) 
           bufin[i*nsamwt+j] = (float) imagep->data[k++];
       }

   /* free space */
   XDestroyImage(imagep);
                                                 
   /* create power spectrum */
   irtflg = power(bufin,nsamwt,nrowwt);
   if (irtflg) {waitcursor(iwin, FALSE, 79);  return 3;}

   /* find maximum and minimum of power spectrum */
   fmin= fmax = bufin[0];
   for (i=1; i < nsamwt * nrowwt; i++)
       {
       fmax = MYMAX(fmax,bufin[i]);
       fmin = MYMIN(fmin,bufin[i]);
       }
   if(fabs((double)(fmax - fmin)) < (double) FLTZER)
       { spout(" Blank power spectrum skipped "); free(bufin); 
         waitcursor(iwin, FALSE, 79); return 4; }

   /* display power spectrum */

   /* conversion factor from floating point to logical *1 range */
   scal = ((float)(nmax - mapgo)) / (fmax - fmin);
   fmsc = - fmin * scal + mapgo + 0.5;
                                                                  
   /* run time allocation of memory space for 8 bit image */
   if ( (lbuf = (char *) malloc(nsamwt * nrowwt * sizeof(char)) ) 
              == (char *) NULL)                                     {
              spout(" *** WARNING: return from comapsav.c ");
              spout(" MALLOC() CAN'T ALLOCATE HEAP MEMORY SPACE");
              waitcursor(iwin, FALSE, 79); free(bufin);   return 5;  }

   if (nmax < 128)
       {
       for (i = 0; i < nsamwt * nrowwt; i++)
           {                   
           lbuf[i] =  (char) ((bufin[i] * scal) + fmsc);
           }
       }
   else
       {
       for (i = 0; i < nrowwt * nsamwt; i++)
           {
           b = bufin[i];
           /* convert 0..255 integers to -128..127 logical *1 */
           lbuf[i] = (((b * scal) + fmsc) < 128   ? 
                       (b * scal) + fmsc          :
                       (b * scal) + fmsc - 256.0 );
           }
       }                                         
 
   /* create image for  the screen in window */
   imagep = (XImage *) NULL;
   if( (imagep  = XCreateImage(idispl, visual,
                     (unsigned int) DefaultDepthOfScreen(iscreen),
                     (int) ZPixmap, 0, (char *) lbuf,
                     (unsigned int) nsamwt,
                     (unsigned int) nrowwt, 8, nsamwt) )
                == (XImage *) NULL)                          {
           spout("***  failed XCreateImage operation ");
           waitcursor(iwin, FALSE, 79);
           free(bufin); free(lbuf); return 2;                }

   /* put image to the screen */
   XPutImage(idispl, iwin, icontx, imagep, 0, 0, ixulw, iyulw,
             (unsigned int) nsamwt, (unsigned int) nrowwt); 

   /* make average image permanent */
   if(perm)
       XPutImage(idispl, imagsav, icontx, imagep, 0, 0, ixulw, iyulw,
                 (unsigned int) nsamwt, (unsigned int) nrowwt); 

   /* make changes visible */
   XFlush(idispl);

   /* free up memory space (note that lbuf is thus destroyed */
   XDestroyImage(imagep);
   free(bufin); 
   waitcursor(iwin, FALSE, 79);
 
   return 0;  
   }                        /*****    wid_power()      **/
