
/*$Header: /net/alwyn/usr1/web/src/RCS/wid_win.c,v 1.4 1995/01/27 16:31:10 liy Exp $*/
/*
C++************************************************************************
C
C WID_WIN
C
C **************************************************************************
C *  AUTHOR:  ArDean Leith                                                 *
C *  COPYRIGHT (C)1991, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C WID_WIN
C
C PURPOSE:      SET UP A WINDOWING SCALE WIDGET FOR 3-D VOLS
C
C PARAMETERS:   IW_PARENT         PARENT WIDGET         READ
C               IW_ITS            WIDGET NUMBERS        WRITE
C               NSAM,NROW,NSLICE  VOLUME EXTENT         READ
C               IXC               UPPER LEFT X          READ
C               IYT               UPPER LEFT Y,LOWER Y  READ/WRITE
C
C CALLED BY:    SURFMEN
C
C NOTE:         SLIDER IS ONLY CREATED IF CORRESPONDING VALUE OF
C               NSAM, NROW, OR NSLICE IS > 0.
C
C***********************************************************************
*/

#include "std.h"
#include "x.h"
#include <Xm/Scale.h>

#define  MAX_ARGS 10

 extern Widget wid_labelg   (Widget, Widget, char *, int, int);
 extern Widget wid_rowcol   (Widget, char , int, int);
 

/************************************************************/

 void wid_win(Widget iw_parent, Widget iw_its[6], 
              int nsam, int nrow, int nslice)

 {   
 Arg          args[MAX_ARGS];    /* arg list  */
 Widget       iw_lab1, iw_lab2, iw_lab3;
 Widget       iw_rowcol, iw_rowcolc, iw_rowcolr, iw_rowcols;
 static int   nsamo, nrowo, nsliceo;
 int          iwid, ihi;

 if (iw_its[0] == (void *)0 && nsam > 0) 
    {  /* must create new col widgets ------------------------------- */

    iwid = 180;
    ihi  = 50;

    iw_rowcol   = wid_rowcol(iw_parent,'v',-1,-1);

    /* create label for cols*/
    iw_rowcolc  = wid_rowcol(iw_rowcol,'h',-1,-1);
    iw_lab2     = wid_labelg(iw_rowcolc, 0, "COLS:  ", -1, -1);

    XtSetArg(args[0], XmNwidth, iwid);                    
    XtSetArg(args[1], XmNheight, ihi);                
    XtSetArg(args[2], XmNscaleWidth, iwid-10);            
    XtSetArg(args[3], XmNscaleHeight, ihi/2);        
    XtSetArg(args[4], XmNminimum, 1);                
    XtSetArg(args[5], XmNmaximum, nsam);             
    XtSetArg(args[6], XmNdecimalPoints, 0);            
    XtSetArg(args[7], XmNvalue, 1);                  
    XtSetArg(args[8], XmNorientation, XmHORIZONTAL);       
    XtSetArg(args[9], XmNshowValue, TRUE);           
 
    iw_its[0] = XmCreateScale(iw_rowcolc, "iw_scale", args, 10);

    XtSetArg(args[7], XmNvalue, nsam);       
    iw_its[1] = XmCreateScale(iw_rowcolc, "iw_scale", args, 10);
    }

 else if (nsam > 0 && nsam != nsamo)
    { /* alter scale size in existing widget */
    XtSetArg(args[0], XmNmaximum, nsam); 
      
    XtSetArg(args[1], XmNvalue, 1);   
    XtSetValues(iw_its[0], args, 2);

    XtSetArg(args[1], XmNvalue, nsam);
    XtSetValues(iw_its[1], args, 2);
    }

  if ( iw_its[2] == (void*)0 && nrow > 0) 
    {  /* must create new nrow widgets ------------------------------ */
   
    /* create label for cols*/
    iw_rowcolr  = wid_rowcol(iw_rowcol,'h',-1,-1);
    iw_lab1     = wid_labelg(iw_rowcolr, 0, "ROWS:  ", -1, -1);

    XtSetArg(args[0], XmNwidth, iwid);                    
    XtSetArg(args[1], XmNheight, ihi);                
    XtSetArg(args[2], XmNscaleWidth, iwid-10);            
    XtSetArg(args[3], XmNscaleHeight, ihi/2);        
    XtSetArg(args[4], XmNminimum, 1);                
    XtSetArg(args[5], XmNmaximum, nrow);             
    XtSetArg(args[6], XmNdecimalPoints, 0);            
    XtSetArg(args[7], XmNvalue, 1);                  
    XtSetArg(args[8], XmNorientation, XmHORIZONTAL);       
    XtSetArg(args[9], XmNshowValue, TRUE);           
 
    iw_its[2] = XmCreateScale(iw_rowcolr, "iw_scale", args, 10);

    XtSetArg(args[7], XmNvalue, nrow);       
    iw_its[3] = XmCreateScale(iw_rowcolr, "iw_scale", args, 10);
    }

 else if (nrow > 0 && nrow != nrowo)
    { /* alter scale size in existing widget */
    XtSetArg(args[0], XmNmaximum, nrow);       
       
    XtSetArg(args[1], XmNvalue, 1);
    XtSetValues(iw_its[2], args, 2);

    XtSetArg(args[1], XmNvalue, nrow);
    XtSetValues(iw_its[3], args, 2);
    }


  if (iw_its[4] == (void *)0 && nslice > 0) 
    {  /* must create new slice widgets ----------------------------- */

    iw_rowcols  = wid_rowcol(iw_rowcol,'h',-1,-1);
    iw_lab3     = wid_labelg(iw_rowcols, 0, "SLICES:", -1, -1);

    /* create scale widget for starting slice */
    XtSetArg(args[0], XmNwidth, iwid);                    
    XtSetArg(args[1], XmNheight, ihi);                
    XtSetArg(args[2], XmNscaleWidth, iwid-10);            
    XtSetArg(args[3], XmNscaleHeight, ihi/2);        
    XtSetArg(args[4], XmNminimum, 1);                
    XtSetArg(args[5], XmNmaximum, nslice);             
    XtSetArg(args[6], XmNdecimalPoints, 0);            
    XtSetArg(args[7], XmNvalue, 1);                  
    XtSetArg(args[8], XmNorientation, XmHORIZONTAL);       
    XtSetArg(args[9], XmNshowValue, TRUE);           
    iw_its[4] = XmCreateScale(iw_rowcols, "iw_scale", args, 10);

    XtSetArg(args[7], XmNvalue, nslice);       
    iw_its[5] = XmCreateScale(iw_rowcols, "iw_scale", args, 10);
    }

 else if (nslice > 0 && nslice != nsliceo)
    { /* alter scale size in existing widget */
    XtSetArg(args[0], XmNmaximum, nslice); 
      
    XtSetArg(args[1], XmNvalue, 1);       
    XtSetValues(iw_its[4], args, 2);

    XtSetArg(args[1], XmNvalue, nslice);       
    XtSetValues(iw_its[5], args, 2);
    }

 if (nsam > 0)
    {
    XtManageChild(iw_its[0]);
    XtManageChild(iw_its[1]);
    }
 if (nrow > 0)
    {
    XtManageChild(iw_its[2]);
    XtManageChild(iw_its[3]);
    }
 if (nslice > 0)
    {
    XtManageChild(iw_its[4]);
    XtManageChild(iw_its[5]);
    }

 nsamo   = nsam;
 nrowo   = nrow;
 nsliceo = nslice;
 }
