
/*$Header: /net/ithaca/usr1/web/src/RCS/widisp.c,v 1.13 1995/08/08 21:08:50 leith Exp $*/

/*
C++*************************************************************************
C
C    WIDISP
C **************************************************************************
C *  AUTHOR:  ArDean Leith                  *
C *  COPYRIGHT (C)1992, WADSWORTH CENTER FOR LABORATORIES AND              *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C WIDISP(ICONTX,FILEDATA,NROW1,NROW2,FMAX,FMIN,IXUL,IYUL,ISKIP)
C
C    PURPOSE:    TO DISPLAY AN IMAGE FROM fp AT IXUL,IYUL
C
C    PARAMETERS:    ICONTX       GRAPHICS CONTEXT
C                   NSAM         NUMBER OF PIXELS ON A LINE
C                   NSAM1        COL IN FILE OF IMAGE START
C                   NSAM2        COL IN FILE OF IMAGE START
C                   NROW1        ROW (RECORD) IN FILE OF IMAGE START
C                   NROW2        ROW OF END OF IMAGE
C                   FMAX         MAX. VALUE OF IMAGE
C                   FMIN         MIN. VALUE OF IMAGE
C                   IXUL         POSITION OF LEFT SIDE OF IMAGE ON SCREEN
C                   IYUL         POSITION OF TOP OF IMAGE ON SCREEN
C                   NMAX         MAX VALUE OF INDEX WRITTEN TO SCREEN
C                   ISKIP        INCREMENT FOR DO LOOP TO SKIP PIXELS
C
C    CALLED BY:     mont3 image
C--*******************************************************************
*/

#include "routines.h"
#include "common.h"


 /****************************  widisp  *****************************/
 
 int widisp(GC icontx, FILEDATA *filedata, int nsam, int nsam1,int nsam2, 
            int nrow1, int nrow2, double fmin, double fmax,
            int ixul,  int iyul,  int nmint,   int    nmaxt, int iskip, 
            int keepimage, XImage ** imaget, int keepimagec)
 { 

 float  *buf;
 char   *cbuf;
 int    nsams, nrows, npix, ival,miskip;
 int    j, m, k,irect;
 float  scal, fm;

 if (fabs(fmax - fmin) < FLTZER)
    { spout("*** BLANK IMAGE SKIPPED"); return FALSE; }

 buf = filedata -> buf;

 /* find number of rows in displayed image and allocate buffer space */
 if (iskip > 1) 
    {
    nsams = 1 + (nsam2 - nsam1) / iskip;
    nrows = 1 + (nrow2 - nrow1) / iskip;
    }
 else if (iskip < -1)
    {
    miskip = -iskip;
    nsams = (nsam2 - nsam1 + 1) * miskip;
    nrows = (nrow2 - nrow1 + 1) * miskip;
    }
 else
    {
    nsams = (nsam2 - nsam1 + 1);
    nrows = (nrow2 - nrow1 + 1);
    } 
    
 if ((cbuf = (char *) malloc(nsams * nrows * sizeof(char))) == (char *) 0)    
    { 
    spout("*** Unable to allocate cbuf memory in widisp.");
    return FALSE;
    }

 /* CONVERSION FACTOR FROM FLOATING POINT TO LOGICAL * 1 RANGE */
 scal = (float) (nmaxt - nmint) / (fmax - fmin);
 fm   = -fmin * scal + nmint + 0.5;

 npix = 0;  

 if (iskip > -2 && iskip < 2)
    {    /* display full size */

    for (irect = nrow1; irect <= nrow2; irect++)
       {
       if (redlin(filedata,buf,nsam,irect) != TRUE) 
          { free(cbuf);  return FALSE; }

       if (nmaxt < 128) 
           {   /* no need to convert numbers > 128 */   
           for (k = nsam1 - 1; k < nsam2; k++)
              {
              ival = (*(buf + k) * scal) + fm;
              *(cbuf + npix++) = ival;
              }
           }
       else
           {   /* need to convert numbers > 128 */   
           for (k = nsam1 - 1; k < nsam2; k++)
              {
              ival = (*(buf + k) * scal) + fm;
              /* convert 0..255  integers to -128...127 */
              if (ival > 127) ival = ival - 256;
              *(cbuf + npix++) = ival;
              }
           }
       }
    }

 else if (iskip > 1)
    {    /* display reduced size image */

    for (irect = nrow1; irect <= nrow2; irect = irect + iskip)
       {
       if (redlin(filedata,buf,nsam,irect) != TRUE) 
          { free(cbuf); return FALSE; }

       if (nmaxt < 128) 
           {   /* no need to convert numbers > 128 */   
           for (k = nsam1 - 1; k < nsam2; k = k + iskip)
              {
              ival = (*(buf + k) * scal) + fm;
              *(cbuf + npix++) = ival;
              }
           }
       else
           {   /* need to convert numbers > 128 */   
           for (k = nsam1 - 1; k < nsam2; k = k + iskip)
              {
              ival = (*(buf + k) * scal) + fm;
              /* convert 0..255  integers to -128...127 */
              if (ival > 127) ival = ival - 256;
              *(cbuf + npix++) = ival;
              }
           }
       }
    }


 else if (iskip < -1)
    {    /* display magnified size image */

    for (irect = nrow1; irect <= nrow2; irect++)
       {
       if (redlin(filedata,buf,nsam,irect) != TRUE) 
          { free(cbuf);   return FALSE; }
       if (nmaxt < 128) 
           {   /* no need to convert numbers > 128 */   
           for (k = nsam1 - 1; k < nsam2; k++)
              {
              ival = (*(buf + k) * scal) + fm;

              for (j = 0; j < miskip ; j++)
                 {
                 for (m = 0; m < miskip; m++)
                   *(cbuf + npix + m * nsams + j) = ival;
                 }
              npix = npix + miskip;
              }
           }
       else
           {   /* need to convert numbers > 128 */   
           for (k = nsam1 - 1; k < nsam2; k++)
              {
              ival = (*(buf+k)) * scal + fm;
              /* convert 0..255  integers to -128...127 */
              if (ival > 127) ival = ival - 256;

              for (j = 0; j < miskip ; j++)
                 {
                 for (m = 0; m < miskip; m++)
                   *(cbuf + npix + m * nsams + j) = ival;
                 }
              npix = npix + miskip;
              }
           }
        npix = (irect - nrow1 + 1) * miskip * nsams + nsam1- 1;
        }
    }

 /* display the image */
 wipic(icontx,cbuf,ixul,iyul,nsams,nrows,TRUE,keepimage,imaget);
 
 /* free up the character buffer unless it is still wanted */
 if (!keepimagec && cbuf)
     {
     free(cbuf);
     cbuf = NULL;
     }	

 /* record ximage for possible later reference */
 filedata -> ximage = *imaget;

 return TRUE;
}

