/******************************************************************
 * 
 * 	xyz_to_eul.c
 *
 *		(created April 24, 1995)
 *
 **************************************************************************
 *  AUTHOR:  YANHONG LI                                                *
 *  COPYRIGHT (C)1991, WADSWORTH CENTER FOR LABORATORIES AND              *
 *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
 *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
 *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
 *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
 *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
 **************************************************************************

 	purpose: The program used to transfer XYZ angles(theta1, theta2,
 	theta3) or (t1,t2,t3) to euler angles (Phi, Theta and Psi)

    	CALLED BY:       surfmen

***********************************************************************
 */

#include <stdio.h>
#include <math.h>
#include "routines.h"

#define	PI	3.1415926

void xyz_to_eul(float *pt1, float *pt2, float *pt3)
{
    float 	inphi, intheta, inpsi;	/* Euler angles */
    float	t1,t2,t3;		/* XYZ angles */

    float	s_phi, s_theta, s_psi;	/* sin values for euler angles */
    float	c_phi, c_theta, c_psi;	/* cos values for euler angles */

    float	s1,s2,s3;		/* sin values for XYZ angles */
    float	c1,c2,c3;		/* cos values for XYZ angles */	

    /* change angle to quatum defination */
    t1 = antiang(*pt1);
    t2 = antiang(*pt2);
    t3 = antiang(*pt3);
  
    /* calculate the sin and cs values */
    s1 = sin(t1); c1 = cos(t1);
    s2 = sin(t2); c2 = cos(t2);
    s3 = sin(t3); c3 = cos(t3);

    /* get the euler angles */
    c_theta = c1*c2;
    s_theta = sqrt(1.0-c_theta*c_theta);

    if (s_theta != 0) {

	/* Theta!=0, general calculation... */
	s_psi = s1*c2/s_theta;
	c_psi = s2/s_theta;

	s_phi = (c1*s2*s3-s1*c3)/s_theta;
	c_phi = (c1*s2*c3+s1*s3)/s_theta;

	/* calculate the angles */
     	if (c_phi >= 0) 	inphi = angle(asin(round(s_phi)));
	else			inphi = 180 - angle(asin(round(s_phi)));

     	if (c_theta >= 0) 	intheta = angle(asin(round(s_theta)));
	else			intheta = 180 - angle(asin(round(s_theta)));

     	if (c_psi >= 0) 	inpsi = angle(asin(round(s_psi)));
	else			inpsi = 180 - angle(asin(round(s_psi)));
	}

    else {
	if (c_theta >0) 	intheta = 0;
	else			intheta = 180;

	if (c_theta < 0) {
	    c_phi = -1*c2*c3;
	    s_phi = c2*s3;
	    }
	else {
	    c_phi = c2*c3;
	    s_phi = c2*s3;
	    }

     	if (c_phi >= 0) 	inphi = angle(asin(round(s_phi)));
	else			inphi = 180 - angle(asin(round(s_phi)));

	inpsi=0;
	}

    /* get the round cutting angles */
    *pt1 = inphi = roundang(inphi);
    *pt2 = intheta = roundang(intheta);
    *pt3 = inpsi = roundang(inpsi);
}
