
/*$Header: /alwyn/usr1/web/src/RCS/ztoxyn.c,v 1.4 94/05/12 12:40:37 leith Exp $*/
/*
C++************************************************************************
C
C ZTOXYN -- CREATED MAR 90
C               CONVERTED TO C, JULY 92 al
C **************************************************************************
C *  AUTHOR:  ArDean Leith                                                 *
C *  COPYRIGHT (C)1990 , WADSWORTH CENTER FOR LABORATORIES AND             *
C *  RESEARCH, NEW YORK STATE DEPARTMENT OF HEALTH, ALBANY, NY 12201.      *
C *    THE CONTENTS OF THIS DOCUMENT ARE PROPRIETARY TO THE CENTER FOR     *
C *    LABORATORIES AND RESEARCH AND ARE NOT TO BE DISCLOSED TO OTHERS OR  *
C *    USED FOR PURPOSES OTHER THAN INTENDED WITHOUT WRITTEN APPROVAL OF   *
C *    THE CENTER FOR LABORATORIES AND RESEARCH			           *
C **************************************************************************
C
C   ZTOXYN(LUNF,NSAM,NSAM1,NSAM2,NROW,NROW1,NROW2,
C           NSLICE,NSLICE1,NSLICE2,AXIS,NMAX,IRTFLG)
C
C    PURPOSE:         DIPLAYING MONTAGE FROM FILE SERIES ON X WORKSTATION
C                     NORMALIZED SLICE BY SLICE
C
C    CALLED BY:       MONT3
C
C    NOTES:
C
C     AXIS      NEW SECTION         NEW ROW            NEW SAMPLE
C     ---        -----------     -----------------    --------------
C      X       (ISAM-NSAM1+1)    (ISLICE-NSLICE1+1)   (IROW-NROW1+1)
C
C     LOC = (NEW SECTION-1)*NSAMS*NROWS + (NEW ROW-1)*NSAMS + NEW SAMPLE
C
C***********************************************************************
*/

#include "files.h"
#include "routines.h"

/**************************  ztoxyn  ********************************/

 int ztoxyn(FILEDATA *filedata,       char *cbuf,
             int nsam,   int nsam1,   int nsam2,
             int nrow,   int nrow1,   int nrow2,
             int nslice, int nslice1, int nslice2,
             char axis,  int nmaxt,   int iskip, int nmint)

 {  
   
 int        nsami, nrowi, nslices, iconx, icony, iconz;
 int        locin, locout, i, npixpsl, icon, nsamu, nrowu, miskip;
 int        miskipm1, iskipz, icons, irect, ipane, iconr, iloc;
 int        isam, irow, islice, l, m, nsliceu, ival, igo;
 float      *buf, *rbuf, *fmint, *fmaxt, tmin, tmax;
 float      fn, fncon, fval; 

 if (axis == 'x')
    {
    nsami   = nrow2 - nrow1 + 1;
    nrowi   = nslice2 - nslice1 + 1;
    nslices = nsam2 - nsam1 + 1;
    }
 else if (axis == 'y')
    {
    nsami   = nsam2 - nsam1 + 1;
    nrowi   = nslice2 - nslice1 + 1;
    nslices = nrow2 - nrow1 + 1;
    } 
 else if (axis == 'z')
    {
    nsami   = nsam2 - nsam1 + 1;
    nrowi   = nrow2 - nrow1 + 1;
    nslices = nslice2 - nslice1 + 1;
    }
 else
    { spout(" *** IN ZTOXYN: UNSUPORTED AXIS"); return FALSE; }


 if (iskip == 1)
    {    /* display full size */
    npixpsl = nrowi * nsami;
    if (axis == 'y')
       icon = -nrow1 * npixpsl - nslice1 * nsami - nsam1;
    else 
       icon = -nsam1 * npixpsl - nslice1 * nsami - nrow1;
    }
 else if (iskip > 1)
    {   /* display at reduced size */
    nsami = nsami / iskip;
    nrowi = nrowi / iskip;
    npixpsl = nrowi * nsami;

    if (axis == 'y')
       {
       icon = -nrow1 * npixpsl - nslice1 * nsami / iskip - 
               nsam1 / iskip;  
       nsamu = nsam2 - (nsam2 - nsam1 + 1) % iskip - iskip;
       }
    else if (axis == 'x')
       {
       icon = -nsam1 * npixpsl - nslice1 * nsami / iskip -
               nrow1 / iskip;
       nrowu = nrow2 - (nrow2-nrow1+1) % iskip - iskip + 1;
       }
    }
 else if (iskip < -1)
    {   /* display magnified image */
    miskip  = -iskip;
    miskipm1= miskip - 1;
    nsami   = nsami * miskip;
    nrowi   = nrowi * miskip;
    npixpsl = nrowi * nsami;
    if (axis == 'y')
       icon = -nrow1 * npixpsl - nslice1 * miskip * nsami - 1;
    else
       icon = -nsam1 * npixpsl - nslice1 * miskip * nsami -
               nrow1 * miskip;
    }
 else
    {
    spout("*** Invalid iskip in ztoxyn");
    return FALSE;
    }

 /* find skip factors for z axis */
 iskipz = MYMAX(1,iskip);
 
 /* find upper end of col. series */
 if (iskip > 1)
    nsliceu = nslice2 - (nslice2-nslice1+1) % iskip - iskip + 1;
 else
    nsliceu = nslice2;

 /* allocate space for range arrays */
 if (((fmint = (float *) malloc(nslices * sizeof(float))) == 
           (float *) 0) ||
     ((fmaxt = (float *) malloc(nslices * sizeof(float))) == 
           (float *) 0) ||
     ((rbuf  = (float *) malloc(nslices * nsami * nrowi * sizeof(float))
           ) ==  (float *) 0))
    {  spout(" *** Unable to allocate memory ztoxyn."); return FALSE; }

 /* set up lpointer to redlin array */
 buf = filedata -> buf;

 for (i=0; i< nslices; i++)
    {    /* zero the slice min/max values */
    fmint[i] = 10e10;  fmaxt[i] = -10e10;
    }

 for (islice=nslice1; islice<=nsliceu; islice=islice+iskipz)
   {
   if (axis == 'y')
      {    /* y axis slicing */
      if (iskip < 0)
         icons = islice * nsami * miskip + icon;
      else if (iskip > 1)
         icons = islice * nsami / iskip  + icon;
      else
         icons = islice * nsami          + icon;

      for (irow=nrow1; irow <= nrow2; irow++)
         {
         irect = (islice - 1) * nrow + irow;
         if (redlin(filedata,buf,nsam,irect) == FALSE) 
             {free(rbuf); 
              free(fmint); free(fmaxt);return FALSE;}

         ipane = irow - nrow1;
         tmin = fmint[ipane];
         tmax = fmaxt[ipane];

         if (iskip == 1)
            {  /* display full size y slice */

            iconr = irow * npixpsl + icons;
            for (isam=nsam1; isam <= nsam2; isam++)
               {
               fval  = buf[isam-1];
               tmin  = MYMIN(fval,tmin);
               tmax  = MYMAX(fval,tmax);

               iloc  = isam + iconr;
               rbuf[iloc] = fval;
               }         
            }  
 
         else if (iskip > 1) 
            {   /* display reduced size image y slice */
            iconr = irow * npixpsl + icons;
            
            for (isam=nsam1; isam <= nsamu; isam=isam+iskip)
               {
               fval  = buf[isam-1];
               tmin  = MYMIN(fval,tmin);
               tmax  = MYMAX(fval,tmax);

               iloc = isam / iskip + iconr;
               rbuf[iloc] = fval;
               }         
            }

         else if (iskip < -1) 
            {   /* display magnified image y slice */

            iloc = irow * npixpsl + icons;
            
            for (isam=nsam1; isam <= nsam2; isam++)
               {
               fval  = buf[isam-1];
               tmin  = MYMIN(fval,tmin);
               tmax  = MYMAX(fval,tmax);

               for (l = 1; l <= miskip; l++)
                  {
                  iloc++;
                  for (m = 0; m <= miskipm1; m++)
                     rbuf[iloc + m * nsami] = fval;
                  }
               }
            }
            fmint[ipane] = tmin;
            fmaxt[ipane] = tmax;
         }
     }
      
   else if (axis == 'x')
      { 
      if (iskip ==1)
         {   /* full size x slice */
         icons = islice * nsami + icon;

         for (irow=nrow1; irow <= nrow2; irow++)
            {
            irect = (islice - 1) * nrow + irow;
            if (redlin(filedata,buf,nsam,irect) == FALSE) 
               {free(rbuf); 
                free(fmint);free(fmaxt);return FALSE;}

            iconr = irow + icons;

            for (isam=nsam1; isam <= nsam2; isam++)
               {
               fval  = buf[isam];
               ipane = isam - nsam1;
               fmint[ipane]  = MYMIN(fval,fmint[ipane]);
               fmaxt[ipane]  = MYMAX(fval,fmaxt[ipane]);

               iloc = isam *npixpsl + iconr;
               rbuf[iloc] = fval;
               }         
            }  
         }
      else if (iskip > 1)
         {   /* reduced size x slice */
         icons = islice * nsami / iskip + icon;

         for (irow=nrow1; irow <= nrowu; irow=irow+iskip)
            {
            irect = (islice - 1) * nrow + irow;
            if (redlin(filedata,buf,nsam,irect) == FALSE) 
               {free(rbuf); free(fmint); 
                free(fmaxt);return FALSE;}
            iconr = irow / iskip + icons;

            for (isam=nsam1; isam <= nsam2; isam++)
               {
               fval  = buf[isam];
               ipane = isam - nsam1;
               fmint[ipane]  = MYMIN(fval,fmint[ipane]);
               fmaxt[ipane]  = MYMAX(fval,fmaxt[ipane]);

               iloc = isam *npixpsl + iconr;
               rbuf[iloc] = fval;
               }         
            }
         }

      else if (iskip < -1)
         {   /* magnifed x slice */ 

         icons = islice * nsami * miskip + icon;

         for (irow=nrow1; irow <= nrow2; irow++)
            {
            irect = (islice - 1) * nrow + irow;
            if (redlin(filedata,buf,nsam,irect) == FALSE) 
               {free(rbuf); free(fmint); 
                free(fmaxt);return FALSE;}

            iconr = irow * miskip + icons;

            for (isam=nsam1; isam <= nsam2; isam++)
               {
               fval  = buf[isam];
               ipane = isam - nsam1;
               fmint[ipane]  = MYMIN(fval,fmint[ipane]);
               fmaxt[ipane]  = MYMAX(fval,fmaxt[ipane]);

               iloc = isam * npixpsl + iconr;

               for (l = 1; l <= miskip; l++)
                  {
                  for (m = 0; m <= miskipm1; m++)
                     rbuf[iloc + m * nsami] = fval;
                  iloc++;
                  }
               }
            }
         }
      }
   }

 /* normalize the floating point montage by slice & convert to 8 bits */
locout = 0;

for (islice=0; islice<nslices; islice++)
    {
    if (abs(fmaxt[islice] - fmint[islice]) < 10e-10)
       {      /* blank slice */
       fn    = 0.0;
       fncon = nmint;
       }
    else
       {      /* FIND SCALE FACTORS FOR IMAGE DENSITY 0...NMAX  */
       fn    = (nmaxt - nmint) / (fmaxt[islice] - fmint[islice]);
       fncon = -fn  * fmint[islice] + nmint + 0.5;
       }

    /* FIND STARTING POSITION IN STORED ARRAY FOR THIS SLICE */
    igo   = islice * npixpsl;

    if (nmaxt < 128)
       {
       for (locin = igo; locin < (igo+npixpsl); locin++)
              cbuf[locout++] = rbuf[locin] * fn + fncon;
       }
    else
       {
       for (locin = igo; locin < (igo+npixpsl); locin++)
          {
          ival = rbuf[locin] * fn + fncon;
          if  (ival > 127) ival = ival - 256;
          cbuf[locout++] = ival;
          }
       }
    }

free(rbuf);
free(fmint);
free(fmaxt);

return TRUE;
}

